@@ -1,18 +1,17 @@
-//
-// Qt Example Widget: DigitalClock (dclock.h)
-//
-// Definition of DigitalClock widget.
-//
-// The DigitalClock widget is a stand-alone widget that you may
-// freely use in your Qt application (at your own risk!)
-// See the widgets/widgets.cpp demo application for how to add
-// a DigitalClock to your application.
-//
+/****************************************************************************
+** $Id: dclock.h,v 1.2 1999/06/03 01:44:08 warwick Exp $
+**
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
 
 #ifndef DCLOCK_H
 #define DCLOCK_H
 
-#include <qlcdnum.h>
+#include <qlcdnumber.h>
 
 
 class DigitalClock : public QLCDNumber		// digital clock widget
