@@ -1,14 +1,14 @@
 /****************************************************************************
-** $Id: drawdemo.cpp,v 2.7 1998/06/16 11:39:32 warwick Exp $
+** $Id: drawdemo.cpp,v 1.3.2.1 1999/09/09 12:36:19 aavit Exp $
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
 **
 *****************************************************************************/
 
-#include <qwindow.h>
+#include <qwidget.h>
 #include <qpainter.h>
 #include <qprinter.h>
 #include <qpushbutton.h>
@@ -27,25 +27,25 @@ void drawColorWheel( QPainter *p )
 {
     QFont f( "times", 18, QFont::Bold );
     p->setFont( f );
-    p->setPen( black );				// black pen outline
+    p->setPen( Qt::black );				// black pen outline
     p->setWindow( 0, 0, 500, 500 );		// defines coordinate system
 
     for ( int i=0; i<36; i++ ) {		// draws 36 rotated rectangles
 
-	QWMatrix matrix;
-	matrix.translate( 250.0F, 250.0F );	// move to center
-	matrix.shear( 0.0F, 0.3F );		// twist it
-	matrix.rotate( (float)i*10 );		// rotate 0,10,20,.. degrees
-	p->setWorldMatrix( matrix );		// use this world matrix
+        QWMatrix matrix;
+        matrix.translate( 250.0F, 250.0F );	// move to center
+        matrix.shear( 0.0F, 0.3F );		// twist it
+        matrix.rotate( (float)i*10 );		// rotate 0,10,20,.. degrees
+        p->setWorldMatrix( matrix );		// use this world matrix
 
-	QColor c;
-	c.setHsv( i*10, 255, 255 );		// rainbow effect
-	p->setBrush( c );			// solid fill with color c
-	p->drawRect( 70, -10, 80, 10 );		// draw the rectangle
+        QColor c;
+        c.setHsv( i*10, 255, 255 );		// rainbow effect
+        p->setBrush( c );			// solid fill with color c
+        p->drawRect( 70, -10, 80, 10 );		// draw the rectangle
 
-	QString n;
-	n.sprintf( "H=%d", i*10 );
-	p->drawText( 80+70+5, 0, n );		// draw the hue number
+        QString n;
+        n.sprintf( "H=%d", i*10 );
+        p->drawText( 80+70+5, 0, n );		// draw the hue number
     }
 }
 
@@ -57,22 +57,21 @@ void drawColorWheel( QPainter *p )
 void drawFonts( QPainter *p )
 {
     static char *fonts[] = { "Helvetica", "Courier", "Times", 0 };
-    static int	 sizes[] = { 10, 12, 18, 24, 0 };
+    static int	 sizes[] = { 10, 12, 18, 24, 36, 0 };
     int f = 0;
-    int s = 0;
     int y = 0;
     while ( fonts[f] ) {
-	s = 0;
-	while ( sizes[s] ) {
-	    QFont font( fonts[f], sizes[s] );
-	    p->setFont( font );
-	    QFontMetrics fm = p->fontMetrics();
-	    y += fm.ascent();
-	    p->drawText( 10, y, "Quartz Glyph Job Vex'd Cwm Finks" );
-	    y += fm.descent();
-	    s++;
-	}
-	f++;
+        int s = 0;
+        while ( sizes[s] ) {
+            QFont font( fonts[f], sizes[s] );
+            p->setFont( font );
+            QFontMetrics fm = p->fontMetrics();
+            y += fm.ascent();
+            p->drawText( 10, y, "Quartz Glyph Job Vex'd Cwm Finks" );
+            y += fm.descent();
+            s++;
+        }
+        f++;
     }
 }
 
@@ -83,12 +82,12 @@ void drawFonts( QPainter *p )
 
 void drawShapes( QPainter *p )
 {
-    QBrush b1( blue );				// solid blue brush
-    QBrush b2( green, Dense6Pattern );		// green 12% fill
-    QBrush b3( NoBrush );			// void brush
-    QBrush b4( CrossPattern );			// black cross pattern
+    QBrush b1( Qt::blue );				// solid blue brush
+    QBrush b2( Qt::green, Qt::Dense6Pattern );		// green 12% fill
+    QBrush b3( Qt::NoBrush );			// void brush
+    QBrush b4( Qt::CrossPattern );			// black cross pattern
 
-    p->setPen( red );
+    p->setPen( Qt::red );
     p->setBrush( b1 );
     p->drawRect( 10, 10, 200, 100 );		// draw some shapes
     p->setBrush( b2 );
@@ -108,6 +107,8 @@ void drawShapes( QPainter *p )
 
 void drawPathText( QPainter *p )
 {
+    p->setWindow( 0, 0, 400, 300 );
+
     QPointArray a( 4 );
     a.setPoint( 0, 100,200 );
     a.setPoint( 1, 150,75 );
@@ -115,34 +116,34 @@ void drawPathText( QPainter *p )
     a.setPoint( 3, 300,200 );
     a = a.quadBezier();				// calculate Bezier curve
 
-    p->setPen( lightGray );			// set light gray pen
+    p->setPen( Qt::lightGray );			// set light gray pen
     p->drawPolyline( a );			// draw Bezier point array
 
-    p->setFont( QFont("Times",24) );		// set fast font
-    p->setPen( black );				// set black pen
+    p->setFont( QFont( "Times", 24 ) );		// set fast font
+    p->setPen( Qt::black );			// set black pen
 
     const char *text = "Troll Tech AS";
 
     int len = strlen(text);
     if ( len == 0 )
-	return;
+        return;
     int ipos = a.size()/len;
     int cpos = ipos;
 
     for ( int i=0; i<len; i++ ) {		// for each char in text...
-	QPoint p1 = a.point( cpos-1 );
-	QPoint p2 = a.point( cpos+1 );
-	QPoint pt = a.point(cpos);
-	float dx = (float)(p2.x() - p1.x());
-	float dy = (float)(p2.y() - p1.y());
-	float angle = (float)atan(dy/dx);	// way too simple
-	angle *= 180.0F/3.14F;
-	QWMatrix m;				// setup world matrix
-	m.translate( (float)pt.x(), (float)pt.y() );
-	m.rotate( angle );
-	p->setWorldMatrix( m );
-	p->drawText( 0,0, &text[i], 1 );
-	cpos += ipos;
+        QPoint p1 = a.point( cpos-1 );
+        QPoint p2 = a.point( cpos+1 );
+        QPoint pt = a.point(cpos);
+        float dx = (float)(p2.x() - p1.x());
+        float dy = (float)(p2.y() - p1.y());
+        float angle = (float)atan(dy/dx);	// way too simple
+        angle *= 180.0F/3.14F;
+        QWMatrix m;				// setup world matrix
+        m.translate( (float)pt.x(), (float)pt.y() );
+        m.rotate( angle );
+        p->setWorldMatrix( m );
+        p->drawText( 0,0, &text[i], 1 );
+        cpos += ipos;
     }
 
 }
@@ -173,11 +174,12 @@ DrawThing ourDrawFunctions[] = {
 // and a text in the table above.
 //
 
-class DrawView : public QWindow
+class DrawView : public QWidget
 {
     Q_OBJECT
 public:
     DrawView();
+    ~DrawView();
 public slots:
     void   updateIt( int );
     void   printIt();
@@ -205,27 +207,27 @@ DrawView::DrawView()
 
     printer = new QPrinter;
 
-  // Create a button group to contain all buttons
+    // Create a button group to contain all buttons
     bgroup = new QButtonGroup( this );
     bgroup->resize( 200, 200 );
     connect( bgroup, SIGNAL(clicked(int)), SLOT(updateIt(int)) );
 
-  // Calculate the size for the radio buttons
+    // Calculate the size for the radio buttons
     int maxwidth = 80;
     int i;
     char *n;
     QFontMetrics fm = bgroup->fontMetrics();
     for ( i=0; (n=ourDrawFunctions[i].name) != 0; i++ ) {
-	int w = fm.width( n );
-	maxwidth = QMAX(w,maxwidth);
+        int w = fm.width( n );
+        maxwidth = QMAX(w,maxwidth);
     }
     maxwidth = maxwidth + 20;			// add 20 pixels
 
     for ( i=0; (n=ourDrawFunctions[i].name) != 0; i++ ) {
-	QRadioButton *rb = new QRadioButton( n, bgroup );
-	rb->setGeometry( 10, i*30+10, maxwidth, 30 );
-	if ( i == 0 )
-	    rb->setChecked( TRUE );
+        QRadioButton *rb = new QRadioButton( n, bgroup );
+        rb->setGeometry( 10, i*30+10, maxwidth, 30 );
+        if ( i == 0 )
+            rb->setChecked( TRUE );
     }
 
     drawindex = 0;				// draw first thing
@@ -233,8 +235,8 @@ DrawView::DrawView()
 
     maxwidth += 40;				// now size of bgroup
 
-  // Create and setup the print button
-    print = new QPushButton( "Print", bgroup );
+    // Create and setup the print button
+    print = new QPushButton( "Print...", bgroup );
     print->resize( 80, 30 );
     print->move( maxwidth/2 - print->width()/2, maxindex*30+20 );
     connect( print, SIGNAL(clicked()), SLOT(printIt()) );
@@ -244,6 +246,13 @@ DrawView::DrawView()
     resize( 640,300 );
 }
 
+//
+// Clean up
+//
+DrawView::~DrawView()
+{
+    delete printer;
+}
 
 //
 // Called when a radio button is clicked.
@@ -252,8 +261,8 @@ DrawView::DrawView()
 void DrawView::updateIt( int index )
 {
     if ( index < maxindex ) {
-	drawindex = index;
-	update();
+        drawindex = index;
+        update();
     }
 }
 
@@ -272,9 +281,9 @@ void DrawView::drawIt( QPainter *p )
 
 void DrawView::printIt()
 {
-    if ( printer->setup(this) ) {
-	QPainter paint( printer );	
-	drawIt( &paint );
+    if ( printer->setup( this ) ) {
+        QPainter paint( printer );	
+        drawIt( &paint );
     }
 }
 
