@@ -1,14 +1,15 @@
-//
-// File: main.cpp
-//
-// A small Qt example application written by Troll Tech.
-//
-// It displays a text in a window and quits when you click
-// the mouse in the window.
-//
+/****************************************************************************
+** $Id: main.cpp,v 1.2 1999/06/03 01:44:40 warwick Exp $
+**
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
 
 #include "hello.h"
-#include <qapp.h>
+#include <qapplication.h>
 
 
 /*
@@ -31,7 +32,7 @@ int main( int argc, char **argv )
     h.setCaption( "Qt says hello" );
     QObject::connect( &h, SIGNAL(clicked()), &a, SLOT(quit()) );
     h.setFont( QFont("times",32,QFont::Bold) );		// default font
-    h.setBackgroundColor( white );			// default bg color
+    h.setBackgroundColor( Qt::white );			// default bg color
     a.setMainWidget( &h );
     h.show();
     return a.exec();
