@@ -1,7 +1,12 @@
-//
-// Qt Example Application: menu
-//
-
+/****************************************************************************
+** $Id: menu.h,v 1.2 1999/06/03 01:44:56 warwick Exp $
+**
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
 
 #ifndef MENU_H
 #define MENU_H
@@ -38,7 +43,7 @@ protected:
     void    resizeEvent( QResizeEvent * );
 
 signals:
-    void    explain( const char * );
+    void    explain( const QString& );
 
 private:
     QMenuBar *menu;
@@ -46,7 +51,6 @@ private:
     bool isBold;
     bool isUnderline;
     int boldID, underlineID;
-    QPopupMenu *options;
 };
 
 
