@@ -1,17 +1,22 @@
-//
-// Qt Example Application: qmag
-//
-// Magnifies parts of the screen.
-//
+/****************************************************************************
+** $Id: qmag.cpp,v 1.2 1999/06/03 01:45:14 warwick Exp $
+**
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
 
-#include <qcombo.h>
-#include <qpushbt.h>
+#include <qcombobox.h>
+#include <qpushbutton.h>
 #include <qpixmap.h>
 #include <qimage.h>
 #include <qlabel.h>
-#include <qfiledlg.h>
+#include <qfiledialog.h>
+#include <qregexp.h>
 
-#include <qapp.h>
+#include <qapplication.h>
 #include <qpainter.h>
 #include <qwmatrix.h>
 
@@ -26,6 +31,7 @@ public slots:
     void	setZoom( int );
     void	setRefresh( int );
     void	save();
+    void	multiSave();
 
 protected:
     void	paintEvent( QPaintEvent * );
@@ -42,6 +48,7 @@ private:
     QComboBox   *zoom;
     QComboBox   *refresh;
     QPushButton *saveButton;
+    QPushButton *multiSaveButton;
     QPushButton *quitButton;
     QPixmap	pm;		// pixmap, magnified
     QPixmap	p;		// pixmap
@@ -50,8 +57,9 @@ private:
     int		yoffset;	// pixels in addition to the actual picture
     int		z;		// magnification factor
     int		r;		// autorefresh rate (index into refreshrates)
-    bool	grabbing;	// true if qmag is currently grabbing
+    bool	grabbing;	// TRUE if qmag is currently grabbing
     int		grabx, graby;
+    QString	multifn;	// filename for multisave
 };
 
 
@@ -71,8 +79,6 @@ static const int timer[] = {
 MagWidget::MagWidget( QWidget *parent, const char *name )
     : QWidget( parent, name)
 {
-    initMetaObject();
-
     zoom = new QComboBox( FALSE, this );
     CHECK_PTR(zoom);
     zoom->insertStrList( zoomfactors, 9 );
@@ -107,11 +113,19 @@ MagWidget::MagWidget( QWidget *parent, const char *name )
     saveButton->setGeometry( x+w+30+2, 2,
 			     10+saveButton->fontMetrics().width("Save"), 20 );
 
+    multiSaveButton = new QPushButton( this );
+    multiSaveButton->setToggleButton(TRUE);
+    CHECK_PTR(multiSaveButton);
+    connect( multiSaveButton, SIGNAL(clicked()), this, SLOT(multiSave()) );
+    multiSaveButton->setText( "MultiSave" );
+    multiSaveButton->setGeometry( saveButton->geometry().right() + 2, 2,
+			     10+multiSaveButton->fontMetrics().width("MultiSave"), 20 );
+
     quitButton = new QPushButton( this );
     CHECK_PTR(quitButton);
     connect( quitButton, SIGNAL(clicked()), qApp, SLOT(quit()) );
     quitButton->setText( "Quit" );
-    quitButton->setGeometry( saveButton->geometry().right() + 2, 2,
+    quitButton->setGeometry( multiSaveButton->geometry().right() + 2, 2,
 			     10+quitButton->fontMetrics().width("Quit"), 20 );
 
     rgb = new QLabel( this );
@@ -167,6 +181,20 @@ void MagWidget::save()
     }
 }
 
+void MagWidget::multiSave()
+{
+    if ( !p.isNull() ) {
+	multifn = ""; // stops saving
+	multifn = QFileDialog::getSaveFileName();
+	if ( multifn.isEmpty() )
+	    multiSaveButton->setOn(FALSE);
+	if ( !r )
+	    p.save( multifn, "BMP" );
+    } else {
+	multiSaveButton->setOn(FALSE);
+    }
+}
+
 
 void MagWidget::grab()
 {
@@ -200,18 +228,17 @@ void MagWidget::grab()
     m.scale( (double)z, (double)z );
     pm = p.xForm( m );
 
-    repaint( FALSE );		// and finally repaint, flicker-free
+    if ( !multiSaveButton->isOn() )
+	repaint( FALSE );		// and finally repaint, flicker-free
 }
 
 
 void MagWidget::paintEvent( QPaintEvent * )
 {
     if ( !pm.isNull() ) {
-	QPainter paint;
-	paint.begin( this );
+	QPainter paint( this );
 	paint.drawPixmap( 0, zoom->height()+4, pm, 
 			  0,0, width(), height()-yoffset );
-	paint.end();
     }
 }
 
@@ -304,6 +331,16 @@ void MagWidget::focusOutEvent( QFocusEvent * )
 void MagWidget::timerEvent( QTimerEvent * )
 {
     grab();
+    if ( multiSaveButton->isOn() && !multifn.isEmpty() ) {
+	QRegExp num("[0-9][0-9]*");
+	int start;
+	int len;
+	if ((start=num.match(multifn,0,&len))>=0)
+	    multifn.replace(num,
+		QString().setNum(multifn.mid(start,len).toInt()+1)
+	    );
+	p.save( multifn, "BMP" );
+    }
 }
 
 
