@@ -1,26 +1,29 @@
 /****************************************************************************
-** $Id: main.cpp,v 1.3 1997/02/07 11:17:18 hanord Exp $
+** $Id: main.cpp,v 1.3 1999/06/23 19:09:49 ettrich Exp $
 **
-** Ritual main() for Qt applications
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
 **
 *****************************************************************************/
 
-#include <qapp.h>
+#include <qapplication.h>
 #include "qwerty.h"
 
 
 int main( int argc, char **argv )
 {
     QApplication a( argc, argv );
+
     int i;
-    for ( i=0; i<argc; i++ ) {
+    for ( i= argc <= 1 ? 0 : 1; i<argc; i++ ) {
 	Editor *e = new Editor;
 	e->resize( 400, 400 );
 	if ( i > 0 )
 	    e->load( argv[i] );
 	e->show();
     }
-    QObject::connect( &a, SIGNAL(lastWindowClosed()),
-		      &a, SLOT(quit()) );
+    a.connect( &a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()) );
     return a.exec();
 }
