@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id: main.cpp,v 2.18 1998/06/16 11:39:34 warwick Exp $
+** $Id: main.cpp,v 1.3 1999/06/24 20:23:39 warwick Exp $
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -19,10 +19,12 @@
 
 int main( int argc, char **argv )
 {
-    if ( argv[1] && 0==strcmp(argv[1], "-m") ) {
+    if ( argc > 1 && QString(argv[1]) == "-m" ) {
 	QApplication::setColorSpec( QApplication::ManyColor );
 	argc--;
 	argv++;
+    } else {
+	QApplication::setColorSpec( QApplication::CustomColor );
     }
 
     QApplication::setFont( QFont("Helvetica", 12) );
@@ -35,13 +37,13 @@ int main( int argc, char **argv )
     if ( argc <= 1 ) {
 	// Create a window which looks after its own existence.
 	ImageViewer *w =
-	    new ImageViewer(0, "new window", WDestructiveClose);
+	    new ImageViewer(0, "new window", Qt::WDestructiveClose);
 	w->show();
     } else {
 	for ( int i=1; i<argc; i++ ) {
 	    // Create a window which looks after its own existence.
 	    ImageViewer *w =
-		new ImageViewer(0, argv[i], WDestructiveClose);
+		new ImageViewer(0, argv[i], Qt::WDestructiveClose);
 	    w->loadImage( argv[i] );
 	    w->show();
 	}
