@@ -1,12 +1,23 @@
+/****************************************************************************
+** $Id: showimg.cpp,v 1.3 1999/06/22 03:50:42 warwick Exp $
+**
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
+
 #include "showimg.h"
 #include <qmenubar.h>
-#include <qfiledlg.h>
-#include <qmsgbox.h>
-#include <qpopmenu.h>
+#include <qfiledialog.h>
+#include <qmessagebox.h>
+#include <qpopupmenu.h>
 #include <qlabel.h>
 #include <qpainter.h>
 #include <qkeycode.h>
-#include <qapp.h>
+#include <qapplication.h>
+#include <qclipboard.h>
 
 
 /*
@@ -16,13 +27,17 @@
   The menu uses a single slot to simplify the process of adding
   more items to the options menu.
 */
-ImageViewer::ImageViewer( QWidget *parent, const char *name )
-    : QWidget( parent, name ),
+ImageViewer::ImageViewer( QWidget *parent, const char *name, int wFlags )
+    : QWidget( parent, name, wFlags ),
       conversion_flags( PreferDither ),
       filename( 0 ),
       helpmsg( 0 )
 {
     pickx = -1;
+    picky = -1;
+    clickx = -1;
+    clicky = -1;
+    alloc_context = 0;
 
     menubar = new QMenuBar(this);
     menubar->setSeparator( QMenuBar::InWindowsStyle );
@@ -39,12 +54,22 @@ ImageViewer::ImageViewer( QWidget *parent, const char *name )
 
     file = new QPopupMenu();
     menubar->insertItem( "&File", file );
+    file->insertItem( "New window", this,  SLOT(newWindow()), CTRL+Key_N );
     file->insertItem( "Open...", this,  SLOT(openFile()), CTRL+Key_O );
     si = file->insertItem( "Save image", saveimage );
     sp = file->insertItem( "Save pixmap", savepixmap );
     file->insertSeparator();
     file->insertItem( "Quit", qApp,  SLOT(quit()), CTRL+Key_Q );
 
+    edit =  new QPopupMenu();
+    menubar->insertItem( "&Edit", edit );
+    edit->insertItem("&Copy", this, SLOT(copy()), ALT+Key_C);
+    edit->insertItem("&Paste", this, SLOT(paste()), ALT+Key_V);
+    edit->insertSeparator();
+    t1 = edit->insertItem( "Convert to &1 bit", this, SLOT(to1Bit()) );
+    t8 = edit->insertItem( "Convert to &8 bit", this, SLOT(to8Bit()) );
+    t32 = edit->insertItem( "Convert to &32 bit", this, SLOT(to32Bit()) );
+
     options =  new QPopupMenu();
     menubar->insertItem( "&Options", options );
     ac = options->insertItem( "AutoColor" );
@@ -61,6 +86,11 @@ ImageViewer::ImageViewer( QWidget *parent, const char *name )
     options->insertSeparator();
     ad = options->insertItem( "PreferDither" );
     dd = options->insertItem( "AvoidDither" );
+    options->insertSeparator();
+    ss = options->insertItem( "Smooth scaling" );
+    cc = options->insertItem( "Use colour context" );
+    if ( QApplication::colorSpec() == QApplication::ManyColor )
+	options->setItemEnabled( cc, FALSE );
     options->setCheckable( TRUE );
     setMenuItemFlags();
 
@@ -79,6 +109,14 @@ ImageViewer::ImageViewer( QWidget *parent, const char *name )
     setMouseTracking( TRUE );
 }
 
+ImageViewer::~ImageViewer()
+{
+    if ( alloc_context )
+	QColor::destroyAllocContext( alloc_context );
+    if ( other == this )
+	other = 0;
+}
+
 /*
   This function modifies the conversion_flags when an options menu item
   is selected, then ensures all menu items are up to date, and reconverts
@@ -86,6 +124,16 @@ ImageViewer::ImageViewer( QWidget *parent, const char *name )
 */
 void ImageViewer::doOption(int item)
 {
+    if ( item == ss || item == cc ) {
+	// Toggle
+	bool newbool = !options->isItemChecked(item);
+	options->setItemChecked(item, newbool);
+	// And reconvert...
+	reconvertImage();
+	repaint(image.hasAlphaBuffer());	// show image in widget
+	return;
+    }
+
     if ( options->isItemChecked( item ) ) return; // They are all radio buttons
 
     int ocf = conversion_flags;
@@ -118,7 +166,7 @@ void ImageViewer::doOption(int item)
 	setMenuItemFlags();
 	// And reconvert...
 	reconvertImage();
-	repaint();			// show image in widget
+	repaint(image.hasAlphaBuffer());	// show image in widget
     }
 }
 
@@ -132,13 +180,19 @@ void ImageViewer::setMenuItemFlags()
     file->setItemEnabled( si, valid_image );
     file->setItemEnabled( sp, valid_image );
 
+    // Edit
+    edit->setItemEnabled( t1, image.depth() != 1 );
+    edit->setItemEnabled( t8, image.depth() != 8 );
+    edit->setItemEnabled( t32, image.depth() != 32 );
+
     // Options
     bool may_need_color_dithering =
-	    image.depth() == 32 && QPixmap::defaultDepth() <= 8;
+	    !valid_image
+	|| image.depth() == 32 && QPixmap::defaultDepth() < 24;
     bool may_need_dithering = may_need_color_dithering
 	 || image.depth() > 1 && options->isItemChecked(mo)
 	 || image.depth() > 1 && QPixmap::defaultDepth() == 1;
-    bool has_alpha_mask = image.hasAlphaBuffer();
+    bool has_alpha_mask = !valid_image || image.hasAlphaBuffer();
 
     options->setItemEnabled( fd, may_need_dithering );
     options->setItemEnabled( bd, may_need_dithering );
@@ -189,7 +243,12 @@ void ImageViewer::updateStatus()
 	    message += moremsg;
 	}
 	if ( image.numColors() > 0 ) {
-	    moremsg.sprintf(", %d colors", image.numColors());
+	    if (image.valid(pickx,picky)) {
+		moremsg.sprintf(", %d/%d colors", image.pixelIndex(pickx,picky),
+		    image.numColors());
+	    } else {
+		moremsg.sprintf(", %d colors", image.numColors());
+	    }
 	    message += moremsg;
 	}
 	if ( image.hasAlphaBuffer() ) {
@@ -225,7 +284,8 @@ void ImageViewer::updateStatus()
 void ImageViewer::saveImage( int item )
 {
     const char* fmt = saveimage->text(item);
-    QString savefilename = QFileDialog::getSaveFileName(0, 0, 0, filename);
+    QString savefilename = QFileDialog::getSaveFileName(QString::null, QString::null,
+					this, filename);
     if ( !savefilename.isEmpty() )
 	if ( !image.save( savefilename, fmt ) )
 	    QMessageBox::warning( this, "Save failed", "Error saving file" );
@@ -237,12 +297,20 @@ void ImageViewer::saveImage( int item )
 void ImageViewer::savePixmap( int item )
 {
     const char* fmt = savepixmap->text(item);
-    QString savefilename = QFileDialog::getSaveFileName(0, 0, 0, filename);
+    QString savefilename = QFileDialog::getSaveFileName(QString::null,
+					QString::null, this, filename);
     if ( !savefilename.isEmpty() )
 	if ( !pmScaled.save( savefilename, fmt ) )
 	    QMessageBox::warning( this, "Save failed", "Error saving file" );
 }
 
+
+void ImageViewer::newWindow()
+{
+    ImageViewer* that = new ImageViewer(0, "new window", WDestructiveClose);
+    that->show();
+}
+
 /*
   This function is the slot for processing the Open menu item.
 */
@@ -271,6 +339,8 @@ bool ImageViewer::loadImage( const char *fileName )
     if ( filename ) {
 	QApplication::setOverrideCursor( waitCursor ); // this might take time
 	ok = image.load(filename, 0);
+	pickx = -1;
+	clickx = -1;
 	if ( ok )
 	    ok = reconvertImage();
 	if ( ok ) {
@@ -303,10 +373,24 @@ bool ImageViewer::reconvertImage()
 {
     bool success = FALSE;
 
+    if ( image.isNull() ) return FALSE;
+
+    if ( alloc_context ) {
+	QColor::destroyAllocContext( alloc_context );
+	alloc_context = 0;
+    }
+    if ( useColorContext() ) {
+	alloc_context = QColor::enterAllocContext();
+	// Clear the image to hide flickering palette
+	QPainter painter(this);
+	painter.eraseRect(0, menubar->heightForWidth( width() ), width(), height());
+    }
+
     QApplication::setOverrideCursor( waitCursor ); // this might take time
     if ( pm.convertFromImage(image, conversion_flags) )
     {
-	pmScaled = pm;
+	pmScaled = QPixmap();
+	scale();
 	resize( width(), height() );
 	success = TRUE;				// load successful
     } else {
@@ -315,9 +399,23 @@ bool ImageViewer::reconvertImage()
     updateStatus();
     setMenuItemFlags();
     QApplication::restoreOverrideCursor();	// restore original cursor
+
+    if ( useColorContext() )
+	QColor::leaveAllocContext();
+
     return success;				// TRUE if loaded OK
 }
 
+bool ImageViewer::smooth() const
+{
+    return options->isItemChecked(ss);
+}
+
+bool ImageViewer::useColorContext() const
+{
+    return options->isItemChecked(cc);
+}
+
 /*
   This functions scales the pixmap in the member variable "pm" to fit the
   widget size and  puts the resulting pixmap in the member variable "pmScaled".
@@ -327,15 +425,22 @@ void ImageViewer::scale()
 {
     int h = height() - menubar->heightForWidth( width() ) - status->height();
 
+    if ( image.isNull() ) return;
+
     QApplication::setOverrideCursor( waitCursor ); // this might take time
     if ( width() == pm.width() && h == pm.height() )
     {						// no need to scale if widget
 	pmScaled = pm;				// size equals pixmap size
     } else {
-	QWMatrix m;				// transformation matrix
-	m.scale(((double)width())/pm.width(),	// define scale factors
-		((double)h)/pm.height());
-	pmScaled = pm.xForm( m );		// create scaled pixmap
+	if (smooth()) {
+	    pmScaled.convertFromImage(image.smoothScale(width(), h),
+		conversion_flags);
+	} else {
+	    QWMatrix m;				// transformation matrix
+	    m.scale(((double)width())/pm.width(),// define scale factors
+		    ((double)h)/pm.height());
+	    pmScaled = pm.xForm( m );		// create scaled pixmap
+	}
     }
     QApplication::restoreOverrideCursor();	// restore original cursor
 }
@@ -361,19 +466,45 @@ void ImageViewer::resizeEvent( QResizeEvent * )
     }
 }
 
+bool ImageViewer::convertEvent( QMouseEvent* e, int& x, int& y)
+{
+    if ( pm.size() != QSize( 0, 0 ) ) {
+	int h = height() - menubar->heightForWidth( width() ) - status->height();
+	int nx = e->x() * image.width() / width();
+	int ny = (e->y()-menubar->heightForWidth( width() )) * image.height() / h;
+	if (nx != x || ny != y ) {
+	    x = nx;
+	    y = ny;
+	    updateStatus();
+	    return TRUE;
+	}
+    }
+    return FALSE;
+}
+
+void ImageViewer::mousePressEvent( QMouseEvent *e )
+{
+    may_be_other = convertEvent(e, clickx, clicky);
+}
+
+void ImageViewer::mouseReleaseEvent( QMouseEvent * )
+{
+    if ( may_be_other )
+	other = this;
+}
+
 /*
   Record the pixel position of interest.
 */
 void ImageViewer::mouseMoveEvent( QMouseEvent *e )
 {
-    if ( pm.size() != QSize( 0, 0 ) ) {
-	int h = height() - menubar->heightForWidth( width() ) - status->height();
-	int npickx = e->x() * image.width() / width();
-	int npicky = (e->y()-menubar->heightForWidth( width() )) * image.height() / h;
-	if (npickx != pickx || npicky != picky ) {
-	    pickx = npickx;
-	    picky = npicky;
-	    updateStatus();
+    if (convertEvent(e,pickx,picky)) {
+	updateStatus();
+	if ((e->state()&LeftButton)) {
+	    may_be_other = FALSE;
+	    if ( clickx >= 0 && other) {
+		copyFrom(other);
+	    }
 	}
     }
 }
@@ -399,7 +530,7 @@ void ImageViewer::paintEvent( QPaintEvent *e )
 void ImageViewer::giveHelp()
 {
     if (!helpmsg) {
-	QString helptext = "Usage: showimg [filename]";
+	QString helptext = "Usage: showimg [-m] [filename]\n\n   -m  - use ManyColor color spec";
 	QStrList support = QImage::inputFormats();
 	helptext += "\n\nSupported input formats:\n";
 	int lastnl = helptext.length();
@@ -423,3 +554,89 @@ void ImageViewer::giveHelp()
     helpmsg->show();
     helpmsg->raise();
 }
+
+void ImageViewer::copyFrom(ImageViewer* s)
+{
+    if ( clickx >= 0 ) {
+	int dx = clickx;
+	int dy = clicky;
+	int sx = s->clickx;
+	int sy = s->clicky;
+	int sw = QABS(clickx - pickx)+1;
+	int sh = QABS(clicky - picky)+1;
+	if ( clickx > pickx ) {
+	    dx = pickx;
+	    sx -= sw-1;
+	}
+	if ( clicky > picky ) {
+	    dy = picky;
+	    sy -= sh-1;
+	}
+	bitBlt( &image, dx, dy, &s->image, sx, sy, sw, sh );
+	reconvertImage();
+	repaint( image.hasAlphaBuffer() );
+    }
+}
+ImageViewer* ImageViewer::other = 0;
+
+void ImageViewer::copy()
+{
+    //QApplication::clipboard()->setPixmap(pm);
+    QApplication::clipboard()->setImage(image); // Less information loss
+}
+
+void ImageViewer::paste()
+{
+    image = QApplication::clipboard()->image();
+
+    if ( !image.isNull() ) {
+	filename = "pasted";
+
+	// From loadImage - could be factored out
+
+	pickx = -1;
+	clickx = -1;
+	setCaption( filename );			// set window caption
+	int w = image.width();
+	int h = image.height();
+
+	const int reasonable_width = 128;
+	if ( w < reasonable_width ) {
+	    // Integer scale up to something reasonable
+	    int multiply = ( reasonable_width + w - 1 ) / w;
+	    w *= multiply;
+	    h *= multiply;
+	}
+
+	h += menubar->heightForWidth(w) + status->height();
+	resize( w, h );				// we resize to fit image
+
+	reconvertImage();
+	repaint( image.hasAlphaBuffer() );
+
+	updateStatus();
+	setMenuItemFlags();
+    }
+}
+
+void ImageViewer::to1Bit()
+{
+    toBitDepth(1);
+}
+
+void ImageViewer::to8Bit()
+{
+    toBitDepth(8);
+}
+
+void ImageViewer::to32Bit()
+{
+    toBitDepth(32);
+}
+
+void ImageViewer::toBitDepth(int d)
+{
+    image = image.convertDepth(d);
+    reconvertImage();
+    repaint( image.hasAlphaBuffer() );
+}
