@@ -1,16 +1,16 @@
-//
-// Qt example: Table
-//
-// A simple, spreadsheet-like widget, made by inheriting  QTableView.
-// 
-// File: main.cpp
-//
-// The main() function, showing the use of the Table widget
-// 
+/****************************************************************************
+** $Id: main.cpp,v 1.2 1999/06/03 01:45:36 warwick Exp $
+**
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
 
 #include "table.h"
-#include <qpushbt.h>
-#include <qapp.h>
+#include <qpushbutton.h>
+#include <qapplication.h>
 
 
 /*
