@@ -1,17 +1,17 @@
-//
-// Qt example: Table
-//
-// A simple, spreadsheet-like widget, made by inheriting  QTableView.
-// 
-// File: table.h
-//
-// Definition of the Table widget.
-//
+/****************************************************************************
+** $Id: table.h,v 1.2 1999/06/03 01:45:38 warwick Exp $
+**
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
 
 #ifndef TABLE_H
 #define TABLE_H
 
-#include <qtablevw.h>
+#include <qtableview.h>
 
 
 class Table : public QTableView
