@@ -1,19 +1,17 @@
 /****************************************************************************
-** $Id: qdragapp.h,v 2.0 1996/09/24 10:15:32 agulbra Exp $
+** $Id: qdragapp.h,v 1.2 1999/06/03 01:45:45 warwick Exp $
 **
-** Definition of QDragApplication class
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** Author  : Eirik Eng
-** Created : 931209
-**
-** Copyright (C) 1994 by Troll Tech AS.  All rights reserved.
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
 **
 *****************************************************************************/
 
 #ifndef QDRAGAPP_H
 #define QDRAGAPP_H
 
-#include "qapp.h"
+#include "qapplication.h"
 
 class QDragger;
 
