@@ -1,10 +1,20 @@
+/****************************************************************************
+** $Id: qtetrix.h,v 1.2 1999/06/03 01:45:47 warwick Exp $
+**
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
+
 #ifndef QTETRIX_H
 #define QTETRIX_H
 
 #include "qtetrixb.h"
 #include <qframe.h>
-#include <qlcdnum.h>
-#include <qpushbt.h>
+#include <qlcdnumber.h>
+#include <qpushbutton.h>
 #include <qpainter.h>
 
 
