@@ -1,3 +1,13 @@
+/****************************************************************************
+** $Id: qtetrixb.cpp,v 1.2 1999/06/03 01:45:48 warwick Exp $
+**
+** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
+
 #include "qtetrixb.h"
 #include "qtetrix.h"
 #include <qtimer.h>
@@ -9,7 +19,6 @@ const int waitAfterLineTime = 500;
 QTetrixBoard::QTetrixBoard( QWidget *p, const char *name )
     : QFrame( p, name )
 {
-    initMetaObject();
     setFrameStyle( QFrame::Panel | QFrame::Sunken );
     paint = 0;
     timer = new QTimer(this);
@@ -35,9 +44,9 @@ QTetrixBoard::QTetrixBoard( QWidget *p, const char *name )
 
 void QTetrixBoard::startGame(int gameType,int fillRandomLines)
 {
-    noGame = FALSE;
     if ( isPaused )
-        pause();   // Turn off the pause function.
+        return;		// ignore if game is paused
+    noGame = FALSE;
     GenericTetrix::startGame( gameType, fillRandomLines );
     // Note that the timer is started by updateLevel!
 }
@@ -68,10 +77,8 @@ void QTetrixBoard::drawSquare(int x,int y,int value)
 
     bool localPainter = paint == 0;
     QPainter *p;
-    if ( localPainter ) {
-	p = new QPainter;    
-	p->begin( this );
-    }
+    if ( localPainter )
+	p = new QPainter( this );    
     else
 	p = paint;
     drawTetrixButton( p, X, Y, blockWidth, blockHeight,
@@ -87,10 +94,8 @@ void QTetrixBoard::drawSquare(int x,int y,int value)
     else
 	p->fillRect( X, Y, blockWidth, blockHeight, backgroundColor() );
 	*/
-    if ( localPainter ) {
-	p->end();
+    if ( localPainter )
 	delete p;
-    }
 }
 
 void QTetrixBoard::drawNextSquare( int x, int y, int value )
