@@ -1,16 +1,22 @@
-//
-// Qt Example Application: Tic-Tac-Toe
-//
+/****************************************************************************
+** $Id: tictac.cpp,v 1.2 1999/06/03 01:45:57 warwick Exp $
+**
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
+**
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
+**
+*****************************************************************************/
 
 #include "tictac.h"
-#include <qapp.h>
+#include <qapplication.h>
 #include <qpainter.h>
-#include <qdrawutl.h>
-#include <qcombo.h>
-#include <qchkbox.h>
+#include <qdrawutil.h>
+#include <qcombobox.h>
+#include <qcheckbox.h>
 #include <qlabel.h>
 #include <stdlib.h>				// rand() function
-#include <qdatetm.h>				// seed for rand()
+#include <qdatetime.h>				// seed for rand()
 
 
 //***************************************************************************
@@ -23,7 +29,6 @@
 
 TicTacButton::TicTacButton( QWidget *parent ) : QButton( parent )
 {
-    initMetaObject();				// initialize meta object
     setBackgroundColor( blue );			// special background color
     t = Blank;					// initial type
 }
@@ -60,8 +65,6 @@ void TicTacButton::drawButton( QPainter *p )
 TicTacGameBoard::TicTacGameBoard( int n, QWidget *parent, const char *name )
     : QWidget( parent, name )
 {
-    initMetaObject();				// initialize meta object
-    setBackgroundColor( lightGray );		// set background color
     st = Init;					// initial state
     nBoard = n;
     n *= n;					// make square
@@ -266,7 +269,7 @@ void TicTacGameBoard::computerMove()
     }
     *btArray = a;				// update model
     updateButtons();				// update buttons
-    delete altv;
+    delete[] altv;
 }
 
 
@@ -304,8 +307,6 @@ void TicTacGameBoard::resizeEvent( QResizeEvent * )
 TicTacToe::TicTacToe( int boardSize, QWidget *parent, const char *name )
     : QWidget( parent, name )
 {
-    initMetaObject();				// initialize meta object
-    setBackgroundColor( lightGray );		// set background color
     resize( 200, 300 );				// resize this widget
 
 // Create the game board and connect the signal finished() to this
