@@ -1,22 +1,24 @@
 /****************************************************************************
-** $Id: main.cpp,v 1.1 1997/03/24 17:17:59 hanord Exp $
+** $Id: main.cpp,v 1.2 1999/06/03 01:46:02 warwick Exp $
 **
-** Ritual main() for Qt applications
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** Copyright (C) 1996 by Troll Tech AS.  All rights reserved.
+** This file is part of an example program for Qt.  This example
+** program may be used, distributed and modified without limitation.
 **
 *****************************************************************************/
 
-#include <qapp.h>
+#include <qapplication.h>
 #include "tooltip.h"
 
-RCSTAG("$Id: main.cpp,v 1.1 1997/03/24 17:17:59 hanord Exp $");
-
-int main( int argc, char ** argv ) {
+int main( int argc, char ** argv ) 
+{
     QApplication a( argc, argv );
+
     TellMe mw;
-    a.setMainWidget( &mw );
     mw.setCaption( "Dynamic Tool Tip Demonstration" );
+    a.setMainWidget( &mw );
     mw.show();
+    
     return a.exec();
 }
