@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id: motor.cpp,v 1.9 1998/06/16 11:39:35 warwick Exp $
+** $Id: motor.cpp,v 1.3 1999/06/21 13:58:46 reggie Exp $
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -11,7 +11,6 @@
 #include "motor.h"
 #include "qlineedit.h"
 #include "qpushbutton.h"
-#include <math.h>
 
 MotorValidator::MotorValidator( QSpinBox * parent, const char * name )
     : QValidator( parent, name )
@@ -41,14 +40,14 @@ void MotorValidator::setRange( int bottom, int top, int step )
 // integer in the range b to t which can be described as n*s+b for
 // some integer n.
 
-QValidator::State MotorValidator::validate( QString & motorSize, int & )
+QValidator::State MotorValidator::validate( QString & motorSize, int & ) const
 {
     bool ok;
     long int tmp = motorSize.toLong( &ok );
     if ( !ok )
-	return QValidator::Invalid;
+        return QValidator::Invalid;
     else if ( tmp < b || tmp > t || ((tmp-b)%s) != 0 )
-	return QValidator::Valid;
+        return QValidator::Valid;
     else
-	return QValidator::Acceptable;
+        return QValidator::Acceptable;
 }
