@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: globjwin.cpp,v 1.4 1998/06/16 11:39:36 warwick Exp $
+** $Id: globjwin.cpp,v 1.6 1999/01/22 08:01:00 aavit Exp $
 **
 ** Implementation of GLObjectWindow widget class
 **
@@ -21,57 +21,53 @@
 GLObjectWindow::GLObjectWindow( QWidget* parent, const char* name )
     : QWidget( parent, name )
 {
-    // Create top-level layout manager
-    QHBoxLayout* hlayout = new QHBoxLayout( this, 20, 20, "hlayout");
 
     // Create a menu
-    QPopupMenu *file = new QPopupMenu();
+    QPopupMenu *file = new QPopupMenu( this );
     file->insertItem( "Exit",  qApp, SLOT(quit()), CTRL+Key_Q );
 
     // Create a menu bar
     QMenuBar *m = new QMenuBar( this );
     m->setSeparator( QMenuBar::InWindowsStyle );
     m->insertItem("&File", file );
-    hlayout->setMenuBar( m );
 
-    // Create a layout manager for the sliders
-    QVBoxLayout* vlayout = new QVBoxLayout( 20, "vlayout");
-    hlayout->addLayout( vlayout );
-
-    // Create a nice frame tp put around the openGL widget
+    // Create a nice frame to put around the OpenGL widget
     QFrame* f = new QFrame( this, "frame" );
     f->setFrameStyle( QFrame::Sunken | QFrame::Panel );
     f->setLineWidth( 2 );
-    hlayout->addWidget( f, 1 );
 
-    // Create a layout manager for the openGL widget
-    QHBoxLayout* flayout = new QHBoxLayout( f, 2, 2, "flayout");
-
-    // Create an openGL widget
+    // Create our OpenGL widget
     GLBox* c = new GLBox( f, "glbox");
-    c->setMinimumSize( 50, 50 );
-    flayout->addWidget( c, 1 );
-    flayout->activate();
 
     // Create the three sliders; one for each rotation axis
     QSlider* x = new QSlider ( 0, 360, 60, 0, QSlider::Vertical, this, "xsl" );
     x->setTickmarks( QSlider::Left );
-    x->setMinimumSize( x->sizeHint() );
-    vlayout->addWidget( x );
     QObject::connect( x, SIGNAL(valueChanged(int)),c,SLOT(setXRotation(int)) );
 
     QSlider* y = new QSlider ( 0, 360, 60, 0, QSlider::Vertical, this, "ysl" );
     y->setTickmarks( QSlider::Left );
-    y->setMinimumSize( y->sizeHint() );
-    vlayout->addWidget( y );
     QObject::connect( y, SIGNAL(valueChanged(int)),c,SLOT(setYRotation(int)) );
 
     QSlider* z = new QSlider ( 0, 360, 60, 0, QSlider::Vertical, this, "zsl" );
     z->setTickmarks( QSlider::Left );
-    z->setMinimumSize( z->sizeHint() );
-    vlayout->addWidget( z );
     QObject::connect( z, SIGNAL(valueChanged(int)),c,SLOT(setZRotation(int)) );
 
-    // Start the geometry management
-    hlayout->activate();
+
+    // Now that we have all the widgets, put them into a nice layout
+
+    // Put the sliders on top of each other
+    QVBoxLayout* vlayout = new QVBoxLayout( 20, "vlayout");
+    vlayout->addWidget( x );
+    vlayout->addWidget( y );
+    vlayout->addWidget( z );
+
+    // Put the GL widget inside the frame
+    QHBoxLayout* flayout = new QHBoxLayout( f, 2, 2, "flayout");
+    flayout->addWidget( c, 1 );
+
+    // Top level layout, puts the sliders to the left of the frame/GL widget
+    QHBoxLayout* hlayout = new QHBoxLayout( this, 20, 20, "hlayout");
+    hlayout->setMenuBar( m );
+    hlayout->addLayout( vlayout );
+    hlayout->addWidget( f, 1 );
 }
