@@ -9,7 +9,8 @@
 // 
 
 #include "globjwin.h"
-#include <qapp.h>
+#include <qapplication.h>
+#include <qgl.h>
 
 /*
   The main program is here. 
@@ -17,8 +18,13 @@
 
 int main( int argc, char **argv )
 {
-    QApplication::setColorSpec( QApplication::ManyColor );
-    QApplication a(argc,argv);			
+    QApplication::setColorSpec( QApplication::CustomColor );
+    QApplication a(argc,argv);
+
+    if ( !QGLFormat::hasOpenGL() ) {
+	qWarning( "This system has no OpenGL support. Exiting." );
+	return -1;
+    }
 
     GLObjectWindow w;
     w.resize( 400, 350 );
