@@ -6,9 +6,13 @@
 //
 
 #include <qgl.h>
-#include <qapp.h>
+#include <qapplication.h>
 #include <math.h>
 
+#if defined(_CC_MSVC_)
+#pragma warning(disable:4305) // init: truncation from const double to float
+#endif
+
 /*
  * Draw a gear wheel.  You'll probably want to call this function when
  * building a display list since we do a lot of trig here.
@@ -145,6 +149,7 @@ static GLfloat angle = 0.0;
 static void draw()
 {
     angle += 2.0;
+    view_roty += 1.0;
 
     glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
 
@@ -166,8 +171,9 @@ static void draw()
     glPopMatrix();
 
     glPushMatrix();
-    glTranslatef( -3.1, 4.2, 0.0 );
-    glRotatef( -2.0*angle-25.0, 0.0, 0.0, 1.0 );
+    glTranslatef( -3.1, 2.2, -1.8 );
+    glRotatef( 90.0, 1.0, 0.0, 0.0 );
+    glRotatef( 2.0*angle-2.0, 0.0, 0.0, 1.0 );
     glCallList(gear3);
     glPopMatrix();
 
@@ -175,68 +181,13 @@ static void draw()
 }
 
 
-static void init( int width, int height )
-{
-    static GLfloat pos[4] = {5.0, 5.0, 10.0, 1.0 };
-    static GLfloat red[4] = {0.8, 0.1, 0.0, 1.0 };
-    static GLfloat green[4] = {0.0, 0.8, 0.2, 1.0 };
-    static GLfloat blue[4] = {0.2, 0.2, 1.0, 1.0 };
-
-    GLfloat w = (float) width / (float) height;
-    GLfloat h = 1.0;
-
-    glLightfv( GL_LIGHT0, GL_POSITION, pos );
-    glEnable( GL_CULL_FACE );
-    glEnable( GL_LIGHTING );
-    glEnable( GL_LIGHT0 );
-    glEnable( GL_DEPTH_TEST );
-
-    static bool first_time = TRUE;
-    if ( first_time ) {
-	first_time = FALSE;
-    } else {
-	glDeleteLists( gear1, 1 );
-	glDeleteLists( gear2, 1 );
-	glDeleteLists( gear3, 1 );
-    }
-
-    /* make the gears */
-    gear1 = glGenLists(1);
-    glNewList(gear1, GL_COMPILE);
-    glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, red );
-    gear( 1.0, 4.0, 1.0, 20, 0.7 );
-    glEndList();
-
-    gear2 = glGenLists(1);
-    glNewList(gear2, GL_COMPILE);
-    glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, green );
-    gear( 0.5, 2.0, 2.0, 10, 0.7 );
-    glEndList();
-
-    gear3 = glGenLists(1);
-    glNewList(gear3, GL_COMPILE);
-    glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, blue );
-    gear( 1.3, 2.0, 0.5, 10, 0.7 );
-    glEndList();
-
-    glEnable( GL_NORMALIZE );
-
-    glViewport( 0, 0, width, height );
-    glMatrixMode(GL_PROJECTION);
-    glLoadIdentity();
-    glFrustum( -w, w, -h, h, 5.0, 60.0 );
-    glMatrixMode(GL_MODELVIEW);
-    glLoadIdentity();
-    glTranslatef( 0.0, 0.0, -40.0 );
-}
-
-
 
 class GearWidget : public QGLWidget
 {
 public:
     GearWidget( QWidget *parent=0, const char *name=0 );
 protected:
+    void initializeGL();
     void resizeGL( int, int );
     void paintGL();
     void timerEvent( QTimerEvent * );
@@ -248,11 +199,57 @@ GearWidget::GearWidget( QWidget *parent, const char *name )
     startTimer( 10 );
 }
 
+void GearWidget::initializeGL()
+{
+    static GLfloat pos[4] = {5.0, 5.0, 10.0, 1.0 };
+    static GLfloat ared[4] = {0.8, 0.1, 0.0, 1.0 };
+    static GLfloat agreen[4] = {0.0, 0.8, 0.2, 1.0 };
+    static GLfloat ablue[4] = {0.2, 0.2, 1.0, 1.0 };
+
+    glLightfv( GL_LIGHT0, GL_POSITION, pos );
+    glEnable( GL_CULL_FACE );
+    glEnable( GL_LIGHTING );
+    glEnable( GL_LIGHT0 );
+    glEnable( GL_DEPTH_TEST );
+
+    /* make the gears */
+    gear1 = glGenLists(1);
+    glNewList(gear1, GL_COMPILE);
+    glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, ared );
+    gear( 1.0, 4.0, 1.0, 20, 0.7 );
+    glEndList();
+
+    gear2 = glGenLists(1);
+    glNewList(gear2, GL_COMPILE);
+    glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, agreen );
+    gear( 0.5, 2.0, 2.0, 10, 0.7 );
+    glEndList();
+
+    gear3 = glGenLists(1);
+    glNewList(gear3, GL_COMPILE);
+    glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, ablue );
+    gear( 1.3, 2.0, 0.5, 10, 0.7 );
+    glEndList();
+
+    glEnable( GL_NORMALIZE );
+}
+
+
 void GearWidget::resizeGL( int width, int height )
 {
-    init( width, height );
+    GLfloat w = (float) width / (float) height;
+    GLfloat h = 1.0;
+
+    glViewport( 0, 0, width, height );
+    glMatrixMode(GL_PROJECTION);
+    glLoadIdentity();
+    glFrustum( -w, w, -h, h, 5.0, 60.0 );
+    glMatrixMode(GL_MODELVIEW);
+    glLoadIdentity();
+    glTranslatef( 0.0, 0.0, -40.0 );
 }
 
+
 void GearWidget::paintGL()
 {
     draw();
@@ -269,6 +266,12 @@ int main( int argc, char **argv )
 {
     QApplication::setColorSpec( QApplication::CustomColor );
     QApplication a( argc, argv );
+
+    if ( !QGLFormat::hasOpenGL() ) {
+	qWarning( "This system has no OpenGL support. Exiting." );
+	return -1;
+    }
+
     GearWidget w;
     a.setMainWidget( &w );
     w.show();
