@@ -1,42 +1,50 @@
 /****************************************************************************
-** $Id: qcursor.h,v 2.2 1997/08/06 15:05:35 hanord Exp $
+** $Id: qcursor.h,v 2.16 1999/04/28 09:55:56 aavit Exp $
 **
 ** Definition of QCursor class
 **
 ** Created : 940219
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #ifndef QCURSOR_H
 #define QCURSOR_H
 
+#ifndef QT_H
 #include "qpoint.h"
 #include "qshared.h"
+#include "qnamespace.h"
+#endif // QT_H
 
 
 struct QCursorData;				// internal cursor data
 
 
-class QCursor					// cursor class
+class Q_EXPORT QCursor				// cursor class
 {
 public:
     QCursor();					// create default arrow cursor
     QCursor( int shape );
     QCursor( const QBitmap &bitmap, const QBitmap &mask,
 	     int hotX=-1, int hotY=-1 );
+    QCursor( const QPixmap &pixmap,
+	     int hotX=-1, int hotY=-1 );
     QCursor( const QCursor & );
    ~QCursor();
     QCursor &operator=( const QCursor & );
@@ -48,7 +56,11 @@ public:
     const QBitmap *mask()   const;
     QPoint	  hotSpot() const;
 
+#if defined(_WS_WIN_)
+    HCURSOR	  handle()  const;
+#elif defined(_WS_X11_)
     HANDLE	  handle()  const;
+#endif
 
     static QPoint pos();
     static void	  setPos( int x, int y );
@@ -58,8 +70,11 @@ public:
     static void	  cleanup();
 
 private:
+    void	  setBitmap( const QBitmap &bitmap, const QBitmap &mask,
+				 int hotX, int hotY );
     void	  update() const;
     QCursorData	 *data;
+    QCursor	 *find_cur(int);
 };
 
 
@@ -76,32 +91,16 @@ inline void QCursor::setPos( const QPoint &p )
 enum QCursorShape {
     ArrowCursor, UpArrowCursor, CrossCursor, WaitCursor, IbeamCursor,
     SizeVerCursor, SizeHorCursor, SizeBDiagCursor, SizeFDiagCursor,
-    SizeAllCursor, BlankCursor, LastCursor=BlankCursor, BitmapCursor=24 };
-
-
-/*****************************************************************************
-  Global cursors
- *****************************************************************************/
-
-extern const QCursor arrowCursor;		// standard arrow cursor
-extern const QCursor upArrowCursor;		// upwards arrow
-extern const QCursor crossCursor;		// crosshair
-extern const QCursor waitCursor;		// hourglass/watch
-extern const QCursor ibeamCursor;		// ibeam/text entry
-extern const QCursor sizeVerCursor;		// vertical resize
-extern const QCursor sizeHorCursor;		// horizontal resize
-extern const QCursor sizeBDiagCursor;		// diagonal resize (/)
-extern const QCursor sizeFDiagCursor;		// diagonal resize (\)
-extern const QCursor sizeAllCursor;		// all directions resize
-extern const QCursor blankCursor;		// blank/invisible cursor
+    SizeAllCursor, BlankCursor, SplitVCursor, SplitHCursor, PointingHandCursor,
+    LastCursor=PointingHandCursor, BitmapCursor=24 };
 
 
 /*****************************************************************************
   QCursor stream functions
  *****************************************************************************/
 
-QDataStream &operator<<( QDataStream &, const QCursor & );
-QDataStream &operator>>( QDataStream &, QCursor & );
+Q_EXPORT QDataStream &operator<<( QDataStream &, const QCursor & );
+Q_EXPORT QDataStream &operator>>( QDataStream &, QCursor & );
 
 
 #endif // QCURSOR_H
