@@ -1,22 +1,25 @@
 /****************************************************************************
-** $Id: qglobal.h,v 2.33 1997/10/09 13:55:22 hanord Exp $
+** $Id: qglobal.h,v 2.95.2.3 1999/09/07 07:43:36 hanord Exp $
 **
 ** Global type declarations and definitions
 **
 ** Created : 920529
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -24,8 +27,8 @@
 #define QGLOBAL_H
 
 
-#define QT_VERSION	131
-#define QT_VERSION_STR	"1.31"
+#define QT_VERSION	202
+#define QT_VERSION_STR	"2.0.2"
 
 
 //
@@ -35,7 +38,7 @@
 //   MSDOS	- MS-DOS and Windows
 //   OS2	- OS/2
 //   OS2EMX	- XFree86 on OS/2 (not PM)
-//   WIN32	- Win32 (Windows 95 and Windows NT)
+//   WIN32	- Win32 (Windows 95/98 and Windows NT)
 //   SUN	- SunOS
 //   SOLARIS	- Sun Solaris
 //   HPUX	- HP-UX
@@ -43,11 +46,15 @@
 //   LINUX	- Linux
 //   FREEBSD	- FreeBSD
 //   NETBSD	- NetBSD
+//   OPENBSD    - OpenBSD
 //   IRIX	- SGI Irix
 //   OSF	- OSF Unix
 //   BSDI	- BSDI Unix
 //   SCO	- SCO of some sort
 //   AIX	- AIX Unix
+//   UNIXWARE	- SCO UnixWare
+//   GNU	- GNU Hurd
+//   DGUX	- DG Unix
 //   UNIX	- Any UNIX bsd/sysv system
 //
 
@@ -63,6 +70,8 @@
 #endif
 #elif defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
 #define _OS_WIN32_
+#elif defined(__MWERKS__) && defined(__INTEL__)
+#define _OS_WIN32_
 #elif defined(sun) || defined(__sun) || defined(__sun__)
 #define _OS_SUN_
 #if defined(_OS_SUN_) && defined(__SVR4)
@@ -79,16 +88,32 @@
 #define _OS_FREEBSD_
 #elif defined(__NetBSD__)
 #define _OS_NETBSD_
+#elif defined(__OpenBSD__)
+#define _OS_OPENBSD_
 #elif defined(sgi) || defined(__sgi)
 #define _OS_IRIX_
 #elif defined(__osf__)
 #define _OS_OSF_
 #elif defined(bsdi) || defined(__bsdi__)
 #define _OS_BSDI_
-#elif defined(_SCO_DS) || defined(M_UNIX) || defined(M_XENIX)
-#define _OS_SCO_
 #elif defined(_AIX)
 #define _OS_AIX_
+#elif defined(__Lynx__)
+#define _OS_LYNXOS_
+#elif defined(_UNIXWARE)
+#define _OS_UNIXWARE_
+#elif defined(__GNU__)
+#define _OS_GNU_
+#elif defined(DGUX)
+#define _OS_DGUX_
+#elif defined(__QNX__)
+#define _OS_QNX_
+#elif defined(_SCO_DS) || defined(M_UNIX) || defined(M_XENIX)
+#define _OS_SCO_
+#elif defined(sco) || defined(_UNIXWARE7)
+#define _OS_UNIXWARE7_
+#elif !defined(_SCO_DS) && defined(__USLC__) && defined(__SCO_VERSION__)
+#define _OS_UNIXWARE7_
 #else
 #error "Qt has not been ported to this OS - talk to qt-bugs@troll.no"
 #endif
@@ -106,6 +131,7 @@
 //
 //   SYM	- Symantec C++ for both PC and Macintosh
 //   MPW	- MPW C++
+//   MWERKS	- Metroworks CodeWarrior
 //   MSVC	- Microsoft Visual C/C++
 //   BOR	- Borland/Turbo C++
 //   WAT	- Watcom C++
@@ -116,12 +142,18 @@
 //   SUN	- Sun C++
 //   DEC	- DEC C++
 //   HP		- HPUX C++
+//   USLC	- SCO UnixWare C++
 //
 
+
+// Should be sorted most-authorative to least-authorative
 #if defined(__SC__)
 #define _CC_SYM_
 #elif defined(applec)
 #define _CC_MPW_
+#elif defined(__MWERKS__)
+#define _CC_MWERKS_
+#define HAS_BOOL_TYPE
 #elif defined(_MSC_VER)
 #define _CC_MSVC_
 #elif defined(__BORLANDC__) || defined(__TURBOC__)
@@ -135,6 +167,11 @@
 #elif defined(como40)
 #define _CC_EDG_
 #define _CC_COMEAU_
+#elif defined(__USLC__)
+#define _CC_USLC_
+#ifdef __EDG__ // UnixWare7
+#define HAS_BOOL_TYPE
+#endif
 #elif defined(__EDG) || defined(__EDG__)
 // one observed on SGI DCC, the other documented
 #define _CC_EDG_
@@ -142,16 +179,21 @@
 #define _CC_OC_
 #elif defined(__SUNPRO_CC)
 #define _CC_SUN_
+#if __SUNPRO_CC >= 0x500
+#define HAS_BOOL_TYPE
+#define Q_SPARCWORKS_FUNCP_BUG
+#endif
 #elif defined(__DECCXX)
 #define _CC_DEC_
 #elif defined(_OS_HPUX_)
-// this test from from aCC online help
+// this test is from aCC online help
 #if __cplusplus >= 199707L
 // this is the aCC
-#define CC_HP_ACC_
+#define _CC_HP_ACC_
+#define HAS_BOOL_TYPE
 #else
 // this is the CC
-#define CC_HP_
+#define _CC_HP_
 #endif // __cplusplus >= 199707L
 #else
 #error "Qt has not been tested with this compiler - talk to qt-bugs@troll.no"
@@ -161,46 +203,64 @@
 #define Q_C_CALLBACKS
 #endif
 
+// Window system setting
 
-//
-// Specify to use macro or template classes (if not set from cmd line)
-//
-
-#define USE_MACROCLASS				/* always use macro classes */
-#define USE_TEMPLATECLASS			/* use template classes */
-
-
-//
-// Some compilers do not support templates
-//
-
-#if defined(_CC_MPW_) || (defined(_CC_MSVC_) && _MSC_VER < 900)
-#define NO_TEMPLATECLASS
+#if defined(_OS_MAC_)
+#define _WS_MAC_
+#elif defined(_OS_MSDOS_)
+#define _WS_WIN16_
+#error "Qt requires Win32 and does not work with Windows 3.x"
+#elif defined(_WIN32_X11_)
+#define _WS_X11_
+#elif defined(_OS_WIN32_)
+#define _WS_WIN32_
+#elif defined(_OS_OS2_)
+#error "Qt does not work with OS/2 Presentation Manager or Workplace Shell"
+#elif defined(UNIX)
+#define _WS_X11_
 #endif
 
-#if defined(NO_TEMPLATECLASS)
-#undef	USE_TEMPLATECLASS			/* templates not wanted */
+#if defined(_WS_WIN16_) || defined(_WS_WIN32_)
+#define _WS_WIN_
 #endif
 
 
 //
-// Smart setting/checking of DEFAULT_ flag
+// Some classes do not permit copies to be made of an object.
+// These classes contains a private copy constructor and operator=
+// to disable copying (the compiler gives an error message).
+// Undefine Q_DISABLE_COPY to turn of this checking.
 //
 
-#if !defined(DEFAULT_MACROCLASS) && !defined(DEFAULT_TEMPLATECLASS)
-#if defined(USE_TEMPLATECLASS)
-#define DEFAULT_TEMPLATECLASS
-#else
-#define DEFAULT_MACROCLASS
+#define Q_DISABLE_COPY
+
+//
+// Create Qt DLL if QT_DLL is defined (Windows only)
+//
+
+#if defined(_OS_WIN32_)
+#if defined(QT_NODLL)
+#undef QT_MAKEDLL
+#undef QT_DLL
 #endif
+#if defined(QT_MAKEDLL)		/* create a Qt DLL library */
+#undef QT_DLL
+#define Q_EXPORT  __declspec(dllexport)
+#define Q_TEMPLATEDLL
+#undef  Q_DISABLE_COPY		/* avoid unresolved externals */
+#endif
+#if defined(QT_DLL)		/* use a Qt DLL library */
+#define Q_EXPORT  __declspec(dllimport)
+#define Q_TEMPLATEDLL
+#undef  Q_DISABLE_COPY		/* avoid unresolved externals */
+#endif
+#else // ! _OS_WIN32_
+#undef QT_MAKEDLL		/* ignore these for other platforms */
+#undef QT_DLL
 #endif
 
-#if !defined(USE_TEMPLATECLASS) && defined(DEFAULT_TEMPLATECLASS)
-#error "Can't use templates as default when USE_TEMPLATECLASS is not defined!"
-#endif
-
-#if defined(DEFAULT_MACROCLASS) && defined(DEFAULT_TEMPLATECLASS)
-#error "Define DEFAULT_MACROCLASS or DEFAULT_TEMPLATECLASS, not both!"
+#ifndef Q_EXPORT
+#define Q_EXPORT
 #endif
 
 
@@ -212,22 +272,19 @@
 #define HAS_BOOL_TYPE
 #elif __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 6)
 #define HAS_BOOL_TYPE
-#elif __BORLANDC__ >= 0x500
+#elif _MSC_VER >= 1100 || __BORLANDC__ >= 0x500
 #define HAS_BOOL_TYPE
 #elif defined(_CC_COMEAU_)
 #define HAS_BOOL_TYPE
-#elif defined(_CC_HP_ACC_)
+#elif defined(sgi) && ( (_COMPILER_VERSION >= 710) || defined(_BOOL) )
 #define HAS_BOOL_TYPE
-#elif defined(sgi) && (_COMPILER_VERSION >= 710)
+#elif defined(__DECCXX) && (__DECCXX_VER >= 60060005)
 #define HAS_BOOL_TYPE
 #endif
 
-#if defined(_CC_MSVC_)
-#define _CC_BOOL_DEF_
-#endif
-
 #if !defined(HAS_BOOL_TYPE)
-#if defined(_CC_BOOL_DEF_)
+#if defined(_CC_MSVC_)
+#define _CC_BOOL_DEF_
 #define bool		int
 #else
 typedef int		bool;
@@ -253,6 +310,17 @@ const bool TRUE = !0;
 #endif
 
 
+#if defined(_CC_MSVC_)
+// Workaround for static const members.
+#define QT_STATIC_CONST static
+#define QT_STATIC_CONST_IMPL
+#else
+#define QT_STATIC_CONST static const
+#define QT_STATIC_CONST_IMPL const
+#endif
+
+
+
 //
 // Utility macros and inline functions
 //
@@ -271,8 +339,10 @@ inline int qRound( double d )
 // Size-dependent types (architechture-dependent byte order)
 //
 
-#if !defined(QT_CLEAN_NAMESPACE) 
-typedef char		INT8;			// 8 bit signed
+// QT_CLEAN_NAMESPACE is not defined by default; it would break too
+// much code.
+#if !defined(QT_CLEAN_NAMESPACE)
+typedef signed char	INT8;			// 8 bit signed
 typedef unsigned char	UINT8;			// 8 bit unsigned
 typedef short		INT16;			// 16 bit signed
 typedef unsigned short	UINT16;			// 16 bit unsigned
@@ -280,7 +350,7 @@ typedef int		INT32;			// 32 bit signed
 typedef unsigned int	UINT32;			// 32 bit unsigned
 #endif
 
-typedef char		Q_INT8;			// 8 bit signed
+typedef signed char	Q_INT8;			// 8 bit signed
 typedef unsigned char	Q_UINT8;		// 8 bit unsigned
 typedef short		Q_INT16;		// 16 bit signed
 typedef unsigned short	Q_UINT16;		// 16 bit unsigned
@@ -289,7 +359,7 @@ typedef unsigned int	Q_UINT32;		// 32 bit unsigned
 
 
 //
-// Data stream functions is provided by many classes (defined in qdstream.h)
+// Data stream functions is provided by many classes (defined in qdatastream.h)
 //
 
 class QDataStream;
@@ -299,8 +369,8 @@ class QDataStream;
 // System information
 //
 
-const char *qVersion();
-bool qSysInfo( int *wordSize, bool *bigEndian );
+Q_EXPORT const char *qVersion();
+Q_EXPORT bool qSysInfo( int *wordSize, bool *bigEndian );
 
 
 //
@@ -319,8 +389,9 @@ bool qSysInfo( int *wordSize, bool *bigEndian );
 #endif
 
 //
-// To get C++ compiler warnings, define CC_WARNINGS or comment out the line
-// "#define NO_WARNINGS"
+// Avoid some particularly useless warnings from some stupid compilers.
+// To get ALL C++ compiler warnings, define CC_WARNINGS or comment out
+// the line "#define NO_WARNINGS"
 //
 
 #if !defined(CC_WARNINGS)
@@ -329,83 +400,99 @@ bool qSysInfo( int *wordSize, bool *bigEndian );
 #if defined(NO_WARNINGS)
 #if defined(_CC_MSVC_)
 #pragma warning(disable: 4244)
+#pragma warning(disable: 4275)
+#pragma warning(disable: 4514)
+#pragma warning(disable: 4800)
 #elif defined(_CC_BOR_)
+#pragma option -w-inl
 #pragma warn -inl
 #pragma warn -pia
 #pragma warn -ccc
 #pragma warn -rch
 #pragma warn -sig
+#elif defined(_CC_MWERKS_)
+#pragma warn_possunwant off
 #endif
 #endif // NO_WARNINGS
 
-//
-// Avoid compiler warning "argument defined but not used"
-//
-
-#if !defined(NOT_USED)
-#define NOT_USED(argument) argument=argument;
-#endif
-
 //
 // Avoid dead code
 //
 
 #if defined(_CC_EDG_) || defined(_CC_WAT_)
-#define NO_DEADCODE
+#define Q_NO_DEAD_CODE
 #endif
 
 //
-// Avoid compiler warning "function defined but not used" (gcc only)
+// Use to avoid "unused parameter" warnings
 //
 
-#if defined(NOT_USED_FN)
-// user provided
-#elif __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7)
-#define NOT_USED_FN __attribute__ ((unused))
-#else
-#define NOT_USED_FN
-#endif
+#define Q_UNUSED(x) x=x;
 
-//
-// Avoid "char ident[54] defined but not used" for the RCS tag
-//
 
-#if !defined(DEBUG)
-#define RCSTAG(string)
-#elif __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7)
-#define RCSTAG(string) static char ident[] __attribute__ ((unused)) = string
-#elif defined(_CC_DEC_) || defined(_CC_COMEAU_)
-#define RCSTAG(string) typedef int QUnusedSemicolon
-#else
-#define RCSTAG(string)
-#endif
-
-void debug( const char *, ... )			// print debug message
+Q_EXPORT void qDebug( const char *, ... )	// print debug message
 #if defined(_CC_GNU_)
     __attribute__ ((format (printf, 1, 2)))
 #endif
 ;
 
-void warning( const char *, ... )		// print warning message
+Q_EXPORT void qWarning( const char *, ... )	// print warning message
 #if defined(_CC_GNU_)
     __attribute__ ((format (printf, 1, 2)))
 #endif
 ;
 
-void fatal( const char *, ... )			// print fatal message and exit
+Q_EXPORT void qFatal( const char *, ... )	// print fatal message and exit
 #if defined(_CC_GNU_)
     __attribute__ ((format (printf, 1, 2)))
 #endif
 ;
 
 
+// QT_CLEAN_NAMESPACE is not defined by default; it would break too
+// much code.
+#if !defined(QT_CLEAN_NAMESPACE)
+// in that case, also define the old ones...
+
+Q_EXPORT void debug( const char *, ... )	// print debug message
+#if defined(_CC_GNU_)
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
+
+Q_EXPORT void warning( const char *, ... )	// print warning message
+#if defined(_CC_GNU_)
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
+
+Q_EXPORT void fatal( const char *, ... )	// print fatal message and exit
+#if defined(_CC_GNU_)
+    __attribute__ ((format (printf, 1, 2)))
+#endif
+;
+
+// okay, that was debug()/warning()/fatal()
+#endif
+
+#if !defined(ASSERT)
+#if defined(CHECK_STATE)
+#if defined(QT_FATAL_ASSERT)
 #define ASSERT(x)  if ( !(x) )\
-	warning("ASSERT: \"%s\" in %s (%d)",#x,__FILE__,__LINE__)
+	qFatal("ASSERT: \"%s\" in %s (%d)",#x,__FILE__,__LINE__)
+#else
+#define ASSERT(x)  if ( !(x) )\
+	qWarning("ASSERT: \"%s\" in %s (%d)",#x,__FILE__,__LINE__)
+#endif
+#else
+#define ASSERT(x)
+#endif
+#endif
 
-bool chk_pointer( bool c, const char *, int );	// fatal error if c is TRUE
+Q_EXPORT bool qt_check_pointer( bool c, const char *, int );
 
 #if defined(CHECK_NULL)
-#define CHECK_PTR(p) (chk_pointer((p)==0,__FILE__,__LINE__))
+#define CHECK_PTR(p) (qt_check_pointer((p)==0,__FILE__,__LINE__))
 #else
 #define CHECK_PTR(p)
 #endif
@@ -413,24 +500,21 @@ bool chk_pointer( bool c, const char *, int );	// fatal error if c is TRUE
 enum QtMsgType { QtDebugMsg, QtWarningMsg, QtFatalMsg };
 
 typedef void (*msg_handler)(QtMsgType, const char *);
-msg_handler qInstallMsgHandler( msg_handler );	// install message handler
+Q_EXPORT msg_handler qInstallMsgHandler( msg_handler );
 
 
-//
-// Enable QT_ADD_GENERIC_MACROS unless QT_REJECT_OBSOLETE is defined
-//
-#if !defined(QT_REJECT_OBSOLETE)
-#define QT_ADD_GENERIC_MACROS
-#endif
-
-void qSuppressObsoleteWarnings( bool = TRUE );
+Q_EXPORT void qSuppressObsoleteWarnings( bool = TRUE );
 
 #if !defined(QT_REJECT_OBSOLETE)
 #define QT_OBSOLETE
-void qObsolete( const char *obj, const char *oldfunc, const char *newfunc );
-void qObsolete( const char *obj, const char *oldfunc );
-void qObsolete( const char *message );
+Q_EXPORT void qObsolete( const char *obj, const char *oldfunc,
+			 const char *newfunc );
+Q_EXPORT void qObsolete( const char *obj, const char *oldfunc );
+Q_EXPORT void qObsolete( const char *message );
 #endif
 
+#ifdef _WS_WIN_
+extern bool qt_winunicode;
+#endif
 
 #endif // QGLOBAL_H
