@@ -1,34 +1,38 @@
 /****************************************************************************
-** $Id: qmenubar.h,v 2.7 1997/09/05 20:00:43 agulbra Exp $
+** $Id: qmenubar.h,v 2.24 1999/06/23 17:53:28 ettrich Exp $
 **
 ** Definition of QMenuBar class
 **
 ** Created : 941209
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #ifndef QMENUBAR_H
 #define QMENUBAR_H
 
-#include "qpopmenu.h"
+#ifndef QT_H
+#include "qpopupmenu.h"
+#endif // QT_H
 
 
-class QMenuBar : public QFrame, public QMenuData
+class Q_EXPORT QMenuBar : public QFrame, public QMenuData
 {
-friend class QPopupMenu;
     Q_OBJECT
 public:
     QMenuBar( QWidget *parent=0, const char *name=0 );
@@ -45,7 +49,9 @@ public:
 
     enum	Separator { Never=0, InWindowsStyle=1 };
     Separator 	separator() const;
-    void	setSeparator( Separator when );
+    virtual void	setSeparator( Separator when );
+
+    bool customWhatsThis() const;
 
 signals:
     void	activated( int itemId );
@@ -58,10 +64,12 @@ protected:
     void	mouseReleaseEvent( QMouseEvent * );
     void	mouseMoveEvent( QMouseEvent * );
     void	keyPressEvent( QKeyEvent * );
+    void	focusOutEvent( QFocusEvent * );
     void	resizeEvent( QResizeEvent * );
     void	leaveEvent( QEvent * );
     void	menuContentsChanged();
     void	menuStateChanged();
+    void 	styleChange( QStyle& );
 
 private slots:
     void	subActivated( int itemId );
@@ -80,19 +88,28 @@ private:
     void	openActPopup();
     void	hidePopups();
 
-    void	setActItem( int, bool );
-    void	setWindowsAltMode( bool, int = 0 );
+    virtual void	setActItem( int, bool );
+    virtual void	setWindowsAltMode( bool, int = 0 );
 
     int		calculateRects( int max_width = -1 );
     int		itemAtPos( const QPoint & );
     QRect	itemRect( int item );
 
+    void	setupAccelerators();
     QAccel     *autoaccel;
     QRect      *irects;
+    int		rightSide;
+
+    uint	mseparator	: 1;
+    uint	windowsaltactive : 1;
+
+    friend class QPopupMenu;
 
 private:	// Disabled copy constructor and operator=
-    QMenuBar( const QMenuBar & ) {}
-    QMenuBar &operator=( const QMenuBar & ) { return *this; }
+#if defined(Q_DISABLE_COPY)
+    QMenuBar( const QMenuBar & );
+    QMenuBar &operator=( const QMenuBar & );
+#endif
 };
 
 
