@@ -1,46 +1,52 @@
 /****************************************************************************
-** $Id: qregion.h,v 2.5 1997/10/09 13:11:05 agulbra Exp $
+** $Id: qregion.h,v 2.28 1999/05/29 17:19:46 warwick Exp $
 **
 ** Definition of QRegion class
 **
 ** Created : 940514
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #ifndef QREGION_H
 #define QREGION_H
 
+#ifndef QT_H
 #include "qshared.h"
 #include "qrect.h"
 #include "qstring.h"
+#endif // QT_H
 
 
-class QRegion
+class Q_EXPORT QRegion
 {
 public:
     enum RegionType { Rectangle, Ellipse };
 
     QRegion();
+    QRegion( int x, int y, int w, int h, RegionType = Rectangle );
     QRegion( const QRect &, RegionType = Rectangle );
     QRegion( const QPointArray &, bool winding=FALSE );
     QRegion( const QRegion & );
+    QRegion( const QBitmap & );
    ~QRegion();
     QRegion &operator=( const QRegion & );
 
-
     bool    isNull()   const;
     bool    isEmpty()  const;
 
@@ -52,39 +58,34 @@ public:
     QRegion unite( const QRegion & )	const;
     QRegion intersect( const QRegion &) const;
     QRegion subtract( const QRegion & ) const;
-
-// Work around clash with the ANSI C++ keyword "xor".
-//
-// Use of QRegion::xor() is deprecated - you should use QRegion::eor().
-// Calls to QRegion::xor() will work for now, but give a warning.
-//
-// If possible, compile the Qt library without this ANSI C++ feature enabled,
-// thus including both the old xor() and new eor() in the library, so old
-// binaries will continue to work (with the warning).
-//
-// We also hide the xor() function if there is a #define for xor, in
-// case someone is using #define xor ^ to work around deficiencies in
-// their compiler that cause problems with some other header files.
-//
-#if !(defined(__STRICT_ANSI__) && defined(_CC_GNU_)) && !defined(_CC_EDG_) && !defined(_CC_HP_) && !defined(_CC_HP_ACC_) && !defined(xor)
-    QRegion xor( const QRegion & )	const;
-#endif
     QRegion eor( const QRegion & )	const;
 
+    QRect   boundingRect() const;
+    QArray<QRect> rects() const;
+
+    QRegion operator|( const QRegion & ) const;
+    QRegion operator+( const QRegion & ) const;
+    QRegion operator&( const QRegion & ) const;
+    QRegion operator-( const QRegion & ) const;
+    QRegion operator^( const QRegion & ) const;
+    QRegion& operator|=( const QRegion & );
+    QRegion& operator+=( const QRegion & );
+    QRegion& operator&=( const QRegion & );
+    QRegion& operator-=( const QRegion & );
+    QRegion& operator^=( const QRegion & );
+
     bool    operator==( const QRegion & )  const;
     bool    operator!=( const QRegion &r ) const
 			{ return !(operator==(r)); }
 
 #if defined(_WS_WIN_)
-    HANDLE  handle() const { return data->rgn; }
-#elif defined(_WS_PM_)
-    HANDLE  handle() const { return data->rgn; }
+    HRGN    handle() const { return data->rgn; }
 #elif defined(_WS_X11_)
     Region  handle() const { return data->rgn; }
 #endif
 
-    friend QDataStream &operator<<( QDataStream &, const QRegion & );
-    friend QDataStream &operator>>( QDataStream &, QRegion & );
+    friend Q_EXPORT QDataStream &operator<<( QDataStream &, const QRegion & );
+    friend Q_EXPORT QDataStream &operator>>( QDataStream &, QRegion & );
 
 private:
     QRegion( bool );
@@ -93,21 +94,15 @@ private:
 #if defined(_WS_WIN_)
     QRegion winCombine( const QRegion &, int ) const;
 #endif
-    void    cmd( int id, void *, const QRegion * = 0, const QRegion * = 0 );
-    void    exec( const QByteArray & );
+    void    exec( const QByteArray &, int ver = 0 );
     struct QRegionData : public QShared {
-	QByteArray bop;
 #if defined(_WS_WIN_)
-	HANDLE rgn;
-#elif defined(_WS_PM_)
-	HANDLE rgn;
+	HRGN   rgn;
 #elif defined(_WS_X11_)
 	Region rgn;
 #endif
+	bool   is_null;
     } *data;
-#if defined(_WS_PM_)
-    static HPS hps;
-#endif
 };
 
 
@@ -120,14 +115,15 @@ private:
 #define QRGN_AND		7
 #define QRGN_SUB		8
 #define QRGN_XOR		9
+#define QRGN_RECTS	       10
 
 
 /*****************************************************************************
   QRegion stream functions
  *****************************************************************************/
 
-QDataStream &operator<<( QDataStream &, const QRegion & );
-QDataStream &operator>>( QDataStream &, QRegion & );
+Q_EXPORT QDataStream &operator<<( QDataStream &, const QRegion & );
+Q_EXPORT QDataStream &operator>>( QDataStream &, QRegion & );
 
 
 #endif // QREGION_H
