@@ -1,32 +1,37 @@
 /****************************************************************************
-** $Id: qsignal.h,v 2.1 1997/08/06 15:05:46 hanord Exp $
+** $Id: qsignal.h,v 2.8 1999/04/28 09:57:37 aavit Exp $
 **
 ** Definition of QSignal class
 **
 ** Created : 941201
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #ifndef QSIGNAL_H
 #define QSIGNAL_H
 
+#ifndef QT_H
 #include "qobject.h"
+#endif // QT_H
 
 
-class QSignal : private QObject			// signal class
+class Q_EXPORT QSignal : private QObject			// signal class
 {
 public:
     QSignal( QObject *parent=0, const char *name=0 );
@@ -47,8 +52,10 @@ private:
     Q_OBJECT_FAKE
 
 private:	// Disabled copy constructor and operator=
-    QSignal( const QSignal & ) {}
-    QSignal &operator=( const QSignal & ) { return *this; }
+#if defined(Q_DISABLE_COPY)
+    QSignal( const QSignal & );
+    QSignal &operator=( const QSignal & );
+#endif
 };
 
 
