@@ -1,39 +1,40 @@
 /****************************************************************************
-** $Id: qsize.h,v 2.3 1997/08/27 13:37:46 agulbra Exp $
+** $Id: qsize.h,v 2.14 1999/04/28 09:57:42 aavit Exp $
 **
 ** Definition of QSize class
 **
 ** Created : 931028
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #ifndef QSIZE_H
 #define QSIZE_H
 
+#ifndef QT_H
 #include "qpoint.h"
-
-#if defined(QSIZE_C) || defined(DEBUG)
-#define QSIZE_DEBUG
-#endif
+#endif // QT_H
 
 
-class QSize
+class Q_EXPORT QSize
 {
 public:
-    QSize()	{ wd = ht = -1; }
+    QSize();
     QSize( int w, int h );
 
     bool   isNull()	const;
@@ -46,15 +47,18 @@ public:
     void   setHeight( int h );
     void   transpose();
 
+    QSize expandedTo( const QSize & ) const;
+    QSize boundedTo( const QSize & ) const;
+
     QCOORD &rwidth();
     QCOORD &rheight();
 
     QSize &operator+=( const QSize & );
     QSize &operator-=( const QSize & );
     QSize &operator*=( int c );
-    QSize &operator*=( float c );
+    QSize &operator*=( double c );
     QSize &operator/=( int c );
-    QSize &operator/=( float c );
+    QSize &operator/=( double c );
 
     friend inline bool	operator==( const QSize &, const QSize & );
     friend inline bool	operator!=( const QSize &, const QSize & );
@@ -62,17 +66,14 @@ public:
     friend inline QSize operator-( const QSize &, const QSize & );
     friend inline QSize operator*( const QSize &, int );
     friend inline QSize operator*( int, const QSize & );
-    friend inline QSize operator*( const QSize &, float );
-    friend inline QSize operator*( float, const QSize & );
-#if defined(QSIZE_DEBUG)
-    friend	  QSize operator/( const QSize &, int );
-    friend	  QSize operator/( const QSize &, float );
-#else
+    friend inline QSize operator*( const QSize &, double );
+    friend inline QSize operator*( double, const QSize & );
     friend inline QSize operator/( const QSize &, int );
-    friend inline QSize operator/( const QSize &, float );
-#endif
+    friend inline QSize operator/( const QSize &, double );
 
 private:
+    static void warningDivByZero();
+
     QCOORD wd;
     QCOORD ht;
 };
@@ -82,14 +83,17 @@ private:
   QSize stream functions
  *****************************************************************************/
 
-QDataStream &operator<<( QDataStream &, const QSize & );
-QDataStream &operator>>( QDataStream &, QSize & );
+Q_EXPORT QDataStream &operator<<( QDataStream &, const QSize & );
+Q_EXPORT QDataStream &operator>>( QDataStream &, QSize & );
 
 
 /*****************************************************************************
   QSize inline functions
  *****************************************************************************/
 
+inline QSize::QSize()
+{ wd = ht = -1; }
+
 inline QSize::QSize( int w, int h )
 { wd=(QCOORD)w; ht=(QCOORD)h; }
 
@@ -129,7 +133,7 @@ inline QSize &QSize::operator-=( const QSize &s )
 inline QSize &QSize::operator*=( int c )
 { wd*=(QCOORD)c; ht*=(QCOORD)c; return *this; }
 
-inline QSize &QSize::operator*=( float c )
+inline QSize &QSize::operator*=( double c )
 { wd=(QCOORD)(wd*c); ht=(QCOORD)(ht*c); return *this; }
 
 inline bool operator==( const QSize &s1, const QSize &s2 )
@@ -150,27 +154,59 @@ inline QSize operator*( const QSize &s, int c )
 inline QSize operator*( int c, const QSize &s )
 {  return QSize(s.wd*c, s.ht*c); }
 
-inline QSize operator*( const QSize &s, float c )
+inline QSize operator*( const QSize &s, double c )
 { return QSize((QCOORD)(s.wd*c), (QCOORD)(s.ht*c)); }
 
-inline QSize operator*( float c, const QSize &s )
+inline QSize operator*( double c, const QSize &s )
 { return QSize((QCOORD)(s.wd*c), (QCOORD)(s.ht*c)); }
 
-#if !defined(QSIZE_DEBUG)
-
 inline QSize &QSize::operator/=( int c )
-{ wd/=(QCOORD)c; ht/=(QCOORD)c; return *this; }
+{
+#if defined(CHECK_MATH)
+    if ( c == 0 )
+	warningDivByZero();
+#endif
+    wd/=(QCOORD)c; ht/=(QCOORD)c;
+    return *this;
+}
 
-inline QSize &QSize::operator/=( float c )
-{ wd=(QCOORD)(wd/c); ht=(QCOORD)(ht/c); return *this; }
+inline QSize &QSize::operator/=( double c )
+{
+#if defined(CHECK_MATH)
+    if ( c == 0.0 )
+	warningDivByZero();
+#endif
+    wd=(QCOORD)(wd/c); ht=(QCOORD)(ht/c);
+    return *this;
+}
 
 inline QSize operator/( const QSize &s, int c )
-{ return QSize(s.wd/c, s.ht/c); }
+{
+#if defined(CHECK_MATH)
+    if ( c == 0 )
+	QSize::warningDivByZero();
+#endif
+    return QSize(s.wd/c, s.ht/c);
+}
 
-inline QSize operator/( const QSize &s, float c )
-{ return QSize((QCOORD)(s.wd/c), (QCOORD)(s.ht/c)); }
+inline QSize operator/( const QSize &s, double c )
+{
+#if defined(CHECK_MATH)
+    if ( c == 0.0 )
+	QSize::warningDivByZero();
+#endif
+    return QSize((QCOORD)(s.wd/c), (QCOORD)(s.ht/c));
+}
 
-#endif // no-debug functions
+inline QSize QSize::expandedTo( const QSize & otherSize ) const
+{
+    return QSize( QMAX(wd,otherSize.wd), QMAX(ht,otherSize.ht) );
+}
+
+inline QSize QSize::boundedTo( const QSize & otherSize ) const
+{
+    return QSize( QMIN(wd,otherSize.wd), QMIN(ht,otherSize.ht) );
+}
 
 
 #endif // QSIZE_H
