@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qmessagebox.cpp,v 2.51.2.2 1999/02/10 12:26:05 warwick Exp $
+** $Id: qmessagebox.cpp,v 2.85 1999/06/22 15:28:27 agulbra Exp $
 **
 ** Implementation of QMessageBox class
 **
@@ -7,17 +7,19 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -25,7 +27,6 @@
 #include "qlabel.h"
 #include "qpushbutton.h"
 #include "qimage.h"
-#include "qkeycode.h"
 #include "qapplication.h"
 
 // Message box icons, from page 210 of the Windows style guide.
@@ -39,78 +40,78 @@ static const char* information_xpm[]={
 "32 32 5 1",
 ". c None",
 "c c #000000",
-"# c #999999",
+"* c #999999",
 "a c #ffffff",
 "b c #0000ff",
-"...........########.............",
-"........###aaaaaaaa###..........",
-"......##aaaaaaaaaaaaaa##........",
-".....#aaaaaaaaaaaaaaaaaa#.......",
-"....#aaaaaaaabbbbaaaaaaaac......",
-"...#aaaaaaaabbbbbbaaaaaaaac.....",
-"..#aaaaaaaaabbbbbbaaaaaaaaac....",
-".#aaaaaaaaaaabbbbaaaaaaaaaaac...",
-".#aaaaaaaaaaaaaaaaaaaaaaaaaac#..",
-"#aaaaaaaaaaaaaaaaaaaaaaaaaaaac#.",
-"#aaaaaaaaaabbbbbbbaaaaaaaaaaac#.",
-"#aaaaaaaaaaaabbbbbaaaaaaaaaaac##",
-"#aaaaaaaaaaaabbbbbaaaaaaaaaaac##",
-"#aaaaaaaaaaaabbbbbaaaaaaaaaaac##",
-"#aaaaaaaaaaaabbbbbaaaaaaaaaaac##",
-"#aaaaaaaaaaaabbbbbaaaaaaaaaaac##",
-".#aaaaaaaaaaabbbbbaaaaaaaaaac###",
-".#aaaaaaaaaaabbbbbaaaaaaaaaac###",
-"..#aaaaaaaaaabbbbbaaaaaaaaac###.",
-"...caaaaaaabbbbbbbbbaaaaaac####.",
-"....caaaaaaaaaaaaaaaaaaaac####..",
-".....caaaaaaaaaaaaaaaaaac####...",
-"......ccaaaaaaaaaaaaaacc####....",
-".......#cccaaaaaaaaccc#####.....",
-"........###cccaaaac#######......",
-"..........####caaac#####........",
-".............#caaac##...........",
-"...............caac##...........",
-"................cac##...........",
-".................cc##...........",
-"..................###...........",
-"...................##..........."};
+"...........********.............",
+"........***aaaaaaaa***..........",
+"......**aaaaaaaaaaaaaa**........",
+".....*aaaaaaaaaaaaaaaaaa*.......",
+"....*aaaaaaaabbbbaaaaaaaac......",
+"...*aaaaaaaabbbbbbaaaaaaaac.....",
+"..*aaaaaaaaabbbbbbaaaaaaaaac....",
+".*aaaaaaaaaaabbbbaaaaaaaaaaac...",
+".*aaaaaaaaaaaaaaaaaaaaaaaaaac*..",
+"*aaaaaaaaaaaaaaaaaaaaaaaaaaaac*.",
+"*aaaaaaaaaabbbbbbbaaaaaaaaaaac*.",
+"*aaaaaaaaaaaabbbbbaaaaaaaaaaac**",
+"*aaaaaaaaaaaabbbbbaaaaaaaaaaac**",
+"*aaaaaaaaaaaabbbbbaaaaaaaaaaac**",
+"*aaaaaaaaaaaabbbbbaaaaaaaaaaac**",
+"*aaaaaaaaaaaabbbbbaaaaaaaaaaac**",
+".*aaaaaaaaaaabbbbbaaaaaaaaaac***",
+".*aaaaaaaaaaabbbbbaaaaaaaaaac***",
+"..*aaaaaaaaaabbbbbaaaaaaaaac***.",
+"...caaaaaaabbbbbbbbbaaaaaac****.",
+"....caaaaaaaaaaaaaaaaaaaac****..",
+".....caaaaaaaaaaaaaaaaaac****...",
+"......ccaaaaaaaaaaaaaacc****....",
+".......*cccaaaaaaaaccc*****.....",
+"........***cccaaaac*******......",
+"..........****caaac*****........",
+".............*caaac**...........",
+"...............caac**...........",
+"................cac**...........",
+".................cc**...........",
+"..................***...........",
+"...................**..........."};
 /* XPM */
 static const char* warning_xpm[]={
 "32 32 4 1",
 ". c None",
 "a c #ffff00",
-"# c #000000",
+"* c #000000",
 "b c #999999",
-".............###................",
-"............#aaa#...............",
-"...........#aaaaa#b.............",
-"...........#aaaaa#bb............",
-"..........#aaaaaaa#bb...........",
-"..........#aaaaaaa#bb...........",
-".........#aaaaaaaaa#bb..........",
-".........#aaaaaaaaa#bb..........",
-"........#aaaaaaaaaaa#bb.........",
-"........#aaaa###aaaa#bb.........",
-".......#aaaa#####aaaa#bb........",
-".......#aaaa#####aaaa#bb........",
-"......#aaaaa#####aaaaa#bb.......",
-"......#aaaaa#####aaaaa#bb.......",
-".....#aaaaaa#####aaaaaa#bb......",
-".....#aaaaaa#####aaaaaa#bb......",
-"....#aaaaaaaa###aaaaaaaa#bb.....",
-"....#aaaaaaaa###aaaaaaaa#bb.....",
-"...#aaaaaaaaa###aaaaaaaaa#bb....",
-"...#aaaaaaaaaa#aaaaaaaaaa#bb....",
-"..#aaaaaaaaaaa#aaaaaaaaaaa#bb...",
-"..#aaaaaaaaaaaaaaaaaaaaaaa#bb...",
-".#aaaaaaaaaaaa##aaaaaaaaaaa#bb..",
-".#aaaaaaaaaaa####aaaaaaaaaa#bb..",
-"#aaaaaaaaaaaa####aaaaaaaaaaa#bb.",
-"#aaaaaaaaaaaaa##aaaaaaaaaaaa#bb.",
-"#aaaaaaaaaaaaaaaaaaaaaaaaaaa#bbb",
-"#aaaaaaaaaaaaaaaaaaaaaaaaaaa#bbb",
-".#aaaaaaaaaaaaaaaaaaaaaaaaa#bbbb",
-"..#########################bbbbb",
+".............***................",
+"............*aaa*...............",
+"...........*aaaaa*b.............",
+"...........*aaaaa*bb............",
+"..........*aaaaaaa*bb...........",
+"..........*aaaaaaa*bb...........",
+".........*aaaaaaaaa*bb..........",
+".........*aaaaaaaaa*bb..........",
+"........*aaaaaaaaaaa*bb.........",
+"........*aaaa***aaaa*bb.........",
+".......*aaaa*****aaaa*bb........",
+".......*aaaa*****aaaa*bb........",
+"......*aaaaa*****aaaaa*bb.......",
+"......*aaaaa*****aaaaa*bb.......",
+".....*aaaaaa*****aaaaaa*bb......",
+".....*aaaaaa*****aaaaaa*bb......",
+"....*aaaaaaaa***aaaaaaaa*bb.....",
+"....*aaaaaaaa***aaaaaaaa*bb.....",
+"...*aaaaaaaaa***aaaaaaaaa*bb....",
+"...*aaaaaaaaaa*aaaaaaaaaa*bb....",
+"..*aaaaaaaaaaa*aaaaaaaaaaa*bb...",
+"..*aaaaaaaaaaaaaaaaaaaaaaa*bb...",
+".*aaaaaaaaaaaa**aaaaaaaaaaa*bb..",
+".*aaaaaaaaaaa****aaaaaaaaaa*bb..",
+"*aaaaaaaaaaaa****aaaaaaaaaaa*bb.",
+"*aaaaaaaaaaaaa**aaaaaaaaaaaa*bb.",
+"*aaaaaaaaaaaaaaaaaaaaaaaaaaa*bbb",
+"*aaaaaaaaaaaaaaaaaaaaaaaaaaa*bbb",
+".*aaaaaaaaaaaaaaaaaaaaaaaaa*bbbb",
+"..*************************bbbbb",
 "....bbbbbbbbbbbbbbbbbbbbbbbbbbb.",
 ".....bbbbbbbbbbbbbbbbbbbbbbbbb.."};
 /* XPM */
@@ -118,40 +119,41 @@ static const char* critical_xpm[]={
 "32 32 4 1",
 ". c None",
 "a c #999999",
-"# c #ff0000",
+"* c #ff0000",
 "b c #ffffff",
-"...........########.............",
-".........############...........",
-".......################.........",
-"......##################........",
-".....####################a......",
-"....######################a.....",
-"...########################a....",
-"..#######b##########b#######a...",
-"..######bbb########bbb######a...",
-".######bbbbb######bbbbb######a..",
-".#######bbbbb####bbbbb#######a..",
-"#########bbbbb##bbbbb#########a.",
-"##########bbbbbbbbbb##########a.",
-"###########bbbbbbbb###########aa",
-"############bbbbbb############aa",
-"############bbbbbb############aa",
-"###########bbbbbbbb###########aa",
-"##########bbbbbbbbbb##########aa",
-"#########bbbbb##bbbbb#########aa",
-".#######bbbbb####bbbbb#######aa.",
-".######bbbbb######bbbbb######aa.",
-"..######bbb########bbb######aaa.",
-"..#######b##########b#######aa..",
-"...########################aaa..",
-"....######################aaa...",
-"....a####################aaa....",
-".....a##################aaa.....",
-"......a################aaa......",
-".......aa############aaaa.......",
-".........aa########aaaaa........",
+"...........********.............",
+".........************...........",
+".......****************.........",
+"......******************........",
+".....********************a......",
+"....**********************a.....",
+"...************************a....",
+"..*******b**********b*******a...",
+"..******bbb********bbb******a...",
+".******bbbbb******bbbbb******a..",
+".*******bbbbb****bbbbb*******a..",
+"*********bbbbb**bbbbb*********a.",
+"**********bbbbbbbbbb**********a.",
+"***********bbbbbbbb***********aa",
+"************bbbbbb************aa",
+"************bbbbbb************aa",
+"***********bbbbbbbb***********aa",
+"**********bbbbbbbbbb**********aa",
+"*********bbbbb**bbbbb*********aa",
+".*******bbbbb****bbbbb*******aa.",
+".******bbbbb******bbbbb******aa.",
+"..******bbb********bbb******aaa.",
+"..*******b**********b*******aa..",
+"...************************aaa..",
+"....**********************aaa...",
+"....a********************aaa....",
+".....a******************aaa.....",
+"......a****************aaa......",
+".......aa************aaaa.......",
+".........aa********aaaaa........",
 "...........aaaaaaaaaaa..........",
 ".............aaaaaaa............"};
+#define QT_END_TEXT qt_dialog_default_key
 
 
 /*!
@@ -159,8 +161,6 @@ static const char* critical_xpm[]={
   \brief Displays a brief message, an icon, and some buttons.
   \ingroup dialogs
 
-  \define QMessageBox::Icon
-
   A message box is a modal dialog that displays an icon, a text and up to
   three push buttons.  It's used for simple messages and questions.
 
@@ -180,6 +180,11 @@ static const char* critical_xpm[]={
   applications) for a simple messages, or two or even three buttons
   for questions.
 
+  The text part of all message box messages can be either rich text or
+  plain text. If you specify a rich text formatted string, it will be
+  rendered using the default stylesheet. See
+  QStyleSheet::defaultSheet() for details.
+
   Here are some examples of how to use the static member functions.
   After these examples you will find an overview of the non-static
   member functions.
@@ -267,7 +272,7 @@ static const char* critical_xpm[]={
   \endcode
 
   The critical() function should be reserved for critical errors.  In
-  this example, errorDetails is a QString or const char *, and QString
+  this example, errorDetails is a QString or const char*, and QString
   is used to concatenate several strings:
 
   \code
@@ -355,7 +360,7 @@ static const char* critical_xpm[]={
 
   QMessageBox has no signals or slots.
 
-  <img src=qmsgbox-m.gif> <img src=qmsgbox-w.gif>
+  <img src=qmsgbox-m.png> <img src=qmsgbox-w.png>
 
   \sa QDialog, <a href="http://www.iarchitect.com/errormsg.htm">Isys on
   error messages,</a>
@@ -387,7 +392,29 @@ static const int LastButton = QMessageBox::Ignore;
 */
 
 static const char *mb_texts[] = {
-    0, "OK", "Cancel", "Yes", "No", "Abort", "Retry", "Ignore", 0
+    0,
+    QT_TRANSLATE_NOOP("QMessageBox","OK"),
+    QT_TRANSLATE_NOOP("QMessageBox","Cancel"),
+    QT_TRANSLATE_NOOP("QMessageBox","Yes"),
+    QT_TRANSLATE_NOOP("QMessageBox","No"),
+    QT_TRANSLATE_NOOP("QMessageBox","Abort"),
+    QT_TRANSLATE_NOOP("QMessageBox","Retry"),
+    QT_TRANSLATE_NOOP("QMessageBox","Ignore"),
+    0
+};
+
+
+// Internal class - don't touch
+
+class QMessageBoxLabel : public QLabel
+{
+    Q_OBJECT
+public:
+    QMessageBoxLabel(QWidget *parent=0, const char *name=0)
+	: QLabel( parent, name )
+	{
+	}
+
 };
 
 
@@ -462,7 +489,8 @@ QMessageBox::QMessageBox( QWidget *parent, const char *name )
   \sa setCaption(), setText(), setIcon()
 */
 
-QMessageBox::QMessageBox( const char *caption, const char *text, Icon icon,
+QMessageBox::QMessageBox( const QString& caption,
+			  const QString &text, Icon icon,
 			  int button0, int button1, int button2,
 			  QWidget *parent, const char *name,
 			  bool modal, WFlags f )
@@ -487,7 +515,7 @@ QMessageBox::~QMessageBox()
 
 void QMessageBox::init( int button0, int button1, int button2 )
 {
-    label = new QLabel( this, "text" );
+    label = new QMessageBoxLabel( this, "text" );
     CHECK_PTR( label );
     label->setAlignment( AlignLeft );
 
@@ -555,10 +583,11 @@ void QMessageBox::init( int button0, int button1, int button2 )
 	if ( i >= mbd->numButtons ) {
 	    mbd->pb[i] = 0;
 	} else {
-	    QString buttonName;
+	    QCString buttonName;
 	    buttonName.sprintf( "button%d", i+1 );
-	    mbd->pb[i] = new QPushButton( mb_texts[mbd->button[i]],
-					  this, buttonName );
+	    mbd->pb[i] = new QPushButton(
+		tr(mb_texts[mbd->button[i]]),
+	        this, buttonName );
 	    if ( mbd->defButton == i ) {
 		mbd->pb[i]->setDefault( TRUE );
 		mbd->pb[i]->setFocus();
@@ -604,11 +633,13 @@ void QMessageBox::resizeButtons()
 
 
 /*!
-  Returns the message box text currently set, or null if no text has been set.
+  Returns the message box text currently set, or a
+  \link QString::operator!() null string\endlink
+  if no text has been set.
   \sa setText()
 */
 
-const char *QMessageBox::text() const
+QString QMessageBox::text() const
 {
     return label->text();
 }
@@ -618,22 +649,25 @@ const char *QMessageBox::text() const
   \sa text()
 */
 
-void QMessageBox::setText( const char *text )
+void QMessageBox::setText( const QString &text )
 {
     label->setText( text );
 }
 
-
 /*!
-  Returns the icon of the message box.
+  \enum QMessageBox::Icon
 
-  The return value is one of the following:
+  This type includes the following values:
   <ul>
   <li> \c QMessageBox::NoIcon
   <li> \c QMessageBox::Information
   <li> \c QMessageBox::Warning
   <li> \c QMessageBox::Critical
   </ul>
+*/
+
+/*!
+  Returns the icon of the message box.
 
   \sa setIcon(), iconPixmap()
 */
@@ -693,7 +727,7 @@ QPixmap QMessageBox::standardIcon( Icon icon, GUIStyle style )
 	QImage image(xpm_data);
 	if ( style == MotifStyle ) {
 	    // All that colour looks ugly in Motif
-	    QColorGroup g = QApplication::palette()->normal();
+	    QColorGroup g = QApplication::palette().normal();
 	    switch ( icon ) {
 	      case Information:
 		image.setColor( 2, 0xff000000 | g.dark().rgb() );
@@ -753,43 +787,19 @@ void QMessageBox::setIconPixmap( const QPixmap &pixmap )
 }
 
 
-/*!
-  This function will be removed in a future version of Qt.
-*/
-
-const char *QMessageBox::buttonText() const
-{
-    return buttonText( Ok );
-}
-
-/*!
-  This function will be removed in a future version of Qt.
-*/
-
-void QMessageBox::setButtonText( const char *text )
-{
-    setButtonText( Ok, text ? text : "OK" );
-}
-
-
 /*!
   Returns the text of the message box button \a button, or null if the
   message box does not contain the button.
 
-  Example:
-  \code
-    QMessageBox mb( QMessageBox::Ok, QMessageBox::Cancel, 0 );
-    mb.buttonText( QMessageBox::Cancel );  // returns "Cancel"
-    mb.buttonText( QMessageBox::Ignore );  // returns 0
-  \endcode
-
   \sa setButtonText()
 */
 
-const char *QMessageBox::buttonText( int button ) const
+QString QMessageBox::buttonText( int button ) const
 {
     int index = indexOf(button);
-    return index >= 0 && mbd->pb[index] ? mbd->pb[index]->text() : 0;
+    return index >= 0 && mbd->pb[index]
+	    ? mbd->pb[index]->text()
+	    : QString::null;
 }
 
 
@@ -798,17 +808,10 @@ const char *QMessageBox::buttonText( int button ) const
   Setting the text of a button that is not in the message box is quietly
   ignored.
 
-  Example:
-  \code
-    QMessageBox mb( QMessageBox::Ok, QMessageBox::Cancel, 0 );
-    mb.setButtonText( QMessageBox::Ok, "All Right" );
-    mb.setButtonText( QMessageBox::Yes, "Yo" );	  // ignored
-  \endcode
-
   \sa buttonText()
 */
 
-void QMessageBox::setButtonText( int button, const char *text )
+void QMessageBox::setButtonText( int button, const QString &text )
 {
     int index = indexOf(button);
     if ( index >= 0 && mbd->pb[index] ) {
@@ -891,8 +894,12 @@ void QMessageBox::resizeEvent( QResizeEvent * )
     mbd->iconLabel.move( border, border );
     if ( mbd->iconLabel.pixmap() && mbd->iconLabel.pixmap()->width() )
 	lmargin += mbd->iconLabel.pixmap()->width() + border;
-    label->move( (width() + lmargin)/2 - label->width()/2,
-		 (height() - border - bh - label->height()) / 2 );
+//     label->move( (width() + lmargin)/2 - label->width()/2,
+// 		 (height() - border - bh - label->height()) / 2 );
+    label->setGeometry( lmargin+border,
+			border,
+			width() - lmargin -2*border,
+			height() - 3*border - bh );
     int space = (width() - bw*n)/(n+1);
     for ( i=0; i<n; i++ ) {
 	mbd->pb[i]->move( space*(i+1)+bw*i,
@@ -931,13 +938,15 @@ void QMessageBox::keyPressEvent( QKeyEvent *e )
   information(), warning() or critical() instead.
 */
 
-int QMessageBox::message( const char *caption,
-			  const char *text,
-			  const char *buttonText,
+int QMessageBox::message( const QString &caption,
+			  const QString& text,
+			  const QString& buttonText,
 			  QWidget    *parent,
 			  const char * )
 {
-    return QMessageBox::information( parent, caption, text, buttonText?buttonText:"Ok" ) == 0;
+    return QMessageBox::information( parent, caption, text,
+				     buttonText.isEmpty()
+				     ? tr("OK") : buttonText ) == 0;
 }
 
 
@@ -950,14 +959,16 @@ int QMessageBox::message( const char *caption,
   information(), warning() or critical() instead.
 */
 
-bool QMessageBox::query( const char *caption,
-			 const char *text,
-			 const char *yesButtonText,
-			 const char *noButtonText,
+bool QMessageBox::query( const QString &caption,
+			 const QString& text,
+			 const QString& yesButtonText,
+			 const QString& noButtonText,
 			 QWidget *parent, const char * )
 {
     return QMessageBox::information( parent, caption, text,
-				     yesButtonText?yesButtonText:"Ok", noButtonText ) == 0;
+				     yesButtonText.isEmpty()
+				     ? tr("OK") : yesButtonText,
+				     noButtonText ) == 0;
 }
 
 
@@ -973,7 +984,7 @@ bool QMessageBox::query( const char *caption,
 */
 
 int QMessageBox::information( QWidget *parent,
-			      const char *caption, const char *text,
+			      const QString& caption, const QString& text,
 			      int button0, int button1, int button2 )
 {
     QMessageBox *mb = new QMessageBox( caption, text, Information,
@@ -998,7 +1009,7 @@ int QMessageBox::information( QWidget *parent,
 */
 
 int QMessageBox::warning( QWidget *parent,
-			  const char *caption, const char *text,
+			  const QString& caption, const QString& text,
 			  int button0, int button1, int button2 )
 {
     QMessageBox *mb = new QMessageBox( caption, text, Warning,
@@ -1023,7 +1034,7 @@ int QMessageBox::warning( QWidget *parent,
 */
 
 int QMessageBox::critical( QWidget *parent,
-			   const char *caption, const char *text,
+			   const QString& caption, const QString& text,
 			   int button0, int button1, int button2 )
 {
     QMessageBox *mb = new QMessageBox( caption, text, Critical,
@@ -1052,8 +1063,8 @@ int QMessageBox::critical( QWidget *parent,
   \sa QWidget::icon() QApplication::mainWidget()
 */
 
-void QMessageBox::about( QWidget *parent, const char *caption,
-			 const char *text )
+void QMessageBox::about( QWidget *parent, const QString &caption,
+			 const QString& text )
 {
     QMessageBox *mb = new QMessageBox( caption, text,
 				       Information,
@@ -1076,13 +1087,11 @@ void QMessageBox::about( QWidget *parent, const char *caption,
 }
 
 
-/*!
-  Reimplemented for implementational reasons.
+/*! \reimp
 */
 
-void QMessageBox::setStyle( GUIStyle s )
+void QMessageBox::styleChanged( QStyle& )
 {
-    QWidget::setStyle( s );
     if ( mbd->icon != NoIcon ) {
 	// Reload icon for new style
 	setIcon( mbd->icon );
@@ -1091,17 +1100,17 @@ void QMessageBox::setStyle( GUIStyle s )
 
 
 static int textBox( QWidget *parent, QMessageBox::Icon severity,
-		    const char *caption, const char *text,
-		    const char *button0Text,
-		    const char *button1Text,
-		    const char *button2Text,
+		    const QString& caption, const QString& text,
+		    const QString& button0Text,
+		    const QString& button1Text,
+		    const QString& button2Text,
 		    int defaultButtonNumber,
 		    int escapeButtonNumber )
 {
     int b[3];
-    b[0] = (button0Text && *button0Text) ? 1 : 0;
-    b[1] = (button1Text && *button1Text) ? 2 : 0;
-    b[2] = (button2Text && *button2Text) ? 3 : 0;
+    b[0] = 1;
+    b[1] = button1Text.isEmpty() ? 0 : 2;
+    b[2] = button2Text.isEmpty() ? 0 : 3;
 
     int i;
     for( i=0; i<3; i++ ) {
@@ -1115,7 +1124,9 @@ static int textBox( QWidget *parent, QMessageBox::Icon severity,
 				       b[0], b[1], b[2],
 				       parent, "information" );
     CHECK_PTR( mb );
-    if ( b[0] )
+    if ( button0Text.isEmpty() )
+	mb->setButtonText( 1, mb_texts[ QMessageBox::Ok ] );
+    else
 	mb->setButtonText( 1, button0Text );
     if ( b[1] )
 	mb->setButtonText( 2, button1Text );
@@ -1150,11 +1161,11 @@ static int textBox( QWidget *parent, QMessageBox::Icon severity,
   \sa warning(), critical()
 */
 
-int QMessageBox::information( QWidget *parent, const char *caption,
-			      const char *text,
-			      const char *button0Text,
-			      const char *button1Text,
-			      const char *button2Text,
+int QMessageBox::information( QWidget *parent, const QString &caption,
+			      const QString& text,
+			      const QString& button0Text,
+			      const QString& button1Text,
+			      const QString& button2Text,
 			      int defaultButtonNumber,
 			      int escapeButtonNumber )
 {
@@ -1187,11 +1198,11 @@ int QMessageBox::information( QWidget *parent, const char *caption,
   \sa information(), critical()
 */
 
-int QMessageBox::warning( QWidget *parent, const char *caption,
-				 const char *text,
-				 const char *button0Text,
-				 const char *button1Text,
-				 const char *button2Text,
+int QMessageBox::warning( QWidget *parent, const QString &caption,
+				 const QString& text,
+				 const QString& button0Text,
+				 const QString& button1Text,
+				 const QString& button2Text,
 				 int defaultButtonNumber,
 				 int escapeButtonNumber )
 {
@@ -1224,11 +1235,11 @@ int QMessageBox::warning( QWidget *parent, const char *caption,
   \sa information() warning()
 */
 
-int QMessageBox::critical( QWidget *parent, const char *caption,
-				  const char *text,
-				  const char *button0Text,
-				  const char *button1Text,
-				  const char *button2Text,
+int QMessageBox::critical( QWidget *parent, const QString &caption,
+				  const QString& text,
+				  const QString& button0Text,
+				  const QString& button1Text,
+				  const QString& button2Text,
 				  int defaultButtonNumber,
 				  int escapeButtonNumber )
 {
@@ -1239,27 +1250,74 @@ int QMessageBox::critical( QWidget *parent, const char *caption,
 
 
 static const char *textAboutQt =
-"This program is developed with Qt, the multi-platform C++ GUI toolkit.\n\n"
-"Qt is a product of Troll Tech AS (http://www.troll.no).\n"
-"Qt is available under two different licenses:\n"
-"- The Free Edition, which may be used free of charge to develop\n"
-"  Free Software on the X Window System.\n"
-"- The Professional Edition, which may be used to develop commercial\n"
-"  software on both X and Microsoft Windows.\n\n"
-"Please contact sales@troll.no for information and pricing.";
+"<h3>This program is developed with Qt, the multi-platform C++ GUI toolkit.</h3>"
+"<p>Qt version running with this application: <tt>%1</tt></p>"
+"<p>Qt is a product of <b>Troll Tech AS </b>(http://www.troll.no).</p>"
+"<p>Qt is available under two different licenses:</p>"
+"<ul><li>The Free Edition, which may be used free of charge to develop"
+"  Free Software on the X Window System.</li>"
+"<li>The Professional Edition, which may be used to develop commercial"
+"  software on both X and Microsoft Windows.</li></ul>";
+void QT_END_TEXT() { QMessageBox::aboutQt(0,"E" "g" "g"); }
 
 
 /*!
   Displays a simple message box about Qt, with window caption \a
-  caption and optionally centered over \a parent.
+  caption and optionally centered over \a parent.  The message includes
+  the version number of Qt being used by the application.
 
   This is neat for inclusion into the Help menu.  See the menu.cpp
   example.
 */
 
-void QMessageBox::aboutQt( QWidget *parent, const char *caption )
+void QMessageBox::aboutQt( QWidget *parent, const QString &caption )
 {
-    if ( !caption )
-	caption = "About Qt";
-    information( parent, caption, textAboutQt );
+    information( parent, caption.isNull()
+	? QString::fromLatin1("About Qt") : caption,
+		 qApp->translate( "QMessageBox", textAboutQt )
+			.arg(QString::fromLatin1(QT_VERSION_STR))
+    );
 }
+
+
+#include <qmetaobject.h>
+
+
+const char *QMessageBoxLabel::className() const
+{
+    return "QMessageBoxLabel";
+}
+
+QMetaObject *QMessageBoxLabel::metaObj = 0;
+
+void QMessageBoxLabel::initMetaObject()
+{
+    if ( metaObj )
+	return;
+    if ( strcmp(QLabel::className(), "QLabel") != 0 )
+	badSuperclassWarning("QMessageBoxLabel","QLabel");
+    if ( !QLabel::metaObject() )
+	QLabel::initMetaObject();
+    metaObj = new QMetaObject( "QMessageBoxLabel", "QLabel",
+	0, 0,
+	0, 0 );
+}
+
+
+/*!
+  \reimp
+*/
+
+void QMessageBox::setIcon( const QPixmap &pix )
+{
+    //reimplemented to avoid compiler warning.
+    QDialog::setIcon( pix );
+}
+
+
+// My own personal favorite minimalist error message popped up whilst
+// testing Freehand 8 last month.  I took a screen shot.  I believe I
+// was trying to convert a file from one format to another.
+// Apparently, I...
+//
+// http://www.people.cornell.edu/pages/mlj8/cant.png
