@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qprogressdialog.cpp,v 2.28.2.1 1998/08/18 19:51:39 warwick Exp $
+** $Id: qprogressdialog.cpp,v 2.46.2.1 1999/07/13 19:58:43 warwick Exp $
 **
 ** Implementation of QProgressDialog class
 **
@@ -7,17 +7,19 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -27,6 +29,7 @@
 #include "qdrawutil.h"
 #include "qdatetime.h"
 #include "qapplication.h"
+#include <limits.h>
 
 // If the operation is expected to take this long (as predicted by
 // progress time), show the progress dialog.
@@ -43,7 +46,7 @@ static const int spacing     = 4;
 struct QProgressData
 {
     QProgressData( QProgressDialog* that, QWidget* parent,
-		   const char* labelText,
+		   const QString& labelText,
 		   int totalSteps ) :
 	creator( parent ),
 	label( new QLabel(labelText,that,"label") ),
@@ -53,8 +56,9 @@ struct QProgressData
 	cancellation_flag( FALSE ),
 	showTime( defaultShowTime )
     {
-	label->setAlignment( that->style() != WindowsStyle ?
-			     AlignCenter : AlignLeft|AlignVCenter );
+	label->setAlignment( that->style() != Qt::WindowsStyle
+			     ? Qt::AlignCenter
+			     : Qt::AlignLeft|Qt::AlignVCenter );
     }
 
     QWidget	 *creator;
@@ -79,6 +83,9 @@ struct QProgressData
   operation is going to take to perform, and to reassure them that the
   application has not frozen.
 
+  QProgressDialog contains a label, a progress bar and a cancellation
+  button.
+
   A potential problem with progress dialogs is that it is difficult to know
   when to use them, as operations take different amounts of time on different
   computer hardware.  QProgressDialog offers a solution to this problem:
@@ -87,7 +94,8 @@ struct QProgressData
 
   Example:
   \code
-    QProgressDialog progress( "Copying files...", "Abort Copy", numFiles, this );
+    QProgressDialog progress( "Copying files...", "Abort Copy", numFiles,
+				this, "progress", TRUE );
     for (int i=0; i<numFiles; i++) {
 	progress.setProgress( i );
 	if ( progress.wasCancelled() )
@@ -97,8 +105,8 @@ struct QProgressData
     progress.setProgress( numFiles );
   \endcode
 
-  <img src=qprogdlg-m.gif> <img src=qprogdlg-w.gif>
-  
+  <img src=qprogdlg-m.png> <img src=qprogdlg-w.png>
+
   \sa QDialog QProgressBar
   <a href="guibooks.html#fowler">GUI Design Handbook: Progress Indicator</a>
 */
@@ -151,7 +159,8 @@ QProgressDialog::QProgressDialog( QWidget *creator, const char *name,
 				  bool modal, WFlags f )
     : QSemiModal( 0, name, modal, f)
 {
-    init( creator, "", "Cancel", 100 );
+    init( creator, QString::fromLatin1(""),
+	tr("Cancel"), 100 );
 }
 
 /*!
@@ -175,8 +184,8 @@ QProgressDialog::QProgressDialog( QWidget *creator, const char *name,
   setTotalSteps()
 */
 
-QProgressDialog::QProgressDialog( const char *labelText,
-				  const char *cancelButtonText,
+QProgressDialog::QProgressDialog( const QString &labelText,
+				  const QString &cancelButtonText,
 				  int totalSteps,
 				  QWidget *creator, const char *name,
 				  bool modal, WFlags f )
@@ -196,7 +205,7 @@ QProgressDialog::~QProgressDialog()
 }
 
 void QProgressDialog::init( QWidget *creator,
-			    const char* lbl, const char* canc,
+			    const QString& lbl, const QString& canc,
 			    int totstps)
 {
     d = new QProgressData(this, creator, lbl, totstps);
@@ -231,7 +240,7 @@ void QProgressDialog::setLabel( QLabel *label )
 	if ( label->parentWidget() == this ) {
 	    label->hide(); // until we resize
 	} else {
-	    label->recreate( this, 0, QPoint(0,0), FALSE );
+	    label->reparent( this, 0, QPoint(0,0), FALSE );
 	}
     }
     resize(sizeHint());
@@ -245,7 +254,7 @@ void QProgressDialog::setLabel( QLabel *label )
   \sa setLabel()
 */
 
-void QProgressDialog::setLabelText( const char *text )
+void QProgressDialog::setLabelText( const QString &text )
 {
     if ( label() ) {
 	label()->setText( text );
@@ -269,7 +278,7 @@ void QProgressDialog::setCancelButton( QPushButton *cancelButton )
 	if ( cancelButton->parentWidget() == this ) {
 	    cancelButton->hide(); // until we resize
 	} else {
-	    cancelButton->recreate( this, 0, QPoint(0,0), FALSE );
+	    cancelButton->reparent( this, 0, QPoint(0,0), FALSE );
 	}
 	connect( d->cancel, SIGNAL(clicked()), this, SIGNAL(cancelled()) );
 	QAccel *accel = new QAccel( this );
@@ -286,9 +295,9 @@ void QProgressDialog::setCancelButton( QPushButton *cancelButton )
   \sa setCancelButton()
 */
 
-void QProgressDialog::setCancelButtonText( const char *cancelButtonText )
+void QProgressDialog::setCancelButtonText( const QString &cancelButtonText )
 {
-    if ( cancelButtonText ) {
+    if ( !cancelButtonText.isNull() ) {
 	if ( d->cancel )
 	    d->cancel->setText(cancelButtonText);
 	else
@@ -310,7 +319,7 @@ void QProgressDialog::setBar( QProgressBar *bar )
 {
     if ( progress() > 0 ) {
 #if defined(CHECK_STATE)
-	warning( "QProgrssDialog::setBar: Cannot set a new progress bar "
+	qWarning( "QProgrssDialog::setBar: Cannot set a new progress bar "
 		 "while the old one is active" );
 #endif
     }
@@ -434,16 +443,28 @@ void QProgressDialog::setProgress( int progress )
 	    }
 	    d->starttime.start();
 	} else {
+	    bool need_show;
 	    int elapsed = d->starttime.elapsed();
-	    if ( !d->showTime || elapsed > minWaitTime ) {
-		int estimate = elapsed * (totalSteps() - progress) / progress;
-		if ( estimate >= d->showTime ) {
-		    resize(sizeHint());
-		    center();
-		    show();
-		    d->shown_once = TRUE;
+	    if ( elapsed >= d->showTime ) {
+		need_show = TRUE;
+	    } else {
+		if ( elapsed > minWaitTime ) {
+		    int estimate;
+		    if ( (totalSteps() - progress) >= INT_MAX / elapsed )
+			estimate = (totalSteps() - progress) / progress * elapsed;
+		    else
+			estimate = elapsed * (totalSteps() - progress) / progress;
+		    need_show = estimate >= d->showTime;
+		} else {
+		    need_show = FALSE;
 		}
 	    }
+	    if ( need_show ) {
+		resize(sizeHint());
+		center();
+		show();
+		d->shown_once = TRUE;
+	    }
 	}
     }
 
@@ -482,7 +503,7 @@ QSize QProgressDialog::sizeHint() const
     int h = margin_tb*2 + bh.height() + sh.height() + spacing;
     if ( d->cancel )
 	h += d->cancel->sizeHint().height() + spacing;
-    return QSize( QMAX(200, sh.width()), h );
+    return QSize( QMAX(200, sh.width() + 2*margin_lr), h );
 }
 
 /*!
@@ -497,7 +518,7 @@ void QProgressDialog::resizeEvent( QResizeEvent * )
 /*!
   Ensures layout conforms to style of GUI.
 */
-void QProgressDialog::styleChange(GUIStyle s)
+void QProgressDialog::styleChange(QStyle& s)
 {
     QSemiModal::styleChange(s);
     layout();
@@ -559,10 +580,21 @@ void QProgressDialog::setMinimumDuration( int ms )
 
 /*!
   Returns the currently set minimum duration for the QProgressDialog
-  
+
   \sa setMinimumDuration()
 */
 int QProgressDialog::minimumDuration() const
 {
     return d->showTime;
 }
+
+
+/*!
+  \reimp
+*/
+
+void QProgressDialog::closeEvent( QCloseEvent *e )
+{
+    emit cancelled();
+    QSemiModal::closeEvent( e );
+}
