@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qtabdialog.cpp,v 2.42 1998/07/03 00:09:29 hanord Exp $
+** $Id: qtabdialog.cpp,v 2.76 1999/06/16 02:22:14 warwick Exp $
 **
 ** Implementation of QTabDialog class
 **
 ** Created : 960825
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -26,10 +28,12 @@
 #include "qobjectlist.h"
 #include "qobjectdict.h"
 #include "qtabbar.h"
+#include "qtabwidget.h"
 #include "qpushbutton.h"
 #include "qpainter.h"
 #include "qpixmap.h"
 #include "qapplication.h"
+#include "qtabwidget.h"
 #include "qwidgetstack.h"
 #include "qlayout.h"
 
@@ -49,15 +53,15 @@
   not offer any way to find out which page is currently visible or to
   set the visible page.
 
-  QTabDialog provides an OK button and optionally Apply, Cancel and
-  Defaults buttons.
+  QTabDialog provides an OK button and optionally Apply, Cancel,
+  Defaults, and Help buttons.
 
   The normal way to use QTabDialog is to do the following in the
-  constructor: <ol> <li> Create a QTabDialog, <li> create a QWidget
+  constructor: <ol> <li> Create a QTabDialog. <li> Create a QWidget
   for each of the pages in the tab dialog, insert children into it,
   set up geometry management for it, and use addTab() to set up a tab
-  and keyboard accelerator for it, <li> set up the buttons for the tab
-  dialog (Apply, Cancel and so on), and finally <li> connect to the
+  and keyboard accelerator for it. <li> Set up the buttons for the tab
+  dialog (Apply, Cancel and so on). <li> Connect to the
   signals and slots. </ol>
 
   The pref.cpp example does all this.
@@ -76,7 +80,10 @@
 
   There are also several other signals which may be useful. <ul> <li>
   cancelButtonPressed() is emitted when the user clicks Cancel.  <li>
-  defaultButtonPressed() is emitted when the user clicks Defaults; the
+  defaultButtonPressed() is emitted when the user clicks Defaults;
+  <li>
+  helpButtonPressed() is emitted when the user clicks Help;
+  the
   slot it is connected to should reset the state of the dialog to the
   application defaults.  <li> aboutToShow() is emitted at the start of
   show(); if there is any chance that the state of the application may
@@ -95,7 +102,7 @@
   While tab dialogs can be a very good way to split up a complex
   dialog, it's also very easy to make a royal mess out of a tab
   dialog.  Here is some advice.  For more, see e.g. the <a
-  href="http://www.uie.com/tabbed.html">UIE web page on tab
+  href="http://world.std.com/~uieweb/tabbed.htm">UIE web page on tab
   dialogs.</a>
 
   <ol><li> Make sure that each page forms a logical whole which is
@@ -149,17 +156,15 @@
   ought not to be joined in a tab dialog.
 
   </ol>
-  
-  Most of the functionality in QTabDialog is provided by a QTabBar (at
-  the top, providing the tabs) and a QWidgetStack (most of the area,
-  organizing the individual pages).
 
-  <img src=qtabdlg-m.gif> <img src=qtabdlg-w.gif>
-  
+  Most of the functionality in QTabDialog is provided by a QTabWidget.
+
+  <img src=qtabdlg-m.png> <img src=qtabdlg-w.png>
+
   \sa QDialog
 */
 
-/*! \fn void QTabDialog::selected( const char * tabLabel );
+/*! \fn void QTabDialog::selected( const QString &tabLabel );
 
   This signal is emitted whenever a tab is selected (raised),
   including during the first show().
@@ -172,24 +177,25 @@
 
 struct QTabPrivate
 {
-    QTabPrivate()
-	: tabs(0), stack(0),
-	  ok(0), cb(0), db(0), ab(0),
-	  tll(0) {}
+    QTabPrivate();
 
-    QTabBar * tabs;
-    QWidgetStack * stack;
-
-    QArrayT<QWidget *> children;
+    QTabWidget* tw;
 
     QPushButton * ok;
     QPushButton * cb;
     QPushButton * db;
+    QPushButton * hb;
     QPushButton * ab;
 
     QBoxLayout * tll;
 };
 
+QTabPrivate::QTabPrivate()
+	: tw(0),
+	  ok(0), cb(0), db(0), hb(0), ab(0),
+	  tll(0)
+{ }
+
 /*!
   Constructs a QTabDialog with only an Ok button.
 */
@@ -201,12 +207,12 @@ QTabDialog::QTabDialog( QWidget *parent, const char *name, bool modal,
     d = new QTabPrivate;
     CHECK_PTR( d );
 
-    d->stack = new QWidgetStack( this, "tab pages" );
-    setTabBar( new QTabBar( this, "tab control" ) );
+    d->tw = new QTabWidget( this, "tab widget" );
+    connect ( d->tw, SIGNAL ( selected( const QString& ) ), this, SIGNAL( selected( const QString& ) ) );
 
     d->ok = new QPushButton( this, "ok" );
     CHECK_PTR( d->ok );
-    d->ok->setText( "OK" );
+    d->ok->setText( tr("OK") );
     d->ok->setDefault( TRUE );
     connect( d->ok, SIGNAL(clicked()),
 	     this, SIGNAL(applyButtonPressed()) );
@@ -265,6 +271,19 @@ bool QTabDialog::hasDefaultButton() const
 }
 
 
+/*!
+  Returns TRUE if the tab dialog has a Help button, FALSE if not.
+
+  \sa setApplyButton() applyButtonPressed() hasCancelButton()
+  hasHelpButton()
+*/
+
+bool QTabDialog::hasHelpButton() const
+{
+     return d->hb != 0;
+}
+
+
 /*!
   \fn void QTabDialog::cancelButtonPressed();
 
@@ -306,6 +325,16 @@ bool QTabDialog::hasCancelButton() const
 */
 
 
+/*!
+  \fn void QTabDialog::helpButtonPressed();
+
+  This signal is emitted when the Help button is pressed.  It
+  should give instructions about how to use the dialog.
+
+  \sa applyButtonPressed() cancelButtonPressed() setHelpButton()
+*/
+
+
 /*!
   Returns TRUE if the tab dialog has an Apply button, FALSE if not.
 
@@ -360,7 +389,7 @@ bool QTabDialog::hasOkButton() const
 void QTabDialog::show()
 {
     if ( topLevelWidget() == this )
-	d->tabs->setFocus();
+	d->tw->setFocus();
     emit aboutToShow();
     setSizes();
     setUpLayout();
@@ -369,20 +398,17 @@ void QTabDialog::show()
 
 
 /*!
-  Ensure that the selected tab's page is visible and appropriately sized.
+  Ensures that the selected tab's page is visible and appropriately sized.
 */
 
 void QTabDialog::showTab( int i )
 {
-    if ( d->stack->widget( i ) ) {
-	d->stack->raiseWidget( i );
-	emit selected( d->tabs->tab( i )->label );
-    }
+    d->tw->showTab( i );
 }
 
 
 /*!
-  Add another tab and page to the tab view.
+  Adds another tab and page to the tab view.
 
   The tab will be labelled \a label and \a child constitutes the new
   page.  Note the difference between the widget name (which you supply
@@ -405,12 +431,22 @@ void QTabDialog::showTab( int i )
   user will be confused.
 */
 
-void QTabDialog::addTab( QWidget * child, const char * label )
+void QTabDialog::addTab( QWidget * child, const QString &label )
 {
-    QTab * t = new QTab();
-    CHECK_PTR( t );
-    t->label = label;
-    addTab( child, t );
+    d->tw->addTab( child, label );
+}
+
+
+
+/*!
+  Adds another tab and page to the tab view.
+
+  This function is the same as addTab() but with an additional
+  iconset.
+ */
+void QTabDialog::addTab( QWidget *child, const QIconSet& iconset, const QString &label)
+{
+    d->tw->addTab( child, iconset, label );
 }
 
 /*!
@@ -421,10 +457,62 @@ void QTabDialog::addTab( QWidget * child, const char * label )
 */
 void QTabDialog::addTab( QWidget * child, QTab* tab )
 {
-    tab->enabled = TRUE;
-    int id = d->tabs->addTab( tab );
-    d->stack->addWidget( child, id );
-    d->tabs->setMinimumSize( d->tabs->sizeHint() );
+    d->tw->addTab( child, tab );
+}
+
+/*!
+  Inserts another tab and page to the tab view.
+
+  The tab will be labelled \a label and \a child constitutes the new
+  page.  Note the difference between the widget name (which you supply
+  to widget constructors and to e.g. setTabEnabled()) and the tab
+  label: The name is internal to the program and invariant, while the
+  label is shown on screen and may vary according to e.g. language.
+
+  \a label is written in the QButton style, where &P makes Qt create
+  an accelerator key on Alt-P for this page.  For example:
+
+  \code
+    td->insertTab( graphicsPane, "&Graphics" );
+    td->insertTab( soundPane, "&Sound" );
+  \endcode
+
+  If \a index is not specified, the tab is simply added. Otherwise
+  it's inserted at the specified position.
+
+  If the user presses Alt-S the sound page of the tab dialog is shown,
+  if the user presses Alt-P the graphics page is shown.
+
+  If you call insertTab() after show(), the screen will flicker and the
+  user will be confused.
+*/
+
+void QTabDialog::insertTab( QWidget * child, const QString &label, int index )
+{
+    d->tw->insertTab( child, label, index );
+}
+
+
+/*!
+  Inserts another tab and page to the tab view.
+
+  This function is the same as insertTab() but with an additional
+  iconset.
+ */
+void QTabDialog::insertTab( QWidget *child, const QIconSet& iconset, const QString &label, int index)
+{
+    d->tw->insertTab( child, iconset, label, index );
+}
+
+/*!
+  This is a lower-level method for inserting tabs, similar to the other
+  insertTab() method.  It is useful if you are using setTabBar() to set a
+  QTabBar subclass with an overridden QTabBar::paint() routine for a
+  subclass of QTab.
+*/
+void QTabDialog::insertTab( QWidget * child, QTab* tab, int index )
+{
+    d->tw->insertTab( child, tab, index );
 }
 
 /*!
@@ -435,11 +523,7 @@ void QTabDialog::addTab( QWidget * child, QTab* tab )
 */
 void QTabDialog::setTabBar( QTabBar* tb )
 {
-    delete d->tabs;
-    d->tabs = tb;
-    connect( d->tabs, SIGNAL(selected(int)),
-	     this,    SLOT(showTab(int)) );
-    d->tabs->setMinimumSize( d->tabs->sizeHint() );
+    d->tw->setTabBar( tb );
     setUpLayout();
 }
 
@@ -449,7 +533,7 @@ void QTabDialog::setTabBar( QTabBar* tb )
 */
 QTabBar* QTabDialog::tabBar() const
 {
-    return d->tabs;
+    return d->tw->tabBar();
 }
 
 /*!  Ensures that \a w is shown.  This is useful mainly for accelerators.
@@ -462,11 +546,11 @@ QTabBar* QTabDialog::tabBar() const
 
 void QTabDialog::showPage( QWidget * w )
 {
-    d->stack->raiseWidget( w );
+    d->tw->showPage( w );
 }
 
 
-/*!
+/*! \obsolete
   Returns TRUE if the page with object name \a name is enabled, and
   false if it is disabled.
 
@@ -476,26 +560,30 @@ void QTabDialog::showPage( QWidget * w )
   \sa setTabEnabled(), QWidget::isEnabled()
 */
 
-bool QTabDialog::isTabEnabled( const char *name ) const
+bool QTabDialog::isTabEnabled( const char* name ) const
 {
-    if ( !name || !*name )
+    if ( !name )
 	return FALSE;
-
     QObjectList * l
-	= ((QTabDialog *)this)->queryList( "QWidget", name, FALSE, FALSE );
-    bool r = FALSE;
+	= ((QTabDialog *)this)->queryList( "QWidget", name, FALSE, TRUE );
     if ( l && l->first() ) {
-	while( l->current() && !l->current()->isWidgetType() )
-	    l->next();
-	if ( l->current() )
-	    r = ((QWidget *)l->current())->isEnabled();
+	QWidget * w;
+	while( l->current() ) {
+	    while( l->current() && !l->current()->isWidgetType() )
+		l->next();
+	    w = (QWidget *)(l->current());
+	    if ( w ) {
+		return d->tw->isTabEnabled( w );
+	    }
+	}
     }
-    delete l;
-    return r;
+    return FALSE;
 }
 
 
-/*! Finds the page with object name \a name, enables/disables it
+/*!\obsolete
+
+  Finds the page with object name \a name, enables/disables it
   according to the value of \a enable, and redraws the page's tab
   appropriately.
 
@@ -512,27 +600,76 @@ bool QTabDialog::isTabEnabled( const char *name ) const
   \sa isTabEnabled(), QWidget::setEnabled()
 */
 
-void QTabDialog::setTabEnabled( const char * name, bool enable )
+void QTabDialog::setTabEnabled( const char* name, bool enable )
 {
-    if ( !name || !*name )
+    if ( !name )
 	return;
-    QObjectList * l = queryList( "QWidget", name, FALSE, FALSE );
+    QObjectList * l
+	= ((QTabDialog *)this)->queryList( "QWidget", name, FALSE, TRUE );
     if ( l && l->first() ) {
-	while( l->current() && !l->current()->isWidgetType() )
-	    l->next();
-	if ( l->current() ) {
-	    int id = d->stack->id( (QWidget *)l->current() );
-	    ((QWidget *)l->current())->setEnabled( enable );
-	    d->tabs->setTabEnabled( id, enable );
+	QObjectListIt it(*l);
+	QObject *o;
+	while( (o = it.current()) ) {
+	    ++it;
+	    if( o->isWidgetType() )
+		d->tw->setTabEnabled( (QWidget*)o, enable );
 	}
     }
-    delete l;
+}
+
+
+/* ### SHOULD THIS BE HERE?
+  Adds an Apply button to the dialog.  The button's text is set to \e
+  text (and defaults to "Apply").
+
+  The Apply button should apply the current settings in the dialog box
+  to the application, while keeping the dialog visible.
+
+  When Apply is clicked, the applyButtonPressed() signal is emitted.
+
+  If \a text is a
+  \link QString::operator!() null string\endlink,
+  no button is shown.
+
+  \sa setCancelButton() setDefaultButton() applyButtonPressed()
+*/
+
+
+/*!
+  Returns TRUE if the page \a w is enabled, and
+  false if it is disabled.
+
+  \sa setTabEnabled(), QWidget::isEnabled()
+*/
+
+bool QTabDialog::isTabEnabled( QWidget* w ) const
+{
+    return d->tw->isTabEnabled( w );
+}
+
+/*!
+  Enables/disables page \a w according to the value of \a enable, and
+  redraws the page's tab appropriately.
+
+  QTabWidget uses QWidget::setEnabled() internally, rather than keep a
+  separate flag.
+
+  Note that even a disabled tab/page may be visible.  If the page is
+  visible already, QTabWidget will not hide it, and if all the pages
+  are disabled, QTabWidget will show one of them.
+
+  \sa isTabEnabled(), QWidget::setEnabled()
+*/
+
+void QTabDialog::setTabEnabled( QWidget* w, bool enable)
+{
+    d->tw->setTabEnabled( w, enable );
 }
 
 
 /*!
   Add an Apply button to the dialog.  The button's text is set to \e
-  text (and defaults to "Apply").
+  text.
 
   The Apply button should apply the current settings in the dialog box
   to the application, while keeping the dialog visible.
@@ -541,62 +678,125 @@ void QTabDialog::setTabEnabled( const char * name, bool enable )
 
   \sa setCancelButton() setDefaultButton() applyButtonPressed()
 */
-
-void QTabDialog::setApplyButton( const char * text )
+void QTabDialog::setApplyButton( const QString &text )
 {
     if ( !text && d->ab ) {
-	delete d->ab;
-	d->ab = 0;
-	setSizes();
+        delete d->ab;
+        d->ab = 0;
+        setSizes();
     } else {
-	if ( !d->ab ) {
-	    d->ab = new QPushButton( this, "apply settings" );
-	    connect( d->ab, SIGNAL(clicked()),
-		     this, SIGNAL(applyButtonPressed()) );
-	    setUpLayout();
-	}
-	d->ab->setText( text );
-	setSizes();
-	d->ab->show();
+        if ( !d->ab ) {
+            d->ab = new QPushButton( this, "apply settings" );
+            connect( d->ab, SIGNAL(clicked()),
+                     this, SIGNAL(applyButtonPressed()) );
+            setUpLayout();
+        }
+        d->ab->setText( text );
+        setSizes();
+        //d->ab->show();
+    }
+}
+
+/*!
+  Adds an Apply button to the dialog.  The button's text is set to
+  a localizable "Apply".
+ */
+void QTabDialog::setApplyButton()
+{
+    setApplyButton( tr("Apply") );
+}
+
+
+/*!
+  Adds a Help button to the dialog.  The button's text is set to \e
+  text.
+
+  When Help is clicked, the helpButtonPressed() signal is emitted.
+
+  If \a text is a
+  \link QString::operator!() null string\endlink,
+  no button is shown.
+
+  \sa setApplyButton() setCancelButton() helpButtonPressed()
+*/
+
+void QTabDialog::setHelpButton( const QString &text )
+{
+    if ( !text ) {
+        delete d->hb;
+        d->hb = 0;
+        setSizes();
+    } else {
+        if ( !d->hb ) {
+            d->hb = new QPushButton( this, "give help" );
+            connect( d->hb, SIGNAL(clicked()),
+                     this, SIGNAL(helpButtonPressed()) );
+            setUpLayout();
+        }
+        d->hb->setText( text );
+        setSizes();
+        //d->hb->show();
     }
 }
 
 
 /*!
-  Add a Defaults button to the dialog.  The button's text is set to \e
-  text (and defaults to "Defaults").
+  Adds a Help button to the dialog.  The button's text is set to
+  a localizable "Help".
+ */
+void QTabDialog::setHelpButton()
+{
+    setHelpButton( tr("Help") );
+}
+
+/*!
+  Adds a Defaults button to the dialog.  The button's text is set to \e
+  text.
 
   The Defaults button should set the dialog (but not the application)
   back to the application defaults.
 
   When Defaults is clicked, the defaultButtonPressed() signal is emitted.
 
+  If \a text is a
+  \link QString::operator!() null string\endlink,
+  no button is shown.
+
   \sa setApplyButton() setCancelButton() defaultButtonPressed()
 */
 
-void QTabDialog::setDefaultButton( const char * text )
+void QTabDialog::setDefaultButton( const QString &text )
 {
     if ( !text ) {
-	delete d->db;
-	d->db = 0;
-	setSizes();
+        delete d->db;
+        d->db = 0;
+        setSizes();
     } else {
-	if ( !d->db ) {
-	    d->db = new QPushButton( this, "back to default" );
-	    connect( d->db, SIGNAL(clicked()),
-		     this, SIGNAL(defaultButtonPressed()) );
-	    setUpLayout();
-	}
-	d->db->setText( text );
-	setSizes();
-	d->db->show();
+        if ( !d->db ) {
+            d->db = new QPushButton( this, "back to default" );
+            connect( d->db, SIGNAL(clicked()),
+                     this, SIGNAL(defaultButtonPressed()) );
+            setUpLayout();
+        }
+        d->db->setText( text );
+        setSizes();
+        //d->db->show();
     }
 }
 
 
 /*!
-  Add a Cancel button to the dialog.  The button's text is set to \e
-  text (and defaults to "Cancel").
+  Adds a Defaults button to the dialog.  The button's text is set to
+  a localizable "Defaults".
+ */
+void QTabDialog::setDefaultButton()
+{
+    setDefaultButton( tr("Defaults") );
+}
+
+/*!
+  Adds a Cancel button to the dialog.  The button's text is set to \e
+  text.
 
   The cancel button should always return the application to the state
   it was in before the tab view popped up, or if the user has clicked
@@ -605,31 +805,46 @@ void QTabDialog::setDefaultButton( const char * text )
   When Cancel is clicked, the cancelButtonPressed() signal is emitted.
   The dialog is closed at the same time.
 
+  If \a text is a
+  \link QString::operator!() null string\endlink,
+  no button is shown.
+
   \sa setApplyButton setDefaultButton() cancelButtonPressed()
 */
 
-void QTabDialog::setCancelButton( const char * text )
+void QTabDialog::setCancelButton( const QString &text )
 {
     if ( !text ) {
-	delete d->cb;
-	d->cb = 0;
-	setSizes();
+        delete d->cb;
+        d->cb = 0;
+        setSizes();
     } else {
-	if ( !d->cb ) {
-	    d->cb = new QPushButton( this, "cancel dialog" );
-	    connect( d->cb, SIGNAL(clicked()),
-		     this, SIGNAL(cancelButtonPressed()) );
-	    connect( d->cb, SIGNAL(clicked()),
-		     this, SLOT(reject()) );
-	    setUpLayout();
-	}
-	d->cb->setText( text );
-	setSizes();
-	d->cb->show();
+        if ( !d->cb ) {
+            d->cb = new QPushButton( this, "cancel dialog" );
+            connect( d->cb, SIGNAL(clicked()),
+                     this, SIGNAL(cancelButtonPressed()) );
+            connect( d->cb, SIGNAL(clicked()),
+                     this, SLOT(reject()) );
+            setUpLayout();
+        }
+        d->cb->setText( text );
+        setSizes();
+        //d->cb->show();
     }
 }
 
 
+/*!
+  Adds a Cancel button to the dialog.  The button's text is set to
+  a localizable "Cancel".
+ */
+
+void QTabDialog::setCancelButton()
+{
+    setCancelButton( tr("Cancel") );
+}
+
+
 /*!  Sets up the layout manager for the tab dialog.
 
   \sa setSizes() setApplyButton() setCancelButton() setDefaultButton()
@@ -651,20 +866,11 @@ void QTabDialog::setUpLayout()
     // top margin
     d->tll->addSpacing( topMargin );
 
-    // tab bar
     QBoxLayout * tmp = new QBoxLayout( QBoxLayout::LeftToRight );
-    d->tll->addLayout( tmp, 0 );
-    tmp->addSpacing( leftMargin );
-    tmp->addWidget( d->tabs, 0 );
-    tmp->addStretch( 1 );
-    tmp->addSpacing( rightMargin + 2 );
-
-    // widget stack.
-    tmp = new QBoxLayout( QBoxLayout::LeftToRight );
-    d->tll->addLayout( tmp, 1 );
-    tmp->addSpacing( leftMargin + 1 );
-    tmp->addWidget( d->stack, 1 );
-    tmp->addSpacing( rightMargin + 2 );
+     d->tll->addLayout( tmp, 1 );
+     tmp->addSpacing( leftMargin );
+     tmp->addWidget( d->tw, 1);
+     tmp->addSpacing( rightMargin + 2 );
 
     d->tll->addSpacing( aboveButtonsMargin + 2 );
     QBoxLayout * buttonRow = new QBoxLayout( QBoxLayout::RightToLeft );
@@ -687,6 +893,11 @@ void QTabDialog::setUpLayout()
 	buttonRow->addSpacing( betweenButtonsMargin );
     }
 
+    if ( d->hb ) {
+	buttonRow->addWidget( d->hb, 0 );
+	buttonRow->addSpacing( betweenButtonsMargin );
+    }
+
     if ( d->ok ) {
 	buttonRow->addWidget( d->ok, 0 );
 	buttonRow->addSpacing( betweenButtonsMargin );
@@ -736,6 +947,14 @@ void QTabDialog::setSizes()
 	    bh = s.height();
     }
 
+    if ( d->hb ) {
+	s = d->hb->sizeHint();
+	if ( s.width() > bw )
+	    bw = s.width();
+	if ( s.height() > bh )
+	    bh = s.height();
+    }
+
     if ( d->cb ) {
 	s = d->cb->sizeHint();
 	if ( s.width() > bw )
@@ -754,12 +973,19 @@ void QTabDialog::setSizes()
 	d->ab->setFixedSize( bw, bh );
     if ( d->db )
 	d->db->setFixedSize( bw, bh );
+    if ( d->hb )
+	d->hb->setFixedSize( bw, bh );
     if ( d->cb )
 	d->cb->setFixedSize( bw, bh );
 
     // fiddle the tab chain so the buttons are in their natural order
     QWidget * w = d->ok;
 
+    if ( d->hb ) {
+	if ( w )
+	    setTabOrder( w, d->hb );
+	w = d->hb;
+    }
     if ( d->db ) {
 	if ( w )
 	    setTabOrder( w, d->db );
@@ -775,7 +1001,7 @@ void QTabDialog::setSizes()
 	    setTabOrder( w, d->cb );
 	w = d->cb;
     }
-    setTabOrder( w, d->tabs );
+    setTabOrder( w, d->tw );
 }
 
 
@@ -797,73 +1023,57 @@ void QTabDialog::resizeEvent( QResizeEvent * e )
 
 void QTabDialog::paintEvent( QPaintEvent * )
 {
-    if ( !d->tabs )
-	return;
-
-    QPainter p;
-    p.begin( this );
-
-    QRect s( d->stack->geometry() );
-
-    QCOORD t = s.top() - 1;
-    QCOORD b = s.bottom() + 2;
-    QCOORD r = s.right() + 2;
-    QCOORD l = s.left() - 1;
-
-    p.setPen( colorGroup().light() );
-    // note - this line overlaps the bottom line drawn by QTabBar
-    p.drawLine( l, t, r - 1, t );
-    p.drawLine( l, t + 1, l, b );
-    p.setPen( black );
-    p.drawLine( r, b, l,b );
-    p.drawLine( r, b-1, r, t );
-    p.setPen( colorGroup().dark() );
-    p.drawLine( l+1, b-1, r-1, b-1 );
-    p.drawLine( r-1, b-2, r-1, t+1 );
-
-    p.end();
 }
 
 
-QRect QTabDialog::childRect() const
-{
-    return d->stack->geometry();
-}
-
 
 /*!
-  Set the OK button's text to \a text (which defaults to "OK").
+  Set the OK button's text to \a text.
 
   When the OK button is clicked, the applyButtonPressed() signal is emitted,
   and the current settings in the dialog box should be applied to
   the application. Then the dialog closes.
 
+  If \a text is a
+  \link QString::operator!() null string\endlink,
+  no button is shown.
+
   \sa setCancelButton() setDefaultButton() applyButtonPressed()
 */
 
-void QTabDialog::setOkButton( const char * text )
+void QTabDialog::setOkButton( const QString &text )
 {
     if ( !text ) {
-	delete d->ok;
-	d->ok = 0;
-	setSizes();
+        delete d->ok;
+        d->ok = 0;
+        setSizes();
     } else {
-	if ( !d->ok ) {
-	    d->ok = new QPushButton( this, "ok" );
-	    connect( d->ok, SIGNAL(clicked()),
-		     this, SIGNAL(applyButtonPressed()) );
-	    setUpLayout();
-	}
-	d->ok->setText( text );
-	setSizes();
-	d->ok->show();
+        if ( !d->ok ) {
+            d->ok = new QPushButton( this, "ok" );
+            connect( d->ok, SIGNAL(clicked()),
+                     this, SIGNAL(applyButtonPressed()) );
+            setUpLayout();
+        }
+        d->ok->setText( text );
+        setSizes();
+        //d->ok->show();
     }
 }
+/*!
+  Adds an OK to the dialog.  The button's text is set to
+  a localizable "OK".
+ */
+
+void QTabDialog::setOkButton()
+{
+    setOkButton( tr("OK") );
+}
+
 
 /*!
   Old version of setOkButton(), provided for backward compatibility.
  */
-void QTabDialog::setOKButton( const char * text )
+void QTabDialog::setOKButton( const QString &text )
 {
     setOkButton( text );
 }
@@ -872,18 +1082,53 @@ void QTabDialog::setOKButton( const char * text )
 /*!  Returns the text in the tab for page \a w.
 */
 
-const char * QTabDialog::tabLabel( QWidget * w )
+QString QTabDialog::tabLabel( QWidget * w )
 {
-    QTab * t = d->tabs->tab( d->stack->id( w ) );
-    return t ? ((const char *)t->label) : 0;
+    return d->tw->tabLabel( w );
 }	
 
 
 /*!  Reimplemented to hndle a change of GUI style while on-screen.
 */
 
-void QTabDialog::styleChange( GUIStyle s )
+void QTabDialog::styleChange( QStyle& s )
 {
     QDialog::styleChange( s );
     setSizes();
 }
+
+
+/*!  Returns a pointer to the page currently being displayed by the
+tab dialog.  The tab dialog does its best to make sure that this value
+is never 0, but if you try hard enough it can be.
+*/
+
+QWidget * QTabDialog::currentPage() const
+{
+    return d->tw->currentPage();
+}
+
+/*!
+  Defines a new label for the tab of page \w
+ */
+void QTabDialog::changeTab( QWidget *w, const QString &label)
+{
+    d->tw->changeTab( w, label );
+}
+
+/*
+  Defines a new iconset and a new label for the tab of page \w
+ */
+void QTabDialog::changeTab( QWidget *w, const QIconSet& iconset, const QString &label)
+{
+    d->tw->changeTab( w, iconset, label );
+}
+
+/*! Removes page \a w from this stack of widgets.  Does not
+  delete \a w.
+  \sa showPage(), QTabWidget::removePage(), QWidgetStack::removeWidget()
+*/
+void QTabDialog::removePage( QWidget * w )
+{
+    d->tw->removePage( w );
+}
