@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qasyncimageio.cpp,v 1.38.2.6 1999/02/13 14:57:55 hanord Exp $
+** $Id: qasyncimageio.cpp,v 1.50 1999/06/02 22:50:23 hanord Exp $
 **
 ** Implementation of asynchronous image/movie loading classes
 **
@@ -7,27 +7,30 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qasyncimageio.h"
 #include "qpainter.h"
 #include "qlist.h"
-#include "qt_gif.h"
+#include "qgif.h"
 #include <stdlib.h>
 
 
+
 /*!
   \class QImageConsumer qasyncimageio.h
   \brief An abstraction used by QImageDecoder.
@@ -92,7 +95,7 @@
   is called.  Of course, the image may not have been decoded by then, in
   which case the frame should not be displayed until it is complete.
   A value of -1 (the assumed default) indicates that the image should
-  be diplayed even while it is only partially loaded.
+  be displayed even while it is only partially loaded.
 */
 
 /*!
@@ -121,7 +124,7 @@ static const int max_header = 32;
 
 
 
-// See qt_gif.h for important information regarding this option
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
 class Q_EXPORT QGIFFormat : public QImageFormat {
 public:
@@ -215,8 +218,8 @@ struct QImageDecoderPrivate {
     static void ensureFactories()
     {
 	if ( !factories ) {
-	    factories = new QListT<QImageFormatType>;
-// See qt_gif.h for important information regarding this option
+	    factories = new QList<QImageFormatType>;
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
 	    gif_decoder_factory = new QGIFFormatType;
 #endif
@@ -226,7 +229,7 @@ struct QImageDecoderPrivate {
 
     static QList<QImageFormatType> * factories;
 
-// See qt_gif.h for important information regarding this option
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
     static QGIFFormatType * gif_decoder_factory;
 #endif
@@ -236,7 +239,7 @@ struct QImageDecoderPrivate {
 };
 
 QList<QImageFormatType> * QImageDecoderPrivate::factories = 0;
-// See qt_gif.h for important information regarding this option
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
 QGIFFormatType * QImageDecoderPrivate::gif_decoder_factory = 0;
 #endif
@@ -246,7 +249,7 @@ void QImageDecoderPrivate::cleanup()
 {
     delete factories;
     factories = 0;
-// See qt_gif.h for important information regarding this option
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
     delete gif_decoder_factory;
     gif_decoder_factory = 0;
@@ -290,7 +293,7 @@ QImageDecoder::~QImageDecoder()
   returns FALSE.
 
   Returns the number of bytes consumed, 0 if consumption is complete,
-  and -1 if decoding fails dur to invalid data.
+  and -1 if decoding fails due to invalid data.
 */
 int QImageDecoder::decode(const uchar* buffer, int length)
 {
@@ -479,7 +482,7 @@ QImageFormat::~QImageFormat()
 	    uchar* buffer, int length)
 
   Returns a decoder for decoding an image which starts with the give bytes.
-  This function should only return a decoder if it is definate that the
+  This function should only return a decoder if it is definite that the
   decoder applies to data with the given header.  Returns 0 if there is
   insufficient data in the header to make a positive identification,
   or if the data is not recognized.
@@ -509,6 +512,10 @@ QImageFormatType::~QImageFormatType()
 }
 
 
+/*!
+  Returns TRUE if Qt was compiled with built-in GIF reading support,
+  otherwise FALSE.
+*/
 bool qt_builtin_gif_reader()
 {
 #if defined(QT_BUILTIN_GIF_READER)
@@ -518,7 +525,7 @@ bool qt_builtin_gif_reader()
 #endif
 }
 
-// See qt_gif.h for important information regarding this option
+// See qgif.h for important information regarding this option
 #if defined(QT_BUILTIN_GIF_READER) && QT_BUILTIN_GIF_READER == 1
 
 /* -- NOTDOC
@@ -551,7 +558,7 @@ QGIFFormat::QGIFFormat()
 */
 QGIFFormat::~QGIFFormat()
 {
-    delete[] globalcmap;
+    if (globalcmap) delete[] globalcmap;
 }
 
 
@@ -912,8 +919,10 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		} else {
 		    if (needfirst) {
 			firstcode=oldcode=code;
-			if (!out_of_bounds && !(preserve_trans && firstcode==trans))
-			    line[y][x] = firstcode;
+			if (!out_of_bounds) {
+			    if (!(preserve_trans && firstcode==trans))
+				line[y][x] = firstcode;
+			}
 			x++;
 			if (x>=swidth) out_of_bounds = TRUE;
 			needfirst=FALSE;
@@ -957,8 +966,10 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 			oldcode=incode;
 			while (sp>stack) {
 			    --sp;
-			    if (!out_of_bounds && !(preserve_trans && *sp==trans))
-				line[y][x] = *sp;
+			    if (!out_of_bounds) {
+				if (!(preserve_trans && *sp==trans))
+				    line[y][x] = *sp;
+			    }
 			    x++;
 			    if (x>=swidth) out_of_bounds = TRUE;
 			    if (x>right) {
@@ -1121,32 +1132,36 @@ void QGIFFormat::nextY(QImage& img, QImageConsumer* consumer)
 	    int i;
 	    my = QMIN(7, bottom-y);
 	    for (i=1; i<=my; i++)
-	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
+		memcpy(img.scanLine(y+i)+left, img.scanLine(y)+left,
+		    right-left+1);
 	    if (consumer && !out_of_bounds)
 	        consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=8;
-	    if (y>bottom) { interlace++; y=4; }
+	    if (y>bottom) { interlace++; y=top+4; }
 	} break;
       case 2:
 	{
 	    int i;
 	    my = QMIN(3, bottom-y);
 	    for (i=1; i<=my; i++)
-	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
+		memcpy(img.scanLine(y+i)+left, img.scanLine(y)+left,
+		    right-left+1);
     	    if (consumer && !out_of_bounds)
 	        consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=8;
-	    if (y>bottom) { interlace++; y=2; }
+	    if (y>bottom) { interlace++; y=top+2; }
 	} break;
       case 3:
 	{
+	    int i;
 	    my = QMIN(1, bottom-y);
-	    for (int i=1; i<=my; i++)
-	        memcpy(img.scanLine(y+i), img.scanLine(y), img.width());
+	    for (i=1; i<=my; i++)
+		memcpy(img.scanLine(y+i)+left, img.scanLine(y)+left,
+		    right-left+1);
 	    if (consumer && !out_of_bounds)
 	        consumer->changed(QRect(left, y, right-left+1, my+1));
 	    y+=4;
-	    if (y>bottom) { interlace++; y=1; }
+	    if (y>bottom) { interlace++; y=top+1; }
 	} break;
       case 4:
 	if (consumer && !out_of_bounds)
