@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qasyncio.h,v 1.3 1997/09/25 15:28:27 hanord Exp $
+** $Id: qasyncio.h,v 1.10 1999/04/28 09:55:40 aavit Exp $
 **
 **		      ***   INTERNAL HEADER FILE   ***
 **
@@ -9,34 +9,39 @@
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #ifndef QASYNCIO_H
 #define QASYNCIO_H
 
+#ifndef QT_H
 #include "qobject.h"
 #include "qsignal.h"
 #include "qtimer.h"
+#endif // QT_H
 
 class QIODevice;
 
-class QAsyncIO {
+class Q_EXPORT QAsyncIO {
 public:
     virtual ~QAsyncIO();
-    void connect(QObject*, const char* member);
+    void connect(QObject*, const char *member);
 
 protected:
     void ready();
@@ -45,7 +50,7 @@ private:
     QSignal signal;
 };
 
-class QDataSink : public QAsyncIO {
+class Q_EXPORT QDataSink : public QAsyncIO {
 public:
     // Call this to know how much I can take.
     virtual int readyToReceive()=0;
@@ -54,7 +59,7 @@ public:
     void maybeReady();
 };
 
-class QDataSource : public QAsyncIO {
+class Q_EXPORT QDataSource : public QAsyncIO {
 public:
     virtual int readyToSend()=0; // returns -1 when never any more ready
     virtual void sendTo(QDataSink*, int count)=0;
@@ -65,7 +70,7 @@ public:
     virtual void rewind();
 };
 
-class QIODeviceSource : public QDataSource {
+class Q_EXPORT QIODeviceSource : public QDataSource {
     const int buf_size;
     uchar *buffer;
     QIODevice* iod;
@@ -82,7 +87,7 @@ public:
     void rewind();
 };
 
-class QDataPump : public QObject {
+class Q_EXPORT QDataPump : public QObject {
     Q_OBJECT
     int interval;
     QTimer timer;
