@@ -1,30 +1,31 @@
 /****************************************************************************
-** $Id: qbitmap.cpp,v 2.1 1997/08/06 15:05:33 hanord Exp $
+** $Id: qbitmap.cpp,v 2.14 1999/06/03 10:27:23 paul Exp $
 **
 ** Implementation of QBitmap class
 **
 ** Created : 941020
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qbitmap.h"
 #include "qimage.h"
 
-RCSTAG("$Id: qbitmap.cpp,v 2.1 1997/08/06 15:05:33 hanord Exp $");
-
 
 /*!
   \class QBitmap qbitmap.h
@@ -59,6 +60,7 @@ RCSTAG("$Id: qbitmap.cpp,v 2.1 1997/08/06 15:05:33 hanord Exp $");
   Classes\endlink
 */
 
+
 /*!
   Constructs a null bitmap.
   \sa QPixmap::isNull()
@@ -69,33 +71,45 @@ QBitmap::QBitmap()
     data->bitmap = TRUE;
 }
 
-/*!
-  Constructs a bitmap with \e w width and \e h height.
 
-  The contents of the bitmap is uninitialized if \e clear is FALSE, otherwise
-  it is filled with pixel value 0 (the QColor \c color0).
+/*!
+  Constructs a bitmap with \a w width and \a h height.
+
+  The contents of the bitmap is uninitialized if \a clear is FALSE, otherwise
+  it is filled with pixel value 0 (the QColor \a Qt::color0).
+
+  The optional \a optimization argument specifies the optimization
+  setting for the bitmap.  The default optimization should be used
+  in most cases.  Games and other pixmap-intensive applications may
+  benefit from setting this argument.
+
+  \sa QPixmap::setOptimization(), QPixmap::setDefaultOptimization()
 */
 
-QBitmap::QBitmap( int w, int h, bool clear )
-    : QPixmap( w, h, 1 )
+QBitmap::QBitmap( int w, int h, bool clear,
+		  QPixmap::Optimization optimization )
+    : QPixmap( w, h, 1, optimization )
 {
     data->bitmap = TRUE;
     if ( clear )
-	fill( color0 );
+	fill( Qt::color0 );
 }
 
+
 /*!
-  Overloaded constructor; takes a QSize parameter instead of \e (w,h).
+  \overload
 */
 
-QBitmap::QBitmap( const QSize &size, bool clear )
-    : QPixmap( size, 1 )
+QBitmap::QBitmap( const QSize &size, bool clear,
+		  QPixmap::Optimization optimization )
+    : QPixmap( size, 1, optimization )
 {
     data->bitmap = TRUE;
     if ( clear )
-	fill( color0 );
+	fill( Qt::color0 );
 }
 
+
 /*!
   Constructs a bitmap with \e w width and \e h height and sets the contents
   to \e bits.
@@ -117,6 +131,7 @@ QBitmap::QBitmap( int w, int h, const uchar *bits, bool isXbitmap )
     data->bitmap = TRUE;
 }
 
+
 /*!
   Overloaded constructor; takes a QSize parameter instead of \e (w,h).
 */
@@ -127,6 +142,7 @@ QBitmap::QBitmap( const QSize &size, const uchar *bits, bool isXbitmap )
     data->bitmap = TRUE;
 }
 
+
 /*!
   Constructs a bitmap which is a copy of \e bitmap.
 */
@@ -136,7 +152,8 @@ QBitmap::QBitmap( const QBitmap &bitmap )
 {
 }
 
-/*!  
+
+/*!
   Constructs a pixmap from the file \e fileName. If the file does not
   exist, or is of an unknown format, the pixmap becomes a null pixmap.
 
@@ -146,7 +163,7 @@ QBitmap::QBitmap( const QBitmap &bitmap )
   QPixmap::save(), QPixmap::imageFormat()
 */
 
-QBitmap::QBitmap( const char *fileName, const char *format )
+QBitmap::QBitmap( const QString& fileName, const char *format )
     : QPixmap() // Will set bitmap to null bitmap, explicit call for clarity
 {
     data->bitmap = TRUE;
@@ -180,16 +197,14 @@ QBitmap &QBitmap::operator=( const QBitmap &bitmap )
 QBitmap &QBitmap::operator=( const QPixmap &pixmap )
 {
     if ( pixmap.isNull() ) {			// a null pixmap
-	QBitmap bm;
-	bm.data->optim = data->optim;
+	QBitmap bm( 0, 0, FALSE, pixmap.optimization() );
 	QBitmap::operator=(bm);
     } else if ( pixmap.depth() == 1 ) {		// 1-bit pixmap
 	if ( pixmap.isQBitmap() ) {		// another QBitmap
 	    QPixmap::operator=(pixmap);		// shallow assignment
 	} else {				// not a QBitmap, but 1-bit
-	    QBitmap bm( pixmap.size() );
+	    QBitmap bm( pixmap.size(), FALSE, pixmap.optimization() );
 	    bitBlt( &bm, 0,0, &pixmap, 0,0,pixmap.width(),pixmap.height() );
-	    bm.data->optim = data->optim;
 	    QBitmap::operator=(bm);
 	}
     } else {					// n-bit depth pixmap
@@ -200,6 +215,7 @@ QBitmap &QBitmap::operator=( const QPixmap &pixmap )
     return *this;
 }
 
+
 /*!
   Converts the image \e image to a bitmap that is assigned to this bitmap.
   Returns a reference to the bitmap.
@@ -216,8 +232,7 @@ QBitmap &QBitmap::operator=( const QImage &image )
 
 
 /*!
-  Transforms the bitmap using \e matrix, and returns the transformed
-  bitmap.
+  Returns a copy of the bitmap that is transformed using \e matrix.
 
   This function does exactly the same as QPixmap::xForm(), except that
   it returns a QBitmap instead of a QPixmap.
@@ -229,6 +244,10 @@ QBitmap QBitmap::xForm( const QWMatrix &matrix ) const
 {
     QPixmap pm = QPixmap::xForm( matrix );
     QBitmap bm;
+    // Here we fake the pixmap to think it's a QBitmap. With this trick,
+    // the QBitmap::operator=(const QPixmap&) will just refer the
+    // pm.data and we do not need to perform a bitBlt.
+    pm.data->bitmap = TRUE;
     bm = pm;
     return bm;
 }
