@@ -1,41 +1,38 @@
 /****************************************************************************
-** $Id: qbrush.h,v 2.2 1997/08/06 15:05:33 hanord Exp $
+** $Id: qbrush.h,v 2.9 1999/04/28 09:55:43 aavit Exp $
 **
 ** Definition of QBrush class
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #ifndef QBRUSH_H
 #define QBRUSH_H
 
+#ifndef QT_H
 #include "qcolor.h"
 #include "qshared.h"
+#endif // QT_H
 
 
-enum BrushStyle					// brush style
-      { NoBrush, SolidPattern,
-	Dense1Pattern, Dense2Pattern, Dense3Pattern, Dense4Pattern,
-	Dense5Pattern, Dense6Pattern, Dense7Pattern,
-	HorPattern, VerPattern, CrossPattern,
-	BDiagPattern, FDiagPattern, DiagCrossPattern, CustomPattern=24 };
-
-
-class QBrush
+class Q_EXPORT QBrush: public Qt
 {
 friend class QPainter;
 public:
@@ -74,8 +71,8 @@ private:
   QBrush stream functions
  *****************************************************************************/
 
-QDataStream &operator<<( QDataStream &, const QBrush & );
-QDataStream &operator>>( QDataStream &, QBrush & );
+Q_EXPORT QDataStream &operator<<( QDataStream &, const QBrush & );
+Q_EXPORT QDataStream &operator>>( QDataStream &, QBrush & );
 
 
 #endif // QBRUSH_H
