@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qclipboard.cpp,v 2.10 1998/07/03 00:09:30 hanord Exp $
+** $Id: qclipboard.cpp,v 2.19 1999/06/11 09:25:56 warwick Exp $
 **
 ** Implementation of QClipboard class
 **
@@ -7,24 +7,31 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qclipboard.h"
 #include "qapplication.h"
+#include "qdragobject.h"
 #include "qpixmap.h"
 
+#if defined(QT_MAKEDLL)
+#define QApplication QBaseApplication
+#endif
+
 /*!
   \class QClipboard qclipboard.h
   \brief The QClipboard class provides access to the window system clipboard.
@@ -34,30 +41,23 @@
   The clipboard offers a simple mechanism to copy and paste data between
   applications.
 
-  QClipboard supports these formats (a format is identified by a string):
-  <ul>
-  <li>"TEXT", zero-terminated char *.
-  <li>"PIXMAP" as provided by QPixmap.
-  </ul>
-
-  The "PIXMAP" format is not implemented in this version of Qt.
+  QClipboard supports the same formats that
+  \link QDragObject drag and drop\endlink supports, and uses much
+  of the same mechanisms.
 
   Only a single QClipboard object may exist in an application. This is
-  because QClipboard is a shared window system resource. It is not
-  possible to create a QClipboard object the standard C++ way (the
-  constructor and destructor are private member functions, but accessible
-  to QApplication since it is a friend class).	Call
+  because QClipboard is a shared window system resource.  Call
   QApplication::clipboard() to access the clipboard.
 
   Example:
   \code
     QClipboard *cb = QApplication::clipboard();
-    const char *text;
+    QString text;
 
     // Copy text from the clipboard (paste)
     text = cb->text();
     if ( text )
-	debug( "The clipboard contains: %s", text );
+	qDebug( "The clipboard contains: %s", text );
 
     // Copy text into the clipboard
     cb->setText( "This text can be pasted by other programs" );
@@ -97,50 +97,6 @@ QClipboard::~QClipboard()
 */
 
 
-/*!
-  Returns the clipboard text, or null if the clipboard does not contains
-  any text.
-  \sa setText()
-*/
-
-const char *QClipboard::text() const
-{
-    return (const char *)data("TEXT");
-}
-
-/*!
-  Copies \e text into the clipboard.
-  \sa text()
-*/
-
-void QClipboard::setText( const char *text )
-{
-    setData( "TEXT", (void *)text );
-}
-
-
-/*!
-  Returns the clipboard pixmap, or null if the clipboard does not contains
-  any pixmap.
-  \sa setText()
-*/
-
-QPixmap *QClipboard::pixmap() const
-{
-    return (QPixmap *)data("PIXMAP");
-}
-
-/*!
-  Copies \e pixmap into the clipboard.
-  \sa pixmap()
-*/
-
-void QClipboard::setPixmap( const QPixmap &pixmap )
-{
-    setData( "PIXMAP", (void *)&pixmap );
-}
-
-
 /*****************************************************************************
   QApplication member functions related to QClipboard.
  *****************************************************************************/
@@ -166,3 +122,87 @@ QClipboard *QApplication::clipboard()
     }
     return (QClipboard *)qt_clipboard;
 }
+
+
+/*!
+  Returns the clipboard text, or a
+  \link QString::operator!() null string\endlink
+  if the clipboard does not contain any text.
+  \sa setText()
+*/
+
+QString QClipboard::text() const
+{
+    QString r;
+    QTextDrag::decode(data(),r);
+    return r;
+}
+
+/*!
+  Copies \e text into the clipboard.
+  \sa text(), setData()
+*/
+
+void QClipboard::setText( const QString &text )
+{
+    setData(new QTextDrag(text));
+}
+
+
+/*!
+  Returns the clipboard image, or null if the clipboard does not contain
+  an image.
+  \sa setText()
+*/
+
+QImage QClipboard::image() const
+{
+    QImage r;
+    QImageDrag::decode(data(),r);
+    return r;
+}
+
+/*!
+  Copies \e image into the clipboard.
+
+  This is just a shorthand for:
+  \code
+    setData(new QImageDrag(image))
+  \endcode
+
+  \sa image(), setData()
+*/
+
+void QClipboard::setImage( const QImage &image )
+{
+    setData(new QImageDrag(image));
+}
+
+
+/*!
+  Returns the clipboard pixmap, or null if the clipboard does not contains
+  any pixmap. Note that this usually looses more information than image().
+  \sa setText(), image()
+*/
+
+QPixmap QClipboard::pixmap() const
+{
+    QPixmap r;
+    QImageDrag::decode(data(),r);
+    return r;
+}
+
+/*!
+  Copies \e pixmap into the clipboard.
+  Note that this usually looses more information than setImage(),
+  as the data may be converted to an image for transfer.
+  \sa pixmap()
+*/
+
+void QClipboard::setPixmap( const QPixmap &pixmap )
+{
+    // *could* just use the handle, but that is X hackery, MIME is better.
+    setData(new QImageDrag(pixmap.convertToImage()));
+}
+
+
