@@ -1,30 +1,34 @@
 /****************************************************************************
-** $Id: qcolor.cpp,v 2.11 1997/09/19 13:12:32 hanord Exp $
+** $Id: qcolor.cpp,v 2.39.2.1 1999/08/10 14:26:15 paul Exp $
 **
 ** Implementation of QColor class
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qcolor.h"
-#include "qdstream.h"
+#include "qpaintdevice.h"
+#include "qnamespace.h"
+#include "qdatastream.h"
 #include <stdlib.h>
-
-RCSTAG("$Id: qcolor.cpp,v 2.11 1997/09/19 13:12:32 hanord Exp $");
+#include <ctype.h>
 
 
 /*!
@@ -35,26 +39,26 @@ RCSTAG("$Id: qcolor.cpp,v 2.11 1997/09/19 13:12:32 hanord Exp $");
   \ingroup drawing
 
   A color is normally specified in terms of RGB (red,green and blue)
-  components, but it is also possible to specify HSV (hue,saturation and
-  value) or set a color name (X11 color database, no support yet
-  on other window systems).
+  components, but it is also possible to specify HSV (hue,saturation
+  and value) or set a color name (the names are copied from from the
+  X11 color database).
 
   In addition to the RGB value, a QColor also has a pixel value.  This
   value is used by the underlying window system to refer to a color.  It
   can be thought of as an index into the display hardware's color table.
 
-  There are 19 predefined global QColor objects:
-  \c black, \c white, \c darkGray, \c gray, \c lightGray, \c red, \c green,
-  \c blue, \c cyan, \c magenta, \c yellow, \c darkRed, \c darkGreen,
-  \c darkBlue, \c darkCyan, \c darkMagenta, \c darkYellow, \c color0 and
-  \c color1.
+  There are 19 predefined QColor objects: \c black, \c white, \c
+  darkGray, \c gray, \c lightGray, \c red, \c green, \c blue, \c cyan,
+  \c magenta, \c yellow, \c darkRed, \c darkGreen, \c darkBlue, \c
+  darkCyan, \c darkMagenta, \c darkYellow, \c color0 and \c color1.
 
-  The colors \c color0 (zero pixel value) and \c color1 (non-zero pixel value)
-  are special colors for drawing in \link QBitmap bitmaps\endlink.
+  The colors \c color0 (zero pixel value) and \c color1 (non-zero
+  pixel value) are special colors for drawing in \link QBitmap
+  bitmaps\endlink.
 
-  The QColor class has an efficient, dynamic color allocation strategy.
-  A color is normally allocated the first time it is used (lazy allocation),
-  that is, whenever the pixel() function is called:
+  The QColor class has an efficient, dynamic color allocation
+  strategy.  A color is normally allocated the first time it is used
+  (lazy allocation), that is, whenever the pixel() function is called:
 
   <ol>
   <li>Is the pixel value valid? If it is, just return it, otherwise,
@@ -68,7 +72,32 @@ RCSTAG("$Id: qcolor.cpp,v 2.11 1997/09/19 13:12:32 hanord Exp $");
   color and save it in the internal table.
   </ol>
 
-  \sa QPalette, QColorGroup, QApplication::setColorSpec()
+  Since many people don't know the HSV color model very well, we'll
+  cover it briefly here.
+
+  The RGB model is hardware-oriented.  Its representation is close to
+  what most monitors show.  In contrast, HSV represents color in a way
+  more suited to traditional human perception of color.  For example,
+  the relationships "stronger than", "darker than", "the opposite of"
+  are easily expressed in HSV, but are much harder to express in RGB.
+
+  HSV, like RGB, has three components.  They are: <ul> <li> H, for
+  hue, is either 0-360 if the color is chromatic (not gray), or
+  meaningless if it is gray.  It represents degrees on the color wheel
+  familiar to most people.  Red is 0 (degrees), green is 120 and blue
+  is 240. <li> S, for saturation, is 0-255 and the bigger it is, the
+  stronger the color is.  Grayish colors have saturation near 0, very
+  strong colors have saturation near 255. <li> V, for value, is 0-255
+  and represents lightness or brightness of the color.  0 is black,
+  255 is far from black as possible. </ul>
+
+  Here are some examples: Pure red is H=0, S=255, V=255.  A dark red,
+  moving slightly towards the magenta, could be H=350 (equvalent to
+  -10), S=255, V=180.  A grayish light red could have H about 0 (say
+  350-359 or 0-10), S about 50-100, and S=255.
+
+  \sa QPalette, QColorGroup, QApplication::setColorSpec(),
+  <a href="http://www.inforamp.net/~poynton/Poynton-color.html">Color FAQ.</a>
 */
 
 /*****************************************************************************
@@ -83,33 +112,43 @@ RCSTAG("$Id: qcolor.cpp,v 2.11 1997/09/19 13:12:32 hanord Exp $");
 #define COLOR1_PIX 1
 #endif
 
-const QColor color0	( 0x00ffffff, COLOR0_PIX );
-const QColor color1	( 0x00000000, COLOR1_PIX );
-const QColor black	(   0,	 0,   0 );
-const QColor white	( 255, 255, 255 );
-const QColor darkGray	( 128, 128, 128 );
-const QColor gray	( 160, 160, 164 );
-const QColor lightGray	( 192, 192, 192 );
-const QColor red	( 255,	 0,   0 );
-const QColor green	(   0, 255,   0 );
-const QColor blue	(   0,	 0, 255 );
-const QColor cyan	(   0, 255, 255 );
-const QColor magenta	( 255,	 0, 255 );
-const QColor yellow	( 255, 255,   0 );
-const QColor darkRed	( 128,	 0,   0 );
-const QColor darkGreen	(   0, 128,   0 );
-const QColor darkBlue	(   0,	 0, 128 );
-const QColor darkCyan	(   0, 128, 128 );
-const QColor darkMagenta( 128,	 0, 128 );
-const QColor darkYellow ( 128, 128,   0 );
+static QColor stdcol[19];
+
+QT_STATIC_CONST_IMPL QColor & Qt::color0 = stdcol[0];
+QT_STATIC_CONST_IMPL QColor & Qt::color1  = stdcol[1];
+QT_STATIC_CONST_IMPL QColor & Qt::black  = stdcol[2];
+QT_STATIC_CONST_IMPL QColor & Qt::white = stdcol[3];
+QT_STATIC_CONST_IMPL QColor & Qt::darkGray = stdcol[4];
+QT_STATIC_CONST_IMPL QColor & Qt::gray = stdcol[5];
+QT_STATIC_CONST_IMPL QColor & Qt::lightGray = stdcol[6];
+QT_STATIC_CONST_IMPL QColor & Qt::red = stdcol[7];
+QT_STATIC_CONST_IMPL QColor & Qt::green = stdcol[8];
+QT_STATIC_CONST_IMPL QColor & Qt::blue = stdcol[9];
+QT_STATIC_CONST_IMPL QColor & Qt::cyan = stdcol[10];
+QT_STATIC_CONST_IMPL QColor & Qt::magenta = stdcol[11];
+QT_STATIC_CONST_IMPL QColor & Qt::yellow = stdcol[12];
+QT_STATIC_CONST_IMPL QColor & Qt::darkRed = stdcol[13];
+QT_STATIC_CONST_IMPL QColor & Qt::darkGreen = stdcol[14];
+QT_STATIC_CONST_IMPL QColor & Qt::darkBlue = stdcol[15];
+QT_STATIC_CONST_IMPL QColor & Qt::darkCyan = stdcol[16];
+QT_STATIC_CONST_IMPL QColor & Qt::darkMagenta = stdcol[17];
+QT_STATIC_CONST_IMPL QColor & Qt::darkYellow = stdcol[18];
 
 
 /*****************************************************************************
   QColor member functions
  *****************************************************************************/
 
-bool QColor::ginit  = FALSE;			// global color not init'ed
-bool QColor::lalloc = TRUE;			// lazy color allocation
+bool QColor::color_init   = FALSE;		// color system not initialized
+bool QColor::globals_init = FALSE;		// global color not initialized
+bool QColor::lazy_alloc = TRUE;			// lazy color allocation
+
+
+QColor* QColor::globalColors()
+{
+    return stdcol;
+}
+
 
 /*!
   Initializes the global colors.  This function is called if a global
@@ -127,30 +166,30 @@ bool QColor::lalloc = TRUE;			// lazy color allocation
   \endcode
 */
 
-void QColor::initglobals()
+void QColor::initGlobalColors()
 {
-    ginit = TRUE;
-    ((QColor*)(&::color0))->pix = COLOR0_PIX;
-    ((QColor*)(&::color1))->pix = COLOR1_PIX;
-    ((QColor*)(&::color0))->rgbVal = 0x00ffffff;
-    ((QColor*)(&::color1))->rgbVal = 0;
-    ((QColor*)(&::black))	->setRgb(   0,	 0,   0 );
-    ((QColor*)(&::white))	->setRgb( 255, 255, 255 );
-    ((QColor*)(&::darkGray))	->setRgb( 128, 128, 128 );
-    ((QColor*)(&::gray))	->setRgb( 160, 160, 164 );
-    ((QColor*)(&::lightGray))	->setRgb( 192, 192, 192 );
-    ((QColor*)(&::red))		->setRgb( 255,	 0,   0 );
-    ((QColor*)(&::green))	->setRgb(   0, 255,   0 );
-    ((QColor*)(&::blue))	->setRgb(   0,	0,  255 );
-    ((QColor*)(&::cyan))	->setRgb(   0, 255, 255 );
-    ((QColor*)(&::magenta))	->setRgb( 255,	0,  255 );
-    ((QColor*)(&::yellow))	->setRgb( 255, 255,   0 );
-    ((QColor*)(&::darkRed))	->setRgb( 128,	0,    0 );
-    ((QColor*)(&::darkGreen))	->setRgb(   0, 128,   0 );
-    ((QColor*)(&::darkBlue))	->setRgb(   0,	0,  128 );
-    ((QColor*)(&::darkCyan))	->setRgb(   0, 128, 128 );
-    ((QColor*)(&::darkMagenta)) ->setRgb( 128,	0,  128 );
-    ((QColor*)(&::darkYellow))	->setRgb( 128, 128,   0 );
+    globals_init = TRUE;
+    stdcol[ 0].pix = COLOR0_PIX;
+    stdcol[ 1].pix = COLOR1_PIX;
+    stdcol[ 0].rgbVal = 0x00ffffff;
+    stdcol[ 1].rgbVal = 0;
+    stdcol[ 2].setRgb(   0,	 0,   0 );
+    stdcol[ 3].setRgb( 255, 255, 255 );
+    stdcol[ 4].setRgb( 128, 128, 128 );
+    stdcol[ 5].setRgb( 160, 160, 164 );
+    stdcol[ 6].setRgb( 192, 192, 192 );
+    stdcol[ 7].setRgb( 255,	 0,   0 );
+    stdcol[ 8].setRgb(   0, 255,   0 );
+    stdcol[ 9].setRgb(   0,	0,  255 );
+    stdcol[10].setRgb(   0, 255, 255 );
+    stdcol[11].setRgb( 255,	0,  255 );
+    stdcol[12].setRgb( 255, 255,   0 );
+    stdcol[13].setRgb( 128,	0,    0 );
+    stdcol[14].setRgb(   0, 128,   0 );
+    stdcol[15].setRgb(   0,	0,  128 );
+    stdcol[16].setRgb(   0, 128, 128 );
+    stdcol[17].setRgb( 128,	0,  128 );
+    stdcol[18].setRgb( 128, 128,   0 );
 }
 
 
@@ -163,26 +202,46 @@ void QColor::initglobals()
   \sa isValid()
 */
 
+
 /*!
   \fn QColor::QColor( int r, int g, int b )
 
-  Constructs a color with the RGB value (r,g,b).
-
-  \e r, \e g and \e b must be in the range 0..255.
+  Constructs a color with the RGB value \a (r,g,b).
+  \a r, \a g and \a b must be in the range 0..255.
 
   \sa setRgb()
 */
 
-/*!
-  Constructs a color with the RGB \e or HSV value \e (x,y,z).
 
-  The \e (x,y,z) triplet defines an RGB value if \e colorSpec == \c
-  QColor::Rgb.	\e x (red), \e y (green) and \e z (blue) must be in the
+/*!
+  Constructs a color with a RGB value and a custom pixel value.
+
+  If the \a pixel = 0xffffffff, then the color uses the RGB value in a
+  standard way.	 If \a pixel is something else, then the pixel value will
+  be set directly to \a pixel (skips the normal allocation procedure).
+*/
+
+QColor::QColor( QRgb rgb, uint pixel )
+{
+    if ( pixel == 0xffffffff ) {
+	setRgb( rgb );
+    } else {
+	rgbVal = rgb | RGB_DIRECT;
+	pix    = pixel;
+    }
+}
+
+
+/*!
+  Constructs a color with the RGB \e or HSV value \a (x,y,z).
+
+  The \e (x,y,z) triplet defines an RGB value if \a colorSpec == \c
+  QColor::Rgb.	\a x (red), \a y (green) and \a z (blue) must be in the
   range 0..255.
 
-  The \e (x,y,z) triplet defines a HSV value if \e colorSpec == \c
-  QColor::Hsv.	\e x (hue) must be in the range -1..360 (-1 means
-  achromatic), and \e y (saturation) and \e z (value) must be in the range
+  The \a (x,y,z) triplet defines a HSV value if \a colorSpec == \c
+  QColor::Hsv.	\a x (hue) must be in the range -1..360 (-1 means
+  achromatic), and \a y (saturation) and \a z (value) must be in the range
   0..255.
 
   \sa setRgb(), setHsv()
@@ -196,51 +255,98 @@ QColor::QColor( int x, int y, int z, Spec colorSpec )
 	setRgb( x, y, z );
 }
 
+
 /*!
-  Constructs a named color, i.e. loads the color from the color database.
+  Constructs a named color in the same way as setNamedColor().
+  \sa setNamedColor()
+*/
+
+QColor::QColor( const QString& name )
+{
+    setNamedColor( name );
+}
+
+
+/*!
+  Constructs a named color in the same way as setNamedColor().
   \sa setNamedColor()
 */
 
 QColor::QColor( const char *name )
 {
-    setNamedColor( name );
+    setNamedColor( QString(name) );
 }
 
+
+
 /*!
-  Constructs a color that is a copy of \e c.
+  Constructs a color that is a copy of \a c.
 */
 
 QColor::QColor( const QColor &c )
 {
-    if ( !ginit )
-	initglobals();
+    if ( !globals_init )
+	initGlobalColors();
     rgbVal = c.rgbVal;
     pix	   = c.pix;
 }
 
 
+inline static int hex2int( QChar hexchar )
+{
+    int v;
+    if ( hexchar.isDigit() )
+	v = hexchar.digitValue();
+    else if ( hexchar >= 'A' && hexchar <= 'F' )
+	v = hexchar.cell() - 'A' + 10;
+    else if ( hexchar >= 'a' && hexchar <= 'f' )
+	v = hexchar.cell() - 'a' + 10;
+    else
+	v = 0;
+    return v;
+}
+
+
 /*!
   Sets the RGB value to that of the named color.
 
-  The "#RRGGBB" color format is supported on all platforms, where
-  RR, GG, and BB and hex digits.
-
-  A color name may also be a color in the X color database (rgb.txt) ,
-  e.g.  "steelblue" or "gainsboro".  The X color names also work under
-  Qt for Windows.
+  The color name may be in one of five formats: <ul>
+  <li> #RGB (each of R, G and B is a single hex digit)
+  <li> #RRGGBB
+  <li> #RRRGGGBBB
+  <li> #RRRRGGGGBBBB
+  <li> A named from the X color database (rgb.txt),  e.g.
+  "steelblue" or "gainsboro".  These color names also work
+  under Qt for Windows.
 */
 
-void QColor::setNamedColor( const char *name )
+void QColor::setNamedColor( const QString& name )
 {
-    if ( strlen(name) == 0 ) {
-	setRgb( 0, 0, 0 );
-    } else if ( name[0]=='#' ) {
-	QString hex = name+1;
-	int bpc = (strlen(name)-1)/3;
-	int i=0;
-	int r = strtol(hex.mid(i,bpc),0,16)*256/(1<<(4*bpc)); i+=bpc;
-	int g = strtol(hex.mid(i,bpc),0,16)*256/(1<<(4*bpc)); i+=bpc;
-	int b = strtol(hex.mid(i,bpc),0,16)*256/(1<<(4*bpc));
+    if ( name.isEmpty() ) {
+	setRgb( 0 );
+    } else if ( name[0] == '#' ) {
+	const QChar *p = name.unicode()+1;
+	int len = name.length()-1;
+	int r, g, b;
+	if ( len == 12 ) {
+	    r = (hex2int(p[0]) << 4) + hex2int(p[1]);
+	    g = (hex2int(p[4]) << 4) + hex2int(p[5]);
+	    b = (hex2int(p[8]) << 4) + hex2int(p[9]);
+	} else if ( len == 9 ) {
+	    r = (hex2int(p[0]) << 4) + hex2int(p[1]);
+	    g = (hex2int(p[3]) << 4) + hex2int(p[4]);
+	    b = (hex2int(p[6]) << 4) + hex2int(p[7]);
+	} else if ( len == 6 ) {
+	    r = (hex2int(p[0]) << 4) + hex2int(p[1]);
+	    g = (hex2int(p[2]) << 4) + hex2int(p[3]);
+	    b = (hex2int(p[4]) << 4) + hex2int(p[5]);
+	} else if ( len == 3 ) {
+	    r = hex2int(p[0]) << 4;
+	    g = hex2int(p[1]) << 4;
+	    b = hex2int(p[2]) << 4;
+	} else {
+	    r = g = b = 0;
+	}
 	setRgb( r, g, b );
     } else {
 	setSystemNamedColor( name );
@@ -254,8 +360,8 @@ void QColor::setNamedColor( const char *name )
 
 QColor &QColor::operator=( const QColor &c )
 {
-    if ( !ginit )
-	initglobals();
+    if ( !globals_init )
+	initGlobalColors();
     rgbVal = c.rgbVal;
     pix	   = c.pix;
     return *this;
@@ -264,7 +370,7 @@ QColor &QColor::operator=( const QColor &c )
 
 /*!
   \fn bool QColor::isValid() const
-  Returns TRUE if the color is invalid, i.e. it was constructed using the
+  Returns FALSE if the color is invalid, i.e. it was constructed using the
   default constructor.
 */
 
@@ -279,7 +385,7 @@ QColor &QColor::operator=( const QColor &c )
 #undef max
 #undef min
 
-/*!
+/*!\obsolete
   Returns the current RGB value as HSV.
 
   \arg \e *h, hue.
@@ -295,9 +401,29 @@ QColor &QColor::operator=( const QColor &c )
 
 void QColor::hsv( int *h, int *s, int *v ) const
 {
-    int r = (int)(rgbVal & 0xff);
-    int g = (int)((rgbVal >> 8) & 0xff);
-    int b = (int)((rgbVal >> 16) & 0xff);
+    getHsv(*h,*s,*v);
+}
+
+
+/*!\obsolete
+  Returns the current RGB value as HSV.
+
+  \arg \e h, hue.
+  \arg \e s, saturation.
+  \arg \e v, value.
+
+  The hue defines the color. Its range is 0..359 if the color is chromatic
+  and -1 if the color is achromatic.  The saturation and value both vary
+  between 0 and 255 inclusive.
+
+  \sa setHsv(), rgb()
+*/
+
+void QColor::getHsv( int &h, int &s, int &v ) const
+{
+    int r = qRed(rgbVal);
+    int g = qGreen(rgbVal);
+    int b = qBlue(rgbVal);
     uint max = r;				// maximum RGB component
     int whatmax = 0;				// r=>0, g=>1, b=>2
     if ( (uint)g > max ) {
@@ -312,29 +438,29 @@ void QColor::hsv( int *h, int *s, int *v ) const
     if ( (uint)g < min ) min = g;
     if ( (uint)b < min ) min = b;
     int delta = max-min;
-    *v = max;					// calc value
-    *s = max ? (510*delta+max)/(2*max) : 0;
-    if ( *s == 0 ) {
-	*h = -1;				// undefined hue
+    v = max;					// calc value
+    s = max ? (510*delta+max)/(2*max) : 0;
+    if ( s == 0 ) {
+	h = -1;				// undefined hue
     } else {
 	switch ( whatmax ) {
 	    case 0:				// red is max component
 		if ( g >= b )
-		    *h = (120*(g-b)+delta)/(2*delta);
+		    h = (120*(g-b)+delta)/(2*delta);
 		else
-		    *h = (120*(g-b+delta)+delta)/(2*delta) + 300;
+		    h = (120*(g-b+delta)+delta)/(2*delta) + 300;
 		break;
 	    case 1:				// green is max component
 		if ( b > r )
-		    *h = 120 + (120*(b-r)+delta)/(2*delta);
+		    h = 120 + (120*(b-r)+delta)/(2*delta);
 		else
-		    *h = 60 + (120*(b-r+delta)+delta)/(2*delta);
+		    h = 60 + (120*(b-r+delta)+delta)/(2*delta);
 		break;
 	    case 2:				// blue is max component
 		if ( r > g )
-		    *h = 240 + (120*(r-g)+delta)/(2*delta);
+		    h = 240 + (120*(r-g)+delta)/(2*delta);
 		else
-		    *h = 180 + (120*(r-g+delta)+delta)/(2*delta);
+		    h = 180 + (120*(r-g+delta)+delta)/(2*delta);
 		break;
 	}
     }
@@ -347,14 +473,14 @@ void QColor::hsv( int *h, int *s, int *v ) const
   \arg \e s, saturation (0..255).
   \arg \e v, value (0..255).
 
-  \sa hsv(), setRgb()
+  \sa getHsv(), setRgb()
 */
 
 void QColor::setHsv( int h, int s, int v )
 {
 #if defined(CHECK_RANGE)
     if ( h < -1 || (uint)s > 255 || (uint)v > 255 ) {
-	warning( "QColor::setHsv: HSV parameters out of range" );
+	qWarning( "QColor::setHsv: HSV parameters out of range" );
 	return;
     }
 #endif
@@ -392,30 +518,49 @@ void QColor::setHsv( int h, int s, int v )
   \fn QRgb QColor::rgb() const
   Returns the RGB value.
 
-  Bits 0-7 = red, bits 8-15 = green, bits 16-23 = blue.
-
   The return type \e QRgb is equivalent to \c unsigned \c int.
 
-  \sa setRgb(), hsv()
+  \sa setRgb(), hsv(), qRed(), qBlue(), qGreen()
 */
 
 /*!
   Returns the red, green and blue components of the RGB value in
-  \e *r, \e *g and \e *b.
+  \e *r, \e *g and \e *b.  The value range for a component is 0..255.
   \sa setRgb(), hsv()
 */
 
 void QColor::rgb( int *r, int *g, int *b ) const
 {
-    *r = (int)(rgbVal & 0xff);
-    *g = (int)((rgbVal >> 8) & 0xff);
-    *b = (int)((rgbVal >> 16) & 0xff);
+    *r = qRed(rgbVal);
+    *g = qGreen(rgbVal);
+    *b = qBlue(rgbVal);
 }
 
-/*!
-  Sets the RGB value to \e rgb.
 
-  Bits 0-7 = red, bits 8-15 = green, bits 16-23 = blue.
+/*!
+  Sets the RGB value to \a (r,g,b).
+  \a r, \a g and \a b must be in the range 0..255.
+  \sa rgb(), setHsv()
+*/
+
+void QColor::setRgb( int r, int g, int b )
+{
+#if defined(CHECK_RANGE)
+    if ( (uint)r > 255 || (uint)g > 255 || (uint)b > 255 )
+	qWarning( "QColor::setRgb: RGB parameter(s) out of range" );
+#endif
+    rgbVal = qRgb(r,g,b);
+    if ( lazy_alloc || !color_init ) {
+	rgbVal |= RGB_DIRTY;			// alloc later
+	pix = 0;
+    } else {
+	alloc();				// alloc now
+    }
+}
+
+
+/*!
+  Sets the RGB value to \a rgb.
 
   The type \e QRgb is equivalent to \c unsigned \c int.
 
@@ -424,26 +569,30 @@ void QColor::rgb( int *r, int *g, int *b ) const
 
 void QColor::setRgb( QRgb rgb )
 {
-    int r = (int)(rgb & 0xff);
-    int g = (int)((rgb >> 8) & 0xff);
-    int b = (int)((rgb >> 16) & 0xff);
-    setRgb( r, g, b );
+    if ( lazy_alloc || !color_init ) {
+	rgbVal = (rgb & 0x00ffffff) | RGB_DIRTY;// alloc later
+	pix = 0;
+    } else {
+	rgbVal = (rgb & 0x00ffffff);
+	alloc();				// alloc now
+    }
 }
 
 
 /*!
   \fn int QColor::red() const
-  Returns the red component of the RGB value.
+  Returns the R (red) component of the RGB value.
 */
 
+
 /*!
   \fn int QColor::green() const
-  Returns the green component of the RGB value.
+  Returns the G (green) component of the RGB value.
 */
 
 /*!
   \fn int QColor::blue() const
-  Returns the blue component of the RGB value.
+  Returns the B (blue) component of the RGB value.
 */
 
 
@@ -483,6 +632,7 @@ QColor QColor::light( int factor ) const
     return c;
 }
 
+
 /*!
   Returns a darker (or lighter) color.
 
@@ -549,7 +699,7 @@ QColor QColor::dark( int factor ) const
 
 void QColor::setLazyAlloc( bool enable )
 {
-    lalloc = enable;
+    lazy_alloc = enable;
 }
 
 
@@ -558,7 +708,8 @@ void QColor::setLazyAlloc( bool enable )
   Returns the pixel value.
 
   This value is used by the underlying window system to refer to a color.
-  It can be thought of as an index into the display hardware's color table.
+  It can be thought of as an index into the display hardware's color table,
+  but the value is an arbitrary 32-bit value.
 
   \sa setLazyAlloc(), alloc()
 */
@@ -572,12 +723,15 @@ void QColor::setLazyAlloc( bool enable )
   \relates QColor
   Writes a color object to the stream.
 
-  Serialization format: RGB value serialized as an UINT32.
+  Serialization format: RGB value serialized as an Q_UINT32.
 */
 
 QDataStream &operator<<( QDataStream &s, const QColor &c )
 {
-    return s << (UINT32)c.rgb();
+    Q_UINT32 p = (Q_UINT32)c.rgb();
+    if ( s.version() == 1 )			// Swap red and blue
+	p = ((p << 16) & 0xff0000) | ((p >> 16) & 0xff) | (p & 0xff00ff00);
+    return s << p;
 }
 
 /*!
@@ -587,9 +741,11 @@ QDataStream &operator<<( QDataStream &s, const QColor &c )
 
 QDataStream &operator>>( QDataStream &s, QColor &c )
 {
-    UINT32 rgb;
-    s >> rgb;
-    c.setRgb( rgb );
+    Q_UINT32 p;
+    s >> p;
+    if ( s.version() == 1 )			// Swap red and blue
+	p = ((p << 16) & 0xff0000) | ((p >> 16) & 0xff) | (p & 0xff00ff00);
+    c.setRgb( p );
     return s;
 }
 
@@ -622,9 +778,7 @@ QDataStream &operator>>( QDataStream &s, QColor &c )
 /*!
   \fn QRgb qRgb( int r, int g, int b )
   \relates QColor
-  Returns the RGB triplet \e (r,g,b).
-
-  Bits 0-7 = \e r (red), bits 8-15 = \e g (green), bits 16-23 = \e b (blue).
+  Returns the RGB triplet \a (r,g,b).
 
   The return type \e QRgb is equivalent to \c unsigned \c int.
 
@@ -634,7 +788,7 @@ QDataStream &operator>>( QDataStream &s, QColor &c )
 /*!
   \fn int qGray( int r, int g, int b )
   \relates QColor
-  Returns a gray value 0..255 from the \e (r,g,b) triplet.
+  Returns a gray value 0..255 from the \a (r,g,b) triplet.
 
   The gray value is calculated using the formula:
   <code>(r*11 + g*16 + b*5)/32</code>.
