@@ -1,38 +1,41 @@
 /****************************************************************************
-** $Id: qdialog.cpp,v 2.10 1997/09/03 15:27:27 warwick Exp $
+** $Id: qdialog.cpp,v 2.51 1999/06/15 12:00:52 warwick Exp $
 **
 ** Implementation of QDialog class
 **
 ** Created : 950502
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qdialog.h"
-#include "qpushbt.h"
-#include "qapp.h"
-#include "qkeycode.h"
-#include "qobjcoll.h"
-
-RCSTAG("$Id: qdialog.cpp,v 2.10 1997/09/03 15:27:27 warwick Exp $");
-
+#include "qpushbutton.h"
+#include "qapplication.h"
+#include "qobjectlist.h"
+#include "qobjectdict.h"
+#include "qwidgetlist.h"
 
 /*!
   \class QDialog qdialog.h
   \brief The QDialog class is the base class of dialog windows.
 
+  \ingroup dialogs
+
   A dialog window is a widget used to communicate with the user. It offers
   mechanisms such as default buttons.
 
@@ -47,14 +50,14 @@ RCSTAG("$Id: qdialog.cpp,v 2.10 1997/09/03 15:27:27 warwick Exp $");
     class Modal : public QDialog {
 	Q_OBJECT
     public:
-	Modal( QWidget *parent, const char *name );
+	Modal( QWidget *parent, QString name );
     };
 
-    Modal::Modal( QWidget *parent, const char *name )
+    Modal::Modal( QWidget *parent, QString name )
 	: QDialog( parent, name, TRUE )
     {
 	QPushButton *ok, *cancel;
-	ok = new QPushButton( "Ok", this );
+	ok = new QPushButton( "OK", this );
 	ok->setGeometry( 10,10, 100,30 );
 	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
 	cancel = new QPushButton( "Cancel", this );
@@ -63,10 +66,9 @@ RCSTAG("$Id: qdialog.cpp,v 2.10 1997/09/03 15:27:27 warwick Exp $");
     }
   \endcode
 
-  Note that the parent widget has a different meaning for modal dialogs
-  than for other types of widgets. A dialog is placed on top of the parent
-  widget. The dialog is centered on the screen if the parent widget is
-  zero.
+  Note that the parent widget has an additional meaning for modal
+  dialogs.  A modal dialog is placed on top of the parent widget. The
+  dialog is centered on the screen if the parent widget is zero.
 
   You would normally call exec() to start a modal dialog. This enters
   a local event loop, which is terminated when the modal dialog calls
@@ -80,8 +82,11 @@ RCSTAG("$Id: qdialog.cpp,v 2.10 1997/09/03 15:27:27 warwick Exp $");
     }
   \endcode
 
-  Modeless dialogs behave just like ordinary widgets. The only difference
+  Modeless dialogs behave just like ordinary toplevel widgets. The only difference
   is that they have the default button mechanism.
+
+  \sa QTabDialog QWidget QSemiModal
+  <a href="guibooks.html#fowler">GUI Design Handbook: Dialogs, Standard.</a>
 */
 
 
@@ -89,21 +94,28 @@ RCSTAG("$Id: qdialog.cpp,v 2.10 1997/09/03 15:27:27 warwick Exp $");
   Constructs a dialog named \e name, which has a parent widget \e parent.
 
   The dialog will by default be modeless, unless you set \e modal to
-  TRUE, which constructs a modal dialog.
+  TRUE to construct a modal dialog.
 
-  The \a f argument is the \link QWidget::QWidget() widget
-  flags \endlink , which can be used to customize the window frame style.
+  The \a f argument is the \link QWidget::QWidget() widget flags,
+  \endlink which can be used to customize the window frame style.
 
-  \warning Creating a modeless dialog with a parent makes it an ordinary
-  child widget, which is probably not what you want. Expect strange
-  behavior (QDialog has a default button mechanism).
+  A dialog is always a top level widget. The optional parent, however,
+  will know about this child and also delete it on
+  destruction. Furthermore, the window system will be able to tell
+  that both the dialog and the parent belong together. This works for
+  Windows and also some X11 window managers, that will for instance
+  provide a common taskbar entry in that case.
+
+  It is recommended to pass a parent.
 */
 
 QDialog::QDialog( QWidget *parent, const char *name, bool modal, WFlags f )
-    : QWidget( parent, name, modal ? (f | WType_Modal) : f )
+    : QWidget( parent, name, (modal ? (f | WType_Modal) : f) | WType_TopLevel | WStyle_Dialog )
 {
     rescode = 0;
-    did_move = did_resize = FALSE;
+    did_move = FALSE;
+    did_resize = FALSE;
+    mainDef = 0;
 }
 
 /*!
@@ -112,6 +124,9 @@ QDialog::QDialog( QWidget *parent, const char *name, bool modal, WFlags f )
 
 QDialog::~QDialog()
 {
+    // Need to hide() here, as our (to-be) overridden hide()
+    // will not be called in ~QWidget.
+    hide();
 }
 
 
@@ -126,11 +141,18 @@ void QDialog::setDefault( QPushButton *pushButton )
     QObjectList *list = queryList( "QPushButton" );
     QObjectListIt it( *list );
     QPushButton *pb;
+    bool hasMain = FALSE;
     while ( (pb = (QPushButton*)it.current()) ) {
+	if ( pb == mainDef )
+	    hasMain = TRUE;
 	if ( pb != pushButton )
 	    pb->setDefault( FALSE );
 	++it;
     }
+    if (!pushButton && hasMain)
+	mainDef->setDefault( TRUE );
+    if (!hasMain)
+	mainDef = pushButton;
     delete list;
 }
 
@@ -148,19 +170,28 @@ void QDialog::setDefault( QPushButton *pushButton )
 */
 
 
-
 /*!
-  Starts the dialog and returns the result code.
+  For modal dialogs: Starts the dialog and returns the result code.
 
   Equivalent to calling show(), then result().
 
-  This function is very useful for modal dialogs. It enters a new local
-  event loop. The event loop is terminated when the dialog is hidden,
-  usually by calling done().
+  This function is very useful for modal dialogs, but makes no sense for
+  modeless dialog. It enters a new local event loop. The event loop is
+  terminated when the dialog is hidden, usually by calling done().
+
+  A warning message is printed if you call this function for a modeless
+  dialog.
+
+  \sa show(), result()
 */
 
 int QDialog::exec()
 {
+#if defined(CHECK_STATE)
+    if ( !testWFlags(WType_Modal) )
+	qWarning( "QDialog::exec: Calling this function for a modeless dialog "
+		 "makes no sense" );
+#endif
     setResult( 0 );
     show();
     return result();
@@ -175,6 +206,12 @@ int QDialog::exec()
   This function is very useful for modal dialogs. It leaves the local
   event loop and returns from the exec() or show() function.
 
+  \warning Although done() will return to the caller also if this
+  dialog is modal, the local event loop is then marked for
+  termination. Hence, a program should not try to do anything that
+  depends on event handling before the corresponding exec() or show()
+  has returned.
+
   \sa accept(), reject()
 */
 
@@ -223,28 +260,40 @@ void QDialog::keyPressEvent( QKeyEvent *e )
 {
     if ( e->state() == 0 ) {
 	switch ( e->key() ) {
-	    case Key_Enter:
-	    case Key_Return: {
-		QObjectList *list = queryList( "QPushButton" );
-		QObjectListIt it( *list );
-		QPushButton *pb;
-		while ( (pb = (QPushButton*)it.current()) ) {
-		    if ( pb->isDefault() ) {
-			delete list;
+	case Key_Enter:
+	case Key_Return: {
+	    QObjectList *list = queryList( "QPushButton" );
+	    QObjectListIt it( *list );
+	    QPushButton *pb;
+	    while ( (pb = (QPushButton*)it.current()) ) {
+		if ( pb->isDefault() && pb->isVisibleToTLW() ) {
+		    delete list;
+		    if ( pb->isEnabled() ) {
 			emit pb->clicked();
-			return;
 		    }
-		    ++it;
+		    return;
 		}
-		delete list;
-		}
-		break;
-	    case Key_Escape:
-		reject();
-		break;
-	    default:
-		e->ignore();
-		return;
+		++it;
+	    }
+	    delete list;
+	}
+	break;
+	case Key_Escape:
+	    reject();
+	    break;
+	case Key_Up:
+	case Key_Left:
+	    // call ours, since c++ blocks us from calling the one
+	    // belonging to focusWidget().
+	    focusNextPrevChild( FALSE );
+	    break;
+	case Key_Down:
+	case Key_Right:
+	    focusNextPrevChild( TRUE );
+	    break;
+	default:
+	    e->ignore();
+	    return;
 	}
     } else {
 	e->ignore();
@@ -285,22 +334,65 @@ void QDialog::closeEvent( QCloseEvent *e )
 
 void QDialog::show()
 {
-    if ( testWFlags(WState_Visible) )
+    if ( testWState(WState_Visible) )
 	return;
     if ( !did_resize )
 	adjustSize();
     if ( !did_move ) {
 	QWidget *w = parentWidget();
 	QPoint p( 0, 0 );
+	int extraw = 0, extrah = 0;
+	QWidget * desk = QApplication::desktop();
 	if ( w )
-	    p = w->mapToGlobal( p );
+	    w = w->topLevelWidget();
+
+	QWidgetList  *list = QApplication::topLevelWidgets();
+	QWidgetListIt it( *list );
+	while ( (extraw == 0 || extrah == 0) &&
+		it.current() != 0 ) {
+	    int w, h;
+	    QWidget * current = it.current();
+	    ++it;
+	    w = current->frameGeometry().width() - current->width();
+	    h = current->frameGeometry().height() - current->height();
+
+	    extraw = QMAX( extraw, w );
+	    extrah = QMAX( extrah, h );
+	}
+	delete list;
+
+	if ( w )
+	    p = QPoint( w->x() + w->width()/2,
+			w->y() + w->height()/ 2 );
 	else
-	    w = QApplication::desktop();
-	move( p.x() + w->width()/2  - width()/2,
-	      p.y() + w->height()/2 - height()/2 );
+	    p = QPoint( desk->width()/2, desk->height()/2 );
+
+	p = QPoint( p.x()-width()/2 - extraw,
+		    p.y()-height()/2 - extraw );
+
+	if ( p.x() + extraw + width() > desk->width() )
+	    p.setX( desk->width() - width() - extraw );
+	if ( p.x() < 0 )
+	    p.setX( 0 );
+
+	if ( p.y() + extrah + height() > desk->height() )
+	    p.setY( desk->height() - height() - extrah );
+	if ( p.y() < 0 )
+	    p.setY( 0 );
+	
+	move( p );
     }
     QWidget::show();
-    if ( testWFlags(WType_Modal) ) qApp->enter_loop();
+    if ( testWFlags(WType_Modal) )
+	qApp->enter_loop();
+}
+
+void QDialog::hide()
+{
+    bool ex = testWState(WState_Visible) && testWFlags(WType_Modal);
+    QWidget::hide();
+    if ( ex )
+	qApp->exit_loop();
 }
 
 
