@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qdrawutil.cpp,v 2.19 1998/07/03 00:09:32 hanord Exp $
+** $Id: qdrawutil.cpp,v 2.39 1999/05/23 20:28:22 agulbra Exp $
 **
 ** Implementation of draw utilities
 **
 ** Created : 950920
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -59,9 +61,12 @@ void qDrawShadeLine( QPainter *p, int x1, int y1, int x2, int y2,
 		     const QColorGroup &g, bool sunken,
 		     int lineWidth, int midLineWidth )
 {
+    if (!( p && lineWidth >= 0 && midLineWidth >= 0 ) )	{
 #if defined(CHECK_RANGE)
-    ASSERT( p && lineWidth >= 0 && midLineWidth >= 0 );
+	qWarning( "qDrawShadeLine invalid parameters." );
 #endif
+	return;
+    }
     int tlw = lineWidth*2 + midLineWidth;	// total line width
     QPen oldPen = p->pen();			// save pen
     if ( sunken )
@@ -171,9 +176,12 @@ void qDrawShadeRect( QPainter *p, int x, int y, int w, int h,
 {
     if ( w == 0 || h == 0 )
 	return;
+    if ( ! ( w > 0 && h > 0 && lineWidth >= 0 && midLineWidth >= 0 ) ) {
 #if defined(CHECK_RANGE)
-    ASSERT( w > 0 && h > 0 && lineWidth >= 0 && midLineWidth >= 0 );
+	qWarning( "qDrawShadeRect() Invalid parameters." );
 #endif
+	return;
+    }
     QPen oldPen = p->pen();			// save pen
     if ( sunken )
 	p->setPen( g.dark() );
@@ -228,7 +236,7 @@ void qDrawShadeRect( QPainter *p, int x, int y, int w, int h,
     if ( fill ) {
 	QBrush oldBrush = p->brush();
 	int tlw = lineWidth + midLineWidth;
-	p->setPen( NoPen );
+	p->setPen( Qt::NoPen );
 	p->setBrush( *fill );
 	p->drawRect( x+tlw, y+tlw, w-2*tlw, h-2*tlw );
 	p->setBrush( oldBrush );
@@ -268,9 +276,11 @@ void qDrawShadePanel( QPainter *p, int x, int y, int w, int h,
 {
     if ( w == 0 || h == 0 )
 	return;
+    if ( !( w > 0 && h > 0 && lineWidth >= 0 ) ) {
 #if defined(CHECK_RANGE)
-    ASSERT( w > 0 && h > 0 && lineWidth >= 0 );
+    	qWarning( "qDrawShadePanel() Invalid parameters." );
 #endif
+    }
     QPen oldPen = p->pen();			// save pen
     QPointArray a( 4*lineWidth );
     if ( sunken )
@@ -316,7 +326,7 @@ void qDrawShadePanel( QPainter *p, int x, int y, int w, int h,
     p->drawLineSegments( a );
     if ( fill ) {				// fill with fill color
 	QBrush oldBrush = p->brush();
-	p->setPen( NoPen );
+	p->setPen( Qt::NoPen );
 	p->setBrush( *fill );
 	p->drawRect( x+lineWidth, y+lineWidth, w-lineWidth*2, h-lineWidth*2 );
 	p->setBrush( oldBrush );
@@ -374,7 +384,7 @@ static void qDrawWinShades( QPainter *p,
 	if ( fill ) {
 	    QBrush oldBrush = p->brush();
 	    p->setBrush( *fill );
-	    p->setPen( NoPen );
+	    p->setPen( Qt::NoPen );
 	    p->drawRect( x+2, y+2, w-4, h-4 );
 	    p->setBrush( oldBrush );
 	}
@@ -410,10 +420,10 @@ void qDrawWinButton( QPainter *p, int x, int y, int w, int h,
 {
     if ( sunken )
 	qDrawWinShades( p, x, y, w, h,
-		       black, g.light(), g.dark(), g.background(), fill );
+		       g.shadow(), g.light(), g.dark(), g.button(), fill );
     else
 	qDrawWinShades( p, x, y, w, h,
-		       g.light(), black, g.midlight(), g.dark(), fill );
+		       g.light(), g.shadow(), g.midlight(), g.dark(), fill );
 }
 
 /*!
@@ -444,10 +454,10 @@ void qDrawWinPanel( QPainter *p, int x, int y, int w, int h,
 {
     if ( sunken )
 	qDrawWinShades( p, x, y, w, h,
-		       g.dark(), g.light(), black, g.midlight(), fill );
+		       g.dark(), g.light(), g.shadow(), g.midlight(), fill );
     else
 	qDrawWinShades( p, x, y, w, h,
-		       g.midlight(), black, g.light(), g.dark(), fill );
+		       g.midlight(), g.shadow(), g.light(), g.dark(), fill );
 }
 
 
@@ -475,17 +485,19 @@ void qDrawPlainRect( QPainter *p, int x, int y, int w, int h, const QColor &c,
 {
     if ( w == 0 || h == 0 )
 	return;
+    if ( !( w > 0 && h > 0 && lineWidth >= 0 ) ) {
 #if defined(CHECK_RANGE)
-    ASSERT( w > 0 && h > 0 && lineWidth >= 0 );
+	qWarning( "qDrawPlainRect() Invalid parameters." );
 #endif
+    }
     QPen   oldPen   = p->pen();
     QBrush oldBrush = p->brush();
     p->setPen( c );
-    p->setBrush( NoBrush );
+    p->setBrush( Qt::NoBrush );
     for ( int i=0; i<lineWidth; i++ )
 	p->drawRect( x+i, y+i, w-i*2, h-i*2 );
     if ( fill ) {				// fill with fill color
-	p->setPen( NoPen );
+	p->setPen( Qt::NoPen );
 	p->setBrush( *fill );
 	p->drawRect( x+lineWidth, y+lineWidth, w-lineWidth*2, h-lineWidth*2 );
     }
@@ -494,28 +506,28 @@ void qDrawPlainRect( QPainter *p, int x, int y, int w, int h, const QColor &c,
 }
 
 
-QRect qItemRect( QPainter *p, GUIStyle gs,
+QRect qItemRect( QPainter *p, Qt::GUIStyle gs,
 		int x, int y, int w, int h,
-		int flags, 
+		int flags,
 		bool enabled,
 		const QPixmap *pixmap,
-		const char *text, int len )
+		const QString& text, int len )
 {
     QRect result;
 
     if ( pixmap ) {
-	if ( (flags & AlignVCenter) == AlignVCenter )
+	if ( (flags & Qt::AlignVCenter) == Qt::AlignVCenter )
 	    y += h/2 - pixmap->height()/2;
-	else if ( (flags & AlignBottom) == AlignBottom)
+	else if ( (flags & Qt::AlignBottom) == Qt::AlignBottom)
 	    y += h - pixmap->height();
-	if ( (flags & AlignRight) == AlignRight )
+	if ( (flags & Qt::AlignRight) == Qt::AlignRight )
 	    x += w - pixmap->width();
-	else if ( (flags & AlignHCenter) == AlignHCenter )
+	else if ( (flags & Qt::AlignHCenter) == Qt::AlignHCenter )
 	    x += w/2 - pixmap->width()/2;
 	result = QRect(x, y, pixmap->width(), pixmap->height());
-    } else if ( text && p ) {
+    } else if ( !text.isNull() && p ) {
 	result = p->boundingRect( x, y, w, h, flags, text, len );
-	if ( gs == WindowsStyle && !enabled ) {
+	if ( gs == Qt::WindowsStyle && !enabled ) {
 	    result.setWidth(result.width()+1);
 	    result.setHeight(result.height()+1);
 	}
@@ -527,30 +539,30 @@ QRect qItemRect( QPainter *p, GUIStyle gs,
 }
 
 
-void qDrawItem( QPainter *p, GUIStyle gs,
+void qDrawItem( QPainter *p, Qt::GUIStyle gs,
 		int x, int y, int w, int h,
-		int flags, 
+		int flags,
 		const QColorGroup &g, bool enabled,
 		const QPixmap *pixmap,
-		const char *text, int len )
+		const QString& text, int len , const QColor* penColor )
 {
-    p->setPen( g.text() );
+    p->setPen( penColor?*penColor:g.text() );
     if ( pixmap ) {
 	QPixmap  pm( *pixmap );
-	bool clip = (flags & DontClip) == 0;
+	bool clip = (flags & Qt::DontClip) == 0;
 	if ( clip ) {
 	    if ( pm.width() < w && pm.height() < h )
 		clip = FALSE;
 	    else
 		p->setClipRect( x, y, w, h );
 	}
-	if ( (flags & AlignVCenter) == AlignVCenter )
+	if ( (flags & Qt::AlignVCenter) == Qt::AlignVCenter )
 	    y += h/2 - pm.height()/2;
-	else if ( (flags & AlignBottom) == AlignBottom)
+	else if ( (flags & Qt::AlignBottom) == Qt::AlignBottom)
 	    y += h - pm.height();
-	if ( (flags & AlignRight) == AlignRight )
+	if ( (flags & Qt::AlignRight) == Qt::AlignRight )
 	    x += w - pm.width();
-	else if ( (flags & AlignHCenter) == AlignHCenter )
+	else if ( (flags & Qt::AlignHCenter) == Qt::AlignHCenter )
 	    x += w/2 - pm.width()/2;
 	if ( !enabled ) {
 	    if ( pm.mask() ) {			// pixmap with a mask
@@ -574,7 +586,7 @@ void qDrawItem( QPainter *p, GUIStyle gs,
 		pm = *mask;
 		if (del) delete mask;
 	    }
-	    if ( gs == WindowsStyle ) {
+	    if ( gs == Qt::WindowsStyle ) {
 		p->setPen( g.light() );
 		p->drawPixmap( x+1, y+1, pm );
 		p->setPen( g.text() );
@@ -583,8 +595,8 @@ void qDrawItem( QPainter *p, GUIStyle gs,
 	p->drawPixmap( x, y, pm );
 	if ( clip )
 	    p->setClipping( FALSE );
-    } else if ( text ) {
-	if ( gs == WindowsStyle && !enabled ) {
+    } else if ( !text.isNull() ) {
+	if ( gs == Qt::WindowsStyle && !enabled ) {
 	    p->setPen( g.light() );
 	    p->drawText( x+1, y+1, w, h, flags, text, len );
 	    p->setPen( g.text() );
@@ -669,22 +681,22 @@ void qDrawPlainRect( QPainter *p, const QRect &r, const QColor &c,
 }
 
 
-static void qDrawWinArrow( QPainter *p, ArrowType type, bool down,
+static void qDrawWinArrow( QPainter *p, Qt::ArrowType type, bool down,
 			   int x, int y, int w, int h,
 			   const QColorGroup &g, bool enabled )
 {
     QPointArray a;				// arrow polygon
     switch ( type ) {
-    case UpArrow:
+    case Qt::UpArrow:
 	a.setPoints( 7, -3,1, 3,1, -2,0, 2,0, -1,-1, 1,-1, 0,-2 );
 	break;
-    case DownArrow:
+    case Qt::DownArrow:
 	a.setPoints( 7, -3,-1, 3,-1, -2,0, 2,0, -1,1, 1,1, 0,2 );
 	break;
-    case LeftArrow:
+    case Qt::LeftArrow:
 	a.setPoints( 7, 1,-3, 1,3, 0,-2, 0,2, -1,-1, -1,1, -2,0 );
 	break;
-    case RightArrow:
+    case Qt::RightArrow:
 	a.setPoints( 7, -1,-3, -1,3, 0,-2, 0,2, 1,-1, 1,1, 2,0 );
 	break;
     }
@@ -697,7 +709,11 @@ static void qDrawWinArrow( QPainter *p, ArrowType type, bool down,
     }
 
     QPen savePen = p->pen();			// save current pen
-    p->fillRect( x, y, w, h, g.background() );
+    if (down)
+	p->setBrushOrigin(p->brushOrigin() + QPoint(1,1));
+    p->fillRect( x, y, w, h, g.brush( QColorGroup::Button ) );
+    if (down)
+	p->setBrushOrigin(p->brushOrigin() - QPoint(1,1));
     if ( enabled ) {
 	a.translate( x+w/2, y+h/2 );
 	p->setPen( g.foreground() );
@@ -724,16 +740,16 @@ static void qDrawWinArrow( QPainter *p, ArrowType type, bool down,
 
 // motif arrows look the same whether they are used or not
 // is this correct?
-static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
+static void qDrawMotifArrow( QPainter *p, Qt::ArrowType type, bool down,
 			     int x, int y, int w, int h,
-			     const QColorGroup &g, bool ) 
+			     const QColorGroup &g, bool )
 {
     QPointArray bFill;				// fill polygon
     QPointArray bTop;				// top shadow.
     QPointArray bBot;				// bottom shadow.
     QPointArray bLeft;				// left shadow.
     QWMatrix	matrix;				// xform matrix
-    bool vertical = type == UpArrow || type == DownArrow;
+    bool vertical = type == Qt::UpArrow || type == Qt::DownArrow;
     bool horizontal = !vertical;
     int	 dim = w < h ? w : h;
     int	 colspec = 0x0000;			// color specification array
@@ -780,7 +796,7 @@ static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
 	}
     }
 
-    if ( type == UpArrow || type == LeftArrow ) {
+    if ( type == Qt::UpArrow || type == Qt::LeftArrow ) {
 	matrix.translate( x, y );
 	if ( vertical ) {
 	    matrix.translate( 0, h - 1 );
@@ -794,7 +810,7 @@ static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
 	else
 	    colspec = horizontal ? 0x1443 : 0x1434;
     }
-    else if ( type == DownArrow || type == RightArrow ) {
+    else if ( type == Qt::DownArrow || type == Qt::RightArrow ) {
 	matrix.translate( x, y );
 	if ( vertical ) {
 	    matrix.translate( w-1, 0 );
@@ -808,7 +824,7 @@ static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
 
     QColor *cols[5];
     cols[0] = 0;
-    cols[1] = (QColor *)&g.background();
+    cols[1] = (QColor *)&g.button();
     cols[2] = (QColor *)&g.mid();
     cols[3] = (QColor *)&g.light();
     cols[4] = (QColor *)&g.dark();
@@ -820,14 +836,14 @@ static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
     QPen     savePen   = p->pen();		// save current pen
     QBrush   saveBrush = p->brush();		// save current brush
     QWMatrix wxm = p->worldMatrix();
-    QPen     pen( NoPen );
-    QBrush   brush( CMID );
+    QPen     pen( Qt::NoPen );
+    QBrush brush = g.brush( QColorGroup::Button );
 
     p->setPen( pen );
     p->setBrush( brush );
     p->setWorldMatrix( matrix, TRUE );		// set transformation matrix
     p->drawPolygon( bFill );			// fill arrow
-    p->setBrush( NoBrush );			// don't fill
+    p->setBrush( Qt::NoBrush );			// don't fill
 
     p->setPen( CLEFT );
     p->drawLineSegments( bLeft );
@@ -847,20 +863,22 @@ static void qDrawMotifArrow( QPainter *p, ArrowType type, bool down,
 }
 
 
-void qDrawArrow( QPainter *p, ArrowType type, GUIStyle style, bool down,
+void qDrawArrow( QPainter *p, Qt::ArrowType type, Qt::GUIStyle style, bool down,
 		 int x, int y, int w, int h,
 		 const QColorGroup &g, bool enabled )
 {
     switch ( style ) {
-	case WindowsStyle:
+	case Qt::WindowsStyle:
 	    qDrawWinArrow( p, type, down, x, y, w, h, g, enabled );
 	    break;
-	case MotifStyle:
+	case Qt::MotifStyle:
 	    qDrawMotifArrow( p, type, down, x, y, w, h, g, enabled );
 	    break;
 	default:
 #if defined(CHECK_RANGE)
-	    warning( "qDrawArrow: Requested GUI style not supported" );
+	    qWarning( "qDrawArrow: Requested GUI style not supported" );
+#else
+	    ;
 #endif
     }
 }
