@@ -1,33 +1,38 @@
 /****************************************************************************
-** $Id: qimage.h,v 2.21 1997/09/22 17:38:24 paul Exp $
+** $Id: qimage.h,v 2.47 1999/04/28 09:56:27 aavit Exp $
 **
 ** Definition of QImage and QImageIO classes
 **
 ** Created : 950207
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #ifndef QIMAGE_H
 #define QIMAGE_H
 
+#ifndef QT_H
 #include "qpixmap.h"
 #include "qstrlist.h"
+#endif // QT_H
 
 
-class QImage
+class Q_EXPORT QImage
 {
 public:
     enum Endian { IgnoreEndian, BigEndian, LittleEndian };
@@ -37,15 +42,20 @@ public:
 	    Endian bitOrder=IgnoreEndian );
     QImage( const QSize&, int depth, int numColors=0,
 	    Endian bitOrder=IgnoreEndian );
-    QImage( const char *fileName, const char *format=0 );
+    QImage( const QString &fileName, const char* format=0 );
     QImage( const char *xpm[] );
+    QImage( const QByteArray &data );
     QImage( const QImage & );
    ~QImage();
 
     QImage     &operator=( const QImage & );
     QImage     &operator=( const QPixmap & );
+    bool   	operator==( const QImage & ) const;
+    bool   	operator!=( const QImage & ) const;
     void	detach();
     QImage	copy()		const;
+    QImage	copy(int x, int y, int w, int h, int conversion_flags=0) const;
+    QImage	copy(QRect&)	const;
 
     bool	isNull()	const	{ return data->bits == 0; }
 
@@ -64,6 +74,9 @@ public:
     bool	hasAlphaBuffer() const;
     void	setAlphaBuffer( bool );
 
+    bool	allGray() const;
+    bool        isGrayscale() const;
+
     uchar      *bits()		const;
     uchar      *scanLine( int ) const;
     uchar     **jumpTable()	const;
@@ -78,33 +91,32 @@ public:
     void	reset();
 
     void	fill( uint pixel );
+    void	invertPixels( bool invertAlpha = TRUE );
 
     QImage	convertDepth( int ) const;
+    QImage	convertDepthWithPalette( int, QRgb* p, int pc, int cf=0 ) const;
     QImage	convertDepth( int, int conversion_flags ) const;
     QImage	convertBitOrder( Endian ) const;
+    QImage	smoothScale(int width, int height) const;
 
-#if defined(HAS_BOOL_TYPE)
-    // Needed for binary compatibility - calls createAlphaMask(int)
-    QImage	createAlphaMask( bool dither=FALSE ) const;
-#else
-    // Needed for source compatibility - calls createAlphaMask(int)
-    QImage	createAlphaMask() const;
-#endif
-
-    QImage	createAlphaMask( int conversion_flags ) const;
+    QImage	createAlphaMask( int conversion_flags=0 ) const;
     QImage	createHeuristicMask( bool clipTight=TRUE ) const;
 
+    QImage	mirror() const;
+    QImage	swapRGB() const;
+
     static Endian systemBitOrder();
     static Endian systemByteOrder();
 
-    static const char *imageFormat( const char *fileName );
+    static const char* imageFormat( const QString &fileName );
     static QStrList inputFormats();
     static QStrList outputFormats();
 
-    bool	load( const char *fileName, const char *format=0 );
+    bool	load( const QString &fileName, const char* format=0 );
     bool	loadFromData( const uchar *buf, uint len,
 			      const char *format=0 );
-    bool	save( const char *fileName, const char *format ) const;
+    bool	loadFromData( QByteArray data, const char *format=0 );
+    bool	save( const QString &fileName, const char* format ) const;
 
     bool	valid( int x, int y ) const;
     int		pixelIndex( int x, int y ) const;
@@ -114,6 +126,7 @@ public:
 private:
     void	init();
     void	freeBits();
+    static void	warningIndexRange( const char *, int );
 
     struct QImageData : public QShared {	// internal image data
 	int	w;				// image width
@@ -126,25 +139,29 @@ private:
 	uchar **bits;				// image data
 	bool	alpha;				// alpha buffer
     } *data;
+
+    friend Q_EXPORT void bitBlt( QImage* dst, int dx, int dy,
+				 const QImage* src, int sx, int sy,
+				 int sw, int sh, int conversion_flags );
 };
 
 
 // QImage stream functions
 
-QDataStream &operator<<( QDataStream &, const QImage & );
-QDataStream &operator>>( QDataStream &, QImage & );
+Q_EXPORT QDataStream &operator<<( QDataStream &, const QImage & );
+Q_EXPORT QDataStream &operator>>( QDataStream &, QImage & );
 
 
 class QIODevice;
 typedef void (*image_io_handler)( QImageIO * ); // image IO handler
 
 
-class QImageIO
+class Q_EXPORT QImageIO
 {
 public:
     QImageIO();
     QImageIO( QIODevice	 *ioDevice, const char *format );
-    QImageIO( const char *fileName, const char *format );
+    QImageIO( const QString &fileName, const char* format );
    ~QImageIO();
 
 
@@ -152,22 +169,22 @@ public:
     int		status()	const	{ return iostat; }
     const char *format()	const	{ return frmt; }
     QIODevice  *ioDevice()	const	{ return iodev; }
-    const char *fileName()	const	{ return fname; }
+    QString	fileName()	const	{ return fname; }
     const char *parameters()	const	{ return params; }
-    const char *description()	const	{ return descr; }
+    QString	description()	const	{ return descr; }
 
     void	setImage( const QImage & );
     void	setStatus( int );
     void	setFormat( const char * );
     void	setIODevice( QIODevice * );
-    void	setFileName( const char * );
+    void	setFileName( const QString &);
     void	setParameters( const char * );
-    void	setDescription( const char * );
+    void	setDescription( const QString &);
 
     bool	read();
     bool	write();
 
-    static const char *imageFormat( const char *fileName );
+    static const char* imageFormat( const QString &fileName );
     static const char *imageFormat( QIODevice * );
     static QStrList inputFormats();
     static QStrList outputFormats();
@@ -181,18 +198,25 @@ public:
 private:
     QImage	im;				// image
     int		iostat;				// IO status
-    QString	frmt;				// image format
+    QCString	frmt;				// image format
     QIODevice  *iodev;				// IO device
     QString	fname;				// file name
     char       *params;				// image parameters
-    char       *descr;				// image description
+    QString     descr;				// image description
 
 private:	// Disabled copy constructor and operator=
-    QImageIO( const QImageIO & ) {}
-    QImageIO &operator=( const QImageIO & ) { return *this; }
+#if defined(Q_DISABLE_COPY)
+    QImageIO( const QImageIO & );
+    QImageIO &operator=( const QImageIO & );
+#endif
 };
 
 
+Q_EXPORT void bitBlt( QImage* dst, int dx, int dy, const QImage* src,
+		      int sx=0, int sy=0, int sw=-1, int sh=-1,
+		      int conversion_flags=0 );
+
+
 /*****************************************************************************
   QImage member functions
  *****************************************************************************/
@@ -227,25 +251,38 @@ inline int QImage::bytesPerLine() const
     return data->h ? data->nbytes/data->h : 0;
 }
 
-#if !(defined(QIMAGE_C) || defined(DEBUG))
+inline QImage QImage::copy(QRect& r) const
+{
+    return copy(r.x(), r.y(), r.width(), r.height());
+}
 
 inline QRgb QImage::color( int i ) const
 {
+#if defined(CHECK_RANGE)
+    if ( i >= data->ncols )
+	warningIndexRange( "color", i );
+#endif
     return data->ctbl ? data->ctbl[i] : (QRgb)-1;
 }
 
 inline void QImage::setColor( int i, QRgb c )
 {
+#if defined(CHECK_RANGE)
+    if ( i >= data->ncols )
+	warningIndexRange( "setColor", i );
+#endif
     if ( data->ctbl )
 	data->ctbl[i] = c;
 }
 
 inline uchar *QImage::scanLine( int i ) const
 {
+#if defined(CHECK_RANGE)
+    if ( i >= data->h )
+	warningIndexRange( "scanLine", i );
+#endif
     return data->bits ? data->bits[i] : 0;
 }
 
-#endif
-
 
 #endif // QIMAGE_H
