@@ -1,25 +1,30 @@
 /****************************************************************************
-** $Id: qmovie.cpp,v 1.26 1997/09/30 11:18:37 hanord Exp $
+** $Id: qmovie.cpp,v 1.61 1999/06/03 16:35:18 warwick Exp $
 **
 ** Implementation of movie classes
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
+// #define QT_SAVE_MOVIE_HACK
+
 #include "qtimer.h"
 #include "qpainter.h"
 #include "qlist.h"
@@ -38,6 +43,8 @@
   \class QMovie qmovie.h
   \brief Incrementally loads an animation or image, signalling as it progresses.
 
+  \ingroup images
+
   A QMovie provides a QPixmap as the framePixmap(), and connections can
   be made via connectResize() and connectUpdate() to receive notification
   of size and pixmap changes.  All decoding is driven by
@@ -64,29 +71,43 @@
   disposal, local color maps, and background colors.  The Netscape looping
   extension is obeyed.
 
+  Archives of animated GIFs and tools for building them can be found
+  at <a href=http://www.yahoo.com/Arts/Visual_Arts/Animation/Computer_Animation/Animated_GIFs/>
+  Yahoo!</a>.
+
   We are required to state: The Graphics Interchange Format(c) is the
   Copyright property of CompuServe Incorporated. GIF(sm) is a Service
   Mark property of CompuServe Incorporated.
 
-  <img src=qmovie.gif>
+  \warning Unisys has changed its position regarding GIF.  If you are
+  in a country where Unisys holds a patent on LZW compression and/or
+  decompression, Unisys may require a license from you.  These
+  countries include Canada, Japan, the USA, France, Germany, Italy and
+  the UK.  There is more information on Unisys web site: <a
+  href="http://corp2.unisys.com/LeadStory/lzwfaq.html">Overview of
+  Unisys' position.</a> GIF support may be removed in a future version
+  of Qt.  We recommend using the PNG format, which is available in the
+  <a href="imageio.html">Qt Image IO Extension</a> package.
+
+  <img src="qmovie.png">
 
   \sa QLabel::setMovie()
 */
 
-class QMoviePrivate : public QObject, public QShared,
+class QMovieFilePrivate : public QObject, public QShared,
 		      private QDataSink, private QImageConsumer
 {
     Q_OBJECT
 
 public: // for QMovie
 
-    // Creates a null QMoviePrivate
-    QMoviePrivate();
+    // Creates a null QMovieFilePrivate
+    QMovieFilePrivate();
 
-    // NOTE:  The ownership of the QDataSource is transferred to the QMoviePrivate
-    QMoviePrivate(QDataSource* src, QMovie* movie, int bufsize);
+    // NOTE:  The ownership of the QDataSource is transferred to the QMovieFilePrivate
+    QMovieFilePrivate(QDataSource* src, QMovie* movie, int bufsize);
 
-    virtual ~QMoviePrivate();
+    virtual ~QMovieFilePrivate();
 
     bool isNull() const;
 
@@ -97,13 +118,15 @@ public: // for QMovie
     void showChanges();
 
     // This as QImageConsumer
-    bool changed(const QRect& rect);
-    bool end();
-    bool frameDone();
+    void changed(const QRect& rect);
+    void end();
+    void preFrameDone(); //util func
+    void frameDone();
+    void frameDone(const QPoint&, const QRect& rect);
     void restartTimer();
-    bool setLooping(int l);
-    bool setFramePeriod(int milliseconds);
-    bool setSize(int w, int h);
+    void setLooping(int l);
+    void setFramePeriod(int milliseconds);
+    void setSize(int w, int h);
 
     // This as QDataSink
     int readyToReceive();
@@ -148,9 +171,15 @@ public:
 
     int error;
     bool empty;
+
+#ifdef QT_SAVE_MOVIE_HACK
+  bool save_image;
+  int image_number;
+#endif
 };
 
-QMoviePrivate::QMoviePrivate()
+
+QMovieFilePrivate::QMovieFilePrivate()
 {
     buffer = 0;
     pump = 0;
@@ -159,8 +188,8 @@ QMoviePrivate::QMoviePrivate()
     init(FALSE);
 }
 
-// NOTE:  The ownership of the QDataSource is transferred to the QMoviePrivate
-QMoviePrivate::QMoviePrivate(QDataSource* src, QMovie* movie, int bufsize) :
+// NOTE:  The ownership of the QDataSource is transferred to the QMovieFilePrivate
+QMovieFilePrivate::QMovieFilePrivate(QDataSource* src, QMovie* movie, int bufsize) :
     that(movie),
     buf_size(bufsize)
 {
@@ -174,30 +203,42 @@ QMoviePrivate::QMoviePrivate(QDataSource* src, QMovie* movie, int bufsize) :
     init(TRUE);
 }
 
-QMoviePrivate::~QMoviePrivate()
+QMovieFilePrivate::~QMovieFilePrivate()
 {
-    delete buffer;
+    if ( buffer )				// Avoid purify complaint
+	delete [] buffer;
     delete pump;
     delete decoder;
     delete source;
 }
 
-bool QMoviePrivate::isNull() const
+bool QMovieFilePrivate::isNull() const
 {
     return !pump;
 }
 
 // Initialize.  Only actually allocate any space if \a fully is TRUE,
-// otherwise, just enough to be a valid null QMoviePrivate.
-void QMoviePrivate::init(bool fully)
+// otherwise, just enough to be a valid null QMovieFilePrivate.
+void QMovieFilePrivate::init(bool fully)
 {
+#ifdef QT_SAVE_MOVIE_HACK
+    save_image = TRUE;
+    image_number = 0;
+#endif
+
     buf_usage = buf_r = buf_w = 0;
-    delete buffer;
+    if ( buffer )				// Avoid purify complaint
+	delete [] buffer;
     buffer = fully ? new uchar[buf_size] : 0;
 
     delete decoder;
     decoder = fully ? new QImageDecoder(this) : 0;
 
+#ifdef AVOID_OPEN_FDS
+    if ( source && !source->isOpen() )
+	source->open(IO_ReadOnly);
+#endif
+
     waitingForFrameTick = FALSE;
     stepping = -1;
     framenumber = 0;
@@ -211,7 +252,7 @@ void QMoviePrivate::init(bool fully)
     empty = TRUE;
 }
 
-void QMoviePrivate::flushBuffer()
+void QMovieFilePrivate::flushBuffer()
 {
     while (buf_usage && !waitingForFrameTick && stepping != 0 && !error) {
 	int used = decoder->decode(buffer + buf_r,
@@ -224,10 +265,11 @@ void QMoviePrivate::flushBuffer()
 	buf_r = (buf_r + used)%buf_size;
 	buf_usage -= used;
     }
+    if(error) frametimer->stop();
     maybeReady();
 }
 
-void QMoviePrivate::updatePixmapFromImage()
+void QMovieFilePrivate::updatePixmapFromImage()
 {
     if (changed_area.isEmpty()) return;
 
@@ -236,9 +278,17 @@ void QMoviePrivate::updatePixmapFromImage()
     QImage img(changed_area.size(), 8, gimg.numColors());
     img.setAlphaBuffer(gimg.hasAlphaBuffer());
 
+#ifdef QT_SAVE_MOVIE_HACK
+    if ( save_image ) {
+	QString name;
+	name.sprintf("movie%i.ppm",image_number++);
+	gimg.save( name, "PPM" );
+    }
+#endif
+
     // Copy color map.
     memcpy(img.colorTable(), gimg.colorTable(),
-	sizeof(QRgb)*gimg.numColors());
+	   sizeof(QRgb)*gimg.numColors());
 
     // Copy pixels.
     const int t = changed_area.top();
@@ -264,7 +314,7 @@ void QMoviePrivate::updatePixmapFromImage()
 	    // Resize to size of image
 	    if (mymask.isNull()) {
 		mymask.resize(gimg.width(), gimg.height());
-		mymask.fill(color1);
+		mymask.fill( Qt::color1 );
 	    }
 	}
 	mypixmap.setMask(QBitmap()); // Remove reference to my mask
@@ -281,7 +331,7 @@ void QMoviePrivate::updatePixmapFromImage()
     }
 }
 
-void QMoviePrivate::showChanges()
+void QMovieFilePrivate::showChanges()
 {
     if (changed_area.isValid()) {
 	updatePixmapFromImage();
@@ -293,20 +343,19 @@ void QMoviePrivate::showChanges()
     }
 }
 
-// QMoviePrivate as QImageConsumer
-bool QMoviePrivate::changed(const QRect& rect)
+// QMovieFilePrivate as QImageConsumer
+void QMovieFilePrivate::changed(const QRect& rect)
 {
-    if (!frametimer->isActive()) frametimer->start(0);
+    if (!frametimer->isActive())
+	frametimer->start(0);
     changed_area = changed_area.unite(rect);
-    return TRUE;
 }
 
-bool QMoviePrivate::end()
+void QMovieFilePrivate::end()
 {
-    return TRUE; // Not that it will do anything.
 }
 
-bool QMoviePrivate::frameDone()
+void QMovieFilePrivate::preFrameDone()
 {
     if (stepping > 0) {
 	stepping--;
@@ -318,13 +367,32 @@ bool QMoviePrivate::frameDone()
 	waitingForFrameTick = TRUE;
 	restartTimer();
     }
+}
+void QMovieFilePrivate::frameDone()
+{
+    preFrameDone();
     showChanges();
     emit dataStatus(QMovie::EndOfFrame);
     framenumber++;
-    return FALSE;
+}
+void QMovieFilePrivate::frameDone(const QPoint& p,
+				const QRect& rect)
+{
+    preFrameDone();
+    const QImage& gimg = decoder->image();
+    if (framenumber==0) {
+	mypixmap.resize(gimg.width(), gimg.height());
+	emit sizeChanged(gimg.size());
+    }
+    bitBlt(&mypixmap, p.x(), p.y(), &gimg, rect.x(), rect.y(),
+		    rect.width(), rect.height());
+    // TODO: transparency
+    emit areaChanged(QRect(p,rect.size()));
+    emit dataStatus(QMovie::EndOfFrame);
+    framenumber++;
 }
 
-void QMoviePrivate::restartTimer()
+void QMovieFilePrivate::restartTimer()
 {
     if (speed > 0) {
 	int i = frameperiod >= 0 ? frameperiod * 100/speed : 0;
@@ -337,7 +405,7 @@ void QMoviePrivate::restartTimer()
     }
 }
 
-bool QMoviePrivate::setLooping(int nloops)
+void QMovieFilePrivate::setLooping(int nloops)
 {
     if (loop == -1) { // Only if we don't already know how many loops!
 	if (source->rewindable()) {
@@ -348,37 +416,35 @@ bool QMoviePrivate::setLooping(int nloops)
 	    loop = -2;
 	}
     }
-
-    return TRUE;
 }
 
-bool QMoviePrivate::setFramePeriod(int milliseconds)
+void QMovieFilePrivate::setFramePeriod(int milliseconds)
 {
     // Animation:  only show complete frame
     frameperiod = milliseconds;
     if (stepping<0 && frameperiod >= 0) restartTimer();
-    return TRUE;
 }
 
-bool QMoviePrivate::setSize(int w, int h)
+void QMovieFilePrivate::setSize(int w, int h)
 {
     if (mypixmap.width() != w || mypixmap.height() != h) {
 	mypixmap.resize(w, h);
 	emit sizeChanged(QSize(w, h));
     }
-    return TRUE;
 }
 
 
-// QMoviePrivate as QDataSink
+// QMovieFilePrivate as QDataSink
 
-int QMoviePrivate::readyToReceive()
+int QMovieFilePrivate::readyToReceive()
 {
+    // Could pre-fill buffer, but more efficient to just leave the
+    // data back at the source.
     return (waitingForFrameTick || !stepping || buf_usage || error)
 	? 0 : buf_size;
 }
 
-void QMoviePrivate::receive(const uchar* b, int bytecount)
+void QMovieFilePrivate::receive(const uchar* b, int bytecount)
 {
     if ( bytecount ) empty = FALSE;
 
@@ -401,11 +467,15 @@ void QMoviePrivate::receive(const uchar* b, int bytecount)
     }
 }
 
-void QMoviePrivate::eof()
+void QMovieFilePrivate::eof()
 {
     if ( empty )
 	emit dataStatus(QMovie::SourceEmpty);
 
+#ifdef QT_SAVE_MOVIE_HACK
+    save_image = FALSE;
+#endif
+
     emit dataStatus(QMovie::EndOfLoop);
 
     if (loop >= 0) {
@@ -420,13 +490,18 @@ void QMoviePrivate::eof()
     } else {
 	delete decoder;
 	decoder = 0;
-	delete buffer;
+	if ( buffer )				// Avoid purify complaint
+	    delete [] buffer;
 	buffer = 0;
 	emit dataStatus(QMovie::EndOfMovie);
+#ifdef AVOID_OPEN_FDS
+	if ( source )
+	    source->close();
+#endif
     }
 }
 
-void QMoviePrivate::pause()
+void QMovieFilePrivate::pause()
 {
     if ( stepping ) {
 	stepping = 0;
@@ -435,8 +510,13 @@ void QMoviePrivate::pause()
     }
 }
 
-void QMoviePrivate::refresh()
+void QMovieFilePrivate::refresh()
 {
+    if (!decoder) {
+	frametimer->stop();
+	return;
+    }
+
     if (frameperiod < 0 && loop == -1) {
 	// Only show changes if probably not an animation
 	showChanges();
@@ -450,7 +530,7 @@ void QMoviePrivate::refresh()
     flushBuffer();
 }
 
-///////////////// End of QMoviePrivate /////////////////
+///////////////// End of QMovieFilePrivate /////////////////
 
 
 
@@ -464,15 +544,12 @@ void QMoviePrivate::refresh()
 */
 QMovie::QMovie()
 {
-    d = new QMoviePrivate();
+    d = new QMovieFilePrivate();
 }
 
-#if 0
-// Removed - QDataSource should be the interface.  QIODevice is not useful,
-// as it doesn't emit signals upon data availability.
-/*
+/*!
   Creates a QMovie which reads an image sequence from the given
-  QIODevice.  The device must be allocated dynamically,
+  QDataSource.  The source must be allocated dynamically,
   as it becomes owned by the QMovie, and will be destroyed
   when the movie is destroyed.
   The movie starts playing as soon as event processing continues.
@@ -482,32 +559,29 @@ QMovie::QMovie()
   value, the better interleaved the movie playback will be with other
   event processing, but the slower the overall processing.
 */
-QMovie::QMovie(QIODevice* src, int bufsize)
+QMovie::QMovie(QDataSource* src, int bufsize)
 {
-    d = new QMoviePrivate(new QIODeviceSource(src), this, bufsize);
+    d = new QMovieFilePrivate(src, this, bufsize);
 }
-#endif
 
 /*!
-  \overload
   Creates a QMovie which reads an image sequence from the named file.
 */
-QMovie::QMovie(const char* fileName, int bufsize)
+QMovie::QMovie(const QString &fileName, int bufsize)
 {
     QFile* file = new QFile(fileName);
     file->open(IO_ReadOnly);
-    d = new QMoviePrivate(new QIODeviceSource(file), this, bufsize);
+    d = new QMovieFilePrivate(new QIODeviceSource(file), this, bufsize);
 }
 
 /*!
-  \overload
   Creates a QMovie which reads an image sequence from given data.
 */
 QMovie::QMovie(QByteArray data, int bufsize)
 {
     QBuffer* buffer = new QBuffer(data);
     buffer->open(IO_ReadOnly);
-    d = new QMoviePrivate(new QIODeviceSource(buffer), this, bufsize);
+    d = new QMovieFilePrivate(new QIODeviceSource(buffer), this, bufsize);
 }
 
 /*!
@@ -615,7 +689,7 @@ int QMovie::frameNumber() const
 */
 bool QMovie::paused() const
 {
-    return d->stepping==0;
+    return d->stepping == 0;
 }
 
 /*!
@@ -653,8 +727,9 @@ void QMovie::pause()
 */
 void QMovie::unpause()
 {
-    if ( d->stepping >= 0 ) {
-	if (d->isNull()) return;
+    if ( d->stepping >= 0 )	{
+	if (d->isNull())
+	    return;
 	d->stepping = -1;
 	d->restartTimer();
     }
@@ -665,8 +740,8 @@ void QMovie::unpause()
 */
 void QMovie::step(int steps)
 {
-    if (d->isNull()) return;
-
+    if (d->isNull())
+	return;
     d->stepping = steps;
     d->frametimer->start(0);
     d->waitingForFrameTick = FALSE; // Full speed ahead!
@@ -686,8 +761,8 @@ void QMovie::step()
 */
 void QMovie::restart()
 {
-    if (d->isNull()) return;
-
+    if (d->isNull())
+	return;
     if (d->source->rewindable()) {
 	d->source->enableRewind(TRUE);
 	d->source->rewind();
@@ -714,22 +789,19 @@ int QMovie::speed() const
 void QMovie::setSpeed(int percent)
 {
     int oldspeed = d->speed;
-
     if ( oldspeed != percent && percent >= 0 ) {
 	d->speed = percent;
-
 	// Restart timer only if really needed
 	if (d->stepping < 0) {
 	    if ( !percent || !oldspeed    // To or from zero
-	    || oldspeed*4 / percent > 4   // More than 20% slower
-	    || percent*4 / oldspeed > 4   // More than 20% faster
-	    )
+		 || oldspeed*4 / percent > 4   // More than 20% slower
+		 || percent*4 / oldspeed > 4   // More than 20% faster
+		 )
 		d->restartTimer();
 	}
     }
 }
 
-
 /*!
   Connects the given member, of type \code void member(const QSize&) \endcode
   such that it is signalled when the movie changes size.
@@ -738,7 +810,7 @@ void QMovie::setSpeed(int percent)
   persist until they are explicitly disconnected with disconnectResize(), or
   until \e every shared copy of the movie is deleted.
 */
-void QMovie::connectResize(QObject* receiver, const char* member)
+void QMovie::connectResize(QObject* receiver, const char *member)
 {
     QObject::connect(d, SIGNAL(sizeChanged(const QSize&)), receiver, member);
 }
@@ -748,7 +820,7 @@ void QMovie::connectResize(QObject* receiver, const char* member)
   previously connected by
   connectResize().
 */
-void QMovie::disconnectResize(QObject* receiver, const char* member)
+void QMovie::disconnectResize(QObject* receiver, const char *member)
 {
     QObject::disconnect(d, SIGNAL(sizeChanged(const QSize&)), receiver, member);
 }
@@ -762,7 +834,7 @@ void QMovie::disconnectResize(QObject* receiver, const char* member)
   persist until they are explicitly disconnected with disconnectUpdate(), or
   until \e every shared copy of the movie is deleted.
 */
-void QMovie::connectUpdate(QObject* receiver, const char* member)
+void QMovie::connectUpdate(QObject* receiver, const char *member)
 {
     QObject::connect(d, SIGNAL(areaChanged(const QRect&)), receiver, member);
 }
@@ -772,7 +844,7 @@ void QMovie::connectUpdate(QObject* receiver, const char* member)
   previously connected by
   connectUpdate().
 */
-void QMovie::disconnectUpdate(QObject* receiver, const char* member)
+void QMovie::disconnectUpdate(QObject* receiver, const char *member)
 {
     QObject::disconnect(d, SIGNAL(areaChanged(const QRect&)), receiver, member);
 }
@@ -802,7 +874,7 @@ void QMovie::disconnectUpdate(QObject* receiver, const char* member)
   persist until they are explicitly disconnected with disconnectStatus(), or
   until \e every shared copy of the movie is deleted.
 */
-void QMovie::connectStatus(QObject* receiver, const char* member)
+void QMovie::connectStatus(QObject* receiver, const char *member)
 {
     QObject::connect(d, SIGNAL(dataStatus(int)), receiver, member);
 }
@@ -812,7 +884,7 @@ void QMovie::connectStatus(QObject* receiver, const char* member)
   previously connected by
   connectStatus().
 */
-void QMovie::disconnectStatus(QObject* receiver, const char* member)
+void QMovie::disconnectStatus(QObject* receiver, const char *member)
 {
     QObject::disconnect(d, SIGNAL(dataStatus(int)), receiver, member);
 }
@@ -821,52 +893,50 @@ void QMovie::disconnectStatus(QObject* receiver, const char* member)
 /* tmake ignore Q_OBJECT */
 
 //       MANUALLY INCLUDED.  Regenerate in vi with:   !Gmoc %
-//       WARNING! All changes made below will be lost
-//
 /****************************************************************************
-** QMoviePrivate meta object code from reading C++ file 'qmovie.cpp'
+** QMovieFilePrivate meta object code from reading C++ file 'standard input'
 **
-** Created: Thu Sep 4 15:31:20 1997
-**      by: The Qt Meta Object Compiler ($Revision: 1.26 $)
+** Created: Fri Aug 21 01:55:09 1998
+**      by: The Qt Meta Object Compiler ($Revision: 1.61 $)
 **
 ** WARNING! All changes made in this file will be lost!
 *****************************************************************************/
 
 #if !defined(Q_MOC_OUTPUT_REVISION)
-#define Q_MOC_OUTPUT_REVISION 2
-#elif Q_MOC_OUTPUT_REVISION != 2
-#error Moc format conflict - please regenerate all moc files
+#define Q_MOC_OUTPUT_REVISION 3
+#elif Q_MOC_OUTPUT_REVISION != 3
+#error "Moc format conflict - please regenerate all moc files"
 #endif
 
-#include <qmetaobj.h>
+#include <qmetaobject.h>
 
 
-const char *QMoviePrivate::className() const
+const char *QMovieFilePrivate::className() const
 {
-    return "QMoviePrivate";
+    return "QMovieFilePrivate";
 }
 
-QMetaObject *QMoviePrivate::metaObj = 0;
+QMetaObject *QMovieFilePrivate::metaObj = 0;
 
-void QMoviePrivate::initMetaObject()
+void QMovieFilePrivate::initMetaObject()
 {
     if ( metaObj )
 	return;
     if ( strcmp(QObject::className(), "QObject") != 0 )
-	badSuperclassWarning("QMoviePrivate","QObject");
+	badSuperclassWarning("QMovieFilePrivate","QObject");
     if ( !QObject::metaObject() )
 	QObject::initMetaObject();
-    typedef void(QMoviePrivate::*m1_t0)();
-    m1_t0 v1_0 = &QMoviePrivate::refresh;
+    typedef void(QMovieFilePrivate::*m1_t0)();
+    m1_t0 v1_0 = &QMovieFilePrivate::refresh;
     QMetaData *slot_tbl = new QMetaData[1];
     slot_tbl[0].name = "refresh()";
     slot_tbl[0].ptr = *((QMember*)&v1_0);
-    typedef void(QMoviePrivate::*m2_t0)(const QSize&);
-    typedef void(QMoviePrivate::*m2_t1)(const QRect&);
-    typedef void(QMoviePrivate::*m2_t2)(int);
-    m2_t0 v2_0 = &QMoviePrivate::sizeChanged;
-    m2_t1 v2_1 = &QMoviePrivate::areaChanged;
-    m2_t2 v2_2 = &QMoviePrivate::dataStatus;
+    typedef void(QMovieFilePrivate::*m2_t0)(const QSize&);
+    typedef void(QMovieFilePrivate::*m2_t1)(const QRect&);
+    typedef void(QMovieFilePrivate::*m2_t2)(int);
+    m2_t0 v2_0 = &QMovieFilePrivate::sizeChanged;
+    m2_t1 v2_1 = &QMovieFilePrivate::areaChanged;
+    m2_t2 v2_2 = &QMovieFilePrivate::dataStatus;
     QMetaData *signal_tbl = new QMetaData[3];
     signal_tbl[0].name = "sizeChanged(const QSize&)";
     signal_tbl[1].name = "areaChanged(const QRect&)";
@@ -874,26 +944,15 @@ void QMoviePrivate::initMetaObject()
     signal_tbl[0].ptr = *((QMember*)&v2_0);
     signal_tbl[1].ptr = *((QMember*)&v2_1);
     signal_tbl[2].ptr = *((QMember*)&v2_2);
-    metaObj = new QMetaObject( "QMoviePrivate", "QObject",
+    metaObj = new QMetaObject( "QMovieFilePrivate", "QObject",
 	slot_tbl, 1,
 	signal_tbl, 3 );
 }
 
-#if !defined(Q_MOC_CONNECTIONLIST_DECLARED)
-#define Q_MOC_CONNECTIONLIST_DECLARED
-#include <qlist.h>
-#if defined(Q_DECLARE)
-Q_DECLARE(QListM,QConnection);
-Q_DECLARE(QListIteratorM,QConnection);
-#else
-// for compatibility with old header files
-declare(QListM,QConnection);
-declare(QListIteratorM,QConnection);
-#endif
-#endif
+#include "qsignalslotimp.h"
 
 // SIGNAL sizeChanged
-void QMoviePrivate::sizeChanged( const QSize& t0 )
+void QMovieFilePrivate::sizeChanged( const QSize& t0 )
 {
     QConnectionList *clist = receivers("sizeChanged(const QSize&)");
     if ( !clist || signalsBlocked() )
@@ -925,7 +984,7 @@ void QMoviePrivate::sizeChanged( const QSize& t0 )
 }
 
 // SIGNAL areaChanged
-void QMoviePrivate::areaChanged( const QRect& t0 )
+void QMovieFilePrivate::areaChanged( const QRect& t0 )
 {
     QConnectionList *clist = receivers("areaChanged(const QRect&)");
     if ( !clist || signalsBlocked() )
@@ -957,7 +1016,7 @@ void QMoviePrivate::areaChanged( const QRect& t0 )
 }
 
 // SIGNAL dataStatus
-void QMoviePrivate::dataStatus( int t0 )
+void QMovieFilePrivate::dataStatus( int t0 )
 {
     activate_signal( "dataStatus(int)", t0 );
 }
