@@ -1,5 +1,5 @@
- /****************************************************************************
-** $Id: qobject.cpp,v 2.52.2.5 1998/11/20 12:28:28 warwick Exp $
+/****************************************************************************
+** $Id: qobject.cpp,v 2.99 1999/06/21 23:50:14 agulbra Exp $
 **
 ** Implementation of QObject class
 **
@@ -7,29 +7,52 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
+#include "qapplication.h"
 #include "qobject.h"
 #include "qobjectlist.h"
 #include "qobjectdict.h"
 #include "qsignalslotimp.h"
 #include "qregexp.h"
 #include <ctype.h>
-#if defined(DEBUG)
-#include "qapplication.h"
-#endif
+
+
+/*! \class Qt qnamespace.h
+
+  \brief The Qt class is a namespace for miscellaneous identifiers
+  that need to be global-like.
+
+  Normally, you can ignore this class.  QObject and a few other
+  classes inherit it, so that all the identifiers in the Qt namespace
+  are visible to you without qualification.
+
+  However, occasionally you may need to say \c Qt::black instead just
+  \c black, particularly in static utility functions (such as many
+  class factories).
+
+*/
+
+/*! \enum Qt::Orientation
+
+  This type is used to signify whether an object should be \a
+  Horizontal or \a Vertical (for example in QScrollBar).
+*/
+
 
 /*!
   \class QObject qobject.h
@@ -140,7 +163,7 @@
 */
 
 
-/*!
+/* (no '!' on purpose since this is an internal class)
   \class QSenderObject qobject.h
   \brief Internal object used for sending signals.
 
@@ -155,7 +178,7 @@
   and slots.  Alternatively, you can use the QSignal class.
 */
 
-/*!
+/*
   \fn void QSenderObject::setSender (QObject* s)
 
   Internal function, used in signal-slot connections.
@@ -187,9 +210,9 @@ static inline bool isSpace( char x )
 #endif
 }
 
-static QString rmWS( const char *src )
+static QCString rmWS( const char *src )
 {
-    QString result( strlen(src)+1 );
+    QCString result( strlen(src)+1 );
     char *d = result.data();
     char *s = (char *)src;
     char last = 0;
@@ -217,14 +240,9 @@ int   qStartTimer( int interval, QObject *obj );
 bool  qKillTimer( int id );
 bool  qKillTimer( QObject *obj );
 
-void  qRemovePostedEvents( QObject * );
+void  qRemovePostedEvents( QObject* );
 
 
-#if !defined(QT_MAKEDLL)
-Q_DECLARE(QDictM,QListM_QConnection);		// dict for multiple signals
-Q_DECLARE(QDictIteratorM,QListM_QConnection);	// dict iterator
-#endif
-
 QMetaObject *QObject::metaObj = 0;
 
 
@@ -251,16 +269,6 @@ static void removeObjFromList( QObjectList *objList, const QObject *obj,
 
   Returns 0 if there is no such child.
 
-  The CHILD macro (defined in qwindowdefs.h) does all this, and also
-  casts the returt type to \a type *.
-
-  Examples:
-  \code
-    QListBox * c = CHILD(myWidget,QListBox,"listboxname");
-    if ( c )
-        c->insertItem( "another string" );
-  \endcode
-
   \code
     QListBox * c = (QListBox *)::qt_find_obj_child(myWidget,QListBox,
                                                    "listboxname");
@@ -316,23 +324,25 @@ QObject::QObject( QObject *parent, const char *name )
 {
     if ( !objectDict )				// will create object dict
 	initMetaObject();
-    objname = name ? qstrdup(name) : 0;		// set object name
-    parentObj = parent;				// set parent
-    childObjects = 0;				// no children yet
-    connections = 0;				// no connections yet
+    objname       = name ? qstrdup(name) : 0;   // set object name
+    parentObj     = 0;				// parent set by insertChild()
+    childObjects  = 0;				// no children yet
+    connections   = 0;				// no connections yet
     senderObjects = 0;				// no signals connected yet
-    eventFilters = 0;				// no filters installed
-    sigSender = 0;				// no sender yet
+    eventFilters  = 0;				// no filters installed
+    sigSender     = 0;				// no sender yet
     isSignal   = FALSE;				// assume not a signal object
     isWidget   = FALSE;				// assume not a widget object
-    hiPriority = FALSE;				// normal priority
     pendTimer  = FALSE;				// no timers yet
     pendEvent  = FALSE;				// no events yet
     blockSig   = FALSE;				// not blocking signals
-    if ( parentObj )				// add object to parent
-	parentObj->insertChild( this );
+    wasDeleted = FALSE;				// double-delete catcher
+    if ( parent )				// add object to parent
+	parent->insertChild( this );
 }
 
+
+
 /*!
   Destroys the object and all its children objects.
 
@@ -347,13 +357,21 @@ QObject::QObject( QObject *parent, const char *name )
 
 QObject::~QObject()
 {
+    if ( wasDeleted ) {
+#if defined(DEBUG)
+	qWarning( "Double QObject deletion detected." );
+#endif
+	return;
+    }
+    wasDeleted = 1;
     emit destroyed();
     if ( objname )
-	delete [] objname;
+	delete [] (char*)objname;
+    objname = 0;
     if ( pendTimer )				// might be pending timers
 	qKillTimer( this );
-    if ( pendEvent )				// pending posted events
-	qRemovePostedEvents( this );
+    if ( pendEvent )
+	QApplication::removePostedEvents( this );
     if ( parentObj )				// remove it from parent object
 	parentObj->removeChild( this );
     register QObject *obj;
@@ -488,12 +506,42 @@ bool QObject::inherits( const char *clname ) const
     return FALSE;
 }
 
+/*!
+  Returns a list of all super classes. If \e includeThis is TRUE
+  then the most derived class name of this object is added, too.
+  The most derived classes appear first in the list. That means that
+  QObject is always the last entry in the list.
+*/
+
+QStringList QObject::superClasses( bool includeThis ) const
+{
+  QStringList lst;
+
+  QMetaObject *meta = queryMetaObject();
+  if ( !includeThis )
+    meta = meta->superClass();
+
+  while ( meta ) {
+    lst.append( QString::fromLatin1(meta->className()) );
+    meta = meta->superClass();
+  }
+
+  return lst;
+}
+
 
 /*!
   \fn const char *QObject::name() const
 
-  Returns the name of this object, or 0 if the object does not have a
-  name.
+  Returns the name of this object. If the object does not have a name,
+  it will return "unnamed", so that printf() (used in debug()) will
+  not be asked to output a null pointer.  If you want a null pointer
+  to be returned for unnamed objects, you can call name(0).
+
+  \code
+    qDebug( "MyClass::setPrecision(): (%s) unable to set precision to %f",
+	    name(), newPrecision );
+  \endcode
 
   The object name is set by the constructor or by the setName() function.
   The object name is not very useful in the current version of Qt, but
@@ -504,24 +552,22 @@ bool QObject::inherits( const char *clname ) const
 
   \sa setName(), className(), queryList()
 */
+const char * QObject::name() const
+{
+    return objname ? objname : "unnamed";
+}
 
 /*!
-  \overload const char *QObject::name( const char * defaultName ) const
-
   Returns the name of this object, or \a defaultName if the object
   does not have a name.
-
-  This version of name() is particularly useful for debug() calls such as:
-
-  \code
-    debug( "MyClass::setPrecision(): (%s) unable to set precision to %f",
-	    name( "unnamed" ), newPrecision );
-  \endcode
-
-  name( "unnamed" ) is used here so that printf() (used in debug())
-  will not be asked to output a null pointer.
 */
 
+const char * QObject::name( const char * defaultName ) const
+{
+    return objname ? objname : defaultName;
+}
+
+
 /*!
   Sets the name of this object to \e name.  The default name is the
   one assigned by the constructor.
@@ -538,10 +584,49 @@ bool QObject::inherits( const char *clname ) const
 void QObject::setName( const char *name )
 {
     if ( objname )
-	delete objname;
+	delete [] (char*) objname;
     objname = name ? qstrdup(name) : 0;
 }
 
+/*!
+  Returns the pointer to a child widget with the required name and type or
+  0 if no child matches. This function works recursive. That means it traverses
+  the entire object tree to find the child. That in turn means that names have
+  to be unique with regard to their toplevel window.
+
+  If multiple widgets with the same name and type are found then it is undefined
+  which one of them is returned.
+
+  If \e type is set to 0 then the only criterion is the objects name.
+
+  This function is useful if you need a widget of a dialog that was created from a ressource file.
+*/
+
+QObject* QObject::child( const char *name, const char *type )
+{
+    const QObjectList *list = children();
+    if ( list ) {
+	QObjectListIt it( *list );
+	QObject *obj;
+	while ( ( obj = it.current() ) ) {
+	    ++it;
+	    if ( ( !type || obj->inherits(type) ) && qstrcmp( name, obj->name() ) == 0 )
+		return obj;
+	}
+
+	// Recursion: Ask our children ...
+	QObjectListIt it2( *list );
+	while ( ( obj = it2.current() ) ) {
+	    ++it2;
+	    QObject* o = obj->child( name, type );
+	    if ( o )
+	      return o;
+	}
+    }
+
+    return 0;
+}
+
 /*!
   \fn bool QObject::isWidgetType() const
   Returns TRUE if the object is a widget, or FALSE if not.
@@ -575,15 +660,21 @@ bool QObject::event( QEvent *e )
 {
 #if defined(CHECK_NULL)
     if ( e == 0 )
-	warning( "QObject::event: Null events are not permitted" );
+	qWarning( "QObject::event: Null events are not permitted" );
 #endif
     if ( eventFilters ) {			// try filters
 	if ( activate_filters(e) )		// stopped by a filter
 	    return TRUE;
     }
-    if ( e->type() == Event_Timer ) {		// timer event
+    switch ( e->type() ) {
+      case QEvent::Timer:
 	timerEvent( (QTimerEvent*)e );
 	return TRUE;
+      case QEvent::ChildInserted: case QEvent::ChildRemoved:
+	childEvent( (QChildEvent*)e );
+	return TRUE;
+      default:
+	break;
     }
     return FALSE;
 }
@@ -592,8 +683,6 @@ bool QObject::event( QEvent *e )
   This event handler can be reimplemented in a subclass to receive
   timer events for the object.
 
-  The default implementation does nothing.
-
   QTimer provides a higher-level interface to the timer functionality,
   and also more general information about timers.
 
@@ -605,6 +694,20 @@ void QObject::timerEvent( QTimerEvent * )
 }
 
 
+/*!
+  This event handler can be reimplemented in a subclass to receive
+  child events.
+
+  Child events are sent to objects when children are inserted or removed.
+
+  \sa event(), QChildEvent
+*/
+
+void QObject::childEvent( QChildEvent * )
+{
+}
+
+
 /*!
   Filters events if this object has been installed as an event filter for
   another object.
@@ -698,14 +801,14 @@ void QObject::blockSignals( bool block )
     MyObject::MyObject( QObject *parent, const char *name )
 	: QObject( parent, name )
     {
-	startTimer( 50 );			// 50 millisec timer
+	startTimer( 50 );			// 50 millisecond timer
 	startTimer( 1000 );			// 1 second timer
 	startTimer( 60000 );			// 1 minute timer
     }
 
     void MyObject::timerEvent( QTimerEvent *e )
     {
-	debug( "timer event, id=%d", e->timerId() );
+	qDebug( "timer event, id=%d", e->timerId() );
     }
   \endcode
 
@@ -775,7 +878,7 @@ static void objSearch( QObjectList *result,
 	    if ( objName )
 		ok = qstrcmp(objName,obj->name()) == 0;
 	    else if ( rx )
-		ok = rx->match(obj->name()) >= 0;
+		ok = rx->match(QString::fromLatin1(obj->name())) >= 0;
 	}
 	if ( ok )				// match!
 	    result->append( obj );
@@ -862,7 +965,7 @@ QObjectList *QObject::queryList( const char *inheritsClass,
     QObjectList *list = new QObjectList;
     CHECK_PTR( list );
     if ( regexpMatch && objName ) {		// regexp matching
-	QRegExp rx = objName;
+	QRegExp rx(QString::fromLatin1(objName));
 	objSearch( list, (QObjectList *)children(), inheritsClass,
 		   0, &rx, recursiveSearch );
     } else {
@@ -884,8 +987,8 @@ QConnectionList *QObject::receivers( const char *signal ) const
 {
     if ( connections && signal ) {
 	if ( *signal == '2' ) {			// tag == 2, i.e. signal
-	    QString s = rmWS( signal+1 );
-	    return connections->find( s );
+	    QCString s = rmWS( signal+1 );
+	    return connections->find( (const char*)s );
 	} else {
 	    return connections->find( signal );
 	}
@@ -894,18 +997,14 @@ QConnectionList *QObject::receivers( const char *signal ) const
 }
 
 
-#if QT_VERSION == 200
-#error "insertChild and removeChild should be virtual."
-#endif
-
 /*!
   Inserts an object \e obj into the list of child objects.
 
   \warning This function cannot be used to make a widget a child
   widget of another.  Child widgets can be created only by setting the
-  parent widget in the constructor or by calling QWidget::recreate().
+  parent widget in the constructor or by calling QWidget::reparent().
 
-  \sa removeChild(), QWidget::recreate()
+  \sa removeChild(), QWidget::reparent()
 */
 
 void QObject::insertChild( QObject *obj )
@@ -913,8 +1012,8 @@ void QObject::insertChild( QObject *obj )
     if ( obj->parentObj ) {
 #if defined(CHECK_STATE)
 	if ( obj->parentObj != this && obj->isWidgetType() )
-	    warning( "QObject::insertChild: Cannot reparent a widget, "
-		     "use QWidget::recreate() instead" );
+	    qWarning( "QObject::insertChild: Cannot reparent a widget, "
+		     "use QWidget::reparent() instead" );
 #endif
 	obj->parentObj->removeChild( obj );
     }
@@ -925,16 +1024,17 @@ void QObject::insertChild( QObject *obj )
     }
 #if defined(CHECK_STATE)
     else if ( childObjects->findRef(obj) >= 0 ) {
-	warning( "QObject::insertChild: Object %s::%s already in list",
+	qWarning( "QObject::insertChild: Object %s::%s already in list",
 		 obj->className(), obj->name( "unnamed" ) );
 	return;
     }
 #endif
     obj->parentObj = this;
-    if ( obj->hiPriority )
-	childObjects->insert( 0, obj );		// high priority inserts
-    else
-	childObjects->append( obj );		// normal priority appends
+    childObjects->append( obj );
+
+    obj->pendEvent = TRUE;
+    QChildEvent *e = new QChildEvent( QEvent::ChildInserted, obj );
+    QApplication::postEvent( this, e );
 }
 
 /*!
@@ -944,7 +1044,7 @@ void QObject::insertChild( QObject *obj )
   This function will not remove a child widget from the screen.
   It will only remove it from the parent widget's list of children.
 
-  \sa insertChild(), QWidget::recreate()
+  \sa insertChild(), QWidget::reparent()
 */
 
 void QObject::removeChild( QObject *obj )
@@ -955,6 +1055,10 @@ void QObject::removeChild( QObject *obj )
 	    delete childObjects;		// last child removed
 	    childObjects = 0;			// reset children list
 	}
+
+	// remove events must be sent, not posted!!!
+	QChildEvent ce( QEvent::ChildRemoved, obj );
+	QApplication::sendEvent( this, &ce );
     }
 }
 
@@ -986,15 +1090,15 @@ void QObject::removeChild( QObject *obj )
     MyWidget::MyWidget( QWidget *parent, const char *name )
 	: QWidget( parent, name )
     {
-	if ( parent )				// has a child widget
+	if ( parent )				// has a parent widget
 	    parent->installEventFilter( this ); // then install filter
     }
 
     bool MyWidget::eventFilter( QObject *, QEvent *e )
     {
-	if ( e->type() == Event_KeyPress ) {	// key press
+	if ( e->type() == QEvent::KeyPress ) {	// key press
 	    QKeyEvent *k = (QKeyEvent*)e;
-	    debug( "Ate key press %d", k->key() );
+	    qDebug( "Ate key press %d", k->key() );
 	    return TRUE;			// eat event
 	}
 	return FALSE;				// standard event processing
@@ -1059,10 +1163,10 @@ static bool check_signal_macro( const QObject *sender, const char *signal,
     int sigcode = (int)(*signal) - '0';
     if ( sigcode != SIGNAL_CODE ) {
 	if ( sigcode == SLOT_CODE )
-	    warning( "QObject::%s: Attempt to %s non-signal %s::%s",
+	    qWarning( "QObject::%s: Attempt to %s non-signal %s::%s",
 		     func, op, sender->className(), signal+1 );
 	else
-	    warning( "QObject::%s: Use the SIGNAL macro to %s %s::%s",
+	    qWarning( "QObject::%s: Use the SIGNAL macro to %s %s::%s",
 		     func, op, sender->className(), signal );
 	return FALSE;
     }
@@ -1073,7 +1177,7 @@ static bool check_member_code( int code, const QObject *object,
 			       const char *member, const char *func )
 {
     if ( code != SLOT_CODE && code != SIGNAL_CODE ) {
-	warning( "QObject::%s: Use the SLOT or SIGNAL macro to "
+	qWarning( "QObject::%s: Use the SLOT or SIGNAL macro to "
 		 "%s %s::%s", func, func, object->className(), member );
 	return FALSE;
     }
@@ -1089,10 +1193,10 @@ static void err_member_notfound( int code, const QObject *object,
 	case SIGNAL_CODE: type = "signal"; break;
     }
     if ( strchr(member,')') == 0 )		// common typing mistake
-	warning( "QObject::%s: Parentheses expected, %s %s::%s",
+	qWarning( "QObject::%s: Parentheses expected, %s %s::%s",
 		 func, type, object->className(), member );
     else
-	warning( "QObject::%s: No such %s %s::%s",
+	qWarning( "QObject::%s: No such %s %s::%s",
 		 func, type, object->className(), member );
 }
 
@@ -1103,9 +1207,32 @@ static void err_info_about_objects( const char * func,
 {
     const char * a = sender->name(), * b = receiver->name();
     if ( a )
-	warning( "QObject::%s:  (sender name:   '%s')", func, a );
+	qWarning( "QObject::%s:  (sender name:   '%s')", func, a );
     if ( b )
-	warning( "QObject::%s:  (receiver name: '%s')", func, b );
+	qWarning( "QObject::%s:  (receiver name: '%s')", func, b );
+}
+
+static void err_info_about_candidates( int code,
+				       const QMetaObject* mo,
+				       const char* member,
+				       const char *func	)
+{
+    if ( strstr(member,"const char*") ) {
+	// porting help
+	QCString newname = member;
+	int p;
+	while ( (p=newname.find("const char*")) >= 0 ) {
+	    newname.replace(p, 11, "const QString&");
+	}
+	QMetaData *rm = 0;
+	switch ( code ) {
+	    case SLOT_CODE:   rm = mo->slot( newname, TRUE );	  break;
+	    case SIGNAL_CODE: rm = mo->signal( newname, TRUE ); break;
+	}
+	if ( rm ) {
+	    qWarning("QObject::%s:  Candidate: %s", func, newname.data());
+	}
+    }
 }
 
 
@@ -1211,13 +1338,16 @@ void QObject::badSuperclassWarning( const char *className,
 				    const char *superclassName )
 {
 #if defined(CHECK_NULL)
-    warning(
+    qWarning(
     "%s::initMetaObject(): Warning:\n"
     "    The class \"%s\" contains the Q_OBJECT macro, but inherits from the\n"
     "    \"%s\" class, which does not contain the Q_OBJECT macro.\n"
     "    Signal/slot behavior is undefined.\n",
-    (const char*)className, (const char*)className,
-    (const char*)superclassName );
+    className, className,
+    superclassName );
+#else
+    Q_UNUSED( className )
+    Q_UNUSED( superclassName )
 #endif
 }
 
@@ -1261,7 +1391,7 @@ void QObject::badSuperclassWarning( const char *className,
 	MyWidget();
     ...
     signals:
-	void aSignal(int);
+	void aSignal();
     ...
     private:
     ...
@@ -1296,7 +1426,7 @@ bool QObject::connect( const QObject *sender,	const char *signal,
 {
 #if defined(CHECK_NULL)
     if ( sender == 0 || receiver == 0 || signal == 0 || member == 0 ) {
-	warning( "QObject::connect: Cannot connect %s::%s to %s::%s",
+	qWarning( "QObject::connect: Cannot connect %s::%s to %s::%s",
 		 sender ? sender->className() : "(null)",
 		 signal ? signal+1 : "(null)",
 		 receiver ? receiver->className() : "(null)",
@@ -1304,8 +1434,8 @@ bool QObject::connect( const QObject *sender,	const char *signal,
 	return FALSE;
     }
 #endif
-    QString signal_name = rmWS( signal );	// white space stripped
-    QString member_name = rmWS( member );
+    QCString signal_name = rmWS( signal );	// white space stripped
+    QCString member_name = rmWS( member );
     signal = signal_name;
     member = member_name;
 
@@ -1322,6 +1452,7 @@ bool QObject::connect( const QObject *sender,	const char *signal,
     if ( !(sm=smeta->signal(signal,TRUE)) ) {	// no such signal
 #if defined(CHECK_RANGE)
 	err_member_notfound( SIGNAL_CODE, sender, signal, "connect" );
+	err_info_about_candidates( SIGNAL_CODE, smeta, signal, "connect" );
 	err_info_about_objects( "connect", sender, receiver );
 #endif
 	return FALSE;
@@ -1349,18 +1480,19 @@ bool QObject::connect( const QObject *sender,	const char *signal,
     if ( !rm ) {				// no such member
 #if defined(CHECK_RANGE)
 	err_member_notfound( membcode, r, member, "connect" );
+	err_info_about_candidates( membcode, rmeta, member, "connect" );
 	err_info_about_objects( "connect", sender, receiver );
 #endif
 	return FALSE;
     }
 #if defined(CHECK_RANGE)
     if ( !s->checkConnectArgs(signal,receiver,member) )
-	warning( "QObject::connect: Incompatible sender/receiver arguments"
+	qWarning( "QObject::connect: Incompatible sender/receiver arguments"
 		 "\n\t%s::%s --> %s::%s",
 		 s->className(), signal,
 		 r->className(), member );
 #endif
-    if ( !s->connections ) {		// create connections dict
+    if ( !s->connections ) {			// create connections dict
 	s->connections = new QSignalDict( 7, TRUE, FALSE );
 	CHECK_PTR( s->connections );
 	s->connections->setAutoDelete( TRUE );
@@ -1403,10 +1535,6 @@ bool QObject::connect( const QObject *sender,	const char *signal,
   involved are destroyed.
 */
 
-#if (QT_VERSION >= 200)
-#error "disconnect const foo"
-#endif
-
 /*!
   Disconnects \e signal in object \e sender from \e member in object \e
   receiver.
@@ -1459,14 +1587,14 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 {
 #if defined(CHECK_NULL)
     if ( sender == 0 || (receiver == 0 && member != 0) ) {
-	warning( "QObject::disconnect: Unexpected null parameter" );
+	qWarning( "QObject::disconnect: Unexpected null parameter" );
 	return FALSE;
     }
 #endif
     if ( !sender->connections )			// no connected signals
 	return FALSE;
-    QString signal_name;
-    QString member_name;
+    QCString signal_name;
+    QCString member_name;
     QMetaData *rm = 0;
     QObject *s = (QObject *)sender;
     QObject *r = (QObject *)receiver;
@@ -1489,6 +1617,7 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	if ( !rm ) {				// no such member
 #if defined(CHECK_RANGE)
 	    err_member_notfound( membcode, r, member, "disconnect" );
+	    err_info_about_candidates( membcode, rmeta, member, "connect" );
 	    err_info_about_objects( "disconnect", sender, receiver );
 #endif
 	    return FALSE;
@@ -1500,6 +1629,7 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
     if ( signal == 0 ) {			// any/all signals
 	QSignalDictIt it(*(s->connections));
 	while ( (clist=it.current()) ) {	// for all signals...
+	    // Tricky hack to avoid UTF conversion.
 	    const char *curkey = it.currentKey();
 	    ++it;
 	    c = clist->first();
@@ -1538,7 +1668,7 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	    if ( !smeta )			// no meta object
 		return FALSE;
 	    if ( !smeta->signal(signal,TRUE) )
-		warning( "QObject::disconnect: No such signal %s::%s",
+		qWarning( "QObject::disconnect: No such signal %s::%s",
 			 s->className(), signal );
 #endif
 	    return FALSE;
@@ -1607,12 +1737,29 @@ QMetaObject *QObject::queryMetaObject() const
     }
 #if defined(CHECK_NULL)
     if ( !m )					// still no meta object: error
-	warning( "QObject: Object %s::%s has no meta object",
+	qWarning( "QObject: Object %s::%s has no meta object",
 		 x->className(), x->name( "unnamed" ) );
 #endif
     return m;
 }
 
+/*!
+  Returns a translated version of \a text, or \a text if there is
+  no appropriate translated version.  All QObject subclasses which use the
+  \link metaobjects.html Q_OBJECT macro\endlink have an overridden
+  version of this.
+
+  \sa QApplication::translate()
+*/
+
+QString QObject::tr( const char *text )
+{
+    if ( qApp )
+	return qApp->translate( "QObject", text );
+    else
+	return QString::fromLatin1(text);
+}
+
 /*!
   Initializes the \link metaObject() meta object\endlink of this
   object. This method is automatically executed on demand from the
@@ -1620,6 +1767,14 @@ QMetaObject *QObject::queryMetaObject() const
   \sa metaObject()
 */
 void QObject::initMetaObject()
+{
+    staticMetaObject();
+}
+
+/*!
+  The functionality of initMetaObject(), provided as a static function.
+*/
+void QObject::staticMetaObject()
 {
     if ( metaObj )
 	return;
@@ -1683,8 +1838,8 @@ void QObject::activate_signal( const char *signal )
 */
 
 
-#define ACTIVATE_SIGNAL_WITH_PARAM(TYPE)				      \
-void QObject::activate_signal( const char *signal, TYPE param )		      \
+#define ACTIVATE_SIGNAL_WITH_PARAM(FNAME,TYPE)				      \
+void QObject::FNAME( const char *signal, TYPE param )			      \
 {									      \
     if ( !connections )							      \
 	return;								      \
@@ -1716,10 +1871,13 @@ void QObject::activate_signal( const char *signal, TYPE param )		      \
 
 // We don't want to duplicate too much text so...
 
-ACTIVATE_SIGNAL_WITH_PARAM( short )
-ACTIVATE_SIGNAL_WITH_PARAM( int )
-ACTIVATE_SIGNAL_WITH_PARAM( long )
-ACTIVATE_SIGNAL_WITH_PARAM( const char * )
+ACTIVATE_SIGNAL_WITH_PARAM( activate_signal, short )
+ACTIVATE_SIGNAL_WITH_PARAM( activate_signal, int )
+ACTIVATE_SIGNAL_WITH_PARAM( activate_signal, long )
+ACTIVATE_SIGNAL_WITH_PARAM( activate_signal, const char * )
+ACTIVATE_SIGNAL_WITH_PARAM( activate_signal_bool, bool )
+ACTIVATE_SIGNAL_WITH_PARAM( activate_signal_string, QString )
+ACTIVATE_SIGNAL_WITH_PARAM( activate_signal_strref, const QString & )
 
 
 /*****************************************************************************
@@ -1731,8 +1889,10 @@ static void dumpRecursive( int level, QObject *object )
 #if defined(DEBUG)
     if ( object ) {
 	QString buf;
-	buf.fill( '\t', level );
-	const char *name = object->name( "unnamed" );
+	buf.fill( '\t', level/2 );
+	if ( level % 2 )
+	    buf += "    ";
+	const char *name = object->name();
 	QString flags="";
 	if ( qApp->focusWidget() == object )
 	    flags += 'F';
@@ -1747,16 +1907,20 @@ static void dumpRecursive( int level, QObject *object )
 		flags += 'I';
 	    }
 	}	
-	debug( "%s%s::%s %s", (const char*)buf, object->className(), name,
-	    (const char*)flags );
+	qDebug( "%s%s::%s %s", (const char*)buf, object->className(), name,
+	    flags.latin1() );
 	if ( object->children() ) {
 	    QObjectListIt it(*object->children());
-	    while ( it ) {
-		dumpRecursive( level+1, it );
+	    QObject * c;
+	    while ( (c=it.current()) != 0 ) {
 		++it;
+		dumpRecursive( level+1, c );
 	    }
 	}
     }
+#else
+    Q_UNUSED( level )
+    Q_UNUSED( object )
 #endif
 }
 
@@ -1781,50 +1945,39 @@ void QObject::dumpObjectTree()
 void QObject::dumpObjectInfo()
 {
 #if defined(DEBUG)
-    debug( "OBJECT %s::%s", className(), name( "unnamed" ) );
-    debug( "  SIGNALS OUT" );
+    qDebug( "OBJECT %s::%s", className(), name( "unnamed" ) );
+    qDebug( "  SIGNALS OUT" );
     int n = 0;
     if ( connections ) {
 	QSignalDictIt it(*connections);
 	QConnectionList *clist;
 	while ( (clist=it.current()) ) {
-	    debug( "\t%s", it.currentKey() );
+	    qDebug( "\t%s", it.currentKey() );
 	    n++;
 	    ++it;
 	    register QConnection *c;
 	    QConnectionListIt cit(*clist);
 	    while ( (c=cit.current()) ) {
 		++cit;
-		debug( "\t  --> %s::%s %s", c->object()->className(),
+		qDebug( "\t  --> %s::%s %s", c->object()->className(),
 		       c->object()->name( "unnamed" ), c->memberName() );
 	    }
 	}
     }
     if ( n == 0 )
-	debug( "\t<None>" );
-    debug( "  SIGNALS IN" );
+	qDebug( "\t<None>" );
+    qDebug( "  SIGNALS IN" );
     n = 0;
     if ( senderObjects ) {
 	QObject *sender = senderObjects->first();
 	while ( sender ) {
-	    debug( "\t%s::%s",
+	    qDebug( "\t%s::%s",
 		   sender->className(), sender->name( "unnamed" ) );
 	    n++;
 	    sender = senderObjects->next();
 	}
     }
     if ( n == 0 )
-	debug( "\t<None>" );
+	qDebug( "\t<None>" );
 #endif
 }
-
-
-
-/*!  Returns a translated version of \a text, or \a text if there is
-  no appropriate translated version.
-*/
-
-const char * QObject::tr( const char * text ) const
-{
-    return text;
-}
