@@ -1,35 +1,35 @@
 /****************************************************************************
-** $Id: qpaintdevice_x11.cpp,v 2.16 1998/07/03 00:09:35 hanord Exp $
+** $Id: qpaintdevice_x11.cpp,v 2.34.2.1 1999/07/01 14:31:38 warwick Exp $
 **
 ** Implementation of QPaintDevice class for X11
 **
 ** Created : 940721
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qpaintdevice.h"
-#include "qpaintdevicedefs.h"
+#include "qpaintdevicemetrics.h"
 #include "qwidget.h"
 #include "qbitmap.h"
 #include "qapplication.h"
-#define	 GC GC_QQQ
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xos.h>
+#include "qt_x11.h"
+
 
 /*!
   \class QPaintDevice qpaintdevice.h
@@ -80,15 +80,14 @@
 // Some global variables - these are initialized by QColor::initialize()
 //
 
-Display *QPaintDevice::dpy = 0;
-Display *QPaintDevice::x_display = 0;
-int	 QPaintDevice::x_screen;
-int	 QPaintDevice::x_depth;
-int	 QPaintDevice::x_cells;
-HANDLE	 QPaintDevice::x_colormap;
-bool	 QPaintDevice::x_defcmap;
-void	*QPaintDevice::x_visual;
-bool	 QPaintDevice::x_defvisual;
+Display *QPaintDevice::x_appdisplay = 0;
+int	 QPaintDevice::x_appscreen;
+int	 QPaintDevice::x_appdepth;
+int	 QPaintDevice::x_appcells;
+HANDLE	 QPaintDevice::x_appcolormap;
+bool	 QPaintDevice::x_appdefcolormap;
+void	*QPaintDevice::x_appvisual;
+bool	 QPaintDevice::x_appdefvisual;
 
 
 /*!
@@ -100,15 +99,15 @@ QPaintDevice::QPaintDevice( uint devflags )
 {
     if ( !qApp ) {				// global constructor
 #if defined(CHECK_STATE)
-	fatal( "QPaintDevice: Must construct a QApplication before a "
+	qFatal( "QPaintDevice: Must construct a QApplication before a "
 	       "QPaintDevice" );
 #endif
 	return;
     }
     devFlags = devflags;
-    if ( dpy == 0 )
-	dpy = qt_xdisplay();
+    painters = 0;
     hd	= 0;
+    x11Data = 0;
 }
 
 /*!
@@ -119,16 +118,79 @@ QPaintDevice::~QPaintDevice()
 {
 #if defined(CHECK_STATE)
     if ( paintingActive() )
-	warning( "QPaintDevice: Cannot destroy paint device that is being "
+	qWarning( "QPaintDevice: Cannot destroy paint device that is being "
 		 "painted" );
 #endif
+    if ( x11Data ) {
+	delete x11Data;
+	x11Data = 0;
+    }	
+}
+
+
+/*
+  \internal
+  Copy X11-specific data (which normally is null).
+*/
+
+void QPaintDevice::copyX11Data( const QPaintDevice *fromDevice )
+{
+    setX11Data( fromDevice ? fromDevice->x11Data : 0 );
+}
+
+
+/*
+  \internal
+  Set the X11-specific data.
+*/
+
+void QPaintDevice::setX11Data( const QPaintDeviceX11Data* d )
+{
+    if ( d ) {
+	if ( !x11Data )
+	    x11Data = new QPaintDeviceX11Data;
+	*x11Data = *d;
+    } else if ( x11Data ) {
+	delete x11Data;
+	x11Data = 0;
+    }
+}
+
+
+/*
+  \internal
+
+  If \a def is FALSE, returns a copy of the x11Data, or 0 if x11Data is 0.
+  If \a def is TRUE, makes a QPaintDeviceX11Data struct filled with the default
+  values.
+  In any case the caller is responsible for deleting the returned struct.
+*/
+
+QPaintDeviceX11Data* QPaintDevice::getX11Data( bool def ) const
+{
+    QPaintDeviceX11Data* res = 0;
+    if ( def ) {
+	res = new QPaintDeviceX11Data;
+	res->x_display = x11AppDisplay();
+	res->x_screen = x11AppScreen();
+	res->x_depth = x11AppDepth();
+	res->x_cells = x11AppCells();
+	res->x_colormap = x11Colormap();
+	res->x_defcolormap = x11AppDefaultColormap();
+	res->x_visual = x11AppVisual();
+	res->x_defvisual = x11AppDefaultVisual();
+    } else if ( x11Data ) {
+	res = new QPaintDeviceX11Data;
+	*res = *x11Data;
+    }
+    return res;
 }
 
 
 /*!
   \fn int QPaintDevice::devType() const
-  Returns the device type identifier: \c PDT_WIDGET, \c PDT_PIXMAP,
-  \c PDT_PRINTER, \c PDT_PICTURE or \c PDT_UNDEF.
+  Returns the device type identifier: \c QInternal::Widget, \c QInternal::Pixmap,
+  \c QInternal::Printer, \c QInternal::Picture or \c QInternal::UndefinedDevice.
 */
 
 /*!
@@ -160,6 +222,51 @@ QPaintDevice::~QPaintDevice()
   \sa handle()
 */
 
+
+/*!
+  Returns the horizontal DPI of the X display (X11 only).
+  <em>Using this function is not portable.</em> See QPaintDeviceMetrics
+  for portable access to related information.
+
+  \sa QPaintDeviceMetrics::logicalDpiX();
+*/
+int QPaintDevice::x11AppDpiX()
+{
+    static int remember = 0;
+    if ( !remember ) {
+	Display *dpy = x11AppDisplay();
+	int scr = x11AppScreen();
+	if ( dpy ) {
+	    remember =
+		(DisplayWidth(dpy,scr) * 254 + DisplayWidthMM(dpy,scr)*5)
+		       / (DisplayWidthMM(dpy,scr)*10);
+	}
+    }
+    return remember;
+}
+
+/*!
+  Returns the vertical DPI of the X display (X11 only).
+  <em>Using this function is not portable.</em> See QPaintDeviceMetrics
+  for portable access to related information.
+
+  \sa QPaintDeviceMetrics::logicalDpiY();
+*/
+int QPaintDevice::x11AppDpiY()
+{
+    static int remember = 0;
+    if ( !remember ) {
+	Display *dpy = x11AppDisplay();
+	int scr = x11AppScreen();
+	if ( dpy )
+	    remember =
+		(DisplayHeight(dpy,scr) * 254 + DisplayHeightMM(dpy,scr)*5)
+		       / (DisplayHeightMM(dpy,scr)*10);
+    }
+    return remember;
+}
+
+
 /*!
   \fn bool QPaintDevice::paintingActive() const
   Returns TRUE if the device is being painted, i.e. someone has called
@@ -178,7 +285,7 @@ QPaintDevice::~QPaintDevice()
 bool QPaintDevice::cmd( int, QPainter *, QPDevCmdParam * )
 {
 #if defined(CHECK_STATE)
-    warning( "QPaintDevice::cmd: Device has no command interface" );
+    qWarning( "QPaintDevice::cmd: Device has no command interface" );
 #endif
     return FALSE;
 }
@@ -192,7 +299,7 @@ bool QPaintDevice::cmd( int, QPainter *, QPDevCmdParam * )
 int QPaintDevice::metric( int ) const
 {
 #if defined(CHECK_STATE)
-    warning( "QPaintDevice::metrics: Device has no metric information" );
+    qWarning( "QPaintDevice::metrics: Device has no metric information" );
 #endif
     return 0;
 }
@@ -291,14 +398,21 @@ static GC cache_mask_gc( Display *dpy, Drawable hd, int mask_no, Pixmap mask )
   The \e rop argument can be one of:
   <ul>
   <li> \c CopyROP:     dst = src.
-  <li> \c OrROP:       dst = dst OR src.
-  <li> \c XorROP:      dst = dst XOR src.
-  <li> \c EraseROP:    dst = (NOT src) AND dst
+  <li> \c OrROP:       dst = src OR dst.
+  <li> \c XorROP:      dst = src XOR dst.
+  <li> \c NotAndROP:   dst = (NOT src) AND dst
   <li> \c NotCopyROP:  dst = NOT src
   <li> \c NotOrROP:    dst = (NOT src) OR dst
   <li> \c NotXorROP:   dst = (NOT src) XOR dst
-  <li> \c NotEraseROP: dst = src AND dst
+  <li> \c AndROP       dst = src AND dst
   <li> \c NotROP:      dst = NOT dst
+  <li> \c ClearROP:    dst = 0
+  <li> \c SetROP:      dst = 1
+  <li> \c NopROP:      dst = dst
+  <li> \c AndNotROP:   dst = src AND (NOT dst)
+  <li> \c OrNotROP:    dst = src OR (NOT dst)
+  <li> \c NandROP:     dst = NOT (src AND dst)
+  <li> \c NorROP:      dst = NOT (src OR dst)
   </ul>
 
   The \e ignoreMask argument (default FALSE) applies where \e src is
@@ -316,7 +430,7 @@ static GC cache_mask_gc( Display *dpy, Drawable hd, int mask_no, Pixmap mask )
 
 void bitBlt( QPaintDevice *dst, int dx, int dy,
 	     const QPaintDevice *src, int sx, int sy, int sw, int sh,
-	     RasterOp rop, bool ignoreMask )
+	     Qt::RasterOp rop, bool ignoreMask )
 {
     if ( !src || !dst ) {
 #if defined(CHECK_NULL)
@@ -334,13 +448,13 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 
     if ( sw <= 0 ) {				// special width
 	if ( sw < 0 )
-	    sw = src->metric( PDM_WIDTH ) - sx;
+	    sw = src->metric( QPaintDeviceMetrics::PdmWidth ) - sx;
 	else
 	    return;
     }
     if ( sh <= 0 ) {				// special height
 	if ( sh < 0 )
-	    sh = src->metric( PDM_HEIGHT ) - sy;
+	    sh = src->metric( QPaintDeviceMetrics::PdmHeight ) - sy;
 	else
 	    return;
     }
@@ -348,29 +462,29 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
     if ( dst->paintingActive() && dst->isExtDev() ) {
 	QPixmap *pm;				// output to picture/printer
 	bool	 tmp_pm = TRUE;
-	if ( ts == PDT_PIXMAP ) {
+	if ( ts == QInternal::Pixmap ) {
 	    pm = (QPixmap*)src;
 	    if ( sx != 0 || sy != 0 ||
 		 sw != pm->width() || sh != pm->height() || ignoreMask ) {
 		QPixmap *tmp = new QPixmap( sw, sh, pm->depth() );
-		bitBlt( tmp, 0, 0, pm, sx, sy, sw, sh, CopyROP, TRUE );
+		bitBlt( tmp, 0, 0, pm, sx, sy, sw, sh, Qt::CopyROP, TRUE );
 		if ( pm->mask() && !ignoreMask ) {
 		    QBitmap mask( sw, sh );
 		    bitBlt( &mask, 0, 0, pm->mask(), sx, sy, sw, sh,
-			    CopyROP, TRUE );
+			    Qt::CopyROP, TRUE );
 		    tmp->setMask( mask );
 		}
 		pm = tmp;
 	    } else {
 		tmp_pm = FALSE;
 	    }
-	} else if ( ts == PDT_WIDGET ) {	// bitBlt to temp pixmap
+	} else if ( ts == QInternal::Widget ) {// bitBlt to temp pixmap
 	    pm = new QPixmap( sw, sh );
 	    CHECK_PTR( pm );
 	    bitBlt( pm, 0, 0, src, sx, sy, sw, sh );
 	} else {
 #if defined(CHECK_RANGE)
-	    warning( "bitBlt: Cannot bitBlt from device" );
+	    qWarning( "bitBlt: Cannot bitBlt from device" );
 #endif
 	    return;
 	}
@@ -378,31 +492,34 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 	QPoint p(dx,dy);
 	param[0].point	= &p;
 	param[1].pixmap = pm;
-	dst->cmd( PDC_DRAWPIXMAP, 0, param );
+	dst->cmd( QPaintDevice::PdcDrawPixmap, 0, param );
 	if ( tmp_pm )
 	    delete pm;
 	return;
     }
 
-    if ( !(ts <= PDT_PIXMAP && td <= PDT_PIXMAP) ) {
+    if ( !(ts <= QInternal::Pixmap && td <= QInternal::Pixmap) ) {
 #if defined(CHECK_RANGE)
-	warning( "bitBlt: Cannot bitBlt to or from device" );
+	qWarning( "bitBlt: Cannot bitBlt to or from device" );
 #endif
 	return;
     }
-    static short ropCodes[] =			// ROP translation table
-	{ GXcopy, GXor, GXxor, GXandInverted,
-	  GXcopyInverted, GXorInverted, GXequiv, GXand, GXinvert };
-    if ( rop > NotROP ) {
+    static short ropCodes[] = {			// ROP translation table
+	GXcopy, GXor, GXxor, GXandInverted,
+	GXcopyInverted, GXorInverted, GXequiv, GXand,
+	GXinvert, GXclear, GXset, GXnoop,
+	GXandReverse, GXorReverse, GXnand, GXnor
+    };
+    if ( rop > Qt::LastROP ) {
 #if defined(CHECK_RANGE)
-	warning( "bitBlt: Invalid ROP code" );
+	qWarning( "bitBlt: Invalid ROP code" );
 #endif
 	return;
     }
 
     if ( dst->handle() == 0 ) {
 #if defined(CHECK_NULL)
-	warning( "bitBlt: Cannot bitBlt to device" );
+	qWarning( "bitBlt: Cannot bitBlt to device" );
 #endif
 	return;
     }
@@ -414,7 +531,7 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
     QPixmap *src_pm;
     QBitmap *mask;
 
-    if ( ts == PDT_PIXMAP ) {
+    if ( ts == QInternal::Pixmap ) {
 	src_pm = (QPixmap*)src;
 	mono_src = src_pm->depth() == 1;
 	mask = ignoreMask ? 0 : src_pm->data->mask;
@@ -422,21 +539,21 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 	src_pm = 0;
 	mono_src = FALSE;
 	mask = 0;
-	include_inferiors = ((QWidget*)src)->testWFlags(WPaintUnclipped);
-	graphics_exposure = td == PDT_WIDGET;
+	include_inferiors = ((QWidget*)src)->testWFlags(Qt::WPaintUnclipped);
+	graphics_exposure = td == QInternal::Widget;
     }
-    if ( td == PDT_PIXMAP ) {
+    if ( td == QInternal::Pixmap ) {
 	mono_dst = ((QPixmap*)dst)->depth() == 1;
 	((QPixmap*)dst)->detach();		// changes shared pixmap
     } else {
 	mono_dst = FALSE;
 	include_inferiors = include_inferiors ||
-	    ((QWidget*)dst)->testWFlags(WPaintUnclipped);
+	    ((QWidget*)dst)->testWFlags(Qt::WPaintUnclipped);
     }
 
     if ( mono_dst && !mono_src ) {	// dest is 1-bit pixmap, source is not
 #if defined(CHECK_RANGE)
-	warning( "bitBlt: Incompatible destination pixmap" );
+	qWarning( "bitBlt: Incompatible destination pixmap" );
 #endif
 	return;
     }
@@ -468,7 +585,7 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 	    }
 	}
 	XSetClipOrigin( dpy, gc, dx-sx, dy-sy );
-	if ( rop != CopyROP )			// use non-default ROP code
+	if ( rop != Qt::CopyROP )		// use non-default ROP code
 	    XSetFunction( dpy, gc, ropCodes[rop] );
 	if ( include_inferiors ) {
 	    XSetSubwindowMode( dpy, gc, IncludeInferiors );
@@ -482,21 +599,21 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 
 	if ( temp_gc )				// delete temporary GC
 	    XFreeGC( dpy, gc );
-	else if ( rop != CopyROP )		// restore ROP
+	else if ( rop != Qt::CopyROP )		// restore ROP
 	    XSetFunction( dpy, gc, GXcopy );
 	return;
     }
 
     gc = qt_xget_temp_gc( mono_dst );		// get a reusable GC
 
-    if ( rop != CopyROP )			// use non-default ROP code
+    if ( rop != Qt::CopyROP )			// use non-default ROP code
 	XSetFunction( dpy, gc, ropCodes[rop] );
 
     if ( mono_src ) {				// src is bitmap
 	XGCValues gcvals;
 	ulong	  valmask = GCBackground | GCForeground | GCFillStyle |
 			    GCStipple | GCTileStipXOrigin | GCTileStipYOrigin;
-	if ( td == PDT_WIDGET ) {		// set GC colors
+	if ( td == QInternal::Widget ) {	// set GC colors
 	    QWidget *w = (QWidget *)dst;
 	    gcvals.background = w->backgroundColor().pixel();
 	    gcvals.foreground = w->foregroundColor().pixel();
@@ -508,8 +625,8 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 	    gcvals.background = 0;
 	    gcvals.foreground = 1;
 	} else {
-	    gcvals.background = white.pixel();
-	    gcvals.foreground = black.pixel();
+	    gcvals.background = Qt::white.pixel();
+	    gcvals.foreground = Qt::black.pixel();
 	}
 
 	gcvals.fill_style  = FillOpaqueStippled;
@@ -563,7 +680,7 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 	    XSetGraphicsExposures( dpy, gc, FALSE );
     }
 
-    if ( rop != CopyROP )			// restore ROP
+    if ( rop != Qt::CopyROP )			// restore ROP
 	XSetFunction( dpy, gc, GXcopy );
 }
 
@@ -576,3 +693,4 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 
   \relates QPaintDevice
 */
+
