@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qpainter.h,v 2.27 1998/07/03 00:09:36 hanord Exp $
+** $Id: qpainter.h,v 2.48.2.1 1999/07/14 19:13:18 aavit Exp $
 **
 ** Definition of QPainter class
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -38,20 +40,10 @@
 #endif // QT_H
 
 
-enum BGMode					// background mode
-    { TransparentMode, OpaqueMode };
-
-enum PaintUnit					// paint unit
-    { PixelUnit, LoMetricUnit, HiMetricUnit, LoEnglishUnit, HiEnglishUnit,
-      TwipsUnit };
+class QTextCodec;
 
 
-#if defined(_WS_WIN_)
-struct QWinFont;
-#endif
-
-
-class QPainter					// painter class
+class Q_EXPORT QPainter : public Qt
 {
 public:
     QPainter();
@@ -119,10 +111,15 @@ public:
     const QWMatrix &worldMatrix() const;	// get/set world xform matrix
     void	setWorldMatrix( const QWMatrix &, bool combine=FALSE );
 
-    void	translate( float dx, float dy );
-    void	scale( float sx, float sy );
-    void	shear( float sh, float sv );
-    void	rotate( float a );
+    void	saveWorldMatrix();
+    void	restoreWorldMatrix();
+
+
+    void	translate( double dx, double dy );
+    void	scale( double sx, double sy );
+    void	shear( double sh, double sv );
+    void	rotate( double a );
+
     void	resetXForm();
 
     QPoint	xForm( const QPoint & ) const;	// map virtual -> device
@@ -204,28 +201,34 @@ public:
 
   // Text drawing functions
 
-    void	drawText( int x, int y, const char *str, int len = -1 );
-    void	drawText( const QPoint &, const char *str, int len = -1 );
+    void	drawText( int x, int y, const QString &, int len = -1 );
+    void	drawText( const QPoint &, const QString &, int len = -1 );
     void	drawText( int x, int y, int w, int h, int flags,
-			  const char *str, int len = -1, QRect *br=0,
+			  const QString&, int len = -1, QRect *br=0,
 			  char **internal=0 );
     void	drawText( const QRect &, int flags,
-			  const char *str, int len = -1, QRect *br=0,
+			  const QString&, int len = -1, QRect *br=0,
 			  char **internal=0 );
 
+    //#####    void	drawText( const QPoint &, const QString &, int flags, int rotation = 0);
+
   // Text drawing functions
 
     QRect	boundingRect( int x, int y, int w, int h, int flags,
-			      const char *str, int len = -1, char **intern=0 );
+			      const QString&, int len = -1, char **intern=0 );
     QRect	boundingRect( const QRect &, int flags,
-			      const char *str, int len = -1, char **intern=0 );
+			      const QString&, int len = -1, char **intern=0 );
 
     int		tabStops() const;
     void	setTabStops( int );
     int	       *tabArray() const;
     void	setTabArray( int * );
 
+#if defined(_WS_WIN_)
+    HDC		handle() const;
+#elif defined(_WS_X11_)
     HANDLE	handle() const;
+#endif
 
     static void initialize();
     static void cleanup();
@@ -248,12 +251,13 @@ private:
     enum { IsActive=0x01, ExtDev=0x02, IsStartingUp=0x04, NoCache=0x08,
 	   VxF=0x10, WxF=0x20, ClipOn=0x40, SafePolygon=0x80, MonoDev=0x100,
 	   DirtyFont=0x200, DirtyPen=0x400, DirtyBrush=0x800,
-	   RGBColor=0x1000, FontMet=0x2000, FontInf=0x4000, CtorBegin=0x8000 };
-    ushort	flags;
-    bool	testf( ushort b ) const { return (flags&b)!=0; }
-    void	setf( ushort b )	{ flags |= b; }
-    void	setf( ushort b, bool v );
-    void	clearf( ushort b )	{ flags &= (ushort)(~b); }
+	   RGBColor=0x1000, FontMet=0x2000, FontInf=0x4000, CtorBegin=0x8000,
+           UsePrivateCx = 0x10000, VolatileDC = 0x20000 };
+    uint	flags;
+    bool	testf( uint b ) const { return (flags&b)!=0; }
+    void	setf( uint b )	{ flags |= b; }
+    void	setf( uint b, bool v );
+    void	clearf( uint b )	{ flags &= (uint)(~b); }
     void	fix_neg_rect( int *x, int *y, int *w, int *h );
 
     QPaintDevice *pdev;
@@ -269,35 +273,53 @@ private:
     int		tabstops;
     int	       *tabarray;
     int		tabarraylen;
+
+    // Transformations
     QCOORD	wx, wy, ww, wh;
     QCOORD	vx, vy, vw, vh;
     QWMatrix	wxmat;
-    int		wm11, wm12, wm21, wm22, wdx, wdy;
-    int		im11, im12, im21, im22, idx, idy;
+
+    // Cached composition (and inverse) of transformations
+    QWMatrix	xmat;
+    QWMatrix	ixmat;
+
+
+
+    double	m11() const { return xmat.m11(); }
+    double      m12() const { return xmat.m12(); }
+    double      m21() const { return xmat.m21(); }
+    double      m22() const { return xmat.m22(); }
+    double      dx() const { return xmat.dx(); }
+    double      dy() const { return xmat.dy(); }
+    double	im11() const { return ixmat.m11(); }
+    double      im12() const { return ixmat.m12(); }
+    double      im21() const { return ixmat.m21(); }
+    double      im22() const { return ixmat.m22(); }
+    double      idx() const { return ixmat.dx(); }
+    double      idy() const { return ixmat.dy(); }
+
     int		txop;
     bool	txinv;
     void       *penRef;				// pen cache ref
     void       *brushRef;			// brush cache ref
     void       *ps_stack;
+    void       *wm_stack;
     void	killPStack();
 
 protected:
 #if defined(_WS_WIN_)
     HDC		hdc;				// device context
-    HANDLE	hpen;				// current pen
-    HANDLE	hbrush;				// current brush
-    HANDLE	hbrushbm;			// current brush bitmap
-    HANDLE	holdpal;
-    uint	pixmapBrush	: 1;
-    uint	nocolBrush	: 1;
-    QWinFont   *winFont;
+    HPEN	hpen;				// current pen
+    HBRUSH	hbrush;				// current brush
+    HBITMAP	hbrushbm;			// current brush bitmap
+    HFONT	hfont;				// current font
+    HPALETTE	holdpal;
+    void       *textmet;			// text metrics
+    uint	pixmapBrush : 1;
+    uint	nocolBrush  : 1;
+    uint	killFont    : 1;
     void       *textMetric();
     void	nativeXForm( bool );
-#elif defined(_WS_PM_)
-    HPS		hps;				// presentation space
-    int		dh;				// device height
-    long	lctrl;				// outline/fill control
-    void	updateCtrl();			// update lctrl
 #elif defined(_WS_X11_)
     Display    *dpy;				// current display
     WId		hd;				// handle to drawable
@@ -308,13 +330,15 @@ protected:
     friend class QFontMetrics;
     friend class QFontInfo;
     friend void qt_format_text( const QFontMetrics& fm, int x, int y, int w, int h,
-		     int tf, const char *str, int len, QRect *brect,
+		     int tf, const QString& str, int len, QRect *brect,
 		     int tabstops, int* tabarray, int tabarraylen,
 		     char **internal, QPainter* painter );
 
 private:	// Disabled copy constructor and operator=
+#if defined(Q_DISABLE_COPY)
     QPainter( const QPainter & );
     QPainter &operator=( const QPainter & );
+#endif
 };
 
 
@@ -369,12 +393,12 @@ inline const QColor &QPainter::backgroundColor() const
     return bg_col;
 }
 
-inline BGMode QPainter::backgroundMode() const
+inline Qt::BGMode QPainter::backgroundMode() const
 {
     return (BGMode)bg_mode;
 }
 
-inline RasterOp QPainter::rasterOp() const
+inline Qt::RasterOp QPainter::rasterOp() const
 {
     return (RasterOp)rop;
 }
@@ -414,17 +438,17 @@ inline int *QPainter::tabArray() const
     return tabarray;
 }
 
+#if defined(_WS_WIN_)
+inline HDC QPainter::handle() const
+{
+    return hdc;
+}
+#elif defined(_WS_X11_)
 inline HANDLE QPainter::handle() const
 {
-#if defined(_WS_WIN_)
-    return hdc;
-#elif defined(_WS_PM_)
-    return hps;
-#elif defined(_WS_X11_)
     return hd;
-#endif
 }
-
+#endif
 
 inline void QPainter::setBrushOrigin( const QPoint &p )
 {
@@ -544,23 +568,32 @@ inline void QPainter::eraseRect( const QRect &r )
     fillRect( r.x(), r.y(), r.width(), r.height(), backgroundColor() );
 }
 
-inline void QPainter::drawText( const QPoint &p, const char *s, int len )
+inline void QPainter::drawText( const QPoint &p, const QString &s, int len )
 {
     drawText( p.x(), p.y(), s, len );
 }
 
 inline void QPainter::drawText( const QRect &r, int tf,
-				const char *str, int len, QRect *br, char **i )
+				const QString& str, int len, QRect *br, char **i )
 {
     drawText( r.x(), r.y(), r.width(), r.height(), tf, str, len, br, i );
 }
 
 inline QRect QPainter::boundingRect( const QRect &r, int tf,
-				     const char *str, int len, char **i )
+				     const QString& str, int len, char **i )
 {
     return boundingRect( r.x(), r.y(), r.width(), r.height(), tf, str, len,
 			 i );
 }
 
+#if defined(_WS_WIN_)
+inline void *QPainter::textMetric()
+{
+    if ( testf(DirtyFont) )
+	updateFont();
+    return textmet;
+}
+#endif
+
 
 #endif // QPAINTER_H
