@@ -1,37 +1,38 @@
 /****************************************************************************
-** $Id: qpen.h,v 2.2 1997/08/06 15:05:40 hanord Exp $
+** $Id: qpen.h,v 2.9 1999/04/28 09:57:01 aavit Exp $
 **
 ** Definition of QPen class
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #ifndef QPEN_H
 #define QPEN_H
 
+#ifndef QT_H
 #include "qcolor.h"
 #include "qshared.h"
+#endif // QT_H
 
 
-enum PenStyle { NoPen, SolidLine, DashLine,	// pen style
-		DotLine, DashDotLine, DashDotDotLine };
-
-
-class QPen
+class Q_EXPORT QPen: public Qt
 {
 friend class QPainter;
 public:
@@ -69,8 +70,8 @@ private:
   QPen stream functions
  *****************************************************************************/
 
-QDataStream &operator<<( QDataStream &, const QPen & );
-QDataStream &operator>>( QDataStream &, QPen & );
+Q_EXPORT QDataStream &operator<<( QDataStream &, const QPen & );
+Q_EXPORT QDataStream &operator>>( QDataStream &, QPen & );
 
 
 #endif // QPEN_H
