@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qpicture.cpp,v 2.7.2.1 1998/11/02 19:29:52 hanord Exp $
+** $Id: qpicture.cpp,v 2.23 1999/06/23 03:19:04 warwick Exp $
 **
 ** Implementation of QPicture class
 **
@@ -7,27 +7,30 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qpicture.h"
-#include "qpaintdevicedefs.h"
 #include "qpainter.h"
 #include "qpixmap.h"
 #include "qimage.h"
 #include "qfile.h"
 #include "qdatastream.h"
+#include "qpaintdevicemetrics.h"
+
 
 /*!
   \class QPicture qpicture.h
@@ -67,23 +70,48 @@
     p.drawPicture( pic );			// draw the picture
     p.end();					// painting done
   \endcode
+
 */
 
 
 static const char  *mfhdr_tag = "QPIC";		// header tag
-static const UINT16 mfhdr_maj = 1;		// major version #
+static const UINT16 mfhdr_maj = 2;		// major version #
 static const UINT16 mfhdr_min = 0;		// minor version #
 
 
 /*!
-  \fn QPicture::QPicture()
   Constructs an empty picture.
+
+  The \a formatVersion parameter may be used to create a QPicture that
+  can be read by programs compiled with earlier versions of
+  Qt. Currently supported is \a formatVersion == 1, which will make a
+  QPicture which is stored in a format that is binary compatible with
+  Qt 1.x programs.
+
+  Reading of Qt 1.x format pictures is supported and needs no special
+  coding; the format is automatically detected.
 */
 
+QPicture::QPicture( int formatVer )
+    : QPaintDevice( QInternal::Picture | QInternal::ExternalDevice )	
+    // set device type
+{
+    if ( formatVer == 1 ) {
+	formatMajor = formatVer;
+	formatMinor = 0;
+	formatOk = FALSE;
+    }
+    else {
+	resetFormat();
+    }
+}
+
 /*!
-  \fn QPicture::~QPicture()
   Destroys the picture.
 */
+QPicture::~QPicture()
+{
+}
 
 
 /*!
@@ -98,7 +126,7 @@ static const UINT16 mfhdr_min = 0;		// minor version #
 */
 
 /*!
-  \fn const char *QPicture::data() const
+  \fn const char* QPicture::data() const
   Returns a pointer to the picture data.  The returned pointer is null
   if the picture contains no data.
   \sa size(), isNull()
@@ -110,12 +138,12 @@ static const UINT16 mfhdr_min = 0;		// minor version #
   \sa data(), size()
 */
 
-void QPicture::setData( const char *data, uint size )
+void QPicture::setData( const char* data, uint size )
 {
     QByteArray a( size );
     memcpy( a.data(), data, size );
     pictb.setBuffer( a );			// set byte array in buffer
-    formatOk = FALSE;				// we'll have to check
+    resetFormat();				// we'll have to check
 }
 
 
@@ -126,7 +154,7 @@ void QPicture::setData( const char *data, uint size )
   \sa save()
 */
 
-bool QPicture::load( const char *fileName )
+bool QPicture::load( const QString &fileName )
 {
     QByteArray a;
     QFile f( fileName );
@@ -136,7 +164,7 @@ bool QPicture::load( const char *fileName )
     f.readBlock( a.data(), (uint)f.size() );	// read file into byte array
     f.close();
     pictb.setBuffer( a );			// set byte array in buffer
-    formatOk = FALSE;				// we'll have to check
+    resetFormat();				// we'll have to check
     return TRUE;
 }
 
@@ -147,7 +175,7 @@ bool QPicture::load( const char *fileName )
   \sa load()
 */
 
-bool QPicture::save( const char *fileName )
+bool QPicture::save( const QString &fileName )
 {
     QFile f( fileName );
     if ( !f.open( IO_WriteOnly ) )
@@ -179,7 +207,7 @@ bool QPicture::play( QPainter *painter )
 	s.readRawBytes( mf_id, 4 );		// read actual tag
 	if ( memcmp(mf_id, mfhdr_tag, 4) != 0 ) { // wrong header id
 #if defined(CHECK_RANGE)
-	    warning( "QPicture::play: Incorrect header" );
+	    qWarning( "QPicture::play: Incorrect header" );
 #endif
 	    pictb.close();
 	    return FALSE;
@@ -190,10 +218,10 @@ bool QPicture::play( QPainter *painter )
 	UINT16 cs,ccs;
 	QByteArray buf = pictb.buffer();	// pointer to data
 	s >> cs;				// read checksum
-	ccs = qchecksum( buf.data() + data_start, buf.size() - data_start );
+	ccs = qChecksum( buf.data() + data_start, buf.size() - data_start );
 	if ( ccs != cs ) {
 #if defined(CHECK_STATE)
-	    warning( "QPicture::play: Invalid checksum %x, %x expected",
+	    qWarning( "QPicture::play: Invalid checksum %x, %x expected",
 		     ccs, cs );
 #endif
 	    pictb.close();
@@ -204,28 +232,34 @@ bool QPicture::play( QPainter *painter )
 	s >> major >> minor;			// read version number
 	if ( major > mfhdr_maj ) {		// new, incompatible version
 #if defined(CHECK_RANGE)
-	    warning( "QPicture::play: Incompatible version %d.%d",
+	    qWarning( "QPicture::play: Incompatible version %d.%d",
 		     major, minor);
 #endif
 	    pictb.close();
 	    return FALSE;
 	}
 	formatOk = TRUE;			// picture seems to be ok
+	formatMajor = major;
+	formatMinor = minor;
     } else {
 	s.device()->at( 10 );			// go directly to the data
     }
 
+    if ( formatMajor == 1 ) {
+	s.setVersion( 1 );			// Qt 1.x compatibility
+    }
+
     UINT8  c, clen;
     UINT32 nrecords;
     s >> c >> clen;
-    if ( c == PDC_BEGIN ) {
+    if ( c == PdcBegin ) {
 	s >> nrecords;
 	if ( !exec( painter, s, nrecords ) )
 	    c = 0;
     }
-    if ( c !=  PDC_BEGIN ) {
+    if ( c !=  PdcBegin ) {
 #if defined(CHECK_RANGE)
-	warning( "QPicture::play: Format error" );
+	qWarning( "QPicture::play: Format error" );
 #endif
 	pictb.close();
 	return FALSE;
@@ -252,7 +286,8 @@ bool QPicture::exec( QPainter *painter, QDataStream &s, int nrecords )
     INT16	i_16, i1_16, i2_16;		// parameters...
     INT8	i_8;
     UINT32	ul;
-    char       *str;
+    QCString	str1;
+    QString	str;
     QPoint	p, p1, p2;
     QRect	r;
     QPointArray a;
@@ -274,134 +309,140 @@ bool QPicture::exec( QPainter *painter, QDataStream &s, int nrecords )
 	strm_pos = s.device()->at();
 #endif
 	switch ( c ) {				// exec cmd
-	    case PDC_NOP:
+	    case PdcNOP:
 		break;
-	    case PDC_DRAWPOINT:
+	    case PdcDrawPoint:
 		s >> p;
 		painter->drawPoint( p );
 		break;
-	    case PDC_MOVETO:
+	    case PdcMoveTo:
 		s >> p;
 		painter->moveTo( p );
 		break;
-	    case PDC_LINETO:
+	    case PdcLineTo:
 		s >> p;
 		painter->lineTo( p );
 		break;
-	    case PDC_DRAWLINE:
+	    case PdcDrawLine:
 		s >> p1 >> p2;
 		painter->drawLine( p1, p2 );
 		break;
-	    case PDC_DRAWRECT:
+	    case PdcDrawRect:
 		s >> r;
 		painter->drawRect( r );
 		break;
-	    case PDC_DRAWROUNDRECT:
+	    case PdcDrawRoundRect:
 		s >> r >> i1_16 >> i2_16;
 		painter->drawRoundRect( r, i1_16, i2_16 );
 		break;
-	    case PDC_DRAWELLIPSE:
+	    case PdcDrawEllipse:
 		s >> r;
 		painter->drawEllipse( r );
 		break;
-	    case PDC_DRAWARC:
+	    case PdcDrawArc:
 		s >> r >> i1_16 >> i2_16;
 		painter->drawArc( r, i1_16, i2_16 );
 		break;
-	    case PDC_DRAWPIE:
+	    case PdcDrawPie:
 		s >> r >> i1_16 >> i2_16;
 		painter->drawPie( r, i1_16, i2_16 );
 		break;
-	    case PDC_DRAWCHORD:
+	    case PdcDrawChord:
 		s >> r >> i1_16 >> i2_16;
 		painter->drawChord( r, i1_16, i2_16 );
 		break;
-	    case PDC_DRAWLINESEGS:
+	    case PdcDrawLineSegments:
 		s >> a;
 		painter->drawLineSegments( a );
 		break;
-	    case PDC_DRAWPOLYLINE:
+	    case PdcDrawPolyline:
 		s >> a;
 		painter->drawPolyline( a );
 		break;
-	    case PDC_DRAWPOLYGON:
+	    case PdcDrawPolygon:
 		s >> a >> i_8;
 		painter->drawPolygon( a, i_8 );
 		break;
-	    case PDC_DRAWQUADBEZIER:
+	    case PdcDrawQuadBezier:
 		s >> a;
 		painter->drawQuadBezier( a );
 		break;
-	    case PDC_DRAWTEXT:
+	    case PdcDrawText:
+		s >> p >> str1;
+		painter->drawText( p, str1 );
+		break;
+	    case PdcDrawTextFormatted:
+		s >> r >> i_16 >> str1;
+		painter->drawText( r, i_16, str1 );
+		break;
+	    case PdcDrawText2:
 		s >> p >> str;
 		painter->drawText( p, str );
-		delete str;
 		break;
-	    case PDC_DRAWTEXTFRMT:
+	    case PdcDrawText2Formatted:
 		s >> r >> i_16 >> str;
 		painter->drawText( r, i_16, str );
-		delete str;
 		break;
-	    case PDC_DRAWPIXMAP: {
+	    case PdcDrawPixmap: {
 		QPixmap pixmap;
 		s >> p >> pixmap;
 		painter->drawPixmap( p, pixmap );
 		}
 		break;
-	    case PDC_DRAWIMAGE: {
+	    case PdcDrawImage: {
 		QImage image;
 		s >> p >> image;
 		painter->drawImage( p, image );
 		}
 		break;
-	    case PDC_BEGIN:
+	    case PdcBegin:
 		s >> ul;			// number of records
 		if ( !exec( painter, s, ul ) )
 		    return FALSE;
 		break;
-	    case PDC_END:
+	    case PdcEnd:
 		if ( nrecords == 0 )
 		    return TRUE;
 		break;
-	    case PDC_SAVE:
+	    case PdcSave:
 		painter->save();
 		break;
-	    case PDC_RESTORE:
+	    case PdcRestore:
 		painter->restore();
 		break;
-	    case PDC_SETBKCOLOR:
+	    case PdcSetBkColor:
 		s >> color;
 		painter->setBackgroundColor( color );
 		break;
-	    case PDC_SETBKMODE:
+	    case PdcSetBkMode:
 		s >> i_8;
-		painter->setBackgroundMode( (BGMode)i_8 );
+		painter->setBackgroundMode( (Qt::BGMode)i_8 );
 		break;
-	    case PDC_SETROP:
+	    case PdcSetROP:
 		s >> i_8;
-		painter->setRasterOp( (RasterOp)i_8 );
+		painter->setRasterOp( (Qt::RasterOp)i_8 );
 		break;
-	    case PDC_SETBRUSHORIGIN:
+	    case PdcSetBrushOrigin:
 		s >> p;
 		painter->setBrushOrigin( p );
 		break;
-	    case PDC_SETFONT:
+	    case PdcSetFont:
 		s >> font;
 		painter->setFont( font );
 		break;
-	    case PDC_SETPEN:
+	    case PdcSetPen:
 		s >> pen;
 		painter->setPen( pen );
 		break;
-	    case PDC_SETBRUSH:
+	    case PdcSetBrush:
 		s >> brush;
 		painter->setBrush( brush );
 		break;
-	    case PDC_SETTABSTOPS:
+	    case PdcSetTabStops:
 		s >> i_16;
 		painter->setTabStops( i_16 );
 		break;
-	    case PDC_SETTABARRAY:
+	    case PdcSetTabArray:
 		s >> i_16;
 		if ( i_16 == 0 ) {
 		    painter->setTabArray( 0 );
@@ -416,37 +457,43 @@ bool QPicture::exec( QPainter *painter, QDataStream &s, int nrecords )
 		    delete [] ta;
 		}
 		break;
-	    case PDC_SETVXFORM:
+	    case PdcSetVXform:
 		s >> i_8;
 		painter->setViewXForm( i_8 );
 		break;
-	    case PDC_SETWINDOW:
+	    case PdcSetWindow:
 		s >> r;
 		painter->setWindow( r );
 		break;
-	    case PDC_SETVIEWPORT:
+	    case PdcSetViewport:
 		s >> r;
 		painter->setViewport( r );
 		break;
-	    case PDC_SETWXFORM:
+	    case PdcSetWXform:
 		s >> i_8;
 		painter->setWorldXForm( i_8 );
 		break;
-	    case PDC_SETWMATRIX:
+	    case PdcSetWMatrix:
 		s >> matrix >> i_8;
 		painter->setWorldMatrix( matrix, i_8 );
 		break;
-	    case PDC_SETCLIP:
+	    case PdcSaveWMatrix:
+		painter->saveWorldMatrix();
+		break;
+	    case PdcRestoreWMatrix:
+		painter->restoreWorldMatrix();
+		break;
+	    case PdcSetClip:
 		s >> i_8;
 		painter->setClipping( i_8 );
 		break;
-	    case PDC_SETCLIPRGN:
+	    case PdcSetClipRegion:
 		s >> rgn;
 		painter->setClipRegion( rgn );
 		break;
 	    default:
 #if defined(CHECK_RANGE)
-		warning( "QPicture::play: Invalid command %d", c );
+		qWarning( "QPicture::play: Invalid command %d", c );
 #endif
 		if ( len )			// skip unknown command
 		    s.device()->at( s.device()->at()+len );
@@ -468,18 +515,20 @@ bool QPicture::cmd( int c, QPainter *, QPDevCmdParam *p )
 {
     QDataStream s;
     s.setDevice( &pictb );
-    if ( c ==  PDC_BEGIN ) {			// begin; write header
+    if ( formatMajor == 1 )
+	s.setVersion( 1 );
+    if ( c ==  PdcBegin ) {			// begin; write header
 	QByteArray empty( 0 );
 	pictb.setBuffer( empty );		// reset byte array in buffer
 	pictb.open( IO_WriteOnly );
 	s.writeRawBytes( mfhdr_tag, 4 );
-	s << (UINT16)0 << mfhdr_maj << mfhdr_min;
+	s << (UINT16)0 << (Q_UINT16)formatMajor << (Q_UINT16)formatMinor;
 	s << (UINT8)c << (UINT8)sizeof(INT32);
 	trecs = 0;
 	s << (UINT32)trecs;			// total number of records
 	formatOk = FALSE;
 	return TRUE;
-    } else if ( c == PDC_END ) {		// end; calc checksum and close
+    } else if ( c == PdcEnd ) {		// end; calc checksum and close
 	trecs++;
 	s << (UINT8)c << (UINT8)0;
 	QByteArray buf = pictb.buffer();
@@ -490,7 +539,7 @@ bool QPicture::cmd( int c, QPainter *, QPDevCmdParam *p )
 	pictb.at( nrecs_start );
 	s << (UINT32)trecs;			// write number of records
 	pictb.at( cs_start );
-	UINT16 cs = (UINT16)qchecksum( buf.data()+data_start, pos-data_start );
+	UINT16 cs = (UINT16)qChecksum( buf.data()+data_start, pos-data_start );
 	s << cs;				// write checksum
 	pictb.close();
 	return TRUE;
@@ -500,70 +549,86 @@ bool QPicture::cmd( int c, QPainter *, QPDevCmdParam *p )
     s << (UINT8)0;				// write dummy length info
     int pos = (int)pictb.at();			// save position
     switch ( c ) {
-	case PDC_DRAWPOINT:
-	case PDC_MOVETO:
-	case PDC_LINETO:
-	case PDC_SETBRUSHORIGIN:
+	case PdcDrawPoint:
+	case PdcMoveTo:
+	case PdcLineTo:
+	case PdcSetBrushOrigin:
 	    s << *p[0].point;
 	    break;
-	case PDC_DRAWLINE:
+	case PdcDrawLine:
 	    s << *p[0].point << *p[1].point;
 	    break;
-	case PDC_DRAWRECT:
-	case PDC_DRAWELLIPSE:
+	case PdcDrawRect:
+	case PdcDrawEllipse:
 	    s << *p[0].rect;
 	    break;
-	case PDC_DRAWROUNDRECT:
-	case PDC_DRAWARC:
-	case PDC_DRAWPIE:
-	case PDC_DRAWCHORD:
+	case PdcDrawRoundRect:
+	case PdcDrawArc:
+	case PdcDrawPie:
+	case PdcDrawChord:
 	    s << *p[0].rect << (INT16)p[1].ival << (INT16)p[2].ival;
 	    break;
-	case PDC_DRAWLINESEGS:
-	case PDC_DRAWPOLYLINE:
-	case PDC_DRAWQUADBEZIER:
+	case PdcDrawLineSegments:
+	case PdcDrawPolyline:
+	case PdcDrawQuadBezier:
 	    s << *p[0].ptarr;
 	    break;
-	case PDC_DRAWPOLYGON:
+	case PdcDrawPolygon:
 	    s << *p[0].ptarr << (INT8)p[1].ival;
 	    break;
-	case PDC_DRAWTEXT:
-	    s << *p[0].point << p[1].str;
+	case PdcDrawText2:
+	    if ( formatMajor == 1 ) {
+		pictb.at( pos - 2 );
+		s << (Q_UINT8)PdcDrawText << (Q_UINT8)0;
+		QCString str1( (*p[1].str).latin1() );
+		s << *p[0].point << str1;
+	    }
+	    else {
+		s << *p[0].point << *p[1].str;
+	    }
 	    break;
-	case PDC_DRAWTEXTFRMT:
-	    s << *p[0].rect << (INT16)p[1].ival << p[2].str;
+	case PdcDrawText2Formatted:
+	    if ( formatMajor == 1 ) {
+		pictb.at( pos - 2 );
+		s << (Q_UINT8)PdcDrawTextFormatted << (Q_UINT8)0;
+		QCString str1( (*p[2].str).latin1() );
+		s << *p[0].rect << (INT16)p[1].ival << str1;
+	    }
+	    else {
+		s << *p[0].rect << (INT16)p[1].ival << *p[2].str;
+	    }
 	    break;
-	case PDC_DRAWPIXMAP:
+	case PdcDrawPixmap:
 	    s << *p[0].point;
 	    s << *p[1].pixmap;
 	    break;
-	case PDC_DRAWIMAGE:
+	case PdcDrawImage:
 	    s << *p[0].point;
 	    s << *p[1].image;
 	    break;
-	case PDC_SAVE:
-	case PDC_RESTORE:
+	case PdcSave:
+	case PdcRestore:
 	    break;
-	case PDC_SETBKCOLOR:
+	case PdcSetBkColor:
 	    s << *p[0].color;
 	    break;
-	case PDC_SETBKMODE:
-	case PDC_SETROP:
+	case PdcSetBkMode:
+	case PdcSetROP:
 	    s << (INT8)p[0].ival;
 	    break;
-	case PDC_SETFONT:
+	case PdcSetFont:
 	    s << *p[0].font;
 	    break;
-	case PDC_SETPEN:
+	case PdcSetPen:
 	    s << *p[0].pen;
 	    break;
-	case PDC_SETBRUSH:
+	case PdcSetBrush:
 	    s << *p[0].brush;
 	    break;
-	case PDC_SETTABSTOPS:
+	case PdcSetTabStops:
 	    s << (INT16)p[0].ival;
 	    break;
-	case PDC_SETTABARRAY:
+	case PdcSetTabArray:
 	    s << (INT16)p[0].ival;
 	    if ( p[0].ival ) {
 		int *ta = p[1].ivec;
@@ -571,25 +636,25 @@ bool QPicture::cmd( int c, QPainter *, QPDevCmdParam *p )
 		    s << (INT16)ta[i];
 	    }
 	    break;
-	case PDC_SETUNIT:
-	case PDC_SETVXFORM:
-	case PDC_SETWXFORM:
-	case PDC_SETCLIP:
+	case PdcSetUnit:
+	case PdcSetVXform:
+	case PdcSetWXform:
+	case PdcSetClip:
 	    s << (INT8)p[0].ival;
 	    break;
-	case PDC_SETWINDOW:
-	case PDC_SETVIEWPORT:
+	case PdcSetWindow:
+	case PdcSetViewport:
 	    s << *p[0].rect;
 	    break;
-	case PDC_SETWMATRIX:
+	case PdcSetWMatrix:
 	    s << *p[0].matrix << (INT8)p[1].ival;
 	    break;
-	case PDC_SETCLIPRGN:
+	case PdcSetClipRegion:
 	    s << *p[0].rgn;
 	    break;
 #if defined(CHECK_RANGE)
 	default:
-	    warning( "QPicture::cmd: Command %d not recognized", c );
+	    qWarning( "QPicture::cmd: Command %d not recognized", c );
 #endif
     }
     int newpos = (int)pictb.at();		// new position
@@ -617,36 +682,42 @@ bool QPicture::cmd( int c, QPainter *, QPDevCmdParam *p )
   Use the QPaintDeviceMetrics class instead.
 
   A picture has the following hard coded values:
-  width=640, height=480. widthMM=236, heightMM=176, numcolors=16777216
-  and depth=24.
+  width=640, height=480, dpi = 72, numcolors=16777216 and depth=24.
 */
 
 int QPicture::metric( int m ) const
 {
     int val;
     switch ( m ) {
-	case PDM_WIDTH:
+	// ### ### ### ### ### ### hard coded value!!!
+	case QPaintDeviceMetrics::PdmWidth:
 	    val = 640;
 	    break;
-	case PDM_HEIGHT:
+	case QPaintDeviceMetrics::PdmHeight:
 	    val = 480;
 	    break;
-	case PDM_WIDTHMM:
-	    val = 236;
+	case QPaintDeviceMetrics::PdmWidthMM:
+	    val = 225;
 	    break;
-	case PDM_HEIGHTMM:
-	    val = 176;
+	case QPaintDeviceMetrics::PdmHeightMM:
+	    val = 169;
 	    break;
-	case PDM_NUMCOLORS:
+	case QPaintDeviceMetrics::PdmDpiX:
+	    val = 72;
+	    break;
+	case QPaintDeviceMetrics::PdmDpiY:
+	    val = 72;
+	    break;
+	case QPaintDeviceMetrics::PdmNumColors:
 	    val = 16777216;
 	    break;
-	case PDM_DEPTH:
+	case QPaintDeviceMetrics::PdmDepth:
 	    val = 24;
 	    break;
 	default:
 	    val = 0;
 #if defined(CHECK_RANGE)
-	    warning( "QPicture::metric: Invalid metric command" );
+	    qWarning( "QPicture::metric: Invalid metric command" );
 #endif
     }
     return val;
@@ -667,3 +738,26 @@ void QPainter::drawPicture( const QPicture &pic )
 {
     ((QPicture*)&pic)->play( (QPainter*)this );
 }
+
+/*!
+  Makes this picture be a deep copy of \a p.
+*/
+QPicture& QPicture::operator= (const QPicture& p)
+{
+    setData(p.data(),p.size());
+    return *this;
+}
+
+
+/*!
+  \internal
+
+  Sets formatOk to FALSE and resets the format version numbers to default
+*/
+
+void QPicture::resetFormat()
+{
+    formatOk = FALSE;
+    formatMajor = mfhdr_maj;
+    formatMinor = mfhdr_min;
+}
