@@ -1,22 +1,25 @@
 /****************************************************************************
-** $Id: qpixmap.cpp,v 2.21 1997/09/29 16:41:55 eiriken Exp $
+** $Id: qpixmap.cpp,v 2.57 1999/06/10 18:32:19 warwick Exp $
 **
 ** Implementation of QPixmap class
 **
 ** Created : 950301
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -24,22 +27,26 @@
 #include "qimage.h"
 #include "qwidget.h"
 #include "qpainter.h"
-#include "qdstream.h"
+#include "qdatastream.h"
 #include "qbuffer.h"
 
-RCSTAG("$Id: qpixmap.cpp,v 2.21 1997/09/29 16:41:55 eiriken Exp $");
-
 
 /*!
   \class QPixmap qpixmap.h
-  \brief The QPixmap class is an off-screen buffer paint device.
+  \brief The QPixmap class is an off-screen pixel-based paint device.
 
   \ingroup drawing
   \ingroup shared
 
-  A common use of the QPixmap class is to enable smooth updating of
+  It is one of the two classes Qt provides for dealing with images,
+  the other being QImage.  QPixmap is designed and optimized for
+  drawing; QImage is designed and optimized for I/O and for direct
+  pixel access/manipulation.  There are (slow) functions to convert
+  between QImage and QPixmap; convertToImage() and convertFromImage().
+
+  One common use of the QPixmap class is to enable smooth updating of
   widgets.  Whenever something complex needs to be drawn, you can use
-  a pixmap to obtain flicker-free drawing.
+  a pixmap to obtain flicker-free drawing, like this:
 
   <ol plain>
   <li> Create a pixmap with the same size as the widget.
@@ -48,52 +55,59 @@ RCSTAG("$Id: qpixmap.cpp,v 2.21 1997/09/29 16:41:55 eiriken Exp $");
   <li> bitBlt() the pixmap contents onto the widget.
   </ol>
 
-  Example of flicker-free update:
-  \code
-    void MyWidget::paintEvent( QPaintEvent * )
-    {
-	QPixmap	 pm( size() );			// create pixmap
-	QPainter p;				// our painter
-	pm.fill( backgroundColor() );		// initialize pixmap
-	p.begin( &pm );				// start painting pixmap
-	...					// draw something
-	p.end();				// painting done
-	bitBlt( this, 0,0, &pm );		// copy pixmap to widget
-    }
-  \endcode
+  Pixel data in a pixmap is internal and managed by the underlying
+  window system.  Pixels can only be accessed through QPainter
+  functions, through bitBlt(), and by converting the QPixmap to a
+  QImage.
 
-  The bitBlt() function has quite a few arguments that are not used in
-  this example.
+  You can display a QPixmap on the screen easily using
+  e.g. QLabel::setPixmap(), and all the \link QButton button classes
+  \endlink support pixmap use.
 
-  Pixel data in a pixmap is internal and managed by the underlying window
-  system.  Pixels can only be accessed through QImage, QPainter functions
-  and the bitBlt().
-
-  You can \link load() load\endlink and \link save() save\endlink
-  pixmaps using several image formats.	You can display a QPixmap on
-  the screen easily using QLabel::setPixmap(), and all the \link
-  QButton button classes \endlink support pixmap use.
-
-  A pixmap can be converted to a QImage to get direct access to the pixels.
-  A QImage can also be converted back to a pixmap.
+  There are also convenience functions to get and set single pixels
+  and to load and save the entire pixmap; these work by converting the
+  pixmap to a QImage internally.
 
   The QPixmap class is optimized by the use of \link shclass.html implicit
   sharing\endlink, so it is very efficient to pass QPixmap objects as
   arguments.
 
+  <strong>Note about Windows 95 and 98:</strong> Because of internal
+  limitations in the operating system, Windows 9x easily crashes if you
+  create around 1000 pixmaps or more.  Qt will work around this problem
+  if you set the \link setOptimization() pixmap optimization\endlink
+  to \c QPixmap::MemoryOptim.  Read more about this in the
+  setOptimization() documentation.
+
   \sa QBitmap, QImage, QImageIO, \link shclass.html Shared Classes\endlink
 */
 
 
+QPixmap::Optimization QPixmap::defOptim = QPixmap::NormalOptim;
+
+
+/*!
+  \internal
+  Private constructor which takes the bitmap flag and the optimization.
+*/
+
+QPixmap::QPixmap( int w, int h, int depth, bool bitmap,
+		  Optimization optimization )
+    : QPaintDevice( QInternal::Pixmap )
+{
+    init( w, h, depth, bitmap, optimization );
+}
+
+
 /*!
   Constructs a null pixmap.
   \sa isNull()
 */
 
 QPixmap::QPixmap()
-    : QPaintDevice( PDT_PIXMAP )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( 0, 0, 0 );
+    init( 0, 0, 0, FALSE, defOptim );
 }
 
 /*!
@@ -111,22 +125,23 @@ QPixmap::QPixmap()
   \sa isNull()
 */
 
-QPixmap::QPixmap( int w, int h, int depth )
-    : QPaintDevice( PDT_PIXMAP )
+QPixmap::QPixmap( int w, int h, int depth, Optimization optimization )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( w, h, depth );
+    init( w, h, depth, FALSE, optimization );
 }
 
 /*!
-  \overload QPixmap::QPixmap( const QSize &size, int depth )
+  \overload QPixmap::QPixmap( const QSize &size, int depth, Optimization optimization )
 */
 
-QPixmap::QPixmap( const QSize &size, int depth )
-    : QPaintDevice( PDT_PIXMAP )
+QPixmap::QPixmap( const QSize &size, int depth, Optimization optimization )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( size.width(), size.height(), depth );
+    init( size.width(), size.height(), depth, FALSE, optimization );
 }
 
+
 /*!
   Constructs a pixmap from the file \e fileName. If the file does not
   exist, or is of an unknown format, the pixmap becomes a null pixmap.
@@ -136,11 +151,11 @@ QPixmap::QPixmap( const QSize &size, int depth )
   \sa isNull(), load(), loadFromData(), save(), imageFormat()
 */
 
-QPixmap::QPixmap( const char *fileName, const char *format,
+QPixmap::QPixmap( const QString& fileName, const char *format,
 	int conversion_flags )
-    : QPaintDevice( PDT_PIXMAP )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( 0, 0, 0 );
+    init( 0, 0, 0, FALSE, defOptim );
     load( fileName, format, conversion_flags );
 }
 
@@ -153,10 +168,10 @@ QPixmap::QPixmap( const char *fileName, const char *format,
   \sa isNull(), load(), loadFromData(), save(), imageFormat()
 */
 
-QPixmap::QPixmap( const char *fileName, const char *format, ColorMode mode )
-    : QPaintDevice( PDT_PIXMAP )
+QPixmap::QPixmap( const QString& fileName, const char *format, ColorMode mode )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( 0, 0, 0 );
+    init( 0, 0, 0, FALSE, defOptim );
     load( fileName, format, mode );
 }
 
@@ -165,62 +180,123 @@ QPixmap::QPixmap( const char *fileName, const char *format, ColorMode mode )
 */
 
 QPixmap::QPixmap( const char *xpm[] )
-    : QPaintDevice( PDT_PIXMAP )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    init( 0, 0, 0 );
+    init( 0, 0, 0, FALSE, defOptim );
     QImage image( xpm );
     if ( !image.isNull() )
 	convertFromImage( image );
 }
 
-
 /*!
-  Special-purpose function that detaches the pixmap from shared pixmap data.
+  Constructs a pixmaps by loading from \a img_data.
+  The data can be in any image format supported by Qt.
 
-  A pixmap is automatically detached by Qt whenever its contents is about
-  to change.  This is done in all QPixmap member functions that modify the
-  pixmap (fill(), resize(), convertFromImage(), load() etc.), in bitBlt()
-  for the destination pixmap and in QPainter::begin() on a pixmap.
-
-  It is possible to modify a pixmap without letting Qt know.
-  You can first obtain the \link handle() system-dependent handle\endlink
-  and then call system-specific functions (for instance BitBlt under Windows)
-  that modifies the pixmap contents.  In this case, you can call detach()
-  to cut the pixmap loose from other pixmaps that share data with this one.
-
-  detach() returns immediately if there is just a single reference or if
-  the pixmap has not been initialized yet.
+  \sa loadFromData()
 */
 
-void QPixmap::detach()
+QPixmap::QPixmap( const QByteArray & img_data )
+    : QPaintDevice( QInternal::Pixmap )
 {
-    if ( data->optim )
-	data->dirty = TRUE;
-    if ( data->uninit || data->count == 1 ) {
-	data->uninit = FALSE;
-	return;
-    }
-    *this = copy();
+    init( 0, 0, 0, FALSE, defOptim );
+    loadFromData( img_data );
 }
 
+
 /*!
-  Returns a 
+  Constructs a pixmap which is a copy of \e pixmap.
+*/
+
+QPixmap::QPixmap( const QPixmap &pixmap )
+    : QPaintDevice( QInternal::Pixmap )
+{
+    if ( pixmap.paintingActive() ) {		// make a deep copy
+	data = 0;
+	operator=( pixmap.copy() );
+    } else {
+	data = pixmap.data;
+	data->ref();
+	devFlags = pixmap.devFlags;		// copy QPaintDevice flags
+#if defined(_WS_WIN_)
+	hdc = pixmap.hdc;			// copy Windows device context
+#elif defined(_WS_X11_)
+	hd = pixmap.hd;				// copy X11 drawable
+	copyX11Data( &pixmap );			// copy x11Data
+#endif
+    }
+}
+
+
+/*!
+  Destroys the pixmap.
+*/
+
+QPixmap::~QPixmap()
+{
+    deref();
+}
+
+
+/*!
+  Returns a
   \link shclass.html deep copy\endlink of the pixmap using the bitBlt()
   function to copy the pixels.
   \sa operator=()
 */
 
-QPixmap QPixmap::copy() const
+QPixmap QPixmap::copy( bool ignoreMask ) const
 {
-    QPixmap tmp( data->w, data->h, data->d );
-    tmp.data->optim  = data->optim;		// copy optim flag
-    tmp.data->bitmap = data->bitmap;		// copy bitmap flag
-    if ( !tmp.isNull() ) {			// copy the bitmap
-	bitBlt( &tmp, 0,0, this, 0,0, data->w, data->h, CopyROP, TRUE );
-	if ( data->mask )			// copy the mask
-	    tmp.setMask( *data->mask );
+    QPixmap pm( data->w, data->h, data->d, data->bitmap, data->optim );
+    if ( !pm.isNull() ) {			// copy the bitmap
+#if defined(_WS_X11_)
+	pm.copyX11Data( this );
+#endif
+	bitBlt( &pm, 0,0, this, 0,0, data->w, data->h, CopyROP, TRUE );
+	if ( !ignoreMask && data->mask )		// copy the mask
+	    pm.setMask( data->selfmask ? *((QBitmap*)&pm) : *data->mask );
     }
-    return tmp;
+    return pm;
+}
+
+
+/*!
+  Assigns the pixmap \e pixmap to this pixmap and returns a reference to
+  this pixmap.
+*/
+
+QPixmap &QPixmap::operator=( const QPixmap &pixmap )
+{
+    if ( paintingActive() ) {
+#if defined(CHECK_STATE)
+	qWarning("QPixmap::operator=: Cannot assign to pixmap during painting");
+#endif
+	return *this;
+    }
+    pixmap.data->ref();				// avoid 'x = x'
+    deref();
+    if ( pixmap.paintingActive() ) {		// make a deep copy
+	init( pixmap.width(), pixmap.height(), pixmap.depth(),
+	      pixmap.data->bitmap, pixmap.data->optim );
+	data->uninit = FALSE;
+	if ( !isNull() ) {
+	    bitBlt( this, 0, 0, &pixmap, pixmap.width(), pixmap.height(),
+		    CopyROP, TRUE );
+	    if ( pixmap.mask() )
+		setMask( pixmap.data->selfmask ? *((QBitmap*)(this))
+					       : *pixmap.mask() );
+	}
+	pixmap.data->deref();
+    } else {
+	data = pixmap.data;
+	devFlags = pixmap.devFlags;		// copy QPaintDevice flags
+#if defined(_WS_WIN_)
+	hdc = pixmap.hdc;
+#elif defined(_WS_X11_)
+	hd = pixmap.hd;				// copy QPaintDevice drawable
+	copyX11Data( &pixmap );			// copy x11Data
+#endif
+    }
+    return *this;
 }
 
 
@@ -292,6 +368,7 @@ QPixmap &QPixmap::operator=( const QImage &image )
 
 /*!
   \fn void QPixmap::fill( const QWidget *widget, const QPoint &ofs )
+
   Fills the pixmap with the widget's background color or pixmap.
   If the background is empty, nothing is done.
 
@@ -337,7 +414,7 @@ void QPixmap::fill( const QWidget *widget, int xofs, int yofs )
 	    QPainter p;
 	    p.begin( this );
 	    p.setPen( NoPen );
-	    p.setBrush( QBrush( black,*widget->backgroundPixmap() ) );
+	    p.setBrush( QBrush( Qt::black,*widget->backgroundPixmap() ) );
 	    p.setBrushOrigin( -xofs, -yofs );
 	    p.drawRect( 0, 0, width(), height() );
 	    p.end();
@@ -363,29 +440,29 @@ void QPixmap::fill( const QWidget *widget, int xofs, int yofs )
 void QPixmap::resize( int w, int h )
 {
     if ( w < 1 || h < 1 ) {			// becomes null
-	QPixmap pm;
-	pm.data->optim	= data->optim;		// keep optimization flag
-	pm.data->bitmap = data->bitmap;		// keep is-a flag
+	QPixmap pm( 0, 0, 0, data->bitmap, data->optim );
 	*this = pm;
 	return;
     }
-
     int d;
     if ( depth() > 0 )
 	d = depth();
     else
 	d = isQBitmap() ? 1 : -1;
-    QPixmap pm( w, h, d );			// create new pixmap
+    // Create new pixmap
+    QPixmap pm( w, h, d, data->bitmap, data->optim );
     if ( !data->uninit && !isNull() )		// has existing pixmap
 	bitBlt( &pm, 0, 0, this, 0, 0,		// copy old pixmap
 		QMIN(width(), w),
 		QMIN(height(),h), CopyROP, TRUE );
-    pm.data->optim  = data->optim;		// keep optim flag
-    pm.data->bitmap = data->bitmap;		// keep bitmap flag
     if ( data->mask ) {				// resize mask as well
-	QBitmap m = *data->mask;
-	m.resize( w, h );
-	pm.setMask( m );
+	if ( data->selfmask ) {			// preserve self-mask
+	    pm.setMask( *((QBitmap*)&pm) );
+	} else {				// independent mask
+	    QBitmap m = *data->mask;
+	    m.resize( w, h );
+	    pm.setMask( m );
+	}
     }
     *this = pm;
 }
@@ -411,38 +488,46 @@ void QPixmap::resize( int w, int h )
   \sa mask(), createHeuristicMask(), QBitmap
 */
 
-void QPixmap::setMask( const QBitmap &mask )
+void QPixmap::setMask( const QBitmap &newmask )
 {
-    if ( mask.handle() && mask.handle() == handle() ) {
-	const QPixmap *tmp = &mask;		// dec cxx bug
-	QPixmap m = tmp->copy();
+    if ( (data == newmask.data) ||
+	 ( newmask.handle() && newmask.handle() == handle() ) ) {
+	const QPixmap *tmp = &newmask;		// dec cxx bug
+	QPixmap m = tmp->copy( TRUE );
 	setMask( *((QBitmap*)&m) );
 	data->selfmask = TRUE;			// mask == pixmap
 	return;
     }
     detach();
     data->selfmask = FALSE;
-    if ( mask.isNull() ) {			// reset the mask
+    if ( newmask.isNull() ) {			// reset the mask
 	delete data->mask;
 	data->mask = 0;
 	return;
     }
-    if ( mask.width() != width() || mask.height() != height() ) {
+    if ( newmask.width() != width() || newmask.height() != height() ) {
 #if defined(CHECK_RANGE)
-	warning( "QPixmap::setMask: The pixmap and the mask must have "
+	qWarning( "QPixmap::setMask: The pixmap and the mask must have "
 		 "the same size" );
 #endif
 	return;
     }
     delete data->mask;
-    data->mask = new QBitmap( mask );
+    QBitmap* newmaskcopy;
+    if ( newmask.mask() ) {
+	const QPixmap *tmp = &newmask;
+	newmaskcopy = (QBitmap*)new QPixmap( tmp->copy( TRUE ) );
+    }
+    else {
+	newmaskcopy = new QBitmap( newmask );
+    }
+    data->mask = newmaskcopy;
 }
 
 
 /*!
   \fn bool QPixmap::selfMask() const
-  Returns TRUE if the pixmap's mask is identical to the pixmap
-  itself.
+  Returns TRUE if the pixmap's mask is identical to the pixmap itself.
   \sa mask()
 */
 
@@ -456,7 +541,7 @@ void QPixmap::setMask( const QBitmap &mask )
   things like:
   \code
     pm->setMask( pm->createHeuristicMask() );
-  \endcode;
+  \endcode
 
   This function is slow because it involves transformation to a QImage,
   non-trivial computations and a transformation back to QBitmap.
@@ -481,29 +566,12 @@ QBitmap QPixmap::createHeuristicMask( bool clipTight ) const
   \sa load(), save()
 */
 
-const char *QPixmap::imageFormat( const char *fileName )
+const char* QPixmap::imageFormat( const QString &fileName )
 {
     return QImageIO::imageFormat(fileName);
 }
 
 
-static bool can_handle_bmp = FALSE;
-static bool did_handle_bmp = FALSE;
-
-bool qt_image_native_bmp()
-{
-    if ( can_handle_bmp ) {
-	did_handle_bmp = TRUE;
-	return TRUE;
-    }
-    return FALSE;
-}
-
-bool qt_image_did_native_bmp()
-{
-    return did_handle_bmp;
-}
-
 /*!
   Loads a pixmap from the file \e fileName.
   Returns TRUE if successful, or FALSE if the pixmap could not be loaded.
@@ -521,28 +589,23 @@ bool qt_image_did_native_bmp()
   \sa loadFromData(), save(), imageFormat(), QImage::load(), QImageIO
 */
 
-bool QPixmap::load( const char *fileName, const char *format,
+bool QPixmap::load( const QString &fileName, const char *format,
 		    int conversion_flags )
 {
     QImageIO io( fileName, format );
-#if defined(_WS_WIN_)
-    can_handle_bmp = TRUE;
-#endif
     bool result = io.read();
     if ( result ) {
-	detach();
+	detach(); // ###hanord: Why detach here, convertFromImage does it
 	result = convertFromImage( io.image(), conversion_flags );
     }
-#if defined(_WS_WIN_)
-    can_handle_bmp = did_handle_bmp = FALSE;
-#endif
     return result;
 }
 
 /*!
   \overload
 */
-bool QPixmap::load( const char *fileName, const char *format,
+
+bool QPixmap::load( const QString &fileName, const char *format,
 		    ColorMode mode )
 {
     int conversion_flags = 0;
@@ -559,10 +622,12 @@ bool QPixmap::load( const char *fileName, const char *format,
     return load( fileName, format, conversion_flags );
 }
 
+
 /*!
   \overload
 */
-bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
+
+bool QPixmap::convertFromImage( const QImage &image, ColorMode mode )
 {
     int conversion_flags = 0;
     switch (mode) {
@@ -575,7 +640,7 @@ bool QPixmap::convertFromImage( const QImage &img, ColorMode mode )
       default:
 	;// Nothing.
     }
-    return convertFromImage( img, conversion_flags );
+    return convertFromImage( image, conversion_flags );
 }
 
 
@@ -604,9 +669,6 @@ bool QPixmap::loadFromData( const uchar *buf, uint len, const char *format,
     QBuffer b( a );
     b.open( IO_ReadOnly );
     QImageIO io( &b, format );
-#if defined(_WS_WIN_)
-    can_handle_bmp = TRUE;
-#endif
     bool result = io.read();
     b.close();
     a.resetRawData( (char *)buf, len );
@@ -614,15 +676,13 @@ bool QPixmap::loadFromData( const uchar *buf, uint len, const char *format,
 	detach();
 	result = convertFromImage( io.image(), conversion_flags );
     }
-#if defined(_WS_WIN_)
-    can_handle_bmp = did_handle_bmp = FALSE;
-#endif
     return result;
 }
 
 /*!
   \overload
 */
+
 bool QPixmap::loadFromData( const uchar *buf, uint len, const char *format,
 			    ColorMode mode )
 {
@@ -640,6 +700,18 @@ bool QPixmap::loadFromData( const uchar *buf, uint len, const char *format,
     return loadFromData( buf, len, format, conversion_flags );
 }
 
+/*!
+  \overload
+*/
+
+bool QPixmap::loadFromData( const QByteArray &buf, const char *format,
+			    int conversion_flags )
+{
+    return loadFromData( (const uchar *)(buf.data()), buf.size(),
+			 format, conversion_flags );
+}
+
+
 /*!
   Saves the pixmap to the file \e fileName, using the image file format
   \e format.  Returns TRUE if successful, or FALSE if the pixmap could not
@@ -647,7 +719,7 @@ bool QPixmap::loadFromData( const uchar *buf, uint len, const char *format,
   \sa load(), loadFromData(), imageFormat(), QImage::save(), QImageIO
 */
 
-bool QPixmap::save( const char *fileName, const char *format ) const
+bool QPixmap::save( const QString &fileName, const char *format ) const
 {
     if ( isNull() )
 	return FALSE;				// nothing to save
@@ -667,19 +739,54 @@ bool QPixmap::save( const char *fileName, const char *format ) const
 */
 
 
+/*!
+  Returns the default pixmap optimization setting.
+  \sa setDefaultOptimization(), setOptimization(), optimization()
+*/
+
+QPixmap::Optimization QPixmap::defaultOptimization()
+{
+    return defOptim;
+}
+
+/*!
+  Sets the default pixmap optimization.
+
+  All \e new pixmaps that are created will use this default optimization.
+  You may also set optimization for individual pixmaps using the
+  setOptimization() function.
+
+  The initial default optimization setting is \c QPixmap::Normal.
+
+  \sa defaultOptimization(), setOptimization(), optimization()
+*/
+
+void QPixmap::setDefaultOptimization( Optimization optimization )
+{
+    if ( optimization != DefaultOptim )
+	defOptim = optimization;
+}
+
+
 /*****************************************************************************
   QPixmap stream functions
  *****************************************************************************/
 
 /*!
   \relates QPixmap
-  Writes a pixmap to the stream as a BMP image.
+  Writes a pixmap to the stream as a PNG image.
   \sa QPixmap::save()
 */
 
 QDataStream &operator<<( QDataStream &s, const QPixmap &pixmap )
 {
-    QImageIO io( s.device(), "BMP" );
+    QImageIO io;
+    io.setIODevice( s.device() );
+    if ( s.version() == 1 )
+	io.setFormat( "BMP" );
+    else
+	io.setFormat( "PNG" );
+
     io.setImage( pixmap.convertToImage() );
     io.write();
     return s;
