@@ -1,45 +1,58 @@
 /****************************************************************************
-** $Id: qpixmap_x11.cpp,v 2.38 1998/07/03 00:09:37 hanord Exp $
+** $Id: qpixmap_x11.cpp,v 2.69 1999/06/24 20:13:38 warwick Exp $
 **
 ** Implementation of QPixmap class for X11
 **
 ** Created : 940501
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
+#include "qglobal.h"
+
 // Uncomment the next line to enable the MIT Shared Memory extension
+//
+// WARNING:  This has some problems:
+//
+//    1. Consumes a 800x600 pixmap
+//    2. Qt does not handle the ShmCompletion message, so you will
+//        get strange effects if you xForm() repeatedly.
+//
 // #define MITSHM
 
+#if defined(_OS_WIN32_) && defined(MITSHM)
+#undef MITSHM
+#endif
+
 #include "qbitmap.h"
+#include "qpaintdevicemetrics.h"
 #include "qimage.h"
-#include "qpaintdevicedefs.h"
 #include "qwmatrix.h"
 #include "qapplication.h"
 #include <stdlib.h>
-#define	 GC GC_QQQ
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xos.h>
+#include "qt_x11.h"
 #ifdef MITSHM
 #include <sys/ipc.h>
 #include <sys/shm.h>
 #include <X11/extensions/XShm.h>
 #endif
 
+
 // For thread-safety:
 //   image->data does not belong to X11, so we must free it ourselves.
 
@@ -68,24 +81,23 @@ static void qt_cleanup_mitshm()
 {
     if ( xshmimg == 0 )
 	return;
-    Display *dpy = QPaintDevice::x__Display();
+    Display *dpy = QPaintDevice::x11AppDisplay();
     if ( xshmpm ) {
 	XFreePixmap( dpy, xshmpm );
 	xshmpm = 0;
     }
-    XShmDetach( dpy, &xshminfo );
-    qSafeXDestroyImage( xshmimg );
-    xshmimg = 0;
+    XShmDetach( dpy, &xshminfo ); xshmimg->data = 0;
+    qSafeXDestroyImage( xshmimg ); xshmimg = 0;
     shmdt( xshminfo.shmaddr );
     shmctl( xshminfo.shmid, IPC_RMID, 0 );
 }
 
-bool qt_create_mitshm_buffer( QPaintDevice* dev, int w, int h )
+
+static bool qt_create_mitshm_buffer( const QPaintDevice* dev, int w, int h )
 {
     static int major, minor;
     static Bool pixmaps_ok;
     Display *dpy = dev->x11Display();
-    int scr	 = dev->x11Screen();
     int dd	 = dev->x11Depth();
     Visual *vis	 = (Visual*)dev->x11Visual();
 
@@ -108,7 +120,8 @@ bool qt_create_mitshm_buffer( QPaintDevice* dev, int w, int h )
 			     IPC_CREAT | 0777 );
     ok = xshminfo.shmid != -1;
     if ( ok ) {
-	xshminfo.shmaddr = xshmimg->data = shmat( xshminfo.shmid, 0, 0 );
+	xshmimg->data = shmat( xshminfo.shmid, 0, 0 );
+	xshminfo.shmaddr = xshmimg->data;
 	ok = xshminfo.shmaddr != 0;
     }
     xshminfo.readOnly = FALSE;
@@ -116,6 +129,7 @@ bool qt_create_mitshm_buffer( QPaintDevice* dev, int w, int h )
 	ok = XShmAttach( dpy, &xshminfo );
     if ( !ok ) {
 	qSafeXDestroyImage( xshmimg );
+	xshmimg = 0;
 	if ( xshminfo.shmaddr )
 	    shmdt( xshminfo.shmaddr );
 	if ( xshminfo.shmid != -1 )
@@ -131,10 +145,12 @@ bool qt_create_mitshm_buffer( QPaintDevice* dev, int w, int h )
 
 #else
 
-bool qt_create_mitshm_buffer( QPaintDevice*, int, int )
-{
-    return FALSE;
-}
+// If extern, need a dummy.
+//
+// static bool qt_create_mitshm_buffer( QPaintDevice*, int, int )
+// {
+//     return FALSE;
+// }
 
 #endif // MITSHM
 
@@ -157,15 +173,27 @@ static uchar *flip_bits( const uchar *bits, int len )
     return newdata;
 }
 
+// Returns position of highest bit set or -1 if none
 static int highest_bit( uint v )
 {
     int i;
-    uint b = (uint)1 << 31;			// get pos of highest set bit
+    uint b = (uint)1 << 31;
     for ( i=31; ((b & v) == 0) && i>=0;	 i-- )
 	b >>= 1;
     return i;
 }
 
+// Returns position of lowest set bit in 'v' as an integer (0-31), or -1
+static int lowest_bit( uint v )
+{
+    int i;
+    ulong lb;
+    lb = 1;
+    for (i=0; ((v & lb) == 0) && i<32;  i++, lb<<=1);
+    return i==32 ? -1 : i;
+}
+
+// Counts the number of bits set in 'v'
 static uint n_bits( uint v )
 {
     int i = 0;
@@ -176,15 +204,16 @@ static uint n_bits( uint v )
     return i;
 }
 
+
 static uint *red_scale_table   = 0;
 static uint *green_scale_table = 0;
 static uint *blue_scale_table  = 0;
 
 static void cleanup_scale_tables()
 {
-    delete red_scale_table;
-    delete green_scale_table;
-    delete blue_scale_table;
+    delete[] red_scale_table;
+    delete[] green_scale_table;
+    delete[] blue_scale_table;
 }
 
 /*
@@ -195,7 +224,7 @@ static void build_scale_table( uint **table, uint nBits )
 {
     if ( nBits > 7 ) {
 #if defined(CHECK_RANGE)
-	warning( "build_scale_table: internal error, nBits = %i", nBits );
+	qWarning( "build_scale_table: internal error, nBits = %i", nBits );
 #endif
 	return;
     }
@@ -218,73 +247,60 @@ static void build_scale_table( uint **table, uint nBits )
   QPixmap member functions
  *****************************************************************************/
 
-bool QPixmap::optimAll = TRUE;
-QPixmap::Optimization QPixmap::defOpt = QPixmap::NormalOptim;
-
-
 /*!
   \internal
   Initializes the pixmap data.
 */
 
-void QPixmap::init( int w, int h, int d )
+void QPixmap::init( int w, int h, int d, bool bitmap, Optimization optim )
 {
     static int serial = 0;
     int dd = x11Depth();
 
+    if ( optim == DefaultOptim )		// use default optimization
+	optim = defOptim;
+
     data = new QPixmapData;
     CHECK_PTR( data );
 
-    data->optim	   = optimAll;
-    data->uninit   = TRUE;
-    data->bitmap   = FALSE;
-    data->selfmask = FALSE;
-    data->ser_no   = ++serial;
-    data->opt	   = defOpt;
-    data->mask	   = 0;
-    data->ximage   = 0;
-    data->maskgc   = 0;
+    memset( data, 0, sizeof(QPixmapData) );
+    data->count  = 1;
+    data->uninit = TRUE;
+    data->bitmap = bitmap;
+    data->ser_no = ++serial;
+    data->optim	 = optim;
 
     bool make_null = w == 0 || h == 0;		// create null pixmap
     if ( d == 1 )				// monocrome pixmap
 	data->d = 1;
     else if ( d < 0 || d == dd )		// def depth pixmap
 	data->d = dd;
-    else					// invalid depth
-	data->d = 0;
     if ( make_null || w < 0 || h < 0 || data->d == 0 ) {
-	data->w = data->h = 0;
-	data->d = 0;
 	hd = 0;
 #if defined(CHECK_RANGE)
 	if ( !make_null )
-	    warning( "QPixmap: Invalid pixmap parameters" );
+	    qWarning( "QPixmap: Invalid pixmap parameters" );
 #endif
 	return;
     }
     data->w = w;
     data->h = h;
-    hd = (HANDLE)XCreatePixmap( dpy, DefaultRootWindow(dpy), w, h, data->d );
+    hd = (HANDLE)XCreatePixmap( x11Display(), DefaultRootWindow(x11Display()),
+				w, h, data->d );
 }
 
 
 void QPixmap::deref()
 {
     if ( data && data->deref() ) {			// last reference lost
-	if ( data->mask ) {
+	if ( data->mask )
 	    delete data->mask;
-	    data->mask = 0;
-	}
-	if ( data->maskgc ) {
-	    XFreeGC( dpy, (GC)data->maskgc );
-	    data->maskgc = 0;
-	}
-	if ( data->ximage ) {
+	if ( data->ximage )
 	    qSafeXDestroyImage( (XImage*)data->ximage );
-	    data->ximage = 0;
-	}
+	if ( data->maskgc )
+	    XFreeGC( x11Display(), (GC)data->maskgc );
 	if ( hd && qApp ) {
-	    XFreePixmap( dpy, hd );
+	    XFreePixmap( x11Display(), hd );
 	    hd = 0;
 	}
 	delete data;
@@ -298,9 +314,9 @@ void QPixmap::deref()
 */
 
 QPixmap::QPixmap( int w, int h, const uchar *bits, bool isXbitmap )
-    : QPaintDevice( PDT_PIXMAP )
+    : QPaintDevice( QInternal::Pixmap )
 {						// for bitmaps only
-    init( 0, 0, 0 );
+    init( 0, 0, 0, FALSE, defOptim );
     if ( w <= 0 || h <= 0 )			// create null pixmap
 	return;
 
@@ -315,39 +331,13 @@ QPixmap::QPixmap( int w, int h, const uchar *bits, bool isXbitmap )
 	flipped_bits = flip_bits( bits, ((w+7)/8)*h );
 	bits = flipped_bits;
     }
-    hd = (HANDLE)XCreateBitmapFromData( dpy, DefaultRootWindow(dpy),
-				(char *)bits, w, h );
+    hd = (HANDLE)XCreateBitmapFromData( x11Display(),
+					DefaultRootWindow(x11Display()),
+					(char *)bits, w, h );
     if ( flipped_bits )				// Avoid purify complaint
 	delete [] flipped_bits;
 }
 
-/*!
-  Constructs a pixmap which is a copy of \e pixmap.
-*/
-
-QPixmap::QPixmap( const QPixmap &pixmap )
-    : QPaintDevice( PDT_PIXMAP )
-{
-    if ( pixmap.paintingActive() ) {		// make a deep copy
-	data = 0;
-	operator=( pixmap );
-    } else {
-	data = pixmap.data;
-	data->ref();
-	devFlags = pixmap.devFlags;		// copy QPaintDevice flags
-	hd = pixmap.hd;				// copy QPaintDevice drawable
-    }
-}
-
-/*!
-  Destroys the pixmap.
-*/
-
-QPixmap::~QPixmap()
-{
-    deref();
-}
-
 
 /*!
   Special-purpose function that detaches the pixmap from shared pixmap data.
@@ -379,50 +369,12 @@ void QPixmap::detach()
 	data->ximage = 0;
     }
     if ( data->maskgc ) {
-	XFreeGC( dpy, (GC)data->maskgc );
+	XFreeGC( x11Display(), (GC)data->maskgc );
 	data->maskgc = 0;
     }
 }
 
 
-/*!
-  Assigns the pixmap \e pixmap to this pixmap and returns a reference to
-  this pixmap.
-*/
-
-QPixmap &QPixmap::operator=( const QPixmap &pixmap )
-{
-    if ( paintingActive() ) {
-#if defined(CHECK_STATE)
-	warning("QPixmap::operator=: Cannot assign to pixmap during painting");
-#endif
-	return *this;
-    }
-    pixmap.data->ref();				// avoid 'x = x'
-    deref();
-
-    if ( pixmap.paintingActive() ) {		// make a deep copy
-	init( pixmap.width(), pixmap.height(), pixmap.depth() );
-	data->uninit = FALSE;
-	data->optim  = pixmap.data->optim;	// copy optimization flag
-	data->bitmap = pixmap.data->bitmap;	// copy bitmap flag
-	data->opt    = pixmap.data->opt;	// copy optimization setting
-	if ( !isNull() ) {
-	    bitBlt( this, 0, 0, &pixmap, pixmap.width(), pixmap.height(),
-		    CopyROP, TRUE );
-	    if ( pixmap.mask() )
-		setMask( *pixmap.mask() );
-	}
-	pixmap.data->deref();
-    } else {
-	data = pixmap.data;
-	devFlags = pixmap.devFlags;		// copy QPaintDevice flags
-	hd = pixmap.hd;				// copy QPaintDevice drawable
-    }
-    return *this;
-}
-
-
 /*!
   Returns the default pixmap depth, i.e. the depth a pixmap gets
   if -1 is specified.
@@ -431,7 +383,7 @@ QPixmap &QPixmap::operator=( const QPixmap &pixmap )
 
 int QPixmap::defaultDepth()
 {
-    return x11Depth();
+    return x11AppDepth();
 }
 
 
@@ -466,94 +418,67 @@ int QPixmap::defaultDepth()
 
   The \a optimization parameter can be:
   <ul>
-  <li> \c QPixmap::NoOptim, avoid optimization. Little or no caching is
-  done. Use this setting if memory is scarce and the speed of pixmap
-  operations is not critical to your application.
+  <li> \c QPixmap::DefaultOptim, sets the default optimization, i.e.
+  QPixmap::defaultOptimization().
+  <li> \c QPixmap::NoOptim, equivalent with MemoryOptim.
+  <li> \c QPixmap::MemoryOptim, use as little memory/resource as possible,
+  with little or no caching.  The tradeoff may be unnoticably slower pixmap
+  operations.  Read the important note about Windows 95/98 below.
   <li> \c QPixmap::NormalOptim, normal optimization to make pixmap drawing
   faster. This option is the default and is suitable for most purposes.
-  <li> \c QPixmap::BestOptim, heavily optimized pixmap drawing. Use this
-  option for pixmap drawn extremely frequently.
+  <li> \c QPixmap::BestOptim, heavily optimized pixmap drawing, but may
+  consume more memory. Use this option for pixmap drawn extremely
+  frequently, e.g. masked pixmaps/sprites.
   </ul>
 
   Use the setDefaultOptimization() to change the default optimization
   for all new pixmaps.
 
+  <strong>Note about Windows 95 and 98:</strong> On Windows 9x, the system
+  crashes if you create around 1000 pixmaps or more, independent of the
+  size of the pixmaps or installed RAM.  The reason is that Windows 9x has
+  a serious resource limitation that comes from the old, 64k segmented
+  memory model. Windows NT does not have this limitation.
+
+  Qt tries to work around the resource limitation.  If you set the pixmap
+  optimization to \c QPixmap::MemoryOptim and the width of your pixmap is
+  less than or equal to 128 pixels, Qt will create a logical pixmap which
+  refers an internal pixmap.  This internal pixmap is shared by hundreds
+  of other pixmaps which have the optimization set to \c
+  QPixmap::MemoryOptim.  The result is that your application may use
+  thousands of pixmaps on Windows 9x and spend only a few percent of the
+  system resources.
+
+  When should you set the memory optimization?  If your application uses
+  hundreds of pixmaps, e.g. on tool bar buttons, in popup menus, and you
+  plan to run it on Windows 95 or Windows 98.
+
+  Here is an example:
+  \code
+    QPixmap::setDefaultOptimization( QPixmap::MemoryOptim );
+    while ( ... ) {
+      // load tool bar pixmaps etc.
+      QPixmap *pixmap = new QPixmap(fileName);
+    }
+    QPixmap::setDefaultOptimization( QPixmap::NormalOptim );
+  \endcode
+
   \sa optimization(), setDefaultOptimization(), defaultOptimization()
 */
 
 void QPixmap::setOptimization( Optimization optimization )
 {
-    if ( optimization == data->opt )
+    if ( optimization == data->optim )
 	return;
-    data->optim = optimization != NoOptim;	// ### compatibility setting
-    data->opt = optimization;
-    if ( optimization == NoOptim && data->ximage ) {
+    detach();
+    data->optim = optimization == DefaultOptim ?
+	    defOptim : optimization;
+    if ( data->optim == MemoryOptim && data->ximage ) {
 	qSafeXDestroyImage( (XImage*)data->ximage );
 	data->ximage = 0;
     }
 }
 
-/*!
-  Returns the default pixmap optimization setting.
-  \sa setDefaultOptimization(), setOptimization(), optimization()
-*/
-
-QPixmap::Optimization QPixmap::defaultOptimization()
-{
-    return defOpt;
-}
-
-/*!
-  Sets the default pixmap optimization.
-
-  All \e new pixmaps that are created will use this default optimization.
-  You may also set optimization for individual pixmaps using the
-  setOptimization() function.
-
-  The initial default optimization setting is \c QPixmap::Normal.
-
-  \sa defaultOptimization(), setOptimization(), optimization()
-*/
-
-void QPixmap::setDefaultOptimization( Optimization optimization )
-{
-    defOpt = optimization;
-    optimAll = optimization != NoOptim;		// ### compatibility setting
-}
-
-
-/*!
-  \fn bool QPixmap::isOptimized() const
-  Deprecated function, replaced by optimization();
-*/
-
-/*!
-  Deprecated function, replaced by setOptimization().
-*/
-
-void QPixmap::optimize( bool enable )
-{
-    setOptimization( enable ? NormalOptim : NoOptim );
-}
-
-/*!
-  Deprecated function, replaced by defaultOptimization().
-*/
-
-bool QPixmap::isGloballyOptimized()
-{
-    return defOpt != NoOptim;
-}
-
-/*!
-  Deprecated function, replaced by setDefaultOptimization.
-*/
-
-void QPixmap::optimizeGlobally( bool enable )
-{
-    setDefaultOptimization( enable ? NormalOptim : NoOptim );
-}
-
 
 /*!
   Fills the pixmap with the color \e fillColor.
@@ -565,8 +490,8 @@ void QPixmap::fill( const QColor &fillColor )
 	return;
     detach();					// detach other references
     GC gc = qt_xget_temp_gc( depth()==1 );
-    XSetForeground( dpy, gc, fillColor.pixel() );
-    XFillRectangle( dpy, hd, gc, 0, 0, width(), height() );
+    XSetForeground( x11Display(), gc, fillColor.pixel() );
+    XFillRectangle( x11Display(), hd, gc, 0, 0, width(), height() );
 }
 
 
@@ -579,32 +504,38 @@ void QPixmap::fill( const QColor &fillColor )
 int QPixmap::metric( int m ) const
 {
     int val;
-    if ( m == PDM_WIDTH || m == PDM_HEIGHT ) {
-	if ( m == PDM_WIDTH )
-	    val = width();
-	else
-	    val = height();
+    if ( m == QPaintDeviceMetrics::PdmWidth )
+	val = width();
+    else if ( m == QPaintDeviceMetrics::PdmHeight ) {
+	val = height();
     } else {
-	int scr = qt_xscreen();
+	Display *dpy = x11Display();
+	int scr = x11Screen();
 	switch ( m ) {
-	    case PDM_WIDTHMM:
+	    case QPaintDeviceMetrics::PdmDpiX:
+		val = QPaintDevice::x11AppDpiX();
+		break;
+	    case QPaintDeviceMetrics::PdmDpiY:
+		val = QPaintDevice::x11AppDpiY();
+		break;
+	    case QPaintDeviceMetrics::PdmWidthMM:
 		val = (DisplayWidthMM(dpy,scr)*width())/
 		      DisplayWidth(dpy,scr);
 		break;
-	    case PDM_HEIGHTMM:
+	    case QPaintDeviceMetrics::PdmHeightMM:
 		val = (DisplayHeightMM(dpy,scr)*height())/
 		      DisplayHeight(dpy,scr);
 		break;
-	    case PDM_NUMCOLORS:
+	    case QPaintDeviceMetrics::PdmNumColors:
 		val = 1 << depth();
 		break;
-	    case PDM_DEPTH:
+	    case QPaintDeviceMetrics::PdmDepth:
 		val = depth();
 		break;
 	    default:
 		val = 0;
 #if defined(CHECK_RANGE)
-		warning( "QPixmap::metric: Invalid metric command" );
+		qWarning( "QPixmap::metric: Invalid metric command" );
 #endif
 	}
     }
@@ -632,7 +563,7 @@ QImage QPixmap::convertToImage() const
     QImage image;
     if ( isNull() ) {
 #if defined(CHECK_NULL)
-	warning( "QPixmap::convertToImage: Cannot convert a null pixmap" );
+	qWarning( "QPixmap::convertToImage: Cannot convert a null pixmap" );
 #endif
 	return image;
     }
@@ -651,7 +582,7 @@ QImage QPixmap::convertToImage() const
 
     XImage *xi = (XImage *)data->ximage;	// any cached ximage?
     if ( !xi )					// fetch data from X server
-	xi = XGetImage( dpy, hd, 0, 0, w, h, AllPlanes,
+	xi = XGetImage( x11Display(), hd, 0, 0, w, h, AllPlanes,
 			mono ? XYPixmap : ZPixmap );
     CHECK_PTR( xi );
 
@@ -754,7 +685,7 @@ QImage QPixmap::convertToImage() const
 			y = h;
 			pixel = 0;		// eliminate compiler warning
 #if defined(CHECK_RANGE)
-			warning( "QPixmap::convertToImage: Invalid depth %d",
+			qWarning( "QPixmap::convertToImage: Invalid depth %d",
 				 bppc );
 #endif
 		}
@@ -801,7 +732,7 @@ QImage QPixmap::convertToImage() const
     } else {
 	/* Typically 2 or 4 bits display depth */
 #if defined(CHECK_RANGE)
-	warning( "QPixmap::convertToImage: Display not supported (bpp=%d)",
+	qWarning( "QPixmap::convertToImage: Display not supported (bpp=%d)",
 		 xi->bits_per_pixel );
 #endif
 	image.reset();
@@ -864,7 +795,8 @@ QImage QPixmap::convertToImage() const
 	XColor *carr = new XColor[ncells];
 	for ( i=0; i<ncells; i++ )
 	    carr[i].pixel = i;
-	XQueryColors( dpy, cmap, carr, ncells );// get default colormap
+	// Get default colormap
+	XQueryColors( x11Display(), cmap, carr, ncells );
 
 	if (msk) {
 	    int trans;
@@ -907,7 +839,7 @@ QImage QPixmap::convertToImage() const
 	}
 	delete [] carr;
     }
-    if ( data->opt == NoOptim )			// throw away image data
+    if ( data->optim != BestOptim )		// throw away image data
 	qSafeXDestroyImage( xi );
     else					// keep ximage data
 	((QPixmap*)this)->data->ximage = xi;
@@ -957,7 +889,7 @@ QImage QPixmap::convertToImage() const
 		This is the default when converting to a pixmap.
      <li> \c AvoidDither - only dither 32-bit images if
 		the image
-		has more than 256 colours and it
+		has more than 256 colors and it
 		is being converted to 8-bits.
 		This is the default when an image is converted
 		for the purpose of saving to a file.
@@ -969,6 +901,11 @@ QImage QPixmap::convertToImage() const
   Note that even though a QPixmap with depth 1 behaves much like a
   QBitmap, isQBitmap() returns FALSE.
 
+  If a pixmap with depth 1 is painted with color0 and color1 and
+  converted to an image, the pixels painted with color0 will produce
+  pixel index 0 in the image and those painted with color1 will produce
+  pixel index 1.
+
   \bug Does not support 2 or 4 bit display hardware.
 
   \sa convertToImage(), isQBitmap(), QImage::convertDepth(), defaultDepth(),
@@ -979,7 +916,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 {
     if ( img.isNull() ) {
 #if defined(CHECK_NULL)
-	warning( "QPixmap::convertFromImage: Cannot convert a null image" );
+	qWarning( "QPixmap::convertFromImage: Cannot convert a null image" );
 #endif
 	return FALSE;
     }
@@ -1028,7 +965,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 
     if ( d == 1 ) {				// 1 bit pixmap (bitmap)
 	if ( hd )				// delete old X pixmap
-	    XFreePixmap( dpy, hd );
+	    XFreePixmap( x11Display(), hd );
 	char  *bits;
 	uchar *tmp_bits;
 	int    bpl = (w+7)/8;
@@ -1060,7 +997,9 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	    bits = (char *)image.bits();
 	    tmp_bits = 0;
 	}
-	hd = (HANDLE)XCreateBitmapFromData( dpy, DefaultRootWindow(dpy), bits, w, h );
+	hd = (HANDLE)XCreateBitmapFromData( x11Display(),
+					    DefaultRootWindow(x11Display()),
+					    bits, w, h );
 	if ( tmp_bits )				// Avoid purify complaint
 	    delete [] tmp_bits;
 	data->w = w;  data->h = h;  data->d = 1;
@@ -1073,6 +1012,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	return TRUE;
     }
 
+    Display *dpy   = x11Display();
     Visual *visual = (Visual *)x11Visual();
     XImage *xi	   = 0;
     bool    trucol = visual->c_class == TrueColor;
@@ -1089,6 +1029,9 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	int   red_shift	  = highest_bit( red_mask )   - 7;
 	int   green_shift = highest_bit( green_mask ) - 7;
 	int   blue_shift  = highest_bit( blue_mask )  - 7;
+	uint  rbits = highest_bit(red_mask) - lowest_bit(red_mask) + 1;
+	uint  gbits = highest_bit(green_mask) - lowest_bit(green_mask) + 1;
+	uint  bbits = highest_bit(blue_mask) - lowest_bit(blue_mask) + 1;
 	int   r, g, b;
 
 	if ( d8 ) {				// setup pixel translation
@@ -1115,6 +1058,54 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	if ( bppc > 8 && xi->byte_order == LSBFirst )
 	    bppc++;
 
+	bool contig_bits = n_bits(red_mask) == rbits &&
+			   n_bits(green_mask) == gbits &&
+			   n_bits(blue_mask) == bbits;
+	bool dither_tc =
+		// Want it
+		(conversion_flags & DitherMode_Mask) != AvoidDither &&
+		// Need it
+		bppc < 24 && !d8 &&
+		// Contiguous bits?
+		contig_bits;
+
+	static bool init=FALSE;
+	static int D[16][16];
+	if ( dither_tc && !init ) {
+	    // I also contributed this code to XV - WWA.
+	    /*
+	    The dither matrix, D, is obtained with this formula:
+
+	    D2 = [ 0 2 ]
+		 [ 3 1 ]
+
+
+	    D2*n = [ 4*Dn       4*Dn+2*Un ]
+		   [ 4*Dn+3*Un  4*Dn+1*Un ]
+	    */
+	    int n,i,j;
+	    init=1;
+
+	    /* Set D2 */
+	    D[0][0]=0;
+	    D[1][0]=2;
+	    D[0][1]=3;
+	    D[1][1]=1;
+
+	    /* Expand using recursive definition given above */
+	    for (n=2; n<16; n*=2) {
+		for (i=0; i<n; i++) {
+		    for (j=0; j<n; j++) {
+		        D[i][j]*=4;
+		        D[i+n][j]=D[i][j]+2;
+		        D[i][j+n]=D[i][j]+3;
+		        D[i+n][j+n]=D[i][j]+1;
+		    }
+		}
+	    }
+	    init=TRUE;
+	}
+
 	for ( int y=0; y<h; y++ ) {
 	    QRgb *p;
 	    src = image.scanLine( y );
@@ -1127,9 +1118,28 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 		    r = qRed  ( *p );
 		    g = qGreen( *p );
 		    b = qBlue ( *p++ );
-		    r = red_shift   > 0 ? r << red_shift   : r >> -red_shift;
-		    g = green_shift > 0 ? g << green_shift : g >> -green_shift;
-		    b = blue_shift  > 0 ? b << blue_shift  : b >> -blue_shift;
+
+		    if ( dither_tc ) {
+			// Dither truecolor
+			int thres = D[x%16][y%16];
+			if ( r <= (255-(1<<(8-rbits))) && ((r<<rbits) & 255)
+				> thres)
+			    r += (1<<(8-rbits));
+			if ( g <= (255-(1<<(8-gbits))) && ((g<<gbits) & 255)
+				> thres)
+			    g += (1<<(8-gbits));
+			if ( b <= (255-(1<<(8-bbits))) && ((b<<bbits) & 255)
+				> thres)
+			    b += (1<<(8-bbits));
+		    }
+
+		    r = red_shift   > 0
+			? r << red_shift   : r >> -red_shift;
+		    g = green_shift > 0
+			? g << green_shift : g >> -green_shift;
+		    b = blue_shift  > 0
+			? b << blue_shift  : b >> -blue_shift;
+
 		    pixel = (b & blue_mask)|(g & green_mask) | (r & red_mask);
 		}
 		switch ( bppc ) {
@@ -1312,7 +1322,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	    newbits = (uchar *)newerbits;
 	} else if ( xi->bits_per_pixel != 8 ) {
 #if defined(CHECK_RANGE)
-	    warning( "QPixmap::convertFromImage: Display not supported "
+	    qWarning( "QPixmap::convertFromImage: Display not supported "
 		     "(bpp=%d)", xi->bits_per_pixel );
 #endif
 	}
@@ -1324,15 +1334,19 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	hd = 0;
     }
     if ( !hd )					// create new pixmap
-	hd = (HANDLE)XCreatePixmap( dpy, DefaultRootWindow(dpy), w, h, dd );
+	hd = (HANDLE)XCreatePixmap( x11Display(),
+				    DefaultRootWindow(x11Display()),
+				    w, h, dd );
 
     XPutImage( dpy, hd, qt_xget_readonly_gc(), xi, 0, 0, 0, 0, w, h );
-    if ( data->opt == NoOptim ) {		// throw away image
+    if ( data->optim != BestOptim ) {		// throw away image
 	qSafeXDestroyImage( xi );
     } else {					// keep ximage that we created
 	data->ximage = xi;
     }
-    data->w = w;  data->h = h;	data->d = dd;
+    data->w = w;
+    data->h = h;
+    data->d = dd;
 
     if ( img.hasAlphaBuffer() ) {
 	QBitmap m;
@@ -1348,7 +1362,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
   Grabs the contents of a window and makes a pixmap out of it.
   Returns the pixmap.
 
-  The argments \e (x,y) specify the offset in the window, while
+  The arguments \e (x,y) specify the offset in the window, while
   \e (w,h) specify the width and height of the area to be copied.
 
   If \e w is negative, the function copies everything to the right
@@ -1369,8 +1383,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 
 QPixmap QPixmap::grabWindow( WId window, int x, int y, int w, int h )
 {
-    if ( !x11DefaultVisual() )			// incompatible depth
-	return QPixmap(0, 0);
+    Display *dpy = x11AppDisplay();
     if ( w <= 0 || h <= 0 ) {
 	if ( w == 0 || h == 0 ) {
 	    QPixmap nullPixmap;
@@ -1394,8 +1407,7 @@ QPixmap QPixmap::grabWindow( WId window, int x, int y, int w, int h )
 
 
 /*!
-  Transforms the pixmap using \e matrix, and returns the transformed
-  pixmap.
+  Returns a copy of the pixmap that is transformed using \e matrix.
 
   Qt uses this function to implement rotated text on window systems
   that do not support such complex features.
@@ -1407,7 +1419,7 @@ QPixmap QPixmap::grabWindow( WId window, int x, int y, int w, int h )
     QPixmap  pm( 8, 8 );
     QPainter p;
     QRect    r;				// text bounding rectangle
-    QPoint   bl;			// text baselink position
+    QPoint   bl;			// text baseline position
 
     p.begin( &pm );			// first get the bounding
     p.setFont( f );			//   text rectangle
@@ -1463,7 +1475,8 @@ QPixmap QPixmap::grabWindow( WId window, int x, int y, int w, int h )
 
 QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
 {
-    int	   w, h;				// size of target pixmap
+    int	   w = 0;
+    int	   h = 0;				// size of target pixmap
     int	   ws, hs;				// size of source pixmap
     uchar *dptr;				// data in target pixmap
     int	   dbpl, dbytes;			// bytes per line/bytes total
@@ -1472,6 +1485,7 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
     int	   bpp;					// bits per pixel
     bool   depth1 = depth() == 1;
     int	   y;
+    Display *dpy = x11Display();
 
     if ( isNull() )				// this is a null pixmap
 	return copy();
@@ -1479,21 +1493,21 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
     ws = width();
     hs = height();
 
-    const float dt = 0.0001;
-    float x1,y1, x2,y2, x3,y3, x4,y4;		// get corners
-    float xx = (float)ws - 1;
-    float yy = (float)hs - 1;
+    const double dt = 0.0001;
+    double x1,y1, x2,y2, x3,y3, x4,y4;		// get corners
+    double xx = (double)ws - 1;
+    double yy = (double)hs - 1;
 
     matrix.map( dt, dt, &x1, &y1 );
     matrix.map( xx, dt, &x2, &y2 );
     matrix.map( xx, yy, &x3, &y3 );
     matrix.map( dt, yy, &x4, &y4 );
 
-    float ymin = y1;				// lowest y value
+    double ymin = y1;				// lowest y value
     if ( y2 < ymin ) ymin = y2;
     if ( y3 < ymin ) ymin = y3;
     if ( y4 < ymin ) ymin = y4;
-    float xmin = x1;				// lowest x value
+    double xmin = x1;				// lowest x value
     if ( x2 < xmin ) xmin = x2;
     if ( x3 < xmin ) xmin = x3;
     if ( x4 < xmin ) xmin = x4;
@@ -1537,7 +1551,7 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
 #endif
     XImage *xi = (XImage*)data->ximage;		// any cached ximage?
     if ( !xi )
-	xi = XGetImage( dpy, handle(), 0, 0, ws, hs, AllPlanes,
+	xi = XGetImage( x11Display(), handle(), 0, 0, ws, hs, AllPlanes,
 			depth1 ? XYPixmap : ZPixmap );
 
     if ( !xi ) {				// error, return null pixmap
@@ -1569,7 +1583,7 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
 	if ( depth1 )				// fill with zeros
 	    memset( dptr, 0, dbytes );
 	else if ( bpp == 8 )			// fill with background color
-	    memset( dptr, white.pixel(), dbytes );
+	    memset( dptr, Qt::white.pixel(), dbytes );
 	else
 	    memset( dptr, 0xff, dbytes );
 #if defined(MITSHM)
@@ -1578,17 +1592,17 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
 
 // #define DEBUG_XIMAGE
 #if defined(DEBUG_XIMAGE)
-    debug( "----IMAGE--INFO--------------" );
-    debug( "width............. %d", xi->width );
-    debug( "height............ %d", xi->height );
-    debug( "xoffset........... %d", xi->xoffset );
-    debug( "format............ %d", xi->format );
-    debug( "byte order........ %d", xi->byte_order );
-    debug( "bitmap unit....... %d", xi->bitmap_unit );
-    debug( "bitmap bit order.. %d", xi->bitmap_bit_order );
-    debug( "depth............. %d", xi->depth );
-    debug( "bytes per line.... %d", xi->bytes_per_line );
-    debug( "bits per pixel.... %d", xi->bits_per_pixel );
+    qDebug( "----IMAGE--INFO--------------" );
+    qDebug( "width............. %d", xi->width );
+    qDebug( "height............ %d", xi->height );
+    qDebug( "xoffset........... %d", xi->xoffset );
+    qDebug( "format............ %d", xi->format );
+    qDebug( "byte order........ %d", xi->byte_order );
+    qDebug( "bitmap unit....... %d", xi->bitmap_unit );
+    qDebug( "bitmap bit order.. %d", xi->bitmap_bit_order );
+    qDebug( "depth............. %d", xi->depth );
+    qDebug( "bytes per line.... %d", xi->bytes_per_line );
+    qDebug( "bits per pixel.... %d", xi->bits_per_pixel );
 #endif
 
     int m11 = qRound((double)mat.m11()*65536.0);
@@ -1674,7 +1688,7 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
 
 		default: {
 #if defined(CHECK_RANGE)
-		warning( "QPixmap::xForm: DISPLAY NOT SUPPORTED (BPP=%d)",bpp);
+		qWarning( "QPixmap::xForm: DISPLAY NOT SUPPORTED (BPP=%d)",bpp);
 #endif
 		QPixmap pm;
 		return pm;
@@ -1727,7 +1741,7 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
 	m22ydy += m22;
 	p += p_inc;
     }
-    if ( data->opt == NoOptim ) {		// throw away ximage
+    if ( data->optim == NoOptim ) {		// throw away ximage
 	qSafeXDestroyImage( xi );
     } else {					// keep ximage that we fetched
 	data->ximage = xi;
@@ -1784,21 +1798,21 @@ QPixmap QPixmap::xForm( const QWMatrix &matrix ) const
 
 QWMatrix QPixmap::trueMatrix( const QWMatrix &matrix, int w, int h )
 {
-    const float dt = 0.0001;
-    float x1,y1, x2,y2, x3,y3, x4,y4;		// get corners
-    float xx = (float)w - 1;
-    float yy = (float)h - 1;
+    const double dt = (double)0.0001;
+    double x1,y1, x2,y2, x3,y3, x4,y4;		// get corners
+    double xx = (double)w - 1;
+    double yy = (double)h - 1;
 
     matrix.map( dt, dt, &x1, &y1 );
     matrix.map( xx, dt, &x2, &y2 );
     matrix.map( xx, yy, &x3, &y3 );
     matrix.map( dt, yy, &x4, &y4 );
 
-    float ymin = y1;				// lowest y value
+    double ymin = y1;				// lowest y value
     if ( y2 < ymin ) ymin = y2;
     if ( y3 < ymin ) ymin = y3;
     if ( y4 < ymin ) ymin = y4;
-    float xmin = x1;				// lowest x value
+    double xmin = x1;				// lowest x value
     if ( x2 < xmin ) xmin = x2;
     if ( x3 < xmin ) xmin = x3;
     if ( x4 < xmin ) xmin = x4;
