@@ -1,41 +1,43 @@
 /****************************************************************************
-** $Id: qpoint.cpp,v 2.1 1997/08/06 15:05:42 hanord Exp $
+** $Id: qpoint.cpp,v 2.16 1999/05/31 10:26:48 warwick Exp $
 **
 ** Implementation of QPoint class
 **
 ** Created : 931028
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
-#define QPOINT_C
 #include "qpoint.h"
-#include "qdstream.h"
-
-RCSTAG("$Id: qpoint.cpp,v 2.1 1997/08/06 15:05:42 hanord Exp $");
+#include "qdatastream.h"
 
 
 /*!
   \class QPoint qpoint.h
   \brief The QPoint class defines a point in the plane.
 
+  \ingroup drawing
+
   A point is specified by an x coordinate and a y coordinate.
 
-  The coordinate type is QCOORD (defined in qwindefs.h as \c short).
-  The minimum value of QCOORD is QCOORD_MIN (-32768) and the maximum
-  value is  QCOORD_MAX (32767).
+  The coordinate type is QCOORD (defined in qwindowdefs.h as \c int).
+  The minimum value of QCOORD is QCOORD_MIN (-2147483648) and the maximum
+  value is  QCOORD_MAX (2147483647).
 
   We have defined many operator functions that make arithmetic on points
   simple and intuitive.
@@ -58,7 +60,7 @@ RCSTAG("$Id: qpoint.cpp,v 2.1 1997/08/06 15:05:42 hanord Exp $");
 
 /*!
   \fn QPoint::QPoint()
-  Constructs a point with undefined x and y values.
+  Constructs a point (0,0) (a \link QPoint::isNull() null point\endlink).
 */
 
 /*!
@@ -233,8 +235,9 @@ RCSTAG("$Id: qpoint.cpp,v 2.1 1997/08/06 15:05:42 hanord Exp $");
   Returns \e p where x and y have opposite signs.
 */
 
-
 /*!
+  \fn QPoint &QPoint::operator/=( int c )
+
   Divides both x and y by \e c, and return a reference to this point.
 
   Example:
@@ -244,17 +247,9 @@ RCSTAG("$Id: qpoint.cpp,v 2.1 1997/08/06 15:05:42 hanord Exp $");
   \endcode
 */
 
-QPoint &QPoint::operator/=( int c )
-{
-    if ( c == 0 )
-#if defined(CHECK_MATH)
-	warning( "QPoint: Attempt to divide point by zero" );
-#endif
-    xp/=(QCOORD)c; yp/=(QCOORD)c;
-    return *this;
-}
-
 /*!
+  \fn QPoint &QPoint::operator/=( double c )
+
   Divides both x and y by \e c, and return a reference to this point.
 
   Example:
@@ -266,44 +261,27 @@ QPoint &QPoint::operator/=( int c )
   Note that the result is truncated.
 */
 
-QPoint &QPoint::operator/=( double c )
-{
-    if ( c == 0.0 )
-#if defined(CHECK_MATH)
-	warning( "QPoint: Attempt to divide point by zero" );
-#endif
-    xp=(QCOORD)(xp/c); yp=(QCOORD)(yp/c);
-    return *this;
-}
-
 /*!
+  \fn QPoint operator/( const QPoint &p, int c )
   \relates QPoint
   Divides both of \e p's components by \e c and returns the result.
 */
 
-QPoint operator/( const QPoint &p, int c )
-{
-    if ( c == 0 )
-#if defined(CHECK_MATH)
-	warning( "QPoint: Attempt to divide point by zero" );
-#endif
-    return QPoint( p.xp/c, p.yp/c );
-}
-
 /*!
+  \fn QPoint operator/( const QPoint &p, double c )
   \relates QPoint
+
   Divides both of \e p's components by \e c and returns the result.
 
   Note that the result is truncated.
 */
 
-QPoint operator/( const QPoint &p, double c )
+
+void QPoint::warningDivByZero()
 {
-    if ( c == 0.0 )
 #if defined(CHECK_MATH)
-	warning( "QPoint: Attempt to divide point by zero" );
+    qWarning( "QPoint: Division by zero error" );
 #endif
-    return QPoint( (QCOORD)(p.xp/c), (QCOORD)(p.yp/c) );
 }
 
 
@@ -315,12 +293,16 @@ QPoint operator/( const QPoint &p, double c )
   \relates QPoint
   Writes a QPoint to the stream and returns a reference to the stream.
 
-  Serialization format: [x (INT16), y (INT16)].
+  Serialization format: [x (Q_INT32), y (Q_INT32)].
 */
 
 QDataStream &operator<<( QDataStream &s, const QPoint &p )
 {
-    return s << (INT16)p.x() << (INT16)p.y();
+    if ( s.version() == 1 )
+	s << (Q_INT16)p.x() << (Q_INT16)p.y();
+    else
+	s << (Q_INT32)p.x() << (Q_INT32)p.y();
+    return s;
 }
 
 /*!
@@ -330,8 +312,30 @@ QDataStream &operator<<( QDataStream &s, const QPoint &p )
 
 QDataStream &operator>>( QDataStream &s, QPoint &p )
 {
-    INT16 x, y;
-    s >> x;  p.rx() = x;
-    s >> y;  p.ry() = y;
+    if ( s.version() == 1 ) {
+	Q_INT16 x, y;
+	s >> x;  p.rx() = x;
+	s >> y;  p.ry() = y;
+    }
+    else {
+	Q_INT32 x, y;
+	s >> x;  p.rx() = x;
+	s >> y;  p.ry() = y;
+    }
     return s;
 }
+
+/*!
+  Returns the sum of the absolute values of x() and y(), traditionally
+  known as the "Manhattan length" of the vector from the origin to the
+  point. The tradition arises since such distances apply
+  to travelers who can only travel on a rectangular grid, like the streets
+  of Manhattan.
+
+  This is a useful approximation to the true length,
+  sqrt(pow(x(),2)+pow(y(),2)).
+*/
+int QPoint::manhattanLength() const
+{
+    return QABS(x())+QABS(y());
+}
