@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qrect.cpp,v 2.9 1998/07/03 00:09:40 hanord Exp $
+** $Id: qrect.cpp,v 2.18.2.1 1999/07/01 06:07:53 warwick Exp $
 **
 ** Implementation of QRect class
 **
@@ -7,17 +7,19 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -35,11 +37,11 @@
   bottom right corner, but it is normally expressed as an upper left
   corner and a size.
 
-  The coordinate type is QCOORD (defined in qwindowdefs.h as \c short).
-  The minimum value of QCOORD is QCOORD_MIN (-32768) and the maximum
-  value is  QCOORD_MAX (32767).
+  The coordinate type is QCOORD (defined in qwindowdefs.h as \c int).
+  The minimum value of QCOORD is QCOORD_MIN (-2147483648) and the maximum
+  value is  QCOORD_MAX (2147483647).
 
-  Note that the size (width and height) of a rectange might be
+  Note that the size (width and height) of a rectangle might be
   different from what you are used to. If the top left corner and the
   bottom right corner are the same, then the height and the width of
   the rectangle will both be 1.
@@ -214,6 +216,38 @@ QRect QRect::normalize() const
   \sa top(), setBottom(), bottomLeft(), bottomRight()
 */
 
+/*!
+  \fn QCOORD &QRect::rLeft()
+
+  Returns the reference to the left coordinate of the rectangle.
+
+  \sa rTop(), rRight(), rBottom()
+*/
+
+/*!
+  \fn QCOORD &QRect::rTop()
+
+  Returns the reference to the top coordinate of the rectangle.
+
+  \sa rLeft(),  rRight(), rBottom()
+*/
+
+/*!
+  \fn QCOORD &QRect::rRight()
+
+  Returns the reference to the right coordinate of the rectangle.
+
+  \sa rLeft(), rTop(), rBottom()
+*/
+
+/*!
+  \fn QCOORD &QRect::rBottom()
+
+  Returns the reference to the bottom coordinate of the rectangle.
+
+  \sa rLeft(), rTop(), rRight()
+*/
+
 /*!
   \fn int QRect::x() const
 
@@ -561,14 +595,35 @@ bool QRect::contains( const QRect &r, bool proper ) const
 }
 
 /*!
-  Returns the union rectangle of this rectangle and \e r.  The union
+  Unites this rectangle with \a r.
+*/
+QRect& QRect::operator|=(const QRect &r)
+{
+    *this = *this | r;
+    return *this;
+}
+
+/*!
+  Intersects this rectangle with \a r.
+*/
+QRect& QRect::operator&=(const QRect &r)
+{
+    *this = *this & r;
+    return *this;
+}
+
+
+/*!
+  Returns the bounding rectangle of this and \a r.
+
+  The bounding
   rectangle of a nonempty rectangle and an empty or invalid rectangle
   is defined to be the nonempty rectangle.
 
-  \sa intersect(), intersects(), contains()
+  \sa operator|=(), operator&(), intersects(), contains()
 */
 
-QRect QRect::unite( const QRect &r ) const
+QRect QRect::operator|(const QRect &r) const
 {
     if ( isValid() ) {
 	if ( r.isValid() ) {
@@ -586,16 +641,24 @@ QRect QRect::unite( const QRect &r ) const
     }
 }
 
+/*!
+  \code r.unite(s) \endcode is equivalent to \code r|s\endcode.
+*/
+QRect QRect::unite( const QRect &r ) const
+{
+    return *this | r;
+}
+
 
 /*!
-  Returns the intersection rectangle of this rectangle and \e r.
+  Returns the intersection of this rectangle and \e r.
 
   Returns an empty rectangle if there is no intersection.
 
-  \sa isEmpty(), intersects(), unite(), contains()
+  \sa operator&=(), operator|(), isEmpty(), intersects(), contains()
 */
 
-QRect QRect::intersect( const QRect &r ) const
+QRect QRect::operator&( const QRect &r ) const
 {
     QRect tmp;
     tmp.x1 = QMAX( x1, r.x1 );
@@ -605,6 +668,14 @@ QRect QRect::intersect( const QRect &r ) const
     return tmp;
 }
 
+/*!
+  \code r.intersect(s) \endcode is equivalent to \code r&s\endcode.
+*/
+QRect QRect::intersect( const QRect &r ) const
+{
+    return *this & r;
+}
+
 /*!
   Returns TRUE if this rectangle intersects with \e r (there is at
   least one pixel which is within both rectangles).
@@ -648,14 +719,19 @@ bool operator!=( const QRect &r1, const QRect &r2 )
 
   Writes a QRect to the stream and returns a reference to the stream.
 
-  Serialization format: [left (INT16), top (INT16), right (INT16),
-  bottom (INT16)].
+  Serialization format: [left (Q_INT32), top (Q_INT32), right (Q_INT32),
+  bottom (Q_INT32)].
 */
 
 QDataStream &operator<<( QDataStream &s, const QRect &r )
 {
-    return s << (INT16)r.left() << (INT16)r.top()
-	     << (INT16)r.right() << (INT16)r.bottom();
+    if ( s.version() == 1 )
+	s << (Q_INT16)r.left() << (Q_INT16)r.top()
+	  << (Q_INT16)r.right() << (Q_INT16)r.bottom();
+    else
+	s << (Q_INT32)r.left() << (Q_INT32)r.top()
+	  << (Q_INT32)r.right() << (Q_INT32)r.bottom();
+    return s;
 }
 
 /*!
@@ -666,8 +742,15 @@ QDataStream &operator<<( QDataStream &s, const QRect &r )
 
 QDataStream &operator>>( QDataStream &s, QRect &r )
 {
-    INT16 x1, y1, x2, y2;
-    s >> x1; s >> y1; s >> x2; s >> y2;
-    r.setCoords( x1, y1, x2, y2 );
+    if ( s.version() == 1 ) {
+	Q_INT16 x1, y1, x2, y2;
+	s >> x1; s >> y1; s >> x2; s >> y2;
+	r.setCoords( x1, y1, x2, y2 );
+    }
+    else {
+	Q_INT32 x1, y1, x2, y2;
+	s >> x1; s >> y1; s >> x2; s >> y2;
+	r.setCoords( x1, y1, x2, y2 );
+    }
     return s;
 }
