@@ -1,32 +1,37 @@
 /****************************************************************************
-** $Id: qrect.h,v 2.5 1997/09/04 12:33:32 warwick Exp $
+** $Id: qrect.h,v 2.14 1999/04/28 09:57:27 aavit Exp $
 **
 ** Definition of QRect class
 **
 ** Created : 931028
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #ifndef QRECT_H
 #define QRECT_H
 
+#ifndef QT_H
 #include "qsize.h"
+#endif // QT_H
 
 
-class QRect					// rectangle class
+class Q_EXPORT QRect					// rectangle class
 {
 public:
     QRect()	{ x1 = y1 = 0; x2 = y2 = -1; }
@@ -43,6 +48,12 @@ public:
     int	   top()	const;
     int	   right()	const;
     int	   bottom()	const;
+    
+    QCOORD &rLeft();
+    QCOORD &rTop();
+    QCOORD &rRight();
+    QCOORD &rBottom();
+	
     int	   x()		const;
     int	   y()		const;
     void   setLeft( int pos );
@@ -78,14 +89,19 @@ public:
     void   setHeight( int h );
     void   setSize( const QSize &s );
 
+    QRect  operator|(const QRect &r) const;
+    QRect  operator&(const QRect &r) const;
+    QRect&  operator|=(const QRect &r);
+    QRect&  operator&=(const QRect &r);
+
     bool   contains( const QPoint &p, bool proper=FALSE ) const;
     bool   contains( const QRect &r, bool proper=FALSE ) const;
     QRect  unite( const QRect &r ) const;
     QRect  intersect( const QRect &r ) const;
     bool   intersects( const QRect &r ) const;
 
-    friend bool operator==( const QRect &, const QRect & );
-    friend bool operator!=( const QRect &, const QRect & );
+    friend Q_EXPORT bool operator==( const QRect &, const QRect & );
+    friend Q_EXPORT bool operator!=( const QRect &, const QRect & );
 
 private:
 #if defined(_OS_MAC_)
@@ -101,15 +117,16 @@ private:
 #endif
 };
 
-bool operator==( const QRect &, const QRect & );
-bool operator!=( const QRect &, const QRect & );
+Q_EXPORT bool operator==( const QRect &, const QRect & );
+Q_EXPORT bool operator!=( const QRect &, const QRect & );
+
 
 /*****************************************************************************
   QRect stream functions
  *****************************************************************************/
 
-QDataStream &operator<<( QDataStream &, const QRect & );
-QDataStream &operator>>( QDataStream &, QRect & );
+Q_EXPORT QDataStream &operator<<( QDataStream &, const QRect & );
+Q_EXPORT QDataStream &operator>>( QDataStream &, QRect & );
 
 
 /*****************************************************************************
@@ -145,6 +162,18 @@ inline int QRect::right() const
 inline int QRect::bottom() const
 { return y2; }
 
+inline QCOORD &QRect::rLeft()
+{ return x1; }
+
+inline QCOORD & QRect::rTop()
+{ return y1; }
+
+inline QCOORD & QRect::rRight()
+{ return x2; }
+
+inline QCOORD & QRect::rBottom()
+{ return y2; }
+
 inline int QRect::x() const
 { return x1; }
 
