@@ -1,31 +1,32 @@
 /****************************************************************************
-** $Id: qtimer.cpp,v 2.4 1997/08/06 15:05:47 hanord Exp $
+** $Id: qtimer.cpp,v 2.15 1999/04/28 09:57:57 aavit Exp $
 **
 ** Implementation of QTimer class
 **
 ** Created : 931111
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qtimer.h"
 #include "qsignal.h"
-#include "qobjcoll.h"
-
-RCSTAG("$Id: qtimer.cpp,v 2.4 1997/08/06 15:05:47 hanord Exp $");
-
+#include "qobjectlist.h"
+#include "qobjectdict.h"
 
 /*!
   \class QTimer qtimer.h
@@ -52,12 +53,12 @@ RCSTAG("$Id: qtimer.cpp,v 2.4 1997/08/06 15:05:47 hanord Exp $");
 
   As a special case, a QTimer with timeout 0 times out as soon as all
   the events in the window system's event queue have been processed.
-
+  
   This can be used to do heavy work while providing a snappy
   user interface: \code
     QTimer *t = new QTimer( myObject );
     connect( t, SIGNAL(timeout()), SLOT(processOneThing()) );
-    t->start( 0, TRUE );
+    t->start( 0, FALSE );
   \endcode
 
   myObject->processOneThing() will be called repeatedly and should
@@ -68,12 +69,16 @@ RCSTAG("$Id: qtimer.cpp,v 2.4 1997/08/06 15:05:47 hanord Exp $");
   on more and more platforms and we expect that null events will
   eventually be replaced by threading.
 
-  An alternative is to call QObject::startTimer() for your object and
-  reimplement the QObject::timerEvent() event handler in your class
-  (it must inherit QObject).  The advantage is that you can have
-  multiple running timers, each having a unique identifier.  The
-  disadvantage is that it does not support such high-level features as
-  single-shot timers or signals.
+  Note that QTimer's accuracy depends on the underlying operating
+  system and hardware.  Most platforms support an accuracy of 20ms;
+  some provide more.  If Qt is unable to deliver the requested number
+  of timer clicks, it will silently discard some.
+
+  An alternative to using QTimer is to call QObject::startTimer() for
+  your object and reimplement the QObject::timerEvent() event handler
+  in your class (which must of course inherit QObject).  The
+  disadvantage is that timerEvent() does not support such high-level
+  features as single-shot timers or signals.
 */
 
 
@@ -90,7 +95,6 @@ const int INV_TIMER = -1;			// invalid timer id
 QTimer::QTimer( QObject *parent, const char *name )
     : QObject( parent, name )
 {
-    initMetaObject();
     id = INV_TIMER;
 }
 
@@ -176,7 +180,7 @@ void QTimer::stop()
 
 bool QTimer::event( QEvent *e )
 {
-    if ( e->type() != Event_Timer )		// ignore all other events
+    if ( e->type() != QEvent::Timer )		// ignore all other events
 	return FALSE;
     if ( single )				// stop single shot timer
 	stop();
@@ -215,7 +219,7 @@ void sst_init()
 class QSingleShotTimer : public QObject
 {
 public:
-    bool    start( int msec, QObject *r, const char *m );
+    bool    start( int msec, QObject *r, const char * m );
 protected:
     bool    event( QEvent * );
 private:
@@ -253,7 +257,7 @@ bool QSingleShotTimer::event( QEvent * )
 
   Example:
   \code
-    #include <qapp.h>
+    #include <qapplication.h>
     #include <qtimer.h>
 
     int main( int argc, char **argv )
