@@ -1,41 +1,43 @@
 /****************************************************************************
-** $Id: qwmatrix.cpp,v 2.1 1997/08/06 15:05:49 hanord Exp $
+** $Id: qwmatrix.cpp,v 2.12 1999/05/20 08:00:58 aavit Exp $
 **
 ** Implementation of QWMatrix class
 **
 ** Created : 941020
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qwmatrix.h"
-#include "qdstream.h"
+#include "qdatastream.h"
 #if defined(_WS_X11_)
-double qsincos( double, bool calcCos );		// defined in qptr_x11.cpp
+double qsincos( double, bool calcCos );		// defined in qpainter_x11.cpp
 #else
 #include <math.h>
 #endif
 
-RCSTAG("$Id: qwmatrix.cpp,v 2.1 1997/08/06 15:05:49 hanord Exp $");
-
-
 /*!
   \class QWMatrix qwmatrix.h
   \brief The QWMatrix class specifies 2D transformations of a
   coordinate system.
 
+  \ingroup drawing
+
   The standard coordinate system of a \link QPaintDevice paint
   device\endlink has the origin located at the top left position. X
   values increase to the right, and Y values increase downwards.
@@ -113,9 +115,9 @@ RCSTAG("$Id: qwmatrix.cpp,v 2.1 1997/08/06 15:05:49 hanord Exp $");
 
   The same example, but using basic matrix operations:
   \code
-    float a    = pi/180 * 25;			// convert 25 to radians
-    float sina = sin(a);
-    float cosa = cos(a);
+    double a    = pi/180 * 25;			// convert 25 to radians
+    double sina = sin(a);
+    double cosa = cos(a);
     QWMatrix m1(0, 0, 0, 0, 10, -20);		// translation matrix
     QWMatrix m2( cosa, sina,			// rotation matrix
 		-sina, cosa, 0, 0 );
@@ -147,16 +149,16 @@ RCSTAG("$Id: qwmatrix.cpp,v 2.1 1997/08/06 15:05:49 hanord Exp $");
 
 QWMatrix::QWMatrix()
 {
-    _m11 = _m22 = (float)1.0;
-    _m12 = _m21 = _dx = _dy = (float)0.0;
+    _m11 = _m22 = 1.0;
+    _m12 = _m21 = _dx = _dy = 0.0;
 }
 
 /*!
   Constructs a matrix with the specified elements.
 */
 
-QWMatrix::QWMatrix( float m11, float m12, float m21, float m22,
-		    float dx, float dy )
+QWMatrix::QWMatrix( double m11, double m12, double m21, double m22,
+		    double dx, double dy )
 {
     _m11 = m11;	 _m12 = m12;
     _m21 = m21;	 _m22 = m22;
@@ -168,8 +170,8 @@ QWMatrix::QWMatrix( float m11, float m12, float m21, float m22,
   Sets the matrix elements to the specified values.
 */
 
-void QWMatrix::setMatrix( float m11, float m12, float m21, float m22,
-			  float dx, float dy )
+void QWMatrix::setMatrix( double m11, double m12, double m21, double m22,
+			  double dx, double dy )
 {
     _m11 = m11;	 _m12 = m12;
     _m21 = m21;	 _m22 = m22;
@@ -178,32 +180,32 @@ void QWMatrix::setMatrix( float m11, float m12, float m21, float m22,
 
 
 /*!
-  \fn float QWMatrix::m11() const
+  \fn double QWMatrix::m11() const
   Returns the X scaling factor.
 */
 
 /*!
-  \fn float QWMatrix::m12() const
+  \fn double QWMatrix::m12() const
   Returns the vertical shearing factor.
 */
 
 /*!
-  \fn float QWMatrix::m21() const
+  \fn double QWMatrix::m21() const
   Returns the horizontal shearing factor.
 */
 
 /*!
-  \fn float QWMatrix::m22() const
+  \fn double QWMatrix::m22() const
   Returns the Y scaling factor.
 */
 
 /*!
-  \fn float QWMatrix::dx() const
+  \fn double QWMatrix::dx() const
   Returns the horizontal translation.
 */
 
 /*!
-  \fn float QWMatrix::dy() const
+  \fn double QWMatrix::dy() const
   Returns the vertical translation.
 */
 
@@ -217,7 +219,7 @@ void QWMatrix::setMatrix( float m11, float m12, float m21, float m22,
   \endcode
 */
 
-void QWMatrix::map( float x, float y, float *tx, float *ty ) const
+void QWMatrix::map( double x, double y, double *tx, double *ty ) const
 {
     *tx = _m11*x + _m21*y + _dx;
     *ty = _m12*x + _m22*y + _dy;
@@ -234,8 +236,10 @@ void QWMatrix::map( float x, float y, float *tx, float *ty ) const
 
 void QWMatrix::map( int x, int y, int *tx, int *ty ) const
 {
-    *tx = qRound(_m11*x + _m21*y + _dx);
-    *ty = qRound(_m12*x + _m22*y + _dy);
+    double fx = x;
+    double fy = y;
+    *tx = qRound(_m11*fx + _m21*fy + _dx);
+    *ty = qRound(_m12*fx + _m22*fy + _dy);
 }
 
 /*!
@@ -244,12 +248,14 @@ void QWMatrix::map( int x, int y, int *tx, int *ty ) const
 
 QPoint QWMatrix::map( const QPoint &p ) const
 {
-    return QPoint( qRound(_m11*p.x() + _m21*p.y() + _dx),
-		   qRound(_m12*p.x() + _m22*p.y() + _dy) );
+    double fx = p.x();
+    double fy = p.y();
+    return QPoint( qRound(_m11*fx + _m21*fy + _dx),
+		   qRound(_m12*fx + _m22*fy + _dy) );
 }
 
 /*!
-  Returns the tranformed rectangle \e r.
+  Returns the transformed rectangle \e r.
 
   If rotation or shearing has been specified, then the bounding rectangle
   will be returned.
@@ -257,13 +263,15 @@ QPoint QWMatrix::map( const QPoint &p ) const
 
 QRect QWMatrix::map( const QRect &r ) const
 {
+    QRect result;
     if ( _m12 == 0.0F && _m21 == 0.0F ) {
-	return QRect( map(r.topLeft()), map(r.bottomRight()) );
+	result = QRect( map(r.topLeft()), map(r.bottomRight()) );
     } else {
 	QPointArray a( r );
 	a = map( a );
-	return a.boundingRect();
+	result = a.boundingRect();
     }
+    return result;
 }
 
 /*!
@@ -292,8 +300,8 @@ QPointArray QWMatrix::map( const QPointArray &a ) const
 
 void QWMatrix::reset()
 {
-    _m11 = _m22 = (float)1.0;
-    _m12 = _m21 = _dx = _dy = (float)0.0;
+    _m11 = _m22 = 1.0;
+    _m12 = _m21 = _dx = _dy = 0.0;
 }
 
 
@@ -306,7 +314,7 @@ void QWMatrix::reset()
   \sa scale(), shear(), rotate()
 */
 
-QWMatrix &QWMatrix::translate( float dx, float dy )
+QWMatrix &QWMatrix::translate( double dx, double dy )
 {
     QWMatrix result( 1.0F, 0.0F, 0.0F, 1.0F, dx, dy );
     return bmul( result );
@@ -321,7 +329,7 @@ QWMatrix &QWMatrix::translate( float dx, float dy )
   \sa translate(), shear(), rotate()
 */
 
-QWMatrix &QWMatrix::scale( float sx, float sy )
+QWMatrix &QWMatrix::scale( double sx, double sy )
 {
     QWMatrix result( sx, 0.0F, 0.0F, sy, 0.0F, 0.0F );
     return bmul( result );
@@ -335,13 +343,13 @@ QWMatrix &QWMatrix::scale( float sx, float sy )
   \sa translate(), scale(), rotate()
 */
 
-QWMatrix &QWMatrix::shear( float sh, float sv )
+QWMatrix &QWMatrix::shear( double sh, double sv )
 {
     QWMatrix result( 1.0F, sv, sh, 1.0F, 0.0F, 0.0F );
     return bmul( result );
 }
 
-const float deg2rad = .017453292519943295769F;	// pi/180
+const double deg2rad = 0.017453292519943295769;	// pi/180
 
 /*!
   Rotates the coordinate system \e a degrees counterclockwise.
@@ -351,15 +359,15 @@ const float deg2rad = .017453292519943295769F;	// pi/180
   \sa translate(), scale(), shear()
 */
 
-QWMatrix &QWMatrix::rotate( float a )
+QWMatrix &QWMatrix::rotate( double a )
 {
     double b = deg2rad*a;			// convert to radians
 #if defined(_WS_X11_)
-    float sina = qsincos(b,FALSE);		// fast and convenient
-    float cosa = qsincos(b,TRUE);
+    double sina = qsincos(b,FALSE);		// fast and convenient
+    double cosa = qsincos(b,TRUE);
 #else
-    float sina = (float)sin(b);
-    float cosa = (float)cos(b);
+    double sina = sin(b);
+    double cosa = cos(b);
 #endif
     QWMatrix result( cosa, sina, -sina, cosa, 0.0F, 0.0F );
     return bmul( result );
@@ -389,10 +397,10 @@ QWMatrix QWMatrix::invert( bool *invertible ) const
 	if ( invertible )
 	    *invertible = TRUE;
 	double dinv = 1.0/det;
-	QWMatrix imatrix( (float)(_m22*dinv),	(float)(-_m12*dinv),
-			  (float)(-_m21*dinv), (float)( _m11*dinv),
-			  (float)((_m21*_dy - _m22*_dx)*dinv),
-			  (float)((_m12*_dx - _m11*_dy)*dinv) );
+	QWMatrix imatrix( (_m22*dinv),	(-_m12*dinv),
+			  (-_m21*dinv), ( _m11*dinv),
+			  ((_m21*_dy - _m22*_dx)*dinv),
+			  ((_m12*_dx - _m11*_dy)*dinv) );
 	return imatrix;
     }
 }
@@ -473,13 +481,18 @@ QWMatrix operator*( const QWMatrix &m1, const QWMatrix &m2 )
   Writes a matrix to the stream and returns a reference to the stream.
 
   Serialization format: m11, m12, m21, m22, dx and dy are serialized as
-  \c float in the listed order.
+  \c double in the listed order.
 */
 
 QDataStream &operator<<( QDataStream &s, const QWMatrix &m )
 {
-    return s << m.m11() << m.m12() << m.m21() << m.m22()
-	     << m.dx()	<< m.dy();
+    if ( s.version() == 1 )
+	s << (float)m.m11() << (float)m.m12() << (float)m.m21() 
+	  << (float)m.m22() << (float)m.dx()  << (float)m.dy();
+    else
+	s << m.m11() << m.m12() << m.m21() << m.m22()
+	  << m.dx() << m.dy();
+    return s;
 }
 
 /*!
@@ -489,9 +502,17 @@ QDataStream &operator<<( QDataStream &s, const QWMatrix &m )
 
 QDataStream &operator>>( QDataStream &s, QWMatrix &m )
 {
-    float m11, m12, m21, m22, dx, dy;
-    s >> m11;  s >> m12;  s >> m21;  s >> m22;
-    s >> dx;   s >> dy;
-    m.setMatrix( m11, m12, m21, m22, dx, dy );
+    if ( s.version() == 1 ) {
+	float m11, m12, m21, m22, dx, dy;
+	s >> m11;  s >> m12;  s >> m21;  s >> m22;
+	s >> dx;   s >> dy;
+	m.setMatrix( m11, m12, m21, m22, dx, dy );
+    }
+    else {
+	double m11, m12, m21, m22, dx, dy;
+	s >> m11;  s >> m12;  s >> m21;  s >> m22;
+	s >> dx;   s >> dy;
+	m.setMatrix( m11, m12, m21, m22, dx, dy );
+    }
     return s;
 }
