@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qdatastream.cpp,v 2.15 1998/07/03 00:09:43 hanord Exp $
+** $Id: qdatastream.cpp,v 2.23 1999/06/22 00:45:31 weis Exp $
 **
 ** Implementation of QDataStream class
 **
 ** Created : 930831
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -89,7 +91,7 @@
 #if defined(CHECK_STATE)
 #undef  CHECK_STREAM_PRECOND
 #define CHECK_STREAM_PRECOND  if ( !dev ) {				\
-				warning( "QDataStream: No device" );	\
+				qWarning( "QDataStream: No device" );	\
 				return *this; }
 #else
 #define CHECK_STREAM_PRECOND
@@ -111,6 +113,7 @@ QDataStream::QDataStream()
     owndev    = FALSE;
     byteorder = BigEndian;			// default byte order
     printable = FALSE;
+    ver	      = 2;			        // 2 is default for Qt 2.0
     noswap    = systemBigEndian;
 }
 
@@ -126,11 +129,12 @@ QDataStream::QDataStream( QIODevice *d )
     owndev    = FALSE;
     byteorder = BigEndian;			// default byte order
     printable = FALSE;
+    ver	      = 2;
     noswap    = systemBigEndian;
 }
 
 /*!
-  Constructs a data stream that operates on a byte array throught an
+  Constructs a data stream that operates on a byte array through an
   internal QBuffer device.
 
   Example:
@@ -155,6 +159,7 @@ QDataStream::QDataStream( QByteArray a, int mode )
     owndev    = TRUE;
     byteorder = BigEndian;			// default byte order
     printable = FALSE;
+    ver	      = 2;
     noswap    = systemBigEndian;
 }
 
@@ -205,7 +210,20 @@ void QDataStream::unsetDevice()
 
 
 /*!
-  \fn bool QDataStream::eof() const
+  \fn bool QDataStream::atEnd() const
+  Returns TRUE if the IO device has reached the end position (end of
+  stream or file) or if there is no IO device set.
+
+  Returns FALSE if the current position of the read/write head of the IO
+  device is somewhere before the end position.
+
+  \sa QIODevice::atEnd()
+*/
+
+/*!\fn bool QDataStream::eof() const
+
+  \obsolete
+
   Returns TRUE if the IO device has reached the end position (end of
   stream or file) or if there is no IO device set.
 
@@ -261,6 +279,25 @@ void QDataStream::setByteOrder( int bo )
 */
 
 
+/*!
+  \fn int QDataStream::version() const
+  Returns the version number of the data serialization format.
+  In Qt 2.0, this number is 2.
+  \sa setVersion()
+*/
+
+/*!
+  \fn void QDataStream::setVersion( int v )
+  Sets the version number of the data serialization format.
+
+  In Qt 2.0, the datastream serialization format of many Qt classes
+  was changed. In order to read data that was created with the
+  QDatastream of Qt 1.x, call this function with the version number \v
+  set to 1.
+
+  \sa version()
+*/
+
 /*****************************************************************************
   QDataStream read functions
  *****************************************************************************/
@@ -500,7 +537,7 @@ QDataStream &QDataStream::readRawBytes( char *s, uint len )
 {
     CHECK_STREAM_PRECOND
     if ( printable ) {				// printable data
-	register char *p = s;
+	register Q_INT8 *p = (Q_INT8*)s;
 	while ( len-- )
 	    *this >> *p++;
     } else {					// read data char array
