@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qdatetime.cpp,v 2.16 1998/07/03 00:09:43 hanord Exp $
+** $Id: qdatetime.cpp,v 2.29.2.1 1999/08/11 07:37:59 aavit Exp $
 **
 ** Implementation of date and time classes
 **
 ** Created : 940124
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -56,6 +58,8 @@ static const uint MSECS_PER_MIN = 60000;
 
 static short monthDays[] ={0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
 
+// ##### Localize.
+
 const char *QDate::monthNames[] = {
     "Jan", "Feb", "Mar", "Apr", "May", "Jun",
     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
@@ -74,13 +78,15 @@ const char *QDate::weekdayNames[] ={
 
   \ingroup time
 
-  The QDate is based on the Gregorian (modern western) calendar. England
-  adopted the Gregorian calendar on September 14th 1752, which is the
-  earliest date that is supported by QDate.  Using earlier dates will give
-  undefined results. Some countries adopted the Gregorian calendar later
-  than England, thus the week day of early dates might be incorrect for
-  these countries (but correct for England).  The end of time is reached
-  around 8000AD, by which time we expect Qt to be obsolete.
+  The QDate is based on the Gregorian (modern western) calendar.  This
+  calendar was first adopted by England in 1752, and subsequently by
+  most other western countries.  QDate computes the wrong day of week
+  for all dates earlier than 14th September 1752, and the day it
+  computes is correct only in some countries for years between 1752
+  and 1923.
+  
+  The end of time is reached around 8000AD, by which time we expect Qt
+  to be obsolete.
 
   \sa QTime, QDateTime
 */
@@ -204,12 +210,16 @@ int QDate::daysInYear() const
   Month 1 == "Jan", month 2 == "Feb" etc.
 */
 
-const char *QDate::monthName( int month) const
+QString QDate::monthName( int month ) const
 {
-#if defined(DEBUG)
-    ASSERT( month > 0 && month <= 12 );
+#if defined(CHECK_RANGE)
+    if ( month < 1 || month > 12 ) {
+	qWarning( "QDate::monthName: Parameter out ouf range." );
+	month = 1;
+    }
 #endif
-    return monthNames[month-1];
+    // ### Remove the fromLatin1 during localization
+    return QString::fromLatin1(monthNames[month-1]);
 }
 
 /*!
@@ -218,12 +228,16 @@ const char *QDate::monthName( int month) const
   Weekday 1 == "Mon", day 2 == "Tue" etc.
 */
 
-const char *QDate::dayName( int weekday) const
+QString QDate::dayName( int weekday) const
 {
-#if defined(DEBUG)
-    ASSERT( weekday > 0 && weekday <= 7 );
+#if defined(CHECK_RANGE)
+    if ( weekday < 1 || weekday > 7 ) {
+	qWarning( "QDate::dayName: Parameter out of range." );
+	weekday = 1;
+    }
 #endif
-    return weekdayNames[weekday-1];
+    // ### Remove the fromLatin1 during localization
+    return QString::fromLatin1(weekdayNames[weekday-1]);
 }
 
 
@@ -235,10 +249,14 @@ const char *QDate::dayName( int weekday) const
 
 QString QDate::toString() const
 {
-    QString buf;
     int y, m, d;
     jul2greg( jd, y, m, d );
-    buf.sprintf( "%s %s %d %d", dayName(dayOfWeek()), monthName(m), d, y);
+    QString buf = dayName(dayOfWeek());
+    buf += ' ';
+    buf += monthName(m);
+    QString t;
+    t.sprintf( " %d %d", d, y);
+    buf += t;
     return buf;
 }
 
@@ -252,7 +270,7 @@ bool QDate::setYMD( int y, int m, int d )
 {
     if ( !isValid(y,m,d) ) {
 #if defined(CHECK_RANGE)
-	 warning( "QDate::setYMD: Invalid date %04d/%02d/%02d", y, m, d );
+	 qWarning( "QDate::setYMD: Invalid date %04d/%02d/%02d", y, m, d );
 #endif
 	 return FALSE;
     }
@@ -553,7 +571,7 @@ bool QTime::setHMS( int h, int m, int s, int ms )
 {
     if ( !isValid(h,m,s,ms) ) {
 #if defined(CHECK_RANGE)
-	warning( "QTime::setHMS Invalid time %02d:%02d:%02d.%03d", h, m, s,
+	qWarning( "QTime::setHMS Invalid time %02d:%02d:%02d.%03d", h, m, s,
 		 ms );
 #endif
 	return FALSE;
@@ -569,9 +587,7 @@ bool QTime::setHMS( int h, int m, int s, int ms )
 
 QTime QTime::addSecs( int nsecs ) const
 {
-    QTime t;
-    t.ds = ((int)ds + nsecs*1000) % MSECS_PER_DAY;
-    return t;
+    return addMSecs(nsecs*1000);
 }
 
 /*!
@@ -596,7 +612,14 @@ int QTime::secsTo( const QTime &t ) const
 QTime QTime::addMSecs( int ms ) const
 {
     QTime t;
-    t.ds = (ds + ms) % MSECS_PER_DAY;
+    if ( ms < 0 ) {
+	// % not well-defined for -ve, but / is.
+	int negdays = (MSECS_PER_DAY-ms) / MSECS_PER_DAY;
+	t.ds = ((int)ds + ms + negdays*MSECS_PER_DAY)
+		% MSECS_PER_DAY;
+    } else {
+	t.ds = ((int)ds + ms) % MSECS_PER_DAY;
+    }
     return t;
 }
 
@@ -658,14 +681,14 @@ QTime QTime::currentTime()
 /*!
   Fetches the current time and returns TRUE if the time is within one
   minute after midnight, otherwise FALSE. The return value is used by
-  QDateTime::currentDateTime to ensure that the date there is correct.
+  QDateTime::currentDateTime() to ensure that the date there is correct.
 */
 
 bool QTime::currentTime( QTime *ct )
 {
     if ( !ct ) {
 #if defined(CHECK_NULL)
-	warning( "QTime::currentTime(QTime *): Null pointer not allowed" );
+	qWarning( "QTime::currentTime(QTime *): Null pointer not allowed" );
 #endif
 	return FALSE;
     }
@@ -701,7 +724,7 @@ bool QTime::currentTime( QTime *ct )
     time_t ltime = tv.tv_sec;
     tm *t = localtime( &ltime );
     ct->ds = (uint)( MSECS_PER_HOUR*t->tm_hour + MSECS_PER_MIN*t->tm_min +
-	            1000*t->tm_sec + tv.tv_usec/1000 );
+		     1000*t->tm_sec + tv.tv_usec/1000 );
     return (t->tm_hour== 0 && t->tm_min == 0);
 
 #else
@@ -737,7 +760,7 @@ bool QTime::isValid( int h, int m, int s, int ms )
     QTime t;
     t.start();				// start clock
     ... // some lengthy task
-    debug( "%d\n", t.elapsed() );	// prints # msecs elapsed
+    qDebug( "%d\n", t.elapsed() );	// prints # msecs elapsed
   \endcode
 
   \sa restart(), elapsed()
@@ -756,13 +779,21 @@ void QTime::start()
   repeated measurements; call start() to start the first measurement,
   then restart() for each later measurement.
 
+  Note that the counter wraps to zero 24 hours after the last call to
+  start() or restart().
+
+  \warning If the system's local time changes, the result is undefined.
+  This can happen e.g. when daylight saving is turned on or off.
+
   \sa start(), elapsed()
 */
 
 int QTime::restart()
 {
     QTime t = currentTime();
-    int	  n = msecsTo( t );
+    int n = msecsTo( t );
+    if ( n < 0 )				// passed midnight
+	n += 86400*1000;
     *this = t;
     return n;
 }
@@ -770,12 +801,22 @@ int QTime::restart()
 /*!
   Returns the number of milliseconds that have elapsed since start() or
   restart() were called.
+
+  Note that the counter wraps to zero 24 hours after the last call to
+  start() or restart.
+
+  \warning If the system's local time changes, the result is undefined.
+  This can happen e.g. when daylight saving is turned on or off.
+
   \sa start(), restart()
 */
 
 int QTime::elapsed()
 {
-    return msecsTo( currentTime() );
+    int n = msecsTo( currentTime() );
+    if ( n < 0 )				// passed midnight
+	n += 86400*1000;
+    return n;
 }
 
 
@@ -865,6 +906,9 @@ QDateTime::QDateTime( const QDate &date, const QTime &time )
   since 00:00:00 on January 1, 1970, Coordinated Universal Time (UTC).
   On systems that do not support timezones this function will behave as if
   local time were UTC.
+
+  Note that Microsoft Windows supports only a limited range of values for
+  \a secsSince1Jan1970UTC.
 */
 
 void QDateTime::setTime_t( uint secsSince1Jan1970UTC )
@@ -872,14 +916,13 @@ void QDateTime::setTime_t( uint secsSince1Jan1970UTC )
     time_t tmp = (time_t) secsSince1Jan1970UTC;
     tm *tM = localtime( &tmp );
     if ( !tM ) {
-	tM = localtime( 0 );
+	tM = gmtime( &tmp );
 	if ( !tM ) {
-#if defined(CHECK_NULL)
-	    warning( "QDateTime::setTime_t: Cannot get localtime" );
-#endif
+	    d.jd = QDate::greg2jul( 1970, 1, 1 );
+	    t.ds = 0;
 	    return;
 	}
-    }
+    } 
     d.jd = QDate::greg2jul( tM->tm_year + 1900, tM->tm_mon + 1, tM->tm_mday );
     t.ds = MSECS_PER_HOUR*tM->tm_hour + MSECS_PER_MIN*tM->tm_min +
 	    1000*tM->tm_sec;
@@ -889,15 +932,20 @@ void QDateTime::setTime_t( uint secsSince1Jan1970UTC )
 /*!
   Returns the datetime as a string.
 
-  The string format is "Sat May 20 1995 03:40:13".
+  The string format is "Sat May 20 03:40:13 1998".
 */
 
 QString QDateTime::toString() const
 {
-    QString buf;
-    QString time = t.toString();
-    buf.sprintf( "%s %s %d %s %d", d.dayName(d.dayOfWeek()),
-		 d.monthName(d.month()), d.day(), (const char*)time, d.year());
+    QString buf = d.dayName(d.dayOfWeek());
+    buf += ' ';
+    buf += d.monthName(d.month());
+    buf += ' ';
+    buf += QString().setNum(d.day());
+    buf += ' ';
+    buf += t.toString();
+    buf += ' ';
+    buf += QString().setNum(d.year());
     return buf;
 }
 
@@ -918,9 +966,18 @@ QDateTime QDateTime::addDays( int ndays ) const
 
 QDateTime QDateTime::addSecs( int nsecs ) const
 {
-    uint dd( d.jd );
-    int tt( t.ds );
-    tt += 1000 * nsecs;
+    uint dd = d.jd;
+    int  tt = t.ds;
+    int  sign = 1;
+    if ( nsecs < 0 ) {
+	nsecs = -nsecs;
+	sign = -1;
+    }
+    if ( nsecs >= (int)SECS_PER_DAY ) {
+	dd += sign*(nsecs/SECS_PER_DAY);
+	nsecs %= SECS_PER_DAY;
+    }
+    tt += sign*nsecs*1000;
     if ( tt < 0 ) {
 	tt = MSECS_PER_DAY - tt - 1;
 	dd -= tt / MSECS_PER_DAY;
@@ -938,7 +995,7 @@ QDateTime QDateTime::addSecs( int nsecs ) const
 
 /*!
   Returns the number of days from this datetime to \a dt, which is
-  negaitive if \a dt is in the past.
+  negative if \a dt is in the past.
 
   \sa addDays() secsTo()
 */
@@ -956,7 +1013,7 @@ int QDateTime::daysTo( const QDateTime &dt ) const
   \code
     QDateTime dt = QDateTime::currentDateTime();
     QDateTime x( QDate(dt.year(),12,24), QTime(17,00) );
-    debug( "There are %d seconds to Christmas", dt.secsTo(x) );
+    qDebug( "There are %d seconds to Christmas", dt.secsTo(x) );
   \endcode
 
   \sa addSecs() daysTo() QTime::secsTo()
