@@ -1,38 +1,68 @@
 /****************************************************************************
-** $Id: qfile.cpp,v 2.10 1997/08/29 10:45:21 paul Exp $
+** $Id: qfile.cpp,v 2.49.2.2 1999/09/03 07:45:54 hanord Exp $
 **
 ** Implementation of QFile class
 **
 ** Created : 930812
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
-#include "qfile.h"
-#include "qfiledef.h"
+#include "qglobal.h"
+#if defined(_OS_WIN32_)
+#ifdef UNICODE
+#ifndef _UNICODE
+#define _UNICODE
+#endif
+#endif
+#endif
 
-RCSTAG("$Id: qfile.cpp,v 2.10 1997/08/29 10:45:21 paul Exp $");
+#include "qfile.h"
+#include "qfiledefs.h"
+
+#if defined(_OS_WIN32_)
+#include <windows.h>
+#include <direct.h>
+#include <tchar.h>
+
+QCString qt_win95Name(const QString s)
+{
+    if ( s[0] == '/' && s[1] == '/' ) {
+	// Win95 cannot handle slash-slash needs slosh-slosh.
+	QString ss(s);
+	ss[0] = '\\';
+	ss[1] = '\\';
+	int n = ss.find('/');
+	if ( n >= 0 )
+	    ss[n] = '\\';
+	return qt_winQString2MB(ss);
+    } else {
+	return qt_winQString2MB(s);
+    }
+}
+#endif
 
 
 /*!
   \class QFile qfile.h
   \brief The QFile class is an I/O device that operates on files.
 
-  \ingroup tools
-  \ingroup files
-  \ingroup stream
+  \ingroup io
 
   QFile is an I/O device for reading and writing binary and text files.	A
   QFile may be used by itself (readBlock and writeBlock) or by more
@@ -77,7 +107,7 @@ QFile::QFile()
   \sa setName()
 */
 
-QFile::QFile( const char *name )
+QFile::QFile( const QString &name )
     : fn(name)
 {
     init();
@@ -106,13 +136,13 @@ void QFile::init()
     fh	   = 0;
     fd	   = 0;
     length = 0;
-    index  = 0;
+    ioIndex  = 0;
     ext_f  = FALSE;				// not an external file handle
 }
 
 
 /*!
-  \fn const char *QFile::name() const
+  \fn QString QFile::name() const
   Returns the name set by setName().
   \sa setName(), QFileInfo::fileName()
 */
@@ -142,40 +172,88 @@ void QFile::init()
   \sa name(), QFileInfo, QDir
 */
 
-void QFile::setName( const char *name )
+void QFile::setName( const QString &name )
 {
     if ( isOpen() ) {
 #if defined(CHECK_STATE)
-	warning( "QFile::setName: File is open" );
+	qWarning( "QFile::setName: File is open" );
 #endif
 	close();
     }
     fn = name;
 }
 
+bool qt_file_access( const QString& fn, int t )
+{
+    if ( fn.isEmpty() )
+	return FALSE;
+#if defined (UNIX)
+    return ACCESS( QFile::encodeName(fn), t ) == 0;
+#elif defined(_OS_WIN32_)
+    if ( qt_winunicode ) {
+	return _taccess((TCHAR*)qt_winTchar(fn,TRUE), t) == 0;
+    } else {
+	return ACCESS(qt_win95Name(fn), t) == 0;
+    }
+#endif
+}
 
 /*!
-  Returns TRUE if the file exists, otherwise FALSE.
+  Returns TRUE if this file exists, otherwise FALSE.
   \sa name()
 */
 
 bool QFile::exists() const
 {
-    if ( fn.isEmpty() )
-	return FALSE;
-    return ACCESS(fn.data(), F_OK) == 0;
+    return qt_file_access( fn, F_OK );
 }
 
 /*!
   Returns TRUE if the file given by \e fileName exists, otherwise FALSE.
 */
 
-bool QFile::exists( const char *fileName )
+bool QFile::exists( const QString &fileName )
 {
+    return qt_file_access( fileName, F_OK );
+}
+
+
+/*!
+  Removes the file specified by the file name currently set.
+  Returns TRUE if successful, otherwise FALSE.
+
+  The file is closed before it is removed.
+*/
+
+bool QFile::remove()
+{
+    close();
+    return remove( fn );
+}
+
+/*!
+  Removes the file \a fileName.
+  Returns TRUE if successful, otherwise FALSE.
+*/
+
+bool QFile::remove( const QString &fileName )
+{
+    if ( fileName.isEmpty() ) {
 #if defined(CHECK_NULL)
-    ASSERT( fileName != 0 );
+	qWarning( "QFile::remove: Empty or null file name" );
+#endif
+	return FALSE;
+    }
+#if defined(UNIX)
+    return unlink( QFile::encodeName(fileName) ) == 0;	// unlink more common in UNIX
+#elif defined(_OS_WIN32_)
+    // use standard ANSI remove
+    if ( qt_winunicode ) {
+	return _tremove((const TCHAR*)qt_winTchar(fileName,TRUE)) == 0;
+    } else {
+	return ::remove(qt_win95Name(fileName)) == 0;
+    }
 #endif
-    return ACCESS( fileName, F_OK ) == 0;
 }
 
 
@@ -198,7 +276,7 @@ bool QFile::exists( const char *fileName )
   <ul>
   <li>\c IO_Raw specified raw (non-buffered) file access.
   <li>\c IO_ReadOnly opens the file in read-only mode.
-  <li>\c IO_WriteOnly opens the file in write-only mode.
+  <li>\c IO_WriteOnly opens the file in write-only mode (and truncates).
   <li>\c IO_ReadWrite opens the file in read/write mode, equivalent to
   \c (IO_ReadOnly|IO_WriteOnly).
   <li>\c IO_Append opens the file in append mode. This mode is very useful
@@ -207,17 +285,17 @@ bool QFile::exists( const char *fileName )
   file index manually using at() in append mode.
   <li>\c IO_Truncate truncates the file.
   <li>\c IO_Translate enables carriage returns and linefeed translation
-  for text files under MS-DOS, Window, OS/2 and Macintosh.
+  for text files under MS-DOS, Windows and OS/2.
   </ul>
 
   The raw access mode is best when I/O is block-operated using 4kB block size
   or greater. Buffered access works better when reading small portions of
   data at a time.
-  
+
   <strong>Important:</strong> When working with buffered files, data may
   not be written to the file at once. Call \link flush() flush\endlink
   to make sure the data is really written.
-  
+
   \warning We have experienced problems with some C libraries when a buffered
   file is opened for both reading and writing. If a read operation takes place
   immediately after a write operation, the read buffer contains garbage data.
@@ -242,13 +320,13 @@ bool QFile::open( int m )
 {
     if ( isOpen() ) {				// file already open
 #if defined(CHECK_STATE)
-	warning( "QFile::open: File already open" );
+	qWarning( "QFile::open: File already open" );
 #endif
 	return FALSE;
     }
     if ( fn.isNull() ) {			// no file name defined
 #if defined(CHECK_NULL)
-	warning( "QFile::open: No file name specified" );
+	qWarning( "QFile::open: No file name specified" );
 #endif
 	return FALSE;
     }
@@ -256,7 +334,7 @@ bool QFile::open( int m )
     setMode( m );
     if ( !(isReadable() || isWritable()) ) {
 #if defined(CHECK_RANGE)
-	warning( "QFile::open: File access not specified" );
+	qWarning( "QFile::open: File access not specified" );
 #endif
 	return FALSE;
     }
@@ -289,17 +367,26 @@ bool QFile::open( int m )
 	if ( isAsynchronous() )
 	    oflags |= OPEN_ASYNC;
 #endif
-	fd = OPEN( (const char *)fn, oflags, 0666 );
+#if defined(UNIX)
+	fd = OPEN( QFile::encodeName(fn), oflags, 0666 );
+#elif defined(_OS_WIN32_)
+	if ( qt_winunicode ) {
+	    fd = _topen((const TCHAR*)qt_winTchar(fn,TRUE), oflags, 0666 );
+	} else {
+	    fd = OPEN(qt_win95Name(fn), oflags, 0666 );
+	}
+#endif
+
 	if ( fd != -1 ) {			// open successful
 	    STATBUF st;
 	    FSTAT( fd, &st );
-	    length = st.st_size;
-	    index  = (flags() & IO_Append) == 0 ? 0 : length;
+	    length = (int)st.st_size;
+	    ioIndex  = (flags() & IO_Append) == 0 ? 0 : length;
 	} else {
 	    ok = FALSE;
 	}
     } else {					// buffered file I/O
-	const char *perm = 0;
+	QCString perm;
 	char perm2[4];
 	bool try_create = FALSE;
 	if ( flags() & IO_Append ) {		// append to end of file?
@@ -326,16 +413,34 @@ bool QFile::open( int m )
 	else
 	    strcat( perm2, "b" );
 #endif
-	fh = fopen( (const char *)fn, perm2 );
-	if ( !fh && try_create ) {
-	    perm2[0] = 'w';			// try "w+" instead of "r+"
-	    fh = fopen( (const char *)fn, perm2 );
+	while (1) { // At most twice
+#if defined(UNIX)
+	    fh = fopen( QFile::encodeName(fn), perm2 );
+#elif defined(_OS_WIN32_)
+	    if ( qt_winunicode ) {
+		TCHAR tperm2[4];
+		tperm2[0] = perm2[0];
+		tperm2[1] = perm2[1];
+		tperm2[2] = perm2[2];
+		tperm2[3] = perm2[3];
+		fh = _tfopen((const TCHAR*)qt_winTchar(fn,TRUE), tperm2 );
+	    } else {
+		fh = fopen(qt_win95Name(fn),
+			    perm2 );
+	    }
+#endif
+	    if ( !fh && try_create ) {
+		perm2[0] = 'w';			// try "w+" instead of "r+"
+		try_create = FALSE;
+	    } else {
+		break;
+	    }
 	}
 	if ( fh ) {
 	    STATBUF st;
 	    FSTAT( FILENO(fh), &st );
-	    length = st.st_size;
-	    index  = (flags() & IO_Append) == 0 ? 0 : length;
+	    length = (int)st.st_size;
+	    ioIndex  = (flags() & IO_Append) == 0 ? 0 : length;
 	} else {
 	    ok = FALSE;
 	}
@@ -361,7 +466,7 @@ bool QFile::open( int m )
   \code
     #include <stdio.h>
 
-    void printError( const char *msg )
+    void printError( const char* msg )
     {
 	QFile f;
 	f.open( IO_WriteOnly, stderr );
@@ -373,8 +478,9 @@ bool QFile::open( int m )
   When a QFile is opened using this function, close() does not actually
   close the file, only flushes it.
 
-  \warning If \e f is one of \c stdin, \c stdout or \c stderr, you may not
-  be able to seek. size() is set to \c INT_MAX (in limits.h).
+  \warning If \e f is \c stdin, \c stdout, \c stderr, you may not
+  be able to seek.  See QIODevice::isSequentialAccess() for more
+  information.
 
   \sa close()
 */
@@ -383,22 +489,24 @@ bool QFile::open( int m, FILE *f )
 {
     if ( isOpen() ) {
 #if defined(CHECK_RANGE)
-	warning( "QFile::open: File already open" );
+	qWarning( "QFile::open: File already open" );
 #endif
 	return FALSE;
     }
     init();
-    setMode( m );
+    setMode( m &~IO_Raw );
     setState( IO_Open );
     fh = f;
     ext_f = TRUE;
-    if ( fh == stdin || fh == stdout || fh == stderr ) {
+    STATBUF st;
+    FSTAT( FILENO(fh), &st );
+    ioIndex = (int)ftell( fh );
+    if ( (st.st_mode & STAT_MASK) != STAT_REG ) {
+	// non-seekable
+	setType( IO_Sequential );
 	length = INT_MAX;
     } else {
-	STATBUF st;
-	FSTAT( FILENO(fh), &st );
-	length = st.st_size;
-	index = ftell( fh );
+	length = (int)st.st_size;
     }
     return TRUE;
 }
@@ -421,22 +529,24 @@ bool QFile::open( int m, int f )
 {
     if ( isOpen() ) {
 #if defined(CHECK_RANGE)
-	warning( "QFile::open: File already open" );
+	qWarning( "QFile::open: File already open" );
 #endif
 	return FALSE;
     }
     init();
-    setMode( m );
+    setMode( m |IO_Raw );
     setState( IO_Open );
     fd = f;
     ext_f = TRUE;
-    if ( fd == 0 || fd == 1 || fd == 2 ) {
+    STATBUF st;
+    FSTAT( fd, &st );
+    ioIndex  = (int)LSEEK(fd, 0, SEEK_CUR);
+    if ( (st.st_mode & STAT_MASK) != STAT_REG ) {
+	// non-seekable
+	setType( IO_Sequential );
 	length = INT_MAX;
     } else {
-	STATBUF st;
-	FSTAT( fd, &st );
-	length = st.st_size;
-	index  = LSEEK(fd, 0, SEEK_CUR);
+	length = (int)st.st_size;
     }
     return TRUE;
 }
@@ -445,29 +555,40 @@ bool QFile::open( int m, int f )
 /*!
   Closes an open file.
 
-  The file is closed even if it was opened with an existing file
-  handle or a file descriptor, \e except that stdin, stdout and stderr
-  are never closed.
+  The file is not closed if it was opened with an existing file handle.
+  If the existing file handle is a \c FILE*, the file is flushed.
+  If the existing file handle is an \c int file descriptor, nothing
+  is done to the file.
+
+  Some "write-behind" filesystems may report an unspecified error on
+  closing the file. These errors only indiciate that something may
+  have gone wrong since the previous open(). In such a case status()
+  reports IO_UnspecifiedError after close(), otherwise IO_Ok.
 
   \sa open(), flush()
 */
 
 void QFile::close()
 {
-    if ( !isOpen() )				// file is not open
-	return;
-    if ( fh ) {					// buffered file
-	if ( ext_f )
-	    fflush( fh );			// cannot close
-	else
-	    fclose( fh );
-    } else {					// raw file
-	if ( ext_f )
-	    ;					// cannot close
-	else
-	    CLOSE( fd );
+    bool ok = FALSE;
+    if ( isOpen() ) {				// file is not open
+	if ( fh ) {				// buffered file
+	    if ( ext_f )
+		ok = fflush( fh ) != -1;	// flush instead of closing
+	    else
+		ok = fclose( fh ) != -1;
+	} else {				// raw file
+	    if ( ext_f )
+		ok = TRUE;			// cannot close
+	    else
+		ok = CLOSE( fd ) != -1;
+	}
+	init();					// restore internal state
     }
-    init();					// restore internal state
+    if (!ok)
+	setStatus (IO_UnspecifiedError);
+
+    return;
 }
 
 
@@ -494,8 +615,17 @@ uint QFile::size() const
     STATBUF st;
     if ( isOpen() )
 	FSTAT( fh ? FILENO(fh) : fd, &st );
-    else
-	STAT( fn, &st );
+    else {
+#if defined (UNIX)
+	STAT( QFile::encodeName(fn), &st );
+#elif defined(_OS_WIN32_)
+	if ( qt_winunicode ) {
+	    _tstat((const TCHAR*)qt_winTchar(fn,TRUE), &st);
+	} else {
+	    _stat(qt_win95Name(fn), &st);
+	}
+#endif
+    }
     return st.st_size;
 }
 
@@ -529,22 +659,22 @@ bool QFile::at( int pos )
 {
     if ( !isOpen() ) {
 #if defined(CHECK_STATE)
-	warning( "QFile::at: File is not open" );
+	qWarning( "QFile::at: File is not open" );
 #endif
 	return FALSE;
     }
     bool ok;
     if ( isRaw() ) {				// raw file
-	pos = LSEEK(fd, pos, SEEK_SET);
+	pos = (int)LSEEK(fd, pos, SEEK_SET);
 	ok = pos != -1;
     } else {					// buffered file
 	ok = fseek(fh, pos, SEEK_SET) == 0;
     }
     if ( ok )
-	index = pos;
+	ioIndex = pos;
 #if defined(CHECK_RANGE)
     else
-	warning( "QFile::at: Cannot set file position %d", pos );
+	qWarning( "QFile::at: Cannot set file position %d", pos );
 #endif
     return ok;
 }
@@ -559,17 +689,15 @@ bool QFile::atEnd() const
 {
     if ( !isOpen() ) {
 #if defined(CHECK_STATE)
-	warning( "QFile::atEnd: File is not open" );
+	qWarning( "QFile::atEnd: File is not open" );
 #endif
 	return FALSE;
     }
-    bool end;
-    if ( isRaw() ) {				// raw file
-	end = (at() == (int)size());
-    } else {					// buffered file
-	end = feof( fh );
+    if ( isDirectAccess() ) {
+	if ( at() < length )
+	    return FALSE;
     }
-    return end;
+    return QIODevice::atEnd();
 }
 
 
@@ -590,23 +718,35 @@ bool QFile::atEnd() const
 
 int QFile::readBlock( char *p, uint len )
 {
+#if defined(CHECK_NULL)
+    if ( !p )
+	qWarning( "QFile::readBlock: Null pointer error" );
+#endif
 #if defined(CHECK_STATE)
-    CHECK_PTR( p );
     if ( !isOpen() ) {				// file not open
-	warning( "QFile::readBlock: File not open" );
+	qWarning( "QFile::readBlock: File not open" );
 	return -1;
     }
     if ( !isReadable() ) {			// reading not permitted
-	warning( "QFile::readBlock: Read operation not permitted" );
+	qWarning( "QFile::readBlock: Read operation not permitted" );
 	return -1;
     }
 #endif
     int nread;					// number of bytes read
-    if ( isRaw() )				// raw file
+    if ( isRaw() ) {				// raw file
 	nread = READ( fd, p, len );
-    else					// buffered file
+	if ( len && nread <= 0 ) {
+	    nread = 0;
+	    setStatus(IO_ReadError);
+	}
+    } else {					// buffered file
 	nread = fread( p, 1, len, fh );
-    index += nread;
+	if ( (uint)nread != len ) {
+	    if ( ferror( fh ) || nread==0 )
+		setStatus(IO_ReadError);
+	}
+    }
+    ioIndex += nread;
     return nread;
 }
 
@@ -628,15 +768,15 @@ int QFile::writeBlock( const char *p, uint len )
 {
 #if defined(CHECK_NULL)
     if ( p == 0 && len != 0 )
-	warning( "QFile::writeBlock: Null pointer error" );
+	qWarning( "QFile::writeBlock: Null pointer error" );
 #endif
 #if defined(CHECK_STATE)
     if ( !isOpen() ) {				// file not open
-	warning( "QFile::writeBlock: File not open" );
+	qWarning( "QFile::writeBlock: File not open" );
 	return -1;
     }
     if ( !isWritable() ) {			// writing not permitted
-	warning( "QFile::writeBlock: Write operation not permitted" );
+	qWarning( "QFile::writeBlock: Write operation not permitted" );
 	return -1;
     }
 #endif
@@ -651,14 +791,14 @@ int QFile::writeBlock( const char *p, uint len )
 	else
 	    setStatus( IO_WriteError );
 	if ( isRaw() )				// recalc file position
-	    index = LSEEK( fd, 0, SEEK_CUR );
+	    ioIndex = (int)LSEEK( fd, 0, SEEK_CUR );
 	else
-	    index = fseek( fh, 0, SEEK_CUR );
+	    ioIndex = fseek( fh, 0, SEEK_CUR );
     } else {
-	index += nwritten;
+	ioIndex += nwritten;
     }
-    if ( index > length )			// update file length
-	length = index;
+    if ( ioIndex > length )			// update file length
+	length = ioIndex;
     return nwritten;
 }
 
@@ -684,11 +824,11 @@ int QFile::readLine( char *p, uint maxlen )
 #if defined(CHECK_STATE)
     CHECK_PTR( p );
     if ( !isOpen() ) {				// file not open
-	warning( "QFile::readLine: File not open" );
+	qWarning( "QFile::readLine: File not open" );
 	return -1;
     }
     if ( !isReadable() ) {			// reading not permitted
-	warning( "QFile::readLine: Read operation not permitted" );
+	qWarning( "QFile::readLine: Read operation not permitted" );
 	return -1;
     }
 #endif
@@ -697,13 +837,46 @@ int QFile::readLine( char *p, uint maxlen )
 	nread = QIODevice::readLine( p, maxlen );
     } else {					// buffered file
 	p = fgets( p, maxlen, fh );
-	nread = p ? strlen( p ) : 0;
+	if ( p ) {
+	    nread = strlen( p );
+	    ioIndex += nread;
+	} else {
+	    nread = -1;
+	    setStatus(IO_ReadError);
+	}
     }
-    index += nread;
     return nread;
 }
 
 
+/*!
+  Reads a line of text.
+
+  Reads bytes from the file until end-of-line is reached, or up to \a
+  maxlen bytes, and returns the number of bytes read, or -1 in case of
+  error.  The terminating newline is not stripped.
+
+  This function is efficient only for buffered files.  Avoid
+  readLine() for files that have been opened with the \c IO_Raw
+  flag.
+
+  Note that the string is read as plain Latin1 bytes, not Unicode.
+
+  \sa readBlock(), QTextStream::readLine()
+*/
+
+int QFile::readLine( QString& s, uint maxlen )
+{
+    QByteArray ba(maxlen);
+    int l = readLine(ba.data(),maxlen);
+    if ( l >= 0 ) {
+	ba.truncate(l);
+	s = QString(ba);
+    }
+    return l;
+}
+
+
 /*!
   Reads a single byte/character from the file.
 
@@ -717,21 +890,32 @@ int QFile::getch()
 {
 #if defined(CHECK_STATE)
     if ( !isOpen() ) {				// file not open
-	warning( "QFile::getch: File not open" );
+	qWarning( "QFile::getch: File not open" );
 	return EOF;
     }
     if ( !isReadable() ) {			// reading not permitted
-	warning( "QFile::getch: Read operation not permitted" );
+	qWarning( "QFile::getch: Read operation not permitted" );
 	return EOF;
     }
 #endif
+
     int ch;
+
+    if ( !ungetchBuffer.isEmpty() ) {
+	int len = ungetchBuffer.length();
+	ch = ungetchBuffer[ len-1 ];
+	ungetchBuffer.truncate( len - 1 );
+	return ch;
+    }
+
     if ( isRaw() ) {				// raw file (inefficient)
 	char buf[1];
 	ch = readBlock( buf, 1 ) == 1 ? buf[0] : EOF;
     } else {					// buffered file
 	if ( (ch = getc( fh )) != EOF )
-	    index++;
+	    ioIndex++;
+	else
+	    setStatus(IO_ReadError);
     }
     return ch;
 }
@@ -748,11 +932,11 @@ int QFile::putch( int ch )
 {
 #if defined(CHECK_STATE)
     if ( !isOpen() ) {				// file not open
-	warning( "QFile::putch: File not open" );
+	qWarning( "QFile::putch: File not open" );
 	return EOF;
     }
     if ( !isWritable() ) {			// writing not permitted
-	warning( "QFile::putch: Write operation not permitted" );
+	qWarning( "QFile::putch: Write operation not permitted" );
 	return EOF;
     }
 #endif
@@ -762,9 +946,11 @@ int QFile::putch( int ch )
 	ch = writeBlock( buf, 1 ) == 1 ? ch : EOF;
     } else {					// buffered file
 	if ( (ch = putc( ch, fh )) != EOF ) {
-	    index++;
-	    if ( index > length )		// update file length
-		length = index;
+	    ioIndex++;
+	    if ( ioIndex > length )		// update file length
+		length = ioIndex;
+	} else {
+	    setStatus(IO_WriteError);
 	}
     }
     return ch;
@@ -785,27 +971,36 @@ int QFile::ungetch( int ch )
 {
 #if defined(CHECK_STATE)
     if ( !isOpen() ) {				// file not open
-	warning( "QFile::ungetch: File not open" );
+	qWarning( "QFile::ungetch: File not open" );
 	return EOF;
     }
     if ( !isReadable() ) {			// reading not permitted
-	warning( "QFile::ungetch: Read operation not permitted" );
+	qWarning( "QFile::ungetch: Read operation not permitted" );
 	return EOF;
     }
 #endif
     if ( ch == EOF )				// cannot unget EOF
 	return ch;
+
+    if ( isSequentialAccess() && !fh) {
+	// pipe or similar => we cannot ungetch, so do it manually
+	ungetchBuffer +=ch;
+	return ch;
+    }
+
     if ( isRaw() ) {				// raw file (very inefficient)
 	char buf[1];
-	at( index-1 );
+	at( ioIndex-1 );
 	buf[0] = ch;
 	if ( writeBlock(buf, 1) == 1 )
-	    at ( index-1 );
+	    at ( ioIndex-1 );
 	else
 	    ch = EOF;
     } else {					// buffered file
 	if ( (ch = ungetc(ch, fh)) != EOF )
-	    index--;
+	    ioIndex--;
+	else
+	    setStatus( IO_ReadError );
     }
     return ch;
 }
@@ -827,7 +1022,83 @@ int QFile::handle() const
     if ( !isOpen() )
 	return -1;
     else if ( fh )
-	return fileno( fh );
+	return FILENO( fh );
     else
 	return fd;
 }
+
+static
+QCString locale_encoder( const QString &fileName )
+{
+    return fileName.local8Bit();
+}
+
+static QFile::EncoderFn encoder = locale_encoder;
+
+/*!
+  When you use QFile, QFileInfo, and QDir to access the filesystem
+  with Qt, you can use Unicode filenames.  On Unix, these filenames
+  are converted to an 8-bit encoding.  If you want to do your own
+  file I/O on Unix, you should convert the filename using this
+  function.  On Windows NT, Unicode filenames are supported directly
+  in the filesystem and this function should be avoided. On Windows 95,
+  non-Latin1 locales are not supported at this time.
+
+  By default, this function converts to the local 8-bit encoding
+  determined by the user's locale.  This is sufficient for
+  filenames that the user chooses.  Filenames hard-coded into the
+  application should only use 7-bit ASCII filename characters.
+
+  The conversion scheme can be changed using setEncodingFunction().
+  This might be useful if you wish to give the user an option to
+  store in filenames in UTF-8, etc., but beware that such filenames
+  would probably then be unrecognizable when seen by other programs.
+
+  \sa decodeName()
+*/
+
+QCString QFile::encodeName( const QString &fileName )
+{
+    return (*encoder)(fileName);
+}
+
+/*!
+  Sets the function for encoding Unicode filenames.
+  The default encodes in the locale-specific 8-bit encoding.
+
+  \sa encodeName()
+*/
+void QFile::setEncodingFunction( EncoderFn f )
+{
+    encoder = f;
+}
+
+static
+QString locale_decoder( const QCString &localFileName )
+{
+    return QString::fromLocal8Bit(localFileName);
+}
+
+static QFile::DecoderFn decoder = locale_decoder;
+
+/*!
+  This does the reverse of QFile::encodeName().
+
+  \sa setDecodingFunction()
+*/
+QString QFile::decodeName( const QCString &localFileName )
+{
+    return (*decoder)(localFileName);
+}
+
+/*!
+  Sets the function for decoding 8-bit filenames.
+  The default uses the locale-specific 8-bit encoding.
+
+  \sa encodeName(), decodeName()
+*/
+
+void QFile::setDecodingFunction( DecoderFn f )
+{
+    decoder = f;
+}
