@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qfileinfo.cpp,v 2.12 1998/07/03 00:09:44 hanord Exp $
+** $Id: qfileinfo.cpp,v 2.32.2.2 1999/08/25 15:37:12 hanord Exp $
 **
 ** Implementation of QFileInfo class
 **
 ** Created : 950628
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -26,6 +28,14 @@
 #define readlink _qt_hide_readlink
 #endif
 
+#if defined(_OS_WIN32_)
+#ifdef UNICODE
+#ifndef _UNICODE
+#define _UNICODE
+#endif
+#endif
+#endif
+
 #include "qfileinfo.h"
 #include "qfiledefs.h"
 #include "qdatetime.h"
@@ -35,6 +45,12 @@
 #include <grp.h>
 #endif
 
+#if defined(_OS_WIN32_)
+#include <windows.h>
+#include <direct.h>
+#include <tchar.h>
+#endif
+
 #if defined(_OS_SUN_)
 #undef readlink
 extern "C" int readlink( const char *, void *, uint );
@@ -42,26 +58,26 @@ extern "C" int readlink( const char *, void *, uint );
 
 #if defined(_OS_FATFS_)
 
-static void convertSeparators( char *n )
+static void slashify( QString& s )
 {
-    if ( !n )
-	return;
-    while ( *n ) {
-	if ( *n ==  '\\' )
-	    *n = '/';
-	n++;
+    for (int i=0; i<(int)s.length(); i++) {
+	if ( s[i] == '\\' )
+	    s[i] = '/';
     }
 }
+extern QCString qt_win95Name(const QString s);
 
 #elif defined(UNIX)
 
-static void convertSeparators( char * )
+static void slashify( QString& )
 {
     return;
 }
 
 #endif
 
+extern bool qt_file_access( const QString& fn, int t );
+
 
 struct QFileInfoCache
 {
@@ -115,14 +131,14 @@ QFileInfo::QFileInfo()
   Constructs a new QFileInfo that gives information about the given file.
   The string given can be an absolute or a relative file path.
 
-  \sa bool setFile(const char*), isRelative(), QDir::setCurrent(),
+  \sa bool setFile(QString ), isRelative(), QDir::setCurrent(),
   QDir::isRelativePath()
 */
 
-QFileInfo::QFileInfo( const char *file )
+QFileInfo::QFileInfo( const QString &file )
 {
     fn	  = file;
-    convertSeparators( fn.data() );
+    slashify( fn );
     fic	  = 0;
     cache = TRUE;
 }
@@ -138,7 +154,7 @@ QFileInfo::QFileInfo( const char *file )
 QFileInfo::QFileInfo( const QFile &file )
 {
     fn	  = file.name();
-    convertSeparators( fn.data() );
+    slashify( fn );
     fic	  = 0;
     cache = TRUE;
 }
@@ -152,10 +168,10 @@ QFileInfo::QFileInfo( const QFile &file )
   \sa isRelative()
 */
 
-QFileInfo::QFileInfo( const QDir &d, const char *fileName )
+QFileInfo::QFileInfo( const QDir &d, const QString &fileName )
 {
     fn	  = d.filePath( fileName );
-    convertSeparators( fn.data() );
+    slashify( fn );
     fic	  = 0;
     cache = TRUE;
 }
@@ -224,8 +240,8 @@ QFileInfo &QFileInfo::operator=( const QFileInfo &fi )
 
     void test()
     {
-	const char *absolute = "/liver/aorta";
-	const char *relative = "liver/aorta";
+	QString absolute = "/liver/aorta";
+	QString relative = "liver/aorta";
 	QFileInfo fi1( absolute );
 	QFileInfo fi2( relative );
 
@@ -241,10 +257,10 @@ QFileInfo &QFileInfo::operator=( const QFileInfo &fi )
   \sa isRelative(), QDir::setCurrent(), QDir::isRelativePath()
 */
 
-void QFileInfo::setFile( const char *file )
+void QFileInfo::setFile( const QString &file )
 {
     fn = file;
-    convertSeparators( fn.data() );
+    slashify( fn );
     delete fic;
     fic = 0;
 }
@@ -260,13 +276,13 @@ void QFileInfo::setFile( const char *file )
 void QFileInfo::setFile( const QFile &file )
 {
     fn	= file.name();
-    convertSeparators( fn.data() );
+    slashify( fn );
     delete fic;
     fic = 0;
 }
 
 /*!
-  Sets the file to obains information about to \e fileName in the
+  Sets the file to obtains information about to \e fileName in the
   directory \e d.
 
   If the directory has a relative path, the QFileInfo will also have one.
@@ -274,10 +290,10 @@ void QFileInfo::setFile( const QFile &file )
   \sa isRelative()
 */
 
-void QFileInfo::setFile( const QDir &d, const char *fileName )
+void QFileInfo::setFile( const QDir &d, const QString &fileName )
 {
     fn	= d.filePath( fileName );
-    convertSeparators( fn.data() );
+    slashify( fn );
     delete fic;
     fic = 0;
 }
@@ -289,10 +305,7 @@ void QFileInfo::setFile( const QDir &d, const char *fileName )
 
 bool QFileInfo::exists() const
 {
-    if ( !fn.isNull() )
-	return ACCESS( fn.data(), F_OK ) == 0;
-    else
-	return FALSE;
+    return qt_file_access( fn, F_OK );
 }
 
 /*!
@@ -346,9 +359,9 @@ void QFileInfo::setCaching( bool enable )
   \sa isRelative(), absFilePath()
 */
 
-const char *QFileInfo::filePath() const
+QString QFileInfo::filePath() const
 {
-    return fn.data();
+    return fn;
 }
 
 /*!
@@ -369,7 +382,7 @@ QString QFileInfo::fileName() const
     if ( p == -1 )
 	return fn.copy();
     else
-	return QString( &fn[p+1] );
+	return fn.mid(p+1);
 }
 
 /*!
@@ -389,7 +402,6 @@ QString QFileInfo::absFilePath() const
 {
     if ( QDir::isRelativePath(fn) ) {
 	QString tmp = QDir::currentDirPath();
-	tmp.detach();
 	tmp += '/';
 	tmp += fn;
 	return QDir::cleanDirPath( tmp );
@@ -427,26 +439,32 @@ QString QFileInfo::baseName() const
 /*!
   Returns the extension name of the file.
 
-  The file name extension consists of all characters in the file name
-  after (but not including) the first '.' character.
+  If \a complete is TRUE (the default), extension() returns the string
+  of all characters in the file name after (but not including) the
+  first '.'  character.  For a file named "archive.tar.gz" this
+  returns "tar.gz".
 
-  For a file named "archive.tar.gz" this function will return "tar.gz".
+  If \a complete is FALSE, extension() returns the string of all
+  characters in the file name after (but not including) the last '.'
+  character.  For a file named "archive.tar.gz" this returns "gz".
 
   Example:
   \code
-     QFileInfo fi( "/tmp/abdomen.lower" );
-     QString ext = fi.extension();		// ext = "lower"
+     QFileInfo fi( "lex.yy.c" );
+     QString ext = fi.extension();		// ext = "yy.c"
+     QString ext = fi.extension( FALSE );	// ext = "c"
   \endcode
 
   \sa fileName(), baseName()
+
 */
 
-QString QFileInfo::extension() const
+QString QFileInfo::extension( bool complete ) const
 {
     QString s = fileName();
-    int pos = s.find( '.' );
-    if ( pos == -1 )
-	return QString( "" );
+    int pos = complete ? s.find( '.' ) : s.findRev( '.' );
+    if ( pos < 0 )
+	return QString::fromLatin1( "" );
     else
 	return s.right( s.length() - pos - 1 );
 }
@@ -469,7 +487,7 @@ QString QFileInfo::dirPath( bool absPath ) const
 	s = fn;
     int pos = s.findRev( '/' );
     if ( pos == -1 )
-	return ".";
+	return QString::fromLatin1(".");
     else
 	return s.left( pos );
 }
@@ -489,6 +507,7 @@ QDir QFileInfo::dir( bool absPath ) const
 }
 
 
+
 /*!
   Returns TRUE if the file is readable.
   \sa isWritable(), isExecutable(), permission()
@@ -496,10 +515,7 @@ QDir QFileInfo::dir( bool absPath ) const
 
 bool QFileInfo::isReadable() const
 {
-    if ( !fn.isNull() )
-	return ACCESS( fn.data(), R_OK ) == 0;
-    else
-	return FALSE;
+    return qt_file_access( fn, R_OK );
 }
 
 /*!
@@ -509,10 +525,7 @@ bool QFileInfo::isReadable() const
 
 bool QFileInfo::isWritable() const
 {
-    if ( !fn.isNull() )
-	return ACCESS( fn.data(), W_OK ) == 0 ;
-    else
-	return FALSE;
+    return qt_file_access( fn, W_OK );
 }
 
 /*!
@@ -522,10 +535,7 @@ bool QFileInfo::isWritable() const
 
 bool QFileInfo::isExecutable() const
 {
-    if ( !fn.isNull() )
-	return ACCESS( fn.data(), X_OK ) == 0;
-    else
-	return FALSE;
+    return qt_file_access( fn, X_OK );
 }
 
 
@@ -539,7 +549,7 @@ bool QFileInfo::isExecutable() const
 
 bool QFileInfo::isRelative() const
 {
-    return QDir::isRelativePath( fn.data() );
+    return QDir::isRelativePath( fn );
 }
 
 /*!
@@ -554,7 +564,7 @@ bool QFileInfo::convertToAbs()
 {
     if ( isRelative() )
 	fn = absFilePath();
-    return QDir::isRelativePath( fn.data() );
+    return QDir::isRelativePath( fn );
 }
 
 
@@ -617,17 +627,18 @@ bool QFileInfo::isSymLink() const
 
 QString QFileInfo::readLink() const
 {
-    QString s;
+    QString r;
+
 #if defined(UNIX) && !defined(_OS_OS2EMX_)
+    char s[PATH_MAX+1];
     if ( !isSymLink() )
-	return s;
-    s.resize( PATH_MAX+1 );
-    int len = readlink( fn.data(), s.data(), s.size() );
-    if ( len < 0 )
-	len = 0;				// error, return empty string
-    s.truncate( len );
+	return QString();
+    int len = readlink( QFile::encodeName(fn).data(), s, PATH_MAX );
+    if ( len >= 0 )
+	r = QFile::decodeName(s);
 #endif
-    return s;
+
+    return r;
 }
 
 
@@ -643,14 +654,14 @@ QString QFileInfo::readLink() const
   \sa ownerId(), group(), groupId()
 */
 
-const char *QFileInfo::owner() const
+QString QFileInfo::owner() const
 {
 #if defined(UNIX)
     passwd *pw = getpwuid( ownerId() );
-    return pw ? pw->pw_name : 0;
-#else
-    return 0;
+    if ( pw )
+	return QFile::decodeName( pw->pw_name );
 #endif
+    return QString::null;
 }
 
 static const uint nobodyID = (uint) -2;
@@ -686,14 +697,14 @@ uint QFileInfo::ownerId() const
   \sa groupId(), owner(), ownerId()
 */
 
-const char *QFileInfo::group() const
+QString QFileInfo::group() const
 {
 #if defined(UNIX)
     struct group *gr = getgrgid( groupId() );
-    return gr ? gr->gr_name : 0;
-#else
-    return 0;
+    if ( gr )
+	return QFile::decodeName( gr->gr_name );
 #endif
+    return QString::null;
 }
 
 /*!
@@ -731,9 +742,9 @@ uint QFileInfo::groupId() const
   \code
     QFileInfo fi( "/tmp/tonsils" );
     if ( fi.permission( QFileInfo::WriteUser | QFileInfo::ReadGroup ) )
-	warning( "Tonsils can be changed by me, and the group can read them.");
+	qWarning( "Tonsils can be changed by me, and the group can read them.");
     if ( fi.permission( QFileInfo::WriteGroup | QFileInfo::WriteOther ) )
-	warning( "Danger! Tonsils can be changed by the group or others!" );
+	qWarning( "Danger! Tonsils can be changed by the group or others!" );
   \endcode
 
   \sa isReadable(), isWritable(), isExecutable()
@@ -768,7 +779,7 @@ bool QFileInfo::permission( int permissionSpec ) const
 	   return (fic->st.st_mode & mask) == mask;
 	} else {
 #if defined(CHECK_NULL)
-	   warning( "QFileInfo::permission: permissionSpec is 0" );
+	   qWarning( "QFileInfo::permission: permissionSpec is 0" );
 #endif
 	   return TRUE;
 	}
@@ -845,15 +856,59 @@ void QFileInfo::doStat() const
     that->fic->isSymLink = FALSE;
 
 #if defined( UNIX ) && defined(S_IFLNK)
-    if ( ::lstat(fn.data(),b) == 0 ) {
+    if ( ::lstat(QFile::encodeName(fn),b) == 0 ) {
 	if ( S_ISLNK( b->st_mode ) )
 	    that->fic->isSymLink = TRUE;
 	else
 	    return;
     }
 #endif
+    int r;
+#if defined (UNIX)
+    r = STAT( QFile::encodeName(fn), b );
+#else
+    if ( qt_winunicode )
+	r = _tstat((const TCHAR*)qt_winTchar(fn,TRUE), b);
+    else
+	r = _stat(qt_win95Name(fn), b);
+    if ( r!=0 ) {
+	bool is_dir=FALSE;
+	if ( fn[0] == '/' && fn[1] == '/'
+	  || fn[0] == '\\' && fn[1] == '\\' )
+	{
+	    // UNC - stat doesn't work for all cases (Windows bug)
+	    int s = fn.find(fn[0],2);
+	    if ( s > 0 ) {
+		// "\\server\..."
+		s = fn.find(fn[0],s+1);
+		if ( s > 0 ) {
+		    // "\\server\share\..."
+		    if ( fn[s+1] ) {
+			// "\\server\share\notfound"
+		    } else {
+			// "\\server\share\"
+			is_dir=TRUE;
+		    }
+		} else {
+		    // "\\server\share"
+		    is_dir=TRUE;
+		}
+	    } else {
+		// "\\server"
+		is_dir=TRUE;
+	    }
+	}
+	if ( is_dir ) {
+	    // looks like a UNC dir, is a dir.
+	    memset(b,0,sizeof(*b));
+	    b->st_mode = STAT_DIR;
+	    b->st_nlink = 1;
+	    r = 0;
+	}
+    }
+#endif
 
-    if ( STAT(fn.data(),b) != 0 ) {
+    if ( r != 0 ) {
 	delete that->fic;
 	that->fic = 0;
     }
