@@ -1,37 +1,38 @@
 /****************************************************************************
-** $Id: qgcache.cpp,v 2.2 1997/08/06 15:05:52 hanord Exp $
+** $Id: qgcache.cpp,v 2.23 1999/06/23 12:06:09 hanord Exp $
 **
 ** Implementation of QGCache and QGCacheIterator classes
 **
 ** Created : 950208
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qgcache.h"
 #include "qlist.h"
 #include "qdict.h"
-#include "qstring.h"				/* used for statistics */
-
-RCSTAG("$Id: qgcache.cpp,v 2.2 1997/08/06 15:05:52 hanord Exp $");
+#include "qstring.h"
 
 
 /*!
   \class QGCache qgcache.h
 
-  \brief The QGCache class is an internal class for implementing QCache and QIntCache.
+  \brief The QGCache class is an internal class for implementing QCache template classes.
 
   QGCache is a strictly internal class that acts as a base class for the
   \link collection.html collection classes\endlink QCache and QIntCache.
@@ -44,13 +45,13 @@ RCSTAG("$Id: qgcache.cpp,v 2.2 1997/08/06 15:05:52 hanord Exp $");
 
 struct QCacheItem
 {
-    QCacheItem( const char *k, GCI d, int c, short p )
+    QCacheItem( void *k, QCollection::Item d, int c, short p )
 	: priority(p), skipPriority(p), cost(c), key(k), data(d), node(0) {}
     short	priority;
     short	skipPriority;
     int		cost;
-    const char *key;
-    GCI		data;
+    void       *key;
+    QCollection::Item data;
     QLNode     *node;
 };
 
@@ -59,11 +60,10 @@ struct QCacheItem
   QCList class (internal list of cache items)
  *****************************************************************************/
 
-Q_DECLARE(QListM,QCacheItem);
-
-class QCList : private QListM(QCacheItem)
+class QCList : private QList<QCacheItem>
 {
 friend class QGCacheIterator;
+friend class QCListIt;
 public:
     QCList()	{}
    ~QCList();
@@ -75,13 +75,13 @@ public:
 
     void	setAutoDelete( bool del ) { QCollection::setAutoDelete(del); }
 
-    bool	removeFirst()	{ return QListM(QCacheItem)::removeFirst(); }
-    bool	removeLast()	{ return QListM(QCacheItem)::removeLast(); }
+    bool	removeFirst()	{ return QList<QCacheItem>::removeFirst(); }
+    bool	removeLast()	{ return QList<QCacheItem>::removeLast(); }
 
-    QCacheItem *first()		{ return QListM(QCacheItem)::first(); }
-    QCacheItem *last()		{ return QListM(QCacheItem)::last(); }
-    QCacheItem *prev()		{ return QListM(QCacheItem)::prev(); }
-    QCacheItem *next()		{ return QListM(QCacheItem)::next(); }
+    QCacheItem *first()		{ return QList<QCacheItem>::first(); }
+    QCacheItem *last()		{ return QList<QCacheItem>::last(); }
+    QCacheItem *prev()		{ return QList<QCacheItem>::prev(); }
+    QCacheItem *next()		{ return QList<QCacheItem>::next(); }
 
 #if defined(DEBUG)
     int		inserts;			// variables for statistics
@@ -112,7 +112,7 @@ void QCList::insert( QCacheItem *ci )
 	item = next();
     }
     if ( item )
-	QListM(QCacheItem)::insert( at(), ci );
+	QList<QCacheItem>::insert( at(), ci );
     else
 	append( ci );
 #if defined(DEBUG)
@@ -123,7 +123,7 @@ void QCList::insert( QCacheItem *ci )
 
 inline void QCList::insert( int i, QCacheItem *ci )
 {
-    QListM(QCacheItem)::insert( i, ci );
+    QList<QCacheItem>::insert( i, ci );
 #if defined(DEBUG)
     ASSERT( ci->node == 0 );
 #endif
@@ -153,6 +153,14 @@ inline void QCList::reference( QCacheItem *ci )
 }
 
 
+class QCListIt: public QListIterator<QCacheItem>
+{
+public:
+    QCListIt( const QCList *p ): QListIterator<QCacheItem>( *p ) {}
+    QCListIt( const QCListIt *p ): QListIterator<QCacheItem>( *p ) {}
+};
+
+
 /*****************************************************************************
   QCDict class (internal dictionary of cache items)
  *****************************************************************************/
@@ -166,16 +174,37 @@ inline void QCList::reference( QCacheItem *ci )
 class QCDict : public QGDict
 {
 public:
-    QCDict( uint size, bool caseSensitive, bool copyKeys, bool trivial )
-	: QGDict( size, caseSensitive, copyKeys, trivial ) {}
+    QCDict( uint size, uint kt, bool caseSensitive, bool copyKeys )
+	: QGDict( size, (KeyType)kt, caseSensitive, copyKeys ) {}
+
+    QCacheItem *find_string(const QString &key) const
+	{ return (QCacheItem*)((QCDict*)this)->look_string(key, 0, 0); }
+    QCacheItem *find_ascii(const char *key) const
+	{ return (QCacheItem*)((QCDict*)this)->look_ascii(key, 0, 0); }
+    QCacheItem *find_int(long key) const
+	{ return (QCacheItem*)((QCDict*)this)->look_int(key, 0, 0); }
+
+    QCacheItem *take_string(const QString &key)
+	{ return (QCacheItem*)QGDict::take_string(key); }
+    QCacheItem *take_ascii(const char *key)
+	{ return (QCacheItem*)QGDict::take_ascii(key); }
+    QCacheItem *take_int(long key)
+	{ return (QCacheItem*)QGDict::take_int(key); }
+
+    bool  insert_string( const QString &key, const QCacheItem *ci )
+	{ return QGDict::look_string(key,(Item)ci,1)!=0;}
+    bool  insert_ascii( const char *key, const QCacheItem *ci )
+	{ return QGDict::look_ascii(key,(Item)ci,1)!=0;}
+    bool  insert_int( long key, const QCacheItem *ci )
+	{ return QGDict::look_int(key,(Item)ci,1)!=0;}
+
+    bool  remove_string( QCacheItem *item )
+	{ return QGDict::remove_string(*((QString*)(item->key)),item); }
+    bool  remove_ascii( QCacheItem *item )
+	{ return QGDict::remove_ascii((const char *)item->key,item); }
+    bool  remove_int( QCacheItem *item )
+	{ return QGDict::remove_int((long)item->key,item);}
 
-    QCacheItem *find(const char *key) const
-		  { return (QCacheItem*)((QCDict*)this)->look(key, 0, 0); }
-    QCacheItem *take(const char *key)
-		  { return (QCacheItem*)QGDict::take(key); }
-    bool  insert( const char *key, const QCacheItem *ci )
-		  { return QGDict::look(key,(GCI)ci,1)!=0;}
-    bool  remove( const char *key )	{ return QGDict::remove(key); }
     void  statistics()			{ QGDict::statistics(); }
 };
 
@@ -189,14 +218,15 @@ public:
   Constructs a cache.
 */
 
-QGCache::QGCache( int maxCost, uint size,
-		  bool caseS, bool copyKeys, bool trivial )
+QGCache::QGCache( int maxCost, uint size, KeyType kt, bool caseSensitive,
+		  bool copyKeys )
 {
+    keytype = kt;
     lruList = new QCList;
     CHECK_PTR( lruList );
     lruList->setAutoDelete( TRUE );
-    copyK   = copyKeys;
-    dict    = new QCDict( size, caseS, FALSE, trivial );
+    copyk   = ((keytype == AsciiKey) && copyKeys);
+    dict    = new QCDict( size, kt, caseSensitive, FALSE );
     CHECK_PTR( dict );
     mCost   = maxCost;
     tCost   = 0;
@@ -218,9 +248,10 @@ QGCache::QGCache( int maxCost, uint size,
 */
 
 QGCache::QGCache( const QGCache & )
+    : QCollection()
 {
 #if defined(CHECK_NULL)
-    fatal( "QGCache::QGCache(QGCache &): Cannot copy a cache" );
+    qFatal( "QGCache::QGCache(QGCache &): Cannot copy a cache" );
 #endif
 }
 
@@ -244,7 +275,7 @@ QGCache::~QGCache()
 QGCache &QGCache::operator=( const QGCache & )
 {
 #if defined(CHECK_NULL)
-    fatal( "QGCache::operator=: Cannot copy a cache" );
+    qFatal( "QGCache::operator=: Cannot copy a cache" );
 #endif
     return *this;				// satisfy the compiler
 }
@@ -292,13 +323,18 @@ void QGCache::setMaxCost( int maxCost )
 /*!
   \internal
   Inserts an item into the cache.
+
+  \warning If this function returns FALSE, you must delete \a data
+  yourself.  Additionally, be very careful about using \a data after
+  calling this function, as any other insertions into the cache, from
+  anywhere in the application, or within Qt itself, could cause the
+  data to be discarded from the cache, and the pointer to become
+  invalid.
 */
 
-bool QGCache::insert( const char *key, GCI data, int cost, int priority )
+bool QGCache::insert_string( const QString &key, QCollection::Item data,
+			     int cost, int priority)
 {
-#if defined(CHECK_NULL)
-    ASSERT( key != 0 && data != 0 );
-#endif
     if ( tCost + cost > mCost ) {
 	if ( !makeRoomFor(tCost + cost - mCost, priority) ) {
 #if defined(DEBUG)
@@ -308,61 +344,134 @@ bool QGCache::insert( const char *key, GCI data, int cost, int priority )
 	}
     }
 #if defined(DEBUG)
+    ASSERT( keytype == StringKey );
     lruList->inserts++;
     lruList->insertCosts += cost;
 #endif
-    if ( copyK )
-	key = qstrdup( key );
-    QCacheItem *ci = new QCacheItem( key, newItem(data), cost,
-				     (short)priority );
+    if ( priority < -32768 )
+	priority = -32768;
+    else if ( priority > 32767 )
+	priority = 32677;
+    QCacheItem *ci = new QCacheItem( new QString(key), newItem(data),
+				     cost, (short)priority );
     CHECK_PTR( ci );
     lruList->insert( 0, ci );
-    dict->insert( key, ci );
+    dict->insert_string( key, ci );
     tCost += cost;
     return TRUE;
 }
 
+
+/*! \internal */
+
+bool QGCache::insert_other( const char *key, QCollection::Item data,
+			    int cost, int priority)
+{
+    if ( tCost + cost > mCost ) {
+	if ( !makeRoomFor(tCost + cost - mCost, priority) ) {
+#if defined(DEBUG)
+	    lruList->insertMisses++;
+#endif
+	    return FALSE;
+	}
+    }
+#if defined(DEBUG)
+    ASSERT( keytype != StringKey );
+    lruList->inserts++;
+    lruList->insertCosts += cost;
+#endif
+    if ( keytype == AsciiKey && copyk )
+	key = qstrdup( key );
+    if ( priority < -32768 )
+	priority = -32768;
+    else if ( priority > 32767 )
+	priority = 32677;
+    QCacheItem *ci = new QCacheItem( (void*)key, newItem(data), cost,
+				     (short)priority );
+    CHECK_PTR( ci );
+    lruList->insert( 0, ci );
+    if ( keytype == AsciiKey )
+	dict->insert_ascii( key, ci );
+    else
+	dict->insert_int( (long)key, ci );
+    tCost += cost;
+    return TRUE;
+}
+
+
 /*!
   \internal
   Removes an item from the cache.
 */
 
-bool QGCache::remove( const char *key )
+bool QGCache::remove_string( const QString &key )
 {
-#if defined(CHECK_NULL)
-    ASSERT( key != 0 );
-#endif
-    GCI d = take( key );
+    Item d = take_string( key );
     if ( d )
 	deleteItem( d );
     return d != 0;
 }
 
+
+/*! \internal */
+
+bool QGCache::remove_other( const char *key )
+{
+    Item d = take_other( key );
+    if ( d )
+	deleteItem( d );
+    return d != 0;
+}
+
+
+/*!
+  \internal
+  Takes an item out of the cache (no delete).
+*/
+
+QCollection::Item QGCache::take_string( const QString &key )
+{
+    QCacheItem *ci = dict->take_string( key );	// take from dict
+    Item d;
+    if ( ci ) {
+	d = ci->data;
+	tCost -= ci->cost;
+	lruList->take( ci );			// take from list
+	delete (QString*)ci->key;
+	delete ci;
+    } else {
+	d = 0;
+    }
+    return d;
+}
+
 /*!
   \internal
   Takes an item out of the cache (no delete).
 */
 
-GCI QGCache::take( const char *key )
+QCollection::Item QGCache::take_other( const char *key )
 {
-#if defined(CHECK_NULL)
-    ASSERT( key != 0 );
-#endif
-    QCacheItem *ci = dict->take( key );		// take from dict
-    GCI d;
+    QCacheItem *ci;
+    if ( keytype == AsciiKey )
+	ci = dict->take_ascii( key );
+    else
+	ci = dict->take_int( (long)key );
+    Item d;
     if ( ci ) {
 	d = ci->data;
 	tCost -= ci->cost;
-	if ( copyK )
-	    delete [] (char *)ci->key;
 	lruList->take( ci );			// take from list
+	if ( copyk )
+	    delete [] (char *)ci->key;
 	delete ci;
-    }
-    else
+    } else {
 	d = 0;
+    }
     return d;
 }
 
+
 /*!
   \internal
   Clears the cache.
@@ -370,12 +479,25 @@ GCI QGCache::take( const char *key )
 
 void QGCache::clear()
 {
-    register QCacheItem *ci;
+    QCacheItem *ci;
     while ( (ci = lruList->first()) ) {
-	dict->remove( ci->key );		// remove from dict
+	switch ( keytype ) {
+	    case StringKey:
+		dict->remove_string( ci );
+		delete (QString*)ci->key;
+		break;
+	    case AsciiKey:
+		dict->remove_ascii( ci );
+		if ( copyk )
+		    delete [] (char*)ci->key;
+		break;
+	    case IntKey:
+		dict->remove_int( ci );
+		break;
+	    case PtrKey:			// unused
+		break;
+	}
 	deleteItem( ci->data );			// delete data
-	if ( copyK )
-	    delete [] (char *)ci->key;
 	lruList->removeFirst();			// remove from list
     }
     tCost = 0;
@@ -387,12 +509,34 @@ void QGCache::clear()
   Finds an item in the cache.
 */
 
-GCI QGCache::find( const char *key, bool ref ) const
+QCollection::Item QGCache::find_string( const QString &key, bool ref ) const
 {
-#if defined(CHECK_NULL)
-    ASSERT( key != 0 );
+    QCacheItem *ci = dict->find_string( key );
+#if defined(DEBUG)
+    lruList->finds++;
 #endif
-    QCacheItem *ci = dict->find( key );
+    if ( ci ) {
+#if defined(DEBUG)
+	lruList->hits++;
+	lruList->hitCosts += ci->cost;
+#endif
+	if ( ref )
+	    lruList->reference( ci );
+	return ci->data;
+    }
+    return 0;
+}
+
+
+/*!
+  \internal
+  Finds an item in the cache.
+*/
+
+QCollection::Item QGCache::find_other( const char *key, bool ref ) const
+{
+    QCacheItem *ci = keytype == AsciiKey ? dict->find_ascii(key)
+					 : dict->find_int((long)key);
 #if defined(DEBUG)
     lruList->finds++;
 #endif
@@ -439,9 +583,22 @@ bool QGCache::makeRoomFor( int cost, int priority )
 	lruList->dumps++;
 	lruList->dumpCosts += ci->cost;
 #endif
-	dict->remove( ci->key );		// remove from dict
-	if ( copyK )
-	    delete [] (char *)ci->key;
+	switch ( keytype ) {
+	    case StringKey:
+		dict->remove_string( ci );
+		delete (QString*)ci->key;
+		break;
+	    case AsciiKey:
+		dict->remove_ascii( ci );
+		if ( copyk )
+		    delete [] (char *)ci->key;
+		break;
+	    case IntKey:
+		dict->remove_int( ci );
+		break;
+	    case PtrKey:			// unused
+		break;
+	}
 	deleteItem( ci->data );			// delete data
 	lruList->removeLast();			// remove from list
     }
@@ -460,28 +617,28 @@ void QGCache::statistics() const
 #if defined(DEBUG)
     QString line;
     line.fill( '*', 80 );
-    debug( line );
-    debug( "CACHE STATISTICS:" );
-    debug( "cache contains %d item%s, with a total cost of %d",
+    qDebug( line.ascii() );
+    qDebug( "CACHE STATISTICS:" );
+    qDebug( "cache contains %d item%s, with a total cost of %d",
 	   count(), count() != 1 ? "s" : "", tCost );
-    debug( "maximum cost is %d, cache is %d%% full.",
+    qDebug( "maximum cost is %d, cache is %d%% full.",
 	   mCost, (200*tCost + mCost) / (mCost*2) );
-    debug( "find() has been called %d time%s",
+    qDebug( "find() has been called %d time%s",
 	   lruList->finds, lruList->finds != 1 ? "s" : "" );
-    debug( "%d of these were hits, items found had a total cost of %d.",
+    qDebug( "%d of these were hits, items found had a total cost of %d.",
 	   lruList->hits,lruList->hitCosts );
-    debug( "%d item%s %s been inserted with a total cost of %d.",
+    qDebug( "%d item%s %s been inserted with a total cost of %d.",
 	   lruList->inserts,lruList->inserts != 1 ? "s" : "",
 	   lruList->inserts != 1 ? "have" : "has", lruList->insertCosts );
-    debug( "%d item%s %s too large or had too low priority to be inserted.",
+    qDebug( "%d item%s %s too large or had too low priority to be inserted.",
 	   lruList->insertMisses, lruList->insertMisses != 1 ? "s" : "",
 	   lruList->insertMisses != 1 ? "were" : "was" );
-    debug( "%d item%s %s been thrown away with a total cost of %d.",
+    qDebug( "%d item%s %s been thrown away with a total cost of %d.",
 	   lruList->dumps, lruList->dumps != 1 ? "s" : "",
 	   lruList->dumps != 1 ? "have" : "has", lruList->dumpCosts );
-    debug( "Statistics from internal dictionary class:" );
+    qDebug( "Statistics from internal dictionary class:" );
     dict->statistics();
-    debug( line );
+    qDebug( line.ascii() );
 #endif
 }
 
@@ -493,14 +650,12 @@ void QGCache::statistics() const
 /*!
   \class QGCacheIterator qgcache.h
 
-  \brief The QGCacheIterator is an internal class for implementing QCacheIterator and QIntCacheIterator.
+  \brief An internal class for implementing QCacheIterator and QIntCacheIterator.
 
   QGCacheIterator is a strictly internal class that does the heavy work for
   QCacheIterator and QIntCacheIterator.
 */
 
-Q_DECLARE(QListIteratorM,QCacheItem);
-
 /*!
   \internal
   Constructs an iterator that operates on the cache \e c.
@@ -508,7 +663,7 @@ Q_DECLARE(QListIteratorM,QCacheItem);
 
 QGCacheIterator::QGCacheIterator( const QGCache &c )
 {
-    it = new QListIteratorM(QCacheItem)( *((QListM(QCacheItem)*)c.lruList) );
+    it = new QCListIt( c.lruList );
 #if defined(DEBUG)
     ASSERT( it != 0 );
 #endif
@@ -521,7 +676,7 @@ QGCacheIterator::QGCacheIterator( const QGCache &c )
 
 QGCacheIterator::QGCacheIterator( const QGCacheIterator &ci )
 {
-    it = new QListIteratorM(QCacheItem)( *ci.it );
+    it = new QCListIt( ci.it );
 #if defined(DEBUG)
     ASSERT( it != 0 );
 #endif
@@ -583,9 +738,9 @@ bool QGCacheIterator::atLast() const
   Sets the list iterator to point to the first item in the cache.
 */
 
-GCI QGCacheIterator::toFirst()
+QCollection::Item QGCacheIterator::toFirst()
 {
-    register QCacheItem *item = it->toFirst();
+    QCacheItem *item = it->toFirst();
     return item ? item->data : 0;
 }
 
@@ -594,9 +749,9 @@ GCI QGCacheIterator::toFirst()
   Sets the list iterator to point to the last item in the cache.
 */
 
-GCI QGCacheIterator::toLast()
+QCollection::Item QGCacheIterator::toLast()
 {
-    register QCacheItem *item = it->toLast();
+    QCacheItem *item = it->toLast();
     return item ? item->data : 0;
 }
 
@@ -605,9 +760,9 @@ GCI QGCacheIterator::toLast()
   Returns the current item.
 */
 
-GCI QGCacheIterator::get() const
+QCollection::Item QGCacheIterator::get() const
 {
-    register QCacheItem *item = it->current();
+    QCacheItem *item = it->current();
     return item ? item->data : 0;
 }
 
@@ -616,21 +771,42 @@ GCI QGCacheIterator::get() const
   Returns the key of the current item.
 */
 
-const char *QGCacheIterator::getKey() const
+QString QGCacheIterator::getKeyString() const
 {
-    register QCacheItem *item = it->current();
-    return item ? item->key : 0;
+    QCacheItem *item = it->current();
+    return item ? *((QString*)item->key) : QString::null;
 }
 
+/*!
+  \internal
+  Returns the key of the current item, as a \0-terminated C string.
+*/
+
+const char *QGCacheIterator::getKeyAscii() const
+{
+    QCacheItem *item = it->current();
+    return item ? (const char *)item->key : 0;
+}
+
+/*!
+  \internal
+  Returns the key of the current item, as a long.
+*/
+
+long QGCacheIterator::getKeyInt() const
+{
+    QCacheItem *item = it->current();
+    return item ? (long)item->key : 0;
+}
 
 /*!
   \internal
   Moves to the next item (postfix).
 */
 
-GCI QGCacheIterator::operator()()
+QCollection::Item QGCacheIterator::operator()()
 {
-    register QCacheItem *item = it->operator()();
+    QCacheItem *item = it->operator()();
     return item ? item->data : 0;
 }
 
@@ -639,9 +815,9 @@ GCI QGCacheIterator::operator()()
   Moves to the next item (prefix).
 */
 
-GCI QGCacheIterator::operator++()
+QCollection::Item QGCacheIterator::operator++()
 {
-    register QCacheItem *item = it->operator++();
+    QCacheItem *item = it->operator++();
     return item ? item->data : 0;
 }
 
@@ -650,9 +826,9 @@ GCI QGCacheIterator::operator++()
   Moves \e jumps positions forward.
 */
 
-GCI QGCacheIterator::operator+=( uint jump )
+QCollection::Item QGCacheIterator::operator+=( uint jump )
 {
-    register QCacheItem *item = it->operator+=(jump);
+    QCacheItem *item = it->operator+=(jump);
     return item ? item->data : 0;
 }
 
@@ -661,9 +837,9 @@ GCI QGCacheIterator::operator+=( uint jump )
   Moves to the previous item (prefix).
 */
 
-GCI QGCacheIterator::operator--()
+QCollection::Item QGCacheIterator::operator--()
 {
-    register QCacheItem *item = it->operator--();
+    QCacheItem *item = it->operator--();
     return item ? item->data : 0;
 }
 
@@ -672,8 +848,8 @@ GCI QGCacheIterator::operator--()
   Moves \e jumps positions backward.
 */
 
-GCI QGCacheIterator::operator-=( uint jump )
+QCollection::Item QGCacheIterator::operator-=( uint jump )
 {
-    register QCacheItem *item = it->operator-=(jump);
+    QCacheItem *item = it->operator-=(jump);
     return item ? item->data : 0;
 }
