@@ -1,41 +1,42 @@
 /****************************************************************************
-** $Id: qglobal.cpp,v 2.12 1997/09/30 10:11:23 hanord Exp $
+** $Id: qglobal.cpp,v 2.30.2.1 1999/07/02 18:20:22 paul Exp $
 **
 ** Global functions
 **
 ** Created : 920604
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qglobal.h"
-#include "qdict.h"
+#include "qasciidict.h"
 #include "qstring.h"
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
 
-RCSTAG("$Id: qglobal.cpp,v 2.12 1997/09/30 10:11:23 hanord Exp $");
-
-
 /*!
   \relates QApplication
   Returns the Qt version number for the library, typically "1.30"
   or "1.31".
 */
 
+Q_EXPORT
 const char *qVersion()
 {
     return QT_VERSION_STR;
@@ -58,10 +59,11 @@ static bool si_bigEndian;
   The \e *bigEndian is set to TRUE if this is a big-endian machine,
   or to FALSE if this is a little-endian machine.
 
-  This function calls fatal() with a message if the computer is truely weird
+  This function calls fatal() with a message if the computer is truly weird
   (i.e. different endianness for 16 bit and 32 bit integers).
 */
 
+Q_EXPORT
 bool qSysInfo( int *wordSize, bool *bigEndian )
 {
 #if defined(CHECK_NULL)
@@ -88,14 +90,14 @@ bool qSysInfo( int *wordSize, bool *bigEndian )
 	 *wordSize != 32 &&
 	 *wordSize != 16 ) {			// word size: 16, 32 or 64
 #if defined(CHECK_RANGE)
-	fatal( "qSysInfo: Unsupported system word size %d", *wordSize );
+	qFatal( "qSysInfo: Unsupported system word size %d", *wordSize );
 #endif
 	return FALSE;
     }
     if ( sizeof(Q_INT8) != 1 || sizeof(Q_INT16) != 2 || sizeof(Q_INT32) != 4 ||
 	 sizeof(float) != 4 || sizeof(double) != 8 ) {
 #if defined(CHECK_RANGE)
-	fatal( "qSysInfo: Unsupported system data type size" );
+	qFatal( "qSysInfo: Unsupported system data type size" );
 #endif
 	return FALSE;
     }
@@ -118,7 +120,7 @@ bool qSysInfo( int *wordSize, bool *bigEndian )
 
     if ( be16 != be32 ) {			// strange machine!
 #if defined(CHECK_RANGE)
-	fatal( "qSysInfo: Inconsistent system byte order" );
+	qFatal( "qSysInfo: Inconsistent system byte order" );
 #endif
 	return FALSE;
     }
@@ -139,24 +141,46 @@ static msg_handler handler = 0;			// pointer to debug handler
   Prints a debug message, or calls the message handler (if it has been
   installed).
 
-  This function takes a format string and a stack arguments, similar to
+  This function takes a format string and a list of arguments, similar to
   the C printf() function.
 
   Example:
   \code
-    debug( "my window handle = %x", myWidget->id() );
+    qDebug( "my window handle = %x", myWidget->id() );
   \endcode
 
   Under X11, the text is printed to stderr.  Under Windows, the text is
   sent to the debugger.
 
+  Note: If DEBUG was not defined when the Qt library was built
+  (i.e. NO_DEBUG was defined), this function does nothing.
+
   \warning The internal buffer is limited to 512 bytes (including the
-  0-terminator.
+  0-terminator).
 
   \sa warning(), fatal(), qInstallMsgHandler(),
   \link debug.html Debugging\endlink
 */
 
+Q_EXPORT
+void qDebug( const char *msg, ... )
+{
+    char buf[512];
+    va_list ap;
+    va_start( ap, msg );			// use variable arg list
+    if ( handler ) {
+	vsprintf( buf, msg, ap );
+	va_end( ap );
+	(*handler)( QtDebugMsg, buf );
+    } else {
+	vfprintf( stderr, msg, ap );
+	va_end( ap );
+	fprintf( stderr, "\n" );		// add newline
+    }
+}
+
+// copied... this looks really bad.
+Q_EXPORT
 void debug( const char *msg, ... )
 {
     char buf[512];
@@ -173,12 +197,14 @@ void debug( const char *msg, ... )
     }
 }
 
+
+
 /*!
   \relates QApplication
   Prints a warning message, or calls the message handler (if it has been
   installed).
 
-  This function takes a format string and a stack arguments, similar to
+  This function takes a format string and a list of arguments, similar to
   the C printf() function.
 
   Example:
@@ -186,7 +212,7 @@ void debug( const char *msg, ... )
     void f( int c )
     {
 	if ( c > 200 )
-	    warning( "f: bad argument, c == %d", c );
+	    qWarning( "f: bad argument, c == %d", c );
     }
   \endcode
 
@@ -194,12 +220,32 @@ void debug( const char *msg, ... )
   sent to the debugger.
 
   \warning The internal buffer is limited to 512 bytes (including the
-  0-terminator.
+  0-terminator).
 
   \sa debug(), fatal(), qInstallMsgHandler(),
   \link debug.html Debugging\endlink
 */
 
+Q_EXPORT
+void qWarning( const char *msg, ... )
+{
+    char buf[512];
+    va_list ap;
+    va_start( ap, msg );			// use variable arg list
+    if ( handler ) {
+	vsprintf( buf, msg, ap );
+	va_end( ap );
+	(*handler)( QtWarningMsg, buf );
+    } else {
+	vfprintf( stderr, msg, ap );
+	va_end( ap );
+	fprintf( stderr, "\n" );		// add newline
+    }
+}
+
+
+// again, copied
+Q_EXPORT
 void warning( const char *msg, ... )
 {
     char buf[512];
@@ -222,7 +268,7 @@ void warning( const char *msg, ... )
   Prints a fatal error message and exits, or calls the message handler (if it
   has been installed).
 
-  This function takes a format string and a stack arguments, similar to
+  This function takes a format string and a list of arguments, similar to
   the C printf() function.
 
   Example:
@@ -230,7 +276,7 @@ void warning( const char *msg, ... )
     int divide( int a, int b )
     {
 	if ( b == 0 )				// program error
-	    fatal( "divide: cannot divide by zero" );
+	    qFatal( "divide: cannot divide by zero" );
 	return a/b;
     }
   \endcode
@@ -239,12 +285,36 @@ void warning( const char *msg, ... )
   sent to the debugger.
 
   \warning The internal buffer is limited to 512 bytes (including the
-  0-terminator.
+  0-terminator).
 
   \sa debug(), warning(), qInstallMsgHandler(),
   \link debug.html Debugging\endlink
 */
 
+Q_EXPORT
+void qFatal( const char *msg, ... )
+{
+    char buf[512];
+    va_list ap;
+    va_start( ap, msg );			// use variable arg list
+    if ( handler ) {
+	vsprintf( buf, msg, ap );
+	va_end( ap );
+	(*handler)( QtFatalMsg, buf );
+    } else {
+	vfprintf( stderr, msg, ap );
+	va_end( ap );
+	fprintf( stderr, "\n" );		// add newline
+#if defined(UNIX) && defined(DEBUG)
+	abort();				// trap; generates core dump
+#else
+	exit( 1 );				// goodbye cruel world
+#endif
+    }
+}
+
+// yet again, copied
+Q_EXPORT
 void fatal( const char *msg, ... )
 {
     char buf[512];
@@ -266,6 +336,9 @@ void fatal( const char *msg, ... )
     }
 }
 
+
+
+
 /*!
   \fn void ASSERT( bool test )
   \relates QApplication
@@ -309,10 +382,6 @@ void fatal( const char *msg, ... )
 
   This is really a macro defined in qglobal.h.
 
-  \warning CHECK_PTR only works for the development release of the Qt
-  library.  In the release library, CHECK_PTR will be substituted with
-  nothing.
-
   Example:
   \code
     int *a;
@@ -330,71 +399,74 @@ void fatal( const char *msg, ... )
 // The CHECK_PTR macro calls this function to check if an allocation went ok.
 //
 
-bool chk_pointer( bool c, const char *n, int l )
+Q_EXPORT
+bool qt_check_pointer( bool c, const char *n, int l )
 {
     if ( c )
-	fatal( "In file %s, line %d: Out of memory", n, l );
+	qFatal( "In file %s, line %d: Out of memory", n, l );
     return TRUE;
 }
 
 
-Q_DECLARE(QDictM,int);
-
 static bool firstObsoleteWarning(const char *obj, const char *oldfunc )
 {
-    static bool firstWarning = TRUE;
-    static QDictM(int) obsoleteDict;
-    if ( firstWarning ) {
-	firstWarning = FALSE;
-	debug(
+    static QAsciiDict<int> *obsoleteDict = 0;
+    if ( !obsoleteDict ) {			// first time func is called
+	obsoleteDict = new QAsciiDict<int>;
+#if defined(DEBUG)
+	qDebug(
       "You are using obsolete functions in the Qt library. Call the function\n"
       "qSuppressObsoleteWarnings() to suppress obsolete warnings.\n"
 	     );
+#endif
     }
-    QString s( obj );
+    QCString s( obj );
     s += "::";
     s += oldfunc;
-    if ( obsoleteDict.find(s) == 0 ) {
-	obsoleteDict.insert( s, (int*) 666 );	// anything different from 0.
+    if ( obsoleteDict->find(s.data()) == 0 ) {
+	obsoleteDict->insert( s.data(), (int*)1 );	// anything different from 0
 	return TRUE;
-    } else {
-	return FALSE;
     }
+    return FALSE;
 }
 
 static bool suppressObsolete = FALSE;
 
+Q_EXPORT
 void qSuppressObsoleteWarnings( bool suppress )
 {
     suppressObsolete = suppress;
 }
 
+Q_EXPORT
 void qObsolete(	 const char *obj, const char *oldfunc, const char *newfunc )
 {
     if ( suppressObsolete )
 	return;
     if ( !firstObsoleteWarning(obj, oldfunc) )
 	return;
-    debug( "%s::%s: This function is obsolete, use %s instead",
+    qDebug( "%s::%s: This function is obsolete, use %s instead",
 	   obj, oldfunc, newfunc );
 }
 
+Q_EXPORT
 void qObsolete(	 const char *obj, const char *oldfunc )
 {
     if ( suppressObsolete )
 	return;
     if ( !firstObsoleteWarning(obj, oldfunc) )
 	return;
-    debug( "%s::%s: This function is obsolete.", obj, oldfunc );
+    qDebug( "%s::%s: This function is obsolete.", obj, oldfunc );
 }
 
+Q_EXPORT
 void qObsolete(	 const char *message )
 {
     if ( suppressObsolete )
 	return;
     if ( !firstObsoleteWarning( "Qt", message) )
 	return;
-    debug( "%s", message );
+    qDebug( "%s", message );
 }
 
 
@@ -420,7 +492,7 @@ void qObsolete(	 const char *message )
 
   Example:
   \code
-    #include <qapp.h>
+    #include <qapplication.h>
     #include <stdio.h>
     #include <stdlib.h>
 
@@ -451,9 +523,15 @@ void qObsolete(	 const char *message )
   \sa debug(), warning(), fatal(), \link debug.html Debugging\endlink
 */
 
+Q_EXPORT
 msg_handler qInstallMsgHandler( msg_handler h )
 {
     msg_handler old = handler;
     handler = h;
     return old;
 }
+
+
+#ifdef _WS_WIN_
+bool qt_winunicode=FALSE;
+#endif
