@@ -1,22 +1,25 @@
 /****************************************************************************
-** $Id: qgvector.cpp,v 2.4 1997/09/12 15:32:04 hanord Exp $
+** $Id: qgvector.cpp,v 2.17 1999/05/23 20:29:20 agulbra Exp $
 **
 ** Implementation of QGVector class
 **
 ** Created : 930907
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -24,12 +27,9 @@
 #include "qgvector.h"
 #include "qglist.h"
 #include "qstring.h"
-#include "qdstream.h"
+#include "qdatastream.h"
 #include <stdlib.h>
 
-RCSTAG("$Id: qgvector.cpp,v 2.4 1997/09/12 15:32:04 hanord Exp $");
-
-
 #define USE_MALLOC				// comment to use new/delete
 
 #undef NEW
@@ -49,18 +49,18 @@ RCSTAG("$Id: qgvector.cpp,v 2.4 1997/09/12 15:32:04 hanord Exp $");
   Default implementation of virtual functions
  *****************************************************************************/
 
-int QGVector::compareItems( GCI d1, GCI d2 )
+int QGVector::compareItems( Item d1, Item d2 )
 {
     return d1 != d2;				// compare pointers
 }
 
-QDataStream &QGVector::read( QDataStream &s, GCI &d )
+QDataStream &QGVector::read( QDataStream &s, Item &d )
 {						// read item from stream
     d = 0;
     return s;
 }
 
-QDataStream &QGVector::write( QDataStream &s, GCI ) const
+QDataStream &QGVector::write( QDataStream &s, Item ) const
 {						// write item to stream
     return s;
 }
@@ -84,16 +84,17 @@ QGVector::QGVector( uint size )			// create vectors with nullptrs
 	vec = 0;
 	return;
     }
-    vec = NEW(GCI,len);
+    vec = NEW(Item,len);
     CHECK_PTR( vec );
-    memset( (void*)vec, 0, len*sizeof(GCI) );	// fill with nulls
+    memset( (void*)vec, 0, len*sizeof(Item) );	// fill with nulls
 }
 
 QGVector::QGVector( const QGVector &a )		// make copy of other vector
+    : QCollection( a )
 {
     len = a.len;
     numItems = a.numItems;
-    vec = NEW(GCI,len);
+    vec = NEW(Item,len);
     CHECK_PTR( vec );
     for ( uint i=0; i<len; i++ ) {
 	vec[i] = a.vec[i] ? newItem( a.vec[i] ) : 0;
@@ -112,7 +113,7 @@ QGVector& QGVector::operator=( const QGVector &v )
     clear();					// first delete old vector
     len = v.len;
     numItems = v.numItems;
-    vec = NEW(GCI,len);				// create new vector
+    vec = NEW(Item,len);				// create new vector
     CHECK_PTR( vec );
     for ( uint i=0; i<len; i++ ) {		// copy elements
 	vec[i] = v.vec[i] ? newItem( v.vec[i] ) : 0;
@@ -122,11 +123,11 @@ QGVector& QGVector::operator=( const QGVector &v )
 }
 
 
-bool QGVector::insert( uint index, GCI d )	// insert item at index
+bool QGVector::insert( uint index, Item d )	// insert item at index
 {
 #if defined(CHECK_RANGE)
     if ( index >= len ) {			// range error
-	warning( "QGVector::insert: Index %d out of range", index );
+	qWarning( "QGVector::insert: Index %d out of range", index );
 	return FALSE;
     }
 #endif
@@ -149,7 +150,7 @@ bool QGVector::remove( uint index )		// remove item at index
 {
 #if defined(CHECK_RANGE)
     if ( index >= len ) {			// range error
-	warning( "QGVector::remove: Index %d out of range", index );
+	qWarning( "QGVector::remove: Index %d out of range", index );
 	return FALSE;
     }
 #endif
@@ -161,15 +162,15 @@ bool QGVector::remove( uint index )		// remove item at index
     return TRUE;
 }
 
-GCI QGVector::take( uint index )		// take out item
+QCollection::Item QGVector::take( uint index )		// take out item
 {
 #if defined(CHECK_RANGE)
     if ( index >= len ) {			// range error
-	warning( "QGVector::take: Index %d out of range", index );
+	qWarning( "QGVector::take: Index %d out of range", index );
 	return 0;
     }
 #endif
-    GCI d = vec[index];				// don't delete item
+    Item d = vec[index];				// don't delete item
     if ( d )
 	numItems--;
     vec[index] = 0;
@@ -212,28 +213,28 @@ bool QGVector::resize( uint newsize )		// resize array
 	    return TRUE;
 	}
 #if defined(DONT_USE_REALLOC)
-	GCI *newvec = NEW(GCI,newsize);		// manual realloc
-	memcpy( newvec, vec, (len < newsize ? len : newsize)*sizeof(GCI) );
+	Item *newvec = NEW(Item,newsize);		// manual realloc
+	memcpy( newvec, vec, (len < newsize ? len : newsize)*sizeof(Item) );
 	DELETE(vec);
 	vec = newvec;
 #else
-	vec = (GCI*)realloc( (char *)vec, newsize*sizeof(GCI) );
+	vec = (Item*)realloc( (char *)vec, newsize*sizeof(Item) );
 #endif
     } else {					// create new vector
-	vec = NEW(GCI,newsize);
+	vec = NEW(Item,newsize);
 	len = numItems = 0;
     }
     CHECK_PTR( vec );
     if ( !vec )					// no memory
 	return FALSE;
     if ( newsize > len )			// init extra space added
-	memset( (void*)&vec[len], 0, (newsize-len)*sizeof(GCI) );
+	memset( (void*)&vec[len], 0, (newsize-len)*sizeof(Item) );
     len = newsize;
     return TRUE;
 }
 
 
-bool QGVector::fill( GCI d, int flen )		// resize and fill vector
+bool QGVector::fill( Item d, int flen )		// resize and fill vector
 {
     if ( flen < 0 )
 	flen = len;				// default: use vector length
@@ -254,7 +255,7 @@ extern "C" {
 
 static int cmp_vec( const void *n1, const void *n2 )
 {
-    return sort_vec->compareItems( *((GCI*)n1), *((GCI*)n2) );
+    return sort_vec->compareItems( *((QCollection::Item*)n1), *((QCollection::Item*)n2) );
 }
 
 #if defined(Q_C_CALLBACKS)
@@ -266,9 +267,9 @@ void QGVector::sort()				// sort vector
 {
     if ( count() == 0 )				// no elements
 	return;
-    register GCI *start = &vec[0];
-    register GCI *end	= &vec[len-1];
-    GCI tmp;
+    register Item *start = &vec[0];
+    register Item *end	= &vec[len-1];
+    Item tmp;
     while ( TRUE ) {				// put all zero elements behind
 	while ( start < end && *start != 0 )
 	    start++;
@@ -283,17 +284,17 @@ void QGVector::sort()				// sort vector
 	}
     }
     sort_vec = (QGVector*)this;
-    qsort( vec, count(), sizeof(GCI), cmp_vec );
+    qsort( vec, count(), sizeof(Item), cmp_vec );
     sort_vec = 0;
 }
 
-int QGVector::bsearch( GCI d ) const		// binary search; when sorted
+int QGVector::bsearch( Item d ) const		// binary search; when sorted
 {
     if ( !len )
 	return -1;
     if ( !d ) {
 #if defined(CHECK_NULL)
-	warning( "QGVector::bsearch: Cannot search for null object" );
+	qWarning( "QGVector::bsearch: Cannot search for null object" );
 #endif
 	return -1;
     }
@@ -318,12 +319,12 @@ int QGVector::bsearch( GCI d ) const		// binary search; when sorted
 }
 
 
-int QGVector::findRef( GCI d, uint index) const // find exact item in vector
+int QGVector::findRef( Item d, uint index) const // find exact item in vector
 {
 #if defined(CHECK_RANGE)
     if ( index >= len ) {			// range error
-	warning( "QGVector::findRef: Index %d out of range", index );
-	return 0;
+	qWarning( "QGVector::findRef: Index %d out of range", index );
+	return -1;
     }
 #endif
     for ( uint i=index; i<len; i++ ) {
@@ -333,12 +334,12 @@ int QGVector::findRef( GCI d, uint index) const // find exact item in vector
     return -1;
 }
 
-int QGVector::find( GCI d, uint index ) const	// find equal item in vector
+int QGVector::find( Item d, uint index ) const	// find equal item in vector
 {
 #if defined(CHECK_RANGE)
     if ( index >= len ) {			// range error
-	warning( "QGVector::find: Index %d out of range", index );
-	return 0;
+	qWarning( "QGVector::find: Index %d out of range", index );
+	return -1;
     }
 #endif
     for ( uint i=index; i<len; i++ ) {
@@ -350,7 +351,7 @@ int QGVector::find( GCI d, uint index ) const	// find equal item in vector
     return -1;
 }
 
-uint QGVector::containsRef( GCI d ) const	// get number of exact matches
+uint QGVector::containsRef( Item d ) const	// get number of exact matches
 {
     uint count = 0;
     for ( uint i=0; i<len; i++ ) {
@@ -360,7 +361,7 @@ uint QGVector::containsRef( GCI d ) const	// get number of exact matches
     return count;
 }
 
-uint QGVector::contains( GCI d ) const		// get number of equal matches
+uint QGVector::contains( Item d ) const		// get number of equal matches
 {
     uint count = 0;
     for ( uint i=0; i<len; i++ ) {
@@ -373,18 +374,7 @@ uint QGVector::contains( GCI d ) const		// get number of equal matches
 }
 
 
-GCI QGVector::at( uint index ) const		// checked indexing
-{
-    if ( index >= len ) {
-#if defined(CHECK_RANGE)
-	warning( "QGVector::operator[]: Index %d out of range", index );
-#endif
-	index = 0;
-    }
-    return vec[index];
-}
-
-bool QGVector::insertExpand( uint index, GCI d )// insert and grow if necessary
+bool QGVector::insertExpand( uint index, Item d )// insert and grow if necessary
 {
     if ( index >= len ) {
 	if ( !resize( index+1 ) )		// no memory
@@ -404,6 +394,17 @@ void QGVector::toList( QGList *list ) const	// store items in list
     }
 }
 
+
+void QGVector::warningIndexRange( uint i )
+{
+#if defined(CHECK_RANGE)
+    qWarning( "QGVector::operator[]: Index %d out of range", i );
+#else
+    Q_UNUSED( i )
+#endif
+}
+
+
 /*****************************************************************************
   QGVector stream functions
  *****************************************************************************/
@@ -425,7 +426,7 @@ QDataStream &QGVector::read( QDataStream &s )	// read vector from stream
     clear();					// clear vector
     resize( num );
     for (uint i=0; i<num; i++) {		// read all items
-	GCI d;
+	Item d;
 	read( s, d );
 	CHECK_PTR( d );
 	if ( !d )				// no memory
