@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qiodevice.h,v 2.6.2.2 1998/08/21 19:13:24 hanord Exp $
+** $Id: qiodevice.h,v 2.16 1999/04/28 09:59:01 aavit Exp $
 **
 ** Definition of QIODevice class
 **
@@ -7,17 +7,19 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -26,6 +28,7 @@
 
 #ifndef QT_H
 #include "qglobal.h"
+#include "qcstring.h"
 #endif // QT_H
 
 
@@ -68,10 +71,7 @@
 #define IO_ConnectError		5		// cannot connect to device
 #define IO_AbortError		6		// abort error
 #define IO_TimeOutError		7		// time out
-
-#undef	TB
-#define TB(x) ((ioMode & (x)) == (x))
-
+#define IO_UnspecifiedError		8		// unspecified error
 
 class Q_EXPORT QIODevice					// IO device class
 {
@@ -83,17 +83,17 @@ public:
     int		 mode()	  const { return ioMode & IO_ModeMask; }
     int		 state()  const { return ioMode & IO_StateMask; }
 
-    bool	 isDirectAccess()     const { return TB(IO_Direct); }
-    bool	 isSequentialAccess() const { return TB(IO_Sequential); }
-    bool	 isCombinedAccess()   const { return TB(IO_Combined); }
-    bool	 isBuffered()	      const { return !TB(IO_Raw); }
-    bool	 isRaw()	      const { return TB(IO_Raw); }
-    bool	 isSynchronous()      const { return !TB(IO_Async); }
-    bool	 isAsynchronous()     const { return TB(IO_Async); }
-    bool	 isTranslated()	      const { return TB(IO_Translate); }
-    bool	 isReadable()	      const { return TB(IO_ReadOnly); }
-    bool	 isWritable()	      const { return TB(IO_WriteOnly); }
-    bool	 isReadWrite()	      const { return TB(IO_ReadWrite); }
+    bool	 isDirectAccess()     const { return ((ioMode & IO_Direct)     == IO_Direct); }
+    bool	 isSequentialAccess() const { return ((ioMode & IO_Sequential) == IO_Sequential); }
+    bool	 isCombinedAccess()   const { return ((ioMode & IO_Combined)   == IO_Combined); }
+    bool	 isBuffered()	      const { return ((ioMode & IO_Raw)        != IO_Raw); }
+    bool	 isRaw()	      const { return ((ioMode & IO_Raw)        == IO_Raw); }
+    bool	 isSynchronous()      const { return ((ioMode & IO_Async)      != IO_Async); }
+    bool	 isAsynchronous()     const { return ((ioMode & IO_Async)      == IO_Async); }
+    bool	 isTranslated()	      const { return ((ioMode & IO_Translate)  == IO_Translate); }
+    bool	 isReadable()	      const { return ((ioMode & IO_ReadOnly)   == IO_ReadOnly); }
+    bool	 isWritable()	      const { return ((ioMode & IO_WriteOnly)  == IO_WriteOnly); }
+    bool	 isReadWrite()	      const { return ((ioMode & IO_ReadWrite)  == IO_ReadWrite); }
     bool	 isInactive()	      const { return state() == 0; }
     bool	 isOpen()	      const { return state() == IO_Open; }
 
@@ -110,9 +110,11 @@ public:
     virtual bool atEnd()  const;
     bool	 reset() { return at(0); }
 
-    virtual int	 readBlock( char *data, uint len ) = 0;
+    virtual int	 readBlock( char *data, uint maxlen ) = 0;
     virtual int	 writeBlock( const char *data, uint len ) = 0;
     virtual int	 readLine( char *data, uint maxlen );
+    int writeBlock( const QByteArray& data );
+    QByteArray readAll();
 
     virtual int	 getch() = 0;
     virtual int	 putch( int ) = 0;
@@ -124,7 +126,7 @@ protected:
     void	 setMode( int );
     void	 setState( int );
     void	 setStatus( int );
-    int		 index;
+    int		 ioIndex;
 
 private:
     int		 ioMode;
@@ -138,7 +140,4 @@ private:	// Disabled copy constructor and operator=
 };
 
 
-#undef	TB
-
-
 #endif // QIODEVICE_H
