@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qregexp.cpp,v 2.12.2.2 1998/11/26 11:14:54 aavit Exp $
+** $Id: qregexp.cpp,v 2.32 1999/06/10 14:48:04 warwick Exp $
 **
 ** Implementation of QRegExp class
 **
@@ -7,17 +7,19 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -29,8 +31,9 @@
   \class QRegExp qregexp.h
   \ingroup tools
   \ingroup misc
-  \brief The QRegExp class provides pattern matching using regular expressions and wildcards.
+  \brief The QRegExp class provides pattern matching using regular expressions or wildcards.
 
+  
   QRegExp knows these regexp primitives:
   <ul plain>
   <li><dfn>c</dfn> matches the character 'c'
@@ -46,13 +49,16 @@
   <li><dfn>\t</dfn> matches the TAB character (9)
   <li><dfn>\n</dfn> matches newline (10)
   <li><dfn>\r</dfn> matches return (13)
-  <li><dfn>\s</dfn> matches white space (9,10,11,12,13,32)
+  <li><dfn>\s</dfn> matches white space (defined as any character
+  where QChar::isSpace() returns TRUE. This includes ASCII characters
+  9 (TAB), 10 (LF), 11 (VT), 12(FF), 13 (CR), and 32 (Space)).
   <li><dfn>\x12</dfn> matches the character 0x12 (18 decimal, 12 hexadecimal).
   <li><dfn>\022</dfn> matches the character 022 (18 decimal, 22 octal).
   </ul>
 
-  In wildcard mode, it only knows three primitives:
+  In wildcard mode, it only knows four primitives:
   <ul plain>
+  <li><dfn>c</dfn> matches the character 'c'
   <li><dfn>?</dfn> matches any character
   <li><dfn>*</dfn> matches any sequence of characters
   <li><dfn>[]</dfn> matches a defined set of characters,
@@ -60,28 +66,50 @@
     and dot, and [^z] matches everything except lower-case z.
   </ul>
 
+  QRegExp supports Unicode both in the pattern strings and in the
+  strings to be matched.
+
   When writing regular expressions in C++ code, remember that the C++
-  preprocessor processes \ characters.  So in order to match a "."
+  preprocessor processes \ characters.  So in order to match e.g. a "."
   character, you must write "\\." in C++ source, not "\.".
+
+  \bug Case insensitive matching is not supported for non-ASCII
+  (non-8bit) characters. Any charcter with a non-zero QChar.row() is
+  matched case sensitively even if the QRegExp is in case insensitive
+  mode.
 */
 
 
 //
-// The regexp pattern is internally represented as an array of ushorts,
-// each element containing an 8-bit character or a 16-bit code (listed below).
-// Character classes are encoded as 256 bits, i.e. 16*16 bits.
+// The regexp pattern is internally represented as an array of uints,
+// each element containing an 16-bit character or a 32-bit code
+// (listed below).  User-defined character classes (e.g. [a-zA-Z])
+// are encoded as this:
+// uint no:	1		2		3		...
+// value:	CCL | n		from | to	from | to
 //
+// where n is the (16-bit) number of following range definitions, and
+// from and to define the ranges inclusive. from <= to is always true,
+// otherwise it is a built-in charclass (Pxx, eg \s - PWS). Single
+// characters in the class are coded as from==to.  Negated classes
+// (e.g. [^a-z]) use CCN instead of CCL.
 
-const ushort CHR	= 0x4000;		// character
-const ushort BOL	= 0x8001;		// beginning of line	^
-const ushort EOL	= 0x8002;		// end of line		$
-const ushort BOW	= 0x8003;		// beginning of word	\<
-const ushort EOW	= 0x8004;		// end of word		\>
-const ushort ANY	= 0x8005;		// any character	.
-const ushort CCL	= 0x8006;		// character class	[]
-const ushort CLO	= 0x8007;		// Kleene closure	*
-const ushort OPT	= 0x8008;		// Optional closure	?
-const ushort END	= 0x0000;
+const uint END	= 0x00000000;
+const uint PWS	= 0x10010000;		// predef whitespace charclass \s
+const uint CCL	= 0x20010000;		// character class	[]
+const uint CCN	= 0x20020000;		// neg character class	[^]
+const uint CHR	= 0x40000000;		// character
+const uint BOL	= 0x80010000;		// beginning of line	^
+const uint EOL	= 0x80020000;		// end of line		$
+const uint BOW	= 0x80030000;		// beginning of word	\<
+const uint EOW	= 0x80040000;		// end of word		\>
+const uint ANY	= 0x80050000;		// any character	.
+const uint CLO	= 0x80070000;		// Kleene closure	*
+const uint OPT	= 0x80080000;		// Optional closure	?
+
+const uint MCC  = 0x20000000;		// character class bitmask
+const uint MCD  = 0xffff0000;		// code mask
+const uint MVL  = 0x0000ffff;		// value mask
 
 //
 // QRegExp::error codes (internal)
@@ -122,7 +150,7 @@ QRegExp::QRegExp()
   \sa setWildcard()
 */
 
-QRegExp::QRegExp( const char *pattern, bool caseSensitive, bool wildcard )
+QRegExp::QRegExp( const QString &pattern, bool caseSensitive, bool wildcard )
 {
     rxstring = pattern;
     rxdata = 0;
@@ -162,7 +190,7 @@ QRegExp::~QRegExp()
 
 QRegExp &QRegExp::operator=( const QRegExp &r )
 {
-    rxstring = (const char *)r.rxstring;
+    rxstring = r.rxstring;
     cs = r.cs;
     wc = r.wc;
     compile();
@@ -174,7 +202,7 @@ QRegExp &QRegExp::operator=( const QRegExp &r )
   The case sensitivity or wildcard options do not change.
 */
 
-QRegExp &QRegExp::operator=( const char *pattern )
+QRegExp &QRegExp::operator=( const QString &pattern )
 {
     rxstring = pattern;
     compile();
@@ -269,7 +297,7 @@ void QRegExp::setCaseSensitive( bool enable )
 
 
 /*!
-  \fn const char *QRegExp::pattern() const
+  \fn QString QRegExp::pattern() const
   Returns the pattern string of the regexp.
 */
 
@@ -281,44 +309,59 @@ void QRegExp::setCaseSensitive( bool enable )
   If \e len is not a null pointer, the length of the match is stored in
   \e *len.
 
+  If \e indexIsStart is TRUE (the default), the position \e index in
+  the string will match the start-of-input primitive (^) in the
+  regexp, if present. Otherwise, position 0 in \e str will match.
+
   Example:
   \code
-    QRegExp r("[0-9]*\.[0-9]+");		// matches floating point
+    QRegExp r("[0-9]*\\.[0-9]+");		// matches floating point
     int len;
     r.match("pi = 3.1416", 0, &len);		// returns 5, len == 6
   \endcode
 */
 
-int QRegExp::match( const char *str, int index, int *len ) const
+int QRegExp::match( const QString &str, int index, int *len,
+		    bool indexIsStart ) const
 {
-    if ( error || str == 0 )			// cannot match
+    if ( !isValid() || isEmpty() )
 	return -1;
-    register char *p = (char *)str + index;
-    ushort *d  = rxdata;
-    char   *ep = 0;
+    if ( str.length() < (uint)index )
+	return -1;
+    const QChar *start = str.unicode();
+    const QChar *p = start + index;
+    uint pl = str.length() - index;
+    uint *d  = rxdata;
+    const QChar *ep = 0;
 
     if ( *d == BOL ) {				// match from beginning of line
-	ep = matchstr( d, p, p );
+	ep = matchstr( d, p, pl, indexIsStart ? p : start );
     } else {
 	if ( *d & CHR ) {
-	    if ( cs ) {				// case sensitive
-		while ( *p && *p != (char)*d )
+	    QChar c( *d );
+	    if ( !cs && !c.row() ) {		// case sensitive, # only 8bit
+		while ( pl && ( p->row() || tolower(p->cell()) != c.cell() ) ) {
 		    p++;
+		    pl--;
+		}
 	    } else {				// case insensitive
-		while ( *p && tolower(*p) != (char)*d )
+		while ( pl && *p != c ) {
 		    p++;
+		    pl--;
+		}
 	    }
 	}
-	while ( *p ) {				// regular match
-	    if ( (ep=matchstr(d,p,(char*)str+index)) )
+	while ( pl ) {				// regular match
+	    if ( (ep=matchstr(d,p,pl, indexIsStart ? start+index : start )) )
 		break;
 	    p++;
+	    pl--;
 	}
     }
     if ( ep ) {					// match
 	if ( len )
 	    *len = ep - p;
-	return (int)((long)p - (long)str);	// return index
+	return (int)(p - start);		// return index
     } else {					// no match
 	if ( len )
 	    *len = 0;
@@ -328,7 +371,43 @@ int QRegExp::match( const char *str, int index, int *len ) const
 
 static inline bool iswordchar( int x )
 {
-    return isalnum(x) || x == '_';
+    return isalnum(x) || x == '_';	//# Only 8-bit support
+}
+
+
+/*!
+  \internal
+  Match character class
+*/
+
+bool matchcharclass( uint *rxd, QChar c )
+{
+    uint *d = rxd;
+    uint clcode = *d & MCD;
+    bool neg = clcode == CCN;
+    if ( clcode != CCL && clcode != CCN)
+	qWarning("QRegExp: Internal error, please report to qt-bugs@troll.no");
+    uint numFields = *d & MVL;
+    uint cval = (((uint)(c.row())) << 8) | ((uint)c.cell());
+    bool found = FALSE;
+    for ( int i = 0; i < (int)numFields; i++ ) {
+	d++;
+	if ( *d == PWS ) {
+	    if ( c.isSpace() ) {
+		found = TRUE;
+		break;
+	    }
+	}
+	else {
+	    uint from = ( *d & MCD ) >> 16;
+	    uint to = *d & MVL;
+	    if ( (cval >= from) && (cval <= to) ) {
+		found = TRUE;
+		break;
+	    }
+	}
+    }
+    return neg ? !found : found;
 }
 
 
@@ -337,39 +416,60 @@ static inline bool iswordchar( int x )
   Recursively match string.
 */
 
-char *QRegExp::matchstr( ushort *rxd, char *str, char *bol ) const
+const QChar *QRegExp::matchstr( uint *rxd, const QChar *str, uint strlength,
+				const QChar *bol ) const
 {
-    register char *p = str;
-    ushort *d = rxd;
+    const QChar *p = str;
+    uint pl = strlength;
+    uint *d = rxd;
+
+    //### in all cases here: handle pl == 0! (don't read past strlen)
     while ( *d ) {
 	if ( *d & CHR ) {			// match char
-	    if ( cs ) {				// case sensitive
-		if ( *p != (char)*d )
+	    if ( !pl )
+		return 0;
+	    QChar c( *d );
+	    if ( !cs && !c.row() ) {		// case insensitive, #Only 8bit
+		if ( p->row() || tolower(p->cell()) != c.cell() )
 		    return 0;
 		p++;
+		pl--;
 	    } else {				// case insensitive
-		if ( tolower(*p) != (char)*d )
+		if ( *p != c )
 		    return 0;
 		p++;
+		pl--;
 	    }
 	    d++;
-	} else switch ( *d++ ) {
-	    case ANY:				// match anything
-		if ( !*p++ )
-		    return 0;
-		break;
-	    case CCL:				// match char class
-		if ( (d[(uchar)*p >> 4] & (1 << ((uchar)*p & 0xf))) == 0 )
+	}
+	else if ( *d & MCC ) {			// match char class
+	    if ( !pl )
+		return 0;
+	    if ( !matchcharclass( d, *p ) )
+		return 0;
+	    p++;
+	    pl--;
+	    d += (*d & MVL) + 1;
+	}
+	else switch ( *d++ ) {
+	    case PWS:				// match whitespace
+		if ( !pl || !p->isSpace() )
 		    return 0;
 		p++;
-		d += 16;
+		pl--;
+		break;
+	    case ANY:				// match anything
+		if ( !pl )
+		    return 0;
+		p++;
+		pl--;
 		break;
 	    case BOL:				// match beginning of line
 		if ( p != bol )
 		    return 0;
 		break;
 	    case EOL:				// match end of line
-		if ( *p )
+		if ( pl )
 		    return 0;
 		break;
 	    case BOW:				// match beginning of word
@@ -382,78 +482,109 @@ char *QRegExp::matchstr( ushort *rxd, char *str, char *bol ) const
 		break;
 	    case CLO:				// Kleene closure
 		{
-		char *first_p = p;
+		const QChar *first_p = p;
 		if ( *d & CHR ) {		// match char
-		    if ( cs ) {			// case sensitive
-			while ( *p && *p == (char)*d )
+		    QChar c( *d );
+		    if ( !cs && !c.row() ) {	// case insensitive, #only 8bit
+			while ( pl && !p->row() && tolower(p->cell())==c.cell() ) {
 			    p++;
+			    pl--;
+			}
 		    }
-		    else {			// case insensitive
-			while ( *p && tolower(*p) == (char)*d )
+		    else {			// case sensitive
+			while ( pl && *p == c ) {
 			    p++;
+			    pl--;
+			}
 		    }
-		} else switch ( *d ) {
-		    case ANY:
-			while ( *p )
-			    p++;
-			break;
-		    case CCL:
-			d++;
-			while ( *p && d[(uchar)*p >> 4] &
-				(1 << ((uchar)*p & 0xf)) )
-			    p++;
-			d += 15;
-			break;
-		    default:			// error
-			return 0;
+		    d++;
 		}
-		d++;
-		d++;
-		char *end;
+		else if ( *d & MCC ) {			// match char class
+		    while( pl && matchcharclass( d, *p ) ) {
+			p++;
+			pl--;
+		    }
+		    d += (*d & MVL) + 1;
+		}
+		else if ( *d == PWS ) {
+		    while ( pl && p->isSpace() ) {
+			p ++;
+			pl--;
+		    }
+		    d++;
+		}
+		else if ( *d == ANY ) {
+		    p += pl;
+		    pl = 0;
+		    d++;
+		}
+		else {
+		    return 0;			// error
+		}
+		d++;				// skip CLO's END
+		const QChar *end;
 		while ( p >= first_p ) {	// go backwards
-		    if ( (end = matchstr(d,p,bol)) )
+		    if ( (end = matchstr(d,p,pl,bol)) )
 			return end;
 		    --p;
+		    ++pl;
 		}
 		}
 		return 0;
 	    case OPT:				// optional closure
 		{
-		char *first_p = p;
+		const QChar *first_p = p;
 		if ( *d & CHR ) {		// match char
-		    if ( cs ) {			// case sensitive
-			if ( *p && *p == (char)*d )
-			    p++;
-		    } else {			// case insensitive
-			if ( *p && tolower(*p) == (char)*d )
+		    QChar c( *d );
+		    if ( !cs && !c.row() ) {	// case insensitive, #only 8bit
+			if ( pl && !p->row() && tolower(p->cell()) == c.cell() ) {
 			    p++;
+			    pl--;
+			}
 		    }
-		}
-		else switch ( *d ) {
-		    case ANY:
-			if ( *p )
+		    else {			// case sensitive
+			if ( pl && *p == c ) {
 			    p++;
-			break;
-		    case CCL:
-			d++;
-			if ( *p &&
-			     d[(uchar)*p >> 4] & (1 << ((uchar)*p & 0xf)) )
-			    p++;
-			d += 15;
-			break;
-		    default:			// error
-			return 0;
+			    pl--;
+			}
+		    }
+		    d++;
 		}
-		d++;
-		d++;
-		char *end;
+		else if ( *d & MCC ) {			// match char class
+		    if ( pl && matchcharclass( d, *p ) ) {
+			p++;
+			pl--;
+		    }
+		    d += (*d & MVL) + 1;
+		}
+		else if ( *d == PWS ) {
+		    if ( pl && p->isSpace() ) {
+			p++;
+			pl--;
+		    }
+		    d++;
+		}
+		else if ( *d == ANY ) {
+		    if ( pl ) {
+			p++;
+			pl--;
+		    }
+		    d++;
+		}
+		else {
+		    return 0;			// error
+		}
+		d++;				// skip OPT's END
+		const QChar *end;
 		while ( p >= first_p ) {	// go backwards
-		    if ( (end = matchstr(d,p,bol)) )
+		    if ( (end = matchstr(d,p,pl,bol)) )
 			return end;
 		    --p;
+		    ++pl;
 		}
 		}
 		return 0;
+
 	    default:				// error
 		return 0;
 	}
@@ -467,13 +598,15 @@ char *QRegExp::matchstr( ushort *rxd, char *str, char *bol ) const
 // Ex:	 *.cpp	==> ^.*\.cpp$
 //
 
-static QString wc2rx( const char *pattern )
+static QString wc2rx( const QString &pattern )
 {
-    register char *p = (char *)pattern;
-    QString wcpattern = "^";
-    char c;
-    while ( (c=*p++) ) {
-	switch ( c ) {
+    int patlen = (int)pattern.length();
+    QString wcpattern = QString::fromLatin1("^");
+
+    QChar c;
+    for( int i = 0; i < patlen; i++ ) {
+	c = pattern[i];
+	switch ( (char)c ) {
 	case '*':				// '*' ==> '.*'
 	    wcpattern += '.';
 	    break;
@@ -488,13 +621,15 @@ static QString wc2rx( const char *pattern )
 	    wcpattern += '\\';
 	    break;
 	case '[':
-	    if ( *p == '^' ) {
+	    if ( (char)pattern[i+1] == '^' ) { // don't quote '^' after '['
 		wcpattern += '[';
-		c = *p++;
+		c = pattern[i+1];
+		i++;
 	    }
 	    break;
 	}
 	wcpattern += c;
+
     }
     wcpattern += '$';
     return wcpattern;				// return new regexp pattern
@@ -505,124 +640,157 @@ static QString wc2rx( const char *pattern )
 // Internal: Get char value and increment pointer.
 //
 
-static int char_val( char **str )		// get char value
+static uint char_val( const QChar **str, uint *strlength )   // get char value
 {
-    register char *p = *str;
-    int len = 1;
-    int v = 0;
-    if ( *p == '\\' ) {				// escaped code
+    const QChar *p = *str;
+    uint pl = *strlength;
+    uint len = 1;
+    uint v = 0;
+    if ( (char)*p == '\\' ) {			// escaped code
 	p++;
-	if ( *p == 0 ) {			// it is just a '\'
+	pl--;
+	if ( !pl ) {				// it is just a '\'
 	    (*str)++;
+	    (*strlength)--;
 	    return '\\';
 	}
 	len++;					// length at least 2
-	switch ( tolower(*p) ) {
+	int i;
+	char c;
+	char ch = tolower((char)*p);
+	switch ( ch ) {
 	    case 'b':  v = '\b';  break;	// bell
 	    case 'f':  v = '\f';  break;	// form feed
 	    case 'n':  v = '\n';  break;	// newline
 	    case 'r':  v = '\r';  break;	// return
 	    case 't':  v = '\t';  break;	// tab
+	    case 's':  v = PWS; break;		// whitespace charclass
+	    case '<':  v = BOW; break;		// word beginning matcher
+	    case '>':  v = EOW; break;		// word ending matcher
 
 	    case 'x': {				// hex code
 		p++;
-		int  c = tolower(*p);
-		bool a = c >= 'a' && c <= 'f';
-		if ( isdigit(c) || a ) {	// hex digit?
-		    v = a ? 10 + c - 'a' : c - '0';
-		    len++;
+		pl--;
+		for ( i = 0; (i < 4) && pl; i++ ) {	//up to 4 hex digits
+		    c = tolower((char)*p);
+		    bool a = ( c >= 'a' && c <= 'f' );
+		    if ( (c >= '0' && c <= '9') || a ) {
+			v <<= 4;
+			v += a ? 10 + c - 'a' : c - '0';
+			len++;
+		    }
+		    else {
+			break;
+		    }
+		    p++;
+		    pl--;
 		}
-		p++;
-		c = tolower(*p);
-		a = c >= 'a' && c <= 'f';
-		if ( isdigit(c) || a ) {	// another hex digit?
-		    v *= 16;
-		    v += a ? 10 + c - 'a' : c - '0';
-		    len++;
-		}
-		}
-		break;
+	    }
+	    break;
 
 	    default: {
-		int i;
-		--len;				// first check if octal
-		for ( i=0; i<3 && *p >= '0' && *p <= '7'; i++ ) {
-		    v *= 8;
-		    v += *p++ - '0';
-		    len++;
+		if ( ch >= '0' && ch <= '7' ) {	//octal code
+		    len--;
+		    for ( i = 0; (i < 3) && pl; i++ ) {	// up to 3 oct digits
+			c = tolower((char)*p);
+			if ( c >= '0' && c <= '7' ) {
+			    v <<= 3;
+			    v += c - '0';
+			    len++;
+			}
+			else {
+			    break;
+			}
+			p++;
+			pl--;
+		    }
 		}
-		if ( i == 0 ) {			// not an octal number
-		    v = *p;
-		    len++;
+		else {				// not an octal number
+		    v = (((uint)(p->row())) << 8) | ((uint)p->cell());
 		}
 	    }
 	}
     } else {
-	v = (uchar)*p;
+	v = (((uint)(p->row())) << 8) | ((uint)p->cell());
     }
     *str += len;
+    *strlength -= len;
     return v;
 }
 
 
 #if defined(DEBUG)
-static ushort *dump( ushort *p )
+static uint *dump( uint *p )
 {
     while ( *p != END ) {
 	if ( *p & CHR ) {
-	    debug( "\tCHR\t%c (%d)", *p&0xff, *p&0xff );
+	    QChar uc = (QChar)*p;
+	    char c = (char)uc;
+	    uint u = (((uint)(uc.row())) << 8) | ((uint)uc.cell());
+	    qDebug( "\tCHR\tU%04x (%c)", u, (c ? c : ' '));
 	    p++;
-	} else switch ( *p++ ) {
+	}
+	else if ( *p & MCC ) {
+	    uint clcode = *p & MCD;
+	    uint numFields = *p & MVL;
+	    if ( clcode == CCL )
+		qDebug( "\tCCL\t%i", numFields );
+	    else if ( clcode == CCN )
+		qDebug( "\tCCN\t%i", numFields );
+	    else
+		qDebug("coding error!");
+	    for ( int i = 0; i < (int)numFields; i++ ) {
+		p++;
+		if ( *p == PWS )
+		    qDebug( "\t\tPWS" );
+		else {
+		    uint from = ( *p & MCD ) >> 16;
+		    uint to = *p & MVL;
+		    char fc = (char)QChar(from);
+		    char tc = (char)QChar(to);
+		    qDebug( "\t\tU%04x (%c) - U%04x (%c)", from,
+			   (fc ? fc : ' '), to, (tc ? tc : ' ') );
+		}
+	    }
+	    p++;
+	}
+	else switch ( *p++ ) {
+	    case PWS:
+		qDebug( "\tPWS" );
+		break;
 	    case BOL:
-		debug( "\tBOL" );
+		qDebug( "\tBOL" );
 		break;
 	    case EOL:
-		debug( "\tEOL" );
+		qDebug( "\tEOL" );
 		break;
 	    case BOW:
-		debug( "\tBOW" );
+		qDebug( "\tBOW" );
 		break;
 	    case EOW:
-		debug( "\tEOW" );
+		qDebug( "\tEOW" );
 		break;
 	    case ANY:
-		debug( "\tANY" );
-		break;
-	    case CCL: {
-		QString s = "";
-		QString buf;
-		for ( int n=0; n<256; n++ ) {
-		    if ( p[n >> 4] & (1 << (n & 0xf)) ) {
-			if ( isgraph(n) )
-			    s += (char)n;
-			else {
-			    buf.sprintf( "\\x%.2X", n );
-			    s += buf;
-			}
-		    }
-		}
-		debug( "\tCCL\t%s", (const char *)s );
-		p += 16;
-		}
+		qDebug( "\tANY" );
 		break;
 	    case CLO:
-		debug( "\tCLO" );
+		qDebug( "\tCLO" );
 		p = dump( p );
 		break;
 	    case OPT:
-		debug( "\tOPT" );
+		qDebug( "\tOPT" );
 		p = dump( p );
 		break;
 	}
     }
-    debug( "\tEND" );
+    qDebug( "\tEND" );
     return p+1;
 }
 #endif // DEBUG
 
 
 static const int maxlen = 1024;			// max length of regexp array
-static ushort rxarray[ maxlen ];		// tmp regexp array
+static uint rxarray[ maxlen ];			// tmp regexp array
 
 /*!
   \internal
@@ -649,101 +817,99 @@ void QRegExp::compile()
 	pattern = wc2rx(rxstring);
     else
 	pattern = rxstring;
-    char   *p = pattern.data();			// pattern pointer
-    ushort *d = rxarray;			// data pointer
-    ushort *prev_d = 0;
+    const QChar *start = pattern.unicode();	// pattern pointer
+    const QChar *p = start;			// pattern pointer
+    uint pl = pattern.length();
+    uint *d = rxarray;				// data pointer
+    uint *prev_d = 0;
 
 #define GEN(x)	*d++ = (x)
 
-    while ( *p ) {
-	switch ( *p ) {
+    while ( pl ) {
+	char ch = (char)*p;
+	switch ( ch ) {
 
 	    case '^':				// beginning of line
 		prev_d = d;
-		GEN( p == pattern.data() ? BOL : *p );
+		GEN( p == start ? BOL : (CHR | ch) );
 		p++;
+		pl--;
 		break;
 
 	    case '$':				// end of line
 		prev_d = d;
-		GEN( *(p+1) == 0 ? EOL : *p );
+		GEN( pl == 1 ? EOL : (CHR | ch) );
 		p++;
+		pl--;
 		break;
 
 	    case '.':				// any char
 		prev_d = d;
 		GEN( ANY );
 		p++;
+		pl--;
 		break;
 
 	    case '[':				// character class
 		{
-		char cc[256];
-		char neg;			// mask for CCL
 		prev_d = d;
-		GEN( CCL );
 		p++;
-		memset( cc, 0, 256 );		// reset char class array
-		if ( *p == '^' ) {		// negate!
-		    neg = 1;
-		    p++;
-		} else {
-		    neg = 0;
-		}
-		if ( *p == ']' )		// bracket, not end
-		    cc[(uchar)(*p++)] = 1;
-		int prev_c = -1;
-		while ( *p && *p != ']' ) {	// scan the char set
-		    if ( *p == '-' && *(p+1) && *(p+1) != ']' ) {
-			p++;			// range!
-			if ( prev_c == -1 ) {	// no previous char
-			    cc[(uchar)'-'] = 1;
-			} else {
-			    int start = prev_c;
-			    int stop = char_val( &p );
-			    if ( start > stop ) { // swap start and stop
-				int tmp = start;
-				start = stop;
-				stop = tmp;
-			    }
-			    while ( start++ < stop )
-				cc[start] = 1;
-			}
-		    } else if ( *p == '\\' && *(p+1) && *(p+1) == 's' ) {
-			// white space primitive
-			cc[9] = 1;
-			cc[10] = 1;
-			cc[11] = 1;
-			cc[12] = 1;
-			cc[13] = 1;
-			cc[32] = 1;
-			p++;
-			p++;
-		    }
-		    else {			// normal char
-			cc[(prev_c=char_val(&p))] = 1;
-		    }
-		}
-		if ( *p != ']' ) {		// missing close bracket
+		pl--;
+		if ( !pl ) {
 		    error = PatSyntax;
 		    return;
 		}
-		if ( d + 16 >= rxarray + maxlen ) {
-		    error = PatOverflow;	// pattern too long
-		    return;
+		bool firstIsEscaped = ( (char)*p == '\\' );
+		uint cch = char_val( &p, &pl );
+		if ( cch == '^' && !firstIsEscaped ) {	// negate!
+		    GEN( CCN );
+		    if ( !pl ) {
+			error = PatSyntax;
+			return;
+		    }
+		    cch = char_val( &p, &pl );
+		} else {
+		    GEN( CCL );
 		}
-		memset( d, 0, 16*sizeof(ushort) );
-		for ( int i=0; i<256; i++ ) {	// set bits
-		    if ( cc[i] ^ neg ) {
-			d[i >> 4] |= (1 << (i & 0xf));
-			if ( !cs && isalpha(i) ) {
-			    int j = islower(i) ? toupper(i) : tolower(i);
-			    d[j >> 4] |= (1 << (j & 0xf));
+		uint numFields = 0;
+		while ( pl ) {
+		    if ((pl>2) && ((char)*p == '-') && ((char)*(p+1) != ']')) {
+			// Found a range
+		       	char_val( &p, &pl ); // Read the '-'
+			uint cch2 = char_val( &p, &pl ); // Read the range end
+			if ( cch > cch2 ) { 		// swap start and stop
+			    int tmp = cch;
+			    cch = cch2;
+			    cch2 = tmp;
 			}
+			GEN( (cch << 16) | cch2 );	// from < to
+			numFields++;
+		    }
+		    else {
+			// Found a single character
+			if ( cch & MCD ) // It's a code; will not be mistaken
+			    GEN( cch );	 // for a range, since from > to
+			else
+			    GEN( (cch << 16) | cch ); // from == to range
+			numFields++;
+		    }
+		    if ( d >= rxarray + maxlen ) {	// pattern too long
+			error = PatOverflow;		
+			return;
+		    }
+		    if ( !pl ) {		// At least ']' should be left
+			error = PatSyntax;
+			return;
+		    }
+		    cch = char_val( &p, &pl );
+		    if ( cch == (uint)']' )
+			break;
+		    if ( !pl ) {		// End, should have seen ']'
+			error = PatSyntax;
+			return;
 		    }
 		}
-		d += 16;
-		p++;
+		*prev_d |= numFields;		// Store number of fields
 		}
 		break;
 
@@ -770,45 +936,31 @@ void QRegExp::compile()
 			error = PatOverflow;	// pattern too long
 			return;
 		    }
-		    memcpy( d, prev_d, ddiff*sizeof(ushort) );
+		    memcpy( d, prev_d, ddiff*sizeof(uint) );
 		    d += ddiff;
 		    prev_d += ddiff;
 		}
-		memmove( prev_d+1, prev_d, ddiff*sizeof(ushort) );
-		*prev_d = *p == '?' ? OPT : CLO;
+		memmove( prev_d+1, prev_d, ddiff*sizeof(uint) );
+		*prev_d = ch == '?' ? OPT : CLO;
 		d++;
 		GEN( END );
 		p++;
+		pl--;
 		}
 		break;
 
 	    default:
 		{
 		prev_d = d;
-		if ( *p == '\\' ) {
-		    if ( *(p+1) == 's' ) {	// white space
-			GEN( CCL );		//   represent as char class
-			if ( d + 16 >= rxarray + maxlen ) {
-			    error = PatOverflow;// pattern too long
-			    return;
-			}
-			memset( d, 0, 16*sizeof(ushort) );
-			d[0] |= 0x3e00;		// 9, 10, 11, 12, 13
-			d[32 >> 4] |= (1 << (32 & 0xf)); // 32
-			d += 16;
-			p++;
-			p++;
-			break;
-		    } else if ( *(p+1) == '<' || *(p+1) == '>' ) {
-			GEN( *++p == '<' ? BOW : EOW );
-			p++;
-			break;
-		    }
+		uint cv = char_val( &p, &pl );
+		if ( cv & MCD ) {			// It's a code
+		    GEN( cv );
+		}
+		else {
+		    if ( !cs && cv <= 0xff )		// #only 8bit support
+			cv = tolower( cv );
+		    GEN( CHR | cv );
 		}
-		int c = char_val(&p);
-		if ( !cs )
-		    c = tolower( c );
-		GEN( CHR | c );
 		}
 	}
 	if ( d >= rxarray + maxlen ) {		// oops!
@@ -818,122 +970,10 @@ void QRegExp::compile()
     }
     GEN( END );
     int len = d - rxarray;
-    rxdata = new ushort[ len ];			// copy from rxarray to rxdata
+    rxdata = new uint[ len ];			// copy from rxarray to rxdata
     CHECK_PTR( rxdata );
-    memcpy( rxdata, rxarray, len*sizeof(ushort) );
+    memcpy( rxdata, rxarray, len*sizeof(uint) );
 #if defined(DEBUG)
     //dump( rxdata );	// uncomment this line for debugging
 #endif
 }
-
-
-/*****************************************************************************
-  QString member functions that use QRegExp
- *****************************************************************************/
-
-/*!
-  Finds the first occurrence of the regular expression \e rx, starting at
-  position \e index.
-
-  Returns the position of the next match, or -1 if \e rx was not found.
-*/
-
-int QString::find( const QRegExp &rx, int index ) const
-{
-    return (uint)index >= size() ? -1 : rx.match( data(), index );
-}
-
-/*!
-  Finds the first occurrence of the regular expression \e rx, starting at
-  position \e index and searching backwards.
-
-  The search will start from the end of the string if \e index is negative.
-
-  Returns the position of the next match (backwards), or -1 if \e rx was not
-  found.
-*/
-
-int QString::findRev( const QRegExp &rx, int index ) const
-{
-    if ( index < 0 ) {				// neg index ==> start from end
-	if ( size() )
-	    index = strlen( data() );
-	else					// empty string
-	    return -1;
-    }
-    else if ( (uint)index >= size() )		// bad index
-	return -1;
-    while( index >= 0 ) {
-	if ( rx.match(data(),index) == index )
-	    return index;
-	index--;
-    }
-    return -1;
-}
-
-/*!
-  Counts the number of overlapping occurrences of \e rx in the string.
-
-  Example:
-  \code
-    QString s = "banana and panama";
-    QRegExp r = QRegExp("a[nm]a", TRUE, FALSE);
-    s.contains( r );				// 4 matches
-  \endcode
-
-  \sa find(), findRev()
-*/
-
-int QString::contains( const QRegExp &rx ) const
-{
-    if ( isEmpty() )
-	return 0;
-    int count = 0;
-    int index = -1;
-    int len = length();
-    while ( index < len ) {			// count overlapping matches
-	index = rx.match( data(), index+1 );
-	if ( index < 0 )
-	    break;
-	count++;
-    }
-    return count;
-}
-
-
-/*!
-  Replaces every occurrence of \e rx in the string with \e str.
-  Returns a reference to the string.
-
-  Example:
-  \code
-    QString s = "banana";
-    s.replace( QRegExp("a.*a"), "" );		// becomes "b"
-
-    QString s = "banana";
-    s.replace( QRegExp("^[bn]a"), " " );	// becomes " nana"
-
-    QString s = "banana";
-    s.replace( QRegExp("^[bn]a"), "" );		// NOTE! becomes ""
-  \endcode
-
-*/
-
-QString &QString::replace( const QRegExp &rx, const char *str )
-{
-    if ( isEmpty() )
-	return *this;
-    int index = 0;
-    int slen  = strlen( str );
-    int len;
-    while ( index < (int)size()-1 ) {
-	if ( (index = rx.match(data(), index, &len)) >= 0 ) {
-	    remove( index, len );
-	    insert( index, str );
-	    index += slen;
-	}
-	else
-	    break;
-    }
-    return *this;
-}
