@@ -1,27 +1,30 @@
 /****************************************************************************
-** $Id: qtextstream.cpp,v 2.17 1998/07/03 00:09:47 hanord Exp $
+** $Id: qtextstream.cpp,v 2.85.2.3 1999/08/20 13:53:55 paul Exp $
 **
 ** Implementation of QTextStream class
 **
 ** Created : 940922
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qtextstream.h"
+#include "qtextcodec.h"
 #include "qbuffer.h"
 #include "qfile.h"
 #include <stdio.h>
@@ -52,19 +55,31 @@
 
   Qt provides several global functions similar to the ones in iostream:
   <ul>
-  <li> \c bin sets the QTextStream to output binary numbers
-  <li> \c oct sets the QTextStream to output octal numbers
-  <li> \c dec sets the QTextStream to output decimal numbers
-  <li> \c hex sets the QTextStream to output hexadecimal numbers
+  <li> \c bin sets the QTextStream to read/write binary numbers
+  <li> \c oct sets the QTextStream to read/write octal numbers
+  <li> \c dec sets the QTextStream to read/write decimal numbers
+  <li> \c hex sets the QTextStream to read/write hexadecimal numbers
   <li> \c endl forces a line break
   <li> \c flush forces the QIODevice to flush any buffered data
   <li> \c ws eats any available white space (on input)
   <li> \c reset resets the QTextStream to its default mode (see reset()).
   </ul>
 
-  The QTextStream class reads and writes ASCII text and it is not
+  \warning By default, QTextStream will automatically detect whether
+  integers in the stream are in decimal, octal, hexadecimal or binary
+  format when reading from the stream. In particular, a leading '0'
+  signifies octal, ie. the sequence "0100" will be interpreted as
+  64.
+
+  The QTextStream class reads and writes text and it is not
   appropriate for dealing with binary data (but QDataStream is).
 
+  By default output of Unicode text (ie. QString) is done using the
+  local 8-bit encoding.  This can be changed using the setEncoding()
+  method.  For input, the QTextStream will auto-detect standard
+  Unicode "byte order marked" text files, but otherwise the local
+  8-bit encoding is used.
+
   \sa QDataStream
 */
 
@@ -123,7 +138,7 @@
 #if defined(CHECK_STATE)
 #undef  CHECK_STREAM_PRECOND
 #define CHECK_STREAM_PRECOND  if ( !dev ) {				\
-				warning( "QTextStream: No device" );	\
+				qWarning( "QTextStream: No device" );	\
 				return *this; }
 #else
 #define CHECK_STREAM_PRECOND
@@ -146,6 +161,8 @@
 #define I_SIGN_MASK	0x0f00
 
 
+static const QChar QEOF = QChar((ushort)0xffff); //guaranteed not to be a character.
+
 const int QTextStream::basefield   = I_BASE_MASK;
 const int QTextStream::adjustfield = ( QTextStream::left |
 				       QTextStream::right |
@@ -153,14 +170,24 @@ const int QTextStream::adjustfield = ( QTextStream::left |
 const int QTextStream::floatfield  = ( QTextStream::scientific |
 				       QTextStream::fixed );
 
-
-int eat_ws( QIODevice *d )
+QChar QTextStream::eat_ws()
 {
-    int c;
-    do { c = d->getch(); } while ( c != EOF && isspace(c) );
+    QChar c;
+    do { c = ts_getc(); } while ( c != QEOF && ts_isspace(c) );
     return c;
 }
 
+void QTextStream::init()
+{
+    ungetcBuf = QEOF;
+    dev = 0;					// no device set
+    fstrm = owndev = FALSE;
+    mapper = 0;
+    decoder = 0;
+    doUnicodeHeader = TRUE;		//default to autodetect
+    latin1 = TRUE; //### mapper = QTextCodec::local
+    internalOrder = QChar::networkOrdered(); //default to network order
+}
 
 /*!
   Constructs a data stream that has no IO device.
@@ -168,8 +195,8 @@ int eat_ws( QIODevice *d )
 
 QTextStream::QTextStream()
 {
-    dev = 0;					// no device set
-    fstrm = owndev = FALSE;
+    init();
+    setEncoding( Locale ); //###
     reset();
 }
 
@@ -179,59 +206,294 @@ QTextStream::QTextStream()
 
 QTextStream::QTextStream( QIODevice *d )
 {
+    init();
+    setEncoding( Locale ); //###
     dev = d;					// set device
-    fstrm = owndev = FALSE;
     reset();
 }
 
+// TODO: use special-case handling of this case in QTextStream, and
+//	 simplify this class to only deal with QChar or QString data.
+class QStringBuffer : public QIODevice {
+public:
+    QStringBuffer( QString* str ) :
+	s(str)
+    {
+    }
+
+   ~QStringBuffer()
+    {
+    }
+
+
+    bool  open( int m )
+    {
+	if ( !s ) {
+#if defined(CHECK_STATE)
+	    qWarning( "QStringBuffer::open: No string" );
+#endif
+	    return FALSE;
+	}
+	if ( isOpen() ) {                           // buffer already open
+#if defined(CHECK_STATE)
+	    qWarning( "QStringBuffer::open: Buffer already open" );
+#endif
+	    return FALSE;
+	}
+	setMode( m );
+	if ( m & IO_Truncate ) {                    // truncate buffer
+	    s->truncate( 0 );
+	}
+	if ( m & IO_Append ) {                      // append to end of buffer
+	    ioIndex = s->length()*sizeof(QChar);
+	} else {
+	    ioIndex = 0;
+	}
+	setState( IO_Open );
+	setStatus( 0 );
+	return TRUE;
+    }
+
+    void  close()
+    {
+	if ( isOpen() ) {
+	    setFlags( IO_Direct );
+	    ioIndex = 0;
+	}
+    }
+
+    void  flush()
+    {
+    }
+
+    uint  size() const
+    {
+	return s ? s->length()*sizeof(QChar) : 0;
+    }
+
+    int   at()   const
+    {
+	return ioIndex;
+    }
+
+    bool  at( int pos )
+    {
+#if defined(CHECK_STATE)
+	if ( !isOpen() ) {
+	    qWarning( "QStringBuffer::at: Buffer is not open" );
+	    return FALSE;
+	}
+#endif
+	if ( (uint)pos >= s->length()*2 ) {
+#if defined(CHECK_RANGE)
+	    qWarning( "QStringBuffer::at: Index %d out of range", pos );
+#endif
+	    return FALSE;
+	}
+	ioIndex = pos;
+	return TRUE;
+    }
+
+
+    int   readBlock( char *p, uint len )
+    {
+#if defined(CHECK_STATE)
+	CHECK_PTR( p );
+	if ( !isOpen() ) {                          // buffer not open
+	    qWarning( "QStringBuffer::readBlock: Buffer not open" );
+	    return -1;
+	}
+	if ( !isReadable() ) {                      // reading not permitted
+	    qWarning( "QStringBuffer::readBlock: Read operation not permitted" );
+	    return -1;
+	}
+#endif
+	if ( (uint)ioIndex + len > s->length()*sizeof(QChar) ) {
+					     	    // overflow
+	    if ( (uint)ioIndex >= s->length()*sizeof(QChar) ) {
+		setStatus( IO_ReadError );
+		return -1;
+	    } else {
+		len = s->length()*2 - (uint)ioIndex;
+	    }
+	}
+	memcpy( p, ((const char*)(s->unicode()))+ioIndex, len );
+	ioIndex += len;
+	return len;
+    }
+
+    int writeBlock( const char *p, uint len )
+    {
+#if defined(CHECK_NULL)
+	if ( p == 0 && len != 0 )
+	    qWarning( "QStringBuffer::writeBlock: Null pointer error" );
+#endif
+#if defined(CHECK_STATE)
+	if ( !isOpen() ) {                          // buffer not open
+	    qWarning( "QStringBuffer::writeBlock: Buffer not open" );
+	    return -1;
+	}
+	if ( !isWritable() ) {                      // writing not permitted
+	    qWarning( "QStringBuffer::writeBlock: Write operation not permitted" );
+	    return -1;
+	}
+	if ( ioIndex&1 ) {
+	    qWarning( "QStringBuffer::writeBlock: non-even index - non Unicode" );
+	    return -1;
+	}
+	if ( len&1 ) {
+	    qWarning( "QStringBuffer::writeBlock: non-even length - non Unicode" );
+	    return -1;
+	}
+#endif
+	s->replace(ioIndex/2, len/2, (QChar*)p, len/2);
+	ioIndex += len;
+	return len;
+    }
+
+    int   getch()
+    {
+#if defined(CHECK_STATE)
+	if ( !isOpen() ) {                          // buffer not open
+	    qWarning( "QStringBuffer::getch: Buffer not open" );
+	    return -1;
+	}
+	if ( !isReadable() ) {                      // reading not permitted
+	    qWarning( "QStringBuffer::getch: Read operation not permitted" );
+	    return -1;
+	}
+#endif
+	if ( (uint)ioIndex >= s->length()*2 ) {           // overflow
+	    setStatus( IO_ReadError );
+	    return -1;
+	}
+	return *((char*)s->unicode() + ioIndex++);
+    }
+
+    int   putch( int ch )
+    {
+	char c = ch;
+	if ( writeBlock(&c,1) < 0 )
+	    return -1;
+	else
+	    return ch;
+    }
+
+    int   ungetch( int ch )
+    {
+#if defined(CHECK_STATE)
+	if ( !isOpen() ) {                          // buffer not open
+	    qWarning( "QStringBuffer::ungetch: Buffer not open" );
+	    return -1;
+	}
+	if ( !isReadable() ) {                      // reading not permitted
+	    qWarning( "QStringBuffer::ungetch: Read operation not permitted" );
+	    return -1;
+	}
+#endif
+	if ( ch != -1 ) {
+	    if ( ioIndex )
+		ioIndex--;
+	    else
+		ch = -1;
+	}
+	return ch;
+    }
+
+protected:
+    QString* s;
+
+private:        // Disabled copy constructor and operator=
+    QStringBuffer( const QStringBuffer & );
+    QStringBuffer &operator=( const QStringBuffer & );
+};
+
 /*!
-  Constructs a text stream that operates on a byte array throught an
-  internal QBuffer device.
+  Constructs a text stream that operates on a Unicode QString through an
+  internal device.
 
   Example:
   \code
     QString str;
-    QTextStream ts( str, IO_WriteOnly );
-    ts << "pi = " << 3.14;			// str == "pi = 3.14"
+    QTextStream ts( &str, IO_WriteOnly );
+    ts << "pi = " << 3.14;			// str == "pi = 3.14..."
   \endcode
 
   Writing data to the text stream will modify the contents of the string.
   The string will be expanded when data is written beyond the end of the
   string.
 
-  Same example, using a QBuffer:
+  Note that since QString is Unicode, you should not use readRawBytes()
+  or writeRawBytes() on such a stream.
+*/
+
+QTextStream::QTextStream( QString* str, int filemode )
+{
+    // TODO: optimize for this case as it becomes more common
+    //        (see QStringBuffer above)
+    init();
+    dev = new QStringBuffer( str );
+    ((QStringBuffer *)dev)->open( filemode );
+    owndev = TRUE;
+    setEncoding(RawUnicode);
+    reset();
+}
+
+/*! \obsolete
+
+  This constructor is equivalent to the constructor taking a QString*
+  parameter.
+*/
+
+QTextStream::QTextStream( QString& str, int filemode )
+{
+    init();
+    dev = new QStringBuffer( &str );
+    ((QStringBuffer *)dev)->open( filemode );
+    owndev = TRUE;
+    setEncoding(RawUnicode);
+    reset();
+}
+
+/*!
+  Constructs a text stream that operates on a byte array through an
+  internal QBuffer device.
+
+  Example:
   \code
-    QString str;
-    QBuffer buf( str );
-    buf.open( IO_WriteOnly );
-    QTextStream ts( &buf );
-    ts << "pi = " << 3.14;			// str == "pi = 3.14"
-    buf.close();
+    QByteArray array;
+    QTextStream ts( array, IO_WriteOnly );
+    ts << "pi = " << 3.14 << '\0';		// array == "pi = 3.14"
   \endcode
 
-  Note that QStrings created in this way will not have NUL terminators.
-  So unless you are using the string purely as an array of bytes, you
-  should terminate them:
+  Writing data to the text stream will modify the contents of the array.
+  The array will be expanded when data is written beyond the end of the
+  string.
+
+  Same example, using a QBuffer:
   \code
-    QString str;
-    ...
-    ts << "pi = " << 3.14;			// str == "pi = 3.14"
-    ts << '\0';
+    QByteArray array;
+    QBuffer buf( array );
+    buf.open( IO_WriteOnly );
+    QTextStream ts( &buf );
+    ts << "pi = " << 3.14 << '\0';		// array == "pi = 3.14"
+    buf.close();
   \endcode
 */
 
 QTextStream::QTextStream( QByteArray a, int mode )
 {
+    init();
     dev = new QBuffer( a );
     ((QBuffer *)dev)->open( mode );
-    fstrm = FALSE;
     owndev = TRUE;
+    setEncoding( Latin1 ); //### Locale???
     reset();
 }
 
 /*!
   Constructs a text stream that operates on an existing file handle \e fh
-  throught an internal QFile device.
+  through an internal QFile device.
 
   Example:
   \code
@@ -243,6 +505,8 @@ QTextStream::QTextStream( QByteArray a, int mode )
 
 QTextStream::QTextStream( FILE *fh, int mode )
 {
+    init();
+    setEncoding( Locale ); //###
     dev = new QFile;
     ((QFile *)dev)->open( mode, fh );
     fstrm = owndev = TRUE;
@@ -252,15 +516,243 @@ QTextStream::QTextStream( FILE *fh, int mode )
 /*!
   Destroys the text stream.
 
-  The destructor will not affect the current IO device.
+  The destructor does not affect the current IO device.
 */
 
 QTextStream::~QTextStream()
 {
     if ( owndev )
 	delete dev;
+    delete decoder;
 }
 
+/*!
+  Equivalent to *this << ws.
+*/
+void QTextStream::skipWhiteSpace()
+{
+    ts_ungetc( eat_ws() );
+}
+
+
+/*!
+  \fn Encoding QTextStream::encoding() const
+
+  Returns the encoding mode of the stream.
+
+  \sa setEncoding()
+*/
+
+/*!
+  Returns one character from the stream, or EOF.
+*/
+QChar QTextStream::ts_getc()
+{
+    QChar r;
+    if ( doUnicodeHeader ) {
+	doUnicodeHeader = FALSE; //only at the top
+	int c1 = dev->getch();
+	if ( c1 == EOF )
+	    return QEOF;
+	int c2 = dev->getch();
+	if ( c1 == 0xfe && c2 == 0xff ) {
+	    mapper = 0;
+	    latin1 = FALSE;
+	    internalOrder = QChar::networkOrdered();   //network order
+	} else if ( c1 == 0xff && c2 == 0xfe ) {
+	    mapper = 0;
+	    latin1 = FALSE;
+	    internalOrder = !QChar::networkOrdered();   //reverse network order
+	} else {
+	    if ( c2 != EOF )
+		dev->ungetch( c2 );
+	    dev->ungetch( c1 );
+	}
+    }
+    if ( mapper ) {
+	if ( ungetcBuf != QEOF ) {
+	    r = ungetcBuf;
+	    ungetcBuf = QEOF;
+	} else {
+	    if ( !decoder )
+		decoder = mapper->makeDecoder();
+	    QString s;
+	    do {
+		char b[1];
+		int c = dev->getch();
+		if ( c == EOF )
+		    return QEOF;
+		b[0] = c;
+		s  = decoder->toUnicode( b, 1 );
+	    } while ( s.isEmpty() );
+	    r = s[0];
+	}
+    } else if ( latin1 ) {
+	int c = dev->getch();
+	if ( c == EOF )
+	    return QEOF;
+	r = (char)c;
+    } else { //Unicode
+	int c1 = dev->getch();
+	if ( c1 == EOF )
+	    return QEOF;
+	int c2 = dev->getch();
+	if ( isNetworkOrder() )
+	    r = QChar( c2, c1 );
+	else
+	    r = QChar( c1, c2 );
+    }
+    return r;
+}
+
+
+/*!
+  Puts one character to the stream.
+*/
+void QTextStream::ts_putc( QChar c )
+{
+    if ( mapper ) {
+	int len = 1;
+	QString s = c;
+	QCString block = mapper->fromUnicode( s, len );
+	dev->writeBlock( block, len );
+    } else if ( latin1 ) {
+	if( c.row() )
+	    dev->putch( '?' ); //######unknown character???
+	else
+	    dev->putch( c.cell() );
+    } else {
+	if ( doUnicodeHeader ) {
+	    doUnicodeHeader = FALSE;
+	    ts_putc( QChar::byteOrderMark );
+	}
+	if ( internalOrder ) {
+	    dev->writeBlock( (char*)&c, sizeof(QChar) );
+	} else if ( isNetworkOrder() ) {
+	    dev->putch(c.row());
+	    dev->putch(c.cell());
+	} else {
+	    dev->putch(c.cell());
+	    dev->putch(c.row());
+	}
+    }
+}
+
+/*!
+  Puts one character to the stream.
+*/
+void QTextStream::ts_putc(int ch)
+{
+    ts_putc(QChar((ushort)ch));
+}
+
+bool QTextStream::ts_isdigit(QChar c)
+{
+    return c.isDigit();
+}
+
+bool QTextStream::ts_isspace( QChar c )
+{
+    return c.isSpace();
+}
+
+void QTextStream::ts_ungetc( QChar c )
+{
+    if ( mapper ) {
+	ungetcBuf = c;
+    } else if ( latin1 ) {
+	dev->ungetch( c );
+    } else {
+	if ( isNetworkOrder() ) {
+	    //stream is network ordered
+	// Reverse of put
+	    dev->ungetch(c.cell());
+	    dev->ungetch(c.row());
+	} else {
+	// Reverse of put
+	    dev->ungetch(c.row());
+	    dev->ungetch( c.cell() );
+	}
+    }
+}
+
+
+
+/*!
+  Reads \e len bytes from the stream into \e e s and returns a reference to
+  the stream.
+
+  The buffer \e s must be preallocated.
+
+  \note No Encoding is done by this function.
+
+  \warning The behaviour of this function is undefined unless the
+  stream's encoding is set to Unicode or Latin1.
+
+  \sa QIODevice::readBlock()
+*/
+
+QTextStream &QTextStream::readRawBytes( char *s, uint len )
+{
+    dev->readBlock( s, len );
+    return *this;
+}
+
+/*!
+  Writes the \e len bytes from \e s to the stream and returns a reference to
+  the stream.
+
+  \note No Encoding is done by this function.
+
+  \sa QIODevice::writeBlock()
+*/
+
+QTextStream &QTextStream::writeRawBytes( const char* s, uint len )
+{
+    dev->writeBlock( s, len );
+    return *this;
+}
+
+
+QTextStream &QTextStream::writeBlock( const char* p, uint len )
+{
+    if ( doUnicodeHeader ) {
+	doUnicodeHeader = FALSE;
+	if ( !mapper && !latin1 )
+	    ts_putc( QChar::byteOrderMark );
+    }
+    //All QCStrings and const char* are defined to be in Latin1
+    if ( !mapper && latin1 ) {
+	dev->writeBlock( p, len );
+    } else if ( !mapper && internalOrder ) {
+	QChar *u = new QChar[len];
+	for (uint i=0; i<len; i++)
+	    u[i] = p[i];
+	dev->writeBlock( (char*)u, len*sizeof(QChar) );
+	delete [] u;
+    } else {
+	for (uint i=0; i<len; i++)
+	    ts_putc( (uchar)p[i] );
+    }
+    return *this;
+}
+
+QTextStream &QTextStream::writeBlock( const QChar* p, uint len )
+{
+    if ( !mapper && !latin1 && internalOrder ) {
+	if ( doUnicodeHeader ) {
+	    doUnicodeHeader = FALSE;
+	    ts_putc( QChar::byteOrderMark );
+	}
+	dev->writeBlock( (char*)p, sizeof(QChar)*len );
+    } else {
+	for (uint i=0; i<len; i++)
+	    ts_putc( p[i] );
+    }
+    return *this;
+}
+
+
 
 /*!
   Resets the text stream.
@@ -315,7 +807,7 @@ void QTextStream::unsetDevice()
 }
 
 /*!
-  \fn bool QTextStream::eof() const
+  \fn bool QTextStream::atEnd() const
   Returns TRUE if the IO device has reached the end position (end of
   stream or file) or if there is no IO device set.
 
@@ -325,6 +817,14 @@ void QTextStream::unsetDevice()
   \sa QIODevice::atEnd()
 */
 
+/*!\fn bool QTextStream::eof() const
+
+  \obsolete
+
+  This function has been renamed to atEnd().
+
+  \sa QIODevice::atEnd()
+*/
 
 /*****************************************************************************
   QTextStream read functions
@@ -333,155 +833,177 @@ void QTextStream::unsetDevice()
 
 /*!
   Reads a \c char from the stream and returns a reference to the stream.
+  Note that whitespace is skipped.
 */
 
 QTextStream &QTextStream::operator>>( char &c )
 {
     CHECK_STREAM_PRECOND
-    c = eat_ws( dev );
+    c = eat_ws();
+    return *this;
+}
+
+/*!
+  Reads a \c char from the stream and returns a reference to the stream.
+  Note that whitespace is \em not skipped.
+*/
+
+QTextStream &QTextStream::operator>>( QChar &c )
+{
+    CHECK_STREAM_PRECOND
+    c = ts_getc();
     return *this;
 }
 
 
-static ulong input_bin( QTextStream *s )
+ulong QTextStream::input_bin()
 {
-    QIODevice *d = s->device();
     ulong val = 0;
-    register int c = eat_ws( d );
-    while ( c == '0' || c == '1' ) {
+    QChar ch = eat_ws();
+    int d = ch.digitValue();
+    while (  d == 0 || d == 1 ) {
 	val <<= 1;
-	val += c - '0';
-	c = d->getch();
+	val += d;
+	ch = ts_getc();
+	d = ch.digitValue();
     }
-    if ( c != EOF )
-	d->ungetch( c );
+    if ( ch != QEOF )
+	ts_ungetc( ch );
     return val;
 }
 
-static ulong input_oct( QTextStream *s )
+ulong QTextStream::input_oct()
 {
-    QIODevice *d = s->device();
     ulong val = 0;
-    register int c = eat_ws( d );
-    while ( c >= '0' && c <= '7' ) {
+    QChar ch = eat_ws();
+    int d = ch.digitValue();
+    while ( d >= 0 && d <= 7 ) {
 	val <<= 3;
-	val += c - '0';
-	c = d->getch();
+	val += d;
+	ch = ts_getc();
+	d = ch.digitValue();
     }
-    if ( c == '8' || c == '9' ) {
-	while ( isdigit(c) )
-	    c = d->getch();
+    if ( d == 8 || d == 9 ) {
+	while ( ts_isdigit(ch) )
+	    ch = ts_getc();
     }
-    if ( c != EOF )
-	d->ungetch( c );
+    if ( ch != QEOF )
+	ts_ungetc( ch );
     return val;
 }
 
-static ulong input_dec( QTextStream *s )
+ulong QTextStream::input_dec()
 {
-    QIODevice *d = s->device();
     ulong val = 0;
-    register int c = eat_ws( d );
-    while ( isdigit(c) ) {
+    QChar ch = eat_ws();
+    int d = ch.digitValue();
+    while ( ts_isdigit(ch) ) {
 	val *= 10;
-	val += c - '0';
-	c = d->getch();
+	val += d;
+	ch = ts_getc();
+	d = ch.digitValue();
     }
-    if ( c != EOF )
-	d->ungetch( c );
+    if ( ch != QEOF )
+	ts_ungetc( ch );
     return val;
 }
 
-static ulong input_hex( QTextStream *s )
+ulong QTextStream::input_hex()
 {
-    QIODevice *d = s->device();
     ulong val = 0;
-    register int c = eat_ws( d );
+    QChar ch = eat_ws();
+    char c = ch;
     while ( isxdigit(c) ) {
 	val <<= 4;
-	if ( isdigit(c) )
+	if ( ts_isdigit(c) )
 	    val += c - '0';
 	else
 	    val += 10 + tolower(c) - 'a';
-	c = d->getch();
+	c = ch = ts_getc();
     }
-    if ( c != EOF )
-	d->ungetch( c );
+    if ( ch != QEOF )
+	ts_ungetc( ch );
     return val;
 }
 
 long QTextStream::input_int()
 {
     long val;
-    int c;
+    QChar ch;
+    char c;
     switch ( flags() & basefield ) {
-	case bin:
-	    val = (long)input_bin( this );
-	    break;
-	case oct:
-	    val = (long)input_oct( this );
-	    break;
-	case dec:
-	    c = eat_ws( dev );
-	    if ( c == EOF ) {
-		val = 0;
-	    } else {
-		if ( !(c == '-' || c == '+') )
-		    dev->ungetch( c );
-		if ( c == '-' ) {
-		    ulong v = input_dec( this );
-		    if ( v ) {		// ensure that LONG_MIN can be read
-			v--;
-			val = -((long)v) - 1;
-		    } else {
-			val = 0;
-		    }
-		} else {
-		    val = (long)input_dec( this );
-		}
-	    }
-	    break;
-	case hex:
-	    val = (long)input_hex( this );
-	    break;
-	default:
+    case bin:
+	val = (long)input_bin();
+	break;
+    case oct:
+	val = (long)input_oct();
+	break;
+    case dec:
+	c = ch = eat_ws();
+	if ( ch == QEOF ) {
 	    val = 0;
-	    c = eat_ws( dev );
-	    if ( c == '0' ) {		// bin, oct or hex
-		c = dev->getch();
-		if ( tolower(c) == 'x' )
-		    val = (long)input_hex( this );
-		else if ( tolower(c) == 'b' )
-		    val = (long)input_bin( this );
-		else {			// octal
-		    if ( c >= '0' && c <= '7' ) {
-			dev->ungetch( c );
-			val = (long)input_oct( this );
-		    } else {
-			val = 0;
-		    }
+	} else {
+	    if ( !(c == '-' || c == '+') )
+		ts_ungetc( ch );
+	    if ( c == '-' ) {
+		ulong v = input_dec();
+		if ( v ) {		// ensure that LONG_MIN can be read
+		    v--;
+		    val = -((long)v) - 1;
+		} else {
+		    val = 0;
+		}
+	    } else {
+		val = (long)input_dec();
+	    }
+	}
+	break;
+    case hex:
+	val = (long)input_hex();
+	break;
+    default:
+	val = 0;
+	c = ch = eat_ws();
+	if ( c == '0' ) {		// bin, oct or hex
+	    c = ch = ts_getc();
+	    if ( tolower(c) == 'x' )
+		val = (long)input_hex();
+	    else if ( tolower(c) == 'b' )
+		val = (long)input_bin();
+	    else {			// octal
+		ts_ungetc( ch );
+		if ( c >= '0' && c <= '7' ) {
+		    val = (long)input_oct();
+		} else {
+		    val = 0;
 		}
 	    }
-	    else if ( c >= '1' && c <= '9' ) {
-		dev->ungetch( c );
-		val = (long)input_dec( this );
-	    }
-	    else if ( c == '-' || c == '+' ) {
-		val = (long)input_dec( this );
-		if ( c == '-' )
-		    val = -val;
+	} else if ( ts_isdigit(ch) ) {
+	    ts_ungetc( ch );
+	    val = (long)input_dec();
+	} else if ( c == '-' || c == '+' ) {
+	    ulong v = input_dec();
+	    if ( c == '-' ) {
+		if ( v ) {		// ensure that LONG_MIN can be read
+		    v--;
+		    val = -((long)v) - 1;
+		} else {
+		    val = 0;
+		}
+	    } else {
+		val = (long)v;
 	    }
+	}
     }
     return val;
 }
 
-
 //
 // We use a table-driven FSM to parse floating point numbers
 // strtod() cannot be used directly since we're reading from a QIODevice
 //
 
-static double input_double( QTextStream *s )
+double QTextStream::input_double()
 {
     const int Init	 = 0;			// states
     const int Sign	 = 1;
@@ -513,10 +1035,9 @@ static double input_double( QTextStream *s )
     int state = Init;				// parse state
     int input;					// input token
 
-    QIODevice *d = s->device();
     char buf[256];
     int i = 0;
-    int c = eat_ws( d );
+    QChar c = eat_ws();
 
     while ( TRUE ) {
 
@@ -545,20 +1066,20 @@ static double input_double( QTextStream *s )
 
 	if  ( state == 0 || state == Done || i > 250 ) {
 	    if ( i > 250 ) {			// ignore rest of digits
-		do { c = d->getch(); } while ( c != EOF && isdigit(c) );
+		do { c = ts_getc(); } while ( c != QEOF && ts_isdigit(c) );
 	    }
-	    if ( c != EOF )
-		d->ungetch( c );
+	    if ( c != QEOF )
+		ts_ungetc( c );
 	    buf[i] = '\0';
 	    char *end;
 	    return strtod( buf, &end );
 	}
 
 	buf[i++] = c;
-	c = d->getch();
+	c = ts_getc();
     }
 
-#if !defined(NO_DEADCODE)
+#if !defined(_CC_EGG_)
     return 0.0;
 #endif
 }
@@ -650,7 +1171,7 @@ QTextStream &QTextStream::operator>>( unsigned long &i )
 QTextStream &QTextStream::operator>>( float &f )
 {
     CHECK_STREAM_PRECOND
-    f = (float)input_double( this );
+    f = (float)input_double();
     return *this;
 }
 
@@ -663,7 +1184,7 @@ QTextStream &QTextStream::operator>>( float &f )
 QTextStream &QTextStream::operator>>( double &f )
 {
     CHECK_STREAM_PRECOND
-    f = input_double( this );
+    f = input_double();
     return *this;
 }
 
@@ -676,16 +1197,16 @@ QTextStream &QTextStream::operator>>( char *s )
 {
     CHECK_STREAM_PRECOND
     int maxlen = width( 0 );
-    int c = eat_ws( dev );
+    QChar c = eat_ws();
     if ( !maxlen )
 	maxlen = -1;
-    while ( c != EOF ) {
-	if ( isspace(c) || maxlen-- == 0 ) {
-	    dev->ungetch( c );
+    while ( c != QEOF ) {
+	if ( ts_isspace(c) || maxlen-- == 0 ) {
+	    ts_ungetc( c );
 	    break;
 	}
 	*s++ = c;
-	c = dev->getch();
+	c = ts_getc();
     }
 
     *s = '\0';
@@ -699,21 +1220,42 @@ QTextStream &QTextStream::operator>>( char *s )
 QTextStream &QTextStream::operator>>( QString &str )
 {
     CHECK_STREAM_PRECOND
-    QString  *dynbuf = 0;
+    str=QString::fromLatin1("");
+    QChar	c = eat_ws();
+
+    while ( c != QEOF ) {
+	if ( ts_isspace(c) ) {
+	    ts_ungetc( c );
+	    break;
+	}
+	str += c;
+	c = ts_getc();
+    }
+    return *this;
+}
+
+/*!
+  Reads a word from the stream and returns a reference to the stream.
+*/
+
+QTextStream &QTextStream::operator>>( QCString &str )
+{
+    CHECK_STREAM_PRECOND
+    QCString  *dynbuf = 0;
     const int buflen = 256;
     char      buffer[buflen];
     char     *s = buffer;
     int	      i = 0;
-    int	      c = eat_ws(dev);
+    QChar	      c = eat_ws();
 
-    while ( c != EOF ) {
-	if ( isspace(c) ) {
-	    dev->ungetch( c );
+    while ( c != QEOF ) {
+	if ( ts_isspace(c) ) {
+	    ts_ungetc( c );
 	    break;
 	}
 	if ( i >= buflen-1 ) {
 	    if ( !dynbuf )  {			// create dynamic buffer
-		dynbuf = new QString(buflen*2);
+		dynbuf = new QCString(buflen*2);
 		memcpy( dynbuf->data(), s, i );	// copy old data
 	    } else if ( i >= (int)dynbuf->size()-1 ) {
 		dynbuf->resize( dynbuf->size()*2 );
@@ -721,7 +1263,7 @@ QTextStream &QTextStream::operator>>( QString &str )
 	    s = dynbuf->data();
 	}
 	s[i++] = c;
-	c = dev->getch();
+	c = ts_getc();
     }
     str.resize( i+1 );
     memcpy( str.data(), s, i );
@@ -744,40 +1286,68 @@ QString QTextStream::readLine()
 {
 #if defined(CHECK_STATE)
     if ( !dev ) {
-	warning( "QTextStream::readLine: No device" );
-	QString nullString;
-	return nullString;
+	qWarning( "QTextStream::readLine: No device" );
+	return QString::null;
     }
 #endif
-    QString  *dynbuf = 0;
-    const int buflen = 256;
-    char      buffer[buflen];
-    char     *s = buffer;
-    int	      i = 0;
-    int	      c = dev->getch();
+    QString result;
+    QChar c = ts_getc();
 
-    while ( c != EOF ) {
+    while ( c != QEOF ) {
 	if ( c == '\n' ) {
 	    break;
 	}
-	if ( i >= buflen-1 ) {
-	    if ( !dynbuf )  {			// create dynamic buffer
-		dynbuf = new QString(buflen*2);
-		memcpy( dynbuf->data(), s, i );	// copy old data
-	    } else if ( i >= (int)dynbuf->size()-1 ) {
-		dynbuf->resize( dynbuf->size()*2 );
-	    }
-	    s = dynbuf->data();
-	}
-	s[i++] = c;
-	c = dev->getch();
+	result += c;
+	c = ts_getc();
     }
-    if ( i > 0 && s[i-1] == '\r' )
-	i--;				// if there are two \r, let one stay
-    QString str(i+1);
-    memcpy( str.data(), s, i );
-    delete dynbuf;
-    return str;
+
+    if ( !!result && result[int(result.length())-1] == '\r' )
+	result.truncate(result.length()-1);
+	    // (if there are two \r, let one stay)
+
+    return result;
+}
+
+
+/*!
+  Reads the entire stream and returns a string containing the text.
+
+  \sa QIODevice::readLine()
+*/
+
+QString QTextStream::read()
+{
+#if defined(CHECK_STATE)
+    if ( !dev ) {
+	qWarning( "QTextStream::readLine: No device" );
+	return QString::null;
+    }
+#endif
+    QString   result;
+    QChar     c = ts_getc();
+    bool      skipped_cr = FALSE;
+
+    while ( c != QEOF ) {
+	if ( c == '\r' ) {
+	    // Only skip single cr's preceeding lf's
+	    if ( skipped_cr ) {
+		result += c;
+	    } else {
+		skipped_cr = TRUE;
+	    }
+	} else {
+	    if ( c != '\n' ) {
+		if ( skipped_cr ) {
+		    // Should not have skipped it, append now
+		    result += '\r';
+		}
+	    }
+	    skipped_cr = FALSE;
+	    result += c;
+	}
+	c = ts_getc();
+    }
+    return result;
 }
 
 
@@ -793,11 +1363,11 @@ QString QTextStream::readLine()
 QTextStream &QTextStream::operator<<( char c )
 {
     CHECK_STREAM_PRECOND
-    dev->putch( c );
+    unsigned char uc = (unsigned char) c;
+    ts_putc( uc );
     return *this;
 }
 
-
 QTextStream &QTextStream::output_int( int format, ulong n, bool neg )
 {
     static char hexdigits_lower[] = "0123456789abcdef";
@@ -870,32 +1440,32 @@ QTextStream &QTextStream::output_int( int format, ulong n, bool neg )
 		*--p = '-';
 	    else if ( flags() & showpos )
 		*--p = '+';
-	    if ( (flags() & internal) && fwidth && !isdigit(*p) ) {
-		dev->putch( *p );		// special case for internal
+	    if ( (flags() & internal) && fwidth && !ts_isdigit(*p) ) {
+		ts_putc( *p );			// special case for internal
 		++p;				//   padding
 		fwidth--;
-		return *this << (const char *)p;
+		return *this << (const char*)p;
 	    }
     }
     if ( fwidth ) {				// adjustment required
 	if ( !(flags() & left) ) {		// but NOT left adjustment
 	    len = strlen(p);
 	    int padlen = fwidth - len;
-	    if ( padlen <= 0 )			// no padding required
-		dev->writeBlock( p, len );
-	    else if ( padlen < (int)(p-buf) ) { // speeds up padding
-		memset( p-padlen, fillchar, padlen );
-		dev->writeBlock( p-padlen, padlen+len );
+	    if ( padlen <= 0 ) {		// no padding required
+		writeBlock( p, len );
+	    } else if ( padlen < (int)(p-buf) ) { // speeds up padding
+		memset( p-padlen, (char)fillchar, padlen );
+		writeBlock( p-padlen, padlen+len );
 	    }
 	    else				// standard padding
-		*this << (const char *)p;
+		*this << (const char*)p;
 	}
 	else
-	    *this << (const char *)p;
+	    *this << (const char*)p;
 	fwidth = 0;				// reset field width
     }
     else
-	dev->writeBlock( p, strlen(p) );
+	writeBlock( p, strlen(p) );
     return *this;
 }
 
@@ -1009,9 +1579,9 @@ QTextStream &QTextStream::operator<<( double f )
     *fs = '\0';
     sprintf( buf, format, f );			// convert to text
     if ( fwidth )				// padding
-	*this << (const char *)buf;
+	*this << (const char*)buf;
     else					// just write it
-	dev->writeBlock( buf, strlen(buf) );
+	writeBlock( buf, strlen(buf) );
     return *this;
 }
 
@@ -1020,7 +1590,7 @@ QTextStream &QTextStream::operator<<( double f )
   Writes a string to the stream and returns a reference to the stream.
 */
 
-QTextStream &QTextStream::operator<<( const char *s )
+QTextStream &QTextStream::operator<<( const char* s )
 {
     CHECK_STREAM_PRECOND
     char padbuf[48];
@@ -1036,20 +1606,46 @@ QTextStream &QTextStream::operator<<( const char *s )
 	    } else {
 		ppad = padbuf;
 	    }
-	    memset( ppad, fillchar, padlen );	// fill with fillchar
+	    memset( ppad, (char)fillchar, padlen );	// fill with fillchar
 	    if ( !(flags() & left) ) {
-		dev->writeBlock( ppad, padlen );
+		writeBlock( ppad, padlen );
 		padlen = 0;
 	    }
-	    dev->writeBlock( s, len );
+	    writeBlock( s, len );
 	    if ( padlen )
-		dev->writeBlock( ppad, padlen );
+		writeBlock( ppad, padlen );
 	    if ( ppad != padbuf )		// delete extra big fill buf
 		delete[] ppad;
 	    return *this;
 	}
     }
-    dev->writeBlock( s, len );
+    writeBlock( s, len );
+    return *this;
+}
+
+QTextStream &QTextStream::operator<<( const QCString & s )
+{
+    return operator<<(s.data());
+}
+
+/*!
+  Writes a string to the stream and returns a reference to the stream.
+*/
+
+QTextStream &QTextStream::operator<<( const QString& s )
+{
+    CHECK_STREAM_PRECOND
+    uint len = s.length();
+    QString s1 = s;
+    if ( fwidth ) {				// field width set
+	if ( !(flags() & left) ) {
+	    s1 = s.rightJustify(fwidth, (char)fillchar);
+	} else {
+	    s1 = s.leftJustify(fwidth, (char)fillchar);
+	}
+	fwidth = 0;				// reset width
+    }
+    writeBlock( s1.unicode(), len );
     return *this;
 }
 
@@ -1072,35 +1668,6 @@ QTextStream &QTextStream::operator<<( void *ptr )
 }
 
 
-/*!
-  Reads \e len bytes from the stream into \e e s and returns a reference to
-  the stream.
-
-  The buffer \e s must be preallocated.
-
-  \sa QIODevice::readBlock()
-*/
-
-QTextStream &QTextStream::readRawBytes( char *s, uint len )
-{
-    dev->readBlock( s, len );
-    return *this;
-}
-
-/*!
-  Writes the \e len bytes from \e s to the stream and returns a reference to
-  the stream.
-
-  \sa QIODevice::writeBlock()
-*/
-
-QTextStream &QTextStream::writeRawBytes( const char *s, uint len )
-{
-    dev->writeBlock( s, len );
-    return *this;
-}
-
-
 /*!
   \fn int QTextStream::flags() const
   Returns the current stream flags. The default value is 0.
@@ -1242,14 +1809,7 @@ QTextStream &flush( QTextStream &s )
 
 QTextStream &ws( QTextStream &s )
 {
-    register QIODevice *d = s.device();
-    if ( d ) {
-	int c = d->getch();
-	while ( c != EOF && isspace(c) )
-	    c = d->getch();
-	if ( c != EOF )
-	    d->ungetch( c );
-    }
+    s.skipWhiteSpace();
     return s;
 }
 
@@ -1258,3 +1818,160 @@ QTextStream &reset( QTextStream &s )
     s.reset();
     return s;
 }
+
+
+/*!
+  \class QTextIStream qtextstream.h
+  \brief A convenience class for input streams.
+
+  For simple tasks, code should be simple.  Hence this
+  class is a shorthand to avoid passing the \e mode argument
+  to the normal QTextStream constructors.
+
+  This makes it easy for example, to write things like this:
+\code
+    QString data = "123 456";
+    int a, b;
+    QTextIStream(&data) >> a >> b;
+\endcode
+
+  \sa QTextOStream
+*/
+
+/*!
+  \fn QTextIStream::QTextIStream( QString *s )
+
+  Constructs a stream to read from string \a s.
+*/
+/*!
+  \fn QTextIStream::QTextIStream( QByteArray ba )
+
+  Constructs a stream to read from the array \a ba.
+*/
+/*!
+  \fn QTextIStream::QTextIStream( FILE *f )
+
+  Constructs a stream to read from the file \a f.
+*/
+
+
+/*!
+  \class QTextOStream qtextstream.h
+  \brief A convenience class for output streams.
+
+  For simple tasks, code should be simple.  Hence this
+  class is a shorthand to avoid passing the \e mode argument
+  to the normal QTextStream constructors.
+
+  This makes it easy for example, to write things like this:
+\code
+    QString result;
+    QTextOStream(&result) << "pi = " << 3.14;
+\endcode
+*/
+
+/*!
+  \fn QTextOStream::QTextOStream( QString *s )
+
+  Constructs a stream to write to string \a s.
+*/
+/*!
+  \fn QTextOStream::QTextOStream( QByteArray ba )
+
+  Constructs a stream to write to the array \a ba.
+*/
+/*!
+  \fn QTextOStream::QTextOStream( FILE *f )
+
+  Constructs a stream to write to the file \a f.
+*/
+
+
+
+/*!
+  Sets the encoding of this stream to \a e, where \a e is one of:
+  <ul>
+  <li> \c Locale Using local file format (Latin1 if locale is not
+  set), but autodetecting Unicode(utf16) on input.
+  <li> \c Unicode Using Unicode(utf16) for input and output. Output
+  will be written in the order most efficient for the current platform
+  (i.e. the order used internally in QString).
+  <li> \c Latin1  ISO-8859-1. Will not autodetect utf16.
+  <li> \c UnicodeNetworkOrder Using network order Unicode(utf16) for
+  input and output. Useful when reading Unicode data that does not
+  start with the byte order marker.
+  <li> \c UnicodeReverse Using reverse network order Unicode(utf16)
+  for input and output. Useful when reading Unicode data that does not
+  start with the byte order marker, or writing data that should be
+  read by buggy Windows applications.
+  <li> \c RawUnicode Like Unicode, but does not write the byte order
+  marker, nor does it autodetect the byte order. Only useful when
+  writing to non-persistent storage used by a single process.
+  </ul>
+
+  \c Locale and all Unicode encodings, except \c RawUnicode, will look
+  at the first two bytes in a input stream to determine the byte
+  order. The initial byte order marker will be stripped off before data is read.
+
+  \note This function should be called before any data is read to/written from the stream.
+  \sa setCodec
+*/
+
+void QTextStream::setEncoding( Encoding e )
+{
+    switch ( e ) {
+    case Unicode:
+	mapper = 0;
+	latin1 = FALSE;
+	doUnicodeHeader = TRUE;
+	internalOrder = TRUE;
+	break;
+    case UnicodeNetworkOrder:
+	mapper = 0;
+	latin1 = FALSE;
+	doUnicodeHeader = TRUE;
+	internalOrder = QChar::networkOrdered();
+	break;
+    case UnicodeReverse:
+	mapper = 0;
+	latin1 = FALSE;
+	doUnicodeHeader = TRUE;
+	internalOrder = !QChar::networkOrdered();   //reverse network ordered
+	break;
+    case RawUnicode:
+	mapper = 0;
+	latin1 = FALSE;
+	doUnicodeHeader = FALSE;
+	internalOrder = TRUE;
+	break;
+    case Locale:
+	latin1 = TRUE; //fallback to Latin 1
+	mapper = QTextCodec::codecForLocale();
+	if ( mapper && mapper->mibEnum() == 4 ) {
+	    // Use optimized latin1 processing
+	    mapper = 0;
+	}
+	doUnicodeHeader = TRUE; // If it reads as Unicode, accept it
+	break;
+    case Latin1:
+	mapper = 0;
+	doUnicodeHeader = FALSE;
+	latin1 = TRUE;
+	break;
+    }
+}
+
+
+/*!
+  Sets the codec for this stream to \a codec. Will not try to autodetect Unicode.
+
+    \note This function should be called before any data is read to/written from the stream.
+    \sa setEncoding
+*/
+
+void QTextStream::setCodec( QTextCodec *codec )
+{
+    mapper = codec;
+    doUnicodeHeader = FALSE;
+}
+
