@@ -1,38 +1,38 @@
 /****************************************************************************
-** $Id: qbutton.cpp,v 2.40 1997/10/09 15:19:36 eiriken Exp $
+** $Id: qbutton.cpp,v 2.95.2.2 1999/09/14 08:12:12 aavit Exp $
 **
 ** Implementation of QButton widget class
 **
 ** Created : 940206
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qbutton.h"
-#include "qbttngrp.h"
+#include "qbuttongroup.h"
 #include "qbitmap.h"
 #include "qpainter.h"
-#include "qkeycode.h"
 #include "qtimer.h"
 #include "qaccel.h"
-#include "qpmcache.h"
+#include "qpixmapcache.h"
+#include "qfocusdata.h"
 #include <ctype.h>
 
-RCSTAG("$Id: qbutton.cpp,v 2.40 1997/10/09 15:19:36 eiriken Exp $");
-
-
 static const int autoRepeatDelay  = 300;
 static const int autoRepeatPeriod = 100;
 
@@ -40,30 +40,40 @@ static const int drawingPixWidth  = 300;
 static const int drawingPixHeight = 100;
 
 
-/* 
+/*
   Returns a pixmap of dimension (drawingPixWidth x drawingPixHeight). The
   pixmap is used by paintEvent for flicker-free drawing.
  */
 
-static QPixmap *getDrawingPixmap()
+static QPixmap *drawpm = 0;
+
+
+static void cleanupButtonPm()
 {
-    QPixmap *pm = QPixmapCache::find( "$qt-button-drawpix" );
-    if ( !pm ) {
-	pm = new QPixmap( drawingPixWidth, drawingPixHeight );
-	CHECK_PTR( pm );
-	QPixmapCache::insert( "$qt-button-drawpix", pm );
+    delete drawpm;
+    drawpm = 0;
+}
+
+
+static inline void makeDrawingPixmap()
+{
+    if ( !drawpm ) {
+	qAddPostRoutine( cleanupButtonPm );
+	drawpm = new QPixmap( drawingPixWidth, drawingPixHeight );
+	CHECK_PTR( drawpm );
     }
-    return pm;
 }
 
 
 struct QButtonData
 {
-    QButtonData() : group(0) {}
+    QButtonData() : group(0), a(0) {}
     QButtonGroup *group;
-    QTimer	  timer;
+    QTimer timer;
+    QAccel *a;
 };
 
+
 void QButton::ensureData()
 {
     if ( !d ) {
@@ -73,17 +83,20 @@ void QButton::ensureData()
     }
 }
 
+
 QButtonGroup *QButton::group() const
 {
     return d ? d->group : 0;
 }
 
+
 void QButton::setGroup( QButtonGroup* g )
 {
     ensureData();
     d->group = g;
 }
 
+
 QTimer *QButton::timer()
 {
     ensureData();
@@ -91,27 +104,11 @@ QTimer *QButton::timer()
 }
 
 
-/*
-  Internal function that returns the shortcut character in a string.
-  Returns zero if no shortcut character was found.
-  Example:
-    shortcutChar("E&xit") returns 'x'.
-*/
-
-static int shortcutChar( const char *str )
-{
-    const char *p = str ? strchr(str, '&') : 0;
-    while ( p && *p && p[1] == '&' )
-	p = strchr( p+2, '&' );
-    return (p && *p && p[1] && p[1] != '&') ? p[1] : 0;
-}
-
-
 /*!
   \class QButton qbutton.h
 
   \brief The QButton class is the abstract base class of button
-  widgets, and provides functionality common to buttons.
+  widgets, providing functionality common to buttons.
 
   \ingroup abstractwidgets
 
@@ -119,8 +116,8 @@ static int shortcutChar( const char *str )
   specify how to reply to user actions and how to draw the button.
 
   QButton provides both push and toggle buttons.  The QRadioButton and
-  QCheckBox classes provide only toggle buttons, QPushButton provides
-  both toggle and push buttons.
+  QCheckBox classes provide only toggle buttons, QPushButton and
+  QToolButton provide both toggle and push buttons.
 
   Any button can have either a text or pixmap label.  setText() sets
   the button to be a text button and setPixmap() sets it to be a
@@ -159,13 +156,15 @@ static int shortcutChar( const char *str )
   <li>clicked() is emitted when the button is first pressed then
   released, or when the accelerator key is typed, or when animateClick()
   is called.
-  <li>toggled() is emitted when the state of a toggle button changes.
+  <li>toggled(bool) is emitted when the state of a toggle button changes.
+  <li>stateChanged(int) is emitted when the state of a tristate
+			toggle button changes.
   </ul>
 
-  If the button is a text button with "&" in its text, QButton creates an
-  automatic accelerator key.  This code creates a push button labelled "Rock
-  & Roll" (where the c is underscored).  The button gets an automatic
-  accelerator key, Alt-C:
+  If the button is a text button with "&" in its text, QButton creates
+  an automatic accelerator key.  This code creates a push button
+  labelled "Rock & Roll" (where the c is underscored).  The button
+  gets an automatic accelerator key, Alt-C:
 
   \code
     QPushButton *p = new QPushButton( "Ro&ck && Roll", this );
@@ -175,17 +174,17 @@ static int shortcutChar( const char *str )
   \link animateClick() animate a click\endlink.
 
   You can also set a custom accelerator using the setAccel() function.
-  This is very useful for pixmap buttons since they have no automatic
-  accelerator.
- 
+  This is useful mostly for pixmap buttons since they have no
+  automatic accelerator.
+
   \code
     QPushButton *p;
-    p->setPixmap( QPixmap("print.gif") );
+    p->setPixmap( QPixmap("print.png") );
     p->setAccel( ALT+Key_F7 );
   \endcode
 
-  All of the buttons provided by Qt (\l QPushButton, \l QCheckBox and \l
-  QRadioButton) support both text and pixmap labels.
+  All of the buttons provided by Qt (\l QPushButton, \l QToolButton,
+  \l QCheckBox and \l QRadioButton) can display both text and pixmaps.
 
   To subclass QButton, you have to reimplement at least drawButton()
   (to draw the button's outskirts) and drawButtonLabel() (to draw its
@@ -198,9 +197,9 @@ static int shortcutChar( const char *str )
   for a subclass of QButton unless you want to take over all drawing.
 
   \sa QButtonGroup
+  <a href="http://www.microsoft.com/win32dev/uiguide/uigui161.htm">Microsoft Style Guide</a>
 */
 
-
 /*!
   Constructs a standard button with a parent widget and a name.
 
@@ -208,14 +207,13 @@ static int shortcutChar( const char *str )
   QButtonGroup::insert().
 */
 
-QButton::QButton( QWidget *parent, const char *name )
-    : QWidget( parent, name )
+QButton::QButton( QWidget *parent, const char *name, WFlags f )
+    : QWidget( parent, name, f )
 {
-    initMetaObject();
     bpixmap    = 0;
-    toggleBt   = FALSE;				// button is not on/off
+    toggleTyp  = SingleShot;			// button is simple
     buttonDown = FALSE;				// button is up
-    buttonOn   = FALSE;				// button is off
+    stat       = Off;				// button is off
     mlbDown    = FALSE;				// mouse left button up
     autoresize = FALSE;				// not auto resizing
     animation  = FALSE;				// no pending animateClick
@@ -275,38 +273,55 @@ QButton::~QButton()
   \sa clicked()
 */
 
+/*!
+  \fn void QButton::stateChanged( int state )
+  This signal is emitted whenever a toggle button changes status.
+  \e state is 2 if the button is on, 1 if it is in the
+  \link QCheckBox::setTristate() "no change" state\endlink
+  or 0 if the button is off.
+
+  This may be the result of a user action, toggle() slot activation,
+  setState(), or because setOn() was called.
+
+  \sa clicked()
+*/
+
 
 /*!
-  \fn const char *QButton::text() const
-  Returns the button text, or 0 if the button has no text.
+  \fn QString QButton::text() const
+  Returns the button text, or 
+  \link QString::operator!() null string\endlink  
+  if the button has no text.
   \sa setText()
 */
 
 /*!
-  Sets the button contents to \e text and redraws the contents.
+  Sets the button to display \e text and repaints
 
-  The button resizes itself if auto-resizing is enabled.
+  The button resizes itself if auto-resizing is enabled, changes its
+  minimum size if autoMinimumSize() is enabled, and sets the
+  appropriate accelerator.
 
-  \sa text(), setPixmap(), setAutoResize()
+  \sa text(), setPixmap(), setAutoMinimumSize(), setAutoResize(),
+  setAccel(), QPixmap::mask()
 */
 
-void QButton::setText( const char *text )
+void QButton::setText( const QString &text )
 {
     if ( btext == text )
 	return;
-    int oldAccelChar = shortcutChar(btext);
-    int newAccelChar = shortcutChar(text);
     btext = text;
+
     if ( bpixmap ) {
 	delete bpixmap;
 	bpixmap = 0;
     }
+
     if ( autoresize )
 	adjustSize();
-    if ( oldAccelChar && !newAccelChar && !accel() )
-	setAccel( 0 );
-    if ( newAccelChar )
-	setAccel( ALT+toupper(newAccelChar) );
+
+    setAccel( QAccel::shortcutKey( btext ) );
+
     repaint( FALSE );
 }
 
@@ -317,7 +332,7 @@ void QButton::setText( const char *text )
 */
 
 /*!
-  Sets the button pixmap to \a pixmap and redraws the contents.
+  Sets the button to display \a pixmap and repaints at once.
 
   If \a pixmap is monochrome (i.e. it is a QBitmap or its \link
   QPixmap::depth() depth\endlink is 1) and it does not have a mask,
@@ -325,35 +340,36 @@ void QButton::setText( const char *text )
   this is to draw transparent bitmaps, which is important for
   e.g. toggle buttons.
 
-  The button resizes itself if auto-resizing is enabled, and always
-  disables any accelerator.
+  The button resizes itself if auto-resizing is enabled, changes its
+  minimum size if autoMinimumSize() is enabled, and always disables
+  any accelerator.
 
-  \sa pixmap(), setText(), setAutoResize(), setAccel(), QPixmap::mask()
+  \sa pixmap(), setText(), setAutoMinimumSize(), setAutoResize(),
+  setAccel(), QPixmap::mask()
 */
 
 void QButton::setPixmap( const QPixmap &pixmap )
 {
-    int  w, h;
+    bool newSize;
     if ( bpixmap ) {
-	w = bpixmap->width();
-	h = bpixmap->height();
+	newSize = pixmap.width() != bpixmap->width() ||
+		  pixmap.height() != bpixmap->height();
+	*bpixmap = pixmap;
     } else {
-	bpixmap = new QPixmap;
+	newSize = TRUE;
+	bpixmap = new QPixmap( pixmap );
 	CHECK_PTR( bpixmap );
-	w = h = -1;
     }
-    bool sameSize = w == bpixmap->width() && h == bpixmap->height();
-    *bpixmap = pixmap;
     if ( bpixmap->depth() == 1 && !bpixmap->mask() )
 	bpixmap->setMask( *((QBitmap *)bpixmap) );
-    int oldAccelChar = shortcutChar(btext);
     if ( !btext.isNull() )
-	btext.resize( 0 );
-    if ( autoresize && !sameSize )
+	btext = QString::null;
+    if ( autoresize && newSize )
 	adjustSize();
-    if ( oldAccelChar )
-	setAccel( 0 );
+    setAccel( 0 );
     repaint( FALSE );
+    if ( autoMask() )
+	updateMask();
 }
 
 
@@ -365,8 +381,7 @@ void QButton::setPixmap( const QPixmap &pixmap )
 
 int QButton::accel() const
 {
-    QAccel *a = CHILD((QObject*)this,QAccel,"buttonAccel");
-    return a ? a->key(0) : 0;
+    return d && d->a ? d->a->key( 0 ) : 0;
 }
 
 /*!
@@ -394,22 +409,15 @@ int QButton::accel() const
 
 void QButton::setAccel( int key )
 {
-    if ( key == -1 ) {				// guess from the text
-	int c = shortcutChar( text() );
-	key = c ? ALT+toupper(c) : 0;
-    }
-
-    QAccel *a = CHILD(this,QAccel,"buttonAccel");
-    if ( key == 0 ) {				// delete accel
-	delete a;
-    } else {
-	if ( a )
-	    a->clear();
-	else
-	    a = new QAccel( this, "buttonAccel" );
-	a->connectItem( a->insertItem(key,0),
-			this, SLOT(animateClick()) );
-    }
+    if ( d && d->a )
+	d->a->clear();
+    if ( !key )
+	return;
+    ensureData();
+    if ( !d->a )
+	d->a = new QAccel( this, "buttonAccel" );
+    d->a->connectItem( d->a->insertItem( key, 0 ),
+		       this, SLOT(animateClick()) );
 }
 
 
@@ -480,13 +488,11 @@ void QButton::setAutoRepeat( bool enable )
   Performs an animated click: The button is pressed and a short while
   later released.
 
-  pressed(), released(), clicked() and toggled() signals are emitted as
-  appropriate.
+  pressed(), released(), clicked(), toggled(), and stateChanged()
+  signals are emitted as appropriate.
 
   This function does nothing if the button is \link setEnabled()
-  disabled\endlink or if it is the enabled toggle button in an
-  exclusive button group (this means, in practice, a \link
-  QRadioButton radio button \endlink in a button group).
+  disabled. \endlink
 
   \sa setAccel()
 */
@@ -495,8 +501,6 @@ void QButton::animateClick()
 {
     if ( !isEnabled() || animation )
 	return;
-    if ( isOn() && group() && group()->isExclusive() )
-	return;
     animation  = TRUE;
     buttonDown = TRUE;
     repaint( FALSE );
@@ -515,9 +519,9 @@ void QButton::animateClick()
   Sets the state of the button to pressed down if \e enable is TRUE
   or to standing up if \e enable is FALSE.
 
-  If the button is a toggle button, it is \e not toggled.
-  Call toggle() as well if you need to do that.
-  The pressed() and released() signals are not emitted by this function.
+  If the button is a toggle button, it is \e not toggled.  Call
+  toggle() as well if you need to do that.  The pressed() and
+  released() signals are not emitted by this function.
 
   This method is provided in case you need to override the mouse event
   handlers.
@@ -534,6 +538,38 @@ void QButton::setDown( bool enable )
     }
 }
 
+/*!
+  \fn QButton::ToggleType QButton::toggleType() const
+
+  Returns the current toggle type.
+
+  \sa setToggleType()
+*/
+
+/*!
+  \fn void QButton::setState(ToggleState)
+  Protected function to set the button state into one of the
+  three states:
+  <ul>
+   <li>\c QButton::Off - isOn() is FALSE
+   <li>\c QButton::On - isOn() is TRUE
+   <li>\c QButton::NoChange - the button is in the
+	    \link QCheckBox::setTristate() NoChanged\endlink state.
+  </ul>
+
+  \sa setToggleType()
+*/
+
+/*!
+  \fn QButton::ToggleState QButton::state() const
+  Returns the state of the button, one of:
+  <ul>
+   <li>\c QButton::Off - isOn() is FALSE
+   <li>\c QButton::On - isOn() is TRUE
+   <li>\c QButton::NoChange - the button is in the
+	    \link QCheckBox::setTristate() NoChanged\endlink state.
+  </ul>
+*/
 
 /*!
   \fn bool QButton::isOn() const
@@ -543,21 +579,30 @@ void QButton::setDown( bool enable )
 */
 
 /*!
+  \fn void QButton::setOn( bool enable )
+
   Switches a toggle button on if \e enable is TRUE or off if \e enable is
   FALSE.  This function should be called only for toggle buttons.
   \sa isOn(), toggleButton()
 */
 
-void QButton::setOn( bool enable )
+void QButton::setState( ToggleState s )
 {
+    if ( !toggleTyp ) {
 #if defined(CHECK_STATE)
-    if ( !toggleBt )
-	warning( "QButton::setOn: Only toggle buttons may be switched" );
+	qWarning( "QButton::setState() / setOn: (%s) Only toggle buttons "
+		 "may be switched", name( "unnamed" ) );
 #endif
-    if ( (bool)buttonOn != enable ) {		// changed state
-	buttonOn = enable;
+	return;
+    }
+
+    if ( (ToggleState)stat != s ) {		// changed state
+	bool was = stat != Off;
+	stat = s;
 	repaint( FALSE );
-	emit toggled( buttonOn );
+	if ( was != (stat != Off) )
+	    emit toggled( stat != Off );
+	emit stateChanged( s );
     }
 }
 
@@ -569,6 +614,8 @@ void QButton::setOn( bool enable )
 */
 
 /*!
+  \fn void QButton::setToggleButton( bool enable )
+
   Makes the button a toggle button if \e enable is TRUE, or a normal button
   if \e enable is FALSE.
 
@@ -580,12 +627,6 @@ void QButton::setOn( bool enable )
   \sa isToggleButton()
 */
 
-void QButton::setToggleButton( bool enable )
-{
-    toggleBt = enable;
-}
-
-
 /*!
   Returns TRUE if \e pos is inside the widget rectangle, or FALSE if it
   is outside.
@@ -610,7 +651,7 @@ void QButton::drawButton( QPainter * )
 }
 
 /*!
-  Draws the button text or pixmap.  The default implementation does nothing.
+  Draws the button text or pixmap.
 
   This virtual function is reimplemented by subclasses to draw real buttons.
 */
@@ -624,15 +665,61 @@ void QButton::drawButtonLabel( QPainter * )
 /*!
   Handles keyboard events for the button.
 
-  Space is the only key that has any effect; it calls animateClick().
+  Space calls animateClick(), the arrow keys cause focus changes.
 */
 
-void QButton::keyPressEvent( QKeyEvent *e ) 
+void QButton::keyPressEvent( QKeyEvent *e )
 {
-    if ( e->key() == Key_Space )
+    if ( e->key() == Key_Space ) {
 	animateClick();
-    else
+    } else if ( group() &&
+	 ( e->key() == Key_Up ||
+	   e->key() == Key_Left ||
+	   e->key() == Key_Down ||
+	   e->key() == Key_Right ) ) {
+	group()->moveFocus( e->key() );
+	return;
+    } else if ( e->key() == Key_Up || e->key() == Key_Left ) {
+	focusNextPrevChild( FALSE );
+    } else if ( e->key() == Key_Down || e->key() == Key_Right ) {
+	focusNextPrevChild( TRUE );
+    } else {
 	e->ignore();
+    }
+}
+
+
+
+/*! \reimp */
+
+bool QButton::focusNextPrevChild( bool next )
+{
+    if ( !group() || !inherits( "QRadioButton" ) )
+	return QWidget::focusNextPrevChild( next );
+
+    QFocusData *f = focusData();
+
+    QWidget *startingPoint = f->home();
+    QWidget *candidate = 0;
+    QWidget *w = next ? f->prev() : f->next();
+
+    do {
+	if ( w != startingPoint &&
+	     !( w->inherits( "QRadioButton" ) &&
+		((QButton*)w)->group() == group() ) &&
+	     ( ( w->focusPolicy() & TabFocus ) == TabFocus ) &&
+	     !w->focusProxy() &&
+	     w->isVisibleToTLW() &&
+	     w->isEnabledToTLW() )
+	    candidate = w;
+	w = next ? f->prev() : f->next();
+    } while( w && !(candidate || w==startingPoint) );
+
+    if ( !candidate )
+	return QWidget::focusNextPrevChild( next );
+
+    candidate->setFocus();
+    return TRUE;
 }
 
 
@@ -663,19 +750,14 @@ void QButton::mousePressEvent( QMouseEvent *e )
 
 void QButton::mouseReleaseEvent( QMouseEvent *e)
 {
-    if ( e->button() != LeftButton )
+    if ( e->button() != LeftButton || !mlbDown )
 	return;
     if ( d )
 	timer()->stop();
     mlbDown = FALSE;				// left mouse button up
-    bool hit = hitButton( e->pos() );
     buttonDown = FALSE;
-    if ( hit ) {				// mouse release on button
-	if ( toggleBt )
-	    buttonOn = !buttonOn;
-	repaint( FALSE );
-	if ( toggleBt )
-	    emit toggled( buttonOn );
+    if ( hitButton( e->pos() ) ) {		// mouse release on button
+	nextState();
 	emit released();
 	emit clicked();
     } else {
@@ -693,8 +775,7 @@ void QButton::mouseMoveEvent( QMouseEvent *e )
 {
     if ( !((e->state() & LeftButton) && mlbDown) )
 	return;					// left mouse button is up
-    bool hit = hitButton( e->pos() );
-    if ( hit ) {				// mouse move in button
+    if ( hitButton( e->pos() ) ) {		// mouse move in button
 	if ( !buttonDown ) {
 	    buttonDown = TRUE;
 	    repaint( FALSE );
@@ -718,37 +799,26 @@ void QButton::mouseMoveEvent( QMouseEvent *e )
 
 void QButton::paintEvent( QPaintEvent *event )
 {
-    QPainter paint;
-    
-    if ( event && width() <= drawingPixWidth && height() <= drawingPixHeight ) {
-	QPixmap *pm = getDrawingPixmap();
-	ASSERT( pm );
-
-	paint.begin( pm );
-	paint.setFont( font() );
-	pm->fill( this, 0, 0 );
+    if ( event &&
+	 width() <= drawingPixWidth &&
+	 height() <= drawingPixHeight ) {
+	makeDrawingPixmap(); // makes file-static drawpm variable
+	drawpm->fill( this, 0, 0 );
+	QPainter paint;
+	paint.begin( drawpm, this );
 	drawButton( &paint );
 	paint.end();
-	
-	bitBlt( this, event->rect().topLeft(), pm, event->rect() );
-    } else {
+
 	paint.begin( this );
-
-	// This optimization is worth it, since we often call repaint()
-	// to draw exactly the whole button.
-	if ( event && !event->rect().contains(rect()) )
-	    paint.setClipRect( event->rect() );
-
-	if ( event )
-	    erase( event->rect() );
-	else
-	    erase();
-
-	drawButton( &paint );
+	// for small buttons it does not make sense to clip, and
+	// besides it led to update bugs for radio buttons
+	paint.drawPixmap( 0, 0, *drawpm );
 	paint.end();
+    } else {
+	erase( event->region() );
+	QPainter paint( this );
+	drawButton( &paint );
     }
-
-
 }
 
 /*!
@@ -759,6 +829,8 @@ void QButton::paintEvent( QPaintEvent *event )
 void QButton::focusInEvent( QFocusEvent * )
 {
     repaint( FALSE );
+    if ( autoMask() )
+	updateMask();
 }
 
 /*!
@@ -769,6 +841,8 @@ void QButton::focusInEvent( QFocusEvent * )
 void QButton::focusOutEvent( QFocusEvent * )
 {
     repaint( FALSE );
+    if ( autoMask() )
+	updateMask();
 }
 
 
@@ -799,19 +873,31 @@ void QButton::animateTimeout()
 	return;
     animation  = FALSE;
     buttonDown = FALSE;
-    if ( toggleBt )
-	buttonOn = !buttonOn;
-    repaint( FALSE );
-    if ( toggleBt )
-	emit toggled( buttonOn );
+    nextState();
     emit released();
     emit clicked();
 }
 
+void QButton::nextState()
+{
+    bool t = isToggleButton() && !( isOn() && isExclusiveToggle() );
+    bool was = stat != Off;
+    if ( t ) {
+	if ( toggleTyp == Tristate )
+	    stat = ( stat + 1 ) % 3;
+	else
+	    stat = stat ? Off : On;
+    }
+    repaint( FALSE );
+    if ( t ) {
+	if ( was != (stat != Off) )
+	    emit toggled( stat != Off );
+	emit stateChanged( stat );
+    }
+}
 
-/*!
-  Reimplemented for implementational reasons.
-*/
+
+/*! \reimp */
 
 void QButton::enabledChange( bool e )
 {
@@ -819,3 +905,48 @@ void QButton::enabledChange( bool e )
 	setDown( FALSE );
     QWidget::enabledChange( e );
 }
+
+
+/*!  if this is a toggle button, toggles it. */
+
+void QButton::toggle()
+{
+    if ( isToggleButton() )
+	 setOn( !isOn() );
+}
+
+/*!
+  Sets the type of toggling behavior.  \a type is one of:
+  <ul>
+   <li>\c SingleShot - pressing the button causes an action, then the
+			button returns to the unpressed state.
+   <li>\c Toggle - pressing the button toggles between an On and and Off
+			state.
+   <li>\c Tristate - pressing the button cycles between three states -
+	    On, Off, and \link QCheckBox::setTristate() NoChanged\endlink.
+  </ul>
+
+  Subclasses use this, and present it with a more comfortable interface.
+*/
+void QButton::setToggleType( ToggleType type )
+{
+    toggleTyp = type;
+    if ( type != Tristate && stat == NoChange ) {
+	setState(On);
+    }
+}
+
+
+
+/*!
+  Returns TRUE if this button behaves exclusively inside a QButtonGroup.
+  In that case, this button can only be toggled off by another buton
+  beinhg toggled on.
+*/
+
+bool QButton::isExclusiveToggle() const
+{
+    return group() && ( group()->isExclusive() ||
+			group()->isRadioButtonExclusive() &&
+			inherits( "QRadioButton" ) );
+}
