@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qheader.cpp,v 2.50.2.1 1998/10/30 14:37:23 paul Exp $
+** $Id: qheader.cpp,v 2.87.2.4 1999/07/17 03:47:36 ettrich Exp $
 **
 ** Implementation of QHeader widget class (table header)
 **
@@ -7,17 +7,19 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -26,79 +28,19 @@
 #include "qdrawutil.h"
 #include "qbitmap.h"
 #include "qbitarray.h"
+#include "qvector.h"
 
-static const int MINSIZE  = 8;
+static const int GRIPMARGIN  = 4;		//half the size of the resize area
 static const int MARKSIZE = 32;
 static const int QH_MARGIN = 4;
 
 
-#if QT_VERSION >= 149
-//#error "Move cursors to QCursor"
-#endif
-
-
-static QCursor *vSplitCur = 0;
-static QCursor *hSplitCur = 0;
-
-#define hsplit_width 32
-#define hsplit_height 32
-static unsigned char hsplit_bits[] = {
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00,
-  0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00,
-  0x00, 0x82, 0x41, 0x00, 0x00, 0x83, 0xc1, 0x00, 0x80, 0x83, 0xc1, 0x01,
-  0xc0, 0xff, 0xff, 0x03, 0xc0, 0xff, 0xff, 0x03, 0x80, 0x83, 0xc1, 0x01,
-  0x00, 0x83, 0xc1, 0x00, 0x00, 0x82, 0x41, 0x00, 0x00, 0x80, 0x01, 0x00,
-  0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00,
-  0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, };
-static unsigned char hsplitm_bits[] = {
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x03, 0x00,
-  0x00, 0xc0, 0x03, 0x00, 0x00, 0xc0, 0x03, 0x00, 0x00, 0xc0, 0x03, 0x00,
-  0x00, 0xc0, 0x03, 0x00, 0x00, 0xc4, 0x23, 0x00, 0x00, 0xc6, 0x63, 0x00,
-  0x00, 0xc7, 0xe3, 0x00, 0x80, 0xc7, 0xe3, 0x01, 0xc0, 0xff, 0xff, 0x03,
-  0xe0, 0xff, 0xff, 0x07, 0xe0, 0xff, 0xff, 0x07, 0xc0, 0xff, 0xff, 0x03,
-  0x80, 0xc7, 0xe3, 0x01, 0x00, 0xc7, 0xe3, 0x00, 0x00, 0xc6, 0x63, 0x00,
-  0x00, 0xc4, 0x23, 0x00, 0x00, 0xc0, 0x03, 0x00, 0x00, 0xc0, 0x03, 0x00,
-  0x00, 0xc0, 0x03, 0x00, 0x00, 0xc0, 0x03, 0x00, 0x00, 0xc0, 0x03, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, };
-
-#define vsplit_width 32
-#define vsplit_height 32
-static unsigned char vsplit_bits[] = {
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x80, 0x01, 0x00, 0x00, 0xc0, 0x03, 0x00, 0x00, 0xe0, 0x07, 0x00,
-  0x00, 0xf0, 0x0f, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00,
-  0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00,
-  0xc0, 0xff, 0xff, 0x03, 0xc0, 0xff, 0xff, 0x03, 0x00, 0x80, 0x01, 0x00,
-  0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x80, 0x01, 0x00,
-  0x00, 0x80, 0x01, 0x00, 0x00, 0xf0, 0x0f, 0x00, 0x00, 0xe0, 0x07, 0x00,
-  0x00, 0xc0, 0x03, 0x00, 0x00, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, };
-static unsigned char vsplitm_bits[] = {
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x00,
-  0x00, 0xc0, 0x03, 0x00, 0x00, 0xe0, 0x07, 0x00, 0x00, 0xf0, 0x0f, 0x00,
-  0x00, 0xf8, 0x1f, 0x00, 0x00, 0xfc, 0x3f, 0x00, 0x00, 0xc0, 0x03, 0x00,
-  0x00, 0xc0, 0x03, 0x00, 0x00, 0xc0, 0x03, 0x00, 0xe0, 0xff, 0xff, 0x07,
-  0xe0, 0xff, 0xff, 0x07, 0xe0, 0xff, 0xff, 0x07, 0xe0, 0xff, 0xff, 0x07,
-  0x00, 0xc0, 0x03, 0x00, 0x00, 0xc0, 0x03, 0x00, 0x00, 0xc0, 0x03, 0x00,
-  0x00, 0xfc, 0x3f, 0x00, 0x00, 0xf8, 0x1f, 0x00, 0x00, 0xf0, 0x0f, 0x00,
-  0x00, 0xe0, 0x07, 0x00, 0x00, 0xc0, 0x03, 0x00, 0x00, 0x80, 0x01, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, };
-
-
 struct QHeaderData
 {
     QArray<QCOORD>	sizes;
-    QArray<char*>	labels;
+    QArray<QCOORD>	heights;
+    QVector<QString>	labels;
+    QVector<QIconSet>	iconsets;
     QArray<int>	        a2l;
     QArray<int>	        l2a;
 
@@ -121,7 +63,7 @@ struct QHeaderData
   This class can be used without a table view, if you need to control
   table-like structures.
 
-  <img src=qheader-m.gif> <img src=qheader-w.gif>
+  <img src=qheader-m.png> <img src=qheader-w.png>
 
   \sa QListView QTableView
   <a href="http://www.microsoft.com/win32dev/uiguide/uigui181.htm">Microsoft Style Guide</a>
@@ -136,7 +78,7 @@ struct QHeaderData
 */
 
 QHeader::QHeader( QWidget *parent, const char *name )
-    : QTableView( parent, name )
+    : QWidget( parent, name )
 {
     orient = Horizontal;
     init( 0 );
@@ -150,7 +92,7 @@ QHeader::QHeader( QWidget *parent, const char *name )
 */
 
 QHeader::QHeader( int n,  QWidget *parent, const char *name )
-    : QTableView( parent, name )
+    : QWidget( parent, name )
 {
     orient = Horizontal;
     init( n );
@@ -161,10 +103,8 @@ QHeader::QHeader( int n,  QWidget *parent, const char *name )
  */
 QHeader::~QHeader()
 {
-    for ( int i=0; i < count(); i++ )
-	if ( data->labels[i] )                      // Avoid purify complaints
-	    delete [] (char *)data->labels[i];
     delete data;
+    data = 0;
 }
 
 /*!
@@ -204,16 +144,28 @@ int QHeader::cellSize( int i ) const
 
 
 /*!
-  Returns the position in pixels of section \a i of the header.  \a i is the
+  Returns the position in pixels of section \a i of the header. The
+  position is measured from the start of the first header. \a i is the
   actual index.
 */
 
 int QHeader::cellPos( int i ) const
 {
-    /* cvs conflict here */
+    int r = cachedPos;
 
-    int r = pPos( i );
-    return r + offset();
+    if ( i >=  cachedIdx  ) {
+	for ( int j = cachedIdx; j < i; j++ )
+	    r += pSize( j );
+    } else { //###
+	for ( int j = cachedIdx-1; j >= i; j-- )
+	    r -= pSize( j );
+    }
+    if ( i != cachedIdx ) {
+	QHeader *This = (QHeader*)this;	//mutable
+	This->cachedIdx = i;
+	This->cachedPos = r;
+    }
+    return r;
 }
 
 
@@ -223,13 +175,13 @@ int QHeader::cellPos( int i ) const
 
 int QHeader::count() const
 {
-    return data->labels.size() - 1;
+    return data->labels.size() - 1;	// Ignore dummy last item
 }
 
 
 
 /*!
-  \fn QHeader::Orientation QHeader::orientation() const
+  \fn Orientation QHeader::orientation() const
 
   Returns \c Horizontal if the header is horizontal, \c Vertical if
   the header is vertical.
@@ -260,35 +212,32 @@ int QHeader::count() const
  */
 void QHeader::init( int n )
 {
-    if ( !hSplitCur )
-	hSplitCur = new QCursor( QBitmap( hsplit_width, hsplit_height, hsplit_bits, TRUE),
-				 QBitmap( hsplit_width, hsplit_height, hsplitm_bits, TRUE)
-				 );
-    if ( !vSplitCur )
-	vSplitCur = new QCursor( QBitmap( vsplit_width, vsplit_height, vsplit_bits, TRUE),
-				 QBitmap( vsplit_width, vsplit_height, vsplitm_bits, TRUE)
-				 );
     state = Idle;
-
+    offs = 0;
+    cachedIdx = 0;
+    cachedPos = 0;
     data = new QHeaderData;
 
-
+    data->labels.setAutoDelete( TRUE );
+    data->iconsets.setAutoDelete( TRUE );
     data->sizes.resize(n+1);
+    data->heights.resize(n+1);
     data->labels.resize(n+1);
+    data->iconsets.resize(n+1);
     data->a2l.resize(n+1);
     data->l2a.resize(n+1);
     data->clicks.resize(n+1);
     data->resize.resize(n+1);
     for ( int i = 0; i < n ; i ++ ) {
-	data->labels[i] = 0;
 	data->sizes[i] = 88;
+	data->heights[i] = fontMetrics().lineSpacing()+6;
 	data->a2l[i] = i;
 	data->l2a[i] = i;
     }
     data->clicks.fill( TRUE );
     data->resize.fill( TRUE );
     data->move = TRUE;
-
+    /*
     setFrameStyle( QFrame::NoFrame );
 
     if ( orient == Horizontal ) {
@@ -302,15 +251,17 @@ void QHeader::init( int n )
 	setNumCols( 1 );
 	setNumRows( n );
     }
+    */
     handleIdx = 0;
-    //################
-    data->labels[n] = 0;
+    //################ ??? What's that supposed to be, Paul?
     data->sizes[n] = 0;
+    data->heights[n] = 0;
     data->a2l[n] = 0;
     data->l2a[n] = 0;
     //#############
     setMouseTracking( TRUE );
     trackingIsOn = FALSE;
+    setBackgroundMode( PaletteButton );
 }
 
 /*!
@@ -324,9 +275,9 @@ void QHeader::init( int n )
 
 void QHeader::setOrientation( Orientation orientation )
 {
-    if (orient==orientation) 
-	return;
+    if (orient==orientation) return;
     orient = orientation;
+    /*
     int n = count();
     if ( orient == Horizontal ) {
 	setCellWidth( 0 );
@@ -340,6 +291,7 @@ void QHeader::setOrientation( Orientation orientation )
 	setNumRows( n );
     }
     updateTableSize();
+    */
     repaint();
 }
 
@@ -413,23 +365,31 @@ void QHeader::unMarkLine( int idx )
 }
 
 /*!
-  Returns the actual index of the section at position \a c, or -1 if outside.
+  Returns the actual index of the section at widget position \a c, or -1 if outside.
  */
 int QHeader::cellAt( int c ) const
 {
-    int i = ( orient == Horizontal ) ?  findCol( c ) :  findRow( c );
+    int pos = c + offset();
+    int i = cachedIdx;
+    if ( pos >= cachedPos ) {
+	while ( i < count() && pos >= cellPos( i+1 ) )
+	    i++;
+    } else {
+	while ( i >= 0 && pos < cellPos( i ) )
+	    i--;
+    }
     return i >= count() ? -1 : i;
 }
 
 
 /*!
-  Tries to find a line that is not a neighbour of  \c handleIdx.
+  Tries to find a line that is not a neighbor of  \c handleIdx.
  */
 int QHeader::findLine( int c )
 {
     int i = cellAt( c );
     if ( i == -1 )
-	return handleIdx; //####### frustrating, but safe behaviour.
+	return handleIdx; //####### frustrating, but safe behavior.
     if ( i == handleIdx )
 	return i;
     if ( i == handleIdx - 1 &&  pPos( handleIdx ) - c > MARKSIZE/2 )
@@ -442,12 +402,24 @@ int QHeader::findLine( int c )
 	return i;
 }
 
-void QHeader::moveAround( int fromIdx, int toIdx )
+void QHeader::moveCell( int fromIdx, int toIdx )
 {
-    if ( fromIdx == toIdx )
+    if ( fromIdx == toIdx ||
+	 fromIdx < 0 || fromIdx > count() ||
+	 toIdx < 0 || toIdx > count() )
 	return;
     int i;
-
+    if ( (fromIdx < cachedIdx) != (toIdx < cachedIdx ) ) {
+	if ( fromIdx < cachedIdx ) {
+	    //lose one section
+	    cachedIdx--;
+	    cachedPos -= pSize( fromIdx );
+	} else {
+	    //gain one section
+	    cachedIdx++;
+	    cachedPos += pSize( fromIdx );
+	}
+    }
     int idx = data->a2l[fromIdx];
     if ( fromIdx < toIdx ) {
 	for ( i = fromIdx; i < toIdx - 1; i++ ) {
@@ -468,91 +440,34 @@ void QHeader::moveAround( int fromIdx, int toIdx )
     }
 }
 
-/*!
-  sets up the painter
-*/
-
-void QHeader::setupPainter( QPainter *p )
-{
-    p->setPen( colorGroup().text() );
-    p->setFont( font() );
-}
-
-
-/*!
-  paints a section of the header
-*/
-
-void QHeader::paintCell( QPainter *p, int row, int col )
-{
-    int i = ( orient == Horizontal ) ? col : row;
-    int size = pSize( i );
-    bool down = (i==handleIdx) && ( state == Pressed || state == Moving );
-
-    QRect fr( 0, 0, orient == Horizontal ?  size : width(),
-	      orient == Horizontal ?  height() : size );
-
-    if ( style() == WindowsStyle )
-	qDrawWinButton( p, fr, colorGroup(), down );
-    else
-	qDrawShadePanel( p, fr, colorGroup(), down );
-
-    int logIdx = mapToLogical(i);
-
-    const char *s = data->labels[logIdx];
-    int d = 0;
-    if ( style() == WindowsStyle  &&
-	 i==handleIdx && ( state == Pressed || state == Moving ) )
-	d = 1;
-
-    QRect r;
-    if (orient == Horizontal )
-      r = QRect( QH_MARGIN+d, 2+d, size - 6, height() - 4 );
-    else
-      r = QRect( QH_MARGIN+d, 2+d, width() - 6, size - 4 );
-
-    if ( s ) {
-	p->drawText ( r, AlignLeft| AlignVCenter|SingleLine, s );
-    } else {
-	QString str;
-	if ( orient == Horizontal )
-	    str.sprintf( "Col %d", logIdx );
-	else
-	    str.sprintf( "Row %d", logIdx );
-	p->drawText ( r, AlignLeft| AlignVCenter|SingleLine, str );
-    }
-}
-
-
 void QHeader::mousePressEvent( QMouseEvent *m )
 {
     if ( m->button() != LeftButton )
 	return;
     handleIdx = 0;
     int c = orient == Horizontal ? m->pos().x() : m->pos().y();
-    int i = 0;
-    while ( i < (int) count() ) {
-	if ( pPos(i+1) - MINSIZE/2 < c &&
-	     c < pPos(i+1) + MINSIZE/2 ) {
-		handleIdx = i+1;
-		oldHIdxSize = cellSize( i );
-	    if ( data->resize.testBit(i) )
-		state = Sliding;
-	    else
-		state = Blocked;
-	    break;
-	} else if (  pPos(i)  < c && c < pPos( i+1 ) ) {
+
+    int i = cellAt( c );
+    if ( i < 0 )
+	return;
+    int p = pPos( i );
+
+    if ( (i != 0 && c < p + GRIPMARGIN) ||
+	 (c > p + pSize( i ) - GRIPMARGIN) ) {
+	if ( c < p + GRIPMARGIN )
 	    handleIdx = i;
-	    moveToIdx = -1;
-	    if ( data->clicks.testBit(i) )
-		state = Pressed;
-	    else
-		state = Blocked;
-	    clickPos = c;
-	    repaint(sRect( handleIdx ));
-	    break;
-	}
-	i++;
+	else
+	    handleIdx = i+1;
+	oldHIdxSize = cellSize( handleIdx - 1 );
+	state = data->resize[ mapToLogical(handleIdx - 1) ]
+		? Sliding : Blocked;
+    } else if ( i >= 0 ) {
+	handleIdx = i;
+	moveToIdx = -1;
+	state = data->clicks[ mapToLogical( i ) ]
+		? Pressed : Blocked;
+	clickPos = c;
+	repaint( sRect( handleIdx ) );
     }
 }
 
@@ -570,13 +485,13 @@ void QHeader::mouseReleaseEvent( QMouseEvent *m )
 	break;
     case Sliding: {
 	int s = orient == Horizontal ? m->pos().x() : m->pos().y();
-	// setCursor( arrowCursor ); // We're probably still there...
+	// unsetCursor(); // We're probably still there...
 	handleColumnResize( handleIdx, s, TRUE );
 	} break;
     case Moving: {
-	setCursor( arrowCursor );
+	unsetCursor();
 	if ( handleIdx != moveToIdx && moveToIdx != -1 ) {
-	    moveAround( handleIdx, moveToIdx );
+	    moveCell( handleIdx, moveToIdx );
 	    emit moved( handleIdx, moveToIdx );
 	    repaint();
 	} else {
@@ -597,85 +512,90 @@ void QHeader::mouseReleaseEvent( QMouseEvent *m )
 
 void QHeader::mouseMoveEvent( QMouseEvent *m )
 {
+    int i, p;
+    bool hit;
+
     int s = orient == Horizontal ? m->pos().x() : m->pos().y();
-    if ( state == Idle ) {
-	bool hit = FALSE;
-	int i = 0;
-	while ( i <= (int) count() ) {
-	    if ( i && pPos(i) - MINSIZE/2 < s && s < pPos(i) + MINSIZE/2 &&
-		 data->resize.testBit(i-1) ) {
+    switch( state ) {
+    case Idle:
+	i = cellAt( s );
+	p = pPos( i );
+	hit = FALSE;
+
+	if (  s < p + GRIPMARGIN || s > p + pSize( i ) - GRIPMARGIN ) {
+	    if ( s < p + GRIPMARGIN )
+		i--;
+	    if ( i >= 0 && data->resize.testBit(mapToLogical(i)) ) {
 		hit = TRUE;
 		if ( orient == Horizontal )
-		    setCursor( *hSplitCur );
+		    setCursor( splitHCursor );
 		else
-		    setCursor( *vSplitCur );
-		break;
+		    setCursor( splitVCursor );
 	    }
-	    i++;
 	}
 	if ( !hit )
-	    setCursor( arrowCursor );
-    } else {
-	switch ( state ) {
-	case Idle:
-	    debug( "QHeader::mouseMoveEvent() (%s) Idle state",
-		   name( "unnamed" ) );
-	    break;
-	case Pressed:
-	case Blocked:
-	    if ( QABS( s - clickPos ) > 4 && data->move ) {
-		state = Moving;
-		moveToIdx = -1;
-		if ( orient == Horizontal )
-		    setCursor( sizeHorCursor );
-		else
-		    setCursor( sizeVerCursor );
-	    }
-	    break;
-	case Sliding:
-	    handleColumnResize( handleIdx, s, FALSE );
-	    break;
-	case Moving: {
-	    int newPos = findLine( s );
-	    if ( newPos != moveToIdx ) {
-		if ( moveToIdx == handleIdx || moveToIdx == handleIdx + 1 )
-		    repaint( sRect(handleIdx) );
-		else
-		    unMarkLine( moveToIdx );
-		moveToIdx = newPos;
-		if ( moveToIdx == handleIdx || moveToIdx == handleIdx + 1 )
-		    paintRect( pPos( handleIdx ), pSize( handleIdx ) );
-		else
-		    markLine( moveToIdx );
-	    }
-	    break;
+	    unsetCursor();
+	break;
+    case Blocked:
+	break;
+    case Pressed:
+	if ( QABS( s - clickPos ) > 4 && data->move ) {
+	    state = Moving;
+	    moveToIdx = -1;
+	    if ( orient == Horizontal )
+		setCursor( sizeHorCursor );
+	    else
+		setCursor( sizeVerCursor );
 	}
-	default:
-	    warning( "QHeader::mouseMoveEvent: (%s) unknown state",
-		     name( "unnamed" ) );
-	    break;
+	break;
+    case Sliding:
+	handleColumnResize( handleIdx, s, FALSE );
+	break;
+    case Moving: {
+	int newPos = findLine( s );
+	if ( newPos != moveToIdx ) {
+	    if ( moveToIdx == handleIdx || moveToIdx == handleIdx + 1 )
+		repaint( sRect(handleIdx) );
+	    else
+		unMarkLine( moveToIdx );
+	    moveToIdx = newPos;
+	    if ( moveToIdx == handleIdx || moveToIdx == handleIdx + 1 )
+		paintRect( pPos( handleIdx ), pSize( handleIdx ) );
+	    else
+		markLine( moveToIdx );
 	}
+	break;
+    }
+    default:
+	qWarning( "QHeader::mouseMoveEvent: (%s) unknown state", name() );
+	break;
     }
 }
 
 void QHeader::handleColumnResize( int index, int s, bool final )
 {
-    int lim = pPos(index-1) + MINSIZE;
+    int lim = pPos(index-1) + 2*GRIPMARGIN;
     if ( s == lim ) return;
     if ( s < lim ) s = lim;
     int oldPos = pPos( index );
     int delta = s - oldPos;
     int lIdx = mapToLogical(index - 1);
     int oldSize = data->sizes[lIdx];
-    int newSize = data->sizes[lIdx] = oldSize + delta;
+    int newSize = oldSize + delta;
+    setCellSize( lIdx, newSize );
     int repaintPos = QMIN( oldPos, s );
-    if ( orient == Horizontal )
-        repaint(repaintPos-2, 0, width(), height());
-    else
-        repaint(0, repaintPos-oldSize+2, width(), height());
+
+    if ( orient == Horizontal ) {
+	if ( repaintPos < width() )
+	    scroll( delta, 0, QRect( repaintPos, 0, width() - repaintPos,  height() ) );
+	repaint( repaintPos - 4, 0, 4, height(), FALSE ); // border between the items
+    } else
+	repaint(0, repaintPos-oldSize+2, width(), height());
+
+
     if ( tracking() && oldSize != newSize )
 	emit sizeChange( lIdx, oldSize, newSize );
-    else if ( final && oldHIdxSize != newSize )
+    else if ( !tracking() && final && oldHIdxSize != newSize )
 	emit sizeChange( lIdx, oldHIdxSize, newSize );
 }
 
@@ -685,37 +605,79 @@ void QHeader::handleColumnResize( int index, int s, bool final )
 
 QRect QHeader::sRect( int i )
 {
-    if ( orient == Horizontal )
+    if ( i < 0 || i >= count() )
+	return rect(); // ### eeeeevil
+    else if ( orient == Horizontal )
 	return QRect( pPos( i ), 0, pSize( i ), height() );
     else
 	return QRect( 0, pPos( i ), width(), pSize( i ) );
 }
 
+
+void QHeader::setLabel( int i, const QIconSet& iconset, const QString &s, int size )
+{
+    if ( i < 0 || i >= count() )
+	return;
+    data->iconsets.insert( i, new QIconSet( iconset ) );
+    setLabel( i, s, size );
+}
+
 /*!
   Sets the text on logical section \a i to \a s. If the section does not exist,
   nothing happens.
   If \a size is non-negative, the section width is set to \a size.
+
+  Any icon set that has been defined for this section remains
+  unchanged.
 */
 
-void QHeader::setLabel( int i, const char *s, int size )
+void QHeader::setLabel( int i, const QString &s, int size )
 {
-    if ( i >= 0 && i < count() ) {
-	if ( data->labels[i] )                      // Avoid purify complaints
-	    delete [] (char *)data->labels[i];
-	data->labels[i] = qstrdup(s);
-	if ( size >= 0 )
-	    data->sizes[i] = size;
-    }
-    repaint();
+    if ( i < 0 || i >= count() )
+	return;
+    data->labels.insert( i, new QString( s ) );
+    if ( size >= 0 )
+	setCellSize( i, size );
+    update();
 }
 
 
 /*!
   Returns the text set on logical section \a i.
 */
-const char* QHeader::label( int i )
+QString QHeader::label( int i ) const
 {
-    return data->labels[i];
+    if ( i < 0 || i >= count() )
+	return QString::null;
+    if ( data->labels[i] )
+	return *(data->labels[i]);
+    else
+	return QString::null;
+}
+
+/*!
+  Returns the icon set set on logical section \a i.
+*/
+QIconSet *QHeader::iconSet( int i) const
+{
+    if ( i < 0 || i >= count() )
+	return 0;
+    return data->iconsets[i];
+}
+
+
+/*!
+  Adds a new section, with icon set \a iconset and label text \a
+  s. Returns the index.  If \a size is non-negative, the section width
+  is set to \a size, otherwise a size currently sufficient for the
+  label is used.
+*/
+int QHeader::addLabel( const QIconSet& iconset, const QString &s, int size )
+{
+    int n = count() + 1; //########### Paul? Why hashes?
+    data->iconsets.resize( n + 1 );
+    data->iconsets.insert( n - 1, new QIconSet( iconset ) );
+    return addLabel( s, size );
 }
 
 /*!
@@ -724,22 +686,39 @@ const char* QHeader::label( int i )
   otherwise a size currently sufficient for the label text is used.
 */
 
-int QHeader::addLabel( const char *s, int size )
+int QHeader::addLabel( const QString &s, int size )
 {
-    int n = count() + 1; //###########
-    data->labels.resize( n + 1 );
-    data->labels[n-1] = qstrdup(s);
+    int n = count() + 1; 		// n is old list size, including dummy
+    data->labels.resize( n + 1 );	// new size including dummy is n+1
+    data->labels.insert( n - 1, new QString( s ) );  // n-1 is last real idx
+    if ( int( data->iconsets.size() ) < n + 1  )
+	data->iconsets.resize( n + 1 );
     data->sizes.resize( n + 1 );
-    if ( size < 0 ) {
-	QFontMetrics fm = fontMetrics();
-        if ( orient == Horizontal )
-            size = -fm.minLeftBearing()
-                   +fm.width( s )
-                   -fm.minRightBearing() + QH_MARGIN*2;
-        else
-            size = fm.lineSpacing() + 6; // Use same size as horizontal QHeader
+    data->heights.resize( n + 1 );
+    int iw = 0;
+    int ih = 0;
+    if ( data->iconsets[n-1] != 0 ) {
+	iw = data->iconsets[n-1]->pixmap( QIconSet::Small, QIconSet::Normal ).width() + 2;
+	ih = data->iconsets[n-1]->pixmap( QIconSet::Small, QIconSet::Normal ).height();
     }
+
+    QFontMetrics fm = fontMetrics();
+    int height = QMAX( fm.lineSpacing() + 6, ih );
+    int width = -fm.minLeftBearing()
+		+fm.width( s )
+		-fm.minRightBearing() + QH_MARGIN*2 + iw;
+
+    if ( size < 0 ) {
+	if ( orient == Horizontal )
+	    size = width;
+	else
+	    size = height;
+    }
+
     data->sizes[n-1] = size;
+    // we abuse the heights as widths for vertical layout
+    data->heights[n-1] = orient == Horizontal ? height : width;
+
     data->a2l.resize( n + 1 );
     data->l2a.resize( n + 1 );
     data->a2l[n-1] = n-1;
@@ -748,44 +727,37 @@ int QHeader::addLabel( const char *s, int size )
     data->resize.resize(n+1);
     data->clicks.setBit(n-1);
     data->resize.setBit(n-1);
-
+#if 0
     //    recalc();
     if ( orient == Horizontal )
 	setNumCols( n );
     else
 	setNumRows( n );
-    repaint(); //####
+#endif
+    update(); //####
     return n - 1;
 }
 
 
-/*!
-  Handles resize events.
-*/
-
-void QHeader::resizeEvent( QResizeEvent * )
-{
-    if ( orient == Horizontal )
-        setCellHeight( height() );
-    else
-        setCellWidth( width() );
-}
-
 /*!
   Returns the recommended size of the QHeader.
 */
 QSize QHeader::sizeHint() const
-{
-    QFontMetrics fm( font() );
-    if ( orient == Horizontal )
+{ // #### seriously buggy if there are lots of columns
+    QFontMetrics fm = fontMetrics();
+    if ( orient == Horizontal ) {
+	int height = fm.lineSpacing() + 6;
+	for ( int i=0 ; i<count() ; i++ )
+	    height = QMAX( height , data->heights[i] );
 	return QSize( count() > 0
 		      ? cellSize( count()-1 ) + cellPos( count()-1 )
 		      : -1,
-		      fm.lineSpacing() + 6 );
+		      height );
+    }
     else {
-        int width = fm.width( " " );
-        for ( int i=0 ; i<count() ; i++ )
-            width = QMAX( width , fm.width( data->labels[i] ) );
+	int width = fm.width( ' ' );
+	for ( int i=0 ; i<count() ; i++ )
+	    width = QMAX( width , data->heights[i] );
 	return QSize( width + 2*QH_MARGIN,
 		      count() > 0
 		      ? cellSize( count()-1 ) + cellPos( count()-1 )
@@ -793,6 +765,19 @@ QSize QHeader::sizeHint() const
     }
 }
 
+/*!
+  A horizontal header may stretch horizontally, but is fixed
+  vertically.
+*/
+
+QSizePolicy QHeader::sizePolicy() const
+{
+    if ( orient == Horizontal )
+	return QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed );
+    else
+	return QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum );
+}
+
 
 /*!
   Scrolls the header such that \a x becomes the leftmost (or uppermost
@@ -801,10 +786,12 @@ QSize QHeader::sizeHint() const
 
 void QHeader::setOffset( int x )
 {
+    int oldOff = offs;
+    offs = x;
     if ( orient == Horizontal )
-	setXOffset( x );
+	scroll( oldOff-offs, 0 );
     else
-	setYOffset( x );
+	scroll( 0, oldOff-offs);
 }
 
 
@@ -812,22 +799,16 @@ void QHeader::setOffset( int x )
 /*!
   Returns the position of actual division line \a i. May return a position
   outside the widget.
+
+  Note that the last division line is numbered count(). (There are one more lines than
+  sections).
  */
 int QHeader::pPos( int i ) const
 {
-    int r = 0;
-    bool ok;
-    if ( orient == Horizontal )
-	ok = colXPos( i, &r );
-    else
-	ok = rowYPos( i, &r );
-    if ( !ok ) {
-	r = 0;
-	for ( int j = 0; j < i; j++ )
-	    r += pSize( j );
-	r -= offset();
-    }
-    return r;
+    if ( i < 0 || i > count() )
+	return 0;
+
+    return cellPos( i ) - offset();
 }
 
 
@@ -836,9 +817,23 @@ int QHeader::pPos( int i ) const
  */
 int QHeader::pSize( int i ) const
 {
+    if ( i < 0 || i >= count() )
+	return 0;
+
     return data->sizes[mapToLogical(i)];
 }
 
+/*!
+  Returns the height of actual section \a i.
+ */
+int QHeader::pHeight( int i ) const
+{
+    if ( i < 0 || i >= count() )
+	return 0;
+
+    return data->heights[mapToLogical(i)];
+}
+
 
 
 /*!
@@ -849,35 +844,9 @@ int QHeader::pSize( int i ) const
 
 int QHeader::offset() const
 {
-     if ( orient == Horizontal )
-	return xOffset();
-    else
-	return yOffset();
+    return offs;
 }
 
-
-/*! \reimp */
-
-int QHeader::cellHeight( int row )
-{
-    if ( orient == Vertical )
-	return pSize( row );
-    else
-	return QTableView::cellHeight();
-}
-
-
-/*! \reimp */
-
-int QHeader::cellWidth( int col )
-{
-    if ( orient == Horizontal )
-	return pSize( col );
-    else
-	return QTableView::cellWidth();
-}
-
-
 /*!
   Translates from actual index \a a to logical index.  Returns -1 if
   \a a is outside the legal range.
@@ -908,30 +877,32 @@ int QHeader::mapToActual( int l ) const
 
 void QHeader::setCellSize( int i, int s )
 {
+    if ( mapToActual(i) < cachedIdx )
+	cachedPos += s - data->sizes[i];
     data->sizes[i] = s;
 }
 
 
 /*!
-  Enable user resizing of column \a i if \a enable is TRUE, disable otherwise.
-  If \a i is negative, resizing is enabled/disabled for all columns.
+  Enable user resizing of logical column \a i if \a enable is TRUE,
+  disable otherwise.  If \a i is negative (as it is by default),
+  resizing is enabled/disabled for all columns.
 
   \sa setMovingEnabled(), setClickEnabled()
 */
 
 void QHeader::setResizeEnabled( bool enable, int i )
 {
-    if ( i < 0 ) {
+    if ( i < 0 )
 	data->resize.fill( enable );
-    } else {
+    else if ( i < count() )
 	data->resize[i] = enable;
-    }
 }
 
 
 /*!
-    Enable the user to exchange  columns if \a enable is TRUE,
-    disable otherwise.
+  Enable the user to exchange  columns if \a enable is TRUE,
+  disable otherwise.
 
   \sa setClickEnabled(), setResizeEnabled()
 */
@@ -943,8 +914,9 @@ void QHeader::setMovingEnabled( bool enable )
 
 
 /*!
-  Enable clicking in column \a i if \a enable is TRUE, disable otherwise.
-  If \a i is negative, clicking is enabled/disabled for all columns.
+  Enable clicking in logical column \a i if \a enable is TRUE, disable
+  otherwise.  If \a i is negative (as it is by default), clicking is
+  enabled/disabled for all columns.
 
   If enabled, the sectionClicked() signal is emitted when the user clicks.
 
@@ -953,9 +925,83 @@ void QHeader::setMovingEnabled( bool enable )
 
 void QHeader::setClickEnabled( bool enable, int i )
 {
-    if ( i < 0 ) {
+    if ( i < 0 )
 	data->clicks.fill( enable );
-    } else {
+    else if ( i < count() )
 	data->clicks[i] = enable;
-    }
 }
+
+
+/*! Paints section \a id of the header, inside rectangle \a fr in
+  widget coordinates.
+*/
+
+void QHeader::paintSection( QPainter *p, int id, QRect fr )
+{
+    bool down = (id==handleIdx) && ( state == Pressed || state == Moving );
+    p->setBrushOrigin( fr.topLeft() );
+    style().drawBevelButton( p, fr.x(), fr.y(), fr.width(), fr.height(),
+			     colorGroup(), down );
+
+    int logIdx = mapToLogical(id);
+    if ( logIdx < 0 )
+	return;
+
+    QString s;
+    if ( data->labels[logIdx] )
+	s = *(data->labels[logIdx]);
+    else if ( orient == Horizontal )
+	s.sprintf( "Col %d", logIdx );
+    else
+	s.sprintf( "Row %d", logIdx );
+
+    int d = 0;
+    if ( style() == WindowsStyle  &&
+	 id==handleIdx && ( state == Pressed || state == Moving ) )
+	d = 1;
+
+    QRect r( fr.x() + QH_MARGIN+d, fr.y() + 2+d,
+	     fr.width() - 6, fr.height() - 4 );
+
+    if ( data->iconsets[logIdx] ) {
+	QIconSet::Mode mode = isEnabled()?QIconSet::Normal:QIconSet::Disabled;
+	QPixmap pixmap = data->iconsets[logIdx]->pixmap( QIconSet::Small, mode );
+	int pixw = pixmap.width();
+	int pixh = pixmap.height();
+	p->drawPixmap( r.left(), r.center().y()-pixh/2, pixmap );
+	r.setLeft( r.left() + pixw + 2 );
+    }
+
+    p->drawText ( r, AlignLeft| AlignVCenter|SingleLine, s );
+}
+
+
+/*!
+  Paints the header.
+*/
+
+void QHeader::paintEvent( QPaintEvent *e )
+{
+    QPainter p( this );
+    p.setPen( colorGroup().buttonText() );
+    int pos = orient == Horizontal
+		     ? e->rect().left()
+		     : e->rect().top();
+    int id = cellAt( pos );
+    if ( id < 0 )
+	if ( pos > 0 )
+	    return;
+	else
+	    id = 0;
+    for ( int i = id; i < count(); i++ ) {
+	QRect r = sRect( i );
+	paintSection( &p, i, r );
+	if ( orient == Horizontal && r. right() >= e->rect().right() ||
+	     orient == Vertical && r. bottom() >= e->rect().bottom() )
+	    return;
+    }
+
+}
+
+
+//#### what about lastSectionCoversAll?
