@@ -1,30 +1,33 @@
 /****************************************************************************
-** $Id: qlcdnumber.cpp,v 2.14 1998/07/03 00:09:49 hanord Exp $
+** $Id: qlcdnumber.cpp,v 2.39.2.2 1999/08/12 11:06:40 aavit Exp $
 **
 ** Implementation of QLCDNumber class
 **
 ** Created : 940518
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qlcdnumber.h"
 #include "qbitarray.h"
 #include "qpainter.h"
-#include <stdio.h>
+//!hanordce #include <stdio.h>
+
 
 /*!
   \class QLCDNumber qlcdnumber.h
@@ -51,19 +54,17 @@
   in the string) and space.  QLCDNumber substitutes spaces for illegal
   characters.
 
-  It is impossible to retrieve the contents of a QLCDNumber object.
+  It is not possible to retrieve the contents of a QLCDNumber object.
   If you need to, we recommend that you connect the signals which feed
   the display() slot to another slot as well and store the value
   there.
 
-  \warning The current version of QLCDNumber does not work well
-  with background pixmaps.
-
   Incidentally, QLCDNumber is the very oldest part of Qt, tracing back
   to a BASIC program on the <a
-  href=http://www.nvg.unit.no/spectrum/>Sinclair Spectrum</a>.
+  href="http://www.nvg.ntnu.no/sinclair/spectrum.htm">Sinclair
+  Spectrum</a>.
 
-  <img src=qlcdnum-m.gif> <img src=qlcdnum-w.gif>
+  <img src=qlcdnum-m.png> <img src=qlcdnum-w.png>
 
   \sa QLabel, QFrame
 */
@@ -99,21 +100,22 @@ static QString int2string( int num, int base, int ndigits, bool *oflow )
 	case QLCDNumber::OCT:
 	    s.sprintf( "%*lo", ndigits, num );
 	    break;
-	case QLCDNumber::BIN: {
-	    char buf[42];
-	    char *p = &buf[41];
-	    uint n = num;
-	    int len = 0;
-	    *p = '\0';
-	    do {
-		*--p = (char)((n&1)+'0');
-		n >>= 1;
-		len++;
-	    } while ( n != 0 );
-	    len = ndigits - len;
-	    if ( len > 0 )
-	    s.fill( ' ', len );
-	    s += p;
+	case QLCDNumber::BIN:
+	    {
+		char buf[42];
+		char *p = &buf[41];
+		uint n = num;
+		int len = 0;
+		*p = '\0';
+		do {
+		    *--p = (char)((n&1)+'0');
+		    n >>= 1;
+		    len++;
+		} while ( n != 0 );
+		len = ndigits - len;
+		if ( len > 0 )
+		s.fill( ' ', len );
+		s += QString::fromLatin1(p);
 	    }
 	    break;
     }
@@ -160,9 +162,9 @@ static QString double2string( double num, int base, int ndigits, bool *oflow )
 }
 
 
-static char *getSegments( char ch )		// gets list of segments for ch
+static const char *getSegments( char ch )		// gets list of segments for ch
 {
-    static char segments[30][8] =
+    static const char segments[30][8] =
        { { 0, 1, 2, 4, 5, 6,99, 0},		// 0	0 / O
 	 { 2, 5,99, 0, 0, 0, 0, 0},		// 1	1
 	 { 0, 2, 3, 4, 6,99, 0, 0},		// 2	2
@@ -294,6 +296,7 @@ void QLCDNumber::init()
     smallPoint = FALSE;
     setNumDigits( ndigits );
     setSegmentStyle( Outline );
+    d = 0;
 }
 
 /*!
@@ -328,7 +331,7 @@ void QLCDNumber::setNumDigits( int numDigits )
 {
     if ( numDigits > 99 ) {
 #if defined(CHECK_RANGE)
-	warning( "QLCDNumber::setNumDigits: (%s) Max 99 digits allowed",
+	qWarning( "QLCDNumber::setNumDigits: (%s) Max 99 digits allowed",
 		 name( "unnamed" ) );
 #endif
 	numDigits = 99;
@@ -412,7 +415,7 @@ QLCDNumber::Mode QLCDNumber::mode() const
 /*!
   Returns the last value set by one of the display() slots.
 
-  If display(const char *) was the last called, 0 is returned.
+  If display(QString ) was the last called, 0 is returned.
 
   \sa intValue(), display()
 */
@@ -426,7 +429,7 @@ double QLCDNumber::value() const
   Returns the last value set by one of the display() slots rounded to the
   nearest integer.
 
-  If display(const char *) was the last called, 0 is returned.
+  If display(QString ) was the last called, 0 is returned.
 
   \sa value(), display()
 */
@@ -450,7 +453,7 @@ void QLCDNumber::display( int num )
     if ( of )
 	emit overflow();
     else
-	internalDisplay( s );
+	internalSetString( s );
 }
 
 
@@ -466,12 +469,12 @@ void QLCDNumber::display( double num )
     if ( of )
 	emit overflow();
     else
-	internalDisplay( s );
+	internalSetString( s );
 }
 
 
 /*!
-  \overload void QLCDNumber::display( const char *s )
+  \overload void QLCDNumber::display( const QString& s )
 
   This version of the function disregards mode() and smallDecimalPoint().
 
@@ -482,10 +485,10 @@ void QLCDNumber::display( double num )
   characters.
 */
 
-void QLCDNumber::display( const char *s )
+void QLCDNumber::display( const QString &s )
 {
     val = 0;
-    internalDisplay( s );
+    internalSetString( s );
 }
 
 /*!
@@ -592,24 +595,12 @@ void QLCDNumber::setSmallDecimalPoint( bool b )
 }
 
 
-/*!
-  Handles resize Events for the QLCDNumber.
-
-  \internal
-
-  FIXME: Why is this implemented and empty?
-*/
-
-void QLCDNumber::resizeEvent( QResizeEvent * )
-{
-}
-
-
 /*!
   Draws the LCD number using painter \e p.  This function is called from
   QFrame::paintEvent().
 */
 
+
 void QLCDNumber::drawContents( QPainter *p )
 {
     if ( smallPoint )
@@ -618,32 +609,32 @@ void QLCDNumber::drawContents( QPainter *p )
 	drawString( digitStr, *p, 0, FALSE );
 }
 
+
 /*!
   \internal
 */
 
-void QLCDNumber::internalDisplay( const char *s )
+void QLCDNumber::internalDisplay( const QString & )
 {
-    QString buffer(ndigits+1);
+    ; // Not used anymore
+}
+
+void QLCDNumber::internalSetString( const QString& s )
+{
+    QString buffer;
     int i;
-    int len = strlen(s);
+    int len = s.length();
     QPainter p( this );
+    QBitArray newPoints(ndigits);
 
     if ( !smallPoint ) {
-	if ( len >= ndigits ) {
-	    for( i=0; i<(int)ndigits; i++ )	// trunc too long string
-		buffer[i] = s[len - ndigits + i];
-	} else {
-	    for( i=0; i<ndigits-len; i++ )	// pad with spaces
-		buffer[i] = ' ';
-	    for( i=0; i<len; i++ )
-		buffer[ndigits - len + i] = s[i];
-	}
-	drawString( buffer, p );
-   } else {
+	if ( len == ndigits )
+	    buffer = s;
+	else
+	    buffer = s.right( ndigits ).rightJustify( ndigits, ' ' );
+    } else {
 	int  index = -1;
 	bool lastWasPoint = TRUE;
-	QBitArray newPoints(ndigits);
 	newPoints.clearBit(0);
 	for ( i=0; i<len; i++ ) {
 	    if ( s[i] == '.' ) {
@@ -675,15 +666,29 @@ void QLCDNumber::internalDisplay( const char *s )
 		newPoints.clearBit(i);
 	    }
 	}
-	drawString(buffer,p,&newPoints);
     }
+
+    if ( buffer == digitStr )
+	return;
+
+    if ( backgroundMode() == FixedPixmap
+	 || colorGroup().brush( QColorGroup::Background ).pixmap() ) {
+	digitStr = buffer;
+	if ( smallPoint )
+	    points = newPoints;
+	repaint( contentsRect() );
+    }
+    else if ( !smallPoint )
+	drawString( buffer, p );
+    else
+	drawString( buffer, p, &newPoints );
 }
 
 /*!
   \internal
 */
 
-void QLCDNumber::drawString( const char *s, QPainter &p,
+void QLCDNumber::drawString( const QString &s, QPainter &p,
 			     QBitArray *newPoints, bool newString )
 {
     QPoint  pos;
@@ -699,7 +704,7 @@ void QLCDNumber::drawString( const char *s, QPainter &p,
     for ( int i=0;  i<ndigits; i++ ) {
 	pos = QPoint( xOffset + xAdvance*i, yOffset );
 	if ( newString )
-	    drawDigit( pos, p, segLen, s[i], digitStr[i] );
+	    drawDigit( pos, p, segLen, s[i], digitStr[i].latin1() );
 	else
 	    drawDigit( pos, p, segLen, s[i]);
 	if ( newPoints ) {
@@ -715,7 +720,7 @@ void QLCDNumber::drawString( const char *s, QPainter &p,
     if ( newString ) {
 	digitStr = s;
 	if ( (int)digitStr.length() > ndigits )
-	    digitStr.resize( ndigits );
+	    digitStr.truncate( ndigits );
 	if ( newPoints )
 	    points = *newPoints;
     }
@@ -736,7 +741,7 @@ void QLCDNumber::drawDigit( const QPoint &pos, QPainter &p, int segLen,
 				// first 9 used if segment table is correct
     int	 nErases;
     int	 nUpdates;
-    char *segs;
+    const char *segs;
     int	 i,j;
 
     const char erase	  = 0;
@@ -786,7 +791,9 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 			      int segLen, bool erase )
 {
     QPoint pt = pos;
-    QColorGroup g = colorGroup();
+    int width = segLen/5;
+
+    const QColorGroup & g = colorGroup();
     QColor lightColor,darkColor,fgColor;
     if ( erase ){
 	lightColor = backgroundColor();
@@ -797,7 +804,6 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	darkColor  = g.dark();
 	fgColor    = g.foreground();
     }
-    int width = segLen/5;
 
 #define LINETO(X,Y) addPoint( a, QPoint(pt.x() + (X),pt.y() + (Y)))
 #define LIGHT
@@ -819,7 +825,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 1 :
-	    pt.ry()++;
+	    pt += QPoint(0 , 1);
 	    p.moveTo(pt);
 	    LIGHT;
 	    LINETO(width,width);
@@ -830,8 +836,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 2 :
-	    pt.rx() += (QCOORD)(segLen - 1);
-	    pt.ry()++;
+	    pt += QPoint(segLen - 1 , 1);
 	    p.moveTo(pt);
 	    DARK;
 	    LINETO(0,segLen - 2);
@@ -841,7 +846,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 3 :
-	    pt.ry() += (QCOORD)segLen;
+	    pt += QPoint(0 , segLen);
 	    p.moveTo(pt);
 	    LIGHT;
 	    LINETO(width,-width/2);
@@ -858,7 +863,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 4 :
-	    pt.ry() += (QCOORD)(segLen + 1);
+	    pt += QPoint(0 , segLen + 1);
 	    p.moveTo(pt);
 	    LIGHT;
 	    LINETO(width,width/2);
@@ -869,8 +874,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 5 :
-	    pt.rx() += (QCOORD)(segLen - 1);
-	    pt.ry() += (QCOORD)(segLen + 1);
+	    pt += QPoint(segLen - 1 , segLen + 1);
 	    p.moveTo(pt);
 	    DARK;
 	    LINETO(0,segLen - 2);
@@ -880,7 +884,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 6 :
-	    pt.ry() += (QCOORD)(segLen*2);
+	    pt += QPoint(0 , segLen*2);
 	    p.moveTo(pt);
 	    LIGHT;
 	    LINETO(width,-width);
@@ -891,10 +895,9 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    break;
 	case 7 :
 	    if ( smallPoint )	// if smallpoint place'.' between other digits
-		pt.rx() += (QCOORD)(segLen + width/2);
+		pt += QPoint(segLen + width/2 , segLen*2);
 	    else
-		pt.rx() += (QCOORD)(segLen/2);
-	    pt.ry() += (QCOORD)(segLen*2);
+		pt += QPoint(segLen/2 , segLen*2);
 	    p.moveTo(pt);
 	    DARK;
 	    LINETO(width,0);
@@ -904,8 +907,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 8 :
-	    pt.ry() += (QCOORD)(segLen/2 + width);
-	    pt.rx() += (QCOORD)(segLen/2 - width/2 + 1);
+	    pt += QPoint(segLen/2 - width/2 + 1 , segLen/2 + width);
 	    p.moveTo(pt);
 	    DARK;
 	    LINETO(width,0);
@@ -915,8 +917,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 9 :
-	    pt.ry() += (QCOORD)(3*segLen/2 + width);
-	    pt.rx() += (QCOORD)(segLen/2 - width/2 + 1);
+	    pt += QPoint(segLen/2 - width/2 + 1 , 3*segLen/2 + width);
 	    p.moveTo(pt);
 	    DARK;
 	    LINETO(width,0);
@@ -927,7 +928,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    break;
 #if defined(CHECK_RANGE)
 	default :
-	    warning( "QLCDNumber::drawSegment: (%s) Internal error."
+	    qWarning( "QLCDNumber::drawSegment: (%s) Internal error."
 		     "  Illegal segment id: %d\n",
 		     name( "unnamed" ), segmentNo );
 #endif
@@ -959,7 +960,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 1 :
-	    pt.ry()++;
+	    pt += QPoint(0,1);
 	    p.moveTo(pt);
 	    LIGHT;
 	    LINETO(width,width);
@@ -970,8 +971,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 2 :
-	    pt.rx() += (QCOORD)(segLen - 1);
-	    pt.ry()++;
+	    pt += QPoint(segLen - 1 , 1);
 	    p.moveTo(pt);
 	    DARK;
 	    LINETO(0,segLen - 2);
@@ -981,7 +981,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 3 :
-	    pt.ry() += (QCOORD)segLen;
+	    pt += QPoint(0 , segLen);
 	    p.moveTo(pt);
 	    LIGHT;
 	    LINETO(width,-width/2);
@@ -998,7 +998,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 4 :
-	    pt.ry() += (QCOORD)(segLen + 1);
+	    pt += QPoint(0 , segLen + 1);
 	    p.moveTo(pt);
 	    LIGHT;
 	    LINETO(width,width/2);
@@ -1009,8 +1009,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 5 :
-	    pt.rx() += (QCOORD)(segLen - 1);
-	    pt.ry() += (QCOORD)(segLen + 1);
+	    pt += QPoint(segLen - 1 , segLen + 1);
 	    p.moveTo(pt);
 	    DARK;
 	    LINETO(0,segLen - 2);
@@ -1020,7 +1019,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 6 :
-	    pt.ry() += (QCOORD)(segLen*2);
+	    pt += QPoint(0 , segLen*2);
 	    p.moveTo(pt);
 	    LIGHT;
 	    LINETO(width,-width);
@@ -1031,10 +1030,9 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    break;
 	case 7 :
 	    if ( smallPoint )	// if smallpoint place'.' between other digits
-		pt.rx() += (QCOORD)(segLen + width/2);
+		pt += QPoint(segLen + width/2 , segLen*2);
 	    else
-		pt.rx() += (QCOORD)(segLen/2);
-	    pt.ry() += (QCOORD)(segLen*2);
+		pt += QPoint(segLen/2 , segLen*2);
 	    p.moveTo(pt);
 	    DARK;
 	    LINETO(width,0);
@@ -1044,8 +1042,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 8 :
-	    pt.ry() += (QCOORD)(segLen/2 + width);
-	    pt.rx() += (QCOORD)(segLen/2 - width/2 + 1);
+	    pt += QPoint(segLen/2 - width/2 + 1 , segLen/2 + width);
 	    p.moveTo(pt);
 	    DARK;
 	    LINETO(width,0);
@@ -1055,8 +1052,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    LINETO(0,0);
 	    break;
 	case 9 :
-	    pt.ry() += (QCOORD)(3*segLen/2 + width);
-	    pt.rx() += (QCOORD)(segLen/2 - width/2 + 1);
+	    pt += QPoint(segLen/2 - width/2 + 1 , 3*segLen/2 + width);
 	    p.moveTo(pt);
 	    DARK;
 	    LINETO(width,0);
@@ -1067,7 +1063,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 	    break;
 #if defined(CHECK_RANGE)
 	default :
-	    warning( "QLCDNumber::drawSegment: (%s) Internal error."
+	    qWarning( "QLCDNumber::drawSegment: (%s) Internal error."
 		     "  Illegal segment id: %d\n",
 		     name( "unnamed" ), segmentNo );
 #endif
@@ -1079,6 +1075,7 @@ void QLCDNumber::drawSegment( const QPoint &pos, char segmentNo, QPainter &p,
 }
 
 
+
 /*!
   Sets the style of the QLCDNumber to \a s.
   <ul>
@@ -1117,7 +1114,7 @@ QLCDNumber::SegmentStyle QLCDNumber::segmentStyle() const
 
 /*!  Returns a suitable size for this LCD number.  The size hint is on
   the small side, so as to be usable as minimum size.
-  
+
   The size hint takes numDigits() and smallDecimalPoint() into
   consideration.
 
@@ -1126,5 +1123,16 @@ QLCDNumber::SegmentStyle QLCDNumber::segmentStyle() const
 
 QSize QLCDNumber::sizeHint() const
 {
-    return QSize( 18 * (numDigits() + (smallDecimalPoint() ? 0 : 1)), 30 );
+    return QSize( 10 + 9 * (numDigits() + (smallDecimalPoint() ? 0 : 1)), 23 );
+}
+
+
+/*!
+  Specifies that this widget may stretch horizontally and
+  vertically beyond the sizeHint().
+*/
+
+QSizePolicy QLCDNumber::sizePolicy() const
+{
+    return QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
 }
