@@ -1,30 +1,31 @@
 /****************************************************************************
-** $Id: qmainwindow.cpp,v 2.28 1998/07/03 00:09:51 hanord Exp $
+** $Id: qmainwindow.cpp,v 2.70.2.4 1999/09/06 16:33:33 paul Exp $
 **
 ** Implementation of QMainWindow class
 **
 ** Created : 980312
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qmainwindow.h"
 
 #include "qlist.h"
-
 #include "qtimer.h"
 #include "qlayout.h"
 #include "qobjectlist.h"
@@ -38,6 +39,9 @@
 #include "qstatusbar.h"
 
 #include "qtooltip.h"
+#include "qwhatsthis.h"
+
+#include "qlayoutengine.h"
 
 /*! \class QMainWindow qmainwindow.h
 
@@ -53,7 +57,7 @@
 
   The central widget is not touched by QMainWindow.  QMainWindow
   manages its geometry, and that is all.  For example, the
-  application/application.cpp exmple (an editor) sets a QMultiLineEdit
+  application/application.cpp example (an editor) sets a QMultiLineEdit
   to be the central widget.
 
   QMainWindow automatically detects the creation of a menu bar or
@@ -82,19 +86,32 @@
   toolbars.  This feature is planned for inclusion in one of the next
   releases.
 
-  <img src=qmainwindow-m.gif> <img src=qmainwindow-w.gif>
+  <img src=qmainwindow-m.png> <img src=qmainwindow-w.png>
 
   \sa QToolBar QStatusBar QMenuBar QToolTipGroup QDialog
 */
 
+/*!
+  \enum QMainWindow::ToolBarDock
+
+  Each toolbar can be in one of the following positions:
+  <ul>
+    <li>\c Top - above the central widget, below the menubar.
+    <li>\c Bottom - below the central widget, above the status bar.
+    <li>\c Left - to the left of the central widget.
+    <li>\c Right - to the left of the central widget.
+  </ul>
+
+  Other values are also defined for future expansion.
+*/
+
 class QMainWindowPrivate {
 public:
     struct ToolBar {
-	ToolBar() : t(0), l(0), nl(FALSE) {}
-	ToolBar( QToolBar * tb, const char * label, bool n=FALSE )
-	    : t(tb), l(label), nl(n) {}
+	ToolBar() : t(0), nl(FALSE) {}
+	ToolBar( QToolBar * tb, bool n=FALSE )
+	    : t(tb), nl(n) {}
 	QToolBar * t;
-	QString l;
 	bool nl;
     };
 	
@@ -157,6 +174,185 @@ public:
 };
 
 
+
+class QToolLayout : public QLayout
+{
+public:
+    QToolLayout( QWidget *parent, int border=0, int space=-1,
+		const char *name=0 )
+	: QLayout( parent, border, space, name ) { init(); }
+    QToolLayout( QLayout* parent, int space=-1, const char *name=0 )
+	: QLayout( parent, space, name )  { init(); }
+    QToolLayout( int space=-1, const char *name=0 )
+	: QLayout( space, name )  { init(); }
+    ~QToolLayout();
+
+    void addItem( QLayoutItem *item);
+    bool hasHeightForWidth() const { return TRUE; }
+    int heightForWidth( int ) const;
+    QSize sizeHint() const { return minimumSize(); }
+    QSize minimumSize() const;
+    QLayoutIterator iterator();
+    QSizePolicy::ExpandData expanding() const { return QSizePolicy::NoDirection; }
+    void setDirection( QBoxLayout::Direction d ) { dir = d; }
+    QBoxLayout::Direction direction() const { return dir; }
+    void newLine();
+    void setRightJustified( bool on ) { fill = on; }
+    bool rightJustified() const { return fill; }
+    void invalidate();
+protected:
+    void setGeometry( const QRect& );
+private:
+    void init();
+    int layoutItems( const QRect&, bool testonly = FALSE );
+    QList<QLayoutItem> list;
+    QArray<QLayoutStruct> *array;
+    QBoxLayout::Direction dir;
+    bool fill;
+    int cached_width;
+    int cached_hfw;
+};
+
+
+void QToolLayout::init()
+{
+    list.setAutoDelete( TRUE );
+    array = 0;
+    dir = QBoxLayout::Down;
+    fill = FALSE;
+    cached_width = 0;
+}
+
+QToolLayout::~QToolLayout()
+{
+    delete array;
+}
+
+QSize QToolLayout::minimumSize() const
+{
+    QSize s;
+    QListIterator<QLayoutItem> it(list);
+    QLayoutItem *o;
+    while ( (o=it.current()) != 0 ) {
+	++it;
+	if ( o->isEmpty() )
+	    continue;
+	s = s.expandedTo( o->minimumSize() );
+    }
+    return s;
+}
+
+void QToolLayout::invalidate()
+{
+    cached_width = 0;
+    delete array;
+    array=0;
+}
+
+int QToolLayout::layoutItems( const QRect &r, bool testonly )
+{
+    int n = list.count();
+    if ( !testonly ) {
+	if ( !array ) {
+	    array = new QArray<QLayoutStruct>(n);
+	    for (int i = 0; i < n; i++ ) {
+		QLayoutStruct &a = (*array)[i];
+		QLayoutItem *o = list.at(i);
+		a.init();
+		a.sizeHint = o->sizeHint().width();
+		a.expansive = o->expanding() & QSizePolicy::Horizontal;
+	    }
+	}
+	ASSERT( array->size() == list.count() );
+    }
+    bool up = dir == QBoxLayout::Up;
+    int y = r.y();
+    int lineh = 0;		//height of this line so far.
+    int linew = 0;
+
+    int start = 0;
+    int idx = 0;
+
+    QLayoutItem *next = list.first();
+    QSize nsh = next->sizeHint();
+    while ( idx < n ) {
+	QSize sh = nsh;
+	idx++;
+	if ( idx < n ) {
+	    next = list.at(idx);
+	    nsh =  next->sizeHint();
+	} else {
+	    next = 0;
+	    nsh = QSize(0,0);
+	}
+	linew = linew + sh.width() + spacing();
+	lineh = QMAX( lineh, sh.height() );
+	if ( !next || idx > start && linew + nsh.width() > r.width() ) {
+	    //linebreak
+	    if ( !testonly ) {
+		int width = fill ? r.width() : linew - spacing();
+		qGeomCalc( *array, start, idx-start, r.left(), width,
+			   spacing() );
+		for ( int i = start; i < idx; i++ ) {
+		    QRect g( (*array)[i].pos,
+			     up ? r.y() + r.bottom() - y - lineh : y,
+			     (*array)[i].size, lineh );
+		    list.at(i)->setGeometry( g );
+		}
+	    }
+	    y = y + lineh + spacing();
+	    linew = 0;
+	    lineh = 0;
+	    start = idx;
+	}
+		
+    }
+    return y - r.y() - spacing();
+}
+
+
+int QToolLayout::heightForWidth( int w ) const
+{
+    if ( cached_width != w ) {
+	//Not all C++ compilers support "mutable" yet:
+	QToolLayout * mthis = (QToolLayout*)this;
+	int h = mthis->layoutItems( QRect(0,0,w,0), TRUE );
+	mthis->cached_hfw = h;
+	return h;
+    }
+    return cached_hfw;
+}
+
+void QToolLayout::addItem( QLayoutItem *item )
+{
+    list.append( item );
+}
+
+
+class QToolLayoutIterator :public QGLayoutIterator
+{
+public:
+    QToolLayoutIterator( QList<QLayoutItem> *l ) :idx(0), list(l)  {}
+    uint count() const { return list->count(); }
+    QLayoutItem *current() { return idx < (int)count() ? list->at(idx) : 0;  }
+    QLayoutItem *next() { idx++; return current(); }
+    QLayoutItem *takeCurrent() { return list->take( idx ); }
+private:
+    int idx;
+    QList<QLayoutItem> *list;
+};
+
+QLayoutIterator QToolLayout::iterator()
+{
+    return QLayoutIterator( new QToolLayoutIterator( &list ) );
+}
+
+void QToolLayout::setGeometry( const QRect &r )
+{
+    QLayout::setGeometry( r );
+    layoutItems( r );
+}
+
 /*!  Constructs an empty main window. */
 
 QMainWindow::QMainWindow( QWidget * parent, const char * name, WFlags f )
@@ -218,7 +414,7 @@ QMenuBar * QMainWindow::menuBar() const
     else
 	b = new QMenuBar( (QMainWindow *)this, "automatic menu bar" );
     delete l;
-    ((QMainWindowPrivate*)d)->mb = b;
+    d->mb = b;
     d->mb->installEventFilter( this );
     ((QMainWindow *)this)->triggerLayout();
     return b;
@@ -240,8 +436,9 @@ void QMainWindow::setStatusBar( QStatusBar * newStatusBar )
     if ( d->sb )
 	delete d->sb;
     d->sb = newStatusBar;
-    connect( toolTipGroup(), SIGNAL(showTip(const char *)),
-	     d->sb, SLOT(message(const char *)) );
+    // ### this code can cause unnecessary creation of a tool tip group
+    connect( toolTipGroup(), SIGNAL(showTip(const QString&)),
+	     d->sb, SLOT(message(const QString&)) );
     connect( toolTipGroup(), SIGNAL(removeTip()),
 	     d->sb, SLOT(clear()) );
     d->sb->installEventFilter( this );
@@ -250,10 +447,10 @@ void QMainWindow::setStatusBar( QStatusBar * newStatusBar )
 
 
 /*!  Returns the status bar for this window.  If there isn't any,
-  menuBar() creates an empty status bar on the fly, and if necessary a
-  tool tip group too.
+  statusBar() creates an empty status bar on the fly, and if necessary
+  a tool tip group too.
 
-  \sa setMenuBar() statusBar() toolTipGroup()
+  \sa  menuBar() toolTipGroup()
 */
 
 QStatusBar * QMainWindow::statusBar() const
@@ -291,8 +488,8 @@ void QMainWindow::setToolTipGroup( QToolTipGroup * newToolTipGroup )
 	delete d->ttg;
     d->ttg = newToolTipGroup;
 
-    connect( toolTipGroup(), SIGNAL(showTip(const char *)),
-	     statusBar(), SLOT(message(const char *)) );
+    connect( toolTipGroup(), SIGNAL(showTip(const QString&)),
+	     statusBar(), SLOT(message(const QString&)) );
     connect( toolTipGroup(), SIGNAL(removeTip()),
 	     statusBar(), SLOT(clear()) );
     triggerLayout();
@@ -354,7 +551,7 @@ void QMainWindow::setDockEnabled( ToolBarDock dock, bool enable )
 	    break;
 	}
     } else {
-	warning( "oops! unimplemented, untested, and not quite thought out." );
+	qWarning( "oops! unimplemented, untested, and not quite thought out." );
     }
 }
 
@@ -384,16 +581,26 @@ bool QMainWindow::isDockEnabled( ToolBarDock dock ) const
 }
 
 
-/*!  Adds \a toolbar to this the \a edge window of this window.
 
+
+/*!  Adds \a toolBar to this the end of \a edge and makes it start a
+new line of tool bars if \a nl is TRUE.
+
+If \a toolBar is already managed by some main window, it is first
+removed from that window.
 */
 
-void QMainWindow::addToolBar( QToolBar * toolBar, const char * label,
-			      ToolBarDock edge, bool nl )
+void QMainWindow::addToolBar( QToolBar * toolBar,
+			      ToolBarDock edge, bool newLine )
 {
     if ( !toolBar )
 	return;
 
+    if ( toolBar->mw ) {
+	toolBar->mw->removeToolBar( toolBar );
+	toolBar->mw = this;
+    }
+
     setDockEnabled( edge, TRUE );
 
     QMainWindowPrivate::ToolBarDock * dl = 0;
@@ -422,16 +629,32 @@ void QMainWindow::addToolBar( QToolBar * toolBar, const char * label,
     if ( !dl )
 	return;
 
-    dl->append( new QMainWindowPrivate::ToolBar( toolBar, label, nl ) );
+    dl->append( new QMainWindowPrivate::ToolBar( toolBar, newLine ) );
     triggerLayout();
 }
 
 
+/*!  Adds \a toolBar to this the end of \a edge, labelling it \a label
+and makes it start a new line of tool bars if \a newLine is TRUE.
+
+If \a toolBar is already managed by some main window, it is first
+removed from that window.
+*/
+
+void QMainWindow::addToolBar( QToolBar * toolBar, const QString &label,
+			      ToolBarDock edge, bool newLine )
+{
+    if ( toolBar ) {
+	toolBar->setLabel( label );
+	addToolBar( toolBar, edge, newLine );
+    }
+}
+
 
 static QMainWindowPrivate::ToolBar * takeToolBarFromDock( QToolBar * t,
 							  QMainWindowPrivate::ToolBarDock *l )
 {
-    if ( !l )
+    if ( !l || !l->count() )
 	return 0;
     QMainWindowPrivate::ToolBar * ct = l->first();
     do {
@@ -444,6 +667,71 @@ static QMainWindowPrivate::ToolBar * takeToolBarFromDock( QToolBar * t,
 }
 
 
+/*!  Moves \a toolBar to this the end of \a edge.
+
+If \a toolBar is already managed by some main window, it is moved from
+that window to this and set to \e not start a new line.
+*/
+
+void QMainWindow::moveToolBar( QToolBar * toolBar, ToolBarDock edge )
+{
+    if ( !toolBar )
+	return;
+    if ( toolBar->mw )
+	toolBar->mw->removeToolBar( toolBar );
+
+    QMainWindowPrivate::ToolBar * ct;
+    ct = takeToolBarFromDock( toolBar, d->top );
+    if ( !ct )
+	ct = takeToolBarFromDock( toolBar, d->left );
+    if ( !ct )
+	ct = takeToolBarFromDock( toolBar, d->right );
+    if ( !ct )
+	ct = takeToolBarFromDock( toolBar, d->bottom );
+    if ( !ct )
+	ct = takeToolBarFromDock( toolBar, d->tornOff );
+    if ( !ct )
+	ct = takeToolBarFromDock( toolBar, d->unmanaged );
+    if ( ct ) {
+	setDockEnabled( edge, TRUE );
+
+	QMainWindowPrivate::ToolBarDock * dl = 0;
+	if ( edge == Top ) {
+	    dl = d->top;
+	    toolBar->setOrientation( QToolBar::Horizontal );
+	    toolBar->installEventFilter( this );
+	} else if ( edge == Left ) {
+	    dl = d->left;
+	    toolBar->setOrientation( QToolBar::Vertical );
+	    toolBar->installEventFilter( this );
+	} else if ( edge == Bottom ) {
+	    dl = d->bottom;
+	    toolBar->setOrientation( QToolBar::Horizontal );
+	    toolBar->installEventFilter( this );
+	} else if ( edge == Right ) {
+	    dl = d->right;
+	    toolBar->setOrientation( QToolBar::Vertical );
+	    toolBar->installEventFilter( this );
+	} else if ( edge == TornOff ) {
+	    dl = d->tornOff;
+	} else if ( edge == Unmanaged ) {
+	    dl = d->unmanaged;
+	}
+
+	if ( !dl ) {
+	    delete ct;
+	    return;
+	}
+
+	dl->append( ct );
+    } else {
+	addToolBar( toolBar, edge );
+    }
+
+    triggerLayout();
+}
+
+
 /*!  Removes \a toolBar from this main window, if \a toolBar is
   non-null and known by this main window.
 */
@@ -465,64 +753,83 @@ void QMainWindow::removeToolBar( QToolBar * toolBar )
     if ( !ct )
 	ct = takeToolBarFromDock( toolBar, d->unmanaged );
     if ( ct ) {
+	toolBar->mw = 0;
 	delete ct;
 	triggerLayout();
     }
 }
 
 
+
+class QToolBoxLayout : public QBoxLayout
+{
+public:
+    QToolBoxLayout( Direction d ) :QBoxLayout(d) {}
+    QSize minimumSize() const;
+};
+
+QSize QToolBoxLayout::minimumSize() const
+{
+    QSize s(0,0);
+    QLayoutIterator it = ((QToolBoxLayout*)this)->iterator();
+    QLayoutItem *o;
+    while ( (o = it.current()) ) {
+	++it;
+	s = s.expandedTo( o->minimumSize() );
+    }
+    s.setHeight(1);
+    return s;
+}
+
+
+
 static void addToolBarToLayout( QMainWindowPrivate::ToolBarDock * dock,
 				QBoxLayout * tl,
 				QBoxLayout::Direction direction,
 				QBoxLayout::Direction dockDirection,
-				bool mayNeedDockLayout,
+				bool /*mayNeedDockLayout*/,
 				bool justify,
-				GUIStyle style )
+				Qt::GUIStyle style )
 {
     if ( !dock || dock->isEmpty() )
 	return;
 
-    bool moreThanOneRow = FALSE;
-    bool anyToolBars = FALSE;
+    QBoxLayout * dockLayout = tl;
 
-    dock->first();
-    while ( dock->next() ) {
-	if ( dock->current()->nl ) {
-	    moreThanOneRow = TRUE;
-	    break;
-	}
-    }
-
-    QBoxLayout * dockLayout;
-    if ( mayNeedDockLayout && moreThanOneRow ) {
-	dockLayout = new QBoxLayout( dockDirection );
-	tl->addLayout( dockLayout );
-    } else {
-	dockLayout = tl;
-    }
-
-    QBoxLayout * toolBarRowLayout = 0;
-    QMainWindowPrivate::ToolBar * t = dock->first();
-    do {
-	if ( !toolBarRowLayout || t->nl ) {
-	    if ( toolBarRowLayout ) {
-		if ( !justify )
-		    toolBarRowLayout->addStretch( 1 );
+    if ( direction == QBoxLayout::RightToLeft || direction == QBoxLayout::LeftToRight ){
+	QToolLayout *layout = 0;
+	QMainWindowPrivate::ToolBar * t = dock->first();
+	while ( t ) {
+	    if ( !layout || t->nl ) {
+		layout = new QToolLayout( tl );
+		layout->setDirection( dockDirection );
+		layout->setRightJustified( justify );
 	    }
-	    toolBarRowLayout = new QBoxLayout( direction );
-	    dockLayout->addLayout( toolBarRowLayout, 0 );
+	    layout->add(t->t);
+	    t = dock->next();
 	}
-	if ( !t->t->testWFlags( WState_DoHide ) ) {
+    } else {
+	QBoxLayout * toolBarRowLayout = 0;
+	QMainWindowPrivate::ToolBar * t = dock->first();
+	bool anyToolBars;
+	do {
+	    bool nl = t->nl;
+	    if ( !toolBarRowLayout || nl ) {
+		if ( toolBarRowLayout && !justify )
+		    toolBarRowLayout->addStretch( 1 );
+		toolBarRowLayout = new QToolBoxLayout( direction );
+		dockLayout->addLayout( toolBarRowLayout, 0 );
+	    }
 	    toolBarRowLayout->addWidget( t->t, 0 );
-	    anyToolBars = TRUE;
-	}
-    } while ( (t=dock->next()) != 0 );
+	    anyToolBars = TRUE;//####
+	} while ( (t=dock->next()) != 0 );
 
-    if ( anyToolBars && style == MotifStyle )
-	dockLayout->addSpacing( 2 );
+	if ( anyToolBars && style == Qt::MotifStyle )
+	    dockLayout->addSpacing( 2 );
 
-    if ( toolBarRowLayout && (!justify || !anyToolBars) )
-	toolBarRowLayout->addStretch( 1 );
+	if ( toolBarRowLayout && (!justify || !anyToolBars) )
+	    toolBarRowLayout->addStretch( 1 );
+    }
 }
 
 
@@ -533,9 +840,6 @@ static void addToolBarToLayout( QMainWindowPrivate::ToolBarDock * dock,
 
 void QMainWindow::setUpLayout()
 {
-    d->timer->stop();
-    delete d->tll;
-    d->tll = new QBoxLayout( this, QBoxLayout::Down );
     if ( !d->mb ) {
 	// slightly evil hack here.  reconsider this after 1.40.
 	QObjectList * l
@@ -544,10 +848,25 @@ void QMainWindow::setUpLayout()
 	    d->mb = menuBar();
 	delete l;
     }
-    if ( d->mb && !d->mb->testWFlags( WState_DoHide ) )
+    if ( !d->sb ) {
+	// as above.
+	QObjectList * l
+	    = ((QObject*)this)->queryList( "QStatusBar", 0, FALSE, FALSE );
+	if ( l && l->count() )
+	    d->sb = statusBar();
+	delete l;
+    }
+
+    d->timer->stop();
+    delete d->tll;
+    d->tll = new QBoxLayout( this, QBoxLayout::Down );
+    //    d->tll->setResizeMode( QLayout::FreeResize );//############# we need floating layout with height for width here!!!!
+
+    if ( d->mb && !d->mb->testWState(Qt::WState_ForceHide) )
 	d->tll->setMenuBar( d->mb );
     if ( style() == WindowsStyle )
 	d->tll->addSpacing( 1 );
+
     addToolBarToLayout( d->top, d->tll,
 			QBoxLayout::LeftToRight, QBoxLayout::Down, FALSE,
 			d->justify, style() );
@@ -556,7 +875,7 @@ void QMainWindow::setUpLayout()
     addToolBarToLayout( d->left, mwl,
 			QBoxLayout::Down, QBoxLayout::LeftToRight, FALSE,
 			d->justify, style() );
-    if ( centralWidget() && !centralWidget()->testWFlags( WState_DoHide ) )
+    if ( centralWidget() && !centralWidget()->testWState(Qt::WState_ForceHide) )
 	mwl->addWidget( centralWidget(), 1 );
     else
 	mwl->addStretch( 1 );
@@ -566,17 +885,12 @@ void QMainWindow::setUpLayout()
     addToolBarToLayout( d->bottom, d->tll,
 			QBoxLayout::LeftToRight, QBoxLayout::Up, TRUE,
 			d->justify, style() );
-    if ( !d->sb ) {
-	// as above.
-	QObjectList * l
-	    = ((QObject*)this)->queryList( "QStatusBar", 0, FALSE, FALSE );
-	if ( l && l->count() )
-	    d->sb = statusBar();
-	delete l;
-    }
-    if ( d->sb && !d->sb->testWFlags( WState_DoHide ) )
+
+    if ( d->sb && !d->sb->testWState(Qt::WState_ForceHide) )
 	d->tll->addWidget( d->sb, 0 );
+    //debug( "act %d, %d", x(), y() );
     d->tll->activate();
+
 }
 
 
@@ -589,12 +903,14 @@ void QMainWindow::show()
 }
 
 
-/*!  Sets the central widget for this window to \a w.  The centail
+/*!  Sets the central widget for this window to \a w.  The central
   widget is the one around which the toolbars etc. are arranged.
 */
 
 void QMainWindow::setCentralWidget( QWidget * w )
 {
+    if ( d->mc )
+	d->mc->removeEventFilter( this );
     d->mc = w;
     d->mc->installEventFilter( this );
     triggerLayout();
@@ -618,34 +934,70 @@ QWidget * QMainWindow::centralWidget() const
 
 void QMainWindow::paintEvent( QPaintEvent * )
 {
-    if ( style() == WindowsStyle ) {
+#if 0
+    // this code should only be used if there's a menu bar, in Windows
+    // Style, and there's a tool bar immediately below it.  or
+    // something like that.  I'll have to figure it out again.
+    if ( style() == WindowsStyle && d->mb ) {
 	QPainter p( this );
-	int y = menuBar()->height();
+	int y = d->mb->height();
 	p.setPen( colorGroup().dark() );
 	p.drawLine( 0, y, width()-1, y );
     }
+#endif
+}
+
+
+/*!
+  \reimp
+*/
+
+bool QMainWindow::eventFilter( QObject* o, QEvent *e )
+{
+    if ( ( e->type() == QEvent::MouseButtonPress ||
+		  e->type() == QEvent::MouseMove ||
+		  e->type() == QEvent::MouseButtonRelease ) &&
+		0 && // 1.4x
+		o && ( d->moving || o->inherits( "QToolBar" ) ) ) {
+	moveToolBar( d->moving ? d->moving : (QToolBar *)o, (QMouseEvent *)e );
+	return TRUE;
+    }
+    return QWidget::eventFilter( o, e );
 }
 
 
 /*!
   Monitors events to ensure layout is updated.
 */
-
-bool QMainWindow::eventFilter( QObject* o, QEvent *e )
+void QMainWindow::resizeEvent( QResizeEvent* )
 {
-    if ( e->type() == Event_Show || e->type() == Event_Hide ) {
-	triggerLayout();
-#if 0
-	// this is nice code, but it does not work.
-    } else if ( ( e->type() == Event_MouseButtonPress ||
-		  e->type() == Event_MouseMove ||
-		  e->type() == Event_MouseButtonRelease ) &&
-		o && ( d->moving || o->inherits( "QToolBar" ) ) ) {
-	moveToolBar( d->moving ? d->moving : (QToolBar *)o, (QMouseEvent *)e );
-	return TRUE;
-#endif
+    //    setUpLayout(); // #####
+}
+
+/*!
+  Monitors events to ensure layout is updated.
+*/
+void QMainWindow::childEvent( QChildEvent* e)
+{
+    if ( e->type() == QEvent::ChildRemoved ) {
+	if ( e->child() == 0 ||
+	     !e->child()->isWidgetType() ||
+	     ((QWidget*)e->child())->testWFlags( WType_TopLevel ) ) {
+	    // nothing
+	} else if ( e->child() == d->sb ) {
+	    d->sb = 0;
+	    triggerLayout();
+	} else if ( e->child() == d->mb ) {
+	    d->mb = 0;
+	    triggerLayout();
+	} else if ( e->child() == d->mc ) {
+	    d->mc = 0;
+	    triggerLayout();
+	} else if ( e->child()->isWidgetType() ) {
+	    removeToolBar( (QToolBar *)(e->child()) );
+	    triggerLayout();
+	}
     }
-    return QWidget::eventFilter( o, e );
 }
 
 /*!
@@ -654,25 +1006,8 @@ bool QMainWindow::eventFilter( QObject* o, QEvent *e )
 
 bool QMainWindow::event( QEvent * e )
 {
-    if ( e->type() == Event_ChildRemoved ) {
-	QChildEvent * c = (QChildEvent *) e;
-	if ( c->child() == 0 ||
-	     c->child()->testWFlags( WType_TopLevel ) ) {
-	    // nothing
-	} else if ( c->child() == d->sb ) {
-	    d->sb = 0;
-	    triggerLayout();
-	} else if ( c->child() == d->mb ) {
-	    d->mb = 0;
-	    triggerLayout();
-	} else if ( c->child() == d->mc ) {
-	    d->mc = 0;
-	    triggerLayout();
-	} else {
-	    removeToolBar( (QToolBar *)(c->child()) );
-	    triggerLayout();
-	}
-    }
+    if ( e->type() == QEvent::LayoutHint )
+	triggerLayout();
     return QWidget::event( e );
 }
 
@@ -754,12 +1089,9 @@ bool QMainWindow::rightJustification() const
 
 void QMainWindow::triggerLayout()
 {
-    if ( isVisibleToTLW() ) {
-	setUpLayout();
-	d->timer->stop();
-    } else {
-	d->timer->start( 0, TRUE );
-    }
+    delete d->tll;
+    d->tll = 0;
+    d->timer->start( 0, TRUE );
 }
 
 
@@ -767,24 +1099,26 @@ void QMainWindow::triggerLayout()
   the funky docking.
 */
 
-void QMainWindow::moveToolBar( QToolBar * , QMouseEvent * )
+void QMainWindow::moveToolBar( QToolBar* /*t*/ , QMouseEvent * /*e*/ )
 {
 #if 0
-    // again. nice code, shame about bugs.
-    if ( e->type() == Event_MouseButtonPress ) {
+    if ( e->type() == QEvent::MouseButtonPress ) {
+	//debug( "saw press" );
 	d->moving = 0;
 	d->offset = e->pos();
 	d->pos = QCursor::pos();
 	return;
-    } else if ( e->type() == Event_MouseButtonRelease ) {
+    } else if ( e->type() == QEvent::MouseButtonRelease ) {
+	//debug( "saw release" );
 	if ( d->moving ) {
-	    releaseMouse();
 	    qApp->removeEventFilter( this );
 	    d->moving = 0;
 	}
 	return;
     }
 
+    //debug( "saw move" );
+
     // with that out of the way, let's concentrate on the moves...
 
     // first, the threshold
@@ -801,9 +1135,7 @@ void QMainWindow::moveToolBar( QToolBar * , QMouseEvent * )
 
     if ( !d->moving ) {
 	d->moving = t;
-	grabMouse();
 	qApp->installEventFilter( this );
-	debug( "grabbing" );
     }
 
     QPoint lp( mapFromGlobal( p ) );
@@ -813,10 +1145,11 @@ void QMainWindow::moveToolBar( QToolBar * , QMouseEvent * )
 	 !rect().contains( lp ) ) {
 	// not a dock
 	if ( t->parentWidget() ) {
-	    t->recreate( 0, 0,
+	    t->reparent( 0, 0,
 			 QPoint( p.x() - d->offset.x(),
 				 p.y() - d->offset.y() ),
 			 TRUE );
+	    QApplication::syncX();
 	    dock = d->tornOff;
 	} else {
 	    t->move( p.x() - d->offset.x(),
@@ -835,13 +1168,13 @@ void QMainWindow::moveToolBar( QToolBar * , QMouseEvent * )
 	// right dock
 	dock = d->right;
     } else {
-	fatal( "never to happen" );
+	qFatal( "never to happen" );
     }
 
     if ( !dock )
 	return;
 
-    debug( "1" );
+    qDebug( "1" );
     // at this point dock points to the new dock
     QMainWindowPrivate::ToolBar * ct;
     ct = takeToolBarFromDock( t, d->top );
@@ -856,13 +1189,13 @@ void QMainWindow::moveToolBar( QToolBar * , QMouseEvent * )
     if ( dock == d->tornOff || ct == 0 )
 	return;
 
-    debug( "2" );
+    qDebug( "2" );
     QMainWindowPrivate::ToolBar * c = dock->first();
     QRect inLine;
     QRect betweenLines;
     int linestart = 0;
     while( c && ct ) {
-	debug( "3 %p %p", c, ct );
+	qDebug( "3 %p %p", c, ct );
 	if ( c->nl ) {
 	    if ( dock == d->top ) {
 		betweenLines.setRect( 0, 0, width(),
@@ -890,7 +1223,7 @@ void QMainWindow::moveToolBar( QToolBar * , QMouseEvent * )
 	    // ct goes in just before c, and takes over nl
 	    dock->insert( dock->at(), ct );
 	    if ( t->parentWidget() != this )
-		t->recreate( this, 0, QPoint( 0, -t->height() ), TRUE );
+		t->reparent( this, 0, QPoint( 0, -t->height() ), TRUE );
 	    t->setOrientation( (dock == d->top || dock == d->bottom )
 			       ? QToolBar::Horizontal : QToolBar::Vertical );
 	    ct->nl = c->nl;
@@ -905,7 +1238,7 @@ void QMainWindow::moveToolBar( QToolBar * , QMouseEvent * )
 		if ( betweenLines.contains( lp ) ) {
 		    dock->insert( linestart, ct );
 		    if ( t->parentWidget() != this )
-			t->recreate( this, 0, QPoint( 0, -t->height() ),
+			t->reparent( this, 0, QPoint( 0, -t->height() ),
 				     TRUE );
 		    t->setOrientation( (dock == d->top || dock == d->bottom )
 				       ? QToolBar::Horizontal
@@ -928,7 +1261,7 @@ void QMainWindow::moveToolBar( QToolBar * , QMouseEvent * )
 		    if ( inLine.contains( lp ) ) {
 			dock->insert( dock->at(), ct );
 			if ( t->parentWidget() != this )
-			    t->recreate( this, 0, QPoint( 0, -t->height() ),
+			    t->reparent( this, 0, QPoint( 0, -t->height() ),
 					 TRUE );
 			t->setOrientation( (dock == d->top ||
 					    dock == d->bottom )
@@ -943,13 +1276,13 @@ void QMainWindow::moveToolBar( QToolBar * , QMouseEvent * )
 	    c = c2;
 	}
     }
-    debug( "4" );
+    qDebug( "4" );
     // okay, is it at the very end?
     if ( ct ) {
-	debug( "4a" );
+	qDebug( "4a" );
 	dock->append( ct );
 	if ( t->parentWidget() != this )
-	    t->recreate( this, 0, QPoint( 0, -t->height() ), TRUE );
+	    t->reparent( this, 0, QPoint( 0, -t->height() ), TRUE );
 	t->setOrientation( (dock == d->top || dock == d->bottom )
 			   ? QToolBar::Horizontal : QToolBar::Vertical );
 	ct->nl = TRUE;
@@ -957,3 +1290,34 @@ void QMainWindow::moveToolBar( QToolBar * , QMouseEvent * )
     }
 #endif
 }
+
+/*!
+    Enters What's This? question mode and returns immediately.
+
+    This is the same as QWhatsThis::enterWhatsThisMode(), but as a slot of of a
+    main window object. This way it can be easily used for popup menus
+    as in the code fragment:
+
+  \code
+    QPopupMenu * help = new QPopupMenu( this );
+    help->insertItem( "What's &This", this , SLOT(whatsThis()), SHIFT+Key_F1);
+  \endcode
+
+  \sa QWhatsThis::enterWhatsThisMode()
+
+ */
+void QMainWindow::whatsThis()
+{
+    QWhatsThis::enterWhatsThisMode();
+}
+
+
+/*!
+\reimp
+*/
+
+void QMainWindow::styleChange( QStyle& old )
+{
+    setUpLayout();
+    QWidget::styleChange( old );
+}
