@@ -1,22 +1,25 @@
 /****************************************************************************
-** $Id: qmenubar.cpp,v 2.37.2.1 1998/01/13 17:38:28 agulbra Exp $
+** $Id: qmenubar.cpp,v 2.97.2.2 1999/07/08 02:35:03 warwick Exp $
 **
 ** Implementation of QMenuBar class
 **
 ** Created : 941209
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -24,18 +27,16 @@
 #include "qmenubar.h"
 #include "qaccel.h"
 #include "qpainter.h"
-#include "qdrawutl.h"
-#include "qapp.h"
+#include "qdrawutil.h"
+#include "qapplication.h"
 #include <ctype.h>
 
-RCSTAG("$Id: qmenubar.cpp,v 2.37.2.1 1998/01/13 17:38:28 agulbra Exp $");
-
-
 /*!
   \class QMenuBar qmenubar.h
   \brief The QMenuBar class provides a horizontal menu bar.
 
   \ingroup realwidgets
+  \ingroup application
 
   It automatically sets its own geometry to the top of the parent
   widget and changes appropriately it when the parent widget is
@@ -43,11 +44,27 @@ RCSTAG("$Id: qmenubar.cpp,v 2.37.2.1 1998/01/13 17:38:28 agulbra Exp $");
 
   menu/menu.cpp is a typical example of QMenuBar and QPopupMenu use.
 
-  <img src=qmenubar-m.gif> <img src=qmenubar-w.gif>
+  <img src=qmenubar-m.png> <img src=qmenubar-w.png>
 
   \sa QPopupMenu
+  <a href="guibooks.html#fowler">GUI Design Handbook: Menu Bar</a>
 */
 
+
+/*! \enum QMenuBar::Separator
+
+  This enum type is used to decide whether QMenuBar should draw a
+  separator line at its bottom.  The possible values are: <ul>
+
+  <li> \c Never - in many applications, there already is a separator,
+  and two looks stupid.
+
+  <li> \c InWindowsStyle - in some other applications, a separator
+  looks good in Windows style but not else.
+
+  </ul>
+ */
+
 /*!
   \fn void QMenuBar::activated( int id )
 
@@ -117,10 +134,10 @@ static const int motifItemVMargin	= 4;	// menu item ver text margin
 QMenuBar::QMenuBar( QWidget *parent, const char *name )
     : QFrame( parent, name, 0, FALSE )
 {
-    initMetaObject();
     isMenuBar = TRUE;
     autoaccel = 0;
     irects    = 0;
+    rightSide = 0; // Right of hear is rigth-aligned content
     mseparator = 0;
     windowsaltactive = 0;
     if ( parent ) {
@@ -158,8 +175,33 @@ QMenuBar::QMenuBar( QWidget *parent, const char *name )
 	default:
 	    break;
     }
+    setBackgroundMode( PaletteButton );
 }
 
+
+
+/*! \reimp */
+
+void QMenuBar::styleChange( QStyle& old )
+{
+    switch ( style().guiStyle() ) {
+	case WindowsStyle:
+	    setFrameStyle( QFrame::NoFrame );
+	    setMouseTracking( TRUE );
+	    break;
+	case MotifStyle:
+	    setFrameStyle( QFrame::Panel | QFrame::Raised );
+	    setLineWidth( motifBarFrame );
+	    break;
+	default:
+	    break;
+    }
+    updateGeometry();
+    QFrame::styleChange( old );
+}
+
+
+
 /*!
   Destroys the menu bar.
 */
@@ -167,7 +209,8 @@ QMenuBar::QMenuBar( QWidget *parent, const char *name )
 QMenuBar::~QMenuBar()
 {
     delete autoaccel;
-    delete [] irects;
+    if ( irects )		// Avoid purify complaint.
+	delete [] irects;
 }
 
 /*!
@@ -176,7 +219,7 @@ QMenuBar::~QMenuBar()
 */
 void QMenuBar::updateItem( int )
 {
- //   repaint( FALSE ); !!!hanord: avoid this until we get a better solution
+ //   repaint( FALSE ); //###H: avoid this until we get a better solution
 }
 
 
@@ -190,6 +233,7 @@ void QMenuBar::updateItem( int )
 
 void QMenuBar::menuContentsChanged()
 {
+    setupAccelerators();
     badSize = TRUE;				// might change the size
     if ( isVisible() ) {
 	calculateRects();
@@ -207,9 +251,10 @@ void QMenuBar::menuContentsChanged()
 
 void QMenuBar::menuStateChanged()
 {
- //   repaint(); !!!hanord: avoid this until we get a better solution
+    repaint();
 }
 
+
 void QMenuBar::menuInsPopup( QPopupMenu *popup )
 {
     popup->parentMenu = this;			// set parent menu
@@ -247,29 +292,31 @@ void QMenuBar::frameChanged()
 
 bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 {
-    if ( object == parent() && event->type() == Event_Resize ) {
+    if ( object == parent() && event->type() == QEvent::Resize ) {
 	QResizeEvent *e = (QResizeEvent *)event;
-	setGeometry( 0, y(), e->size().width(), height() );
-	calculateRects();
+	int w = e->size().width();
+	setGeometry( 0, y(), w, heightForWidth(w) );
 	return FALSE;
     }
 
     if ( style() != WindowsStyle ||
 	 !isVisibleToTLW() ||
 	 !object->isWidgetType() ||
-	 !( event->type() == Event_Accel ||
-	    event->type() == Event_KeyPress ||
-	    event->type() == Event_KeyRelease ) )
+	 !( event->type() == QEvent::Accel ||
+	    event->type() == QEvent::KeyPress ||
+	    event->type() == QEvent::KeyRelease ) )
 	return FALSE;
 
     // look for Alt press and Alt-anything press
-    if ( event->type() == Event_Accel ) {
+    if ( event->type() == QEvent::Accel ) {
 	QWidget * f = ((QWidget *)object)->focusWidget();
 	QKeyEvent * ke = (QKeyEvent *) event;
 	if ( f ) { // ### this thinks alt and meta are the same
 	    if ( ke->key() == Key_Alt || ke->key() == Key_Meta ) {
 		if ( windowsaltactive || actItem >= 0 ) {
 		    setWindowsAltMode( FALSE, -1 );
+ 		    ke->accept();
+ 		    return TRUE;
 		} else {
 		    windowsaltactive = 1;
 		    if ( f != object )
@@ -281,19 +328,20 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 	    }
 	}
 	// ### ! block all accelerator events when the menu bar is active
-	if ( qApp && qApp->focusWidget() == this )
+	if ( qApp && qApp->focusWidget() == this ) {
 	    return TRUE;
+	}
     }
 
     // look for Alt release
     if ( ((QWidget*)object)->focusWidget() == object ||
 	 (object->parent() == 0 && ((QWidget*)object)->focusWidget() == 0) ) {
 	if ( windowsaltactive &&
-	     event->type() == Event_KeyRelease &&
+	     event->type() == QEvent::KeyRelease &&
 	     (((QKeyEvent *)event)->key() == Key_Alt ||
 	      ((QKeyEvent *)event)->key() == Key_Meta) ) {
-	    actItem = 0;
-	    setFocus();
+	    windowsaltactive = FALSE; // trigger update
+	    setWindowsAltMode( TRUE, 0 );
 	    if ( object->parent() )
 		object->removeEventFilter( this );
 	    QWidget * tlw = ((QWidget *)object)->topLevelWidget();
@@ -304,8 +352,9 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 		tlw->removeEventFilter( this );
 		tlw->installEventFilter( this );
 	    }
-	} else if ( (event->type() == Event_KeyPress ||
-		     event->type() == Event_KeyRelease) &&
+	    return TRUE;
+	} else if ( (event->type() == QEvent::KeyPress ||
+		     event->type() == QEvent::KeyRelease) &&
 		    !(((QKeyEvent *)event)->key() == Key_Alt ||
 		      ((QKeyEvent *)event)->key() == Key_Meta) ) {
 	    if ( object->parent() )
@@ -314,14 +363,6 @@ bool QMenuBar::eventFilter( QObject *object, QEvent *event )
 	}
     }
 
-    // watch for top-level window focus changes
-    if ( windowsaltactive &&
-	 event->type() == Event_FocusOut &&
-	 qApp && qApp->focusWidget() != this ) {
-	setWindowsAltMode( FALSE, -1 );
-	if ( object->parent() )
-	    object->removeEventFilter( this );
-    }
 
     return FALSE;				// don't stop event
 }
@@ -399,8 +440,8 @@ bool QMenuBar::tryMouseEvent( QPopupMenu *popup, QMouseEvent *e )
     if ( !rect().contains( pos ) )		// outside
 	return FALSE;
     int item = itemAtPos( pos );
-    if ( item == -1 && (e->type() == Event_MouseButtonPress ||
-			e->type() == Event_MouseButtonRelease) ) {
+    if ( item == -1 && (e->type() == QEvent::MouseButtonPress ||
+			e->type() == QEvent::MouseButtonRelease) ) {
 	hidePopups();
 	goodbye();
 	return FALSE;
@@ -430,20 +471,31 @@ void QMenuBar::openActPopup()
 	return;
     setWindowsAltMode( FALSE, actItem );
     QPopupMenu *popup = mitems->at(actItem)->popup();
-    if ( !popup )
+    if ( !popup || !popup->isEnabled() )
 	return;
 
     QRect  r = itemRect( actItem );
     QPoint pos = r.bottomLeft() + QPoint(0,1);
-    if ( popup->badSize )
-	popup->updateSize();
+    int ph = popup->sizeHint().height();
     pos = mapToGlobal(pos);
     int sh = QApplication::desktop()->height();
-    int ph = popup->height();
     if ( pos.y() + ph > sh ) {
 	pos = mapToGlobal( r.topLeft() );
 	pos.ry() -= (QCOORD)ph;
     }
+
+    //avoid circularity
+    if ( popup->isVisible() )
+	return;
+
+    if (popup->parentMenu != this ){
+	// reuse
+	if (popup->parentMenu)
+	    popup->parentMenu->menuDelPopup(popup);
+	popup->selfItem  = mitems->at(actItem);
+	menuInsPopup(popup);
+    }
+
     popup->popup( pos );
 }
 
@@ -471,37 +523,9 @@ void QMenuBar::hidePopups()
 
 void QMenuBar::show()
 {
-    QMenuItemListIt it(*mitems);
-    register QMenuItem *mi;
-    QWidget *w = parentWidget();
-    while ( (mi=it.current()) ) {
-	++it;
-	QString s = mi->text();
-	if ( !s.isEmpty() ) {
-	    int i = s.find( '&' );
-	    if ( i >= 0 && isalnum(s[i+1]) ) {
-		int k = s[i+1];
-		if ( isalpha(k) )
-		    k = toupper(k) - 'A' + Key_A;
-		if ( !autoaccel ) {
-		    autoaccel = new QAccel( this );
-		    CHECK_PTR( autoaccel );
-		    connect( autoaccel, SIGNAL(activated(int)),
-			     SLOT(accelActivated(int)) );
-		    connect( autoaccel, SIGNAL(destroyed()),
-			     SLOT(accelDestroyed()) );
-		}
-		autoaccel->insertItem( ALT+k, mi->id() );
-	    }
-	}
-	if ( mi->popup() ) {
-	    mi->popup()->updateAccel( this );
-	    if ( !mi->popup()->isEnabled() )
-		mi->popup()->enableAccel( FALSE );
-	}
-    }
-    if ( w )
-	resize( w->width(), height() );
+    setupAccelerators();
+    if ( parentWidget() )
+	resize( parentWidget()->width(), height() );
     calculateRects();
     QWidget::show();
     raise();
@@ -524,10 +548,13 @@ void QMenuBar::hide()
   Needs to change the size of the menu bar when a new font is set.
 */
 
-void QMenuBar::fontChange( const QFont & )
+void QMenuBar::fontChange( const QFont & f )
 {
     badSize = TRUE;
-    repaint();
+    updateGeometry();
+    if ( isVisible() )
+	calculateRects();
+    QWidget::fontChange( f );
 }
 
 
@@ -547,13 +574,11 @@ int QMenuBar::calculateRects( int max_width )
     bool update = ( max_width < 0 );
 
     if ( update ) {
-        if (width() == 1) { // gcc Linux/Alpha
-	    show();
-	    return 0;
-	}
+	rightSide = 0;
 	if ( !badSize )				// size was not changed
 	    return 0;
-	delete [] irects;
+	if ( irects )		// Avoid purify complaint.
+	    delete [] irects;
 	if ( mitems->isEmpty() ) {
 	    irects = 0;
 	    return 0;
@@ -570,7 +595,7 @@ int QMenuBar::calculateRects( int max_width )
     int y = motifBarFrame + motifBarVMargin;
     int i = 0;
     int separator = -1;
-    if ( gs == WindowsStyle )	// !!!hanord
+    if ( gs == WindowsStyle )	//###H
 	x = y = 2;
     while ( i < (int)mitems->count() ) {	// for each menu item...
 	QMenuItem *mi = mitems->at(i);
@@ -578,8 +603,11 @@ int QMenuBar::calculateRects( int max_width )
 	if ( mi->pixmap() ) {			// pixmap item
 	    w = mi->pixmap()->width();
 	    h = mi->pixmap()->height();
-	} else if ( mi->text() ) {		// text item
-	    w = fm.width(mi->text()) + 2*motifItemHMargin;
+	} else if ( !mi->text().isNull() ) {	// text item
+	    w = fm.width(mi->text())
+		    - fm.minLeftBearing()
+		    - fm.minRightBearing()
+		    + 2*motifItemHMargin;
 	    h = fm.height() + motifItemVMargin;
 	} else if ( mi->isSeparator() ) {	// separator item
 	    separator = i;
@@ -607,9 +635,12 @@ int QMenuBar::calculateRects( int max_width )
     if ( update ) {
 	if ( separator >= 0 && style() == MotifStyle ) {
 	    int moveBy = max_width - x;
+	    rightSide = x;
 	    while( --i > separator ) {
 		irects[i].moveBy( moveBy, 0 );
 	    }
+	} else {
+	    rightSide = width()-frameWidth();
 	}
 	if ( max_height != height() )
 	    resize( max_width, max_height );
@@ -666,10 +697,6 @@ int QMenuBar::itemAtPos( const QPoint &pos )
 }
 
 
-/*# QT_VERSION 200
-  Move the flag for separator usage down from QMenuData to QMenuBar.
-*/
-
 /*!
   When a menubar is used above an unframed widget, it may look better
   with a separating line when displayed with \link QWidget::style()
@@ -679,6 +706,8 @@ int QMenuBar::itemAtPos( const QPoint &pos )
   QMenuBar::Never, or QMenuBar::InWindowsStyle.
 
   The default is QMenuBar::Never.
+
+  \sa Separator separator()
 */
 void QMenuBar::setSeparator( Separator when )
 {
@@ -686,7 +715,9 @@ void QMenuBar::setSeparator( Separator when )
 }
 
 /*!
-  Returns the currently set \link setSeparator() separator usage\endlink.
+  Returns the currently set Separator usage.
+
+  \sa Separator setSeparator()
 */
 QMenuBar::Separator QMenuBar::separator() const
 {
@@ -705,12 +736,12 @@ QMenuBar::Separator QMenuBar::separator() const
 void QMenuBar::drawContents( QPainter *p )
 {
     QColorGroup g;
-    GUIStyle gs = style();
+    GUIStyle gs = style().guiStyle();
     bool e;
 
     for ( int i=0; i<(int)mitems->count(); i++ ) {
 	QMenuItem *mi = mitems->at( i );
-	if ( mi->text() || mi->pixmap() ) {
+	if ( !mi->text().isNull() || mi->pixmap() ) {
 	    QRect r = irects[i];
 	    e = mi->isEnabled();
 	    if ( e )
@@ -718,31 +749,38 @@ void QMenuBar::drawContents( QPainter *p )
 	    else
 		g = palette().disabled();
 		
-	    if ( gs == WindowsStyle ) {
-		p->fillRect( r, palette().normal().background() );
+	    if ( gs == WindowsStyle || style().defaultFrameWidth() < 2) {
+		p->fillRect( r,palette().normal().brush( QColorGroup::Button ) );
 		if ( i == actItem ) {
-		    QBrush b( palette().normal().background() );
+		    QBrush b = palette().normal().brush( QColorGroup::Button );
 		    bool sunken = !windowsaltactive ||
 				  (mi->popup() && mi->popup()->isVisible());
+		    if (sunken)
+			p->setBrushOrigin(p->brushOrigin() + QPoint(1,1));
 		    qDrawShadeRect( p,
 				    r.left(), r.top(), r.width(), r.height(),
 				    g, sunken, 1, 0, &b );
-		    if ( sunken )
+		    if ( sunken ) {
 			r.setRect( r.left()+2, r.top()+2,
 				   r.width()-2, r.height()-2 );
+			p->setBrushOrigin(p->brushOrigin() - QPoint(1,1));
+		    }
 		}
 	    } else { // motif
 		if ( i == actItem ) // active item
 		    qDrawShadePanel( p, r, palette().normal(), FALSE,
-				     motifItemFrame );
+				     motifItemFrame,
+			    &palette().normal().brush( QColorGroup::Button ));
 		else // other item
-		    qDrawPlainRect( p, r, palette().normal().background(),
-				    motifItemFrame );
+		    p->fillRect(r,
+			    palette().normal().brush( QColorGroup::Button ));
+// 		    qDrawPlainRect( p, r, palette().normal().button(),
+// 				    motifItemFrame, &palette().normal().fillButton() );
 	    }
-
-	    qDrawItem( p, gs, r.left(), r.top(), r.width(), r.height(),
-		       AlignCenter|ShowPrefix|DontClip|SingleLine,
-		       g, e, mi->pixmap(), mi->text() );
+	    QColor btext = g.buttonText();
+	    style().drawItem( p, r.left(), r.top(), r.width(), r.height(),
+			      AlignCenter|ShowPrefix|DontClip|SingleLine,
+			      g, e, mi->pixmap(), mi->text(), -1, &btext );
 
 	}
     }
@@ -761,7 +799,8 @@ void QMenuBar::drawContents( QPainter *p )
 
 void QMenuBar::mousePressEvent( QMouseEvent *e )
 {
-    if ( e->button() != LeftButton &&  e->button() != RightButton )
+    if ( e->button() != LeftButton &&
+	 e->button() != RightButton )
 	return;
     mouseBtDn = TRUE;				// mouse button down
     int item = itemAtPos( e->pos() );
@@ -797,7 +836,8 @@ void QMenuBar::mousePressEvent( QMouseEvent *e )
 
 void QMenuBar::mouseReleaseEvent( QMouseEvent *e )
 {
-    if ( e->button() != LeftButton &&  e->button() != RightButton )
+    if ( e->button() != LeftButton &&
+	 e->button() != RightButton )
 	return;
     mouseBtDn = FALSE;				// mouse button up
     int item = itemAtPos( e->pos() );
@@ -895,15 +935,15 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
 
     QMenuItem  *mi = 0;
     QPopupMenu *popup;
-    int d = 0;
+    int dx = 0;
 
     switch ( e->key() ) {
     case Key_Left:
-	d = -1;
+	dx = -1;
 	break;
 
     case Key_Right:
-	d = 1;
+	dx = 1;
 	break;
 
     case Key_Up:
@@ -926,12 +966,12 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
 	break;
     }
 
-    if ( d ) {					// highlight next/prev
+    if ( dx ) {					// highlight next/prev
 	register int i = actItem;
 	int c = mitems->count();
 	int n = c;
 	while ( n-- ) {
-	    i = i + d;
+	    i = i + dx;
 	    if ( i == c )
 		i = 0;
 	    else if ( i < 0 )
@@ -956,8 +996,8 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
 		}
 	    }
 	}
-    } else if ( !e->state() && e->key() >= Key_0 && e->key() <= Key_Z ) {
-	char c = '0' + e->key() - Key_0;
+    } else if ( !e->state() && e->text().length()==1 ) {
+	QChar c = e->text()[0].upper();
 
 	QMenuItemListIt it(*mitems);
 	register QMenuItem *m;
@@ -967,9 +1007,9 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
 	    QString s = m->text();
 	    if ( !s.isEmpty() ) {
 		int i = s.find( '&' );
-		if ( i >= 0 && isalnum(s[i+1]) ) {
-		    int k = s[i+1];
-		    if ( toupper(k) == c ) {
+		if ( i >= 0 )
+		{
+		    if ( s[i+1].upper() == c ) {
 			mi = m;
 			break;
 		    }
@@ -996,14 +1036,19 @@ void QMenuBar::keyPressEvent( QKeyEvent *e )
   Handles resize events for the menu bar.
 */
 
-void QMenuBar::resizeEvent( QResizeEvent *e )
+void QMenuBar::resizeEvent( QResizeEvent * )
 {
-    QSize oldSize = e->oldSize();
-    QSize newSize = e->size();
-    if(oldSize == newSize) return;
+    QRect fr = frameRect();
+    if ( !fr.isNull() ) {
+	QRect r( fr.x(), fr.y(), width(), height() );
+	setFrameRect( r );
+    }
 
+    if ( badSize )
+	return;
     badSize = TRUE;
-    calculateRects();
+    if ( isVisible() )
+	calculateRects();
 }
 
 
@@ -1045,7 +1090,7 @@ void QMenuBar::setActItem( int i, bool clear )
 /*!  Sets the windows alt-key mode to \a enable, optionally setting
   item \a index to be the active item.
 
-  Windows alt-key mode lets the user use the keys to move alond the
+  Windows alt-key mode lets the user use the keys to move along the
   menu bar.  You get into this mode by pressing and releasing alt or
   by moving the mouse onto the menu bar.
 */
@@ -1055,20 +1100,80 @@ void QMenuBar::setWindowsAltMode( bool enable, int index )
     if ( enable ) {
 	if ( !windowsaltactive ) {
 	    windowsaltactive = 1;
-	    if ( style() == WindowsStyle )
+	    if ( style() == WindowsStyle ) {
+		setUpdatesEnabled( FALSE );
 		setFocus();
+		setUpdatesEnabled( TRUE );
+	    }
 	}
-	if ( index == actItem ) // work around setActItem overoptimization
-	    actItem = -1;
 	setActItem( index, FALSE );
     } else {
 	if ( windowsaltactive ) {
-	    if ( style() == WindowsStyle && focusWidget() )
+	    if ( style() == WindowsStyle && focusWidget() ) {
+		setUpdatesEnabled( FALSE );
 		focusWidget()->setFocus();
+		setUpdatesEnabled( TRUE );
+	    }
 	    windowsaltactive = 0;
 	}
-	if ( index == actItem ) // work around setActItem overoptimization
-	    actItem = -1;
 	setActItem( index, FALSE );
     }
 }
+
+
+/*!  Sets up keyboard accelerators for the menu bar. */
+
+void QMenuBar::setupAccelerators()
+{
+    delete autoaccel;
+    autoaccel = 0;
+
+    QMenuItemListIt it(*mitems);
+    register QMenuItem *mi;
+    while ( (mi=it.current()) ) {
+	++it;
+	QString s = mi->text();
+	if ( !s.isEmpty() ) {
+	    int i = QAccel::shortcutKey( s );
+	    if ( i ) {
+		if ( !autoaccel ) {
+		    autoaccel = new QAccel( this );
+		    CHECK_PTR( autoaccel );
+		    autoaccel->setIgnoreWhatsThis( TRUE );
+		    connect( autoaccel, SIGNAL(activated(int)),
+			     SLOT(accelActivated(int)) );
+		    connect( autoaccel, SIGNAL(destroyed()),
+			     SLOT(accelDestroyed()) );
+		}
+		autoaccel->insertItem( i, mi->id() );
+	    }
+	}
+	if ( mi->popup() ) {
+	    // reuse
+	    QPopupMenu* popup = mi->popup();
+	    if (popup->parentMenu)
+		popup->parentMenu->menuDelPopup(popup);
+	    popup->selfItem  = mi;
+	    menuInsPopup(popup);
+	    popup->updateAccel( this );
+	    if ( !popup->isEnabled() )
+		popup->enableAccel( FALSE );
+	}
+    }
+}
+
+/*!\reimp
+ */
+bool QMenuBar::customWhatsThis() const
+{
+    return TRUE;
+}
+
+
+/*!\reimp
+ */
+void QMenuBar::focusOutEvent( QFocusEvent * )
+{
+    if ( windowsaltactive )
+	setWindowsAltMode( FALSE, -1 );
+}
