@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qmenudata.cpp,v 2.20.2.1 1998/11/24 14:53:41 ettrich Exp $
+** $Id: qmenudata.cpp,v 2.39 1999/06/16 21:32:59 warwick Exp $
 **
 ** Implementation of QMenuData class
 **
@@ -7,17 +7,19 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -26,6 +28,11 @@
 #include "qpopupmenu.h"
 #include "qapplication.h"
 
+
+// Not used yet...
+class QMenuItemData { };
+class QMenuDataData { };
+
 /*!
   \class QMenuData qmenudata.h
   \brief The QMenuData class is a base class for QMenuBar and QPopupMenu.
@@ -52,19 +59,24 @@
 QMenuItem::QMenuItem()
 {
     ident	 = -1;
-    is_separator = is_checked = FALSE;
+    is_separator = FALSE;
+    is_checked   = FALSE;
     is_enabled	 = TRUE;
     is_dirty	 = TRUE;
+    iconset_data	 = 0;
     pixmap_data	 = 0;
     popup_menu	 = 0;
     accel_key	 = 0;
     signal_data	 = 0;
+    d = 0; // FOR EXTENSION (eg. non-ascii accels)
 }
 
 QMenuItem::~QMenuItem()
 {
+    delete iconset_data;
     delete pixmap_data;
     delete signal_data;
+    delete d;
 }
 
 
@@ -91,8 +103,12 @@ QMenuData::QMenuData()
     CHECK_PTR( mitems );
     mitems->setAutoDelete( TRUE );
     parentMenu = 0;				// assume top level
-    isPopupMenu = isMenuBar = mouseBtDn = FALSE;
+    isPopupMenu = FALSE;
+    isMenuBar = FALSE;
+    mouseBtDn = FALSE;
     badSize = TRUE;
+    avoid_circularity = 0;
+    d = 0; // FOR EXTENSION
 }
 
 /*!
@@ -108,6 +124,7 @@ QMenuData::~QMenuData()
 	mi = mitems->next();
     }
     delete mitems;				// delete menu item list
+    delete d;
 }
 
 
@@ -166,59 +183,43 @@ uint QMenuData::count() const
 }
 
 
-/*!
-  Internal function that insert a menu item.  Called by all insertItem()
+
+    /*!
+  Internal function that insert a menu item.  Called by all insert()
   functions.
 */
 
-int QMenuData::insertAny( const char *text, const QPixmap *pixmap,
-			  QPopupMenu *popup, int id, int index )
+int QMenuData::insertAny( const QString *text, const QPixmap *pixmap,
+			  QPopupMenu *popup, const QIconSet* iconset, int id, int index )
 {
     if ( index > (int)mitems->count() ) {
 #if defined(CHECK_RANGE)
-	warning( "QMenuData::insertItem: Index %d out of range", index );
+	qWarning( "QMenuData::insertItem: Index %d out of range", index );
 #endif
 	return 0;
     }
     if ( index < 0 )				// append
 	index = mitems->count();
-    if ( id < -1 )				// -2, -3 etc.
+    if ( id < 0 )				// -2, -3 etc.
 	id = get_seq_id();
-    if ( popup && popup->parentMenu ){		// popup already in use
-#if defined(CHECK_STATE)
-	warning( "QMenuData::insertItem: "
-		 "popup already in use" );
-#endif
-  	return 0;
-    }
+
     register QMenuItem *mi = new QMenuItem;
     CHECK_PTR( mi );
-    mi->ident = id == -1 ? index : id;
+    mi->ident = id;
     if ( text == 0 && pixmap == 0 && popup == 0 ) {
 	mi->is_separator = TRUE;		// separator
 	mi->ident	 = -1;
     } else {
-	mi->text_data = text;
+	mi->text_data = text?*text:QString::null;
 	if ( pixmap )
 	    mi->pixmap_data = new QPixmap( *pixmap );
 	mi->popup_menu = popup;
-	if ( popup ) {
+	if ( popup )
 	    popup->selfItem = mi;
-	    menuInsPopup( popup );
-	    if ( isPopupMenu ) {		// circular popups?
-		QPopupMenu *p = (QPopupMenu*)this;
-		while ( p && p != popup )
-		    p = (QPopupMenu*)p->parentMenu;
-		if ( p ) {
-#if defined(CHECK_STATE)
-		    warning( "QMenuData::insertItem: "
-			     "Circular popup menu ignored" );
-#endif
-		    return 0;
-		}
-	    }
-	}
+	if ( iconset )
+	    mi->iconset_data = new QIconSet( *iconset );
     }
+
     mitems->insert( index, mi );
     menuContentsChanged();			// menu data changed
     return mi->ident;
@@ -266,11 +267,10 @@ void QMenuData::setAllDirty( bool dirty )
 }
 
 
-/*!
-  Inserts a menu item with a text and an optional accelerator key, and
-  connects it to an object/slot.
 
-  Returns a unique menu item identifier (negative integer \<= -2).
+/*!
+  Inserts a menu item with a text, an accelerator key, an id and an
+  optional index and connects it to an object/slot.
 
   Example:
   \code
@@ -281,93 +281,20 @@ void QMenuData::setAllDirty( bool dirty )
     mainMenu->insertItem( "File", fileMenu );
   \endcode
 
+  If you will need to translate accelerators, use QAccel::stringToKey()
+  to calculate the accelerator key:
+  \code
+    fileMenu->insertItem( tr("Open"), myView, SLOT(open()),
+			  QAccel::stringToKey( tr("Ctrl+O") ) );
+  \endcode
+
   In the example above, pressing CTRL+N or selecting "open" from the
   menu activates the myView->open() function.
 
-  Note that accelerators only work for QPopupMenu items that live in a
-  menu bar. For stand-alone popup menus, use an independent QAccel
-  object.
-
-  \warning Be careful when passing a literal 0 to insertItem(), as
-	some C++ compilers choose the wrong overloaded function.
-	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
-
-  \sa removeItem(), changeItem(), setAccel(), connectItem(), QAccel,
-  qkeycode.h
-*/
-
-int QMenuData::insertItem( const char *text,
-			   const QObject *receiver, const char *member,
-			   int accel )
-{
-    int id = insertAny( text, 0, 0, -2, -1 );
-    connectItem( id, receiver, member );
-    if ( accel )
-	setAccel( accel, id );
-    return id;
-}
-
-/*!
-  Inserts a menu item with a pixmap and an optional accelerator key, and
-  connects it to an object/slot.
-
-  Returns a unique menu item identifier (negative integer \<= -2).
-
-  \warning Be careful when passing a literal 0 to insertItem(), as
-	some C++ compilers choose the wrong overloaded function.
-	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
-
-  \sa removeItem(), changeItem(), setAccel(), connectItem(), QAccel,
-  qkeycode.h
-*/
-
-int QMenuData::insertItem( const QPixmap &pixmap,
-			   const QObject *receiver, const char *member,
-			   int accel )
-{
-    int id = insertAny( 0, &pixmap, 0, -2, -1 );
-    connectItem( id, receiver, member );
-    if ( accel )
-	setAccel( accel, id );
-    return id;
-}
-
-/*!
-  Inserts a menu item with a pixmap, a text and an optional accelerator
-  key, and connects it to an object/slot. The pixmap will be displayed to
-  the left of the text in the item.
-
-  Returns a unique menu item identifier (negative integer \<= -2).
-
-  \warning Be careful when passing a literal 0 to insertItem(), as
-	some C++ compilers choose the wrong overloaded function.
-	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
-
-  \sa removeItem(), changeItem(), setAccel(), connectItem(), QAccel,
-  qkeycode.h
-*/
-
-int QMenuData::insertItem( const QPixmap &pixmap, const char *text,
-			   const QObject *receiver, const char *member,
-			   int accel )
-{
-    int id = insertAny( text, &pixmap, 0, -2, -1 );
-    connectItem( id, receiver, member );
-    if ( accel )
-	setAccel( accel, id );
-    return id;
-}
-
-/*!
-  Inserts a menu item with a text, an accelerator key, an id and an
-  optional index and connects it to an object/slot.
-
   The menu item is assigned the identifier \a id or an automatically
-  generated identifier.	 It works as follows: If \a id \>= 0, this
-  identifier is assigned.  If \a id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \a id is
-  any other negative integer, for instance -2, a unique identifier
-  (negative integer \<= -2) is generated.
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
 
   The \a index specifies the position in the menu.  The menu item is
   appended at the end of the list if \a index is negative.
@@ -381,14 +308,51 @@ int QMenuData::insertItem( const QPixmap &pixmap, const char *text,
 	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
 
   \sa removeItem(), changeItem(), setAccel(), connectItem(), QAccel,
-  qkeycode.h
+  qnamespace.h
 */
 
-int QMenuData::insertItem( const char *text,
-			   const QObject *receiver, const char *member,
+int QMenuData::insertItem( const QString &text,
+			   const QObject *receiver, const char* member,
 			   int accel, int id, int index )
 {
-    int actualID = insertAny( text, 0, 0, id, index );
+    int actualID = insertAny( &text, 0, 0, 0, id, index );
+    connectItem( actualID, receiver, member );
+    if ( accel )
+	setAccel( accel, actualID );
+    return actualID;
+}
+
+/*!
+  Inserts a menu item with an icon, a text, an accelerator key, an id
+  and an optional index and connects it to an object/slot. The icon
+  will be displayed to the left of the text in the item.
+
+  The menu item is assigned the identifier \a id or an automatically
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
+
+  The \a index specifies the position in the menu.  The menu item is
+  appended at the end of the list if \a index is negative.
+
+  Note that accelerators only work for QPopupMenu items that live in a
+  menu bar. For stand-alone popup menus, use an independent QAccel
+  object.
+
+  \warning Be careful when passing a literal 0 to insertItem(), as
+	some C++ compilers choose the wrong overloaded function.
+	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
+
+  \sa removeItem(), changeItem(), setAccel(), connectItem(), QAccel,
+  qnamespace.h
+*/
+
+int QMenuData::insertItem( const QIconSet& icon,
+			   const QString &text,
+			   const QObject *receiver, const char* member,
+			   int accel, int id, int index )
+{
+    int actualID = insertAny( &text, 0, 0, &icon, id, index );
     connectItem( actualID, receiver, member );
     if ( accel )
 	setAccel( accel, actualID );
@@ -400,11 +364,9 @@ int QMenuData::insertItem( const char *text,
   optional index and connects it to an object/slot.
 
   The menu item is assigned the identifier \a id or an automatically
-  generated identifier.	 It works as follows: If \a id \>= 0, this
-  identifier is assigned.  If \a id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \a id is
-  any other negative integer, for instance -2, a unique identifier
-  (negative integer \<= -2) is generated.
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
 
   The \a index specifies the position in the menu.  The menu item is
   appended at the end of the list if \a index is negative.
@@ -418,30 +380,30 @@ int QMenuData::insertItem( const char *text,
 	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
 
   \sa removeItem(), changeItem(), setAccel(), connectItem(), QAccel,
-  qkeycode.h
+  qnamespace.h
 */
 
 int QMenuData::insertItem( const QPixmap &pixmap,
-			   const QObject *receiver, const char *member,
+			   const QObject *receiver, const char* member,
 			   int accel, int id, int index )
 {
-    int actualID = insertAny( 0, &pixmap, 0, id, index );
+    int actualID = insertAny( 0, &pixmap, 0, 0, id, index );
     connectItem( actualID, receiver, member );
     if ( accel )
 	setAccel( accel, actualID );
     return actualID;
 }
 
+
 /*!
-  Inserts a menu item with a pixmap, a text, an accelerator key, an id and
-  an optional index and connects it to an object/slot.
+  Inserts a menu item with an icon, a pixmap, an accelerator key, an id
+  and an optional index and connects it to an object/slot. The icon
+  will be displayed to the left of the pixmap in the item.
 
   The menu item is assigned the identifier \a id or an automatically
-  generated identifier.	 It works as follows: If \a id \>= 0, this
-  identifier is assigned.  If \a id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \a id is
-  any other negative integer, for instance -2, a unique identifier
-  (negative integer \<= -2) is generated.
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
 
   The \a index specifies the position in the menu.  The menu item is
   appended at the end of the list if \a index is negative.
@@ -455,29 +417,29 @@ int QMenuData::insertItem( const QPixmap &pixmap,
 	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
 
   \sa removeItem(), changeItem(), setAccel(), connectItem(), QAccel,
-  qkeycode.h
+  qnamespace.h
 */
 
-int QMenuData::insertItem( const QPixmap &pixmap, const char *text,
-			   const QObject *receiver, const char *member,
+int QMenuData::insertItem( const QIconSet& icon,
+			   const QPixmap &pixmap,
+			   const QObject *receiver, const char* member,
 			   int accel, int id, int index )
 {
-    int actualID = insertAny( text, &pixmap, 0, id, index );
+    int actualID = insertAny( 0, &pixmap, 0, &icon, id, index );
     connectItem( actualID, receiver, member );
     if ( accel )
 	setAccel( accel, actualID );
     return actualID;
 }
 
+
 /*!
   Inserts a menu item with a text.  Returns the menu item identifier.
 
   The menu item is assigned the identifier \a id or an automatically
-  generated identifier.	 It works as follows: If \a id \>= 0, this
-  identifier is assigned.  If \a id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \a id is
-  any other negative integer, for instance -2, a unique identifier
-  (negative integer \<= -2) is generated.
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
 
   The \a index specifies the position in the menu.  The menu item is
   appended at the end of the list if \a index is negative.
@@ -489,26 +451,49 @@ int QMenuData::insertItem( const QPixmap &pixmap, const char *text,
   \sa removeItem(), changeItem(), setAccel(), connectItem()
 */
 
-int QMenuData::insertItem( const char *text, int id, int index )
+int QMenuData::insertItem( const QString &text, int id, int index )
 {
-    return insertAny( text, 0, 0, id, index );
+    return insertAny( &text, 0, 0, 0, id, index );
+}
+
+/*!
+  Inserts a menu item with an icon and a text.  The icon will be
+  displayed to the left of the text in the item. Returns the menu
+  item identifier.
+
+  The menu item is assigned the identifier \a id or an automatically
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
+
+  The \a index specifies the position in the menu.  The menu item is
+  appended at the end of the list if \a index is negative.
+
+  \warning Be careful when passing a literal 0 to insertItem(), as
+	some C++ compilers choose the wrong overloaded function.
+	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
+
+  \sa removeItem(), changeItem(), setAccel(), connectItem()
+*/
+
+int QMenuData::insertItem( const QIconSet& icon,
+			   const QString &text, int id, int index )
+{
+    return insertAny( &text, 0, 0, &icon, id, index );
 }
 
 /*!
   Inserts a menu item with a text and a sub menu.
   Returns the menu item identifier.
 
-  The \a popup must be deleted by the programmer.  It is not deleted when
-  this menu item is removed or when the menu is deleted.  Note that all
-  popups are automatically deleted when the application terminates. (The
-  QApplication destructor destroys all remaining widgets.)
+  The \a popup must be deleted by the programmer or by its parent
+  widget.  It is not deleted when this menu item is removed or when
+  the menu is deleted.
 
   The menu item is assigned the identifier \a id or an automatically
-  generated identifier.	 It works as follows: If \a id \>= 0, this
-  identifier is assigned.  If \a id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \a id is
-  any other negative integer, for instance -2, a unique identifier
-  (negative integer \<= -2) is generated.
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
 
   The \a index specifies the position in the menu.  The menu item is
   appended at the end of the list if \a index is negative.
@@ -520,21 +505,50 @@ int QMenuData::insertItem( const char *text, int id, int index )
   \sa removeItem(), changeItem(), setAccel(), connectItem()
 */
 
-int QMenuData::insertItem( const char *text, QPopupMenu *popup,
+int QMenuData::insertItem( const QString &text, QPopupMenu *popup,
 			   int id, int index )
 {
-    return insertAny( text, 0, popup, id, index );
+    return insertAny( &text, 0, popup, 0, id, index );
+}
+
+/*!
+  Inserts a menu item with an icon, a text and a sub menu. The icon
+  will be displayed to the left of the text in the item. Returns the
+  menu item identifier.
+
+  The \a popup must be deleted by the programmer or by its parent
+  widget.  It is not deleted when this menu item is removed or when
+  the menu is deleted.
+
+  The menu item is assigned the identifier \a id or an automatically
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
+
+  The \a index specifies the position in the menu.  The menu item is
+  appended at the end of the list if \a index is negative.
+
+  \warning Be careful when passing a literal 0 to insertItem(), as
+	some C++ compilers choose the wrong overloaded function.
+	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
+
+  \sa removeItem(), changeItem(), setAccel(), connectItem()
+*/
+
+int QMenuData::insertItem( const QIconSet& icon,
+			   const QString &text, QPopupMenu *popup,
+			   int id, int index )
+{
+    return insertAny( &text, 0, popup, &icon, id, index );
 }
 
 /*!
   Inserts a menu item with a pixmap.  Returns the menu item identifier.
 
   The menu item is assigned the identifier \a id or an automatically
-  generated identifier.	 It works as follows: If \a id \>= 0, this
-  identifier is assigned.  If \a id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \a id is
-  any other negative integer, for instance -2, a unique identifier
-  (negative integer \<= -2) is generated.
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
 
   The \a index specifies the position in the menu.  The menu item is
   appended at the end of the list if \a index is negative.
@@ -548,24 +562,49 @@ int QMenuData::insertItem( const char *text, QPopupMenu *popup,
 
 int QMenuData::insertItem( const QPixmap &pixmap, int id, int index )
 {
-    return insertAny( 0, &pixmap, 0, id, index );
+    return insertAny( 0, &pixmap, 0, 0, id, index );
 }
 
 /*!
-  Inserts a menu item with a pixmap and a sub menu.
-  Returns the menu item identifier.
-
-  The \a popup must be deleted by the programmer.  It is not deleted when
-  this menu item is removed or when the menu is deleted.  Note that all
-  popups are automatically deleted when the application terminates. (The
-  QApplication destructor destroys all remaining widgets.)
+  Inserts a menu item with an icon and a pixmap.  The icon will be
+  displayed to the left of the pixmap in the item. Returns the menu
+  item identifier.
 
   The menu item is assigned the identifier \a id or an automatically
-  generated identifier.	 It works as follows: If \a id \>= 0, this
-  identifier is assigned.  If \a id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \a id is
-  any other negative integer, for instance -2, a unique identifier
-  (negative integer \<= -2) is generated.
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
+
+  The \a index specifies the position in the menu.  The menu item is
+  appended at the end of the list if \a index is negative.
+
+  \warning Be careful when passing a literal 0 to insertItem(), as
+	some C++ compilers choose the wrong overloaded function.
+	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
+
+  \sa removeItem(), changeItem(), setAccel(), connectItem()
+*/
+
+int QMenuData::insertItem( const QIconSet& icon,
+			   const QPixmap &pixmap, int id, int index )
+{
+    return insertAny( 0, &pixmap, 0, &icon, id, index );
+}
+
+
+/*!
+  Inserts a menu item with a pixmap and a sub menu. The icon
+  will be displayed to the left of the pixmap in the item.  Returns
+  the menu item identifier.
+
+  The \a popup must be deleted by the programmer or by its parent
+  widget.  It is not deleted when this menu item is removed or when
+  the menu is deleted.
+
+  The menu item is assigned the identifier \a id or an automatically
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
 
   The \a index specifies the position in the menu.  The menu item is
   appended at the end of the list if \a index is negative.
@@ -580,19 +619,23 @@ int QMenuData::insertItem( const QPixmap &pixmap, int id, int index )
 int QMenuData::insertItem( const QPixmap &pixmap, QPopupMenu *popup,
 			   int id, int index )
 {
-    return insertAny( 0, &pixmap, popup, id, index );
+    return insertAny( 0, &pixmap, popup, 0, id, index );
 }
 
+
 /*!
-  Inserts a menu item with a pixmap and a text.  Returns the menu item
-  identifier.
+  Inserts a menu item with an icon, a pixmap and a sub menu. The icon
+  will be displayed to the left of the pixmap in the item.  Returns
+  the menu item identifier.
+
+  The \a popup must be deleted by the programmer or by its parent
+  widget.  It is not deleted when this menu item is removed or when
+  the menu is deleted.
 
   The menu item is assigned the identifier \a id or an automatically
-  generated identifier.	 It works as follows: If \a id \>= 0, this
-  identifier is assigned.  If \a id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \a id is
-  any other negative integer, for instance -2, a unique identifier
-  (negative integer \<= -2) is generated.
+  generated identifier if \a id is < 0. The generated identifiers
+  (negative integers) are guaranteed to be unique within the entire
+  application.
 
   The \a index specifies the position in the menu.  The menu item is
   appended at the end of the list if \a index is negative.
@@ -604,43 +647,14 @@ int QMenuData::insertItem( const QPixmap &pixmap, QPopupMenu *popup,
   \sa removeItem(), changeItem(), setAccel(), connectItem()
 */
 
-int QMenuData::insertItem( const QPixmap &pixmap, const char *text,
+int QMenuData::insertItem( const QIconSet& icon,
+			   const QPixmap &pixmap, QPopupMenu *popup,
 			   int id, int index )
 {
-    return insertAny( text, &pixmap, 0, id, index );
+    return insertAny( 0, &pixmap, popup, &icon, id, index );
 }
 
-/*!
-  Inserts a menu item with a pixmap, a text and a sub menu.
-  Returns the menu item identifier.
 
-  The \a popup must be deleted by the programmer.  It is not deleted when
-  this menu item is removed or when the menu is deleted.  Note that all
-  popups are automatically deleted when the application terminates. (The
-  QApplication destructor destroys all remaining widgets.)
-
-  The menu item is assigned the identifier \a id or an automatically
-  generated identifier.	 It works as follows: If \a id \>= 0, this
-  identifier is assigned.  If \a id == -1 (default), the identifier is
-  set equal to the menu item's real index (see below).	If \a id is
-  any other negative integer, for instance -2, a unique identifier
-  (negative integer \<= -2) is generated.
-
-  The \a index specifies the position in the menu.  The menu item is
-  appended at the end of the list if \a index is negative.
-
-  \warning Be careful when passing a literal 0 to insertItem(), as
-	some C++ compilers choose the wrong overloaded function.
-	Cast the 0 to what you mean, eg. <tt>(QObject*)0</tt>.
-
-  \sa removeItem(), changeItem(), setAccel(), connectItem()
-*/
-
-int QMenuData::insertItem( const QPixmap &pixmap, const char *text,
-			   QPopupMenu *popup, int id, int index )
-{
-    return insertAny( text, &pixmap, popup, id, index );
-}
 
 /*!
   Inserts a separator at position \a index.
@@ -655,7 +669,7 @@ int QMenuData::insertItem( const QPixmap &pixmap, const char *text,
 
 void QMenuData::insertSeparator( int index )
 {
-    insertAny( 0, 0, 0, 0, index );
+    insertAny( 0, 0, 0, 0, 0, index );
 }
 
 /*!
@@ -673,7 +687,7 @@ void QMenuData::removeItemAt( int index )
 {
     if ( index < 0 || index >= (int)mitems->count() ) {
 #if defined(CHECK_RANGE)
-	warning( "QMenuData::removeItem: Index %d out of range", index );
+	qWarning( "QMenuData::removeItem: Index %d out of range", index );
 #endif
 	return;
     }
@@ -712,7 +726,7 @@ void QMenuData::clear()
 /*!
   Returns the accelerator key that has been defined for the menu item \a id,
   or 0 if it has no accelerator key.
-  \sa setAccel(), QAccel, qkeycode.h
+  \sa setAccel(), QAccel, qnamespace.h
 */
 
 int QMenuData::accel( int id ) const
@@ -725,8 +739,8 @@ int QMenuData::accel( int id ) const
   Defines an accelerator key for the menu item \a id.
 
   An accelerator key consists of a key code and a combination of the modifiers
-  \c SHIFT, \c CTRL and \c ALT (OR'ed or added).
-  The header file qkeycode.h contains a list of key codes.
+  \c SHIFT, \c CTRL, \c ALT, or \c UNICODE_ACCEL (OR'ed or added).
+  The header file qnamespace.h contains a list of key codes.
 
   Defining an accelerator key generates a text which is added to the
   menu item, for instance, \c CTRL + \c Key_O generates "Ctrl+O".  The
@@ -740,15 +754,21 @@ int QMenuData::accel( int id ) const
     QMenuBar   *mainMenu = new QMenuBar;
     QPopupMenu *fileMenu = new QPopupMenu;	// file sub menu
     fileMenu->insertItem( "Open Document", 67 );// add "Open" item
-    fileMenu->setAccel( CTRL + Key_O, 67 );
+    fileMenu->setAccel( CTRL + Key_O, 67 );     // Control and O to open
     fileMenu->insertItem( "Quit", 69 );		// add "Quit" item
     fileMenu->setAccel( CTRL + ALT + Key_Delete, 69 );
     mainMenu->insertItem( "File", fileMenu );	// add the file menu
   \endcode
 
+  If you will need to translate accelerators, use QAccel::stringToKey():
+
+  \code
+    fileMenu->setAccel( QAccel::stringToKey(tr("Ctrl+O")), 67 );
+  \endcode
+
   You can also specify the accelerator in the insertItem() function.
 
-  \sa accel(), insertItem(), QAccel, qkeycode.h
+  \sa setAccel(), accel(), insertItem(), QAccel, qnamespace.h
 */
 
 void QMenuData::setAccel( int key, int id )
@@ -761,23 +781,35 @@ void QMenuData::setAccel( int key, int id )
     }
 }
 
-
 /*!
-  Returns the text that has been set for menu item \a id, or 0 if no text
-  has been set.
-  \sa changeItem(), pixmap()
+  Returns the icon set that has been set for menu item \a id, or 0 if no icon
+  set has been set.
+  \sa changeItem(), text(), pixmap()
 */
 
-const char *QMenuData::text( int id ) const
+QIconSet* QMenuData::iconSet( int id ) const
 {
     QMenuItem *mi = findItem( id );
-    return mi ? mi->text() : 0;
+    return mi ? mi->iconSet() : 0;
+}
+
+/*!
+  Returns the text that has been set for menu item \a id, or a
+  \link QString::operator!() null string\endlink
+  if no text has been set.
+  \sa changeItem(), pixmap(), iconSet()
+*/
+
+QString QMenuData::text( int id ) const
+{
+    QMenuItem *mi = findItem( id );
+    return mi ? mi->text() : QString::null;
 }
 
 /*!
   Returns the pixmap that has been set for menu item \a id, or 0 if no pixmap
   has been set.
-  \sa changeItem(), text()
+  \sa changeItem(), text(), iconSet()
 */
 
 QPixmap *QMenuData::pixmap( int id ) const
@@ -786,12 +818,52 @@ QPixmap *QMenuData::pixmap( int id ) const
     return mi ? mi->pixmap() : 0;
 }
 
-/*!
-  Changes the text of the menu item \a id.
+/*!\obsolete
+
+  Changes the text of the menu item \a id. If the item has an icon,
+  the icon remains unchanged.
+
   \sa text()
 */
 
-void QMenuData::changeItem( const char *text, int id )
+void QMenuData::changeItem( const QString &text, int id )
+{
+    changeItem( id, text);
+}
+
+/*!\obsolete
+
+  Changes the pixmap of the menu item \a id. If the item has an icon,
+  the icon remains unchanged.
+
+  \sa pixmap()
+*/
+
+void QMenuData::changeItem( const QPixmap &pixmap, int id )
+{
+    changeItem( id, pixmap );
+}
+
+/*!\obsolete
+
+  Changes the icon and text of the menu item \a id.
+
+  \sa pixmap()
+*/
+
+void QMenuData::changeItem( const QIconSet &icon, const QString &text, int id )
+{
+    changeItem( id, icon, text );
+}
+
+
+/*!
+  Changes the text of the menu item \a id. If the item has an icon,
+  the icon remains unchanged.
+  \sa text()
+*/
+
+void QMenuData::changeItem( int id, const QString &text )
 {
     QMenuData *parent;
     QMenuItem *mi = findItem( id, &parent );
@@ -808,11 +880,12 @@ void QMenuData::changeItem( const char *text, int id )
 }
 
 /*!
-  Changes the pixmap of the menu item \a id.
+  Changes the pixmap of the menu item \a id. If the item has an icon,
+  the icon remains unchanged.
   \sa pixmap()
 */
 
-void QMenuData::changeItem( const QPixmap &pixmap, int id )
+void QMenuData::changeItem( int id, const QPixmap &pixmap )
 {
     QMenuData *parent;
     QMenuItem *mi = findItem( id, &parent );
@@ -823,7 +896,7 @@ void QMenuData::changeItem( const QPixmap &pixmap, int id )
 	    i->height() == pixmap.height() &&
 	    !mi->text();
 	if ( !mi->text_data.isNull() )		// delete text
-	    mi->text_data.resize( 0 );
+	    mi->text_data = QString::null;
 	mi->pixmap_data = new QPixmap( pixmap );
 	delete i; // old mi->pixmap_data, could be &pixmap
 	if ( fast_refresh )
@@ -833,27 +906,44 @@ void QMenuData::changeItem( const QPixmap &pixmap, int id )
     }
 }
 
-
 /*!
-  Changes the pixmap and text of the menu item \a id.
+  Changes the icon and text of the menu item \a id.
   \sa pixmap()
 */
 
-void QMenuData::changeItem( const QPixmap &pixmap, const char *text, int id )
+void QMenuData::changeItem( int id, const QIconSet &icon, const QString &text )
+{
+    changeItem(id, text);
+    changeItemIconSet(id,  icon);
+}
+
+/*!
+  Changes the icon and pixmap of the menu item \a id.
+  \sa pixmap()
+*/
+
+void QMenuData::changeItem( int id, const QIconSet &icon, const QPixmap &pixmap )
+{
+    changeItem(id, pixmap);
+    changeItemIconSet(id,  icon);
+}
+
+
+
+/*!
+  Changes the icon of the menu item \a id.
+  \sa pixmap()
+*/
+
+void QMenuData::changeItemIconSet( int id, const QIconSet &icon )
 {
     QMenuData *parent;
     QMenuItem *mi = findItem( id, &parent );
     if ( mi ) {					// item found
-	QPixmap *i = mi->pixmap_data;
-	bool fast_refresh = i != 0 &&
-	    i->width() == pixmap.width() &&
-	    i->height() == pixmap.height();
-	if ( mi->text_data != text ) {
-	    fast_refresh = FALSE;
-	    mi->text_data = text;
-	}
-	mi->pixmap_data = new QPixmap( pixmap );
-	delete i; // old mi->pixmap_data, could be &pixmap
+	register QIconSet *i = mi->iconset_data;
+	bool fast_refresh = i != 0;
+	mi->iconset_data = new QIconSet( icon );
+	delete i; // old mi->iconset_data, could be &icon
 	if ( fast_refresh )
 	    parent->updateItem( id );
 	else
@@ -1001,7 +1091,7 @@ int QMenuData::indexOf( int id ) const
 
 int QMenuData::idAt( int index ) const
 {
-    return index < (int)mitems->count() ?
+    return index < (int)mitems->count() && index >= 0 ?
 	   mitems->at(index)->id() : -1;
 }
 
@@ -1027,7 +1117,7 @@ void QMenuData::setId( int index, int id )
 */
 
 bool QMenuData::connectItem( int id, const QObject *receiver,
-			     const char *member )
+			     const char* member )
 {
     QMenuItem *mi = findItem( id );
     if ( !mi )					// no such identifier
@@ -1046,10 +1136,42 @@ bool QMenuData::connectItem( int id, const QObject *receiver,
 */
 
 bool QMenuData::disconnectItem( int id, const QObject *receiver,
-				const char *member )
+				const char* member )
 {
     QMenuItem *mi = findItem( id );
     if ( !mi || !mi->signal_data )		// no identifier or no signal
 	return FALSE;
     return mi->signal_data->disconnect( receiver, member );
 }
+
+/*!
+  Sets a Whats This help for a certain menu item.
+
+  \arg \e id is the menu item id.
+  \arg \e text is the Whats This help text in QML format
+
+  \sa whatsThis()
+ */
+void QMenuData::setWhatsThis( int id, const QString& text )
+{
+
+    QMenuData *parent;
+    QMenuItem *mi = findItem( id, &parent );
+    if ( mi ) {
+	mi->setWhatsThis( text );
+	parent->menuContentsChanged();
+    }
+}
+
+/*!
+  Returns the Whats This help text for the specified item \e id or
+  QString::null if no text has been defined yet.
+
+  \sa setWhatsThis()
+ */
+QString QMenuData::whatsThis( int id ) const
+{
+
+    QMenuItem *mi = findItem( id );
+    return mi? mi->whatsThis() : QString::null;
+}
