@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qprogressbar.cpp,v 2.21.2.1 1998/08/12 16:55:15 agulbra Exp $
+** $Id: qprogressbar.cpp,v 2.47.2.1 1999/09/22 19:26:03 reggie Exp $
 **
 ** Implementation of QProgressBar class
 **
@@ -7,17 +7,19 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -25,6 +27,7 @@
 #include "qpainter.h"
 #include "qdrawutil.h"
 #include "qapplication.h"
+#include <limits.h>
 
 /*!
   \class QProgressBar qprogressbar.h
@@ -37,8 +40,8 @@
   QProgressBar only implements the basic progress display, while
   QProgressDialog provides a fuller encapsulation.
 
-  <img src=qprogbar-m.gif> <img src=qprogbar-w.gif>
-  
+  <img src=qprogbar-m.png> <img src=qprogbar-w.png>
+
   \sa QProgressDialog
   <a href="guibooks.html#fowler">GUI Design Handbook: Progress Indicator</a>
 */
@@ -57,14 +60,14 @@
 
 QProgressBar::QProgressBar( QWidget *parent, const char *name, WFlags f )
     : QFrame( parent, name, f ),
-	total_steps( 100 ),
-	progress_val( -1 ),
-	percentage( -1 )
+      total_steps( 100 ),
+      progress_val( -1 ),
+      percentage( -1 ),
+      center_indicator( TRUE ),
+      auto_indicator( TRUE ),
+      d( 0 )
 {
-    if ( style() == MotifStyle ) {
-	setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
-	setLineWidth( 2 );
-    }
+    initFrame();
 }
 
 
@@ -86,14 +89,14 @@ QProgressBar::QProgressBar( QWidget *parent, const char *name, WFlags f )
 QProgressBar::QProgressBar( int totalSteps,
 			    QWidget *parent, const char *name, WFlags f )
     : QFrame( parent, name, f ),
-	total_steps( totalSteps ),
-	progress_val( -1 ),
-	percentage( -1 )
+      total_steps( totalSteps ),
+      progress_val( -1 ),
+      percentage( -1 ),
+      center_indicator( TRUE ),
+      auto_indicator( TRUE ),
+      d( 0 )
 {
-    if ( style() == MotifStyle ) {
-	setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
-	setLineWidth( 2 );
-    }
+    initFrame();
 }
 
 
@@ -108,6 +111,8 @@ void QProgressBar::reset()
     percentage = -1;
     setIndicator(progress_str, progress_val, total_steps);
     update();
+    if ( autoMask() )
+	updateMask();
 }
 
 
@@ -124,10 +129,14 @@ void QProgressBar::reset()
 
 void QProgressBar::setTotalSteps( int totalSteps )
 {
+    bool clear = totalSteps != total_steps;
     total_steps = totalSteps;
     if ( isVisible() ) {
-	if ( setIndicator(progress_str, progress_val, total_steps) )
-	    repaint( FALSE );
+	if ( setIndicator(progress_str, progress_val, total_steps) ) {
+	    repaint( clear );
+	    if ( autoMask() )
+		updateMask();
+	}
     }
 }
 
@@ -151,8 +160,11 @@ void QProgressBar::setProgress( int progress )
 	return;
     progress_val = progress;
     if ( isVisible() ) {
-	if ( setIndicator(progress_str, progress_val, total_steps) )
+	if ( setIndicator(progress_str, progress_val, total_steps) ) {
 	    repaint( FALSE );
+	    if ( autoMask() )
+		updateMask();
+	}
     }
 }
 
@@ -164,14 +176,101 @@ void QProgressBar::setProgress( int progress )
 QSize QProgressBar::sizeHint() const
 {
     QFontMetrics fm = fontMetrics();
-    return QSize(-1, fm.height()+8);
+    return QSize( fm.height()*4, fm.height()+8);
 }
 
 
+
+/*!
+  \reimp
+*/
+QSize QProgressBar::minimumSizeHint() const
+{
+    //so we can implement it later
+    return QWidget::minimumSizeHint();
+}
+
+
+/*!
+  Specifies that this widget can use more, but is able to survive on
+  less, horizontal space; and is fixed vertically.
+*/
+
+QSizePolicy QProgressBar::sizePolicy() const
+{
+    return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
+}
+
+/*!
+  \fn bool QProgressBar::centerIndicator() const
+
+  Returns where the indicator string should be displayed if
+  indicatorFollowsStyle() is TRUE.
+
+  \sa setCenterIndicator(), indicatorFollowsStyle(),
+      setIndicatorFollowsStyle(), setIndicator()
+*/
+
+/*!
+  If set to TRUE (the default) the progress bar always shows the indicator
+  text at the center of the progress bar, regardless of the GUI style
+  currently set.  If set to FALSE the progress bar always shows the
+  indicator text outside the progress bar, regardless of the GUI style
+  currently set.
+
+  Calling this function always sets indicatorFollowsStyle() to FALSE.
+
+  \sa centerIndicator(), indicatorFollowsStyle(), setIndicatorFollowsStyle(),
+      setIndicator()
+ */
+void QProgressBar::setCenterIndicator( bool on )
+{
+    if ( !auto_indicator && on == center_indicator )
+	return;
+    auto_indicator   = FALSE;
+    center_indicator = on;
+    repaint( FALSE );
+    if ( autoMask() )
+	updateMask();
+}
+
+/*!
+  \fn bool QProgressBar::indicatorFollowsStyle() const
+
+  Returns whether the display of the indicator string should follow the
+  GUI style or not.
+
+  \sa setIndicatorFollowsStyle(), setCenterIndicator(), centerIndicator()
+      setIndicator()
+*/
+
+/*!
+  When set to TRUE (the default) the positioning of the indicator string
+  follows the GUI style. When set to FALSE the indicator posion is decided
+  by the value of indicatorFollowsStyle().
+
+  \sa indicatorFollowsStyle(), centerIndicator(), setCenterIndicator(),
+      setIndicator()
+ */
+void QProgressBar::setIndicatorFollowsStyle( bool on )
+{
+    if ( on == auto_indicator )
+	return;
+    auto_indicator = on;
+    repaint( FALSE );
+    if ( autoMask() )
+	updateMask();
+}
+
 void QProgressBar::show()
 {
     setIndicator( progress_str, progress_val, total_steps );
-    // ### 2.0: Move this next stuff to styleChange()
+    QFrame::show();
+}
+
+
+void QProgressBar::initFrame()
+{
     if ( style() == MotifStyle ) {
 	setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
 	setLineWidth( 2 );
@@ -180,7 +279,14 @@ void QProgressBar::show()
 	setFrameStyle(QFrame::NoFrame);
 	setLineWidth( 1 );
     }
-    QFrame::show();
+}
+
+/*! \reimp
+ */
+void QProgressBar::styleChange( QStyle& old )
+{
+    initFrame();
+    QFrame::styleChange( old );
 }
 
 
@@ -199,15 +305,21 @@ void QProgressBar::show()
   progress bar.
 */
 
-bool QProgressBar::setIndicator( QString& indicator, int progress,
+bool QProgressBar::setIndicator( QString & indicator, int progress,
 				 int totalSteps )
 {
     if ( !totalSteps )
 	return FALSE;
     if ( progress < 0 ) {
-	indicator = "";
+	indicator = QString::fromLatin1("");
 	return TRUE;
     } else {
+	// Get the values down to something usable.
+	if ( totalSteps > INT_MAX/1000 ) {
+	    progress /= 1000;
+	    totalSteps /= 1000;
+	}
+
 	int np = progress * 100 / totalSteps;
 	if ( np != percentage ) {
 	    percentage = np;
@@ -223,7 +335,7 @@ bool QProgressBar::setIndicator( QString& indicator, int progress,
 /*!
   Handles paint events for the progress bar.
   In WindowsStyle, \link QColorGroup::text() colorGroup().text()\endlink
-  and QApplication::winStyleHighlightColor() are used.  In MotifStyle,
+  and colorGroup::highlight() are used.  In MotifStyle,
   \link QColorGroup::base() colorGroup().base()\endlink is also used.
 */
 
@@ -233,19 +345,27 @@ void QProgressBar::drawContents( QPainter *p )
     const int unit_height = 12;
     const QRect bar = contentsRect();
 
-    if ( style() == WindowsStyle ) {
+    if ( style() != MotifStyle && auto_indicator ||
+	 !auto_indicator && !center_indicator ) {
 	// Draw nu units out of a possible u of unit_width width, each
 	// a rectangle bordered by background color, all in a sunken panel
 	// with a percentage text display at the end.
 
 	QFontMetrics fm = p->fontMetrics();
-	int textw = fm.width("100%");
+	int textw = fm.width(QString::fromLatin1("100%"));
 	int u = (bar.width() - textw - 2/*panel*/) / unit_width;
 	int ox = ( bar.width() - (u*unit_width+textw) ) / 2;
 
 	if (total_steps) { // Sanity check
 	    // ### This part doesn't change as often as percentage does.
-	    int nu = ( u * progress_val + total_steps/2 ) / total_steps;
+	    int p_v = progress_val;
+	    int t_s = total_steps;
+	    if ( u > 0 && progress_val >= INT_MAX / u && t_s >= u ) {
+		// scale down to something usable.
+		p_v /= u;
+		t_s /= u;
+	    }
+	    int nu = ( u * p_v + t_s/2 ) / t_s;
 	    int x = bar.x() + ox;
 	    int uh = QMIN(bar.height()-4, unit_height);
 	    int vm = (bar.height()-4 - uh)/2 + 2;
@@ -253,7 +373,7 @@ void QProgressBar::drawContents( QPainter *p )
 	    for (int i=0; i<nu; i++) {
 		p->fillRect( x+2, bar.y()+vm,
 			     unit_width-2, bar.height()-vm-vm,
-			     QApplication::winStyleHighlightColor() );
+			     colorGroup().brush( QColorGroup::Highlight ) );
 		x += unit_width;
 	    }
 	}
@@ -265,23 +385,52 @@ void QProgressBar::drawContents( QPainter *p )
 	// ### This part changes every percentage change.
 	p->setPen( colorGroup().text() );
 	p->fillRect( r.x()+r.width(), bar.y(), textw, bar.height(),
-	    backgroundColor() );
+		     colorGroup().brush(QColorGroup::Background ) );
 	p->drawText( r.x()+r.width(), bar.y(), textw, bar.height(),
 	    AlignRight | AlignVCenter, progress_str );
     } else {
 	if (total_steps) { // Sanity check
 	    int pw = bar.width() * progress_val / total_steps;
 
-	    p->setPen( colorGroup().base() );
+	    p->setPen( colorGroup().highlightedText() );
 	    p->setClipRect( bar.x(), bar.y(), pw, bar.height() );
-	    p->fillRect( bar, QApplication::winStyleHighlightColor() );
+	    p->fillRect( bar, colorGroup().brush( QColorGroup::Highlight ) );
 	    p->drawText( bar, AlignCenter, progress_str );
 
-	    p->setPen( QApplication::winStyleHighlightColor() );
 	    p->setClipRect( bar.x()+pw, bar.y(), bar.width()-pw, bar.height() );
 	}
-	p->fillRect( bar, colorGroup().base() );
+	p->fillRect( bar, colorGroup().brush( QColorGroup::Base ) );
 	p->setPen( colorGroup().text() );
 	p->drawText( bar, AlignCenter, progress_str );
     }
 }
+
+
+/*!
+  Draws the progressbar contents mask using the painter \e p.
+  Used only in transparent mode.
+
+  \sa QWidget::setAutoMask();
+*/
+void QProgressBar::drawContentsMask( QPainter *p )
+{
+    const int unit_width  = 9;	    // includes 2 bg pixels
+    const QRect bar = contentsRect();
+
+    if ( style() != MotifStyle ) {
+	// ### This part doesn't actually change.
+	QFontMetrics fm = p->fontMetrics();
+	int textw = fm.width(QString::fromLatin1("100%"));
+	int u = (bar.width() - textw - 2/*panel*/) / unit_width;
+	int ox = ( bar.width() - (u*unit_width+textw) ) / 2;
+
+	const QRect r( ox + bar.x(), bar.y(), u*unit_width + 2, bar.height() );
+	p->drawRect( r );
+
+	// ### This part changes every percentage change.
+	p->drawText( r.x()+r.width(), bar.y(), textw, bar.height(),
+	    AlignRight | AlignVCenter, progress_str );
+    } else {
+	p->drawRect( bar );
+    }
+}
