@@ -1,30 +1,34 @@
 /****************************************************************************
-** $Id: qscrollbar.cpp,v 2.39 1998/07/03 00:09:52 hanord Exp $
+** $Id: qscrollbar.cpp,v 2.78.2.1 1999/08/18 17:21:57 paul Exp $
 **
 ** Implementation of QScrollBar class
 **
 ** Created : 940427
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qscrollbar.h"
 #include "qpainter.h"
 #include "qbitmap.h"
-#include "qkeycode.h"
+#include "qapplication.h"
+#include "qtimer.h"
+#include <limits.h>
 
 /*!
   \class QScrollBar qscrollbar.h
@@ -60,6 +64,10 @@
   and maxValue() of the scrollbar.  (QScrollBar has a convenience
   constructor with which you can set most of that.)
 
+  Some GUI styles, for example the provided Windows and Motif styles,
+  also use the pageStep() value to calculate the size of the sliding
+  thumb (scroll indicator).
+
   In addition to the access functions from QRangeControl, QScrollBar
   has a comprehensive set of signals: <ul>
 
@@ -95,7 +103,7 @@
   default focusPolicy() of \c NoFocus. Use setFocusPolicy() to
   enable keyboard focus.
 
-  <img src=qscrbar-m.gif> <img src=qscrbar-w.gif>
+  <img src=qscrbar-m.png> <img src=qscrbar-w.png>
 
   \sa QSlider QSpinBox
   <a href="guibooks.html#fowler">GUI Design Handbook: Scroll Bar</a>
@@ -150,40 +158,14 @@
 */
 
 
-enum ScrollControl { ADD_LINE = 0x1 , SUB_LINE = 0x2 , ADD_PAGE = 0x4,
-		     SUB_PAGE = 0x8 , FIRST    = 0x10, LAST	= 0x20,
-		     SLIDER   = 0x40, NONE     = 0x80 };
-
-
-class QScrollBar_Private : public QScrollBar
-{
-public:
-    void	  sliderMinMax( int *, int * )		const;
-    void	  metrics( int *, int *, int * )	const;
-
-    ScrollControl pointOver( const QPoint &p )		const;
-
-    int		  rangeValueToSliderPos( int val )	const;
-    int		  sliderPosToRangeValue( int  val )	const;
-
-    void	  action( ScrollControl control );
-
-    void	  drawControls( uint controls, uint activeControl ) const;
-    void	  drawControls( uint controls, uint activeControl,
-				QPainter *p ) const;
-};
-
-
-#undef PRIV
-#define PRIV	((QScrollBar_Private *)this)
 
 static const int thresholdTime = 500;
-static const int repeatTime	= 10;
+static const int repeatTime	= 50;
 
 #define HORIZONTAL	(orientation() == Horizontal)
 #define VERTICAL	!HORIZONTAL
 #define MOTIF_BORDER	2
-#define SLIDER_MIN	9 // ### motif says 6 but that's too small
+#define SLIDER_MIN	9
 
 
 /*!
@@ -193,7 +175,7 @@ static const int repeatTime	= 10;
 */
 
 QScrollBar::QScrollBar( QWidget *parent, const char *name )
-    : QWidget( parent, name )
+    : QWidget( parent, name, WResizeNoErase )
 {
     orient = Vertical;
     init();
@@ -209,7 +191,7 @@ QScrollBar::QScrollBar( QWidget *parent, const char *name )
 
 QScrollBar::QScrollBar( Orientation orientation, QWidget *parent,
 			const char *name )
-    : QWidget( parent, name )
+    : QWidget( parent, name, WResizeNoErase )
 {
     orient = orientation;
     init();
@@ -221,7 +203,7 @@ QScrollBar::QScrollBar( Orientation orientation, QWidget *parent,
   \arg \e minValue is the minimum scroll bar value.
   \arg \e maxValue is the maximum scroll bar value.
   \arg \e lineStep is the line step value.
-  \arg \e pageStep is the page step value.
+  \arg \e pageStep is the page step value. It is also used to calculate the size of the sliding thumb (scroll indicator).
   \arg \e value is the initial value.
   \arg \e orientation must be QScrollBar::Vertical or QScrollBar::Horizontal.
 
@@ -231,7 +213,7 @@ QScrollBar::QScrollBar( Orientation orientation, QWidget *parent,
 QScrollBar::QScrollBar( int minValue, int maxValue, int lineStep, int pageStep,
 			int value,  Orientation orientation,
 			QWidget *parent, const char *name )
-    : QWidget( parent, name ),
+    : QWidget( parent, name, WResizeNoErase ),
       QRangeControl( minValue, maxValue, lineStep, pageStep, value )
 {
     orient = orientation;
@@ -240,11 +222,15 @@ QScrollBar::QScrollBar( int minValue, int maxValue, int lineStep, int pageStep,
 
 void QScrollBar::init()
 {
-    track	     = TRUE;
-    sliderPos	     = 0;
-    pressedControl   = NONE;
-    clickedAt	     = FALSE;
+    track = TRUE;
+    sliderPos = 0;
+    pressedControl = QStyle::NoScroll;
+    clickedAt = FALSE;
     setFocusPolicy( NoFocus );
+
+    repeater = 0;
+    d = 0;
+
     if ( style() == MotifStyle )
 	setBackgroundMode( PaletteMid );
     else
@@ -280,7 +266,7 @@ void QScrollBar::setOrientation( Orientation orientation )
   If tracking is enabled (default), the scroll bar emits the
   valueChanged() signal whenever the slider is being dragged.  If
   tracking is disabled, the scroll bar emits the valueChanged() signal
-  when the user relases the mouse button (unless the value happens to
+  when the user releases the mouse button (unless the value happens to
   be the same as before).
 
   \sa tracking()
@@ -298,12 +284,12 @@ void QScrollBar::setOrientation( Orientation orientation )
 
 /*!
   Returns TRUE if the user has clicked the mouse on the slider
-  and is currenly dragging it, or FALSE if not.
+  and is currently dragging it, or FALSE if not.
 */
 
 bool QScrollBar::draggingSlider() const
 {
-    return pressedControl == SLIDER;
+    return pressedControl == QStyle::Slider;
 }
 
 
@@ -329,13 +315,25 @@ void QScrollBar::setPalette( const QPalette &p )
 
 QSize QScrollBar::sizeHint() const
 {
-    QSize s( size() );
     if ( orient == Horizontal ) {
-	s.setHeight( 16 );
+	return QSize( 30, 16 );
     } else {
-	s.setWidth( 16 );
+	return QSize( 16, 30 );
     }
-    return s;
+}
+
+/*!
+  Specifies that this widget can use more, but is able to survive on
+  less, space in the orientation() direction; and is fixed in the other
+  direction.
+*/
+
+QSizePolicy QScrollBar::sizePolicy() const
+{
+    if ( orient == Horizontal )
+	return QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed );
+    else
+	return QSizePolicy(  QSizePolicy::Fixed, QSizePolicy::Minimum );
 }
 
 
@@ -349,7 +347,7 @@ void QScrollBar::valueChange()
     int tmp = sliderPos;
     positionSliderFromValue();
     if ( tmp != sliderPos )
-	PRIV->drawControls( ADD_PAGE | SLIDER | SUB_PAGE , pressedControl );
+	drawControls( QStyle::AddPage | QStyle::Slider | QStyle::SubPage , pressedControl );
     emit valueChanged(value());
 }
 
@@ -371,8 +369,9 @@ void QScrollBar::stepChange()
 void QScrollBar::rangeChange()
 {
     positionSliderFromValue();
-    PRIV->drawControls( ADD_LINE | ADD_PAGE | SLIDER | SUB_PAGE | SUB_LINE,
-			pressedControl );
+    drawControls( QStyle::AddLine | QStyle::AddPage | QStyle::Slider |
+		  QStyle::SubPage | QStyle::SubLine,
+		  pressedControl );
 }
 
 
@@ -380,17 +379,60 @@ void QScrollBar::rangeChange()
   Handles timer events for the scroll bar.
 */
 
-void QScrollBar::timerEvent( QTimerEvent * )
+void QScrollBar::doAutoRepeat()
 {
-    if ( !isTiming )
-	return;
-    if ( !thresholdReached ) {
-	thresholdReached = TRUE;	// control has been pressed for a time
-	killTimers();			// kill the threshold time timer
-	startTimer( repeatTime );	//   and start repeating
+    if ( clickedAt ){
+	if ( repeater )
+	    repeater->changeInterval( repeatTime );
+	action( (QStyle::ScrollControl) pressedControl );
+	QApplication::syncX();
+    } else {
+	stopAutoRepeat();
     }
-    if ( clickedAt )
-	PRIV->action( (ScrollControl) pressedControl );
+}
+
+
+/*! Starts the auto-repeat logic.  Some time after this function is
+called, the auto-repeat starts taking effect, and from then on repeats
+until stopAutoRepeat() is called.
+*/
+
+void QScrollBar::startAutoRepeat()
+{
+    if ( !repeater ) {
+	repeater = new QTimer( this, "auto-repeat timer" );
+	connect( repeater, SIGNAL(timeout()),
+		 this, SLOT(doAutoRepeat()) );
+    }
+    repeater->start( thresholdTime, FALSE );
+}
+
+
+/*! Stops the auto-repeat logic. */
+
+void QScrollBar::stopAutoRepeat()
+{
+    delete repeater;
+    repeater = 0;
+}
+
+
+/*!
+  Handles wheel events for the scroll bar.
+*/
+void QScrollBar::wheelEvent( QWheelEvent *e ){
+    static float offset = 0;
+    static QScrollBar* offset_owner = 0;
+    if (offset_owner != this){
+	offset_owner = this;
+	offset = 0;
+    }
+    e->accept();
+    offset += -e->delta()*QMAX(pageStep()/8,2*lineStep())/120;
+    if (QABS(offset)<1)
+	return;
+    setValue( value() + int(offset) );
+    offset -= int(offset);
 }
 
 
@@ -403,27 +445,25 @@ void QScrollBar::keyPressEvent( QKeyEvent *e )
     switch ( e->key() ) {
     case Key_Left:
 	if ( orient == Horizontal )
-	    setValue( value() - lineStep() );
+	    subtractLine();
 	break;
     case Key_Right:
 	if ( orient == Horizontal )
-	    setValue( value() + lineStep() );
+	    addLine();
 	break;
     case Key_Up:
 	if ( orient == Vertical )
-	    setValue( value() - lineStep() );
+	    subtractLine();
 	break;
     case Key_Down:
 	if ( orient == Vertical )
-	    setValue( value() + lineStep() );
+	    addLine();
 	break;
     case Key_PageUp:
-	if ( orient == Vertical )
-	    setValue( value() - pageStep() );
+	subtractPage();
 	break;
     case Key_PageDown:
-	if ( orient == Vertical )
-	    setValue( value() + pageStep() );
+	addPage();
 	break;
     case Key_Home:
 	setValue( minValue() );
@@ -445,6 +485,7 @@ void QScrollBar::keyPressEvent( QKeyEvent *e )
 void QScrollBar::resizeEvent( QResizeEvent * )
 {
     positionSliderFromValue();
+    repaint(rect());
 }
 
 
@@ -452,24 +493,13 @@ void QScrollBar::resizeEvent( QResizeEvent * )
   Handles paint events for the scroll bar.
 */
 
-void QScrollBar::paintEvent( QPaintEvent *event )
+void QScrollBar::paintEvent( QPaintEvent * )
 {
     QPainter p( this );
-    if ( event )
-	p.setClipRect( event->rect() );
-
-    qDrawShadePanel( &p, rect(), colorGroup(), TRUE );
-    if ( hasFocus() ) {
-	if ( style() != WindowsStyle ) {
-	    p.setPen( black );
-	    p.drawRect(  1, 1, width() - 2, height() - 2 );
-	}
-    }
-    PRIV->drawControls( ADD_LINE | SUB_LINE | ADD_PAGE | SUB_PAGE | SLIDER,
+    drawControls( QStyle::AddLine | QStyle::SubLine | QStyle::AddPage | QStyle::SubPage | QStyle::Slider,
 			pressedControl, &p );
 }
 
-
 static QCOORD sliderStartPos = 0;
 
 /*!
@@ -479,41 +509,40 @@ static QCOORD sliderStartPos = 0;
 void QScrollBar::mousePressEvent( QMouseEvent *e )
 {
     if ( !(e->button() == LeftButton ||
-	   (style() == MotifStyle && e->button() == MidButton) ) )
+	   (/*style() == MotifStyle &&*/ e->button() == MidButton) ) )
 	return;
 
     if ( maxValue() == minValue() ) // nothing to be done
 	return;
 
     clickedAt	   = TRUE;
-    pressedControl = PRIV->pointOver( e->pos() );
+    pressedControl = pointOver( e->pos() );
 
-    if ( (pressedControl == ADD_PAGE ||
-	  pressedControl == SUB_PAGE ||
-	  pressedControl == SLIDER ) &&
-	 style() == MotifStyle &&
+    if ( (pressedControl == QStyle::AddPage ||
+	  pressedControl == QStyle::SubPage ||
+	  pressedControl == QStyle::Slider ) &&
+	 /*style() == MotifStyle &&*/
 	 e->button() == MidButton ) {
-	int dummy1, dummy2, sliderLength;
-	PRIV->metrics( &dummy1, &dummy2, &sliderLength );
+	int sliderMin, dummy2, dummy3, sliderLength;
+	metrics( sliderMin, dummy2, sliderLength, dummy3 );
 	int newSliderPos = (HORIZONTAL ? e->pos().x() : e->pos().y())
 			   - sliderLength/2;
-	setValue( PRIV->sliderPosToRangeValue(newSliderPos) );
+	newSliderPos = QMAX( sliderMin, newSliderPos );
+	setValue( sliderPosToRangeValue(newSliderPos) );
 	sliderPos = newSliderPos;
-	pressedControl = SLIDER;
+	pressedControl = QStyle::Slider;
     }
 
-    if ( pressedControl == SLIDER ) {
+    if ( pressedControl == QStyle::Slider ) {
 	clickOffset = (QCOORD)( (HORIZONTAL ? e->pos().x() : e->pos().y())
 				- sliderPos );
 	slidePrevVal   = value();
 	sliderStartPos = sliderPos;
 	emit sliderPressed();
-    } else if ( pressedControl != NONE ) {
-	PRIV->drawControls( pressedControl, pressedControl );
-	PRIV->action( (ScrollControl) pressedControl );
-	thresholdReached = FALSE;	// wait before starting repeat
-	startTimer(thresholdTime);
-	isTiming = TRUE;
+    } else if ( pressedControl != QStyle::NoScroll ) {
+	drawControls( pressedControl, pressedControl );
+	action( (QStyle::ScrollControl) pressedControl );
+	startAutoRepeat();
     }
 }
 
@@ -524,30 +553,22 @@ void QScrollBar::mousePressEvent( QMouseEvent *e )
 
 void QScrollBar::mouseReleaseEvent( QMouseEvent *e )
 {
-    if ( !clickedAt || !(e->button() == LeftButton ||
-			 (style() == MotifStyle && e->button() == MidButton)) )
+    if ( !clickedAt || !( e->button() == LeftButton ||
+			  e->button() == MidButton ) )
 	return;
-    ScrollControl tmp = (ScrollControl) pressedControl;
+    QStyle::ScrollControl tmp = (QStyle::ScrollControl) pressedControl;
     clickedAt = FALSE;
-    if ( isTiming )
-	killTimers();
+    stopAutoRepeat();
     mouseMoveEvent( e );  // Might have moved since last mouse move event.
-    pressedControl = NONE;
+    pressedControl = QStyle::NoScroll;
 
-    switch( tmp ) {
-	case SLIDER: // Set value directly, we know we don't have to redraw.
-	    directSetValue( calculateValueFromSlider() );
-	    emit sliderReleased();
-	    if ( value() != prevValue() )
-		emit valueChanged( value() );
-	    break;
-	case ADD_LINE:
-	case SUB_LINE:
-	    PRIV->drawControls( tmp, pressedControl );
-	    break;
-	default:
-	    break;
+    if (tmp == QStyle::Slider) {
+	directSetValue( calculateValueFromSlider() );
+	emit sliderReleased();
+	if ( value() != prevValue() )
+	    emit valueChanged( value() );
     }
+    drawControls( tmp, pressedControl );
 }
 
 
@@ -561,30 +582,39 @@ void QScrollBar::mouseMoveEvent( QMouseEvent *e )
 	clickedAt = FALSE;
 	return;
     }
+
     if ( !clickedAt || !(e->state() & LeftButton ||
-			 ((e->state() & MidButton) && style() == MotifStyle)) )
+			 ((e->state() & MidButton) /*&&
+			  style() == MotifStyle*/)) )
 	return;
     int newSliderPos;
-    if ( pressedControl == SLIDER ) {
+    if ( pressedControl == QStyle::Slider ) {
 	int sliderMin, sliderMax;
-	PRIV->sliderMinMax( &sliderMin, &sliderMax );
+	sliderMinMax( sliderMin, sliderMax );
 	QRect r = rect();
-	if ( orientation() == Horizontal )
-	    r.setRect( r.x() - 20, r.y() - 40, r.width() + 40, r.height() + 80 );
+	int m = style().maximumSliderDragDistance();
+	if ( m >= 0 ) {
+	    if ( orientation() == Horizontal )
+		r.setRect( r.x() - m, r.y() - 2*m, r.width() + 2*m, r.height() + 4*m );
+	    else
+		r.setRect( r.x() - 2*m, r.y() - m, r.width() + 4*m, r.height() + 2*m );
+	    if ( style() == WindowsStyle && !r.contains( e->pos() ) )
+		newSliderPos = sliderStartPos;
+	    else
+		newSliderPos = (HORIZONTAL ? e->pos().x() :
+				e->pos().y()) -clickOffset;
+	}
 	else
-	    r.setRect( r.x() - 40, r.y() - 20, r.width() + 80, r.height() + 40 );
-	if ( style() == WindowsStyle && !r.contains( e->pos() ) )
-	    newSliderPos = sliderStartPos;
-        else
 	    newSliderPos = (HORIZONTAL ? e->pos().x() :
-			                 e->pos().y()) -clickOffset;
+			    e->pos().y()) -clickOffset;
+	
 	if ( newSliderPos < sliderMin )
 	    newSliderPos = sliderMin;
 	else if ( newSliderPos > sliderMax )
 	    newSliderPos = sliderMax;
 	if ( newSliderPos == sliderPos )
 	    return;
-	int newVal = PRIV->sliderPosToRangeValue(newSliderPos);
+	int newVal = sliderPosToRangeValue(newSliderPos);
 	if ( newVal != slidePrevVal )
 	    emit sliderMoved( newVal );
 	if ( track && newVal != value() ) {
@@ -593,7 +623,19 @@ void QScrollBar::mouseMoveEvent( QMouseEvent *e )
 	}
 	slidePrevVal = newVal;
 	sliderPos = (QCOORD)newSliderPos;
-	PRIV->drawControls( ADD_PAGE | SLIDER | SUB_PAGE, pressedControl );
+	drawControls( QStyle::AddPage | QStyle::Slider | QStyle::SubPage, pressedControl );
+    }
+    else if ( style() == WindowsStyle ) {
+	// stop scrolling when the mouse pointer leaves a control
+	// similar to push buttons
+	if ( (int)pressedControl != pointOver( e->pos() ) ) {
+	    drawControls( pressedControl, QStyle::NoScroll );
+	    stopAutoRepeat();
+	} else if ( !repeater ) {
+	    drawControls( pressedControl, pressedControl );
+	    action( (QStyle::ScrollControl) pressedControl );
+	    startAutoRepeat();
+	}
     }
 }
 
@@ -613,8 +655,8 @@ void QScrollBar::mouseMoveEvent( QMouseEvent *e )
 
 QRect QScrollBar::sliderRect() const
 {
-    int sliderMin, sliderMax, sliderLength;
-    PRIV->metrics( &sliderMin, &sliderMax, &sliderLength );
+    int sliderMin, sliderMax, sliderLength, buttonDim;
+    metrics( sliderMin, sliderMax, sliderLength, buttonDim );
     int b = style() == MotifStyle ? MOTIF_BORDER : 0;
 
     if ( HORIZONTAL )
@@ -627,254 +669,109 @@ QRect QScrollBar::sliderRect() const
 
 void QScrollBar::positionSliderFromValue()
 {
-    sliderPos = (QCOORD)PRIV->rangeValueToSliderPos( value() );
+    sliderPos = (QCOORD)rangeValueToSliderPos( value() );
 }
 
 int QScrollBar::calculateValueFromSlider() const
 {
-    return PRIV->sliderPosToRangeValue( sliderPos );
+    return sliderPosToRangeValue( sliderPos );
 }
 
 
-/*****************************************************************************
-  QScrollBar_Private member functions
- *****************************************************************************/
-
-void QScrollBar_Private::sliderMinMax( int *sliderMin, int *sliderMax) const
+void QScrollBar::sliderMinMax( int &sliderMin, int &sliderMax) const
 {
-    int dummy;
-    metrics( sliderMin, sliderMax, &dummy );
+    int dummy1, dummy2;
+    metrics( sliderMin, sliderMax, dummy1, dummy2 );
 }
 
 
-void QScrollBar_Private::metrics( int *sliderMin, int *sliderMax,
-				  int *sliderLength ) const
+void QScrollBar::metrics( int &sliderMin, int &sliderMax,
+			  int &sliderLength, int& buttonDim ) const
 {
-    int buttonDim, maxLength;
-    int b = style() == MotifStyle ? MOTIF_BORDER : 0;
-    int length = HORIZONTAL ? width()  : height();
-    int extent = HORIZONTAL ? height() : width();
 
-    if ( length > ( extent - b*2 - 1 )*2 + b*2 + SLIDER_MIN )
-	buttonDim = extent - b*2;
-    else
-	buttonDim = ( length - b*2 - SLIDER_MIN )/2 - 1;
-
-    *sliderMin = b + buttonDim;
-    maxLength  = length - b*2 - buttonDim*2;
-
-    if ( maxValue() == minValue() ) {
-	*sliderLength = maxLength;
-    } else {
-	*sliderLength = (pageStep()*maxLength)/
-			(maxValue()-minValue()+pageStep());
-	if ( *sliderLength < SLIDER_MIN )
-	    *sliderLength = SLIDER_MIN;
-	if ( *sliderLength > maxLength )
-	    *sliderLength = maxLength;
-    }
-    *sliderMax = *sliderMin + maxLength - *sliderLength;
+    style().scrollBarMetrics( this, sliderMin, sliderMax, sliderLength, buttonDim);
 }
 
 
-ScrollControl QScrollBar_Private::pointOver(const QPoint &p) const
+QStyle::ScrollControl QScrollBar::pointOver(const QPoint &p) const
 {
-    if ( !rect().contains( p ) )
-	return NONE;
-    int sliderMin, sliderMax, sliderLength, pos;
-    metrics( &sliderMin, &sliderMax, &sliderLength );
-    pos = HORIZONTAL ? p.x() : p.y();
-    if ( pos < sliderMin )
-	return SUB_LINE;
-    if ( pos < sliderStart() )
-	return SUB_PAGE;
-    if ( pos < sliderStart() + sliderLength )
-	return SLIDER;
-    if ( pos < sliderMax + sliderLength )
-	return ADD_PAGE;
-    return ADD_LINE;
+    return style().scrollBarPointOver(this, sliderStart(), p);
 }
 
 
-int QScrollBar_Private::rangeValueToSliderPos( int v ) const
-{
-    int smin, smax;
-    sliderMinMax( &smin, &smax );
-    if ( maxValue() == minValue() )
-	return smin;
-    int sliderMin=smin, sliderMax=smax;
-    return ((sliderMax-sliderMin)*2*(v-minValue())+1)/
-	   ((maxValue()-minValue())*2) + sliderMin;
-}
-
-int QScrollBar_Private::sliderPosToRangeValue( int pos ) const
+int QScrollBar::rangeValueToSliderPos( int v ) const
 {
     int sliderMin, sliderMax;
-    sliderMinMax( &sliderMin, &sliderMax );
-    if ( pos <= sliderMin || sliderMax == sliderMin )
-	return minValue();
-    if ( pos >= sliderMax )
-	return maxValue();
-    return (maxValue() - minValue() + 1)*(pos - sliderMin)/
-	   (sliderMax - sliderMin) + minValue();
+    sliderMinMax( sliderMin, sliderMax );
+    return positionFromValue( v, sliderMax-sliderMin ) + sliderMin;
+}
+
+int QScrollBar::sliderPosToRangeValue( int pos ) const
+{
+    int sliderMin, sliderMax;
+    sliderMinMax( sliderMin, sliderMax );
+    return  valueFromPosition( pos - sliderMin, sliderMax - sliderMin );
 }
 
 
-void QScrollBar_Private::action( ScrollControl control )
+void QScrollBar::action( QStyle::ScrollControl control )
 {
     switch( control ) {
-	case ADD_LINE:
+	case QStyle::AddLine:
 	    emit nextLine();
 	    addLine();
 	    break;
-	case SUB_LINE:
+	case QStyle::SubLine:
 	    emit prevLine();
 	    subtractLine();
 	    break;
-	case ADD_PAGE:
+	case QStyle::AddPage:
 	    emit nextPage();
 	    addPage();
 	    break;
-	case SUB_PAGE:
+	case QStyle::SubPage:
 	    emit prevPage();
 	    subtractPage();
 	    break;
-#if defined(CHECK_RANGE)
 	default:
-	    warning( "QScrollBar_Private::action: (%s) internal error",
+#if defined(CHECK_RANGE)
+	    qWarning( "QScrollBar::action: (%s) internal error",
 		     name( "unnamed" ) );
+#else
+	    ;
 #endif
     }
 }
 
 
-void QScrollBar_Private::drawControls( uint controls,
-				       uint activeControl ) const
+void QScrollBar::drawControls( uint controls, uint activeControl ) const
 {
     QPainter p ( this );
     drawControls( controls, activeControl, &p );
 }
 
 
-void QScrollBar_Private::drawControls( uint controls, uint activeControl,
-				       QPainter *p ) const
+void QScrollBar::drawControls( uint controls, uint activeControl,
+			       QPainter *p ) const
 {
-#define ADD_LINE_ACTIVE ( activeControl == ADD_LINE )
-#define SUB_LINE_ACTIVE ( activeControl == SUB_LINE )
-    QColorGroup g  = colorGroup();
 
-    int sliderMin, sliderMax, sliderLength;
-    metrics( &sliderMin, &sliderMax, &sliderLength );
+    style().drawScrollBarControls(p, this, sliderStart(), controls, activeControl);
+    return;
 
-    int b = style() == MotifStyle ? MOTIF_BORDER : 0;
-    int dimB = sliderMin - b;
-    QRect addB;
-    QRect subB;
-    QRect addPageR;
-    QRect subPageR;
-    QRect sliderR;
-    int addX, addY, subX, subY;
-    int length = HORIZONTAL ? width()  : height();
-    int extent = HORIZONTAL ? height() : width();
+}
 
-    if ( HORIZONTAL ) {
-	subY = addY = ( extent - dimB ) / 2;
-	subX = b;
-	addX = length - dimB - b;
-    } else {
-	subX = addX = ( extent - dimB ) / 2;
-	subY = b;
-	addY = length - dimB - b;
-    }
+/*!\reimp
+ */
+void QScrollBar::styleChange( QStyle& old )
+{
+    if ( style() == MotifStyle )
+	setBackgroundMode( PaletteMid );
+    else
+	setBackgroundMode( PaletteBackground );
 
-    subB.setRect( subX,subY,dimB,dimB );
-    addB.setRect( addX,addY,dimB,dimB );
-
-    int sliderEnd = sliderStart() + sliderLength;
-    int sliderW = extent - b*2;
-    if ( HORIZONTAL ) {
-	subPageR.setRect( subB.right() + 1, b,
-			  sliderStart() - subB.right() - 1 , sliderW );
-	addPageR.setRect( sliderEnd, b, addX - sliderEnd, sliderW );
-	sliderR .setRect( sliderStart(), b, sliderLength, sliderW );
-    } else {
-	subPageR.setRect( b, subB.bottom() + 1, sliderW,
-			  sliderStart() - subB.bottom() - 1 );
-	addPageR.setRect( b, sliderEnd, sliderW, addY - sliderEnd );
-	sliderR .setRect( b, sliderStart(), sliderW, sliderLength );
-    }
-
-    if ( style() == WindowsStyle ) {
-	bool maxedOut = (maxValue() == minValue());
-	if ( controls & ADD_LINE ) {
-	    qDrawWinPanel( p, addB.x(), addB.y(),
-			   addB.width(), addB.height(), g,
-			   ADD_LINE_ACTIVE );
-	    qDrawArrow( p, VERTICAL ? DownArrow : RightArrow,
-			WindowsStyle, ADD_LINE_ACTIVE, addB.x()+2, addB.y()+2,
-			addB.width()-4, addB.height()-4, g, !maxedOut );
-	}
-	if ( controls & SUB_LINE ) {
-	    qDrawWinPanel( p, subB.x(), subB.y(),
-			   subB.width(), subB.height(), g,
-			   SUB_LINE_ACTIVE );
-	    qDrawArrow( p, VERTICAL ? UpArrow : LeftArrow,
-			WindowsStyle, SUB_LINE_ACTIVE, subB.x()+2, subB.y()+2,
-			subB.width()-4, subB.height()-4, g, !maxedOut );
-	}
-	p->setBrush( QBrush(white,Dense4Pattern) );
-	p->setPen( NoPen );
-	p->setBackgroundMode( OpaqueMode );
-	if ( maxedOut ) {
-	    p->drawRect( sliderR );
-	} else {
-	    if ( controls & SUB_PAGE )
-		p->drawRect( subPageR );
-	    if ( controls & ADD_PAGE )
-		p->drawRect( addPageR );
-	    if ( controls & SLIDER ) {
-		if ( !maxedOut ) {
-		    QBrush fill( g.background() );
-		    qDrawWinPanel( p, sliderR.x(), sliderR.y(),
-				   sliderR.width(), sliderR.height(), g,
-				   FALSE, &fill );
-		}
-	    }
-	}
-	// ### perhaps this should not be able to accept focus if maxedOut?
-	if ( hasFocus() && (controls & SLIDER) )
-	    p->drawWinFocusRect( sliderR.x()+2, sliderR.y()+2,
-				 sliderR.width()-5, sliderR.height()-5,
-				 backgroundColor() );
-    } else {
-	if ( controls & ADD_LINE )
-	    qDrawArrow( p, VERTICAL ? DownArrow : RightArrow, MotifStyle,
-			ADD_LINE_ACTIVE, addB.x(), addB.y(),
-			addB.width(), addB.height(), g, value()<maxValue() );
-	if ( controls & SUB_LINE )
-	    qDrawArrow( p, VERTICAL ? UpArrow : LeftArrow, MotifStyle,
-			SUB_LINE_ACTIVE, subB.x(), subB.y(),
-			subB.width(), subB.height(), g, value()>minValue() );
-	if ( controls & SUB_PAGE )
-	    p->fillRect( subPageR, g.mid() );
-	if ( controls & ADD_PAGE )
-	    p->fillRect( addPageR, g.mid() );
-	if ( controls & SLIDER ) {
-	    QBrush fill( g.background() );
-	    qDrawShadePanel( p, sliderR, g, FALSE, 2, &fill );
-	}
-
-    }
+    QWidget::styleChange( old );
 }
 
 
 #undef ADD_LINE_ACTIVE
 #undef SUB_LINE_ACTIVE
-
-
-void qDrawArrow( QPainter *p, ArrowType type, GUIStyle style, bool down,
-		 int x, int y, int w, int h,
-		 const QColorGroup & g )
-{
-    qDrawArrow( p, type, style, down, x, y, w, h, g, TRUE );
-}
