@@ -1,37 +1,36 @@
 /****************************************************************************
-** $Id: qslider.cpp,v 2.44 1997/09/26 10:57:41 eiriken Exp $
+** $Id: qslider.cpp,v 2.92.2.1 1999/07/23 11:10:52 paul Exp $
 **
 ** Implementation of QSlider class
 **
 ** Created : 961019
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qslider.h"
 #include "qpainter.h"
-#include "qdrawutl.h"
+#include "qdrawutil.h"
 #include "qtimer.h"
-#include "qkeycode.h"
-
-RCSTAG("$Id: qslider.cpp,v 2.44 1997/09/26 10:57:41 eiriken Exp $");
+#include "qbitmap.h"
 
 
 static const int motifBorder = 2;
-static const int motifLength = 30;
-static const int winLength = 9; // Must be odd
 static const int thresholdTime = 500;
 static const int repeatTime    = 100;
 
@@ -45,19 +44,45 @@ static int sliderStartVal = 0; //##### class member?
   \brief The QSlider widget provides a vertical or horizontal slider.
   \ingroup realwidgets
 
-  A slider is used to let the user control a value within a
-  program-definable range. In contrast to a QScrollBar, the QSlider
-  widget has a constant size slider and no arrow buttons.
+  The slider is the classic widget for controlling a bounded value.
+  It lets the user move a slider along a horizontal or vertical
+  groove, and translates the slider's position into an integer value
+  in the legal range.
 
-  QSlider only offers integer ranges.
+  QSlider inherits QRangeControl, which provides the "integer" side of
+  the slider.  setRange() and value() are likely to be used by
+  practically all slider users; see the \l QRangeControl documentation
+  for information about the many other functions that class provides.
 
-  The recommended thickness of a slider is given by sizeHint().
+  The main functions offered by the slider itself are tickmark and
+  orientation control; you can use setTickmarks() to indicate where
+  you want the tickmarks to be, setTickInterval() to indicate how many
+  of them you want, and setOrientation() to indicate whether the
+  slider is to e horizontal or vertical.
 
-  Tickmarks may be added using setTickmarks().
+  A slider has a default focusPolicy() of \a TabFocus, and a suitable
+  keyboard interface.
 
-  A slider has a default focusPolicy() of \a TabFocus.
+  <img src=qslider-m.png> <img src=qslider-w.png>
 
-  <img src=qslider-m.gif> <img src=qslider-w.gif> */
+  \sa QScrollBar QSpinBox
+  <a href="guibooks.html#fowler">GUI Design Handbook: Slider</a>
+*/
+
+
+/*! \enum QSlider::TickSetting
+
+  This enum specifies where the tick marks are to be drawn, relative
+  to the slider's groove and the handle the user moves.  The possible
+  values are \c NoMarks (no tickmarks are drawn), \c Above, \c Below,
+  \c Left, \c Right and \c Both.
+
+  \c NoMarks means to not draw any tickmarks; \c Both means to draw
+  tickmarks on both sides of the groove.  \c Above and \c Below mean
+  to draw tickmarks above and below the (horizontal) slider.  \c Left
+  and \c Right mean to draw tickmarks to the left and right of the
+  (vertical) slider.
+*/
 
 
 /*!
@@ -67,7 +92,7 @@ static int sliderStartVal = 0; //##### class member?
 */
 
 QSlider::QSlider( QWidget *parent, const char *name )
-    : QWidget( parent, name )
+    : QWidget( parent, name, WResizeNoErase )
 {
     orient = Vertical;
     init();
@@ -82,7 +107,7 @@ QSlider::QSlider( QWidget *parent, const char *name )
 */
 
 QSlider::QSlider( Orientation orientation, QWidget *parent, const char *name )
-    : QWidget( parent, name )
+    : QWidget( parent, name, WResizeNoErase )
 {
     orient = orientation;
     init();
@@ -100,14 +125,15 @@ QSlider::QSlider( Orientation orientation, QWidget *parent, const char *name )
   The \e parent and \e name arguments are sent to the QWidget constructor.
 */
 
-QSlider::QSlider( int minValue, int maxValue, int step,
+QSlider::QSlider( int minValue, int maxValue, int pageStep,
 		  int value, Orientation orientation,
 		  QWidget *parent, const char *name )
-    : QWidget( parent, name ),
-      QRangeControl( minValue, maxValue, 1, step, value )
+    : QWidget( parent, name, WResizeNoErase ),
+      QRangeControl( minValue, maxValue, 1, pageStep, value )
 {
     orient = orientation;
     init();
+    sliderVal = value;
 }
 
 
@@ -200,15 +226,13 @@ void QSlider::setTracking( bool enable )
 */
 
 /*!
-  Calculates slider position corresponding to value \a v. Does not perform
-  rounding.
+  Calculates slider position corresponding to value \a v.
 */
 
 int QSlider::positionFromValue( int v ) const
 {
     int  a = available();
-    int range = maxValue() - minValue();
-    return range > 0 ? ( (v - minValue() ) * a ) / (range): 0;
+    return QRangeControl::positionFromValue( v, a );
 }
 
 /*!
@@ -220,27 +244,27 @@ int QSlider::available() const
     int a;
     switch ( style() ) {
     case WindowsStyle:
-	a = (orient == Horizontal) ? width() - winLength
-	    : height() - winLength;
+	a = (orient == Horizontal) ? width() - style().sliderLength()
+	    : height() - style().sliderLength();
 	break;
     default:
     case MotifStyle:
-	a = (orient == Horizontal) ? width() -motifLength - 2*motifBorder 
-	    : height() - motifLength - 2*motifBorder;
+	a = (orient == Horizontal) ? width() -style().sliderLength() - 2*motifBorder
+	    : height() - style().sliderLength() - 2*motifBorder;
 	break;
     }
     return a;
 }
 
 /*!
-  Calculates value corresponding to slider position \a p. Performs rounding.
+  Calculates value corresponding to slider position \a p.
 */
 
 int QSlider::valueFromPosition( int p ) const
 {
     int a = available();
-    int range = maxValue() - minValue();
-    return   minValue() + ( a > 0 ? (2 * p * range + a ) / ( 2*a ): 0 );
+    return QRangeControl::valueFromPosition( p, a );
+
 }
 
 /*!
@@ -251,7 +275,7 @@ void QSlider::rangeChange()
 {
     int newPos = positionFromValue( value() );
     if ( newPos != sliderPos ) {
-	reallyMoveSlider( newPos ); 
+	reallyMoveSlider( newPos );
     }
 }
 
@@ -278,6 +302,9 @@ void QSlider::resizeEvent( QResizeEvent * )
 {
     rangeChange();
     initTicks();
+    if ( autoMask() )
+	updateMask();
+    repaint( rect() );
 }
 
 
@@ -330,187 +357,47 @@ QRect QSlider::sliderRect() const
     switch ( style() ) {
     case WindowsStyle:
 	if (orient == Horizontal )
-	    r.setRect( sliderPos, tickOffset, 
-		       winLength, thickness()  );
+	    r.setRect( sliderPos, tickOffset,
+		       style().sliderLength(), thickness()  );
 	else
 	    r.setRect ( tickOffset, sliderPos,
-			thickness(), winLength  );
+			thickness(), style().sliderLength()  );
 	break;
     default:
     case MotifStyle:
 	if (orient == Horizontal )
-	    r.setRect ( sliderPos + motifBorder, tickOffset + motifBorder, 
-			motifLength, thickness() - 2 * motifBorder );
+	    r.setRect ( sliderPos + motifBorder, tickOffset + motifBorder,
+			style().sliderLength(), thickness() - 2 * motifBorder );
 	else
-	    r.setRect ( tickOffset + motifBorder, sliderPos + motifBorder, 
-			thickness() - 2 * motifBorder, motifLength );
+	    r.setRect ( tickOffset + motifBorder, sliderPos + motifBorder,
+			thickness() - 2 * motifBorder, style().sliderLength() );
 	break;
     }
     return r;
 }
-enum SlDir {SlUp,SlDown,SlLeft,SlRight};
-
-static void drawWinPointedSlider( QPainter *p,
-				  const QRect r,
-				  const QColorGroup &g,
-				  SlDir dir)
-{
-    // 3333330
-    // 3444410
-    // 3422210
-    // 3422210
-    // 3422210
-    // 3422210
-    //  34210
-    //   340
-    //    0
-
-    const QColor c0 = black;
-    const QColor c1 = g.dark();
-    const QColor c2 = g.background();
-    const QColor c3 = g.midlight();
-    const QColor c4 = g.light();
-
-    int x1 = r.left();
-    int x2 = r.right();
-    int y1 = r.top();
-    int y2 = r.bottom();
-
-
-    QBrush oldBrush = p->brush();
-    p->setBrush( c2 );
-    p->setPen( NoPen );
-    p->drawRect( r );
-    p->setBrush( oldBrush );
-
-
-    switch ( dir ) {
-    case SlUp:
-	y1 = y1 + r.width()/2;
-	break;
-    case SlDown:
-	y2 = y2 - r.width()/2;
-	break;
-    case SlLeft:
-	x1 = x1 + r.height()/2;
-	break;
-    case SlRight:
-	x2 = x2 - r.height()/2;
-	break;
-    }
-
-    if ( dir != SlUp ) {
-	p->setPen( c4 );
-	p->drawLine( x1, y1, x2, y1 );
-	p->setPen( c3 );
-	p->drawLine( x1, y1+1, x2, y1+1 );
-    }
-    if ( dir != SlLeft ) {
-	p->setPen( c3 );
-	p->drawLine( x1+1, y1+1, x1+1, y2 );
-	p->setPen( c4 );
-	p->drawLine( x1, y1, x1, y2 );
-    }
-    if ( dir != SlRight ) {
-	p->setPen( c0 );
-	p->drawLine( x2, y1, x2, y2 );
-	p->setPen( c1 );
-	p->drawLine( x2-1, y1+1, x2-1, y2-1 );
-    }
-    if ( dir != SlDown ) {
-	p->setPen( c0 );
-	p->drawLine( x1, y2, x2, y2 );
-	p->setPen( c1 );
-	p->drawLine( x1+1, y2-1, x2-1, y2-1 );
-    }
-
-    int d;
-    switch ( dir ) {
-	case SlUp:
-	    p->setPen( c4 );
-	    d =  (r.width() + 1) / 2 - 1;
-	    p->drawLine( x1, y1, x1+d, y1-d);
-	    p->setPen( c0 );
-	    d = r.width() - d - 1;
-	    p->drawLine( x2, y1, x2-d, y1-d);
-	    p->setPen( c1 );
-	    d--;
-	    p->drawLine( x2-1, y1, x2-1-d, y1-d);
-	    break;
-	case SlDown:
-	    p->setPen( c4 );
-	    d =  (r.width() + 1) / 2 - 1;
-	    p->drawLine( x1, y2, x1+d, y2+d);
-	    p->setPen( c0 );
-	    d = r.width() - d - 1;
-	    p->drawLine( x2, y2, x2-d, y2+d);
-	    p->setPen( c1 );
-	    d--;
-	    p->drawLine( x2-1, y2, x2-1-d, y2+d);
-	    break;
-	case SlLeft:
-	    p->setPen( c4 );
-	    d =  (r.height() + 1) / 2 - 1;
-	    p->drawLine( x1, y1, x1-d, y1+d);
-	    p->setPen( c0 );
-	    d = r.height() - d - 1;
-	    p->drawLine( x1, y2, x1-d, y2-d);
-	    p->setPen( c1 );
-	    d--;
-	    p->drawLine( x1, y2-1, x1-d, y2-1-d);
-	    break;
-	case SlRight:
-	    p->setPen( c4 );
-	    d =  (r.height() + 1) / 2 - 1;
-	    p->drawLine( x2, y1, x2+d, y1+d);
-	    p->setPen( c0 );
-	    d = r.height() - d - 1;
-	    p->drawLine( x2, y2, x2+d, y2-d);
-	    p->setPen( c1 );
-	    d--;
-	    p->drawLine( x2, y2-1, x2+d, y2-1-d);
-	    break;
-    }
-
-}
 
 
 /*!
-  Paints the slider button using painter \a p with size and
-  position given by \a r. Reimplement this function to change the
-  look of the slider button.  
+  Paints the slider button using painter \a p with size, a colorgroup
+  and position given by \a r. Reimplement this function to change the
+  look of the slider button.
+
+  Setting the colorgroup is useful to reuse the code to draw a mask if
+  the slider supports transparceny.
+
+  \sa setAutoMask(), updateMask()
 */
 
-void QSlider::paintSlider( QPainter *p, const QRect &r )
-{
-    QColorGroup g = colorGroup();
-    QBrush fill( g.background() );
+void QSlider::paintSlider( QPainter *p, const QColorGroup &g, const QRect &r )
+{ //####### should this one be removed? private? non-virtual?
+    QBrush fill = g.brush( QColorGroup::Button );
 
-    switch ( style() ) {
-    case WindowsStyle:
-	if ( ticks == NoMarks || ticks == Both ) {
-	    qDrawWinButton( p, r, g, FALSE, &fill );
-	} else {
-	    SlDir d = ( orient == Horizontal ) ?
-		      (ticks == Above) ? SlUp : SlDown
-		    : (ticks == Left) ? SlLeft : SlRight;
-	    drawWinPointedSlider( p, r, g, d );
-	}
-	break;
-    default:
-    case MotifStyle:
-	qDrawShadePanel( p, r, g, FALSE, 2, &fill );
-	if ( orient == Horizontal ) {
-	    QCOORD mid = ( r.left() + r.right() + 1) / 2;
-	    qDrawShadeLine( p, mid,  r.top(), mid,  r.bottom() - 1,
-			    g, TRUE, 1);
-	} else {
-	    QCOORD mid = ( r.top() + r.bottom() + 1) / 2;
-	    qDrawShadeLine( p, r.left(), mid,  r.right() - 1, mid,
-			    g, TRUE, 1);
-	}
-	break;
-    }
+    QPoint bo = p->brushOrigin();
+    p->setBrushOrigin(r.topLeft());
+
+    style().drawSlider( p, r.x(), r.y(), r.width(), r.height(), g, orient,
+			ticks & Above, ticks & Below );
+    p->setBrushOrigin(bo);
 }
 
 /*!
@@ -527,20 +414,22 @@ void QSlider::reallyMoveSlider( int newPos )
     if ( orient == Horizontal ) {
 	if ( oldR.left() < newR.left() )
 	    oldR.setRight( QMIN ( oldR.right(), newR.left()));
-	else           //oldR.right() >= newR.right() 
+	else           //oldR.right() >= newR.right()
 	    oldR.setLeft( QMAX ( oldR.left(), newR.right()));
     } else {
 	if ( oldR.top() < newR.top() )
 	    oldR.setBottom( QMIN ( oldR.bottom(), newR.top()));
-	else           //oldR.bottom() >= newR.bottom() 
+	else           //oldR.bottom() >= newR.bottom()
 	    oldR.setTop( QMAX ( oldR.top(), newR.bottom()));
     }
     repaint( oldR );
     repaint( newR, FALSE );
+    if ( autoMask() )
+	updateMask();
 }
 
+/*!\obsolete
 
-/*!
   Draws the "groove" on which the slider moves, using the painter \a p.
   \a c gives the distance from the top (or left) edge of the widget to
   the center of the groove.
@@ -550,11 +439,11 @@ void QSlider::drawWinGroove( QPainter *p, QCOORD c )
 {
     if ( orient == Horizontal ) {
 	qDrawWinPanel( p, 0, c - 2,  width(), 4, colorGroup(), TRUE );
-	p->setPen( black );
+	p->setPen( colorGroup().foreground() );
 	p->drawLine( 1, c - 1, width() - 3, c - 1 );
     } else {
 	qDrawWinPanel( p, c - 2, 0, 4, height(), colorGroup(), TRUE );
-	p->setPen( black );
+	p->setPen( colorGroup().foreground() );
 	p->drawLine( c - 1, 1, c - 1, height() - 3 );
     }
 }
@@ -564,65 +453,33 @@ void QSlider::drawWinGroove( QPainter *p, QCOORD c )
   Handles paint events for the slider.
 */
 
-void QSlider::paintEvent( QPaintEvent *e )
+void QSlider::paintEvent( QPaintEvent * )
 {
-    
-    QPainter p;
-    p.begin( this );
-    QRect paintRect = e->rect();
-    p.setClipRect( paintRect );
-    QRect sliderR = sliderRect();
-    QColorGroup g = colorGroup();
-    switch ( style() ) {
-    case WindowsStyle:
-	if ( hasFocus() ) {
-	    QRect r;
-	    if ( orient == Horizontal )
-		r.setRect( 0, tickOffset-1, width(), thickness()+2 );
-	    else
-		r.setRect( tickOffset-1, 0, thickness()+2, height() );
-	    r = r.intersect( rect() );
-	    qDrawPlainRect( &p, r, g.background() );
-	    p.drawWinFocusRect( r, backgroundColor() );
-	}
-	{
-	    int mid = tickOffset + thickness()/2;
-	    if ( ticks & Above )
-		mid += winLength / 8;
-	    if ( ticks & Below )
-		mid -= winLength / 8;
-	    drawWinGroove( &p, mid );
-	}
-	paintSlider( &p, sliderR );
-	break;
-    default:
-    case MotifStyle:
-	if ( orient == Horizontal ) {
-	    qDrawShadePanel( &p, 0, tickOffset, width(), thickness(),
-			     g, TRUE );
-	    p.fillRect( 0, 0, width(), tickOffset, g.background() );
-	    p.fillRect( 0, tickOffset + thickness(),
-			width(), height()/*###*/, g.background() ); 
-	} else {
-	    qDrawShadePanel( &p, tickOffset, 0, thickness(), height(),
-			     g, TRUE );
-	    p.fillRect( 0, 0,  tickOffset, height(), g.background() );
-	    p.fillRect( tickOffset + thickness(), 0,
-			width()/*###*/, height(), g.background() ); 
-	}
 
-	if ( hasFocus() ) {
-	    p.setPen( black );
-	    if ( orient == Horizontal )
-		p.drawRect(  1, tickOffset + 1, width() - 2, thickness() - 2 );
-	    else
-		p.drawRect( tickOffset + 1, 1, thickness() - 2, height() - 2 );
+    QPainter p( this );
+    const QRect & sliderR = sliderRect();
+    const QColorGroup & g = colorGroup();
+    {
+	int mid = thickness()/2;
+	if ( ticks & Above )
+	    mid += style().sliderLength() / 8;
+	if ( ticks & Below )
+	    mid -= style().sliderLength() / 8;
+ 	if ( orient == Horizontal ) {
+ 	    style().drawSliderGroove(&p, 0, tickOffset, width(), thickness(),
+ 				     g, mid, Horizontal );
+ 	    p.fillRect( 0, 0, width(), tickOffset, g.background() );
+ 	    p.fillRect( 0, tickOffset + thickness(),
+ 			width(), height()/*###*/, g.background() );
+	}
+	else {
+ 	    style().drawSliderGroove( &p, tickOffset, 0, thickness(), height(),
+				      g, mid, Vertical );
+ 	    p.fillRect( 0, 0,  tickOffset, height(), g.background() );
+ 	    p.fillRect( tickOffset + thickness(), 0,
+ 			width()/*###*/, height(), g.background() );
 	}
-	paintSlider( &p, sliderR );
-	break;
     }
-
-    
     int interval = tickInt;
     if ( interval <= 0 ) {
 	interval = lineStep();
@@ -630,17 +487,83 @@ void QSlider::paintEvent( QPaintEvent *e )
 	    interval = pageStep();
     }
     if ( ticks & Above )
-	drawTicks( &p, 0, tickOffset - 2, interval );
+	drawTicks( &p, g, 0, tickOffset - 2, interval );
 	
     if ( ticks & Below ) {
 	int avail = (orient == Horizontal) ? height() : width();
 	avail -= tickOffset + thickness();
-	drawTicks( &p, tickOffset + thickness() + 1, avail - 2, interval );
+	drawTicks( &p, g, tickOffset + thickness() + 1, avail - 2, interval );
     }
-    p.end();
+    if ( hasFocus() ) {
+	QRect r;
+	if ( orient == Horizontal )
+	    r.setRect( 0, tickOffset-1, width(), thickness()+2 );
+	else
+	    r.setRect( tickOffset-1, 0, thickness()+2, height() );
+	r = r.intersect( rect() );
+	if (style() == MotifStyle)
+	    style().drawFocusRect(&p, QRect(r.x()+1, r.y()+1, r.width()-2, r.height()-2), g);
+	else
+	    style().drawFocusRect(&p, r, g);
+    }
+    paintSlider( &p, g, sliderR );
+
 }
 
 
+/*!
+
+ Reimplementation of QWidget::updateMask(). Draws the mask of the
+ slider when transparency is required.
+
+ \sa QWidget::setAutoMask()
+*/
+void QSlider::updateMask()
+{
+    QBitmap bm( size() );
+    bm.fill( color0 );
+
+    {
+	QPainter p( &bm, this );
+	QRect sliderR = sliderRect();
+	QColorGroup g(color1, color1, color1, color1, color1, color1, color1, color1, color0);
+	QBrush fill (color1);
+	int mid = tickOffset + thickness()/2;
+	if ( ticks & Above )
+	    mid += style().sliderLength() / 8;
+	if ( ticks & Below )
+	    mid -= style().sliderLength() / 8;
+ 	if ( orient == Horizontal ) {
+ 	    style().drawSliderGrooveMask(&p, 0, tickOffset, width(), thickness(),
+					 mid, Horizontal );
+	}
+	else {
+ 	    style().drawSliderGrooveMask( &p, tickOffset, 0, thickness(), height(),
+					  mid, Vertical );
+	}
+	style().drawSliderMask( &p, sliderR.x(), sliderR.y(),
+				sliderR.width(), sliderR.height(),
+				orient, ticks & Above, ticks & Below );
+
+	int interval = tickInt;
+	if ( interval <= 0 ) {
+	    interval = lineStep();
+	    if ( positionFromValue( interval ) - positionFromValue( 0 ) < 3 )
+		interval = pageStep();
+	}
+	if ( ticks & Above )
+	    drawTicks( &p, g, 0, tickOffset - 2, interval );
+	
+	if ( ticks & Below ) {
+	    int avail = (orient == Horizontal) ? height() : width();
+	    avail -= tickOffset + thickness();
+	    drawTicks( &p, g, tickOffset + thickness() + 1, avail - 2, interval );
+	}
+
+    }
+    setMask( bm );
+}
+
 /*!
   Handles mouse press events for the slider.
 */
@@ -649,24 +572,20 @@ void QSlider::mousePressEvent( QMouseEvent *e )
 {
     resetState();
     sliderStartVal = sliderVal;
-    if ( e->button() == MidButton ) {
-	int pos = goodPart( e->pos() );
-	moveSlider( pos - slideLength() / 2 );
-	return;
-    }
-    if ( e->button() != LeftButton )
-	return;
     QRect r = sliderRect();
 
-    if ( r.contains( e->pos() ) ) {
+    if ( e->button() == RightButton ) {
+	return;
+    } else if ( r.contains( e->pos() ) ) {
 	state = Dragging;
 	clickOffset = (QCOORD)( goodPart( e->pos() ) - sliderPos );
 	emit sliderPressed();
-    } else if ( funnyWindowsStyle && style() == WindowsStyle) {
+    } else if ( e->button() == MidButton ||
+		(funnyWindowsStyle && style() == WindowsStyle) ) {
 	int pos = goodPart( e->pos() );
 	moveSlider( pos - slideLength() / 2 );
 	state = Dragging;
-	clickOffset = slideLength() / 2;     
+	clickOffset = slideLength() / 2;
     } else if ( orient == Horizontal && e->pos().x() < r.left() //### goodPart
 		|| orient == Vertical && e->pos().y() < r.top() ) {
 	state = TimingDown;
@@ -674,7 +593,7 @@ void QSlider::mousePressEvent( QMouseEvent *e )
 	if ( !timer )
 	    timer = new QTimer( this );
 	connect( timer, SIGNAL(timeout()), SLOT(repeatTimeout()) );
-	timer->start( thresholdTime, TRUE ); 
+	timer->start( thresholdTime, TRUE );
     } else if ( orient == Horizontal && e->pos().x() > r.right() //### goodPart
 		|| orient == Vertical && e->pos().y() > r.bottom() ) {
 	state = TimingUp;
@@ -682,7 +601,7 @@ void QSlider::mousePressEvent( QMouseEvent *e )
 	if ( !timer )
 	    timer = new QTimer( this );
 	connect( timer, SIGNAL(timeout()), SLOT(repeatTimeout()) );
-	timer->start( thresholdTime, TRUE ); 
+	timer->start( thresholdTime, TRUE );
     }
 }
 
@@ -692,43 +611,54 @@ void QSlider::mousePressEvent( QMouseEvent *e )
 
 void QSlider::mouseMoveEvent( QMouseEvent *e )
 {
-    
-    if ( style() == WindowsStyle ) {
-	QRect r = rect();
-	if ( orientation() == Horizontal )
-	    r.setRect( r.x() - 20, r.y() - 30, r.width() + 40, r.height() + 60 );
-	else
-	    r.setRect( r.x() - 30, r.y() - 20, r.width() + 60, r.height() + 40 );
-	if ( !r.contains( e->pos() ) ) { 
-	    moveSlider( positionFromValue( sliderStartVal) );
-	    return;
-	}
-    }
-
-    if ( (e->state() & MidButton) ) { 		// middle button wins
-	int pos = goodPart( e->pos() );
-	moveSlider( pos - slideLength() / 2 );
-	return;	
-    }
-    if ( !(e->state() & LeftButton) )
-	return;					// left mouse button is up
     if ( state != Dragging )
 	return;
 
+    if ( style() == WindowsStyle ) {
+	QRect r = rect();
+	int m = style().maximumSliderDragDistance();
+	if ( m >= 0 ) {
+	    if ( orientation() == Horizontal )
+		r.setRect( r.x() - m, r.y() - 2*m/3,
+			   r.width() + 2*m, r.height() + 3*m );
+	    else
+		r.setRect( r.x() - 2*m/3, r.y() - m,
+			   r.width() + 3*m, r.height() + 2*m );
+	    if ( !r.contains( e->pos() ) ) {
+		moveSlider( positionFromValue( sliderStartVal) );
+		return;
+	    }
+	}
+    }
+
     int pos = goodPart( e->pos() );
-    
     moveSlider( pos - clickOffset );
 }
 
+/*!
+  Handles wheel events for the slider.
+*/
+void QSlider::wheelEvent( QWheelEvent * e){
+    static float offset = 0;
+    static QSlider* offset_owner = 0;
+    if (offset_owner != this){
+	offset_owner = this;
+	offset = 0;
+    }
+    offset += -e->delta()*QMAX(pageStep(),lineStep())/120;
+    if (QABS(offset)<1)
+	return;
+    setValue( value() + int(offset) );
+    offset -= int(offset);
+}
+
 
 /*!
   Handles mouse release events for the slider.
 */
 
-void QSlider::mouseReleaseEvent( QMouseEvent *e )
+void QSlider::mouseReleaseEvent( QMouseEvent * )
 {
-    if ( !(e->button() & LeftButton) )
-	return;
     resetState();
 }
 
@@ -739,10 +669,23 @@ void QSlider::mouseReleaseEvent( QMouseEvent *e )
 void QSlider::focusInEvent( QFocusEvent * )
 {
     repaint( FALSE );
+    if ( autoMask() )
+	updateMask();
 }
 
 /*!
-  Moves the left (or top) edge of the slider to position 
+  Handles focus out events for the slider.
+*/
+
+void QSlider::focusOutEvent( QFocusEvent * )
+{
+    repaint( visibleRect() );
+    if ( autoMask() )
+	updateMask();
+}
+
+/*!
+  Moves the left (or top) edge of the slider to position
   \a pos. Performs snapping.
 */
 
@@ -767,7 +710,7 @@ void QSlider::moveSlider( int pos )
     default:
     case MotifStyle:
 	break;
-    }	    
+    }	
 
     if ( sliderPos != newPos )
 	reallyMoveSlider( newPos );
@@ -796,7 +739,7 @@ void QSlider::resetState()
     case Idle:
 	break;
     default:
-	warning("QSlider: in wrong state");
+	qWarning("QSlider: (%s) in wrong state", name( "unnamed" ) );
     }
     state = Idle;
 }
@@ -842,7 +785,6 @@ void QSlider::keyPressEvent( QKeyEvent *e )
 	e->ignore();
 	return;
     }
-    e->accept();
 }
 
 
@@ -850,14 +792,14 @@ void QSlider::keyPressEvent( QKeyEvent *e )
   Returns the length of the slider.
 */
 
-int QSlider::slideLength() const 
+int QSlider::slideLength() const
 {
     switch ( style() ) {
     case WindowsStyle:
-	return winLength;
+	return style().sliderLength();
     default:
     case MotifStyle:
-	return motifLength;
+	return style().sliderLength();
     }
 }
 
@@ -904,7 +846,7 @@ void QSlider::repeatTimeout()
 	connect( timer, SIGNAL(timeout()), SLOT(subtractStep()) );
     else if ( state == TimingUp )
 	connect( timer, SIGNAL(timeout()), SLOT(addStep()) );
-    timer->start( repeatTime, FALSE );   
+    timer->start( repeatTime, FALSE );
 }
 
 
@@ -917,15 +859,14 @@ int QSlider::goodPart( const QPoint &p ) const
     return (orient == Horizontal) ?  p.x() : p.y();
 }
 
-/*!  
-  Returns the recommended size of the slider. Only the thickness is
-  relevant.
+/*!
+  Returns the recommended size of the slider.
 */
 
 QSize QSlider::sizeHint() const
 {
     const int length = 84;
-    int thick = 16;
+    int thick = style() == MotifStyle ? 24 : 16;
     const int tickSpace = 5;
 
     if ( ticks & Above )
@@ -933,7 +874,7 @@ QSize QSlider::sizeHint() const
     if ( ticks & Below )
 	thick += tickSpace;
     if ( style() == WindowsStyle && ticks != Both && ticks != NoMarks )
-	thick += winLength / 4;	    // pointed slider
+	thick += style().sliderLength() / 4;	    // pointed slider
     if ( orient == Horizontal )
 	return QSize( length, thick );
     else
@@ -941,9 +882,43 @@ QSize QSlider::sizeHint() const
 }
 
 
+
 /*!
-  Returns the number of pixels to use for the business part of the 
-  slider (i.e. the non-tickmark portion). The remaining space is shared 
+  \reimp
+*/
+
+QSize QSlider::minimumSizeHint() const
+{
+    QSize s = sizeHint();
+    int length = style().sliderLength();
+    if ( orient == Horizontal )
+	s.setWidth( length );
+    else
+	s.setHeight( length );
+
+    return s;
+}
+
+
+
+/*!
+  Specifies that this widget can use more, but is able to survive on
+  less, space in the orientation() direction; and is fixed in the other
+  direction.
+*/
+
+QSizePolicy QSlider::sizePolicy() const
+{
+    if ( orient == Horizontal )
+	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
+    else
+	return QSizePolicy(  QSizePolicy::Fixed, QSizePolicy::Expanding );
+}
+
+
+/*!
+  Returns the number of pixels to use for the business part of the
+  slider (i.e. the non-tickmark portion). The remaining space is shared
   equally between the tickmark regions. This function and  sizeHint()
   are closely related; if you change one, you almost certainly
   have to change the other.
@@ -962,7 +937,7 @@ int QSlider::thickness() const
 
     int thick = 6;	// Magic constant to get 5 + 16 + 5
     if ( style() == WindowsStyle && ticks != Both && ticks != NoMarks ) {
-	thick += winLength / 4;
+	thick += style().sliderLength() / 4;
     }
     space -= thick;
     //### the two sides may be unequal in size
@@ -971,15 +946,40 @@ int QSlider::thickness() const
     return thick;
 }
 
-
-/*!  
+/*!
   Using \a p, draws tickmarks at a distance of \a d from the edge
-  of the widget, using \a w pixels and with an interval of \a i.  
+  of the widget, using \a w pixels and with an interval of \a i.
+
+  Do not reimplement this function, it's mainly there for
+  compatibility reasons. It simply calls
+
+  drawTicks( QPainter *p, const QColorGroup& g, int d, int w, int i ) const
+
+  which is the right function to overload.
+
+  \sa colorGroup(), drawTicks( QPainter *p, const QColorGroup& g, int d, int w, int i ) const,
+  paintSlider
+
 */
 
 void QSlider::drawTicks( QPainter *p, int d, int w, int i ) const
 {
-    p->setPen( colorGroup().foreground() );
+    drawTicks( p, colorGroup(), d, w, i);
+}
+
+/*!
+  Using \a p, draws tickmarks at a distance of \a d from the edge
+  of the widget, using \a w pixels and with an interval of \a i.
+
+  Setting the colorgroup is useful to reuse the code to draw a mask if
+  the slider supports transparceny.
+
+\sa setAutoMask(), updateMask()
+*/
+
+void QSlider::drawTicks( QPainter *p, const QColorGroup& g, int d, int w, int i ) const
+{
+    p->setPen( g.foreground() );
     int v = minValue();
     int fudge = slideLength() / 2 + 1;
     while ( v <= maxValue() + 1 ) {
@@ -994,7 +994,7 @@ void QSlider::drawTicks( QPainter *p, int d, int w, int i ) const
 
 
 /*!
-  Sets the way tickmarks are displayed by the slider. \a s can take 
+  Sets the way tickmarks are displayed by the slider. \a s can take
   the following values:
   <ul>
   <li> \c NoMarks
@@ -1013,6 +1013,8 @@ void QSlider::setTickmarks( TickSetting s )
     ticks = s;
     initTicks();
     update();
+    if ( autoMask() )
+	updateMask();
 }
 
 
@@ -1031,9 +1033,9 @@ void QSlider::setTickmarks( TickSetting s )
 */
 
 /*!
-  Sets the interval between tickmarks to \a i. This is a value interval, 
+  Sets the interval between tickmarks to \a i. This is a value interval,
   not a pixel interval. If \a i is 0, the slider
-  will choose between lineStep() and pageStep(). The initial value of 
+  will choose between lineStep() and pageStep(). The initial value of
   tickInterval() is 0.
   \sa tickInterval(), QRangeControl::lineStep(), QRangeControl::pageStep()
 */
@@ -1042,6 +1044,8 @@ void QSlider::setTickInterval( int i )
 {
     tickInt = QMAX( 0, i );
     update();
+    if ( autoMask() )
+	updateMask();
 }
 
 
@@ -1051,3 +1055,16 @@ void QSlider::setTickInterval( int i )
   chooses between pageStep() and lineStep().
   \sa setTickInterval()
 */
+
+
+/*\reimp
+ */
+void QSlider::styleChange( QStyle& old )
+{
+    if ( style() == MotifStyle )
+	setBackgroundMode( PaletteMid );
+    else
+	setBackgroundMode( PaletteBackground );
+    QWidget::styleChange( old );
+}
+
