@@ -1,23 +1,25 @@
 /****************************************************************************
-** $Id: qspinbox.cpp,v 2.38 1998/07/06 10:23:16 aavit Exp $
+** $Id: qspinbox.cpp,v 2.73.2.1 1999/08/22 14:00:25 aavit Exp $
 **
 ** Implementation of QSpinBox widget class
 **
 ** Created : 1997
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -26,7 +28,6 @@
 #include <qspinbox.h>
 #include <qpushbutton.h>
 #include <qpainter.h>
-#include <qkeycode.h>
 #include <qbitmap.h>
 #include <qlineedit.h>
 #include <qvalidator.h>
@@ -40,7 +41,7 @@
 
   \ingroup realwidgets
 
-  QSpinBox allows the user to choose a numeric value, either by
+  QSpinBox allows the user to choose an integer value, either by
   clicking the up/down buttons to increase/decrease the value
   currently displayed, or by typing the value directly into the spin
   box.
@@ -60,7 +61,7 @@
   circular behavior.
 
   The displayed value can be appended and/or prepended with an
-  arbitray string indicating for example the unit of measurement. See
+  arbitrary string indicating for example the unit of measurement. See
   setPrefix() and setSuffix().
 
   It is often desirable to give the user a special, often default,
@@ -76,7 +77,7 @@
   the virtual functions mapValueToText() and mapTextToValue() and
   setting another, suitable validator using setValidator().
 
-  <img src=qspinbox-m.gif> <img src=qspinbox-w.gif>
+  <img src=qspinbox-m.png> <img src=qspinbox-w.png>
 
   \sa QScrollBar QSlider
   <a href="guibooks.html#fowler">GUI Design Handbook: Spin Box</a>
@@ -94,8 +95,9 @@ struct QSpinBoxData {
   \sa minValue(), maxValue(), setRange(), lineStep(), setSteps()
 */
 
-QSpinBox::QSpinBox( QWidget * parent , const char * name )
-    : QFrame( parent, name )
+QSpinBox::QSpinBox( QWidget * parent , const char *name )
+    : QFrame( parent, name ),
+      QRangeControl()
 {
     initSpinBox();
 }
@@ -124,6 +126,7 @@ void QSpinBox::initSpinBox()
 {
     extra = 0; 			// not used; reserved for future expansion
     wrap = FALSE;
+    edited = FALSE;
 
     up = new QPushButton( this, "up" );
     up->setFocusPolicy( QWidget::NoFocus );
@@ -133,12 +136,12 @@ void QSpinBox::initSpinBox()
     down->setFocusPolicy( QWidget::NoFocus );
     down->setAutoRepeat( TRUE );
 
-    validator = new QIntValidator( minValue(), maxValue(), this, "validator" );
-    vi = new QLineEdit( this, "lineeditor" );
+    validate = new QIntValidator( minValue(), maxValue(), this, "validator" );
+    vi = new QLineEdit( this, "line editor" );
     vi->setFrame( FALSE );
     setFocusProxy( vi );
     setFocusPolicy( StrongFocus );
-    vi->setValidator( validator );
+    vi->setValidator( validate );
     vi->installEventFilter( this );
 
     if ( style() == WindowsStyle )
@@ -147,15 +150,18 @@ void QSpinBox::initSpinBox()
 	setFrameStyle( Panel | Sunken );
     setLineWidth( 2 );
 
+    setPalettePropagation( AllChildren );
+    setFontPropagation( AllChildren );
+
     updateDisplay();
 
     connect( up, SIGNAL(pressed()), SLOT(stepUp()) );
     connect( down, SIGNAL(pressed()), SLOT(stepDown()) );
-    connect( vi, SIGNAL(textChanged(const char *)), SLOT(textChanged()) );
+    connect( vi, SIGNAL(textChanged(const QString&)), SLOT(textChanged()) );
 }
 
 /*!
-  Deletes the spin box, freeing all memory and other resoures.
+  Deletes the spin box, freeing all memory and other resources.
 */
 
 QSpinBox::~QSpinBox()
@@ -169,7 +175,7 @@ QSpinBox::~QSpinBox()
   \sa value()
 */
 
-const char * QSpinBox::text() const
+QString QSpinBox::text() const
 { 	
     return vi->text();
 }
@@ -186,13 +192,13 @@ const char * QSpinBox::text() const
 QString QSpinBox::cleanText() const
 {
     QString s = QString(text()).stripWhiteSpace();
-    if ( prefix() ) {
+    if ( !prefix().isEmpty() ) {
 	QString px = QString(prefix()).stripWhiteSpace();
 	int len = px.length();
 	if ( len && s.left(len) == px )  // Remove _only_ if it is the prefix
 	    s.remove( 0, len );
     }
-    if ( suffix() ) {
+    if ( !suffix().isEmpty() ) {
 	QString sx = QString(suffix()).stripWhiteSpace();
 	int len = sx.length();
 	if ( len && s.right(len) == sx )  // Remove _only_ if it is the suffix
@@ -204,7 +210,7 @@ QString QSpinBox::cleanText() const
 
 /*!
   Sets the special-value text to \a text. If set, the spin box will
-  display this text in stead of a numeric value whenever the current
+  display this text instead of a numeric value whenever the current
   value is equal to minVal(). Typically used for indicating that this
   choice has a special (default) meaning.
 
@@ -227,13 +233,13 @@ QString QSpinBox::cleanText() const
   displayed.
 
   To turn off the special-value text display, call this function with
-  0 or an empty string as parameter. The default is no special-value
-  text, i.e. the numeric value is shown as usual.
+  an empty string as parameter. The default is no special-value text,
+  i.e. the numeric value is shown as usual.
 
   \sa specialValueText()
 */
 
-void QSpinBox::setSpecialValueText( const char* text )
+void QSpinBox::setSpecialValueText( const QString &text )
 {
     specText = text;
     updateDisplay();
@@ -241,16 +247,16 @@ void QSpinBox::setSpecialValueText( const char* text )
 
 
 /*!
-  Returns the currently special-value text, or 0 if no special-value
-  text is currently set.
+  Returns the currently special-value text, or a null string if no
+  special-value text is currently set.
 
   \sa setSpecialValueText()
 */
 
-const char* QSpinBox::specialValueText() const
+QString QSpinBox::specialValueText() const
 {
     if ( specText.isEmpty() )
-	return 0;
+	return QString::null;
     else
 	return specText;
 }
@@ -261,13 +267,13 @@ const char* QSpinBox::specialValueText() const
   the displayed value. Typical use is to indicate the unit of
   measurement to the user.
 
-  To turn off the prefix display, call this function with 0 or an
-  empty string as parameter. The default is no prefix.
+  To turn off the prefix display, call this function with an empty
+  string as parameter. The default is no prefix.
 
   \sa prefix(), setSuffix(), suffix()
 */
 
-void QSpinBox::setPrefix( const char* text )
+void QSpinBox::setPrefix( const QString &text )
 {
     pfix = text;
     updateDisplay();
@@ -279,13 +285,13 @@ void QSpinBox::setPrefix( const char* text )
   displayed value. Typical use is to indicate the unit of measurement
   to the user.
 
-  To turn off the suffix display, call this function with 0 or an
-  empty string as parameter. The default is no suffix.
+  To turn off the suffix display, call this function with an empty
+  string as parameter. The default is no suffix.
 
   \sa suffix(), setPrefix(), prefix()
 */
 
-void QSpinBox::setSuffix( const char* text )
+void QSpinBox::setSuffix( const QString &text )
 {
     sfix = text;
     updateDisplay();
@@ -293,32 +299,32 @@ void QSpinBox::setSuffix( const char* text )
 
 
 /*!
-  Returns the currently set prefix, or 0 if no prefix is currently
-  set.
+  Returns the currently set prefix, or a null string if no prefix is
+  currently set.
 
   \sa setPrefix(), setSuffix(), suffix()
 */
 
-const char* QSpinBox::prefix() const
+QString QSpinBox::prefix() const
 {
     if ( pfix.isEmpty() )
-	return 0;
+	return QString::null;
     else
 	return pfix;
 }
 
 
 /*!
-  Returns the currently set suffix, or 0 if no suffix is currently
-  set.
+  Returns the currently set suffix, or a null string if no suffix is
+  currently set.
 
   \sa setSuffix(), setPrefix(), suffix()
 */
 
-const char* QSpinBox::suffix() const
+QString QSpinBox::suffix() const
 {
     if ( sfix.isEmpty() )
-	return 0;
+	return QString::null;
     else
 	return sfix;
 }
@@ -361,7 +367,7 @@ QSize QSpinBox::sizeHint() const
     if ( h < 12 ) 	// ensure enough space for the button pixmaps
 	h = 12;
     int w = 35; 	// minimum width for the value
-    int wx = fm.width( "  " );
+    int wx = fm.width( ' ' )*2;
     QString s;
     s.setNum( minValue() );
     s.prepend( prefix() );
@@ -371,9 +377,10 @@ QSize QSpinBox::sizeHint() const
     s.prepend( prefix() );
     s.append( suffix() );
     w = QMAX( w, fm.width( s ) + wx );
-    s = specialValueText();
-    w = QMAX( w, fm.width( s ) + wx );
-
+    if ( !specialValueText().isEmpty() ) {
+	s = specialValueText();
+	w = QMAX( w, fm.width( s ) + wx );
+    }
     QSize r( h // buttons AND frame both sides - see resizeevent()
 	     + 6 // right/left margins
 	     + w, // widest value
@@ -384,6 +391,16 @@ QSize QSpinBox::sizeHint() const
     return r;
 }
 
+/*!
+  Specifies that this widget may stretch horizontally, but is fixed
+  vertically.
+*/
+
+QSizePolicy QSpinBox::sizePolicy() const
+{
+    return QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed );
+}
+
 
 /*!
   Sets the current value of the spin box to \a value. This is
@@ -450,7 +467,7 @@ void QSpinBox::stepDown()
 
 
 /*!
-  \fn void QSpinBox::valueChanged( const char* valueText )
+  \fn void QSpinBox::valueChanged( const QString& valueText )
 
   This signal is emitted whenever the valueChanged( int ) signal is
   emitted, i.e. every time the value of the spin box changes (whatever
@@ -475,22 +492,19 @@ bool QSpinBox::eventFilter( QObject* obj, QEvent* ev )
     if ( obj != vi )
 	return FALSE;
 
-    if ( ev->type() == Event_FocusOut ) {
+    if ( ev->type() == QEvent::FocusOut || ev->type() == QEvent::Leave ) {
 	interpretText();
-    } else if ( ev->type() == Event_KeyPress ) {
+    } else if ( ev->type() == QEvent::KeyPress ) {
 	QKeyEvent* k = (QKeyEvent*)ev;
 	if ( k->key() == Key_Up ) {
 	    stepUp();
-	    k->accept();
 	    return TRUE;
 	} else if ( k->key() == Key_Down ) {
 	    stepDown();
-	    k->accept();
 	    return TRUE;
-	} else if ( k->key() == Key_Return ) { // Workaround for use in dialogs
+	} else if ( k->key() == Key_Return ) {
 	    interpretText();
-	    k->accept();
-	    return TRUE;
+	    return FALSE;
 	}
     }
     return FALSE;
@@ -535,7 +549,7 @@ void QSpinBox::resizeEvent( QResizeEvent* )
 	p.end();
 	up->setPixmap( bm );
     }
-    
+
     if ( down->size() != bs ) {
 	down->resize( bs );
 	QBitmap bm( bms );
@@ -552,6 +566,8 @@ void QSpinBox::resizeEvent( QResizeEvent* )
 	down->setPixmap( bm );
     }
 
+    repaint( rect() );
+
     int y = style() == WindowsStyle ? frameWidth() : 0;
     int x = width() - y - bs.width();
     up->move( x, y );
@@ -563,6 +579,26 @@ void QSpinBox::resizeEvent( QResizeEvent* )
 	vi->setGeometry( contentsRect() );
 }
 
+/*!
+  Handles wheel events for the spinbox.
+*/
+void QSpinBox::wheelEvent( QWheelEvent * e )
+{
+    static float offset = 0;
+    static QSpinBox* offset_owner = 0;
+    if (offset_owner != this) {
+	offset_owner = this;
+	offset = 0;
+    }
+    offset += -e->delta()/120;
+    if (QABS(offset) < 1)
+	return;
+    int ioff = int(offset);
+    int i;
+    for (i=0; i<QABS(ioff); i++)
+	offset > 0 ? stepDown() : stepUp();
+    offset -= ioff;
+}
 
 
 
@@ -586,24 +622,40 @@ void QSpinBox::valueChange()
 
 void QSpinBox::rangeChange()
 {
-    if ( validator->inherits( "QIntValidator" ) )
-	((QIntValidator*)validator)->setRange( minValue(), maxValue() );
+    if ( validate->inherits( "QIntValidator" ) )
+	((QIntValidator*)validate)->setRange( minValue(), maxValue() );
     updateDisplay();
 }
 
 
 /*!
-  Sets the validator to \a v. The default is to use a suitable
-  QIntValidator.
+  Sets the validator to \a v. The validator controls what keyboard
+  input is accepted when the user is editing in the value field. The
+  default is to use a suitable QIntValidator.
+
+  Use setValidator(0) to turn off input validation (entered input will
+  still be clamped to the range of the spinbox).
 */
 
-void QSpinBox::setValidator( QValidator* v )
+void QSpinBox::setValidator( const QValidator* v )
 {
     if ( vi )
 	vi->setValidator( v );
 }
 
 
+/*!  
+  Returns the validator which constrains editing for this spin box if
+  there is any, or else 0.
+
+  \sa setValidator() QValidator
+*/
+
+const QValidator * QSpinBox::validator() const
+{
+    return vi ? vi->validator() : 0;
+}
+
 /*!
   Updates the contents of the embedded QLineEdit to reflect current
   value, using mapValueToText(). Also enables/disables the push
@@ -632,7 +684,7 @@ void QSpinBox::interpretText()
     bool ok = TRUE;
     bool done = FALSE;
     int newVal = 0;
-    if ( specialValueText() ) {
+    if ( !specialValueText().isEmpty() ) {
 	QString s = QString(text()).stripWhiteSpace();
 	QString t = QString(specialValueText()).stripWhiteSpace();
 	if ( s == t ) {
@@ -737,7 +789,7 @@ int QSpinBox::mapTextToValue( bool* ok )
 {
     QString s = text();
     int newVal = s.toInt( ok );
-    if ( !(*ok) && ( prefix() || suffix() ) ) {	// Try removing any pre/suffix
+    if ( !(*ok) && !( !prefix() && !suffix() ) ) {// Try removing any pre/suffix
 	s = cleanText();
 	newVal = s.toInt( ok );
     }
@@ -753,10 +805,9 @@ int QSpinBox::mapTextToValue( bool* ok )
 QString QSpinBox::currentValueText()
 {
     QString s;
-    if ( (value() == minValue()) && specialValueText() ) {
+    if ( (value() == minValue()) && !specialValueText().isEmpty() ) {
 	s = specialValueText();
-    }
-    else {
+    } else {
 	s = prefix();
 	s.append( mapValueToText( value() ) );
 	s.append( suffix() );
@@ -765,61 +816,33 @@ QString QSpinBox::currentValueText()
 }
 
 
+
 /*!
-  Reimplemented for internal purposes.
+  \reimp
 */
 
-void QSpinBox::paletteChange( const QPalette& )
+void QSpinBox::setEnabled( bool on )
 {
-    vi->setPalette( palette() );
-    up->setPalette( palette() );
-    down->setPalette( palette() );
+    if ( on == isEnabled() )
+	return;
+    QFrame::setEnabled( on );
+    vi->setEnabled( on );
+    updateDisplay();
     update();
 }
 
 
-/*!
-  Reimplemented for internal purposes.
-*/
 
-void QSpinBox::enabledChange( bool )
+/*! \reimp */
+
+void QSpinBox::styleChange( QStyle& old )
 {
-    if ( !isEnabled() ) {
-	vi->setEnabled( FALSE );
-	up->setEnabled( FALSE );
-	down->setEnabled( FALSE );
-    }
-    else {
-	vi->setEnabled( TRUE );
-	updateDisplay();
-    }
-    update();
-}
-
-
-/*!
-  Reimplemented for internal purposes.
-*/
-
-void QSpinBox::fontChange( const QFont& )
-{
-    vi->setFont( font() );
-}
-
-
-/*!
-  Reimplemented for internal purposes.
-*/
-
-void QSpinBox::styleChange( GUIStyle )
-{
-    vi->setStyle( style() );
-    up->setStyle( style() );
-    down->setStyle( style() );
+//#####    vi->setStyle( style() );
+//#####    up->setStyle( style() );
+//#####    down->setStyle( style() );
     if ( style() == WindowsStyle )
 	setFrameStyle( WinPanel | Sunken );
     else
 	setFrameStyle( Panel | Sunken );
-    resizeEvent( 0 );
-    update();
+    QFrame::styleChange( old );
 }
