@@ -1,35 +1,44 @@
 /****************************************************************************
-** $Id: qtabbar.cpp,v 2.29 1997/09/26 10:57:41 eiriken Exp $
+** $Id: qtabbar.cpp,v 2.77 1999/06/22 12:03:57 paul Exp $
 **
 ** Implementation of QTabBar class
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.31.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qtabbar.h"
-#include "qkeycode.h"
 #include "qaccel.h"
+#include "qbitmap.h"
 
 #include <ctype.h>
 
-RCSTAG("$Id: qtabbar.cpp,v 2.29 1997/09/26 10:57:41 eiriken Exp $");
+/*!
+  \class QTab qtabbar.h
+  \brief The structures in a QTabBar.
 
+  For custom QTabBar tab headings.
+
+  \sa QTabBar
+*/
 
 QTab::~QTab()
 {
-    // gcc says: just give me a vtbl and I'll be satisfied
+    delete iconset;
 }
 
 // this struct can be expanded without breaking binary compatibility
@@ -48,6 +57,8 @@ struct QTabPrivate {
   \brief The QTabBar class provides a tab bar, for use in e.g. tabbed
   dialogs.
 
+  \ingroup realwidgets
+
   The class is quite simple; it draws the tabs in one of four shapes
   and emits a signal when one is selected.  It can be subclassed to
   tailor the look and feel.
@@ -85,7 +96,7 @@ struct QTabPrivate {
   addTab() creates a new tab and adds it to the bar. <li> selectTab()
   decides which, if any, tab the user selects with the mouse. </ul>
 
-  <img src=qtabbar-m.gif> <img src=qtabbar-w.gif>
+  <img src=qtabbar-m.png> <img src=qtabbar-w.png>
 */
 
 /*!
@@ -100,10 +111,10 @@ struct QTabPrivate {
 */
 
 /*!
-  Create a new, empty tab bar.
+  Creates a new, empty tab bar.
 */
 
-QTabBar::QTabBar( QWidget * parent, const char * name )
+QTabBar::QTabBar( QWidget * parent, const char *name )
     : QWidget( parent, name )
 {
     d = new QTabPrivate;
@@ -111,8 +122,9 @@ QTabBar::QTabBar( QWidget * parent, const char * name )
     d->focus = 0;
     d->a = new QAccel( this, "tab accelerators" );
     d->s = RoundedAbove;
-    l = new QListT<QTab>;
-    l->setAutoDelete( TRUE );
+    l = new QList<QTab>;
+    lstatic = new QList<QTab>;
+    lstatic->setAutoDelete( TRUE );
     setFocusPolicy( TabFocus );
 
     connect( d->a, SIGNAL(activated(int)), this, SLOT(setCurrentTab(int)) );
@@ -120,53 +132,79 @@ QTabBar::QTabBar( QWidget * parent, const char * name )
 
 
 /*!
-  Delete the tab control and free the memory it used.
+  Deletes the tab control and free the memory it used.
 */
 
 QTabBar::~QTabBar()
 {
     delete d;
+    d = 0;
     delete l;
+    l = 0;
+    delete lstatic;
+    lstatic = 0;
 }
 
 
 /*!
-  Add \a newTab to the tab control.
+  Adds \a newTab to the tab control.
 
-  Allocate a new id, set t's id, locate it just to the right of the
-  existing tabs, insert an accelerator if the tab's label contains the
-  string "&p" for some value of p, add it to the bar, and return the
+  Allocates a new id, sets \a newTab's id, locates it just to the right of the
+  existing tabs, inserts an accelerator if the tab's label contains the
+  string "&p" for some value of p, adds it to the bar, and returns the
   newly allocated id.
 */
 
 int QTabBar::addTab( QTab * newTab )
 {
-    QFontMetrics fm = fontMetrics();
-    QTab  *t = l->first();
-    int lw = fm.width( newTab->label );
-    if ( t ) {
-	QRect r( t->r );
-	while ( (t = l->next()) != 0 )
-	    r = r.unite( t->r );
-	newTab->r.setRect( r.right()-3, 0, lw + 24,
-			   QMAX( r.height(), fm.height() + 10 ) );
-    } else {
-	newTab->r.setRect( 0, 0, lw + 24, fm.height() + 10 );
-    }
+    return insertTab( newTab );
+}
 
+
+/*!
+  Inserts \a newTab to the tab control.
+
+  If \a index is not specified, the tab is simply added. Otherwise
+  it's inserted at the specified position.
+
+  Allocates a new id, sets \a newTab's id, locates it respectively,
+  inserts an accelerator if the tab's label contains the string "&p"
+  for some value of p, adds it to the bar, and returns the newly
+  allocated id.
+*/
+
+int QTabBar::insertTab( QTab * newTab, int index )
+{
     newTab->id = d->id++;
-    l->append( newTab );
+    l->insert( 0, newTab );
+    if ( index < 0 || index > int(lstatic->count()) )
+	lstatic->append( newTab );
+    else
+	lstatic->insert( index, newTab );
 
-    const char * p = strchr( newTab->label, '&' );
-    while( p && *p && p[1] == '&' )
-	p = strchr( p+2, '&' );
-    if ( p && *p && isalpha(p[1]) )
-	d->a->insertItem( ALT + toupper(p[1]), newTab->id );
+    layoutTabs();
+
+    int p = QAccel::shortcutKey( newTab->label );
+    if ( p )
+	d->a->insertItem( p, newTab->id );
 
     return newTab->id;
 }
 
 
+/*!
+  Removes \a tab from the tab control.
+*/
+void QTabBar::removeTab( QTab * tab )
+{
+    //#### accelerator labels??
+    l->remove( tab );
+    lstatic->remove( tab );
+    layoutTabs();
+    repaint();
+}
+
+
 /*!
   Enable tab \a id if \a enable is TRUE, or disable it if \a enable is
   FALSE.  If \a id is currently selected, setTabEnabled() makes
@@ -200,7 +238,7 @@ void QTabBar::setTabEnabled( int id, bool enabled )
 				       (p2.y() - p1.y())*(p2.y() - p1.y());
 			    if ( distance < m ) {
 				t = n;
-				distance = m;
+				m = distance;
 			    }
 			}
 		    }
@@ -210,6 +248,7 @@ void QTabBar::setTabEnabled( int id, bool enabled )
 			emit selected( t->id );
 		    }
 		}
+		updateMask();
 		repaint( r );
 	    }
 	    return;
@@ -219,7 +258,7 @@ void QTabBar::setTabEnabled( int id, bool enabled )
 
 
 /*!
-  Return TRUE if the tab with id \a id is enabled, or FALSE if it
+  Returns TRUE if the tab with id \a id is enabled, or FALSE if it
   is disabled or there is no such tab.
 
   \sa setTabEnabled()
@@ -255,6 +294,17 @@ QSize QTabBar::sizeHint() const
 }
 
 
+/*!
+  Specifies that this widget can use more, but is able to survive on
+  less, horizontal space; and has a fixed height.
+*/
+
+QSizePolicy QTabBar::sizePolicy() const
+{
+    return QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed );
+}
+
+
 /*!  Paint the single tab \a t using \a p.  If and only if \a selected
   is TRUE, \a t is currently selected.
 
@@ -266,102 +316,19 @@ QSize QTabBar::sizeHint() const
 void QTabBar::paint( QPainter * p, QTab * t, bool selected ) const
 {
     QRect r( t->r );
-    if ( d->s == RoundedAbove ) {
-	if ( selected ) {
-	    p->setPen( colorGroup().background() );
-	    p->drawLine( r.left()+1, r.bottom(), r.right()-2, r.bottom() );
-	    p->drawLine( r.left()+1, r.bottom(), r.left()+1, r.top()+2 );
-	    p->setPen( colorGroup().light() );
-	    QFont bold( font() );
-	    bold.setWeight( QFont::Bold );
-	    p->setFont( bold );
-	} else {
-	    p->setPen( colorGroup().light() );
-	    p->drawLine( r.left(), r.bottom(), r.right(), r.bottom() );
-	    r.setRect( r.left() + 2, r.top() + 2,
-		       r.width() - 4, r.height() - 2 );
-	    p->setFont( font() );
-	}
-    
-	p->drawLine( r.left(), r.bottom(), r.left(), r.top() + 2 );
-	p->drawPoint( r.left()+1, r.top() + 1 );
-	p->drawLine( r.left()+2, r.top(),
-		     r.right() - 2, r.top() );
 
-	p->setPen( colorGroup().dark() );
-	p->drawLine( r.right() - 1, r.top() + 2,
-		     r.right() - 1, r.bottom() - 1 ); 
-	p->setPen( colorGroup().foreground() );
-	p->drawPoint( r.right() - 1, r.top() + 1 );
-	p->drawLine( r.right(), r.top() + 2, r.right(), r.bottom() - 1 ); 
-    } else if ( d->s == RoundedBelow ) {
-	if ( selected ) {
-	    p->setPen( colorGroup().background() );
-	    p->drawLine( r.left()+1, r.top(), r.right()-2, r.top() );
-	    p->drawLine( r.left()+1, r.top(), r.left()+1, r.bottom()-2 );
-	    p->setPen( colorGroup().dark() );
-	    QFont bold( font() );
-	    bold.setWeight( QFont::Bold );
-	    p->setFont( bold );
-	} else {
-	    p->setPen( colorGroup().dark() );
-	    p->drawLine( r.left(), r.top(), r.right(), r.top() );
-	    r.setRect( r.left() + 2, r.top(),
-		       r.width() - 4, r.height() - 2 );
-	    p->setFont( font() );
-	}
+    style().drawTab( p, this, t, selected );
 
-	p->drawLine( r.right() - 1, r.top(),
-		     r.right() - 1, r.bottom() - 2 );
-	p->drawPoint( r.right() - 2, r.bottom() - 2 );
-	p->drawLine( r.right() - 2, r.bottom() - 1,
-		     r.left() + 1, r.bottom() - 1 );
-	p->drawPoint( r.left() + 1, r.bottom() - 2 );
-    
-	p->setPen( colorGroup().foreground() );
-	p->drawLine( r.right(), r.top(),
-		     r.right(), r.bottom() - 1 );
-	p->drawPoint( r.right() - 1, r.bottom() - 1 );
-	p->drawLine( r.right() - 1, r.bottom(),
-		     r.left() + 2, r.bottom() );
+    p->setFont( font() );
 
-	p->setPen( colorGroup().light() );
-	p->drawLine( r.left(), r.top(),
-		     r.left(), r.bottom() - 2 );
-    } else {
-	// triangular, above or below
-	int y;
-	int x;
-	QPointArray a( 10 );
-	a.setPoint( 0, 0, -1 );
-	a.setPoint( 1, 0, 0 );
-	y = t->r.height()-2;
-	x = y/3;
-	a.setPoint( 2, x++, y-1 );
-	a.setPoint( 3, x++, y );
-	a.setPoint( 3, x++, y++ );
-	a.setPoint( 4, x, y );
-
-	int i;
-	int right = t->r.width() - 1;
-	for ( i = 0; i < 5; i++ )
-	    a.setPoint( 9-i, right - a.point( i ).x(), a.point( i ).y() );
-
-	if ( d->s == TriangularAbove )
-	    for ( i = 0; i < 10; i++ )
-		a.setPoint( i, a.point(i).x(),
-			    t->r.height() - 1 - a.point( i ).y() );
-
-	a.translate( t->r.left(), t->r.top() );
-
-	p->setBrush( colorGroup().background() );
-	p->setPen( colorGroup().foreground() );
-	p->drawPolygon( a );
-	p->setBrush( NoBrush );
+    int iw = 0;
+    int ih = 0;
+    if ( t->iconset != 0 ) {
+	iw = t->iconset->pixmap( QIconSet::Small, QIconSet::Normal ).width() + 2;
+	ih = t->iconset->pixmap( QIconSet::Small, QIconSet::Normal ).height();
     }
-
-    int w = p->fontMetrics().width( t->label ) + 4;
-    int h = p->fontMetrics().height() + 4;
+    int w = iw + p->fontMetrics().width( t->label ) + 4;
+    int h = QMAX(p->fontMetrics().height() + 4, ih );
     paintLabel( p, QRect( r.left() + (r.width()-w)/2 - 3,
 			  r.top() + (r.height()-h)/2,
 			  w, h ), t, t->id == keyboardFocusTab() );
@@ -369,23 +336,35 @@ void QTabBar::paint( QPainter * p, QTab * t, bool selected ) const
 
 /*!
   Paints the label of tab \a t centered in rectangle \a br using
-  painter \a p and draws a focus indication iff \a has_focus is TRUE.
-
-  In Qt 2.0, this will be virtual.
+  painter \a p and draws a focus indication if \a has_focus is TRUE.
 */
 
 void QTabBar::paintLabel( QPainter* p, const QRect& br,
                           QTab* t, bool has_focus ) const
 {
-    if ( t->enabled ) {
+
+    QRect r = br;
+    if ( t->iconset) {
+	// the tab has an iconset, draw it in the right mode
+	QIconSet::Mode mode = t->enabled?QIconSet::Normal:QIconSet::Disabled;
+	if ( mode == QIconSet::Normal && has_focus )
+	    mode = QIconSet::Active;
+	QPixmap pixmap = t->iconset->pixmap( QIconSet::Small, mode );
+	int pixw = pixmap.width();
+	int pixh = pixmap.height();
+	r.setLeft( r.left() + pixw + 2 );
+	p->drawPixmap( br.left()+2, br.center().y()-pixh/2, pixmap );
+    }
+
+   if ( t->enabled ) {
 	p->setPen( palette().normal().text() );
-	p->drawText( br, AlignCenter | ShowPrefix, t->label );
+	p->drawText( r, AlignCenter | ShowPrefix, t->label );
     } else if ( style() == MotifStyle ) {
 	p->setPen( palette().disabled().text() );
 	p->drawText( br, AlignCenter | ShowPrefix, t->label );
     } else { // Windows style, disabled
 	p->setPen( colorGroup().light() );
-	QRect wr = br;
+	QRect wr = r;
 	wr.moveBy( 1, 1 );
 	p->drawText( wr, AlignCenter | ShowPrefix, t->label );
 	p->setPen( palette().disabled().text() );
@@ -402,6 +381,39 @@ void QTabBar::paintLabel( QPainter* p, const QRect& br,
 }
 
 
+/*!
+  Draws the mask for this tab bar.
+
+  \internal
+  This is not totally right - a few corner pixels missing.
+*/
+
+void  QTabBar::updateMask()
+{
+    if ( !autoMask() )
+	return;
+    QBitmap bm( size() );
+    bm.fill( color0 );
+
+    QPainter p;
+    p.begin( &bm, this );
+    p.setBrush(color1);
+    p.setPen(color1);
+
+    QTab * t;
+    t = l->first();
+    do {
+	QTab * n = l->next();
+	if ( t )
+	    style().drawTabMask( &p, this, t, n == 0 );
+	t = n;
+    } while ( t != 0 );
+
+
+    p.end();
+    setMask( bm );
+}
+
 /*!
   Repaints the tab row.  All the painting is done by paint();
   paintEvent() only decides which tabs need painting and in what
@@ -412,11 +424,12 @@ void QTabBar::paintLabel( QPainter* p, const QRect& br,
 
 void QTabBar::paintEvent( QPaintEvent * e )
 {
-    QPainter p;
+    QPainter p( this );
 
-    p.begin( this );
-    p.setClipping( TRUE );
-    p.setClipRect( e->rect() );
+    if (backgroundPixmap() ) {
+	p.setBrushOrigin( rect().bottomLeft() );
+	p.fillRect(0, 0, width(), height(), QBrush( colorGroup().background(), *backgroundPixmap() ));
+    }
 
     QTab * t;
     t = l->first();
@@ -427,7 +440,6 @@ void QTabBar::paintEvent( QPaintEvent * e )
 	t = n;
     } while ( t != 0 );
 
-    p.end();
 }
 
 
@@ -489,6 +501,7 @@ void QTabBar::mouseReleaseEvent( QMouseEvent * e )
 	    l->append( l->take() );
 
 	d->focus = t->id;
+	updateMask();
 	if ( t->r.intersects( r ) ) {
 	    repaint( r.unite( t->r ) );
 	} else {
@@ -506,8 +519,6 @@ void QTabBar::mouseReleaseEvent( QMouseEvent * e )
 void QTabBar::show()
 {
     QTab * t = l->last();
-    if ( !isVisible() && (t = l->first()) != 0 )
-	l->append( l->take() );
     QWidget::show();
     if ( t )
 	emit selected( t->id );
@@ -521,7 +532,7 @@ void QTabBar::show()
   the user can see the relevant page, or that the tab \link
   isTabEnabled() is enabled.\endlink
 
-  When when you need to display something, the return value from this
+  When you need to display something, the return value from this
   function represents the best page to display.  That's all.
 
   \sa selected()
@@ -535,7 +546,7 @@ int QTabBar::currentTab() const
 }
 
 
-/*!  "Raises" the tab with ID \a id and emits the selected() signal.
+/*! Raises the tab with ID \a id and emits the selected() signal.
 
   \sa currentTab() selected() tab()
 */
@@ -546,7 +557,7 @@ void QTabBar::setCurrentTab( int id )
 }
 
 
-/*! "Raises" \a tab and emits the selected() signal unless the tab was
+/*! Raises \a tab and emits the selected() signal unless the tab was
   already current.
 
   \sa currentTab() selected()
@@ -563,6 +574,7 @@ void QTabBar::setCurrentTab( QTab * tab )
 	    l->append( l->take() );
 
 	d->focus = tab->id;
+	updateMask();
 	if ( tab->r.intersects( r ) ) {
 	    repaint( r.unite( tab->r ) );
 	} else {
@@ -585,7 +597,7 @@ int QTabBar::keyboardFocusTab() const
 
 /*!  Handles the tab bar's keyboard interface (if enabled).
 
-  The right and left arrow keys move a selector, the space bar makes
+  The right and left arrow keys move a selector, the spacebar makes
   the tab with the selector active.  All other keys are ignored.
 */
 
@@ -596,34 +608,32 @@ void QTabBar::keyPressEvent( QKeyEvent * e )
     if ( e->key() == Key_Left ) {
 	// left - skip past any disabled ones
 	if ( d->focus > 0 ) {
-	    QTab * t;
+	    QTab * t = lstatic->last();
+	    while ( t && t->id != d->focus )
+		t = lstatic->prev();
 	    do {
-		d->focus--;
-		t = l->first();
-		while ( t && t->id != d->focus )
-		    t = l->next();
-	    } while ( d->focus >= 0 && t && !t->enabled );
+		t = lstatic->prev();
+	    } while ( t && !t->enabled);
+	    if (t)
+		d->focus = t->id;
 	}
 	if ( d->focus < 0 )
 	    d->focus = old;
-	e->accept();
     } else if ( e->key() == Key_Right ) {
-	// right - ditto
-	if ( d->focus < d->id-1 ) {
-	    QTab * t;
-	    do {
-		d->focus++;
-		t = l->first();
-		while ( t && t->id != d->focus )
-		    t = l->next();
-	    } while ( d->focus < d->id && t && !t->enabled );
-	}
-	if ( d->focus >= d->id )
-	    d->focus = old;
-	e->accept();
+	QTab * t = lstatic->first();
+	while ( t && t->id != d->focus )
+	    t = lstatic->next();
+	do {
+	    t = lstatic->next();
+	} while ( t && !t->enabled);
+    if (t)
+	d->focus = t->id;
+    if ( d->focus >= d->id )
+	d->focus = old;
     } else {
 	// other keys - ignore
 	e->ignore();
+	return;
     }
 
     // if the focus moved, repaint and signal
@@ -637,7 +647,7 @@ void QTabBar::keyPressEvent( QKeyEvent * e )
 	if ( t ) {
 	    l->append( l->take() );
 	    r = r.unite( t->r );
-	    e->accept();
+	    updateMask();
 	    if ( r.isValid() )
 		repaint( r );
 	    emit selected( t->id );
@@ -662,7 +672,7 @@ QTab * QTabBar::tab( int id )
 /*!
   The list of QTab objects added.
 */
-QListT<QTab> * QTabBar::tabList()
+QList<QTab> * QTabBar::tabList()
 {
     return l;
 }
@@ -681,9 +691,46 @@ QTabBar::Shape QTabBar::shape() const
 
 void QTabBar::setShape( Shape s )
 {
-    if ( !d && d->s == s )
+    if ( !d || d->s == s )
 	return;
-
+    //######### must recalculate heights
     d->s = s;
+    updateMask();
     repaint();
 }
+
+
+
+/*!
+  Layout all existing tabs (i.e. setting their r attribute ) according
+  to their label and their iconset.
+ */
+void QTabBar::layoutTabs()
+{
+    if ( l->isEmpty() )
+	return;
+
+    QTab* t = lstatic->first();
+    QRect r( t->r );
+    while ( (t = lstatic->next()) != 0 )
+	r = r.unite( t->r );
+
+    int hframe, vframe, overlap;
+    style().tabbarMetrics( this, hframe, vframe, overlap );
+    QFontMetrics fm = fontMetrics();
+    int x = 0;
+    for ( t = lstatic->first(); t; t = lstatic->next() ) {
+	int lw = fm.width( t->label );
+	int iw = 0;
+	int ih = 0;
+	if ( t->iconset != 0 ) {
+	    iw = t->iconset->pixmap( QIconSet::Small, QIconSet::Normal ).width() + 2;
+	    ih = t->iconset->pixmap( QIconSet::Small, QIconSet::Normal ).height();
+	}
+	int h = QMAX( fm.height(), ih );
+
+	h += vframe;
+	t->r.setRect( x, 0, lw + hframe + iw, QMAX(r.height(), h) );
+	x += t->r.width() - overlap;
+    }
+}
