@@ -1,23 +1,25 @@
 /**********************************************************************
-** $Id: qtableview.cpp,v 2.51 1998/07/06 11:22:18 aavit Exp $
+** $Id: qtableview.cpp,v 2.89 1999/06/21 23:50:24 agulbra Exp $
 **
 ** Implementation of QTableView class
 **
 ** Created : 941115
 **
-** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.40.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -43,26 +45,26 @@ enum ScrollBarDirtyFlags {
 };
 
 
-class CornerSquare : public QWidget		// internal class
+class QCornerSquare : public QWidget		// internal class
 {
 public:
-    CornerSquare( QWidget *, const char * = 0 );
+    QCornerSquare( QWidget *, const char* = 0 );
     void paintEvent( QPaintEvent * );
 };
 
-CornerSquare::CornerSquare( QWidget *parent, const char *name )
+QCornerSquare::QCornerSquare( QWidget *parent, const char *name )
 	: QWidget( parent, name )
 {
 }
 
-void CornerSquare::paintEvent( QPaintEvent * )
+void QCornerSquare::paintEvent( QPaintEvent * )
 {
 }
 
 
 /*!
   \class QTableView qtableview.h
-  \brief This is the abstract base class of all the table views.
+  \brief This is an abstract base class for implementing tables
 
   \ingroup abstractwidgets
 
@@ -72,7 +74,7 @@ void CornerSquare::paintEvent( QPaintEvent * )
   column 0.
 
   The behavior of the widget can be finely tuned using
-  setTableFlags(); subclasses like QListBox are little more than a
+  setTableFlags(); a typical subclass will consist of little more than a
   call to setTableFlags(), some table content manipulation, and an
   implementation of paintCell().  Subclasses that need cells with
   variable width or height must reimplement cellHeight() and/or
@@ -107,14 +109,19 @@ void CornerSquare::paintEvent( QPaintEvent * )
   the opposite order of all pixel operations, which take first the
   horizontal (x), then the vertical (y).
 
-  <img src=qtablevw-m.gif> <img src=qtablevw-w.gif>
+  <img src=qtablevw-m.png> <img src=qtablevw-w.png>
 
-  \warning in the current (and previous) version of the library, the
-  functions setNumRows(), setNumCols(), setCellHeight(),
+  \warning the functions setNumRows(), setNumCols(), setCellHeight(),
   setCellWidth(), setTableFlags() and clearTableFlags() may cause
-  virtual functions like cellWidth() and cellHeight() to be called, even
-  if autoUpdate() is FALSE.
-  This may cause errors if relevant state variables are not initialized.
+  virtual functions like cellWidth() and cellHeight() to be called,
+  even if autoUpdate() is FALSE.  This may cause errors if relevant
+  state variables are not initialized.
+
+  \warning Experience has shown that use of this widget tends to bring
+  more bugs than expected, and our analysis indicates that widget's
+  very flexibility is the problem.  If QScrollView or QListBox can
+  easily be made to do the job you need, we recommend subclassing
+  those widgets rather than QTableView.
 
   \sa QScrollView
   <a href="guibooks.html#fowler">GUI Design Handbook: Table</a>
@@ -133,7 +140,7 @@ void CornerSquare::paintEvent( QPaintEvent * )
   cell width\endlink are set to 0.
 
   Frame line shapes (QFrame::HLink and QFrame::VLine) are disallowed,
-  see QFrame::setStyle().
+  see QFrame::setFrameStyle().
 
   Note that the \a f argument is \e not \link setTableFlags() table
   flags \endlink but rather \link QWidget::QWidget() widget
@@ -241,14 +248,17 @@ void QTableView::show()
 
 void QTableView::repaint( int x, int y, int w, int h, bool erase )
 {
-    if ( !isVisible() )				// ignore if not visible
+    if ( !isVisible() || testWState(WState_BlockUpdates) )
 	return;
     if ( w < 0 )
 	w = width()  - x;
     if ( h < 0 )
 	h = height() - y;
-    QPaintEvent e( QRect(x,y,w,h) );
-    if ( erase )
+    QRect r( x, y, w, h );
+    if ( r.isEmpty() )
+	return; // nothing to do
+    QPaintEvent e( r );
+    if ( erase && backgroundMode() != NoBackground )
 	eraseInPaint = TRUE;			// erase when painting
     paintEvent( &e );
     eraseInPaint = FALSE;
@@ -278,19 +288,24 @@ void QTableView::setNumRows( int rows )
 {
     if ( rows < 0 ) {
 #if defined(CHECK_RANGE)
-	warning( "QTableView::setNumRows: (%s) Negative argument %d.",
+	qWarning( "QTableView::setNumRows: (%s) Negative argument %d.",
 		 name( "unnamed" ), rows );
 #endif
 	return;
     }
     if ( nRows == rows )
 	return;
-    int oldRows = nRows;
-    nRows = rows;
+
     if ( autoUpdate() && isVisible() ) {
-	int maxRow  = lastRowVisible();
-	if ( maxRow >= oldRows || maxRow >= nRows )
-	    repaint();
+	int oldLastVisible = lastRowVisible();
+	int oldTopCell = topCell();
+	nRows = rows;
+	if ( autoUpdate() && isVisible() &&
+	     ( oldLastVisible != lastRowVisible() || oldTopCell != topCell() ) )
+		repaint( oldTopCell != topCell() );
+    } else {
+	// Be more careful - if destructing, bad things might happen.
+	nRows = rows;
     }
     updateScrollBars( verRange );
 }
@@ -314,7 +329,7 @@ void QTableView::setNumCols( int cols )
 {
     if ( cols < 0 ) {
 #if defined(CHECK_RANGE)
-	warning( "QTableView::setNumCols: (%s) Negative argument %d.",
+	qWarning( "QTableView::setNumCols: (%s) Negative argument %d.",
 		 name( "unnamed" ), cols );
 #endif
 	return;
@@ -491,7 +506,7 @@ void QTableView::setOffset( int x, int y, bool updateScrBars )
 	}
     } else {
 	int xn=0, xcd=0, col = 0;
-	while ( col < nCols && x >= xn+(xcd=cellWidth(col)) ) {
+	while ( col < nCols-1 && x >= xn+(xcd=cellWidth(col)) ) {
 	    xn += xcd;
 	    col++;
 	}
@@ -516,7 +531,7 @@ void QTableView::setOffset( int x, int y, bool updateScrBars )
 	yCellDelta  = (short)(y % cellH);
     } else {
 	int yn=0, yrd=0, row=0;
-	while ( row < nRows && y >= yn+(yrd=cellHeight(row)) ) {
+	while ( row < nRows-1 && y >= yn+(yrd=cellHeight(row)) ) {
 	    yn += yrd;
 	    row++;
 	}
@@ -580,7 +595,7 @@ void QTableView::setCellWidth( int cellWidth )
 	return;
 #if defined(CHECK_RANGE)
     if ( cellWidth < 0 || cellWidth > SHRT_MAX ) {
-	warning( "QTableView::setCellWidth: (%s) Argument out of range (%d)",
+	qWarning( "QTableView::setCellWidth: (%s) Argument out of range (%d)",
 		 name( "unnamed" ), cellWidth );
 	return;
     }
@@ -632,7 +647,7 @@ void QTableView::setCellHeight( int cellHeight )
 	return;
 #if defined(CHECK_RANGE)
     if ( cellHeight < 0 || cellHeight > SHRT_MAX ) {
-	warning( "QTableView::setCellHeight: (%s) Argument out of range (%d)",
+	qWarning( "QTableView::setCellHeight: (%s) Argument out of range (%d)",
 		 name( "unnamed" ), cellHeight );
 	return;
     }
@@ -647,7 +662,7 @@ void QTableView::setCellHeight( int cellHeight )
 /*!
   Returns the total width of the table in pixels.
 
-  This function is virtual and should be reimplmented by subclasses that
+  This function is virtual and should be reimplemented by subclasses that
   have variable cell widths and a non-trivial cellWidth() function, or a
   large number of columns in the table.
 
@@ -670,7 +685,7 @@ int QTableView::totalWidth()
 /*!
   Returns the total height of the table in pixels.
 
-  This function is virtual and should be reimplmented by subclasses that
+  This function is virtual and should be reimplemented by subclasses that
   have variable cell heights and a non-trivial cellHeight() function, or a
   large number of rows in the table.
 
@@ -925,10 +940,6 @@ void QTableView::clearTableFlags( uint f )
 
 void QTableView::setAutoUpdate( bool enable )
 {
-#if QT_VERSION == 200
-#error "Remove unused flag obsoleteUpd"
-#endif
-    obsoleteUpd = enable; //### binary compatibility contortions
     if ( isUpdatesEnabled() == enable )
 	return;
     setUpdatesEnabled( enable );
@@ -1070,7 +1081,7 @@ void QTableView::coverCornerSquare( bool enable )
 {
     coveringCornerSquare = enable;
     if ( !cornerSquare && enable ) {
-	cornerSquare = new CornerSquare( this );
+	cornerSquare = new QCornerSquare( this );
 	CHECK_PTR( cornerSquare );
 	cornerSquare->setGeometry( maxViewX() + frameWidth() + 1,
 				   maxViewY() + frameWidth() + 1,
@@ -1234,8 +1245,6 @@ void QTableView::verSbSlidingDone( )
   is started. It can be reimplemented by subclasses that want to
   to set up the painter in a special way and that do not want to
   do so for each cell.
-
-  The default implementation does nothing.
 */
 
 void QTableView::setupPainter( QPainter * )
@@ -1271,10 +1280,9 @@ void QTableView::setupPainter( QPainter * )
 void QTableView::paintEvent( QPaintEvent *e )
 {
     QRect updateR = e->rect();			// update rectangle
-#if DEBUG_INFO
-    debug("Update rect = ( %i, %i, %i, %i )",
-	   updateR.x(),updateR.y(), updateR.width(), updateR.height() );
-#endif
+
+    //debug("Update rect = ( %i, %i, %i, %i )",
+    //updateR.x(),updateR.y(), updateR.width(), updateR.height() );
 
     if ( sbDirty ) {
 	bool e = eraseInPaint;
@@ -1361,21 +1369,30 @@ void QTableView::paintEvent( QPaintEvent *e )
 	yPos = nextY;
     }
 
-    if ( eraseInPaint ) {
-	// If we are erasing while painting any areas in the view that
-	// are not covered by cells but are covered by the paint event
-	// rectangle these must be erased. We know that xPos is the last
-	// x pixel updated + 1 and that yPos is the last y pixel updated + 1.
-	if ( xPos > maxX + 1 )
-	    xPos = maxX + 1;
-	if ( yPos > maxY + 1 )
-	    yPos = maxY + 1;
-	if ( xPos <= maxX )	// erase to the right along full view
-	    paint.eraseRect( xPos, frameWidth(),
-			     maxX - xPos + 1, maxY - frameWidth() + 1 );
-	if ( yPos <= maxY )	// erase under cells updated above
-	    paint.eraseRect( frameWidth(), yPos,
-			     xPos - frameWidth(), maxY - yPos + 1 );
+    // while painting we have to erase any areas in the view that
+    // are not covered by cells but are covered by the paint event
+    // rectangle these must be erased. We know that xPos is the last
+    // x pixel updated + 1 and that yPos is the last y pixel updated + 1.
+
+    // Note that this needs to be done regardless whether we do
+    // eraseInPaint or not. Reason: a subclass (for example
+    // QMultiLineEdit) may implement flicker-freeness and encourage
+    // the use of repaint(FALSE). The subclass, however, cannot draw
+    // all pixels, just those inside the cells. So QTableView is
+    // reponsible for all pixels outside the cells.
+
+    QRect viewR = viewRect();
+
+    if ( xPos < maxX ) {
+	QRect r = viewR;
+	r.setLeft( xPos );
+	r.setBottom( yPos<maxY?yPos:maxY );
+	paint.eraseRect( r.intersect( updateR ) );
+    }
+    if ( yPos < maxY ) {
+	QRect r = viewR;
+	r.setTop( yPos );
+	paint.eraseRect( r.intersect( updateR ) );
     }
 }
 
@@ -1387,19 +1404,13 @@ void QTableView::paintEvent( QPaintEvent *e )
 
 void QTableView::resizeEvent( QResizeEvent * )
 {
-    bool update = autoUpdate();
-    setAutoUpdate( FALSE );
-    int maxX = maxXOffset();			// ### can be slow
-    int maxY = maxYOffset();
-    if ( xOffs > maxX )
-	setXOffset( maxX );
-    if ( yOffs > maxY )
-	setYOffset( maxY );
-    setAutoUpdate( update );
-    updateScrollBars( horSteps | horGeometry | horRange |
+    updateScrollBars( horValue | verValue | horSteps | horGeometry | horRange |
 		      verSteps | verGeometry | verRange );
     showOrHideScrollBars();
     updateFrameSize();
+    int maxX = QMIN( xOffs, maxXOffset() );			// ### can be slow
+    int maxY = QMIN( yOffs, maxYOffset() );
+    setOffset( maxX, maxY );
 }
 
 
@@ -1418,11 +1429,12 @@ void QTableView::updateView()
   values, use findRow() to translate to cell numbers.
 */
 
-const QScrollBar *QTableView::verticalScrollBar() const
+QScrollBar *QTableView::verticalScrollBar() const
 {
     QTableView *that = (QTableView*)this; // semantic const
     if ( !vScrollBar ) {
 	QScrollBar *sb = new QScrollBar( QScrollBar::Vertical, that );
+	sb->setCursor( arrowCursor );
 	sb->resize( sbDim, 16 ); // 16 is irrelevant
 	CHECK_PTR(sb);
 	sb->setTracking( FALSE );
@@ -1446,11 +1458,12 @@ const QScrollBar *QTableView::verticalScrollBar() const
   values, use findCol() to translate to cell numbers.
 */
 
-const QScrollBar *QTableView::horizontalScrollBar() const
+QScrollBar *QTableView::horizontalScrollBar() const
 {
     QTableView *that = (QTableView*)this; // semantic const
     if ( !hScrollBar ) {
 	QScrollBar *sb = new QScrollBar( QScrollBar::Horizontal, that );
+	sb->setCursor( arrowCursor );
 	sb->resize( 16, sbDim ); // 16 is irrelevant
 	sb->setFocusPolicy( NoFocus );
 	CHECK_PTR(sb);
@@ -1555,7 +1568,7 @@ int QTableView::findRawRow( int yPos, int *cellMaxY, int *cellMinY,
     if ( goOutsideView || yPos >= minViewY() && yPos <= maxViewY() ) {
 	if ( yPos < minViewY() ) {
 #if defined(CHECK_RANGE)
-	    warning( "QTableView::findRawRow: (%s) internal error: "
+	    qWarning( "QTableView::findRawRow: (%s) internal error: "
 		     "yPos < minViewY() && goOutsideView "
 		     "not supported. (%d,%d)",
 		     name( "unnamed" ), yPos, yOffs );
@@ -1602,7 +1615,7 @@ int QTableView::findRawCol( int xPos, int *cellMaxX, int *cellMinX ,
     if ( goOutsideView || xPos >= minViewX() && xPos <= maxViewX() ) {
 	if ( xPos < minViewX() ) {
 #if defined(CHECK_RANGE)
-	    warning( "QTableView::findRawCol: (%s) internal error: "
+	    qWarning( "QTableView::findRawCol: (%s) internal error: "
 		     "xPos < minViewX() && goOutsideView "
 		     "not supported. (%d,%d)",
 		     name( "unnamed" ), xPos, xOffs );
@@ -1762,7 +1775,11 @@ bool QTableView::colXPos( int col, int *xPos ) const
   Moves the visible area of the table rightwards by \e xPixels and
   downwards by \e yPixels pixels.  Both may be negative.
 
-  This function is \e not the same as QWidget::scroll().
+  \warning You might find that QScrollView offers a higher-level of
+	functionality than using QTableView and this function.
+
+  This function is \e not the same as QWidget::scroll(), in particular,
+  the signs of \a xPixels and \a yPixels have the reverse semantics.
 
   \sa setXOffset(), setYOffset(), setOffset(), setTopCell(),
   setLeftCell(), setTopLeftOffset()
@@ -1770,118 +1787,7 @@ bool QTableView::colXPos( int col, int *xPos ) const
 
 void QTableView::scroll( int xPixels, int yPixels )
 {
-    if ( xPixels == 0 && yPixels == 0 )
-	return;
-    if ( xPixels != 0 && yPixels != 0 ||
-	 QABS(xPixels) > viewWidth() || QABS(yPixels) > viewHeight() ) {
-	repaint();
-	return;
-    }
-
-    int xStart, yStart, width, height;
-
-    if ( xPixels != 0 ) {
-	yStart = minViewY();
-	height = viewHeight();
-	xStart = xPixels < 0 ? minViewX() : minViewX() + xPixels;
-
-	width = viewWidth() - QABS(xPixels);
-
-	// If we are cutting cells horizontally
-	// the rightmost limit for drawing will vary
-	// this makes scrolling tricky since we might
-	// have to erase an area no longer used for
-	// cell drawing.
-
-	int oldLim;
-#if 0
-	int newLim;
-#endif
-	if ( testTableFlags(Tbl_cutCellsH) ) {
-	    int maxX = maxViewX();
-	    int oldLastMaxX, oldLastMinX;
-	    int newLastMaxX, newLastMinX;
-	    findRawCol( maxX - xPixels,
-			&oldLastMaxX, &oldLastMinX, TRUE );
-	    findRawCol( maxX,&newLastMaxX, &newLastMinX, TRUE );
-	    oldLastMaxX += xPixels;
-	    oldLastMinX += xPixels;
-	    oldLim = oldLastMaxX <= maxX ? oldLastMaxX : oldLastMinX - 1;
-#if 0
-	    newLim = newLastMaxX <= maxX ? newLastMaxX : newLastMinX - 1;
-#endif
-	    width -= maxX - oldLim;
-
-	    bitBlt( this, xStart - xPixels, yStart,
-		    this, xStart, yStart, width, height );
-
-	    if ( xPixels < 0 )
-		repaint( minViewX(), yStart, -xPixels, height );
-	    else
-		repaint( minViewX() + width, yStart,
-			 maxX - minViewX() - width, height );
-	} else {
-	    bitBlt( this, xStart - xPixels, yStart,
-		    this, xStart, yStart, width, height );
-	    if ( xPixels < 0 )
-		repaint( minViewX(), yStart, -xPixels, height );
-	    else
-		repaint( minViewX() + width, yStart, xPixels, height );
-	}
-    }
-
-    if ( yPixels != 0 ) {
-	xStart = minViewY();
-	width  = viewWidth();
-	yStart = yPixels < 0 ? minViewY() : minViewY() + yPixels;
-
-	height = viewHeight() - QABS(yPixels);
-
-      // If we are cutting cells vertically
-      // the lower limit for drawing will vary
-      // this makes scrolling tricky since we might
-      // have to erase an area no longer used for
-      // cell drawing.
-
-	int oldLim;
-#if 0
-	int newLim;
-#endif
-	if ( testTableFlags(Tbl_cutCellsV) ) {
-	    int maxY = maxViewY();
-	    int oldLastMaxY, oldLastMinY;
-	    int newLastMaxY, newLastMinY;
-	    findRawRow( maxY - yPixels,
-			&oldLastMaxY, &oldLastMinY, TRUE );
-	    findRawRow( maxY,&newLastMaxY, &newLastMinY, TRUE );
-	    oldLastMaxY += yPixels;
-	    oldLastMinY += yPixels;
-	    oldLim = oldLastMaxY <= maxY ? oldLastMaxY : oldLastMinY - 1;
-#if 0
-	    newLim = newLastMaxY <= maxY ? newLastMaxY : newLastMinY - 1;
-#endif
-	    height -= maxY - oldLim;
-
-	    bitBlt( this, xStart, yStart - yPixels,
-		    this, xStart, yStart, width, height );
-
-	    if ( yPixels < 0 ) {
-		repaint( xStart, minViewY(), width, -yPixels );
-	    } else {
-		repaint( xStart, minViewY() + height,
-			 width, maxY - minViewY() - height );
-	    }
-	} else {
-	    bitBlt( this, xStart, yStart - yPixels,
-		    this, xStart, yStart, width, height );
-
-	    if ( yPixels < 0 ) {
-		repaint( xStart, minViewY(), width, -yPixels );
-	    } else {
-		repaint( xStart, minViewY() + height, width, yPixels );
-	    }
-	}
-    }
+    QWidget::scroll( -xPixels, -yPixels, contentsRect() );
 }
 
 
@@ -2073,7 +1979,7 @@ void QTableView::updateScrollBars( uint f )
 
 	if ( sbDirty & horSteps ) {
 	    if ( cellW )
-		hScrollBar->setSteps( cellW, viewWidth() );
+		hScrollBar->setSteps( QMIN(cellW,viewWidth()/2), viewWidth() );
 	    else
 		hScrollBar->setSteps( 16, viewWidth() );
 	}
@@ -2096,7 +2002,7 @@ void QTableView::updateScrollBars( uint f )
 
 	if ( sbDirty & verSteps ) {
 	    if ( cellH )
-		vScrollBar->setSteps( cellH, viewHeight() );
+		vScrollBar->setSteps( QMIN(cellH,viewHeight()/2), viewHeight() );
 	    else
 		vScrollBar->setSteps( 16, viewHeight() );  // fttb! ###
 	}
@@ -2131,11 +2037,15 @@ void QTableView::updateFrameSize()
 	rh = 0;
     int fh = frameRect().height();
     int fw = frameRect().width();
-    if ( rw > fw )
-	update( fw - frameWidth(), 0, frameWidth(), fh );
-    if ( rh > fh )
-	update( 0, fh - frameWidth(), fw, frameWidth() );
+
     setFrameRect( QRect(0,0,rw,rh) );
+
+    if ( autoUpdate() ) {
+	if ( rw != fw )
+	    repaint( QMIN(fw,rw) - frameWidth(), 0, frameWidth(), rh );
+	if ( rh != fh )
+	    repaint( 0, QMIN(fh,rh) - frameWidth(), rw, frameWidth() );
+    }
 }
 
 
@@ -2333,3 +2243,4 @@ void QTableView::updateTableSize()
 		      verSteps |  verRange );
     showOrHideScrollBars();
 }
+
