@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qtoolbutton.cpp,v 2.30.2.1 1999/02/01 10:44:11 hanord Exp $
+** $Id: qtoolbutton.cpp,v 2.56 1999/06/16 09:10:20 agulbra Exp $
 **
 ** Implementation of QToolButton class
 **
@@ -7,17 +7,19 @@
 **
 ** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of Qt Free Edition, version 1.45.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/free-license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** Please see http://www.troll.no/pricing.html for information about 
-** Qt Professional Edition, which is this same library but with a
-** license which allows creation of commercial/proprietary software.
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
@@ -41,7 +43,8 @@ class QToolButtonPrivate
 };
 
 
-/*! \class QToolButton qtoolbutton.h
+/*!
+  \class QToolButton qtoolbutton.h
 
   \brief The QToolButton class provides a push button whose appearance
   has been tailored for use in a QToolBar.
@@ -51,7 +54,7 @@ class QToolButtonPrivate
   This means that it implements the ridiculous Microsoft auto-raise
   feature using QIconSet.  Apart from that, it's pretty much like a
   QPushButton.  The two classes may at some point be merged.
-  
+
   \sa QPushButton QToolButton
   <a href="guibooks.html#fowler">GUI Design Handbook: Push Button</a>
 */
@@ -59,7 +62,7 @@ class QToolButtonPrivate
 
 /*!  Constructs an empty tool button. */
 
-QToolButton::QToolButton( QWidget * parent, const char * name )
+QToolButton::QToolButton( QWidget * parent, const char *name )
     : QButton( parent, name )
 {
     init();
@@ -79,6 +82,10 @@ void QToolButton::init()
     ubp = TRUE;
 
     s = 0;
+    son = 0;
+
+    setBackgroundMode( PaletteButton);
+    setFocusPolicy( NoFocus );
 }
 
 
@@ -90,10 +97,10 @@ void QToolButton::init()
   in object \a receiver, and returns the button.
 */
 
-QToolButton::QToolButton( const QPixmap & pm, const char * textLabel,
-			  const char * grouptext,
-			  QObject * receiver, const char * slot,
-			  QToolBar * parent, const char * name )
+QToolButton::QToolButton( const QPixmap &pm, const QString &textLabel,
+			  const QString &grouptext,
+			  QObject * receiver, const char *slot,
+			  QToolBar * parent, const char *name )
     : QButton( parent, name )
 {
     init();
@@ -108,8 +115,8 @@ QToolButton::QToolButton( const QPixmap & pm, const char * textLabel,
     } else {
 	setUsesBigPixmap( FALSE );
     }
-    if ( textLabel && *textLabel ) {
-	if ( grouptext && *grouptext )
+    if ( !textLabel.isEmpty() ) {
+	if ( !grouptext.isEmpty() )
 	    QToolTip::add( this, textLabel,
 			   parent->mainWindow()->toolTipGroup(), grouptext );
 	else
@@ -126,10 +133,10 @@ QToolButton::QToolButton( const QPixmap & pm, const char * textLabel,
   in object \a receiver, and returns the button.
 */
 
-QToolButton::QToolButton( QIconSet iconSet, const char * textLabel,
-			  const char * grouptext,
-			  QObject * receiver, const char * slot,
-			  QToolBar * parent, const char * name )
+QToolButton::QToolButton( const QIconSet& iconSet, const QString &textLabel,
+			  const QString& grouptext,
+			  QObject * receiver, const char *slot,
+			  QToolBar * parent, const char *name )
     : QButton( parent, name )
 {
     init();
@@ -144,8 +151,8 @@ QToolButton::QToolButton( QIconSet iconSet, const char * textLabel,
     } else {
 	setUsesBigPixmap( FALSE );
     }
-    if ( textLabel && *textLabel ) {
-	if ( grouptext && *grouptext )
+    if ( !textLabel.isEmpty() ) {
+	if ( !grouptext.isEmpty() )
 	    QToolTip::add( this, textLabel,
 			   parent->mainWindow()->toolTipGroup(), grouptext );
 	else
@@ -187,9 +194,10 @@ void QToolButton::setToggleButton( bool enable )
 
 QSize QToolButton::sizeHint() const
 {
-    int w, h;
+    int w = 0;
+    int h = 0;
 
-    if ( text() ) {
+    if ( !text().isNull()) {
 	w = fontMetrics().width( text() );
 	h = fontMetrics().height(); // boundingRect()?
     } else if ( usesBigPixmap() ) {
@@ -208,6 +216,16 @@ QSize QToolButton::sizeHint() const
 }
 
 
+/*!
+  Specifies that this widget may grow.
+*/
+
+QSizePolicy QToolButton::sizePolicy() const
+{
+    return QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
+}
+
+
 
 /* \fn bool QToolButton::usesBigPixmap() const
 
@@ -223,7 +241,7 @@ QSize QToolButton::sizeHint() const
 */
 
 
-/*! \fn const char * QToolButton::textLabel() const
+/*! \fn QString QToolButton::textLabel() const
 
   Returns the text label in use by this tool button, or 0.
 
@@ -236,7 +254,12 @@ QSize QToolButton::sizeHint() const
   if \a enable is TRUE, and to use the small ones else.
 
   QToolButton automatically connects this slot to the relevant signal
-  in the QMainWindow in which is resides.
+  in the QMainWindow in which it resides.  You're strongly urged to
+  use QMainWindow::setUsesBigPixmaps() instead.
+
+  \warning If you set some buttons (in a QMainWindow) to have big and
+  others small pixmaps, QMainWindow may have trouble getting the
+  geometry correct.
 */
 
 void QToolButton::setUsesBigPixmap( bool enable )
@@ -245,9 +268,7 @@ void QToolButton::setUsesBigPixmap( bool enable )
 	return;
 
     ubp = enable;
-
-    if ( parent() )
-	QApplication::postEvent( parent(), new QEvent( Event_LayoutHint ) );
+    updateGeometry();
 }
 
 
@@ -273,8 +294,7 @@ void QToolButton::setUsesTextLabel( bool enable )
 
     utl = enable;
 
-    if ( parent() )
-	QApplication::postEvent( parent(), new QEvent( Event_LayoutHint ) );
+    updateGeometry();
 }
 
 
@@ -326,7 +346,8 @@ void QToolButton::toggle()
 
 void QToolButton::drawButton( QPainter * p )
 {
-    if ( uses3D() || isOn() ) {
+    if ( uses3D() || (isOn()&&!son) ) {
+#if 0	
 	QPointArray a;
 	a.setPoints( 3, 0, height()-1, 0, 0, width()-1, 0 );
 	if ( isOn() && !isDown() && !uses3D() ) {
@@ -351,6 +372,10 @@ void QToolButton::drawButton( QPainter * p )
 		   ? colorGroup().light()
 		   : colorGroup().dark() );
 	p->drawPolyline( a );
+#else
+	style().drawBevelButton( p, 0, 0, width(), height(), colorGroup(), (isOn()&&!son)||isDown(),
+				&colorGroup().brush( QColorGroup::Button ) );
+#endif	
     }
     drawButtonLabel( p );
 
@@ -372,42 +397,47 @@ void QToolButton::drawButton( QPainter * p )
 
 void QToolButton::drawButtonLabel( QPainter * p )
 {
-    if ( text() ) {
-	qDrawItem( p, style(), 1, 1, width()-2, height()-2,
-		   AlignCenter + ShowPrefix,
-		   colorGroup(), isEnabled(),
-		   0, text() );
+    int sx = 0;
+    int sy = 0;
+    if (isDown() || (isOn()&&!son) )
+	style().getButtonShift(sx, sy);
+    if ( !text().isNull() ) {
+	
+	style().drawItem( p, 1 + sx, 1 + sy, width()-2, height()-2,
+			  AlignCenter + ShowPrefix,
+			  colorGroup(), isEnabled(),
+			  0, text() );
     } else {
 
 	QPixmap pm;
 	if ( usesBigPixmap() ) {
 	    if ( !isEnabled() )
-		pm = iconSet().pixmap( QIconSet::Large, QIconSet::Disabled );
+		pm = iconSet( isOn() ).pixmap( QIconSet::Large, QIconSet::Disabled );
 	    else if ( uses3D() )
-		pm = iconSet().pixmap( QIconSet::Large, QIconSet::Active );
+		pm = iconSet( isOn() ).pixmap( QIconSet::Large, QIconSet::Active );
 	    else
-		pm = iconSet().pixmap( QIconSet::Large, QIconSet::Normal );
+		pm = iconSet( isOn() ).pixmap( QIconSet::Large, QIconSet::Normal );
 	} else {
 	    if ( !isEnabled() )
-		pm = iconSet().pixmap( QIconSet::Small, QIconSet::Disabled );
+		pm = iconSet( isOn() ).pixmap( QIconSet::Small, QIconSet::Disabled );
 	    else if ( uses3D() )
-		pm = iconSet().pixmap( QIconSet::Small, QIconSet::Active );
+		pm = iconSet( isOn() ).pixmap( QIconSet::Small, QIconSet::Active );
 	    else
-		pm = iconSet().pixmap( QIconSet::Small, QIconSet::Normal );
+		pm = iconSet( isOn() ).pixmap( QIconSet::Small, QIconSet::Normal );
 	}
 
 	if ( usesTextLabel() ) {
 	    int fh = fontMetrics().height();
-	    qDrawItem( p, style(), 1, 1, width()-2, height() - 2 - fh - 6,
-		       AlignCenter, colorGroup(), TRUE, &pm, 0 );
+	    style().drawItem( p, 1 + sx, 1 + sy, width()-2, height() - 2 - fh - 6,
+			      AlignCenter, colorGroup(), TRUE, &pm, QString::null );
 	    p->setFont( font() );
-	    qDrawItem( p, style(), 1, height() - 4 - fh, width()-2, fh,
-		       AlignCenter + ShowPrefix,
-		       colorGroup(), isEnabled(),
-		       0, textLabel() );
+	    style().drawItem( p, 1 + sx, height() - 4 - fh + sy, width()-2, fh,
+			      AlignCenter + ShowPrefix,
+			      colorGroup(), isEnabled(),
+			      0, textLabel() );
 	} else {
-	    qDrawItem( p, style(), 1, 1, width()-2, height() - 2,
-		       AlignCenter, colorGroup(), TRUE, &pm, 0 );
+	    style().drawItem( p, 1 + sx, 1 + sy, width()-2, height() - 2,
+			      AlignCenter, colorGroup(), TRUE, &pm, QString::null );
 
 	}
     }
@@ -451,7 +481,7 @@ bool QToolButton::uses3D() const
   sets it as tool tip too if \a tipToo is TRUE.
 */
 
-void QToolButton::setTextLabel( const char * newLabel , bool tipToo )
+void QToolButton::setTextLabel( const QString &newLabel , bool tipToo )
 {
     tl = newLabel;
     if ( !tipToo )
@@ -470,33 +500,48 @@ void QToolButton::setTextLabel( const char * newLabel , bool tipToo )
 /*!  Sets this tool button to display the icons in \a set.
   (setPixmap() is effectively a wrapper for this function.)
 
+  For toggle buttons it is possible to set an extra icon set with \a
+  on equals TRUE, which will be used exclusively for the on-state.
+
   QToolButton makes a copy of \a set, so you must delete \a set
   yourself.
 
-  \sa iconSet() QIconSet
+  \sa iconSet() QIconSet, setToggleButton(), isOn()
 */
 
-void QToolButton::setIconSet( const QIconSet & set )
+void QToolButton::setIconSet( const QIconSet & set, bool on )
 {
-    if ( s )
-	delete s;
-    s = new QIconSet( set );
+    if ( !on ) {
+	if ( s )
+	    delete s;
+	s = new QIconSet( set );
+    } else {
+	if ( son )
+	    delete son;
+	son = new QIconSet( set );
+    }
 }
 
 
 /*!  Returns a copy of the icon set in use.  If no icon set has been
-  set, iconSeT() creates one from the pixmap().
+  set, iconSet() creates one from the pixmap().
 
   If the button doesn't have a pixmap either, iconSet()'s return value
   is meaningless.
 
+  If \a on equals TRUE, the special icon set for the on-state of the
+  button is returned.
+
   \sa setIconSet() QIconSet
 */
 
-QIconSet QToolButton::iconSet() const
+QIconSet QToolButton::iconSet( bool on ) const
 {
     QToolButton * that = (QToolButton *)this;
 
+    if ( on && that->son )
+	return *that->son;
+
     if ( pixmap() && (!that->s || (that->s->pixmap().serialNumber() !=
 				   pixmap()->serialNumber())) )
 	that->setIconSet( *pixmap() );
