@@ -1,29 +1,30 @@
 /****************************************************************************
-** $Id: qtooltip.cpp,v 2.33.2.1 1997/11/18 12:40:13 agulbra Exp $
+** $Id: qtooltip.cpp,v 2.84.2.2 1999/07/13 19:59:33 warwick Exp $
 **
 ** Tool Tips (or Balloon Help) for any widget or rectangle
 **
-** Copyright (C) 1992-1997 Troll Tech AS.  All rights reserved.
+** Copyright (C) 1992-1999 Troll Tech AS.  All rights reserved.
 **
-** This file is part of the free edition of Qt 1.33.
+** This file is part of the Qt GUI Toolkit.
 **
-** See the file LICENSE included in the distribution for the usage
-** and distribution terms, or http://www.troll.no/qt/license.html.
+** This file may be distributed under the terms of the Q Public License
+** as defined by Troll Tech AS of Norway and appearing in the file
+** LICENSE.QPL included in the packaging of this file.
 **
-** IMPORTANT NOTE: You may NOT copy this file or any part of it into
-** your own programs or libraries.
+** Licensees holding valid Qt Professional Edition licenses may use this
+** file in accordance with the Qt Professional Edition License Agreement
+** provided with the Qt Professional Edition.
 **
-** (This file differs from the one in the commercial edition of Qt
-** only by this comment.)
+** See http://www.troll.no/pricing.html or email sales@troll.no for
+** information about the Professional Edition licensing, or see
+** http://www.troll.no/qpl/ for QPL licensing information.
 **
 *****************************************************************************/
 
 #include "qtooltip.h"
 #include "qlabel.h"
 #include "qptrdict.h"
-#include "qapp.h"
-
-RCSTAG("$Id: qtooltip.cpp,v 2.33.2.1 1997/11/18 12:40:13 agulbra Exp $");
+#include "qapplication.h"
 
 // Magic value meaning an entire widget - if someone tries to insert a
 // tool tip on this part of a widget it will be interpreted as the
@@ -31,10 +32,37 @@ RCSTAG("$Id: qtooltip.cpp,v 2.33.2.1 1997/11/18 12:40:13 agulbra Exp $");
 
 static inline QRect entireWidget()
 {
-    return QRect( QCOORD_MIN, QCOORD_MIN,
-		  QCOORD_MAX-QCOORD_MIN, QCOORD_MAX-QCOORD_MIN );
+    return QRect( -QWIDGETSIZE_MAX, -QWIDGETSIZE_MAX,
+		  2*QWIDGETSIZE_MAX, 2*QWIDGETSIZE_MAX );
 }
 
+static bool use_style_override=FALSE;
+
+// Internal class - don't touch
+
+class QTipLabel : public QLabel
+{
+    Q_OBJECT
+public:
+    QTipLabel(const QString& text) : QLabel( 0, "toolTipTip",
+			  WStyle_StaysOnTop +
+			  WStyle_Customize + WStyle_NoBorder + WStyle_Tool )
+    {
+	setMargin(1);
+	setIndent(0);
+	setAutoMask( FALSE );
+	if ( use_style_override ) {
+	    setFont( QToolTip::font() );
+	    setPalette( QToolTip::palette(), TRUE );
+	}
+	setFrameStyle( QFrame::Plain | QFrame::Box );
+	setLineWidth( 1 );
+	setAlignment( AlignLeft | AlignTop );
+	polish();
+	setText(text);
+	adjustSize();
+    }
+};
 
 // Internal class - don't touch
 
@@ -57,8 +85,8 @@ public:
     };
 
     bool    eventFilter( QObject * o, QEvent * e );
-    void    add( QWidget *, const QRect &, const char *, 
-		 QToolTipGroup *, const char *, QToolTip *, bool );
+    void    add( QWidget *, const QRect &, const QString& ,
+		 QToolTipGroup *, const QString& , QToolTip *, bool );
     void    remove( QWidget *, const QRect & );
     void    remove( QWidget * );
 
@@ -82,72 +110,12 @@ private:
     QLabel	     *label;
     QPoint	      pos;
     QWidget	     *widget;
-    QTipManager::Tip *currentTip;
-    QTipManager::Tip *previousTip;
+    Tip *currentTip;
+    Tip *previousTip;
     bool isApplicationFilter;
 };
 
 
-// moc stuff - included by hand.
-
-/****************************************************************************
-** QTipManager meta object code from reading C++ file 'qtooltip.cpp'
-**
-** Created: Mon Mar 17 12:39:34 1997
-**      by: The Qt Meta Object Compiler ($Revision: 2.33.2.1 $)
-**
-** WARNING! All changes made in this file will be lost!
-*****************************************************************************/
-
-#if !defined(Q_MOC_OUTPUT_REVISION)
-#define Q_MOC_OUTPUT_REVISION 2
-#elif Q_MOC_OUTPUT_REVISION != 2
-#error Moc format conflict - please regenerate all moc files
-#endif
-
-#include <qmetaobj.h>
-
-
-const char *QTipManager::className() const
-{
-    return "QTipManager";
-}
-
-QMetaObject *QTipManager::metaObj = 0;
-
-void QTipManager::initMetaObject()
-{
-    if ( metaObj )
-	return;
-    if ( strcmp(QObject::className(), "QObject") != 0 )
-	badSuperclassWarning("QTipManager","QObject");
-    if ( !QObject::metaObject() )
-	QObject::initMetaObject();
-    typedef void(QTipManager::*m1_t0)();
-    typedef void(QTipManager::*m1_t1)();
-    typedef void(QTipManager::*m1_t2)();
-    typedef void(QTipManager::*m1_t3)();
-    m1_t0 v1_0 = &QTipManager::labelDestroyed;
-    m1_t1 v1_1 = &QTipManager::clientWidgetDestroyed;
-    m1_t2 v1_2 = &QTipManager::showTip;
-    m1_t3 v1_3 = &QTipManager::hideTip;
-    QMetaData *slot_tbl = new QMetaData[4];
-    slot_tbl[0].name = "labelDestroyed()";
-    slot_tbl[1].name = "clientWidgetDestroyed()";
-    slot_tbl[2].name = "showTip()";
-    slot_tbl[3].name = "hideTip()";
-    slot_tbl[0].ptr = *((QMember*)&v1_0);
-    slot_tbl[1].ptr = *((QMember*)&v1_1);
-    slot_tbl[2].ptr = *((QMember*)&v1_2);
-    slot_tbl[3].ptr = *((QMember*)&v1_3);
-    metaObj = new QMetaObject( "QTipManager", "QObject",
-	slot_tbl, 4,
-	0, 0 );
-}
-
-// moc code ends here
-
-
 // We have a global, internal QTipManager object
 
 static QTipManager *tipManager	  = 0;
@@ -176,7 +144,6 @@ static void initTipManager()
 QTipManager::QTipManager()
     : QObject( 0, "toolTipManager" )
 {
-    initMetaObject();
     tips = new QPtrDict<QTipManager::Tip>( 313 );
     currentTip = 0;
     previousTip = 0;
@@ -215,8 +182,8 @@ QTipManager::~QTipManager()
     delete label;
 }
 
-void QTipManager::add( QWidget *w, const QRect &r, const char *s,
-		       QToolTipGroup *g, const char *gs,
+void QTipManager::add( QWidget *w, const QRect &r, const QString &s,
+		       QToolTipGroup *g, const QString& gs,
 		       QToolTip *tt, bool a )
 {
     QTipManager::Tip *h = (*tips)[ w ];
@@ -235,7 +202,8 @@ void QTipManager::add( QWidget *w, const QRect &r, const char *s,
     tips->insert( w, t );
     connect( w, SIGNAL(destroyed()), this, SLOT(clientWidgetDestroyed()) );
     if ( a ) {
-	showTip();
+	if ( t->rect.contains( pos ) )
+	    showTip();
 	tips->take( w );
 	if( t->next )
 	    tips->insert( w, t->next );
@@ -260,13 +228,8 @@ void QTipManager::remove( QWidget *w, const QRect & r )
 
     if ( t->rect == r ) {
 	tips->take( w );
-	if ( t->next ) {
+	if ( t->next )
 	    tips->insert( w, t->next );
-	} else {
-	    // ### w->setMouseTracking( FALSE );
-	    // ### need to disable sometimes
-	    // w->removeEventFilter( tipManager );
-	}
 	delete t;
     } else {
 	while( t->next && t->next->rect != r )
@@ -349,43 +312,54 @@ void QTipManager::removeFromGroup( QToolTipGroup *g )
 
 bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 {
-    // avoid dumping core in case of application madness
-    if ( !tips || !e || !obj || !obj->isWidgetType() )
+    // avoid dumping core in case of application madness, and return
+    // quickly for some common but irrelevant events
+    if ( !qApp || !qApp->focusWidget() ||
+	 !obj || !obj->isWidgetType() || // isWidgetType() catches most stuff
+	 !e ||
+	 e->type() == QEvent::Paint ||
+	 e->type() == QEvent::Timer ||
+	 e->type() == QEvent::SockAct ||
+	 !tips )
 	return FALSE;
     QWidget *w = (QWidget *)obj;
 
+    if ( e->type() == QEvent::FocusOut || e->type() == QEvent::FocusIn ) {
+	// user moved focus somewhere - hide the tip
+	hideTip();
+	fallAsleep.stop();
+	leaveWindow.stop();
+	return FALSE;
+    }
+	
     QTipManager::Tip *t = 0;
     while( w && !t ) {
 	t = (*tips)[ w ];
 	if ( !t )
-	    w = w->testWFlags(WType_TopLevel) ? 0 : w->parentWidget();
+	    w = w->isTopLevel() ? 0 : w->parentWidget();
     }
 
     if ( !t ) {
-	if ( e->type() >= Event_MouseButtonPress &&
-	     e->type() <= Event_Leave )
-	    wakeUp.stop();
+	if ( e->type() >= QEvent::MouseButtonPress &&
+	     e->type() <= QEvent::Leave )
+	    hideTip();
 	return FALSE;
     }
 
     // with that out of the way, let's get down to action
 
     switch( e->type() ) {
-    case Event_Timer: // fall through
-    case Event_Paint:
-	// no processing at all
-	break;
-    case Event_MouseButtonPress:
-    case Event_MouseButtonRelease:
-    case Event_MouseButtonDblClick:
-    case Event_KeyPress:
-    case Event_KeyRelease:
+    case QEvent::MouseButtonPress:
+    case QEvent::MouseButtonRelease:
+    case QEvent::MouseButtonDblClick:
+    case QEvent::KeyPress:
+    case QEvent::KeyRelease:
 	// input - turn off tool tip mode
 	hideTip();
 	fallAsleep.stop();
 	leaveWindow.stop();
 	break;
-    case Event_MouseMove:
+    case QEvent::MouseMove:
 	{ // a whole scope just for one variable
 	    QMouseEvent * m = (QMouseEvent *)e;
 
@@ -401,13 +375,17 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 
 	    wakeUp.stop();
 	    if ( m->state() == 0 ) {
-		if ( (label && label->isVisible()) )
+		if ( (label && label->isVisible()) ) {
 		    return TRUE;
-		else if ( fallAsleep.isActive() )
-		    wakeUp.start( 100, TRUE );
-		else {
-		    previousTip = 0;
-		    wakeUp.start( 1000, TRUE );
+		} else {
+		    if ( fallAsleep.isActive() ) {
+			wakeUp.start( 100, TRUE );
+		    } else {
+			previousTip = 0;
+			wakeUp.start( 700, TRUE );
+		    }
+		    if ( t->group && !t->group->d && !t->groupText.isEmpty() )
+			emit t->group->showTip( t->groupText );
 		}
 		widget = w;
 		pos = mousePos;
@@ -417,17 +395,19 @@ bool QTipManager::eventFilter( QObject *obj, QEvent *e )
 	    }
 	}
 	break;
-    case Event_Enter:
+    case QEvent::Enter:
 	if ( label && label->isVisible() && w == widget )
 	    leaveWindow.stop();
 	else if ( w ) // test in event_leave below should always hit first
-	    wakeUp.stop();
+	    hideTip();
 	break;
-    case Event_Leave:
+    case QEvent::Leave:
 	if ( label && label->isVisible() )
 	    leaveWindow.start( 50, TRUE );
 	else if ( widget != w )
-	    wakeUp.stop();
+	    hideTip();
+	if ( t->group && !t->group->d && !t->groupText.isEmpty() )
+	    emit t->group->showTip( QString::null );
 	break;
     default:
 	hideTip();
@@ -448,7 +428,7 @@ void QTipManager::showTip()
     QTipManager::Tip *t = (*tips)[ widget ];
     while ( t && !t->rect.contains( pos ) )
 	t = t->next;
-    if ( t == 0 || t == previousTip )
+    if ( t == 0 )
 	return;
 
     if ( t->tip ) {
@@ -458,36 +438,28 @@ void QTipManager::showTip()
 
     if ( label ) {
 	label->setText( t->text );
+	label->adjustSize();
     } else {
-	label = new QLabel( 0, "toolTipTip",
-			    WStyle_Customize | WStyle_NoBorder | WStyle_Tool );
+	label = new QTipLabel(t->text);
 	CHECK_PTR( label );
 	connect( label, SIGNAL(destroyed()), SLOT(labelDestroyed()) );
-	label->setFont( QToolTip::font() );
-	label->setPalette( QToolTip::palette() );
-	label->setText( t->text );
-	if ( QApplication::style() == MotifStyle )
-	    label->setFrameStyle( QFrame::Plain | QFrame::Box );
-	else
-	    label->setFrameStyle( QFrame::Raised | QFrame::Panel );
-	label->setLineWidth( 1 );
-	label->setMargin( 3 );
-	label->setAlignment( AlignLeft | AlignTop );
-	label->setAutoResize( TRUE );
     }
     QPoint p = widget->mapToGlobal( pos ) + QPoint( 2, 16 );
     if ( p.x() + label->width() > QApplication::desktop()->width() )
 	p.setX( QApplication::desktop()->width() - label->width() );
     if ( p.y() + label->height() > QApplication::desktop()->height() )
 	p.setY( p.y() - 20 - label->height() );
-    label->move( p );
-    label->show();
-    label->raise();
+    if ( label->text().length() ) {
+	label->move( p );
+	label->show();
+	label->raise();
+    }
 
-    fallAsleep.start( 4000, TRUE );
+    if ( !label->text() && label->text().length() < 25 )
+	fallAsleep.start( 5000, TRUE );
     leaveWindow.stop();
 
-    if ( t->group && !t->groupText.isEmpty() )
+    if ( t->group && t->group->d && !t->groupText.isEmpty() )
 	emit t->group->showTip( t->groupText );
     currentTip = t;
     previousTip = 0;
@@ -498,13 +470,18 @@ void QTipManager::hideTip()
 {
     if ( label && label->isVisible() ) {
 	label->hide();
-	fallAsleep.start( 5000, TRUE );
+	fallAsleep.start( 10000, TRUE );
+	wakeUp.stop();
 	if ( currentTip && currentTip->group )
 	    emit currentTip->group->removeTip();
     } else if ( wakeUp.isActive() ) {
 	wakeUp.stop();
+	if ( currentTip && currentTip->group &&
+	     !currentTip->group->d && !currentTip->groupText.isEmpty() )
+	    emit currentTip->group->removeTip();
     }
 
+
     if ( currentTip && currentTip->autoDelete )
 	delete currentTip;
     else
@@ -521,6 +498,8 @@ void QTipManager::hideTip()
   \brief The QToolTip class provides tool tips (sometimes called
   balloon help) for any widget or rectangular part of a widget.
 
+  \ingroup application
+
   The tip is a short, one-line text reminding the user of the widget's
   or rectangle's function.  It is drawn immediately below the region,
   in a distinctive black on yellow combination.  In Motif style, Qt's
@@ -540,12 +519,12 @@ void QTipManager::hideTip()
   a key, lets the mouse rest for five seconds, or moves the mouse
   outside \e all tip-equpped regions for at least a second.
 
-  There are no less than three APIs for QToolTip: <ol> <li> Adding a
-  tip to an entire widget. <li> Adding a tip to a fixed rectangle
-  within a widget. <li> Adding a tip to a dynamic rectangle within a
-  widget. </ol>
+  The QToolTip class can be used in three different ways: <ol> <li>
+  Adding a tip to an entire widget. <li> Adding a tip to a fixed
+  rectangle within a widget. <li> Adding a tip to a dynamic rectangle
+  within a widget. </ol>
 
-  To add a tip to a widget, call the static function QToolTip::add()
+  To add a tip to a widget, call the \e static function QToolTip::add()
   with the widget and tip as arguments:
 
   \code
@@ -560,10 +539,10 @@ void QTipManager::hideTip()
     QToolTip::remove( quitButton );
   \endcode
 
-  You can also display another text (typically in a status bar),
-  courtesy of QToolTipGroup.  This example assumes that \e g is a
-  <code>QToolTipGroup *</code> and already connected to the
-  appropriate status bar:
+  You can also display another text (typically in a \link QStatusBar
+  status bar),\endlink courtesy of QToolTipGroup.  This example
+  assumes that \e g is a <code>QToolTipGroup *</code> and already
+  connected to the appropriate status bar:
 
   \code
     QToolTip::add( quitButton, "Leave the application", g,
@@ -574,25 +553,29 @@ void QTipManager::hideTip()
 
   To add a tip to a fixed rectangle within a widget, call the static
   function QToolTip::add() with the widget, rectangle and tip as
-  arguments.  (See the tellme/tellme.cpp example.)  Again, you can supply a
+  arguments.  (See the tooltip/tooltip.cpp example.)  Again, you can supply a
   QToolTipGroup * and another text if you want.
 
   Both of the above are one-liners and cover the vast majority of
-  cases.  The third and most general API uses a pure virtual function
-  to decide whether to pop up a tool tip.  The tellme/tellme.cpp
-  example demonstrates this too.  This mode can be used to implement
-  e.g. tips for text that can move as the user scrolls.
+  cases.  The third and most general way to use QToolTip uses a pure
+  virtual function to decide whether to pop up a tool tip.  The
+  tooltip/tooltip.cpp example demonstrates this too.  This mode can be
+  used to implement e.g. tips for text that can move as the user
+  scrolls.
 
-  To use this API, you need to subclass QToolTip and reimplement
-  maybeTip().  maybeTip() will be called when there's a chance that a
-  tip should pop up.  It must decide whether to show a tip, and
-  possibly call add() with the rectangle the tip applies to, the tip's
-  text and optionally the QToolTipGroup details.  The tip will
-  disappear once the mouse moves outside the rectangle you supply, and
-  \e not \e reappear - maybeTip() will be called again if the user
-  lets the mouse rest within the same rectangle again.  You can
-  forcibly remove the tip by calling remove() with no arguments.  This
-  is handy if the widget scrolls.
+  To use QToolTip like this, you need to subclass QToolTip and
+  reimplement maybeTip().  maybeTip() will be called when there's a
+  chance that a tip should pop up.  It must decide whether to show a
+  tip, and possibly call add() with the rectangle the tip applies to,
+  the tip's text and optionally the QToolTipGroup details.  The tip
+  will disappear once the mouse moves outside the rectangle you
+  supply, and \e not \e reappear - maybeTip() will be called again if
+  the user lets the mouse rest within the same rectangle again.  You
+  can forcibly remove the tip by calling remove() with no arguments.
+  This is handy if the widget scrolls.
+
+  \sa QStatusBar QWhatsThis QToolTipGroup
+  <a href="guibooks.html#fowler">GUI Design Handbook: Tool Tip</a>
 */
 
 
@@ -608,14 +591,14 @@ void QToolTip::initialize()
 {
     if ( ttFont )				// already initialized
 	return;
+
     qAddPostRoutine( cleanup );
-    ttFont = new QFont;
-    CHECK_PTR( ttFont );
-    QColorGroup cg( black, QColor(255,255,220),
-		    QColor(96,96,96), black, black,
-		    black, QColor(255,255,220) );
-    ttPalette = new QPalette( cg, cg, cg );
-    CHECK_PTR( ttPalette );
+
+    QTipLabel t("");
+    ttFont = new QFont(QApplication::font(&t));
+    ttPalette = new QPalette(QApplication::palette(&t));
+
+    use_style_override = TRUE;
 }
 
 
@@ -651,6 +634,7 @@ void QToolTip::setFont( const QFont &font )
     if ( !ttFont )
 	initialize();
     *ttFont = font;
+    use_style_override = TRUE;
 }
 
 
@@ -677,6 +661,7 @@ void QToolTip::setPalette( const QPalette &palette )
     if ( !ttPalette )
 	initialize();
     *ttPalette = palette;
+    use_style_override = TRUE;
 }
 
 
@@ -696,7 +681,8 @@ QToolTip::QToolTip( QWidget * parent, QToolTipGroup * group )
     p = parent;
     g = group;
     initTipManager();
-    tipManager->add( p, entireWidget(), 0, g, 0, this, FALSE );
+    tipManager->add( p, entireWidget(),
+		     QString::null, g, QString::null, this, FALSE );
 }
 
 
@@ -709,10 +695,10 @@ QToolTip::QToolTip( QWidget * parent, QToolTipGroup * group )
   and so on.
 */
 
-void QToolTip::add( QWidget *widget, const char *text )
+void QToolTip::add( QWidget *widget, const QString &text )
 {
     initTipManager();
-    tipManager->add( widget, entireWidget(), text, 0, 0, 0, FALSE );
+    tipManager->add( widget, entireWidget(), text, 0, QString::null, 0, FALSE );
 }
 
 
@@ -726,8 +712,8 @@ void QToolTip::add( QWidget *widget, const char *text )
   Normally, \a longText is shown in a status bar or similar.
 */
 
-void QToolTip::add( QWidget *widget, const char *text,
-		    QToolTipGroup *group, const char *longText )
+void QToolTip::add( QWidget *widget, const QString &text,
+		    QToolTipGroup *group, const QString& longText )
 {
     initTipManager();
     tipManager->add( widget, entireWidget(), text, group, longText, 0, FALSE );
@@ -747,16 +733,16 @@ void QToolTip::remove( QWidget * widget )
 	tipManager->remove( widget, entireWidget() );
 }
 
-/*!  
+/*!
   Adds a tool tip to a fixed rectangle within \a widget.  \a text is
   the text shown in the tool tip.  QToolTip makes a deep copy of this
   string.
 */
 
-void QToolTip::add( QWidget * widget, const QRect & rect, const char * text )
+void QToolTip::add( QWidget * widget, const QRect & rect, const QString &text )
 {
     initTipManager();
-    tipManager->add( widget, rect, text, 0, 0, 0, FALSE );
+    tipManager->add( widget, rect, text, 0, QString::null, 0, FALSE );
 }
 
 
@@ -772,8 +758,8 @@ void QToolTip::add( QWidget * widget, const QRect & rect, const char * text )
 */
 
 void QToolTip::add( QWidget *widget, const QRect &rect,
-		    const char *text,
-		    QToolTipGroup *group, const char *groupText )
+		    const QString& text,
+		    QToolTipGroup *group, const QString& groupText )
 {
     initTipManager();
     tipManager->add( widget, rect, text, group, groupText, 0, FALSE );
@@ -801,36 +787,51 @@ void QToolTip::remove( QWidget * widget, const QRect & rect )
   QToolTip offers.
 
   It is called when there is a chance that a tool tip should be shown,
-  and must decide whether there is a tool tip for the point \a p and
-  what rectangle
+  and must decide whether there is a tool tip for the point \a p in
+  the widget this QToolTip object relates to.
+
+  \a p is given in that widget's local coordinates.  Most maybeTip()
+  implementation will be of the form:
+
+  \code
+    if ( \\<something\\> ) {
+	tip( \\<something\\>, \\<something\\> );
+    }
+  \endcode
+
+  The first argument to tip() (a rectangle) should include the \a p,
+  or QToolTip, the user or both can be confused.
+
+  \sa tip()
 */
 
 
 /*!
-  Pop up a tip saying \a text right now, and remove that tip once
-  the cursor moves out of rectangle \a rect.
+  Pops up a tip saying \a text right now, and removes that tip once
+  the cursor moves out of rectangle \a rect (which is given in the
+  coordinate system of the widget this QToolTip relates to).
 
   The tip will not come back if the cursor moves back; your maybeTip()
   has to reinstate it each time.
 */
 
-void QToolTip::tip( const QRect & rect, const char * text )
+void QToolTip::tip( const QRect & rect, const QString &text )
 {
     initTipManager();
-    tipManager->add( parentWidget(), rect, text, 0, 0, 0, TRUE );
+    tipManager->add( parentWidget(), rect, text, 0, QString::null, 0, TRUE );
 }
 
 
 /*!
-  Pop up a tip saying \a text right now, and remove that tip once
+  Pops up a tip saying \a text right now, and removes that tip once
   the cursor moves out of rectangle \a rect.
 
   The tip will not come back if the cursor moves back; your maybeTip()
   has to reinstate it each time.
 */
 
-void QToolTip::tip( const QRect & rect, const char * text,
-		    const char * groupText )
+void QToolTip::tip( const QRect & rect, const QString &text,
+		    const QString& groupText )
 {
     initTipManager();
     tipManager->add( parentWidget(), rect, text, group(), groupText, 0, TRUE );
@@ -838,7 +839,7 @@ void QToolTip::tip( const QRect & rect, const char * text,
 
 
 /*!
-  Remove all tool tips for this widget immediately.
+  Removes all tool tips for this widget immediately.
 */
 
 void QToolTip::clear()
@@ -867,7 +868,7 @@ void QToolTip::clear()
   isn't a member of any group.
 
   The tool tip group is the object responsible for relaying contact
-  betweeen tool tips and a status bar or something else which can show
+  between tool tips and a status bar or something else which can show
   a longer help text.
 
   \sa parentWidget(), QToolTipGroup
@@ -877,8 +878,7 @@ void QToolTip::clear()
 /*!
   \class QToolTipGroup qtooltip.h
 
-  \brief The QToolTipGroup class provides a way to group tool tips
-  into natural groups.
+  \brief The QToolTipGroup class collects tool tips into natural groups.
 
   Tool tips can display \e two texts, the one in the tip and
   optionally another one, typically in a status bar.  QToolTipGroup
@@ -889,8 +889,8 @@ void QToolTip::clear()
 
   \code
     QToolTipGroup * g = new QToolTipGroup( this, "tool tip relay" );
-    connect( g, SIGNAL(showTip(const char *)),
-             myLabel, SLOT(setText(const char *)) );
+    connect( g, SIGNAL(showTip(const QString&)),
+             myLabel, SLOT(setText(const QString&)) );
     connect( g, SIGNAL(removeTip()),
              myLabel, SLOT(clear()) );
     QToolTip::add( giraffeButton, "feed giraffe",
@@ -907,7 +907,7 @@ void QToolTip::clear()
   Deleting a tool tip group removes the tool tips in it.
 */
 
-/*! \fn void QToolTipGroup::showTip (const char* longText)
+/*! \fn void QToolTipGroup::showTip (const QString &longText)
 
   This signal is emitted when one of the tool tips in the group is
   displayed.  \a longText is the supplementary text for the displayed
@@ -932,7 +932,7 @@ void QToolTip::clear()
 QToolTipGroup::QToolTipGroup( QObject *parent, const char *name )
     : QObject( parent, name )
 {
-    initMetaObject();
+    // nothing
 }
 
 
@@ -945,3 +945,114 @@ QToolTipGroup::~QToolTipGroup()
     if ( tipManager )
 	tipManager->removeFromGroup( this );
 }
+
+
+/*!  Returns TRUE if the group text is shown delayed (at the same time
+as the tip) and FALSE if it is shown immediately.
+
+\sa setDelay()
+*/
+
+bool QToolTipGroup::delay() const
+{
+    return d;
+}
+
+
+/*!  Sets the group to show its text immediately if \a enable is
+FALSE, and delayed (at the same time as the tip text) if \a enable is
+TRUE.  The default is TRUE.
+
+\sa delay()
+*/
+
+void QToolTipGroup::setDelay( bool enable )
+{
+#if 0
+    if ( enable && !d ) {
+	// maybe we should show the text at once?
+    }
+#endif
+    d = enable;
+}
+
+
+
+
+// moc stuff - included by hand.  !Gmoc %
+
+/****************************************************************************
+** QTipLabel meta object code from reading C++ file 'qtooltip.cpp'
+**
+** Created: Sun Aug 23 21:50:26 1998
+**      by: The Qt Meta Object Compiler ($Revision: 2.84.2.2 $)
+**
+** WARNING! All changes made in this file will be lost!
+*****************************************************************************/
+
+#if !defined(Q_MOC_OUTPUT_REVISION)
+#define Q_MOC_OUTPUT_REVISION 3
+#elif Q_MOC_OUTPUT_REVISION != 3
+#error "Moc format conflict - please regenerate all moc files"
+#endif
+
+#include <qmetaobject.h>
+
+
+const char *QTipLabel::className() const
+{
+    return "QTipLabel";
+}
+
+QMetaObject *QTipLabel::metaObj = 0;
+
+void QTipLabel::initMetaObject()
+{
+    if ( metaObj )
+	return;
+    if ( strcmp(QLabel::className(), "QLabel") != 0 )
+	badSuperclassWarning("QTipLabel","QLabel");
+    if ( !QLabel::metaObject() )
+	QLabel::initMetaObject();
+    metaObj = new QMetaObject( "QTipLabel", "QLabel",
+	0, 0,
+	0, 0 );
+}
+
+
+const char *QTipManager::className() const
+{
+    return "QTipManager";
+}
+
+QMetaObject *QTipManager::metaObj = 0;
+
+void QTipManager::initMetaObject()
+{
+    if ( metaObj )
+	return;
+    if ( strcmp(QObject::className(), "QObject") != 0 )
+	badSuperclassWarning("QTipManager","QObject");
+    if ( !QObject::metaObject() )
+	QObject::initMetaObject();
+    typedef void(QTipManager::*m1_t0)();
+    typedef void(QTipManager::*m1_t1)();
+    typedef void(QTipManager::*m1_t2)();
+    typedef void(QTipManager::*m1_t3)();
+    m1_t0 v1_0 = &QTipManager::labelDestroyed;
+    m1_t1 v1_1 = &QTipManager::clientWidgetDestroyed;
+    m1_t2 v1_2 = &QTipManager::showTip;
+    m1_t3 v1_3 = &QTipManager::hideTip;
+    QMetaData *slot_tbl = new QMetaData[4];
+    slot_tbl[0].name = "labelDestroyed()";
+    slot_tbl[1].name = "clientWidgetDestroyed()";
+    slot_tbl[2].name = "showTip()";
+    slot_tbl[3].name = "hideTip()";
+    slot_tbl[0].ptr = *((QMember*)&v1_0);
+    slot_tbl[1].ptr = *((QMember*)&v1_1);
+    slot_tbl[2].ptr = *((QMember*)&v1_2);
+    slot_tbl[3].ptr = *((QMember*)&v1_3);
+    metaObj = new QMetaObject( "QTipManager", "QObject",
+	slot_tbl, 4,
+	0, 0 );
+}
