@@ -10,7 +10,6 @@
 #include <qwidget.h>
 
 class QScrollBar;
-class QLCDNumber;
 
 
 class LCDRange : public QWidget
@@ -20,16 +19,17 @@ public:
     LCDRange( QWidget *parent=0, const char *name=0 );
 
     int value() const;
+
 public slots:
     void setValue( int );
     void setRange( int minVal, int maxVal );
+
 signals:
     void valueChanged( int );
-protected:
-    void resizeEvent( QResizeEvent * );
+
 private:
     QScrollBar  *sBar;
-    QLCDNumber  *lcd;
 };
 
+
 #endif // LCDRANGE_H
