@@ -3,12 +3,12 @@
 ** Qt tutorial 12
 **
 ****************************************************************/
-
-#include <qapp.h>
-#include <qpushbt.h>
-#include <qscrbar.h>
-#include <qlcdnum.h>
+#include <qapplication.h>
+#include <qpushbutton.h>
+#include <qscrollbar.h>
+#include <qlcdnumber.h>
 #include <qfont.h>
+#include <qlayout.h>
 
 #include "lcdrange.h"
 #include "cannon.h"
@@ -18,14 +18,6 @@ class MyWidget : public QWidget
 {
 public:
     MyWidget( QWidget *parent=0, const char *name=0 );
-protected:
-    void resizeEvent( QResizeEvent * );
-private:
-    QPushButton *quit;
-    QPushButton *shoot;
-    LCDRange    *angle;
-    LCDRange    *force;
-    CannonField *cannonField;
 };
 
 
@@ -34,22 +26,21 @@ MyWidget::MyWidget( QWidget *parent, const char *name )
 {
     setMinimumSize( 500, 355 );
 
-    quit = new QPushButton( "Quit", this, "quit" );
-    quit->setGeometry( 10, 10, 75, 30 );
+    QPushButton *quit = new QPushButton( "Quit", this, "quit" );
     quit->setFont( QFont( "Times", 18, QFont::Bold ) );
 
     connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );
 
-    angle  = new LCDRange( "ANGLE", this, "angle" );
+    LCDRange *angle  = new LCDRange( "ANGLE", this, "angle" );
     angle->setRange( 5, 70 );
-    angle->setGeometry( 10, quit->y() + quit->height() + 10, 75, 130 );
 
-    force  = new LCDRange( "FORCE", this, "force" );
+    LCDRange *force  = new LCDRange( "FORCE", this, "force" );
     force->setRange( 10, 50 );
-    force->setGeometry( 10, angle->y() + angle->height() + 10, 75, 130 );
 
-    cannonField = new CannonField( this, "cannonField" );
-    cannonField->move( angle->x() + angle->width() + 10, angle->y() );
+    QPushButton *shoot = new QPushButton( "Shoot", this, "shoot" );
+    shoot->setFont( QFont( "Times", 18, QFont::Bold ) );
+
+    CannonField *cannonField = new CannonField( this, "cannonField" );
     cannonField->setBackgroundColor( QColor( 250, 250, 200) );
 
     connect( angle,SIGNAL(valueChanged(int)), cannonField,SLOT(setAngle(int)));
@@ -58,22 +49,25 @@ MyWidget::MyWidget( QWidget *parent, const char *name )
     connect( force,SIGNAL(valueChanged(int)), cannonField,SLOT(setForce(int)));
     connect( cannonField,SIGNAL(forceChanged(int)), force,SLOT(setValue(int)));
 
-    connect( cannonField, SIGNAL(hit()), cannonField, SLOT(newTarget()) );
+    connect( shoot, SIGNAL(clicked()), cannonField, SLOT(shoot()) );
+
+    QGridLayout *grid = new QGridLayout( this, 2, 2, 10 );
+    grid->addWidget( quit, 0, 0 );
+    grid->addWidget( cannonField, 1, 1 );
+    grid->setColStretch( 1, 10 );
+    
+    QVBoxLayout *leftBox = new QVBoxLayout;
+    grid->addLayout( leftBox, 1, 0 );
+    leftBox->addWidget( angle );
+    leftBox->addWidget( force );
+
+    QHBoxLayout *topBox = new QHBoxLayout;
+    grid->addLayout( topBox, 0, 1 );
+    topBox->addWidget( shoot );
+    topBox->addStretch( 1 );
 
     angle->setValue( 60 );
     force->setValue( 25 );
-
-    shoot = new QPushButton( "Shoot", this, "shoot" );
-    shoot->setGeometry( 90, 10, 75, 30 );
-    shoot->setFont( QFont( "Times", 18, QFont::Bold ) );
-
-    connect( shoot, SIGNAL(clicked()), cannonField, SLOT(shoot()) );
-}
-
-void MyWidget::resizeEvent( QResizeEvent * )
-{
-    cannonField->resize( width()  - cannonField->x() - 10,
-			 height() - cannonField->y() - 10 );
 }
 
 int main( int argc, char **argv )
@@ -82,8 +76,14 @@ int main( int argc, char **argv )
     QApplication a( argc, argv );
 
     MyWidget w;
-    w.setGeometry( 100, 100, 500, 355 );
     a.setMainWidget( &w );
     w.show();
     return a.exec();
 }
+
+
+
+
+
+
+
