@@ -4,7 +4,7 @@
 **
 ****************************************************************/
 
-#include <qapp.h>
+#include <qapplication.h>
 
 #include "gamebrd.h"
 
@@ -15,7 +15,6 @@ int main( int argc, char **argv )
     QApplication a( argc, argv );
 
     GameBoard gb;
-    gb.setGeometry( 100, 100, 500, 355 );
     a.setMainWidget( &gb );
     gb.show();
     return a.exec();
