@@ -7,7 +7,7 @@
 #include "cannon.h"
 #include <qpainter.h>
 #include <qpixmap.h>
-#include <qdatetm.h>
+#include <qdatetime.h>
 #include <qfont.h>
 
 #include <math.h>
@@ -87,7 +87,7 @@ void CannonField::setGameOver()
 }
 
 void CannonField::restartGame()
-{    
+{
     if ( shooting )
 	stopShooting();
     gameEnded = FALSE;
@@ -227,15 +227,15 @@ QRect CannonField::shotRect() const
     const double gravity = 4;
 
     double time      = timerCount / 4.0;
-    double velocity  = shoot_f/0.7; 
+    double velocity  = shoot_f;
     double radians   = shoot_ang*3.14159265/180;
-    
+
     double velx      = velocity*cos( radians );
     double vely      = velocity*sin( radians );
     double x0        = ( barrel_rect.right()  + 5 )*cos(radians);
     double y0        = ( barrel_rect.right()  + 5 )*sin(radians);
     double x         = x0 + velx*time;
-    double y         = y0 + vely*time - gravity*time*time;
+    double y         = y0 + vely*time - 0.5*gravity*time*time;
 
     QRect r = QRect( 0, 0, 6, 6 );
     r.moveCenter( QPoint( qRound(x), height() - 1 - qRound(y) ) );
@@ -262,3 +262,13 @@ bool CannonField::barrelHit( const QPoint &p ) const
     mtx = mtx.invert();
     return barrel_rect.contains( mtx.map(p) );
 }
+
+QSize CannonField::sizeHint() const
+{
+    return QSize( 400, 300 );
+}
+
+QSizePolicy CannonField::sizePolicy() const
+{
+    return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
+}
